#ifndef PLOT_WINDOW_H
#define PLOT_WINDOW_H

#include <QMainWindow>
#include <QAction>
#include <QMenuBar>
#include <QSlider>
#include <QFileDialog>
#include <QDialog>
#include <QLayout>
#include <QLabel>
#include <QLineEdit>
#include <QComboBox>
#include <QPrintDialog>
#include <QPrinter>

#include "IntervalDialog.h"
#include "QtPLWidget.h"
#include "qprofile.h"


class PlotWindow : public QMainWindow
{
    Q_OBJECT
            
	public:
		PlotWindow(int id, int width=500, int height=500, double saveWidth=600, double saveHeight=600, int resolution=72, QPrinter::PageSize pageSize=QPrinter::A4, QPrinter::Orientation orientation=QPrinter::Portrait, QWidget* parent=NULL);
		
			QtPLWidget * plot;
                        
			void print_iden();
                        
			void setSmooth(bool b);
                        
			void setId(int id);
                        
			int getId();
                        
			void setPLFocus();
                        
			void setSize(double width, double height); //for save only
			
			void setTitle(const QString& title);
			
			void fixAspectRatio(double &saveWidth, double &saveHeight);
			
			QPrinter::PageSize pageSize;
			QPrinter::Orientation orientation;
			int resolution; //dpi
			double weight; // used for print and vector graphics plots to weight all lines
			IntervalDialog * intervalDialog;
			QMenu * printWeightMenu;
			QString saveDirectory;
			
	signals:
		
		void clicked(double x,double y, int pos_x, int pos_y); // interface between widget and window
		void doubleClicked(double x,double y, int pos_x, int pos_y); // interface between widget and window
		
		void rescaleScalar(double start, double end); // interface between intervalDialog and window
		void rescaleTime(const QmjdTimeInterval& interval);
				
    public slots:
        
	void savePlotPNG(); // Functions for Mac
	void savePlotJPG();
	void savePlotPDF();
	void savePlotSVG();
	void setWeightL();
	void setWeightM();
	void setWeightH();
	void setPrintRes(int);	

//         void savePlot();

	void printPlot();
	
	void changeDisplayedRatio(double r);
        
	protected:
            
            int id;
            
            PLINT strm;
            
            double saveWidth, saveHeight;
	    
	    QString title;


};

// extern PlotWindow *QPlot_window;

extern QList<PlotWindow*> GlobalPlotWindows;


// void QPlotmakeWindow(QWidget*,char*);

PlotWindow* focusPlotWindow(int id, double width=-1., double height=-1., int resolution=72, QPrinter::PageSize pageSize=QPrinter::A4, QPrinter::Orientation orientation=QPrinter::Portrait);
PlotWindow* newPlotWindow(double width, double height, int* id, int resolution=72, QPrinter::PageSize pageSize=QPrinter::A4, QPrinter::Orientation orientation=QPrinter::Portrait);

void clearPlotWindows();


#endif

