#ifndef QSAS_EDITORS_H_
#define QSAS_EDITORS_H_

#include "qslot.h"
#include "qt_version.h"
#include "Qdos.h"
#include <QObject.h>

#include <QHBoxLayout>
#include "IntervalWidgets.h"

class argSlider : public QWidget {
  
Q_OBJECT

public:
  argSlider(QString title, int indx, int maxN, QWidget *parent=0, char * name=0 );
  ~argSlider();
  
  QguWLinput *numField;
  int num;
  int maxNum;
  int id;
  
public slots:
  void decRec();
  void incRec();
  void adjustRec();
  
signals:
   void QbEchanged(int id, int num);
   
protected:
  
};


class QbEditor : public QGroupBox {

Q_OBJECT

public slots:
   void updateForRec();

signals:
   void editorHasInput();

public:
  QbEditor(QdObject_var dobj, QWidget *parent=0, const char *name=0 );
  ~QbEditor();
  virtual void makeCopy(QdObject_var dobj){;}
  void showDep0Field(bool OKtoShow);
  void showValueField(bool OKtoShow);
  virtual void setReadOnly(bool readOnly);

  
  QWidget * valueBox;
  QWidget * dep0Box;

  QdTimeSeq_var ttags;
  QdRScalarSeq_var stags;
  QdObject_var D0;
  QdObject_var edited_var;

  int record;
  int maxRec;

  QLabel *valueLabel;
  QguWLinput *valueField;
  QguWLinput *SdepField;
  
#if QT_VERSION < 0x040300
  QbDateTimeLineEdit *timeField;
#else
  QbDateTimeWidget *timeField;
#endif

protected:
  QString editorType;
  
};

class QbScalarEditor : public QbEditor {

Q_OBJECT

public slots:
  void valueChanged();
  void timeValueChanged();
  void SdepValueChanged();

signals:
 
public:
  QbScalarEditor(QdObject_var dobj, QWidget *parent=0, const char *name=0 );
  void setData(QdObject_var dobj);
  void makeCopy(QdObject_var dobj);
  
  QdRScalar_var S_var;
  
protected:
  
};

class QbScalarSeqEditor : public QbEditor {

Q_OBJECT

public slots:
  void valueChanged();
  void timeValueChanged();
  void SdepValueChanged();

signals:
 
public:
  QbScalarSeqEditor(QdObject_var dobj, QWidget *parent=0, const char *name=0 );
  void setData(QdObject_var dobj);
  void makeCopy(QdObject_var dobj);
  void updateForRec();

  QdRScalarSeq_var Sseq_var;
  
protected:
  
};

class QbTtagEditor : public QbEditor {

Q_OBJECT

public slots:
  void valueChanged();

signals:
 
public:
  QbTtagEditor(QdObject_var dobj, QWidget *parent=0, const char *name=0 );
  void makeCopy(QdObject_var dobj);
  void updateForRec();
  void setData(QdObject_var dobj);

  QdTimeSeq_var Ttag_var;
  
protected:
  
};


class QbTimeObjEditor : public QbEditor {

Q_OBJECT

public slots:
  void valueChanged();
  void setValue();

signals:
 
public:
  QbTimeObjEditor(QdObject_var dobj, QWidget *parent=0, const char *name=0 );
  void makeCopy(QdObject_var dobj);
  void setData(QdObject_var dobj);

  QdTime_var T_var;
  
protected:
  
};

class QbMatrixEditor : public QbEditor {

Q_OBJECT

public slots:
  void valueChanged();
  void timeValueChanged();
  void SdepValueChanged();
  void changeElement(int, int);

signals:
 
public:
  QbMatrixEditor(QdObject_var dobj, QWidget *parent=0, const char *name=0 );
  void makeCopy(QdObject_var dobj);
  void setData(QdObject_var dobj);
  
  QdRMatrix_var M_var;
  QdSMatrix_var SM_var;

  int dimSize;  
  vector<int> index;

protected:
  
};

class QbMatrixSeqEditor : public QbEditor {

Q_OBJECT

public slots:
  void valueChanged();
  void timeValueChanged();
  void SdepValueChanged();
  void changeElement(int,int);

signals:
 
public:
  QbMatrixSeqEditor(QdObject_var dobj, QWidget *parent=0, const char *name=0 );
  void makeCopy(QdObject_var dobj);
  void updateForRec();
  void setData(QdObject_var dobj);

  QdRMatrixSeq_var Mseq_var;
  QdSMatrixSeq_var SMseq_var;
  
  int dimSize;  
  vector<int> index;
  
protected:
  
};

class QbTextEditor : public QbEditor {

Q_OBJECT

public slots:
  void valueChanged();
  void timeValueChanged();
  void SdepValueChanged();

signals:
 
public:
  QbTextEditor(QdObject_var dobj, QWidget *parent=0, const char *name=0 );
  void makeCopy(QdObject_var dobj);
  void setData(QdObject_var dobj);
  
  QdString_var Str_var;
  
protected:
  
};

class QbTextSeqEditor : public QbEditor {

Q_OBJECT

public slots:
  void valueChanged();
  void timeValueChanged();
  void SdepValueChanged();

signals:
 
public:
  QbTextSeqEditor(QdObject_var dobj, QWidget *parent=0, const char *name=0 );
  void makeCopy(QdObject_var dobj);
  void updateForRec();
  void setData(QdObject_var dobj);

  QdStringSeq_var Strseq_var;
  
protected:
  
};


# endif // QSAS_EDITORS_H_
