#ifndef TREE_VIEW_H
#define TREE_VIEW_H

#include <QTreeView>
#include <QItemDelegate>
#include <QHeaderView>
#include <QPainter>
#include "WLItem.h"
#include "WLModel.h"
#include "qbu.h"



class WLEditor: public QItemDelegate
{
	Q_OBJECT
	
	public:
		WLEditor(QObject *parent = 0);

		QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const;

		void setEditorData(QWidget *editor, const QModelIndex &index) const;
		
		void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const;

		void updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const;
};

class WLView: public QTreeView
{
	Q_OBJECT
	
	public:
	
		WLView(QWidget* parent=NULL);
		
		inline WLModel* wlmodel() const
		{
			return static_cast<WLModel*>(model());
		}
		
		void displayMenu(const QPoint&);
	
	signals:
		void exportET();
		
	public slots:
		
		void newFolder();
		
		void moveUp();
		
		void moveDown();

		void sortWLSelection();
		void sortWL();
		void sortWLPrefs();
		
		void setSortWLname(bool);
		void setSortWLtime(bool);
		void setSortWLtype(bool);
		void setSortRev(bool);

		
		void clear();
		
// 		void fill();

		void copy();
		
		void insert();
		
		void rename();
		
		void browseObject(const QModelIndex& index=QModelIndex());
		void editObject(const QModelIndex& index=QModelIndex());
		
		void newObject();
		void createText();
		void createTextArr();
		void createTime();
		void createTInterval();
		void createConstant();
		void createVConstant();
		void createArray();
		
		void duplicateObj();
		
		void sliceObj();
				
		void tif2ev();
		
		void ev2tif();
		
// 		void exportET();
		void deleteObjectFromPath(const QString& path);
		void deleteObject();
		
		void removeTimeTags();
		
		void selectAllData();
		void deleteAllDataItems();
		void deleteAllTimeIntervalItems();
		void deleteAllConstantItems();
		void clearCacheSelected();
		void clearCache();
		
		void newSelectedIndexes(const QModelIndexList&);
		void setCurrentItem(WLItem* i);
	protected slots:
		void dragEnterEvent(QDragEnterEvent *event);
	protected:
		void drawRow(QPainter * painter, const QStyleOptionViewItem& option, const QModelIndex& index) const;

		QList<WLItem*> selectedItems();
		
		void mousePressEvent(QMouseEvent* e);

		QMenu * RButtonMenu;
		QDialog* objTypeSelector;
		QDialog* sortTypeSelector;
		arrDialog *arrTypeSelector;
		WLEditor* delegate;
		string sortOn;
		bool sortReverse;
		
};

#endif
