#include "qcalcCommon.h"
#include "qcalcSlots.h"
#include "qcalcCalcItem.h"
#include "qcalcInputItem.h"
#include "qcalcOperatorItem.h"
#include "qcalcOutputItem.h"
#include "qcalcExtractionItem.h"
#include "qar.h"

// The evaluators are the classes that do the actual operations for the operator items. This avoid too much sub-classing for VOperatorItem

class QEvaluator
{
	public:
		QEvaluator(){}
		
		virtual ~QEvaluator(){}
		
		// checks that the inputs in "in" are properly connected
		// does the computation (when sub-classed) and puts the results in the "out" slots
		// ok and message indicate the status of the computation
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message);
};

class QEvaluatorAdd: public QEvaluator
{
	public:
		QEvaluatorAdd():QEvaluator(){}
		
		virtual ~QEvaluatorAdd(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message);
};

class QEvaluatorSubtract: public QEvaluator
{
	public:
		QEvaluatorSubtract():QEvaluator(){}
		
		virtual ~QEvaluatorSubtract(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message);
};

class QEvaluatorMultiply: public QEvaluator
{
	public:
		QEvaluatorMultiply():QEvaluator(){}
		
		virtual ~QEvaluatorMultiply(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message);
		
	protected:
		void setProductDepends(QdObject_var resultObj, QdObject_var obj1, QdObject_var obj2);
};

class QEvaluatorDivide: public QEvaluator
{
	public:
		QEvaluatorDivide():QEvaluator(){}
		
		virtual ~QEvaluatorDivide(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorMod: public QEvaluator
{
	public:
		QEvaluatorMod():QEvaluator(){}
		
		virtual ~QEvaluatorMod(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorPow: public QEvaluator
{
	public:
		QEvaluatorPow():QEvaluator(){}
		
		virtual ~QEvaluatorPow(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorOpp: public QEvaluator
{
	public:
		QEvaluatorOpp():QEvaluator(){}
		
		virtual ~QEvaluatorOpp(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorRemainder: public QEvaluator
{
	public:
		QEvaluatorRemainder():QEvaluator(){}
		
		virtual ~QEvaluatorRemainder(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorAbs: public QEvaluator
{
	public:
		QEvaluatorAbs():QEvaluator(){}
		
		virtual ~QEvaluatorAbs(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorLog: public QEvaluator
{
	public:
		QEvaluatorLog():QEvaluator(){}
		
		virtual ~QEvaluatorLog(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorLog10: public QEvaluator
{
	public:
		QEvaluatorLog10():QEvaluator(){}
		
		virtual ~QEvaluatorLog10(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorInverse: public QEvaluator
{
	public:
		QEvaluatorInverse():QEvaluator(){}
		
		virtual ~QEvaluatorInverse(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorSqrt: public QEvaluator
{
	public:
		QEvaluatorSqrt():QEvaluator(){}
		
		virtual ~QEvaluatorSqrt(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorExp: public QEvaluator
{
	public:
		QEvaluatorExp():QEvaluator(){}
		
		virtual ~QEvaluatorExp(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorTrace: public QEvaluator
{
	public:
		QEvaluatorTrace():QEvaluator(){}
		
		virtual ~QEvaluatorTrace(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorVectProd: public QEvaluator
{
	public:
		QEvaluatorVectProd():QEvaluator(){}
		
		virtual ~QEvaluatorVectProd(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorDotProd: public QEvaluator
{
	public:
		QEvaluatorDotProd():QEvaluator(){}
		
		virtual ~QEvaluatorDotProd(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorDiff3: public QEvaluator
{
	public:
		QEvaluatorDiff3():QEvaluator(){}
		
		virtual ~QEvaluatorDiff3(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorDiff5: public QEvaluator
{
	public:
		QEvaluatorDiff5():QEvaluator(){}
		
		virtual ~QEvaluatorDiff5(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorIntDef: public QEvaluator
{
	public:
		QEvaluatorIntDef():QEvaluator(){}
		
		virtual ~QEvaluatorIntDef(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorIntIndef: public QEvaluator
{
	public:
		QEvaluatorIntIndef():QEvaluator(){}
		
		virtual ~QEvaluatorIntIndef(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorUnits: public QEvaluator
{
	public:
		QEvaluatorUnits():QEvaluator(){}
		
		virtual ~QEvaluatorUnits(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorFills: public QEvaluator
{
	public:
		QEvaluatorFills():QEvaluator(){}
		
		virtual ~QEvaluatorFills(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorAngle360: public QEvaluator
{
	public:
		QEvaluatorAngle360():QEvaluator(){}
		
		virtual ~QEvaluatorAngle360(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorTTtoSec: public QEvaluator
{
	public:
		QEvaluatorTTtoSec():QEvaluator(){}
		
		virtual ~QEvaluatorTTtoSec(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorTIVLtoSec: public QEvaluator
{
	public:
		QEvaluatorTIVLtoSec():QEvaluator(){}
		
		virtual ~QEvaluatorTIVLtoSec(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorTT2_TT1toSec: public QEvaluator
{
	public:
		QEvaluatorTT2_TT1toSec():QEvaluator(){}
		
		virtual ~QEvaluatorTT2_TT1toSec(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorT2_T1toSec: public QEvaluator
{
	public:
		QEvaluatorT2_T1toSec():QEvaluator(){}
		
		virtual ~QEvaluatorT2_T1toSec(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};


class QEvaluatorLinReg: public QEvaluator
{
	public:
		QEvaluatorLinReg():QEvaluator(){}
		
		virtual ~QEvaluatorLinReg(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorAngle180: public QEvaluator
{
	public:
		QEvaluatorAngle180():QEvaluator(){}
		
		virtual ~QEvaluatorAngle180(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorTrig: public QEvaluator
{
	public:
		QEvaluatorTrig(const QCalcOperation& op=QCALC_COS):QEvaluator()
		{
			this->op=op;
		}
		
		virtual ~QEvaluatorTrig(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
		
		QCalcOperation op;
};

class QEvaluatorPlugin: public QEvaluator
{
    public:
        QEvaluatorPlugin(const QString& pluginName);

        virtual ~QEvaluatorPlugin(){delete qtplFile;}
        
        virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
        
        QString pluginName;

        QtplParserFile * qtplFile;

        QString PathAndNameSO;
        QString try_entry_name;
        int num_in;
        int num_out;
};

class QEvaluatorPiecewise: public QEvaluator
{
	public:
		QEvaluatorPiecewise():QEvaluator(){}
		
		virtual ~QEvaluatorPiecewise(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};

class QEvaluatorPwDiv: public QEvaluator
{
	public:
		QEvaluatorPwDiv():QEvaluator(){}
		
		virtual ~QEvaluatorPwDiv(){}
		
		virtual void evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& err, QString& message);
};
