#ifndef QCALC_SCENE_H
#define QCALC_SCENE_H

#include <QGraphicsScene>
#include <QGraphicsSceneMouseEvent>
#include <QMenu>
#include <QGraphicsSceneDragDropEvent>
#include "qcalcGraphicElements.h"

// Handles all the graphic items, their positions and the global status of the calculator (adding something, deleting something...)
class CalcScene: public QGraphicsScene
{
	Q_OBJECT
	
	public:
		CalcScene(QObject* parent=NULL);
		
		virtual ~CalcScene();

        void emitSelectionChanged()
        {
            emit selectionChanged();
        }
		
		QDomDocument toXML(bool onlySelectedItems=false);
		void fromXML(QDomDocument& doc, bool restore=true);
	
		QPointF interactivePosition();

		// Utility to simplify adding connectors after loading/including a session. Makes sure
		// that connectors match underlying virtual connections
		void addMissingConnectors();

		// "Esc" pressed, ends tracking
		void escPressed();
		
		static int currentPage;
		static QList<VCalcItem*> currentItems; // all the items in the calculator

		// Typically for a change of page
		void updateVisibleItems(bool restore=false);
		
		// What I called tracking is acquiring mouse position and drawing crosshairs, for items positioning
		bool tracking;
		bool trackingSuccess;
		bool draggingSelectionRect;
		
	public slots:
		void evaluate();
		void evaluatePage();

        bool saveAs(const QString& path, bool onlySelectedItems=false);
        void open(const QString& path, bool restore=true);
		bool exportAs(const QString& path);
		void deleteSelection();
		
		void selectAll();
		void clearPage();
		void clearAll();

	signals:
		
		void refreshContextWidget();
		void trackingStarted();
		void trackingEnded();
		void currentPageChanged();
		
		// This signal is emitted so that the context widget knows that an input
		// has been added to the calculator, and so a default time line
		// can be created based on it if necessary
		void newInput(const QString&);
		
	protected:
		void setTracking();
		void mousePressEvent(QGraphicsSceneMouseEvent* mouseEvent);
		void mouseMoveEvent(QGraphicsSceneMouseEvent* mouseEvent);
		void mouseReleaseEvent(QGraphicsSceneMouseEvent* mouseEvent);
		void keyPressEvent(QKeyEvent* keyEvent);
		void dragEnterEvent(QGraphicsSceneDragDropEvent * event);
		void dragMoveEvent(QGraphicsSceneDragDropEvent * event);
		void dropEvent(QGraphicsSceneDragDropEvent * event);

		void getItemsBoundaries(bool onlySelectedItems, double& minX, double& minY, double& maxX, double& maxY);

        void selectItem(QGraphicsItem*);
		
		// All of this is status management, so that the scene remembers what it's doing
		Connector* currentConnector;
		bool movingConnector;
		bool rotoZooming;
		CalcItem* currentItem;
		QPointF reference;
		QMatrix referenceMatrix;
		bool closing;
		QGraphicsRectItem* selectionRect;
		QGraphicsLineItem* tracking1;
		QGraphicsLineItem* tracking2;

		QPointF trackingPosition;
};

#endif
