#ifndef QSAS_QDU_H_
#define QSAS_QDU_H_

#define REMOVE_GAP "remove gaps"
#define ZERO_FILL_GAP "zero-fill"
#define FILL_GAP "fill value"
#define NEAREST_NEIGHBOUR_GAP "nearest neighbour"
#define NEW_NAME "new name"
#define LINEAR_INTERP_GAP "linear"
#define LINEAR "linear"
#define BOXCAR "boxcar"

#define QDUTIL_OK 0
#define QDUTIL_ERR 1
#define QDUTIL_WARN 2

#include <stdlib.h>
#include <QMainWindow>
#include "Qdos.h"
#include "DvObj.h"
#include "Xrefs.h"
#include "qprofile.h"
#include <qcolordialog.h>
#include <qcheckbox.h>

#if QT_VERSION >= 0x040000
#include <QLayout>
#endif

#include <QDomDocument>

namespace QSAS
{

extern SequenceMask UNITmsk;

   void QuMultiJoin( QdObjectSeq_var      join_obj_seq,
					 QdObject_var         join_pts,
					 QdObjectSeq_var      joined_obj_seq,
					 vector <double>	  &gap_or_width,
				  	 QdStringSeq_var	  gap_method,
					 QdStringSeq_var	  join_method);
   
   QdObject_var QuLinearJoin(QdObject_var obj, QdObject_var target, double gap = -1., string gap_opt=string(LINEAR_INTERP_GAP), bool doGaps=true, SequenceMask &Gmsk=UNITmsk);
   QdRScalarSeq_var QuLinearJoinScalar(SequenceMask &msk, QdRScalarSeq_var Sobj, QdRScalarSeq_var sso, QdRScalarSeq_var ss, double gap, String gap_opt, double fillval);
   QdRMatrixSeq_var QuLinearJoinMatrix(SequenceMask &msk, QdRMatrixSeq_var Mobj, QdRScalarSeq_var sso, QdRScalarSeq_var ss, double gap, String gap_opt, double fillval);
   double LinearInterpScalar(QdRScalarSeq_var Sobj, QdRScalarSeq_var sso, int i_obj, double fillval, double target);
   void LinearInterpMatrix( QDOS::Matrix<double> &result, QdRMatrixSeq_var Mobj, QdRScalarSeq_var sso, int i_obj, double fillval, double target);
   void QuJoinXrefs(QdObject_var preJoinObj, QdObject_var joinedObj, double gap_width, bool useBoxcar=false);

   QdObject_var QuBoxcarJoin(QdObject_var obj, QdObject_var target, double width = -1., string gap_opt=string(ZERO_FILL_GAP), bool doGaps=true, SequenceMask &Gmsk=UNITmsk);
   QdRScalarSeq_var QuBoxcarJoinScalar(SequenceMask &msk, QdRScalarSeq_var Sobj, QdRScalarSeq_var sso, QdRScalarSeq_var ss, double halfWidth, String gap_opt, double fillval);
   QdRMatrixSeq_var QuBoxcarJoinMatrix(SequenceMask &msk, QdRMatrixSeq_var Mobj, QdRScalarSeq_var sso, QdRScalarSeq_var ss, double halfWidth, String gap_opt, double fillval);
   bool BoxcarInterpScalar(QDOS::Scalar<double> &result, QdRScalarSeq_var Sobj, QdRScalarSeq_var sso, int i_obj, double halfWidth, double fillval, double target);
   bool BoxcarInterpMatrix( QDOS::Matrix<double> &result, QdRMatrixSeq_var Mobj, QdRScalarSeq_var sso, int i_obj, double halfWidth, double fillval, double target);
   
   QdObject_var QuRemoveGaps(QdObject_var obj, SequenceMask &msk, QdObject_var target);

} // end namespace QSAS

using namespace std;

typedef vector <int> dimTriple;
typedef vector <dimTriple> dimTripleStack;

#define EVENT_0_STR " < 0 >"
#define R_STR_CMP " [ r (mag) ]"
#define X_STR_CMP " [ x ]"
#define Y_STR_CMP " [ y ]"
#define Z_STR_CMP " [ z ]"
#define Ld_STR_CMP " [ Lat (deg) ]"
#define Lr_STR_CMP " [ Lat (rad) ]"
#define Td_STR_CMP " [ theta (deg) ]"
#define Tr_STR_CMP " [ theta (rad) ]"
#define Pd_STR_CMP " [ phi (deg) ]"
#define Pr_STR_CMP " [ phi (rad) ]"
#define T_STR_START " [ Start Time ]"
#define T_STR_CENTRE " [ Centre Time ]"
#define T_STR_END " [ End Time ]"
#define T_STR_0_START " <0> [ Start Time ]"

typedef enum QguComp_e
{
  UNKNOWN_COMP=-1,
  WL_OBJ=0,
  R_COMP=1,
  X_COMP=2,
  Y_COMP=3,
  Z_COMP=4,
  LAT_DEG_COMP=5,
  THETA_DEG_COMP=6,
  PHI_DEG_COMP=7,
  LAT_RAD_COMP=8,
  THETA_RAD_COMP=9,
  PHI_RAD_COMP=10,
  ARRAY_COMP=11,
  XREF_COMP=12,
  T_START_COMP=13,
  T_CENTRE_COMP=14,
  T_END_COMP=15,
  XREF_OBJ=16,
  SLICE_COMP=17,
  T_0_START_COMP=18,
} QguComp;


QDomElement createTextXMLNode(QDomDocument& doc, const QString& parentName, const QString&);
QDomElement createTextXMLNode(QDomDocument& doc, const QString& parentName, const std::string&);
QDomElement createTextXMLNode(QDomDocument& doc, const QString& parentName, const char*);
QDomElement createTextXMLNode(QDomDocument& doc, const QString& parentName, const double);
QDomElement createTextXMLNode(QDomDocument& doc, const QString& parentName, const int);
QDomElement createTextXMLNode(QDomDocument& doc, const QString& parentName, const bool);
bool getXMLNodeText(QDomElement& el, const QString& tag, QString& val);

QdRScalarSeq_var QuRegression(QdObject_var dobj, QdObject_var dep0=NULL);
QdString_var QuSIC_ratio(QdString_var SI1, QdString_var SI2);
char * QuNewSTR(const char * in_str);
int QuAreUnitsSame(QdObject_var ptr1, QdObject_var ptr2);
int QuAreFramesSame(QdObject_var ptr1, QdObject_var ptr2);
QString encode(const QString& input);
QString decode(const QString& input);
QString QuGetDefaultComp(const QString& WL, const QString& compDef);
int QuEpochType(QdObject_var obj);
void QuReplaceTokens(QString &newTxt, QString &oldTxt);
bool QuIsMatch(double value, double fill);
QCheckBox * checkBadData(QWidget *parent=0);
QCheckBox * checkFills(QWidget *parent=0);
bool QuSlotDebug();
string QuCommonLabl(string str1, string str2);
QdObject_var QuGetObjByName(const QString &name);
QdObject_var QuTakeSlice(vector <string> objectNameList, QdObject_var dobj, dimTriple sliceTriple, bool getInterval);
QdObject_var QuGetSubset(QdObject_var dobj, int recStart, int recEnd);
QdObject_var QuGetSubset(QdObject_var obj, QdTimeInterval_var tint);
QString baseObjectText(QString);
QguComp compEntry(const char*,const char*);
QguComp compEntry(const char*);
QString compText(QString);
QString compText(QString, const dimTriple&);
QString getXrefName(QString);
dimTriple getDimTriple(QString &compTxt);
dimTriple getSliceTriple(const QString &compTxt);
dimTripleStack getDimTripleStack(QString &compTxt);
QdObject_var QuGetObjByNameAndComp(const QString& name);
QdObject_var QuGetIntervalsByNameAndComp(const QString& name);
QdObject_var QuGetTimeByNameAndComp(const QString& name);
void QuSetTxtAttr(const char *attrName, const char *value, QdObject_var obj);
bool QuIsBadData(double value, double fill);
void applyMaskToXrefs(int oldSeqLen, QdObject_var ms_out, SequenceMask msk);
void applyAliasToXrefs(int oldSeqLen, QdObject_var dobj, int nElem);
QdObject_var QuAliasArray(QdObject_var doa, int indx, int fromK, int toK );
QdObject_var QuCleanObject(QdObject_var dobj);
void QuMsgBar(const char * text, bool error);
void QuMsgBar(const char* text1, const char* text2, bool error);
void QuSetOK();
int QuStrcpy(char * To, const char * From, int ToLen);
int QuStrcat(char * To, const char * From, int ToLen);
string QuGetAttrCondensed(const char *attrName, QdObject_var obj);
string QuGetAttrText(const char* attrName, QdObject_var obj);
string QuToUpper(const char *InStr);
string QuToUpper(string InStr);
void QuSetFrame(QdObject_var obj, const char *txt, const char*frm=0, const char*rep=0);
string QuGetFrameAttr(QdObject_var obj);
string QuGetFrameAttr(QdObject_var obj1, QdObject_var obj2);
string QuGetOrder(QdObject_var obj);
string QuGetFrame(QdObject_var obj);
string QuGetToFrame(QdObject_var obj);
string QuGetRep(QdObject_var obj);
bool QuIsThreeVector(QdObject_var obj);
bool QuIsScalar(QdObject_var obj);
string QuGetVectorRep(QdObject_var obj);
QdObject_var QuGetVectorXYZ(QdObject_var dov);
void QuSetRep(const char* rep, QdObject_var do_out);
QdObject_var QuAngleMod(QdObject_var depObj, int angMax, const char * rad_deg);
bool QuIsAngleDegrees(QdObject_var dobj);
bool QuIsAngle(QdObject_var dobj, bool &isDeg);
bool QuIsPhiAngle(QdObject_var dobj, bool isDeg, int index, double &minVal, int &kStart, int &kEnd);
string QuAngleUnits(QdObject_var dobj);
double ConvertThetaRad(double theta);
double ConvertThetaDeg(double theta);
bool QuIsTSRegular(QdTimeSeq_var tt, double *spacing);
bool QuIsSSRegular(QdRScalarSeq_var ss, double *spacing);
string QuSanitizeName ( const char * oldName);
bool QuIsJoined(QdObject_var obj1, QdObject_var target);
bool QuAreJoined(QdObject_var obj1, QdObject_var obj2);
int QuArrayDims(QdObject_var obj);
bool copyXref(const char * name, QdObject_var from_ptr, QdObject_var to_ptr);
bool copyDimXrefs( QdObject_var from_ptr, QdObject_var to_ptr);
void innerProdDimXrefs( QdObject_var A_ptr,QdObject_var B_ptr, QdObject_var to_ptr);
QdObject_var QuGetArrayComponent(QdObject_var doa, dimTripleStack dimStack, string arrComp_xref="");
QdObject_var QuGetVectorComponent(QdObject_var dov, const char *vcomp);
QdObject_var QuGetTimeComponent(QdObject_var dot, const char *tcomp);
QdObject_var QuSliceArray(QdObject_var doa, int indx, int entry );
QdObject_var QuSumDown(QdRMatrix &Mat_in, int indx, int fromK, int toK, bool average, double fill );
QdObject_var QuSumExterior(QdRMatrix &Mat_in, int indx, int fromK, int toK, bool average, double fill );
QdObject_var QuSumArray(QdObject_var doa, int indx, int from, int to, bool average );
QdObject_var QuSubsetArray(QdObject_var doa, int indx, int from, int to );
bool QuStrCmp(const char *A, const char *B);
QColor QuGetColour(QColor inC, bool *newChoice, QWidget * parent);
void QuGetBounds(QdTimeSeq_var tt, int &recL, int &recU, const QmjdTime& t);
bool QuGetIvlRecBound(QdTimeSeq_var tt, int &recStart, int &recEnd, const QmjdTime& tStart, const QmjdTime& tEnd);
QdObject_var QuSetCompFromObjText(const QString&, QString&);
QdTimeInterval_var QuGetIvlFromSlice(QdObject_var, const dimTriple&);
bool QuGetSliceFromIvl(QdObject_var,QdTimeInterval_var,dimTriple&);
QdObject_var get_Depend_0(const QdObject_var& obj);
QdObject_var getcopy_Depend_0(const QdObject_var& obj);
QdTimeSeq_var get_timetags(const QdObject_var& obj);
QdTimeInterval_var get_timeinterval(const QdObject_var& obj);
QdTimeSeq_var get_timetags_inrange(const QdTimeSeq_var& ts_in, const QdTimeInterval_var& tint);
double get_time_spacing(const QdObject_var& obj);
double get_scalar_spacing(const QdObject_var& obj);
bool get_ScalarInterval(const QdObject_var& obj, double &start, double &end);
void set_timetags(QdObject_var obj, QdTimeSeq_var ts);
int num_timetags_inrange(const QdTimeSeq_var& ts, const QdTimeInterval_var& tint);
bool is_empty(const QdObject_var& obj);
void QuNow(const char * txt, bool reset=false);
int QuLength(QdObject_var obj);
bool QuSet_Array(int ndata, double *floatArray, QdObject_var obj, QdTime_var offset=0);
QdObject_var makeSequence(QdObject_var inObj, int seqSize );
bool QuSetObjByName(QdObject_var ToSetobj, QString name);
void QuRenameObject(QString oldName, QString newName);
void QuDeleteObject(const QString& name);
void QuReplaceObject(const QString& name, QdObject_var new_obj);
QdObject_var QuDuplicateObject(QString &name);
QString QuNameOnly(QString nameContext);
QString QuTextInputDialog(const char *caption, const char *label, const char * defText, bool *OK, QWidget * parent);
double getSpacingTolerance(QdRScalarSeq_var rseq);
bool QuGetDataRange(QdObject_var obj, double &min, double &max);
bool QuIsBad(double value, double fill);
void setRestoreVersion(const char *version);
QMainWindow* getMainWindow();
QdObject_var QuMergeObjects(QdObject_var &dobj1, QdObject_var &dobj2);
QdObject_var QuMergeVar(int nMask, vector <int> &pairMask, QdObject_var &dobj1, QdObject_var &dobj2);
QdObject_var QuStripDuplicates(QdObject_var dobj, double tolerence);
QdObject_var QuMakeMonotonic(QdObject_var dobj);
QdObject_var applyOrder( vector <int> &order, QdObject_var dobj);
void QuParseSIC(string &str, double &multiplier);

void cleanWLtext(string &text);
string QuSIConv_inverse(QdObject_var in1_ptr);
string QuSIConv_product(QdObject_var in1_ptr, QdObject_var in2_ptr);
string QuSIConv_power(QdObject_var in1_ptr, double constB);

inline bool isFill(double val, double fill){
	static double epsilon = 10.*DBL_EPSILON;
	if( fabs((val - fill) / (val + DBL_EPSILON)) < epsilon) return true;
	return false;
}

inline bool notFill(double val, double fill){
	static double epsilon = 10.*DBL_EPSILON;
	if( fabs((val - fill) / (val + DBL_EPSILON)) > epsilon) return true;
	return false;
}
	
DvObject_var toDVOS(QdObject_var &qobj, int id=0);
void addXrefsToDVOS(QdObject_var &qobj, DvObject_var &dobj);

QdObject_var toQDOS(DvObject_var &dobj);
void addXrefsToQDOS(DvObject_var &dobj, QdObject_var &qobj);

					 
# endif // QSAS_QDU_H_
