#ifndef QSETTINGS_H
#define QSETTINGS_H

#include <iostream>
#include <fstream>
#include <vector>
#include <stdlib.h>

#include <QString>

#include "ColourTable.h"

#ifndef WINDOWS
#define DEFAULT_FONT_SIZE 13
#else
#define DEFAULT_FONT_SIZE 11
#endif

#define DEFAULT_DECIMAL_SEC 3
#define DEFAULT_GREGORIAN_START "1582-10-15"

QString addTokens(const QString& in);
QString removeTokens(const QString& in);

class QProfile
{
	public:
		QProfile(QString name="Default");
		
		QProfile(QProfile* p);
		
		QString getName(){return name;}
		
		void setName(const QString& name) {this->name=name;}
		
		// Paths Settings
		
		QString SaveRestorePath;
		QString UserLibPath;
		QString ConfigPath;
		QString DataPath;
		QString FilterBankPath;
		QString BlackBoxPath;
		//Plot Settings
		
		QString Vector3D;
		QString Array3Dto0D;
		QString Array3Dto1D;
		QString Array3Dto2D;
		QString PageSize;
		QString PageOrientation;
		int PageDPI;
		double PageWidth; // inches
		double PageHeight; // inches
		QString TimeRep;
		bool UseKPrinter;
		
		QString SansFont;
		QString SerifFont;
		QString MonotypeFont;
		QString DecorativeFont;
		
		bool VectoriseSVGFonts;
		
// 		QString ColourTable;
		
		double LineWidth;
		double BaseFontSize;

        double gapTolerance;
        int gapToleranceType;
        int axisScaleType;

        ColourTablePalette palette;
		
		// UI
		
		int Height;
		int QpuiHeight;
		int QpuiWidth;
		int fontSize;
		int secDecimal;
		QString gregorian_start;
		int GUImargins;
		int slotLR;
		int timeIntervalButton; //0: both, 1: right click only
		
		// MISC
		
		bool Debug;
		bool QrtnDebug;
		bool dbgWL;
		
	protected:
		
		QString name;
};

class QProfilesList
{
	public:
		QProfilesList();
		
		~QProfilesList();
		
		bool load();
		bool save();
		QProfile * current();
		QProfile * fromName(const QString& name);
		bool setCurrent(const QString& name);
		void names(std::vector<QString>& names);
		bool add(QProfile * prof);
		bool remove(const QString& name);
		
	protected:
		std::vector<QProfile*> presets;
		QString currentName;
		QProfile * defaultProfile;
};

extern QProfilesList * QSAS_profiles;

#endif
