/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include "qar.h"


int  QarTestConformal(QdObject_var in1_ptr, 
                      QdObject_var in2_ptr){


 if(in1_ptr.is_nil() || in2_ptr.is_nil() ) return QAR_ERR;
 
 
// if one item is constant don't need to join

 
 if( in1_ptr->sequence_size() != 1 && in2_ptr->sequence_size() !=1 ){
   
    if (QarTestJoined(in1_ptr, in2_ptr) != QAR_OK)   return QAR_ERR;
   
 }
 // only get here if operation is possible
 
  return QAR_OK;

} // end QarTestConformal

// ***************************************************

int  QarTestConformalForVecProds(QdObject_var in1_ptr, 
                                 QdObject_var in2_ptr){ 
  // NOTE that test on joined TS is already done in TestConformal
   
  if (!QuIsThreeVector(in1_ptr) || !QuIsThreeVector(in2_ptr)) {
      return QAR_ERR;
  }          

   // only get here if operation is possible
   return QarAreFramesSame(in1_ptr, in2_ptr) ;
                     
} // end QarTestConformalForVecProds


// *************************************************************************

int QarTestJoined (QdObject_var do1, 
                   QdObject_var do2){

	// always accept if one object is not a sequence
  if(do1->sequence_size() == 1 || do2->sequence_size() == 1 ) return QAR_OK;

	// if sequences they must be the same length
  if(do1->sequence_size() != do2->sequence_size()) return QAR_ERR;

  // both are sequences so must be joined if they have time tags
 
  QdObject_var D0_1 =  get_Depend_0(do1);
  QdObject_var D0_2 =  get_Depend_0(do2);
 
  // if one Depend_0 is not sequence only care about length
 
  if(D0_1.is_nil() || D0_2.is_nil()) return QAR_OK;
  
  
  if (D0_1->sequence_size() != D0_2->sequence_size()) return QAR_ERR;
  
  QdTimeSeq_var tt1 = QdTimeSeq_var::narrow(D0_1);
  QdTimeSeq_var tt2 = QdTimeSeq_var::narrow(D0_2);
  
  if(!tt1.is_nil() && !tt2.is_nil()){
  	// same object is OK
  	if( (*tt1) == (*tt2) ) return QAR_OK;
	
  	for(int i=0; i<tt1->sequence_size(); i++){
    	// allow for 1 msec jitter in double rep for epoch
    	if( tt1[i] != tt2[i] ) return QAR_ERR;
  	}
  }
  else {
	QdRScalarSeq_var ss1 = QdRScalarSeq_var::narrow(D0_1);
  	QdRScalarSeq_var ss2 = QdRScalarSeq_var::narrow(D0_2);
	if( (*ss1) == (*ss2) ) return QAR_OK;
	
  	if(!ss1.is_nil() && !ss2.is_nil()){
  		for(int i=0; i<ss1->sequence_size(); i++){
    		// allow for 1 msec jitter in double rep for epoch
    		if( ss1[i] != ss2[i] ) return QAR_ERR;
  		}
	}
	else return QAR_ERR; // not both tt or scalar sequences
  }
  
  return QAR_OK;

} // end QarTestJoined


// *************************************************************************

int QarAreFramesSame(QdObject_var ptr1, 
                     QdObject_var ptr2){
    return QuAreFramesSame(ptr1, ptr2);
  
} // end QarAreFramesSame

// *************************************************************************

bool QarSameBaseSI(const char *SIc1, const char *SIc2){
 std::string base_str1;
 std::string base_str2;
 char * ptr1, * ptr2;
 
  base_str1 = DvUnitGetBaseSI(SIc1); 
           
  base_str2 = DvUnitGetBaseSI(SIc2);
  
  ptr1 = (char*)strstr(base_str1.c_str(), ">");
  ptr2 = (char*)strstr(base_str2.c_str(), ">");

  if(ptr1 == NULL || ptr2 == NULL || strcmp(ptr1, ptr2) !=0 ) return false;
  
  return true;
  
}

int QarAreUnitsSame(QdObject_var ptr1, 
                    QdObject_var ptr2){

 string SIc1, SIc2;
 string base_str1;
 string base_str2;

 int status;

  status = QAR_OK;
  
  SIc1 = QuGetAttrText("SI_conversion", ptr1);
  if (SIc1.empty())  status = QAR_WARN;

  SIc2 = QuGetAttrText("SI_conversion", ptr2);
  if (SIc2.empty() )  status = QAR_WARN;

  if(status != QAR_WARN){
    base_str1 = DvUnitGetBaseSI(SIc1.c_str());  
    base_str2 = DvUnitGetBaseSI(SIc2.c_str());

    if(base_str1 != base_str2) status = QAR_ERR;
  
  }
   
  return status;
  
} // end QarAreUnitsSame


// *************************************************************************

double QarConvFactor(const char * SIc){

  char * SIcopy = QplugNewSTR(SIc);
  char * ptr;
  double value=0.0;
  ptr = strstr(SIcopy, ">");
  if (ptr != NULL){
    ptr[0] = '\0';
    value = (double) Qar_atof(SIcopy);
  }
  
  free(SIcopy);
  return value;
  
}

double Qar_atof(const char * vtxt){

  double value=0.0;
  char *ptr = (char*)strstr(vtxt, "10^");
  if (ptr != NULL){
    double power = atof(&(ptr[3]));
    ptr[0] = '\0'; 
    value = (double) atof(vtxt);
    if(value < 1e-13) value = 1.0; // if vtxt of form "10^6" value is zero since there is no mantissa
    value *= pow(10., power);
  }
  else
      value = (double) atof(vtxt);

  return value;
  
}


// *************************************************************************

QdObject_var QarConvertUnitsTo(QdObject_var ptr, const char * SIcTarget, const char* newUnits){

 std::string SIc;
 char * SI_ptr;
 QdObject_var newObj;
 double conversion;
 double convTarget;
 
 QdObject_var xref;
 char temp_string[30];
 double val_double;
 QString val_string;
 
  SIc = string(QuGetAttrText("SI_conversion", ptr));
  if (strcmp(SIc.c_str(), "") == 0)  return newObj;  // nil object return
  if( !QarSameBaseSI(SIc.c_str(), SIcTarget) ) {
    QuMsgBar("input object units cannot convert to ", SIcTarget, true);
    return newObj;
  }

  convTarget = QarConvFactor(SIcTarget);
  
   if( convTarget < 1.0e-60 ) {
    QuMsgBar("target units too small ", SIcTarget, true);
    return newObj;
  }
 
  // Conversion factor
  conversion = QarConvFactor(SIc.c_str());
  conversion  = conversion / convTarget;
  QdRScalar_var conv_var = new QdRScalar(conversion);
  newObj = ptr * conv_var;

  newObj->copy_xrefs_from(ptr);
  newObj->change_xref("SI_conversion", new QdString(SIcTarget));
  
  // Now convert the parameters and xrefs that have to be converted
  try
  {
      xref=newObj->get_xref(DELTA_PLUS);
      xref=QarConvertUnitsTo(xref, SIcTarget, newUnits);
      newObj->change_xref(DELTA_PLUS, xref);
  }
  catch(Exception& e)
  {;}
  
  try
  {
      xref=newObj->get_xref(DELTA_MINUS);
      xref=QarConvertUnitsTo(xref, SIcTarget, newUnits);
      newObj->change_xref(DELTA_MINUS, xref);
  }
  catch(Exception& e)
  {;}
  
  try
  {
      xref=newObj->get_xref(SCALEMIN);
      xref=QarConvertUnitsTo(xref, SIcTarget, newUnits);
      newObj->change_xref(SCALEMIN, xref);
  }
  catch(Exception& e)
  {;}
  
  try
  {
      xref=newObj->get_xref(SCALEMAX);
      xref=QarConvertUnitsTo(xref, SIcTarget, newUnits);
      newObj->change_xref(SCALEMAX, xref);
  }
  catch(Exception& e)
  {;}
  
  try
  {
      xref=newObj->get_xref(VALIDMIN);
      xref=QarConvertUnitsTo(xref, SIcTarget, newUnits);
      newObj->change_xref(VALIDMIN, xref);
  }
  catch(Exception& e)
  {;}
  
  try
  {
      xref=newObj->get_xref(VALIDMAX);
      xref=QarConvertUnitsTo(xref, SIcTarget, newUnits);
      newObj->change_xref(VALIDMAX, xref);
  }
  catch(Exception& e)
  {;}
  
  
  //int index=QString(SIcTarget).find('>');
  if(strlen(newUnits)>0)
  {
      //QString unit=QString(SIcTarget).right(QString(SIcTarget).length()-index-1);
      QuSetTxtAttr("UNITS", newUnits, newObj);
  }
  
  return newObj;
  
} // end QarConvertUnitsTo

// *************************************************************************

 string QarFrame_product(QdObject_var in1_ptr, 
                        QdObject_var in2_ptr){

 string frame;
 string frame_scalar = "scalar>na";
 string unk_frame =  "vector>_??_??";
 
 // This is used for non-vector operation multiply
 // input may be rotation matrix 
 
 bool isRotn1 = false;
 string fr = QuGetOrder(in1_ptr);
 if (QuToUpper(fr).find("ROT") != string::npos) isRotn1 = true;
 
 bool isRotn2 = false;
 fr = QuGetOrder(in2_ptr);
 if (QuToUpper(fr).find("ROT") != string::npos) isRotn2 = true;
 
 // if vector  times scalar use frame of vector...
  
  bool vec_1=QuIsThreeVector(in1_ptr);
  bool vec_2=QuIsThreeVector(in2_ptr);

  if(vec_1 && vec_2)  // both 3 vectors
    frame = frame_scalar;
  else if(vec_1 && isRotn2){
    frame = unk_frame; 
    QuMsgBar("CAUTION: ROTATION MATRIX should pre-multiply", true);
  }
  else if(vec_1)
    frame = QuGetFrameAttr( in1_ptr);
  else if(vec_2 && isRotn1)
    frame = QuGetAttrText("ToFrame", in1_ptr);
  else if(vec_2)
    frame = QuGetFrameAttr( in2_ptr);
  else if(isRotn1 && isRotn2){
    frame = QuGetFrameAttr( in2_ptr);           // R2 is applied 1st to vector, so from Frame from R2
    // ToFrame will be from R1 
  }
  else  // scalar
    frame = QuGetFrameAttr(in1_ptr, in2_ptr); // will use frame if found else scalar
 
  
  return frame;
  
} // end QarFrame_product

// ***************************************************
string QarUNITS_product(QdObject_var in1_ptr, 
                        QdObject_var in2_ptr){

string units;
 
 // note QuGetAttrTextn returns a valid empty string if xref not found
 
  units = QuGetAttrText("UNITS", in1_ptr);
  units += " ";
  units += QuGetAttrText("UNITS", in2_ptr);

  return units;
  
} // end QarUNITS_product

// ***************************************************

string QarUNITS_power(QdObject_var in1_ptr, double constB){

string units;
char  exponent[20];

   sprintf(&(exponent[0]),"%g",(double) constB);

   units = "(";
   units += QuGetAttrText("UNITS", in1_ptr);
   units += ")^";
   units += exponent;
 
  return units;
   
} // end QarUNITS_power

// *************************************************************************


string QarSIConv_product(QdObject_var in1_ptr, 
                         QdObject_var in2_ptr){

 string siconv1, siconv2, siconv;
 char base_str[20];
 float value1, value2, value;
 size_t ptr1, ptr2;
 
 string null_unit = "1 > ()";

  siconv1 = QuGetAttrText("SI_conversion", in1_ptr);
  if( siconv1.empty()) {  
    siconv1 = null_unit;
  }
  
  
  siconv2 = QuGetAttrText("SI_conversion", in2_ptr);
  if(siconv2.empty()) {
  	siconv2 = null_unit;
  }

  ptr1 = siconv1.find(">");
  if(ptr1 == string::npos) return string("");
  
  value1 = atof(siconv1.c_str());
  ptr1 ++;
  
  ptr2 = siconv2.find(">");
  if(ptr2 == string::npos) return string("");
  
  value2 = atof(siconv2.c_str());
  ptr2 ++;
  
  value = value1 * value2;
   
  sprintf(base_str, "%16g >", value);

  siconv = string(&(base_str[0])); 
  siconv += siconv1.substr(ptr1);  
  siconv += " "; 
  siconv += siconv2.substr(ptr2);
    
  siconv =  DvUnitGetTidySI(siconv.c_str());  
  
  return siconv;
  
} // end QarSIConv_product

// *************************************************************************

string QarSIConv_inverse(QdObject_var in1_ptr){

 string siconv;

  siconv = QuGetAttrText("SI_conversion", in1_ptr);
     
  siconv =  DvUnitGetPowerSI(siconv.c_str(), -1);
    
  return siconv;
  
} // end QarSIConv_inverse

// *************************************************************************

string QarSIConv_power(QdObject_var in1_ptr, double constB){

 string siconv;

  siconv = QuGetAttrText("SI_conversion", in1_ptr);

  siconv =  DvUnitGetPowerSI(siconv.c_str(), constB);
    
  return siconv;
  
} // end QarSIConv_power

// *************************************************************************

int QarHasUnits(QdObject_var in_ptr){

 int flag;
 string siconv;
 
  siconv = QuGetAttrText("SI_conversion", in_ptr);
  if (siconv.empty() ) return 0;
  
  flag = DvUnitHasUnits (siconv.c_str());
  
  return flag;
}


// *************************************************************************


