#include "qOperators.h"

QdObject_var QdVecProd( QString& rqstrErrorMsg, QdObject_var qdOperand1, QdObject_var qdOperand2 )
{

    QdObject_var  qdResult;

    if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
    {
        // The object will be empty - this should never happen
	// The qdResult object is NULL at this point
        return ( qdResult );
    }

 	// Test validity of inputs...
	int nQarStatus =  QarTestConformal( qdOperand1, qdOperand2 );
	if ( nQarStatus == QAR_ERR )
	{
		QcalcAppendTextDisplay( "Sequence size or join error: Notify csc_support" );
		return qdResult;
	}
	
     if ( ! QuIsThreeVector(qdOperand1) )  QcalcAppendTextDisplay( "Warning: Vect Prod: Input A may not be true 3-vector");
   
     if ( ! QuIsThreeVector(qdOperand2) )  QcalcAppendTextDisplay( "Warning: Vect Prod: Input B may not be true 3-vector");
	
	int frameOK = QarAreFramesSame(qdOperand1, qdOperand2 );
     if(frameOK == QAR_WARN) {
		QcalcAppendTextDisplay( "Warning: Vect Prod: Cannot confirm vexctor frame");
	}
	else if(frameOK == QAR_ERR){
		QcalcAppendTextDisplay( "Error: Vect Prod: Frames differ");
		return ( qdResult );
	}
	
	
	string attrPtr;

	// do arithmetic...
	try
	{
		qdResult = outer( qdOperand1, qdOperand2 );
	}
	catch( Exception &e )
	{
		QcalcAppendTextDisplay ( e.msg().c_str() );
		QcalcAppendTextDisplay( "The operation Vect Prod is not yet supported between these input types");
		rqstrErrorMsg = QString( "The operation Vect Prod is not yet supported between these input types");
		return ( qdResult );
	}

	if ( qdResult.is_nil() )
	{
		QcalcAppendTextDisplay( "Arithmetic>> Not valid for these input types" );
		rqstrErrorMsg = QString( "Arithmetic>> Not valid for these input types" );
		return ( qdResult );
	}


	// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
	QdObject_var qd0 = get_Depend_0( qdOperand1 );
	if ( qd0.is_nil() ) qd0 = get_Depend_0( qdOperand2 );
	if ( !qd0.is_nil() ) qdResult->change_xref(DEPEND_0 , qd0 );

	// Set QSAS essential attriutes
	// SI_conversion
	attrPtr = QarSIConv_product( qdOperand1, qdOperand2 );
	if ( !attrPtr.empty() )
	{
		QuSetTxtAttr( "SI_conversion", attrPtr.c_str(), qdResult );
	}


	// Frame
	QuSetFrame( qdResult, QuGetFrameAttr(  qdOperand1, qdOperand2 ).c_str() );

	// UNITS
	attrPtr = QarUNITS_product( qdOperand1, qdOperand2 );
	if ( !attrPtr.empty() )
	{
		QuSetTxtAttr("UNITS", attrPtr.c_str(), qdResult);
	}

    return ( qdResult );
}

