


#include "qOperators.h"

string AxisLabelMakeStem ( QdObject_var data_obj )
{
	string   stem_ptr;

	//  For the stem, use LABLAXIS attribute if it exists, else try for
	//  LABL_PTR (may exist for vector data objects) and perform string
	//  comparision on its entries to synthesise a stem from common parts.
	//  If neither LABLAXIS nor LABL_PTR_1 exists, try for FIELDNAM attribute,
	//  and if that is absent finally fall back to data object name itself.


	//  first use LABLAXIS attribute if it exists

	stem_ptr = QuGetAttrText( "LABLAXIS", data_obj );

	if ( stem_ptr.empty() )
	{
		stem_ptr = QuGetAttrText("FIELDNAM", data_obj);

		// fall back to using FIELDNAM attribute if it exists

	}

	return stem_ptr;
}


QdObject_var QdComponent( QString& rqstrErrorMsg, QdObject_var qdOperand1, QdObject_var qdOperand2 )
{
	QdObject_var   qdResult;
	QdRScalar_var  componentdo;
	int            icomponent;
	string         fieldnam;                  // attributes
	string         siconversion;
	string         units;
	string         lablaxis;
	string         stem;
	double         hs = 0. , hsp = 0.;
	int            ndata_in = 0;                                  /* number of data points  */
													               /* array new'ed below  */

	if ( qdOperand1.is_nil() )
    {
        QcalcAppendTextDisplay ( "VectorTSDecomp>> ERROR - Operand1 is NULL" );
		rqstrErrorMsg = QString( "VectorTSDecomp>> ERROR - Operand1 is NULL" );
	    return ( qdResult );
    }

	if ( qdOperand2.is_nil() )
    {
     	QcalcAppendTextDisplay ( "VectorTSDecomp>> ERROR - Operand2 is NULL" );
		rqstrErrorMsg = QString( "VectorTSDecomp>> ERROR - Operand2 is NULL" );
        return ( qdResult );
    }


	if ( !QuIsThreeVector( qdOperand1 ) )
	{
		QcalcAppendTextDisplay ( "VectorTSDecomp>> ERROR - Operand1 not a Vector Time Series!" );
		rqstrErrorMsg = QString( "VectorTSDecomp>> ERROR - Operand1 not a Vector Time Series!" );
		return ( qdResult );
	}

	ndata_in    = qdOperand1->sequence_size();
	componentdo = QdRScalar_var::narrow( qdOperand2 );
	if ( componentdo.is_nil() )
	{
		QcalcAppendTextDisplay ( "VectorTSDecomp>> ERROR - Operand2 not an integer flag!" );
		rqstrErrorMsg = QString( "VectorTSDecomp>> ERROR - Operand2 not an integer flag!" );
		return ( qdResult );
	}

	icomponent = ( int )( *componentdo );

	stem = AxisLabelMakeStem( qdOperand1 );

	fieldnam = QuGetAttrText( "FIELDNAM", qdOperand1 );

	switch( icomponent )
	{
		case( 0 ):            // x component
			units = QuGetAttrText( "UNITS", qdOperand1 );

			siconversion = QuGetAttrText( "SI_conversion", qdOperand1 );
			if ( siconversion.empty() )
			{
				siconversion = "1>(unknown)";     //  empty if fail
			}

			lablaxis = stem;
			lablaxis += "_x";
			fieldnam += "_x" ;
			qdResult = QuGetVectorComponent( qdOperand1, "x" );
			break;

		case( 1 ):            // y component
			units = QuGetAttrText( "UNITS", qdOperand1 );

			siconversion = QuGetAttrText( "SI_conversion", qdOperand1 );
			if ( siconversion.empty() )
			{                                            
				siconversion = "1>(unknown)";     //  empty if fail
			}      

			lablaxis = stem;
			lablaxis += "_y";
			fieldnam += "_y" ;
			qdResult = QuGetVectorComponent( qdOperand1, "y" );
			break;

		case( 2 ):            // z component
			units = QuGetAttrText( "UNITS", qdOperand1 );

			siconversion = QuGetAttrText( "SI_conversion", qdOperand1 );
			if ( siconversion.empty() )
			{                                            
				siconversion = "1>(unknown)";     //  empty if fail
			}

			lablaxis = stem;
			lablaxis += "_z";
			fieldnam += "_z";
			qdResult = QuGetVectorComponent( qdOperand1, "z" );
			break;

		case( 3 ):            // r component
			units = QuGetAttrText( "UNITS", qdOperand1 );

			siconversion = QuGetAttrText( "SI_conversion", qdOperand1 );
			if ( siconversion.empty() )
			{                                         // set SI_conversion
				siconversion = "1>(unknown)" ;     //  empty if fail
			}      

			lablaxis = "|";
			lablaxis += stem;
			lablaxis += "|" ;
			fieldnam += "_magnitude";
			qdResult = QuGetVectorComponent( qdOperand1, "r(mag)" );
			break;

		case( 4 ):            // Lat(deg) component
			units = "deg" ;
			siconversion = "1>degree";
			lablaxis = "Lat_";
			lablaxis += stem;
			fieldnam += "_Lat(deg)";
			qdResult = QuGetVectorComponent( qdOperand1, "Lat(deg)" );
			break;

		case( 5 ):            // theta(deg) component
			units = "deg";
			siconversion = "1>degree";
			lablaxis = "theta_";
			lablaxis += stem;
			fieldnam += "_theta(deg)";
			qdResult = QuGetVectorComponent( qdOperand1, "theta(deg)" );
			break;

		case( 6 ):            // phi(deg) component
			units = "deg";
			siconversion = "1>degree";
			lablaxis = "phi_";
			lablaxis += stem;
			fieldnam += "_phi(deg)";
			qdResult = QuGetVectorComponent( qdOperand1, "phi(deg)" );
			break;

		case( 7 ):            // Lat(rad) component
			units = "rad";
			siconversion = "1>rad";
			lablaxis = "Lat_";
			lablaxis += stem;
			fieldnam += "_Lat(rad)";
			qdResult = QuGetVectorComponent( qdOperand1, "Lat(rad)" );
			break;

		case( 8 ):            // theta(rad) component
			units = "rad";
			siconversion = "1>rad" ;
			lablaxis = "theta_";
			lablaxis += stem;
			fieldnam += "_theta(rad)";
			qdResult = QuGetVectorComponent( qdOperand1, "theta(rad)" );
			break;

		case( 9 ):            // phi(rad) component
			units = "rad";
			siconversion = "1>rad";
			lablaxis = "phi_";
			lablaxis += stem;
			fieldnam += "_phi(deg)";
			qdResult = QuGetVectorComponent( qdOperand1, "phi(rad)" );
			break;

		default:                     // Shouldn't get here!
			QcalcAppendTextDisplay( "VectorTSDecompAVG>> Unknown component requested; Failure" );
			rqstrErrorMsg = QString( "VectorTSDecompAVG>> Unknown component requested; Failure" );
			return ( qdResult );
			break;
	}                                 // end switch(icomp)


	// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
	QdObject_var qd0 = get_Depend_0( qdOperand1 );
	if ( !qd0.is_nil() ) qdResult->change_xref(DEPEND_0 , qd0 );

	// load all the attributes, and the half spacing/sample period:

	QuSetFrame(qdResult, "scalar", "",  "" );
	QuSetTxtAttr( "SI_conversion", siconversion.c_str(), qdResult );
	QuSetTxtAttr( "UNITS",         units.c_str(),        qdResult );
	QuSetTxtAttr( "LABLAXIS",      lablaxis.c_str(),     qdResult) ;


	QcalcAppendTextDisplay ( "VectorTSDecomp>> data object created" );
	rqstrErrorMsg = QString( "VectorTSDecomp>> data object created" );

	return ( qdResult );

}


