#include "qcalc.h"
#include "qwui.h"
#include "qdutil.h"

using namespace QSAS;

extern qwuiWindow* qsasWindow;


void QcalcListView::deleteAllItems()
{
	if  ( QuSlotDebug() )
		printf( "QcalcListView::deleteAllItems %ld\n", (long)this );

	QcalcItem *rootItem = calcModel()->_root;
	if(rootItem == NULL) return;
	
	while(rootItem->_children.size() >0)
	{
		QcalcItem *item = rootItem->_children.at(0);
		calcModel()->deleteItem(item);;
	}
}

void QcalcListView::deleteListItems()
{
	if  ( QuSlotDebug() )
		printf( "QcalcListView::deleteListItems %ld\n", (long)this );
	
	this->blockSignals(true);
	
	// deleting items affects the QModelIndex List, so collect items
	QModelIndexList indexes_temp=selectionModel()->selectedRows();
	QList <QcalcItem*> itemList;
	itemList.clear();
	
	foreach(QModelIndex index, indexes_temp)
	{
		QcalcItem *item = calcModel()->itemFromIndex(index);
		itemList.push_back(item);
	}
	clearSelection();
	
	// select current to something never deleted
	selectionModel()->setCurrentIndex(rootIndex(), QItemSelectionModel::NoUpdate );


	// now safe to start deleting
	for(int i=0; i<itemList.size(); i++)
	{
		QcalcItem* item=itemList[i];	
		// ignore any item whose ancestor is also selected
		if( item->ancestorInList(itemList) ) continue;
		
		calcModel()->deleteItem(item);
	
	}

	
	this->blockSignals(false);

	this->owner->_TagSource->reValidate();
	
    emit listItemDeleted(); 
			
	
}

QcalcItem* QcalcListView::insertEntry(const QString& context)
{
	if(context.contains("__root__")) return NULL;
	
	calcModel()->emitLayoutAboutToBeChanged();
	QcalcItem *newItem = new QcalcItem( this, context, context  );
	calcModel()->emitLayoutChanged();

	return newItem;
}


void QcalcListView::insertVirtualItem()
{
	calcModel()->emitLayoutAboutToBeChanged();
	QcalcItem* newItem =  new QcalcItem   ( this, "????", "");
	newItem->m_pcalcTab->objAliase_field->setText( newItem->_alias );
	calcModel()->emitLayoutChanged();

}


void QcalcListView::validateWLitems(){

	if(QuSlotDebug())
	{
		printf("QcalcListView::validateWLitems %ld\n", (long)this);
		fflush(stdout);
	}
  
	QList<QcalcItem*> items = listItems();
	foreach(QcalcItem *item, items)
	{
		QString ctxt = item->WLcontext;

		if(item->m_eObjType == WORKING_LIST_OBJECT && qsasWindow->wlview->wlmodel()->itemFromPath(ctxt) == NULL ){
			item->setColour( false);
		}
		else{
			item->setColour( true);
		}
	}


}

void QcalcListView::propagateSelection(const char * compString, const char * compType){

 if(QuSlotDebug())
 {
	printf("QcalcListView::propagateSelection %ld %s\n", (long)this, compString);
	fflush(stdout);
 }
 

	QList<QcalcItem*> items = selectedItems();
	foreach(QcalcItem *item, items)
	{		
		// ignore virtual items
		if(item->m_eObjType == WORKING_LIST_OBJECT && item != calcModel()->itemFromIndex(this->currentIndex()) ) 
		{
			// only modify items of the correct object type
			QguWLinput * objSlot = item->m_pcalcTab->operandStack[0]->slot;
			
			if ( strcmp(compType , "VECTOR" ) == 0){
				if(item->getIsVector()){
					QString baseName = objSlot->textObject();
					objSlot->setText(baseName + QString(compString));
					objSlot->reValidate(); // propagate change to rest of GUI
				}
			}
			else if ( strcmp(compType , "ARRAY") == 0){
				if(  !(item->virtualArrDim.is_nil()) && !(this->selectedObjDim.is_nil()) ) {
					if(*(item->virtualArrDim) == *(this->selectedObjDim) ) {
						QString baseName = objSlot->textObject();
						objSlot->setText(baseName + QString(compString));
						objSlot->reValidate(); // propagate change to rest of GUI
					}
				}
			}
			else {
				cout << "Multiple selection is ignored for Xref (attribute) and Time menu choice" << endl;
			}
		}
	}
}


void QcalcListView::tabChanged( QWidget* qwdgt )
{
	if  ( QuSlotDebug() )
		printf("QcalcListView::tabChanged, tab %ld of list %ld \n", (long)qwdgt, (long)this);

	QcalcTab* qTab = dynamic_cast< QcalcTab* >( qwdgt );
	if ( qTab == NULL ) return;

	if ( qTab->virtualItem == NULL ) return;

	qTab->virtualItem->m_pcalcTab->setThisSelected();
}





