#include "qcalc.h"
#include "qdutil.h"
#include "qwui.h"

using namespace QSAS;

extern qwuiWindow* qsasWindow;

void QcalcWindow::defaultAll()
 {

 if(QuSlotDebug())
 {
  printf("QcalcWindow::defaultAll %ld\n", (long)this);
  fflush(stdout);
  }

	QList<QcalcItem*> items = _join_listview->listItems();
	foreach(QcalcItem *item, items)
	{
		item->m_pcalcTab->_width_field->setText(      _width_field_Def );
		item->m_pcalcTab->_gap->setCurrentIndex(       _gap_Def         );
		item->m_pcalcTab->_gap_field->setText(        _gap_field_Def   );
		item->m_pcalcTab->_algorithm->setCurrentIndex( _algorithm_Def   );

	}
  
  _TimeIvlSelect->setCurrentIndex(  _TimeIvlSelect_Def );
  _IvlSource->setText(             _IvlSource_Def     );
  _TimeTagSelect->setCurrentIndex(  _TimeTagSelect_Def );
  _TagSource->setText(             _TagSource_Def     );

}

void QcalcWindow::setDefaults()
{

  _width_field_Def   = QString("60");
  _width_field_cpy   = QString("60");

  _gap_Def           = 0;
  _gap_cpy           = _gap_Def;

  _gap_field_Def     = QString("6");
  _gap_field_cpy     = QString("6");

  _algorithm_Def     = 0;
  _algorithm_cpy     = _algorithm_Def;

  _IvlSource_Def     = QString(" ");
  _IvlSource_cpy     = QString(" ");

  _TimeIvlSelect_Def = 0;
  _TimeIvlSelect_cpy = _TimeIvlSelect_Def;

  _TagSource_Def     = QString(" ");
  _TagSource_cpy     = QString(" ");

  _TimeTagSelect_Def = 1;
  _TimeTagSelect_cpy = _TimeTagSelect_Def;


}

 void QcalcWindow::keepChoices()
 {

 if ( QuSlotDebug() )
 {
	printf("QcalcWindow::keepChoices %ld\n", (long)this);
	fflush(stdout);
 }
 
	 QList<QcalcItem*> items = _join_listview->listItems();
	 foreach(QcalcItem *item, items)
	 {
		if( item->m_pcalcTab->isVisible())
  	 	{
			// keep the choices of the visible tab
 		 	_width_field_cpy   = QString(item->m_pcalcTab->_width_field->text().toLatin1().data());
 		 	_gap_cpy           = item->m_pcalcTab->_gap->currentIndex();
 		 	_gap_field_cpy     = QString(item->m_pcalcTab->_gap_field->text().toLatin1().data());
 		 	_algorithm_cpy     = item->m_pcalcTab->_algorithm->currentIndex();
			 break;
  		}
	}
  
  _IvlSource_cpy     = QString(_IvlSource->text().toLatin1().data());
  _TimeIvlSelect_cpy = _TimeIvlSelect->currentIndex();
  _TagSource_cpy     = QString(_TagSource->text().toLatin1().data());
  _TimeTagSelect_cpy = _TimeTagSelect->currentIndex();


}

 void QcalcWindow::applyChoices()
 {

 if ( QuSlotDebug() )
 {
	printf( "QcalcWindow::applyChoices %ld\n", (long)this );
	fflush(stdout);
 }

	 QList<QcalcItem*> items=_join_listview->selectedItems();
	 foreach(QcalcItem *item, items)
	{		
		item->m_pcalcTab->_width_field->setText(          _width_field_cpy   );
  		item->m_pcalcTab->_gap->setCurrentIndex(           _gap_cpy           );
  		item->m_pcalcTab->_gap_field->setText(            _gap_field_cpy     );
  		item->m_pcalcTab->_algorithm->setCurrentIndex(     _algorithm_cpy     );

  	}
	
  _TimeIvlSelect->setCurrentIndex( _TimeIvlSelect_cpy );
  _IvlSource->setText(            _IvlSource_cpy     );
  _TimeTagSelect->setCurrentIndex( _TimeTagSelect_cpy );
  _TagSource->setText(            _TagSource_cpy     );

}

 void QcalcWindow::setTTivl( int item )
 {
 static QString lastIvlObj = QString("");
 
 if(QuSlotDebug())
 {
	printf("QcalcWindow::setTTivl %ld, %d\n", (long)this, item);
	fflush(stdout);
 }

  if( item == 0 )
  {
    // from data
    if( !(_IvlSource->text().isEmpty()) ) lastIvlObj = QString(_IvlSource->text().toLatin1().data());
    
    IvlSource_label->setText( " " );
    _IvlSource->setToolTip( "not active");
    _IvlSource->setText( " " ); 
    _IvlSource->active = false;
  }
  else if ( item == 1 )
  {
    IvlSource_label->setText( "Interval Object" );
    _IvlSource->setToolTip("drag and drop object here" );
    _IvlSource->active       = true;
    _IvlSource->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_NOT_SEQ, QS_ANY));
	_IvlSource->addAcceptable(Qcondition(QS_NARROW, QS_ANY, QS_ANY, QS_SEQ, QS_ANY));
    _IvlSource->setText( lastIvlObj );
    _IvlSource->lineBox->setFocus();
  }

    _IvlSource->reValidate( true );
}


void QcalcWindow::setTTsource( int item )
{
 static QString lastObj = "";
 static QString lastNumber = "";
 
 if ( QuSlotDebug() )
 {
	printf( "QcalcWindow::setTTsource %ld, %d\n", (long)this, item );
	fflush(stdout);
 }

  if( item == 0 )
  {
    if ( !(_TagSource->text().isEmpty()) ) lastObj = QString(_TagSource->text().toLatin1().data());
    // create regular
    TagSource_label->setText("Spacing ");
    _TagSource->setToolTip( "enter spacing here (in seconds for time series)");
    _TagSource->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
    _TagSource->setText(lastNumber);
  }
  else if ( item == 1 )
  {
    if ( !(_TagSource->text().isEmpty()) ) lastNumber = QString(_TagSource->text().toLatin1().data());
    TagSource_label->setText("Target Tag Object");
    _TagSource->setToolTip( "drag and drop any object with DEPEND_0 (or DEPEND_0 object itself)");
    _TagSource->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_SEQ, QS_ANY));
	_TagSource->addAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_SEQ, QS_ANY));
    _TagSource->setText(lastObj);
  }

  _TagSource->reValidate( ) ; // updates T display if valid
  _TagSource->lineBox->setFocus();

}

void QcalcWindow::setTimeFeedback()
{

	if ( QuSlotDebug() )
	{
		printf( "QcalcWindow::setTimeFeedback %ld\n", (long)this );
		fflush(stdout);
	}

	QdTimeInterval_var ivl;
	int                Ntags=0;
	double             Spacing;
	double             Interval;

	// set interval
	ivl = QcalcGetInterval( );

	if ( ivl.is_nil() )
	{
		double start, end;
		if(QcalcGetScalarInterval(start, end)){
			QPalette palette;
			palette.setColor(Ntags_field->foregroundRole(), QColor("#000000"));
			Start_field->setPalette(palette);
			End_field->setPalette(palette);
			Interval = end - start;
			Start_field->setText( QString("%1").arg(start) );
			End_field->setText( QString("%1").arg(end) );
		}
		else{
			QPalette palette;
			palette.setColor(Ntags_field->foregroundRole(), QColor("#880000"));
			Ntags_field->setPalette(palette);
			Start_field->setPalette(palette);
			End_field->setPalette(palette);
			spacing_field->setPalette(palette);
			Ntags_field->setText(  "??");
			spacing_field->setText("??");
			Start_field->setText(  "0000-00-00 00:00:00.000");
			End_field->setText(    "0000-00-00 00:00:00.000");
			Interval = -1.0;
			return;
		}
	}
	else
	{
		QPalette palette;
		palette.setColor(Ntags_field->foregroundRole(), QColor("#000000"));
		Start_field->setPalette(palette);
		End_field->setPalette(palette);
		Start_field->setText( QString( ivl->start().getISOstring().c_str()) );
		End_field->setText(   QString( ivl->end().getISOstring().c_str()) );
		Interval = ivl->duration_abs();
	}
	
	// set tags

	if( _TimeTagSelect->currentIndex() == 0 )
	{
		// create regular tags
		bool OK;
		Spacing = _TagSource->text().toDouble(&OK);
		if( !OK || Spacing < 1.e-20 )
		{
			Spacing = -1.0;
			Ntags   = -1;
		}
		else
		{
			Ntags = (int) (Interval/Spacing + 1.0);
		}

	}
	else
	{
		// tags from object, get_timetags() is safe against nil object
		QdTimeSeq_var ts  = get_timetags(_TagSource->getSlotObject());
		if( ts.is_nil() || ivl.is_nil()){
			Ntags = -1;
			Spacing = -1.0;
			QdObject_var targetObj = _TagSource->getSlotObject();
				
			Spacing = get_scalar_spacing(targetObj);
			if(Spacing > 0) Ntags = (int) (Interval / Spacing);
		}
		else{
			Spacing              = get_time_spacing(ts);
			QdTimeInterval overlap = ivl->int_intersect(QdTimeInterval(ts[0], ts[ts->sequence_size()-1]) );
			Ntags                = (int) (overlap.duration_abs()/Spacing );
		}
	}

	QString txt;
	if(Ntags > 0)
	{
		QPalette palette;
		palette.setColor(Ntags_field->foregroundRole(), QColor("#000000"));
		Ntags_field->setPalette(palette);

		Ntags_field->setText(txt.setNum(Ntags));
	}
	else
	{
		QPalette palette;
		palette.setColor(Ntags_field->foregroundRole(), QColor("#880000"));
		Ntags_field->setPalette(palette);

		Ntags_field->setText("None, check interval");
	}

	if(Spacing > -1)
	{
		QPalette palette;
		palette.setColor(Ntags_field->foregroundRole(), QColor("#000000"));
		spacing_field->setPalette(palette);
		spacing_field->setText(txt.setNum(Spacing));
	}
	else
	{
		QPalette palette;
		palette.setColor(Ntags_field->foregroundRole(), QColor("#880000"));
		spacing_field->setPalette(palette);
		spacing_field->setText("??");
	}

}



void QcalcWindow::dragEnterEvent(QDragEnterEvent *event)
{
 if ( QuSlotDebug() )
 {
	printf( "QcalcWindow::dragEnterEvent %ld\n", (long)this );
	fflush(stdout);
 }

  this->activateWindow();
  this->raise();
  this->setAcceptDrops(TRUE);
}

