
#include "qcalc.h"

using namespace QSAS;


// Class QcalcTab
/*!
    \class QcalcTab
    \brief  This class defines the widget that is added to the calculator tab view when a new operation is created (added to
	the current list view).

	This class specialises the QT QWidget class\n
*/


QcalcTab::QcalcTab()
: QWidget()
{
  this->opWasJoin = false; // initialise flag
}

QcalcTab::~QcalcTab()
{

}


void QcalcTab::createCalcTabWidgets(QcalcItem * VirtItem)
{
   if  ( QuSlotDebug() ){
		printf("QcalcTab::createCalcTabWidgets %ld of %ld \n", (long)this, (long)VirtItem);	
		QDateTime  qdatetime ; 
		qdatetime = QDateTime::currentDateTime();
		printf(" ---- START tabs -----  %s\n", qdatetime.toString( "dd.MM.yyyy-hh:mm:ss.zzz" ).toLatin1().data());

   }
   
   if ( calcListview == NULL )  return;
   
   this->virtualItem = VirtItem;
   this->WLnameUnset = true;
   this->nSlots = 0;
   
  // Fill in the CALC tab
  QVBoxLayout *pageTab2layout = new QVBoxLayout( this );
  pageTab2layout->setSpacing( 2 );

  // Selected object details
  details_groupbox = new QGroupBox(this);

  details_groupbox->setTitle( "New Operation" );

//   details_widget = new QWidget( details_groupbox, "details_widget" );

  details_layout = new QGridLayout;
  details_layout->setSpacing( 2 );
  details_layout->setMargin(0);


  // Aliase of object being selected
  objAlias_label = new QLabel( "Object Alias: ");
  details_layout->addWidget( objAlias_label, 0, 0, Qt::AlignRight );

  objAliase_field = new QguWLinput( details_groupbox,  "objAliase_field" );
  objAliase_field->setMinimumWidth( 270 );
  objAliase_field->setMaxLength( 200 );
  objAliase_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
  objAliase_field->setText("");  
  objAliase_field->setEnabled( true);
  connect(objAliase_field, SIGNAL(edited_OK()), this, SLOT(reNameAliasTab()));  

    
  objAliase_field->resize( objAliase_field->sizeHint() );
  details_layout->addWidget( objAliase_field, 0, 1, Qt::AlignLeft );
  
 // Check BOX
  saveWL_check = new QCheckBox( "Save to WL as:  ");
  saveWL_check->setEnabled( TRUE );
  saveWL_check->resize( saveWL_check->sizeHint() );
  details_layout->addWidget( saveWL_check, 1, 0, Qt::AlignRight  );
  connect( saveWL_check, SIGNAL( clicked() ), this, SLOT( toggleWLName() ) );

 // Name of the working list Edit box
  _nameWL_field = new QguWLinput( details_groupbox, "nameWL_field" );
  _nameWL_field->setMinimumWidth( 270 );
  _nameWL_field->setMaxLength( 200 );
  _nameWL_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
  _nameWL_field->resize( _nameWL_field->sizeHint() );
  _nameWL_field->setToolTip( "Name of the object as saved within the working list" );
  details_layout->addWidget( _nameWL_field, 1, 1, Qt::AlignLeft  );
  connect( _nameWL_field, SIGNAL( edited_OK()), this, SLOT(setNameForSave()));
  
  saveWL_check->setChecked( FALSE );
  _nameWL_field->setEnabled( FALSE );
 
  details_groupbox->setLayout(details_layout);
  pageTab2layout->addWidget( details_groupbox );


  // Operation group box

  operation_groupbox = new QGroupBox(this);
  operation_groupbox->setTitle( " Operation " );
  
  operation_layout = new QGridLayout;

  operation_layout->setSpacing( 2 );
  operation_layout->setMargin(0);

  operator_label = new QLabel( "Operator   ");

  operation_layout->addWidget( operator_label, 0, 0, Qt::AlignRight );


  tb4 = new QPushButton( "            ");

  menu = new QMenu( tb4 );
  tb4->setText(strOpJoin);
  actJoin = menu->addAction(strOpJoin);
  connect(menu, SIGNAL(triggered(QAction*)), this, SLOT(SetOperator(QAction*)) );

  arithmetic = menu->addMenu("Arith f (A, B)...");

  // Pop menu arithmetic
  actAdd = arithmetic->addAction( strOpAdd );
  actAdd->setToolTip(strAddDescription);
  actSub = arithmetic->addAction( strOpSub);
  actSub->setToolTip(strSubDescription);
  actMult = arithmetic->addAction( strOpMult);
  actMult->setToolTip(strMultDescription);
  actDiv = arithmetic->addAction( strOpDiv);
  actDiv->setToolTip(strDivDescription);
  actPower = arithmetic->addAction( strOpPower);
  actPower->setToolTip(strPowDescription);
  actRemainder = arithmetic->addAction(strOpRemainder);
  actRemainder->setToolTip(strRemainderDescription);
  actPeicewise = arithmetic->addAction( strOpPiecewise);
  actPeicewise->setToolTip(strPiecewiseDescription);
  actPwDiv = arithmetic->addAction( strOpPwDiv);
  actPwDiv->setToolTip(strPwDivDescription);

  // Pop menu Filter
  filter =  menu->addMenu("Arith f (A)..." );

  actMod = filter->addAction( strOpMod);
  actMod->setToolTip(strModDescription);
  actAbsValue = filter->addAction(strOPAbsValue);
  actAbsValue->setToolTip(strAbsValueDescription);
  actLog = filter->addAction(strOpLog);
  actLog10 = filter->addAction(strOpLog10);
  actChSign = filter->addAction( strOpChSign);
  actChSign->setToolTip(strChSignDescription);
  actInvers = filter->addAction(strOpInvers);
  actInvers->setToolTip(strInversDescription);
  actSqrt = filter->addAction(strOpSqrt);
  actExp = filter->addAction(strOpExp);


  // Pop menu Trig
  trig = menu->addMenu("Trig...");
  
  actCos = trig->addAction(strOpCos);
  actSin = trig->addAction(strOpSin);
  actTan = trig->addAction(strOpTan);
  actCot = trig->addAction(strOpCot);
  actACos = trig->addAction(strOpACos);
  actASin = trig->addAction(strOpASin);
  actATan = trig->addAction(strOpATan);
  actATan2 = trig->addAction(strOpATan2);
  actACot = trig->addAction(strOpACot);
  trig->addSeparator();
  actCosh = trig->addAction(strOpCosh);
  actSinh = trig->addAction(strOpSinh);
  actTanh = trig->addAction(strOptanh);
  actCoth = trig->addAction(strOpCoth);
  actACosh = trig->addAction(strOpACosh);
  actASinh = trig->addAction(strOpASinh);
  actATanh = trig->addAction(strOpAtanh);
  actACoth = trig->addAction(strOpACoth);


  // Pop menu Vector Ops
  vectorOps = menu->addMenu("Vector Ops...");

  actVecProd = vectorOps->addAction(strOpVecProd);
  actDotProd = vectorOps->addAction(strOpDotProd);


 

    // Pop menu Modify
  modify =   menu->addMenu("Modify...");

  actUnits = modify->addAction(strOpUnits);
  actFills = modify->addAction(strOpFills);

  
  tb4->setMenu(menu);
  tb4->resize( tb4->sizeHint() );
  operation_layout->addWidget( tb4, 0, 1, Qt::AlignLeft );
  operation_layout->setColumnStretch(0,0);
  operation_layout->setColumnStretch(1,10);
  operation_layout->setColumnStretch(2,5);
  operation_layout->setColumnStretch(3,5);

  
    // Operands Stuff

   operand_layout = new QGridLayout;
   operand_layout->setSpacing( 1 );
   operand_layout->setMargin(0);

// operation_layout->addWidget(operand_widget);
  
  // create all slots, but hide them
   for(int nOps=0; nOps<MAX_N_OPERANDS; nOps++)
   {
	   QString title = getOpName(nOps);
	   QcalcInput *OpInput_i = new QcalcInput( operation_groupbox, title, title, this ); // its name is also title to be unique
	   OpInput_i->slot->setText("");
	   operandStack.push_back(OpInput_i);
	   operand_layout->addWidget( OpInput_i, nOps, 0);  
	// connect the underlying QlineEdit signal as the QguWLinput signal is only emitted if box has focus. 
	// Needed for pulldown updates for components etc

	   connect(calcListview, SIGNAL(listItemDeleted()), this, SLOT(reValidateSlot()) );
	   connect(OpInput_i->slot, SIGNAL(validated_WL_OBJ( const QString & ) ), calcListview, SLOT(validateWLitems()) );
	   connect(OpInput_i->slot, SIGNAL(hadDropEvent()), this, SLOT(setThisSelected()) );
	   connect(OpInput_i->slot, SIGNAL(componentSelected(const char *, const char * )), calcListview, SLOT(propagateSelection(const char *, const char * )) );
   }
   operation_layout->addLayout(operand_layout, 1, 0, 2, 2);

   

     // Extras

  extras_layout = new QGridLayout;
  extras_layout->setSpacing( 1 );
   
  AngleUnits = new QcalcInput(operation_groupbox, "AngleUnits", "Angle in", this);
  extras_layout->addWidget( AngleUnits, 0, 0 );
  AngleUnits->setAnglePulldown();

  AngleUnits->setToolTip( "Angle Units for Trig Function if not in SI_conversion" );
  AngleUnits->hide();
  operation_layout->addLayout(extras_layout, 0, 3, 1, 2);
  
  operation_groupbox->setLayout(operation_layout);

  pageTab2layout->addWidget( operation_groupbox );

  
    // Join Setting group box

  join_groupbox = new QGroupBox(this);
  join_groupbox->setTitle("Join Settings");

  // Widget Join Setting
//   qwJoinSetting = new QWidget( join_groupbox, "page_join_widget" );

  // ( 1 ) Create the GRID
  join_layout = new QGridLayout;
  join_layout->setSpacing( 2 );

  // ( 2 ) Algoritm label
  algorithm_label = new QLabel( "Join Algorithm ");
  join_layout->addWidget( algorithm_label, 0, 0, Qt::AlignRight );
  
  // Combo box
  _algorithm = new QComboBox( );
  _algorithm->clear();
  _algorithm->addItem( LINEAR);
  _algorithm->addItem( BOXCAR);
  join_layout->addWidget( _algorithm, 0, 1, Qt::AlignLeft );
  connect( _algorithm, SIGNAL( activated( int ) ), this,  SLOT( ValidateAlgorithm( int ) ) );


  // Average width
  width_label = new QLabel( "Ave width ");
  join_layout->addWidget( width_label, 1, 0, Qt::AlignRight );

  _width_field = new QguWLinput( join_groupbox, "width_field" );
  _width_field->setMinimumWidth( 150 );
  _width_field->setMaxLength( 250 );
  _width_field->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
  _width_field->setEnabled(false);
  connect( _width_field,  SIGNAL( edited_OK() ), this, SLOT( setWidthField() ) );
  _width_field->setToolTip( "width of the boxcar (in seconds for time series)" );
  join_layout->addWidget( _width_field, 1, 1 );

  // Gap hadling label
  gap_label = new QLabel("Gap Handling ");
  join_layout->addWidget( gap_label, 3, 0, Qt::AlignRight );

   // Gap handling combo box
  _gap = new QComboBox( );
  _gap->clear();
  _gap->addItem( LINEAR_INTERP_GAP);
  _gap->addItem( ZERO_FILL_GAP);
  _gap->addItem( FILL_GAP);
  _gap->addItem( NEAREST_NEIGHBOUR_GAP );
  _gap->addItem( REMOVE_GAP );

  connect( _gap, SIGNAL( activated(int) ), this, SLOT( setGap(int) ) );
  join_layout->addWidget( _gap, 3, 1, Qt::AlignLeft );

  // Gap threshold label
  gapSize_label = new QLabel( "Gap threshold ");
  join_layout->addWidget( gapSize_label, 4, 0, Qt::AlignRight ); 

  // Gap field input box
  _gap_field = new QguWLinput( join_groupbox, "gap_field" );
  _gap_field->setMinimumWidth( 150 );
  _gap_field->setMaxLength( 100 );
  _gap_field->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
  _gap_field->setEnabled( true);
  connect( _gap_field, SIGNAL(edited_OK()), this, SLOT(setGapField()) );
  _gap_field->setToolTip( "minimum interval to be treated as a gap if empty (in seconds for time series)" );
  join_layout->addWidget( _gap_field, 4, 1 );
  
  join_groupbox->setLayout(join_layout);
  pageTab2layout->addWidget( join_groupbox );


}

void QcalcTab::reValidateSlot()
{
	for(int i=0; i<operandStack.size(); i++) operandStack[i]->slot->reValidate(true);
}

void QcalcTab::SetTabFeedback( ) 
{
 
	if  ( QuSlotDebug() )
		printf("QcalcTab::SetTabFeedback %ld of %ld \n", (long)this, (long)virtualItem);
		
	virtualItem->SetItemName(false);

	// set save to WL name here AFTER the name has been set in text(Obj_Name_)
	// don't update once the user has entered their own choice
	if(this->WLnameUnset || this->_nameWL_field->text() == QString("") )
	{
		this->WLnameUnset = true; // reset if field was blank
   	  	QString joined_name = QString(virtualItem->_name);
		if(virtualItem->m_qstrOperator == strOpJoin)
		{
			// for WL item add _J and remove path
			int compPosn = joined_name.indexOf( " [ " );
			if (compPosn != -1) {
			   joined_name.replace(compPosn+1, 1, "(");
			   compPosn = joined_name.indexOf( "]" );
			   if (compPosn != -1) joined_name.replace(compPosn, 1, ")");
			}
			else{
				joined_name        += "_J";
			}
			// remove path
     		int posn            = joined_name.indexOf( "/" );
     		joined_name.remove( 0, posn+1 );
		}
		
		// sanitise / from divide op
		int posn = joined_name.indexOf(" / ");
		while (posn >=0){
			joined_name.replace(posn, 3, " | ");
			posn = joined_name.indexOf(" / ");
		}
		
		this->_nameWL_field->setText(joined_name);
		virtualItem->m_NameForSave = QString(joined_name.toLatin1().data()); // used only to allow name to be held when toggle is off
	}
	 confirmAliasSlot(); // this will silently set alias in slot to item->text(Alias_)
	 
	 // set Angle slot on/off according to availability of SI_conversion
	 if(virtualItem->m_qstrOperator == strOpCos ||
 	  virtualItem->m_qstrOperator == strOpSin  ||
	  virtualItem->m_qstrOperator == strOpTan  ||
	  virtualItem->m_qstrOperator == strOpCot  ){
	  
		bool SIc_unset = true;
		QdObject_var dobj = this->operandStack[0]->slot->getSlotObject();
		string SIc = QuToUpper(QuGetAttrText( "SI_conversion", dobj )); // safe against nil dobj
		if(SIc.find("RAD") != string::npos || SIc.find("DEG") != string::npos){
			// SI conversion should be used for units
			SIc_unset = false;
		}
		if( SIc_unset ) this->AngleUnits->show(); // not hyperbolic trig
		else this->AngleUnits->hide(); // hyperbolic trig or has deg/rad set in SI_conversion
	 }
}

void QcalcTab::SetTab( ) 
{
	if  ( QuSlotDebug() )
		printf("QcalcTab::SetTab %ld of %ld \n", (long)this, (long)virtualItem);
		
	SetTabFeedback();	
	
	updateFlags();

	bool isSelfLinked = false;
	QcalcListView * qView = virtualItem->_view;
	if(qView && this->isVisible()){
		// test for self link
		for(int i=0; i<this->nSlots; i++){
		   if( this->objAliase_field->text() == this->operandStack[i]->slot->textObject() ){
			  QPalette palette = QPalette(qApp->palette());
			  palette.setColor(QPalette::Active, QPalette::Base, QPalette::Link);
			  palette.setColor(QPalette::Active, QPalette::Text, QPalette::Dark);
			  this->operandStack[i]->slot->setPalette(palette);

			  qView->owner->messageHandler( "Self Link!", true );
			  isSelfLinked = true;
		   }
		}
		if(!isSelfLinked) {
			qView->owner->messageHandler(" ", false);
		}
	}

}


void QcalcTab::updateFlags() 
{
	if  ( QuSlotDebug() )
		printf("QcalcTab::updateFlags %ld  %ld\n", (long)this, (long)virtualItem);
	
	if(virtualItem->setArrayFlags()){ 

		// if flags changed, recursively update virtual items that depend on it
		if(virtualItem->_view){
			QList<QcalcItem*> items = virtualItem->_view->listItems();
			foreach(QcalcItem *item, items)
			{
				if (item == 0 || item == virtualItem || item->m_eObjType == WORKING_LIST_OBJECT) continue; // skip this as already done
				for (int i=0; i<item->m_pcalcTab->nSlots; i++){
					bool updateThis = false;
					if( item->m_pcalcTab->operandStack[i]->slot->textObject() == virtualItem->_alias ){
				   		updateThis = true;
						item->m_pcalcTab->operandStack[i]->slot->reValidate();
					}
					if(updateThis) item->m_pcalcTab->updateFlags();
				}
			}
		}
	}
}

bool QcalcTab::AddValid( ) 
{
	QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
	QdObject_var qdOperand2 = this->operandStack[1]->slot->getSlotObject();
	
    if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
    {
        // The object will be empty - this should never happen
		// The qdResult object is NULL at this point
		// Display error message within the window status
		QcalcAppendTextDisplay( "NULL input object to Add/Sub" );
		return false;
    }

    // Get base SI for input data
    int nQarStatus = QarAreUnitsSame( qdOperand1, qdOperand2 );
    if( nQarStatus == QAR_ERR )
    {
          QcalcAppendTextDisplay( "This Operation requires inputs in same units (from SI_conversion)" );
     	  return true; // allow Add and Subtract to try to convert on the fly
    }
    else if ( nQarStatus == QAR_WARN )
    {
		QcalcAppendTextDisplay( "WARNING: units (SI_conversion) unknown for at least one input - Proceed with caution!" );
    }

    //  Test frames...
    nQarStatus = QarAreFramesSame( qdOperand1, qdOperand2 );
    if( nQarStatus == QAR_ERR ) {
		QcalcAppendTextDisplay( "Frames differ for inputs" );
		return false;
	}
    
    return true;
}

bool QcalcTab::DivideValid( ) 
{
	QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
	QdObject_var qdOperand2 = this->operandStack[1]->slot->getSlotObject();
	
    if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
    {
        // The object will be empty - this should never happen
		// The qdResult object is NULL at this point
		// Display error message within the window status
		QcalcAppendTextDisplay( "NULL input object to divide" );
		return false;
    }

	// trap divide by vector or array

	if ( !QuIsScalar( qdOperand2 ) )
	{
		QcalcAppendTextDisplay( "Only divide by scalars or scalar sequences is allowed" );
		return false;
	}
	
	return true;
}

bool QcalcTab::VectorProdValid( ) 
{
	QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
	QdObject_var qdOperand2 = this->operandStack[1]->slot->getSlotObject();
	
    if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
    {
        // The object will be empty - this should never happen
		// The qdResult object is NULL at this point
		// Display error message within the window status
		QcalcAppendTextDisplay( "NULL input object to Vector Prod" );
		return false;
    }
	
	// test special for multiply ...
	int nQarStatus =  QarTestConformalForVecProds( qdOperand1, qdOperand2 );
	if ( nQarStatus == QAR_ERR )
	{
		QcalcAppendTextDisplay( "Inner and Outer product only valid between vectors or tensors" );
		return false;
	}
	
	return true;
}

bool QcalcTab::MultiplyValid( ) 
{
	QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
	QdObject_var qdOperand2 = this->operandStack[1]->slot->getSlotObject();
	
    if (  qdOperand1.is_nil()  )
    {
        // The object will be empty - this should never happen
		// The qdResult object is NULL at this point
		// Display error message within the window status
		QcalcAppendTextDisplay( "NULL input object 1 to Multiply" );
		return false;
    }
    if (  qdOperand2.is_nil()  )
    {
        // The object will be empty - this should never happen
		// The qdResult object is NULL at this point
		// Display error message within the window status
		QcalcAppendTextDisplay( "NULL input object 2 to Multiply" );
		return false;
    }
	
	return true;
}

bool QcalcTab::PiecewiseValid( ) 
{
	QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
	QdObject_var qdOperand2 = this->operandStack[1]->slot->getSlotObject();
	
    if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
    {
        // The object will be empty - this should never happen
		// The qdResult object is NULL at this point
		// Display error message within the window status
		QcalcAppendTextDisplay( "NULL input object to Piecewise Multiply" );
		return false;
    }
	
	Dim_var dim1;
	QdRMatrix_var m1 = QdRMatrix_var::narrow(qdOperand1);
	if(!m1.is_nil())
	{
		dim1 = m1->dimvar();
	}
	else
	{
		QdRMatrixSeq_var ms1 = QdRMatrixSeq_var::narrow(qdOperand1);
		if(ms1.is_nil() || ms1->sequence_size() == 0) return false;
		dim1 = ms1[0].dimvar();
	}
	
	Dim_var dim2;
	QdRMatrix_var m2 = QdRMatrix_var::narrow(qdOperand2);
	if(!m2.is_nil())
	{
		dim2 = m2->dimvar();
	}
	else
	{
		QdRMatrixSeq_var ms2 = QdRMatrixSeq_var::narrow(qdOperand2);
		if(ms2.is_nil() || ms2->sequence_size() == 0) return false;
		dim2 = ms2[0].dimvar();
	}
	
	// compare dimensions
	if(dim1->rank() != dim2->rank()) return false;
	for(size_t i=0; i<dim1->rank(); i++) if((*dim1)[i] != (*dim2)[i]) return false;
	
	return true;
}

bool QcalcTab::FilterValid( ) 
{
	QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
	
    if ( qdOperand1.is_nil()  )
    {
        // The object will be empty - this should never happen
		// The qdResult object is NULL at this point
		// Display error message within the window status
		QcalcAppendTextDisplay( "NULL input object to Filter" );
		return false;
    }
	
	// test special for exp 
	if (QarHasUnits( qdOperand1) )
	{
		QcalcAppendTextDisplay( "WARNING!  Operation Exp() or Log() -> Subject has units" );
	}
	
	return true;
}

bool QcalcTab::PowerValid( ) 
{
	QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
	QdObject_var qdOperand2 = this->operandStack[1]->slot->getSlotObject();
	
    if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
    {
        // The object will be empty - this should never happen
		// The qdResult object is NULL at this point
		// Display error message within the window status
		QcalcAppendTextDisplay( "NULL input object" );
		return false;
    }

	// get base SI for input data
	if( QarHasUnits( qdOperand2 ) )
	{
		// issue warning
		QcalcAppendTextDisplay( "WARNING! Operation Power - Exponent has units.");
	}

	return true;
}



bool QcalcTab::isOpAllowed( ) 
{
	if  ( QuSlotDebug() )
		printf("QcalcTab::isOpAllowed %ld of %ld \n", (long)this, (long)virtualItem);
		
	QcalcListView * qView = virtualItem->_view;
		
	bool isVector0 = false;
	bool isVector1 = false;
		
	QcalcItem * qItem = qView->findAlias( this->operandStack[0]->slot->textObject() );
		if(qItem) isVector0 = qItem->getIsVector();
		
		if(isVector0){
			// allow for component selection in slot
			if(this->operandStack[0]->slot->getComponent() != WL_OBJ) isVector0 = false;
		}
		
		if (nSlots > 1){
		 	qItem = qView->findAlias( this->operandStack[1]->slot->textObject() );
		     if(qItem) isVector1 = qItem->getIsVector();
			
			if(isVector1) {
				// allow for component selection in slot
				if(this->operandStack[1]->slot->getComponent() != WL_OBJ) isVector1 = false;
			}

		}

		// JOIN
		if(virtualItem->m_qstrOperator == strOpJoin){
			return true; // always allowed
		}
		
		// ADD or SUBTRACT
		else if(virtualItem->m_qstrOperator == strOpAdd || virtualItem->m_qstrOperator == strOpSub){
			if( ( isVector0 && !isVector1 ) || ( !isVector0 && isVector1 ) ) {
				QcalcAppendTextDisplay( "Cannot do operation: objects of different dimensionality, e.g. scalar + vector" );
				return false; 
			}
			if( !(this->AddValid()) ) {
				QcalcAppendTextDisplay( "Mismatch in Xrefs for inputs to Add or Subtract" );
				return false;
			}
		}
		
		// MULTIPLY scalars and matrices
		else if(virtualItem->m_qstrOperator == strOpMult){
			if( isVector0 &&  isVector1)  {
				QcalcAppendTextDisplay( "Use vector operations for multiplying vectors" );
				return false; 
			}
			if( !(this->MultiplyValid()) ) {
				QcalcAppendTextDisplay( "Mismatch in Xrefs for inputs to Multiply" );
				return false;
			}
		}
		
		// Piecewise multiplication
		else if( virtualItem->m_qstrOperator == strOpPiecewise){
			if( !(this->PiecewiseValid()) ) {
				QcalcAppendTextDisplay( "Piecewise Multiply requires arrays with same dimensions" );
				return false;
			}
		}
		
		// Piecewise division
		else if( virtualItem->m_qstrOperator == strOpPwDiv){
			if( !(this->PiecewiseValid()) ) {
				QcalcAppendTextDisplay( "Piecewise Multiply requires arrays with same dimensions" );
				return false;
			}
		}
		
		// DIVIDE
		else if(virtualItem->m_qstrOperator == strOpDiv){
			// only numerator may be vector
			if( isVector1 )  {
				QcalcAppendTextDisplay( "cannot divide by vector or array" );
				return false; 
			}
			if( !(this->DivideValid()) ) {
				QcalcAppendTextDisplay( "Mismatch in Xrefs for inputs to Divide" );
				return false;
			}
		}
		
		// MOD
		else if(virtualItem->m_qstrOperator == strOpMod){
			// always scalar but allowed for vector
		}
		
		// POWER
		else if(virtualItem->m_qstrOperator == strOpPower){
			// always scalar but allowed for vector
			// only base may be vector
			if( isVector1 )  {
				QcalcAppendTextDisplay( "cannot raise to power of vector or array" );
				return false; 
			}
			
			if( !(this->PowerValid()) ) {
				QcalcAppendTextDisplay( "Mismatch in Xrefs for inputs to Power" );
				return false;
			}
		}
		
		// CHANGE SIGN
		else if(virtualItem->m_qstrOperator == strOpChSign){
		}
		
		// TRIG
		else if(virtualItem->m_qstrOperator == strOpCos){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpSin){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpTan){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpCot){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpCosh){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpSinh){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOptanh){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpCoth){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpACos){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpASin){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpATan){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpACot){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpACosh){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpASinh){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpAtanh){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpACoth){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		
		// CROSS PRODUCT
		else if(virtualItem->m_qstrOperator == strOpVecProd){
			if( !isVector0 || !isVector1 ) {
				QcalcAppendTextDisplay( "Operation is only valid between vectors or tensors" );
				return false; 
			}
			if( !(this->VectorProdValid()) ) {
				QcalcAppendTextDisplay( "Mismatch in Xrefs for inputs to Cross Product" );
				return false;
			}
		}
		
		// DOT PRODUCT
		else if(virtualItem->m_qstrOperator == strOpDotProd){
			if( !isVector0 || !isVector1 ) {
				QcalcAppendTextDisplay( "Operation is only valid between vectors or tensors" );
				return false; 
			}
			if( !(this->VectorProdValid()) ) {
				QcalcAppendTextDisplay( "Mismatch in Xrefs for inputs to Inner Product" );
				return false;
			}
		}
		
		// REMAINDER
		else if(virtualItem->m_qstrOperator == strOpRemainder){
			if( isVector0 ){
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		
		// ABS
		else if(virtualItem->m_qstrOperator == strOPAbsValue){
			if( isVector0 ) {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		
		// LOG e
		else if(virtualItem->m_qstrOperator == strOpLog){
			if( isVector0 ){
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
			if( !(this->FilterValid()) ) {
				QcalcAppendTextDisplay( "Input not compatible with Log e" );
				return false;
			}
		}
		
		// LOG 10
		else if(virtualItem->m_qstrOperator == strOpLog10){
			if( isVector0 ) {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
			if( !(this->FilterValid()) ) {
				QcalcAppendTextDisplay( "Input not compatible with Log 10" );
				return false;
			}
		}
		
		// INVERSE
		else if(virtualItem->m_qstrOperator == strOpInvers){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		
		// SQRT
		else if(virtualItem->m_qstrOperator == strOpSqrt){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		
		// EXPONENT
		else if(virtualItem->m_qstrOperator == strOpExp){
			if( isVector0 ) {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
			if( !(this->FilterValid()) ) {
				QcalcAppendTextDisplay( "Input not compatible with Exp" );
				return false;
			}
		}
		
		// CONVERT UNITS
		else if(virtualItem->m_qstrOperator == strOpUnits){
			QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
			if( !QarHasUnits( qdOperand1 ) ){
				QcalcAppendTextDisplay( "Input needs SI_conversion for this filter " );
				return false;
			}
			QdObject_var qdOperand2 = this->operandStack[1]->slot->getSlotObject();
			if( !QarHasUnits( qdOperand2 ) ){
				QdString_var strObj = QdString_var::narrow(qdOperand2);
				if(strObj.is_nil() || strstr(strObj->c_str(), ">") == NULL ) {
					QcalcAppendTextDisplay( "SI_conversion slot must be text or object with SI_conversion" );
					return false;
				}
			}
		}
		
		// CONVERT FILLS
		else if(virtualItem->m_qstrOperator == strOpFills){
			QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
			if( !qdOperand1->xref_exists(FILL_VALUE)){
				QcalcAppendTextDisplay( "Input needs FILLVAL for this filter (ONLY works on WL object for now" );
				return false;
			}
		}
		
	return true;
}


void QcalcTab::reNameAliasTab()
{
	if  ( QuSlotDebug() )
		printf("QcalcTab::reNameAliasTab %ld  \n", (long)this);
		 
	if (calcListview == NULL) return;

	// stop revalidation from non-user input
	if ( !(this->objAliase_field->lineBox->hasFocus()) ) return;

	QString NewVal = this->objAliase_field->lineBox->text();
	if ( virtualItem->AliasAlreadyUsed( NewVal ) )
	{
		QString qstrMsg = QString("Alias ") + this->objAliase_field->lineBox->text() + QString(" already used. Please re-enter a valid alias");
		calcListview->owner->messageHandler( qstrMsg, true );
		return;
	}
	
	calcListview->owner->messageHandler( " ", false );
	virtualItem->reNameAlias( this->objAliase_field->lineBox->text() );
	
}

void QcalcTab::ValidateAlgorithm( int nIndex )
{
	if ( _algorithm )
	{
		if (_width_field  )
		{
			if ( nIndex == 1  )
			{
				_width_field->setEnabled( TRUE );
			}
			else
			{
				_width_field->setEnabled( FALSE );
			}
		}
		
		if( !(_algorithm->isVisible()) ) return; // ensure only active tab can update selection 
				
		// now instruct any selected item to also update
		QList<QcalcItem*> items = virtualItem->_view->selectedItems();
		foreach(QcalcItem *item, items)
		{
			if (item == virtualItem) continue; // this item
			
			// set algorithm for selected item
			item->m_pcalcTab->_algorithm->setCurrentIndex(_algorithm->currentIndex()); // may emit validate signal which is trapped above
			if (item->m_pcalcTab->_width_field  )
			{
				if ( nIndex == 1  )
				{
					item->m_pcalcTab->_width_field->setEnabled( TRUE );
				}
				else
				{
					item->m_pcalcTab->_width_field->setEnabled( FALSE );
				}
			}
		}
	}
	
}


void QcalcTab::setGap(int nIndex)
{

	if ( QuSlotDebug() )
		printf( "QcalcTab::setGap %ld\n", (long)this );

	if ( _gap )
	{
		if( !(_gap->isVisible()) ) return; // ensure only active tab can validate - avoid incorrect reset and inf loop

		// now instruct any selected item to also update
		QList<QcalcItem*> items = virtualItem->_view->selectedItems();
		foreach(QcalcItem *item, items)
		{
			if (item == virtualItem) continue; // this item
		   // set gap for selected item gap;
		   item->m_pcalcTab->_gap->setCurrentIndex(_gap->currentIndex()); // this may emit validate signal which is trapped above
		}
	}

}


void QcalcTab::setWidthField()
{

	if ( QuSlotDebug() )
		printf( "QcalcTab::setWidthField %ld\n", (long)this );

	// now instruct any selected item to also update
	QList<QcalcItem*> items = virtualItem->_view->selectedItems();
	foreach(QcalcItem *item, items)
	{
		if (item == virtualItem) continue; // this item
		// set gap for selected item gap;
		item->m_pcalcTab->_width_field->setText(this->_width_field->text()); // this may emit validate signal which is trapped above
	}

}


void QcalcTab::setGapField()
{

	if  ( QuSlotDebug() )
		printf("QcalcTab::setGapField %ld\n", (long)this);

	// now instruct any seleceted item to also update
	QList<QcalcItem*> items = virtualItem->_view->selectedItems();
	foreach(QcalcItem *item, items)
	{
		if (item == virtualItem) continue; // this item

		// set gap for selected item gap;
		item->m_pcalcTab->_gap_field->setText(this->_gap_field->text()); // this may emit validate signal which is trapped above
	}

}

void QcalcTab::confirmAliasSlot()
{

	if  ( QuSlotDebug() )
		printf("QcalcTab::confirmAliasSlot %ld\n", (long)this);

	if(  this->objAliase_field->text() != this->virtualItem->_alias)
	{
		// if user goes elsewhere before valid alias is entered, reset to existing alias
		this->objAliase_field->setText(this->virtualItem->_alias);
	}

}

void QcalcTab::SetOperand(  )
{
	if ( QuSlotDebug() ) printf("QcalcTab:: SetOperand %ld \n", (long)this);
		
	if ( this->virtualItem == NULL ) return;
	
	if(this->virtualItem->m_qstrOperator.isNull() ) return;

	// update the time range in case object subsampled and TI is from data
	calcListview->owner->setTimeFeedback();

	SetTab();
     return;
}


void QcalcTab::SetLink( const QString & objStr )
{
	if ( QuSlotDebug() )
		printf("QcalcTab::SetLink %ld %s\n", (long)this, objStr.toLatin1().data());
	
	if (this->virtualItem == NULL  ) return;

	if( this->virtualItem->m_qstrOperator == strOpJoin )
	{
		this->virtualItem->WLcontext = QString(objStr.toLatin1().data());
		// rename item on list
		this->virtualItem->setName(objStr);
	}
		
	SetTab();

     return;
}


void QcalcTab::toggleWLName()
{
	if  ( QuSlotDebug() )
        printf("QcalcTab::toggleWLName %ld\n", (long)this);fflush(stdout);

	if ( this->virtualItem == NULL ) return;

	if ( this->_nameWL_field == NULL || this->saveWL_check == NULL)	return;
	
	if ( this->saveWL_check->isChecked() )
	{
		this->_nameWL_field->setEnabled( TRUE );
		this->_nameWL_field->setText(this->virtualItem->m_NameForSave);
	}
	else
	{
		this->_nameWL_field->setEnabled( FALSE );
		this->_nameWL_field->setText("");
	}
	// propagate the same choice to other selected items
	QList<QcalcItem*> items = virtualItem->_view->selectedItems();
	foreach(QcalcItem *item, items)
	{
		if (!item || item == this->virtualItem) continue; // already done

		// set toggle for other selected item
		if(this->saveWL_check->isChecked()){
			item->m_pcalcTab->saveWL_check->setChecked(TRUE);
			item->m_pcalcTab->_nameWL_field->setEnabled( TRUE );
			item->m_pcalcTab->_nameWL_field->setText(item->m_NameForSave);
		}
		else
		{
			item->m_pcalcTab->saveWL_check->setChecked(FALSE);
			item->m_pcalcTab->_nameWL_field->setEnabled( FALSE );
			item->m_pcalcTab->_nameWL_field->setText("");
		}
	}
}    
  
void QcalcTab::setNameForSave()
{
	if  ( QuSlotDebug() )
		printf("QcalcTab::setNameForSave %ld\n", (long)this);

	if ( this->virtualItem == NULL ) return;
	
	if (this->_nameWL_field->lineBox->hasFocus()) 
	{
		this->virtualItem->m_NameForSave = this->_nameWL_field->lineBox->text();
	
		this->WLnameUnset = false;  // stops auto-set overwriting user choice
	}
}


void QcalcTab::SetOperatorByName(const char *name )
{
	QAction *act=0;
	QList <QAction*> actions = menu->actions();
	for(int i=0; i<actions.size(); i++)
	{
		if(strcmp(name, actions.at(i)->text().toLatin1().data()) == 0) act = actions.at(i);
	}
	if(act) 
	{
		SetOperator(act);
		return;
	}
	
	QList <QAction*> arithActions = arithmetic->actions();
	for(int i=0; i<arithActions.size(); i++)
	{
		if(strcmp(name, arithActions.at(i)->text().toLatin1().data()) == 0) act = arithActions.at(i);
	}
	if(act) 
	{
		SetOperator(act);
		return;
	}
	
	QList <QAction*> trigActions = trig->actions();
	for(int i=0; i<trigActions.size(); i++)
	{
		if(strcmp(name, trigActions.at(i)->text().toLatin1().data()) == 0) act = trigActions.at(i);
	}
	if(act) 
	{
		SetOperator(act);
		return;
	}
	
	QList <QAction*> vecActions = vectorOps->actions();
	for(int i=0; i<vecActions.size(); i++)
	{
		if(strcmp(name, vecActions.at(i)->text().toLatin1().data()) == 0) act = vecActions.at(i);
	}
	if(act) 
	{
		SetOperator(act);
		return;
	}
	
	QList <QAction*> filterActions = filter->actions();
	for(int i=0; i<filterActions.size(); i++)
	{
		if(strcmp(name, filterActions.at(i)->text().toLatin1().data()) == 0) act = filterActions.at(i);
	}
	if(act) 
	{
		SetOperator(act);
		return;
	}
	
	QList <QAction*> modifyActions = modify->actions();
	for(int i=0; i<modifyActions.size(); i++)
	{
		if(strcmp(name, modifyActions.at(i)->text().toLatin1().data()) == 0) act = modifyActions.at(i);
	}
	if(act) 
	{
		SetOperator(act);
		return;
	}
	
}

void QcalcTab::SetOperator(QAction *act )
{
	QString strOperator = QString(act->text());
	if ( QuSlotDebug() )
		printf("QcalcTab::SetOperator %ld %s \n", (long)this, strOperator.toLatin1().data());
	
	if( strOperator.isEmpty() ) return;
	
	if (this->virtualItem == NULL ) return;
	this->virtualItem->m_qstrOperator = QString(strOperator);
	
	SetSlotsForOperator(strOperator);

	QString joined_name;
	if ( this->tb4 && !(strOperator.isNull()) )
	{
		QdOperator qdOpDetails; // this is initialised with defaults
		this->tb4->setText( strOperator );
	     if(strOperator != strOpJoin)  
		{
			this->opWasJoin = false; // flag used to empty slot A if operation changes FROM join (as WL item is lost)
			calcListview->owner->m_mapOps.FindOperator( strOperator, qdOpDetails ) ;
		
			this->nSlots = qdOpDetails.GetNumOperands();
			if ( qdOpDetails.GetTrigFlag() ) //  atan2(y,x) does NOT have trig flag set
			{
				bool SIc_set = false;
				QdObject_var dobj = this->operandStack[0]->slot->getSlotObject();
				string SIc = QuToUpper(QuGetAttrText( "SI_conversion", dobj )); // safe against nil dobj

				if(SIc.find("RAD") != string::npos || SIc.find("DEG") != string::npos){
					// SI conversion should be used for units
					SIc_set = true;
				}

				if(           strOperator == strOpCosh
				           || strOperator == strOpSinh
				           || strOperator == strOptanh
				           || strOperator == strOpCoth
				           || strOperator == strOpACosh
				           || strOperator == strOpASinh
				           || strOperator == strOpAtanh
				           || strOperator == strOpACoth) this->AngleUnits->hide(); // hyperbolic trig
				else if ( (  strOperator == strOpCos
				           || strOperator == strOpSin
				           || strOperator == strOpTan
				           || strOperator == strOpCot)
					    && SIc_set ) this->AngleUnits->hide(); // trig with SI_conversion
				else {
					this->AngleUnits->show(); // inverse trig or trig with no SI_conversion
					this->AngleUnits->pulldown->setCurrentIndex( 0 );
				}
			}
			else if( strOperator == strOpATan2)
			{
				this->AngleUnits->show(); // atan2 on its own as has 2 args
			}
			else
			{
				this->AngleUnits->hide(); // not trig menu
			}
		}
		else{
			this->AngleUnits->hide();		
		}
		
		for(int i = this->nSlots; i < MAX_N_OPERANDS; i++) 
		{
			this->operandStack[i]->hide();
			this->operandStack[i]->slot->active = false;
		}
		
		for(int i = 0; i < this->nSlots; i++)
		{
			this->operandStack[i]->show();
			this->operandStack[i]->slot->active = true;
			this->operandStack[i]->slot->reValidate();
		}

		// Set the operator name within the selected list view item
		this->virtualItem->SetItemName( true );

		// set save to WL name here AFTER the name has been set in text(Obj_Name_)
		// don't update once the user has entered their own choice
		if(this->WLnameUnset || this->_nameWL_field->text() == QString("") )
		{
   	  		this->WLnameUnset = true; // reset if field was blank
			joined_name = QString(this->virtualItem->_name.toLatin1().data());
			if(strOperator == strOpJoin)
			{
				// for WL item add _J and remove path
				int compPosn = joined_name.indexOf( " [ " );
				if (compPosn != -1) {
					joined_name.replace(compPosn+1, 1, "(");
					compPosn = joined_name.indexOf( "]" );
					if (compPosn != -1) joined_name.replace(compPosn, 1, ")");
				}
			
				joined_name        += "_J";
				
     			int posn            = joined_name.indexOf( "/" );
     			if(posn != -1) joined_name.remove( 0, posn+1 );
			}

			// sanitise / from divide op. Will be } from text() sanitisation
			int posn = joined_name.indexOf(" / ");
			while (posn >=0){
				joined_name.replace(posn, 3, " | ");
				posn = joined_name.indexOf(" / ");
			}

			this->_nameWL_field->setText(joined_name);
			this->virtualItem->m_NameForSave = QString(joined_name.toLatin1().data()); // used only to allow name to be held when toggle is off
		}
	}
}

void QcalcTab::SetSlotsForOperator(const QString& strOperator )
{	
	this->operandStack[1]->inputLabel->setText("Operand B    "); // gets reset by Filter units
	if( this->opWasJoin ) {
		this->operandStack[0]->slot->setText("");
	}
	
	// all operations are virtual except join

	if(strOperator != strOpUnits){
		this->operandStack[0]->setToolTip( " Drop object from WL or calculator list, or type in value " );
		this->operandStack[1]->setToolTip( " Drop object from WL or calculator list, or type in value " );
	}
			
	// Arithmetic
	if(     strOperator == strOpMod
		 || strOperator == strOpChSign ){
	  this->operandStack[0]->slot->setAcceptable(Qcondition(QS_ALIAS, QS_DBL));
	}
	else if(strOperator == strOpDiv 
	     || strOperator == strOpPower){
	  this->operandStack[0]->slot->setAcceptable(Qcondition(QS_ALIAS, QS_DBL));
	  this->operandStack[1]->slot->setAcceptable(Qcondition(QS_ALIAS, QS_DBL, QS_0D));
	  this->operandStack[1]->slot->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ));
	}
	else if(strOperator == strOpAdd 
	     || strOperator == strOpSub
		 || strOperator == strOpMult
		 || strOperator == strOpPiecewise
		 || strOperator == strOpPwDiv){
		// note multiply will operate on matrices, so set no extra constraints
		this->operandStack[0]->slot->setAcceptable(Qcondition(QS_ALIAS, QS_DBL));
		this->operandStack[0]->slot->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ));
		this->operandStack[1]->slot->setAcceptable(Qcondition(QS_ALIAS, QS_DBL));
		this->operandStack[1]->slot->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ));
	}
	// Trig
	else if(strOperator == strOpCos 
	     || strOperator == strOpSin
		 || strOperator == strOpTan
		 || strOperator == strOpCot
		 || strOperator == strOpCosh
		 || strOperator == strOpSinh
		 || strOperator == strOptanh
		 || strOperator == strOpCoth
		 || strOperator == strOpACos
		 || strOperator == strOpASin
		 || strOperator == strOpATan
		 || strOperator == strOpACot
		 || strOperator == strOpACosh
		 || strOperator == strOpASinh
		 || strOperator == strOpAtanh
		 || strOperator == strOpACoth){
	
		this->operandStack[0]->slot->setAcceptable(Qcondition(QS_ALIAS, QS_DBL, QS_0D));
		this->operandStack[0]->slot->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ));

	}
	else if(strOperator == strOpATan2){ 
		this->operandStack[0]->slot->setAcceptable(Qcondition(QS_ALIAS, QS_DBL, QS_0D));
		this->operandStack[0]->slot->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ));
		this->operandStack[1]->slot->setAcceptable(Qcondition(QS_ALIAS, QS_DBL, QS_0D));
		this->operandStack[1]->slot->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ));
	}
	// Vector Ops
	else if(strOperator == strOpVecProd
		|| strOperator == strOpDotProd){ 

		this->operandStack[0]->slot->setAcceptable(Qcondition(QS_ALIAS, QS_DBL, QS_VECTOR));
		this->operandStack[1]->slot->setAcceptable(Qcondition(QS_ALIAS, QS_DBL, QS_VECTOR));
	}
     // Filters
	else if(strOperator == strOpRemainder){
		this->operandStack[0]->slot->setAcceptable(Qcondition(QS_ALIAS, QS_DBL, QS_0D));
		this->operandStack[0]->slot->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ));
		this->operandStack[1]->slot->setAcceptable(Qcondition(QS_ALIAS, QS_DBL, QS_0D));
		this->operandStack[1]->slot->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ));
	}
	else if(strOperator == strOpInvers){
	  	// will accept square matrices
		this->operandStack[0]->slot->setAcceptable(Qcondition(QS_ALIAS, QS_DBL));
		this->operandStack[0]->slot->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ));
	}
	else if(strOperator == strOpUnits){
	  	// will accept anything
		this->operandStack[0]->slot->setAcceptable(Qcondition(QS_ALIAS, QS_DBL));
		this->operandStack[0]->slot->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ));

		this->operandStack[1]->slot->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ));
		this->operandStack[1]->slot->addAcceptable(Qcondition(QS_WL_OBJ));

	  	this->operandStack[1]->inputLabel->setText("SI_conv/Object  ");
		this->operandStack[0]->setToolTip( " Drop object from WL or calculator list, or type in value " );
		this->operandStack[1]->setToolTip( " Accepts SI_conversion text (e.g. 1.0e3>m), or object with SI_conversion xref ");
	}
	else if(strOperator == strOpFills){
		// will accept anything
		this->operandStack[0]->slot->setAcceptable(Qcondition(QS_ALIAS));

		this->operandStack[1]->slot->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ));
		this->operandStack[1]->slot->addAcceptable(Qcondition(QS_ALIAS, QS_DBL, QS_0D, QS_NOT_SEQ));
		this->operandStack[1]->inputLabel->setText("Replace fill w/  ");
	}
	else if(strOperator == strOPAbsValue
		|| strOperator == strOpLog
		|| strOperator == strOpLog10
		|| strOperator == strOpSqrt
		|| strOperator == strOpExp){
		this->operandStack[0]->slot->setAcceptable(Qcondition(QS_ALIAS, QS_DBL, QS_0D, QS_NOT_SEQ));
		this->operandStack[0]->slot->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ));
	}
	// Join only
	else if(strOperator == strOpJoin){
		this->operandStack[1]->inputLabel->setText("Operand B   "); // gets reset by Filter units
	
		this->opWasJoin = true;
		this->nSlots = 1;
			
		this->operandStack[0]->slot->setAcceptable(Qcondition(QS_WL_OBJ));
		this->operandStack[0]->setToolTip( "Drop object from WL" );
	
		if ( this->menu )
		{
			this->menu->setToolTip( QString("returns object or component joined") );
		}
	}
	else{
		cout << "UNKNOWN OPERATION, NOTIFY CSC SUPPORT [" << strOperator.toLatin1().data() << "]" << endl;
	}

}




void QcalcTab::setThisSelected(){
	// process other signals first
	qApp->processEvents();

  if(calcListview && virtualItem){
	  QModelIndex idx = calcListview->calcModel()->indexFromItem(virtualItem, 0);
	  calcListview->setCurrentIndex(idx);  
	  calcListview->showTab(this);
  }
}







