#include "qcalcEvaluator.h"


void QEvaluatorAdd::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdObject_var result;
	string attrPtr;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	QdObject_var val2=in[1]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[1]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	// Test dimensionality of inputs
	bool isVector1=QuIsThreeVector(val1);
	bool isVector2=QuIsThreeVector(val2);
	
	if( (isVector1 && !isVector2) || (!isVector1 && isVector2) )
	{
		message=QString("Inputs of different dimensionality");
		ok=false;
		return;
	}
	
	// Test no mismatch in Xrefs
	
	int nQarStatus=QarAreFramesSame(val1, val2);
	if( nQarStatus == QAR_ERR)
	{
		message=QString("Frames differ for inputs");
		ok=false;
		return;
	}
	
	// Test validity of inputs...
	nQarStatus=QarTestConformal(val1, val2);
	if ( nQarStatus == QAR_ERR )
	{
		message=QString("Non conformal inputs");
		ok=false;
		return;
	}
	
	// some validity tests are done in isOpAllowed() called by DoOperation() before (pairwise) joining 

	if(QarAreUnitsSame(val1, val2) == QAR_ERR )
	{
		// try to convert into same units 
		try
		{
// 			QcalcAppendTextDisplay( "Trying to convert to same units" );
			std::string SIcTarget = std::string(QuGetAttrText("SI_conversion", val1));
			val2 = QarConvertUnitsTo(val2, SIcTarget.c_str() ); 
			// this creates new converted object, copies xrefs across and sets SI_conversion
		}
		catch(Exception &e)
		{
			message=QString("Units conversion failed: ")+QString(e.msg().c_str());
			ok=false;
			return;
		}
		if(val2.is_nil())
		{
			message=QString("Units conversion failed");
			ok=false;
			return;
		}
	}
	
	try
	{
		result = val1 + val2;
	}
	catch ( Exception &e )
	{
		message=QString("Operator failed: ")+QString(e.msg().c_str());
		ok=false;
		return;
	}
	
	if(result.is_nil())
	{
		message="This Operation Add is not yet supported between these input types";
		ok=false;
		return;
	}


	// get xrefs from inputs if requested
	result->use_xrefs_from(val1, val2);
	
	// 6 - Add timeline: Either timeline is safe if it exists

	// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
	QdObject_var qd0 = get_Depend_0( val1 );
	if ( qd0.is_nil() ) qd0 = get_Depend_0( val2 );
	if ( !qd0.is_nil() ) result->change_xref(DEPEND_0 , qd0 );

	
	// Set QSAS essential attriutes
	// for addition they stay the same as inputs

	// SI_conversion
	attrPtr = QuGetAttrText( "SI_conversion", val1 );
	if ( !attrPtr.empty()  )
	{
		QuSetTxtAttr("SI_conversion", attrPtr.c_str(), result );
	}
	else
	{
		attrPtr = QuGetAttrText( "SI_conversion", val2 );
		QuSetTxtAttr("SI_conversion", attrPtr.c_str(), result );
	}

	// Frame
	QuSetFrame(result, QuGetFrameAttr(val1, val2).c_str());
   
	// UNITS
	attrPtr = QuGetAttrText("UNITS", val1);
	if ( !attrPtr.empty()  )
	{
		QuSetTxtAttr("UNITS", attrPtr.c_str(), result);
	}
	else
	{
		attrPtr = QuGetAttrText("UNITS", val2);
		QuSetTxtAttr("UNITS", attrPtr.c_str(), result);
	}

 	// LABLAXIS
	string labl_s = QuGetAttrText( "LABLAXIS", val1 );
	labl_s += " + ";
	labl_s += QuGetAttrText( "LABLAXIS", val2 );
	QuSetTxtAttr( "LABLAXIS", labl_s.c_str(), result );

 	// DEPEND_i
	if(val1->xref_exists(DEPEND_1)) 
	{
		QdObject_var d1_xref = val1->get_xref(DEPEND_1);
		result->change_xref(DEPEND_1, d1_xref);
	}
	else if (val2->xref_exists(DEPEND_1))
	{
		QdObject_var d1_xref = val2->get_xref(DEPEND_1);
		result->change_xref(DEPEND_1, d1_xref);
	}

	if (val1->xref_exists(DEPEND_2)) 
	{
		QdObject_var d1_xref = val1->get_xref(DEPEND_2);
		result->change_xref(DEPEND_2, d1_xref);
	}
	else if (val2->xref_exists(DEPEND_2))
	{
		QdObject_var d1_xref = val2->get_xref(DEPEND_2);
		result->change_xref(DEPEND_2, d1_xref);
	}

	if (val1->xref_exists(DEPEND_3)) 
	{
		QdObject_var d1_xref = val1->get_xref(DEPEND_3);
		result->change_xref(DEPEND_3, d1_xref);
	}
	else if (val2->xref_exists(DEPEND_3))
	{
		QdObject_var d1_xref = val2->get_xref(DEPEND_3);
		result->change_xref(DEPEND_3, d1_xref);
	}
	out[0]->setValue(result, true);
}

