#include "qcalcEvaluator.h"


void QEvaluatorDotProd::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdObject_var result;
	string attrPtr;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	QdObject_var val2=in[1]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[1]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	bool isVector1=QuIsThreeVector(val1);
	bool isVector2=QuIsThreeVector(val2);
	
	if( !isVector1 ||  !isVector2)
	{
		message=QString("Operation is only valid between vectors or tensors");
		ok=false;
		return; 
	}
	
	// Test validity of inputs...
	int nQarStatus =  QarTestConformalForVecProds( val1, val2 );
	if ( nQarStatus == QAR_ERR )
	{
		message=QString("Inner and Outer product only valid between vectors or tensors" );
		ok=false;
		return; 
	}
	
	int frameOK = QarAreFramesSame(val1, val2 );
	if(frameOK == QAR_WARN)
	{
		message=QString("Warning: Vect Prod: Cannot confirm vexctor frame");
		ok=false;
		return; //TODO warning rather than error
	}
	else if(frameOK == QAR_ERR)
	{
		message=QString("Error: Vect Prod: Frames differ");
		ok=false;
		return;
	}

	// do arithmetic...
	try
	{
		result = inner(val1, val2);
	}
	catch( Exception &e )
	{
		message=QString(e.msg().c_str());
		message+=QString(" The Operation Dot Prod is not yet supported between these input types");
		ok=false;
		return;
	}

	if ( result.is_nil() )
	{
		message=QString("Operation Not valid for these input types" );
		ok=false;
		return;
	}
	
	// get xrefs from inputs if requested
	result->use_xrefs_from(val1, val2);


	// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
	QdObject_var qd0 = get_Depend_0( val1 );
	if ( qd0.is_nil() ) qd0 = get_Depend_0( val2 );
	if ( !qd0.is_nil() ) result->change_xref(DEPEND_0 , qd0 );

	// Set QSAS essential attriutes

	// Frame
	QuSetTxtAttr( "Frame", "scalar>na", result );

	// UNITS
	attrPtr = QarUNITS_product( val1, val2 );
	if ( !attrPtr.empty() )
	{
		QuSetTxtAttr( "UNITS", attrPtr.c_str(), result );
	}
	
	// SI_conversion
	attrPtr = QarSIConv_product( val1, val2 );
	if ( !attrPtr.empty() )
	{
		QuSetTxtAttr( "SI_conversion", attrPtr.c_str(), result );
	}
	
	out[0]->setValue(result, true);
}
