#include "qcalcEvaluator.h"


void QEvaluatorExp::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdObject_var result;
	string attrPtr;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	bool isVector1=QuIsThreeVector(val1);
	if(isVector1)
	{
		message=QString("Operation is not valid for vector objects");
		ok=false;
		return;
	}
	
	if (QarHasUnits(val1))
	{
		message=QString("WARNING!  Operation Exp() or Log() -> Subject has units");
		ok=false;
		return; //TODO warning rather than error
	}
	
	// do arithmetic...
	try
	{
		result = exp(val1);
	}
	catch( Exception &e )
	{
		message=QString(e.msg().c_str());
		message+=QString( " The operation Exp is not yet supported for the input type");
		ok=false;
		return;
	}

	if ( result.is_nil() )
	{
		message=QString( "Arithmetic>> Exp is not yet supported for the input type" );
		ok=false;
		return;
	}

	// get xrefs from inputs if requested
	result->use_xrefs_from(val1);

	// Add timeline (or DEPEND_0)	
	QdObject_var qd0 = get_Depend_0( val1 );
	if ( !qd0.is_nil() ) result->change_xref(DEPEND_0 , qd0 );

 	// Set QSAS essential attributes

   	// SI_conversion

    // Not meaningfol for a exp whatever the inputs

   	// UNITS

	// Not meaningfol for a exp whatever the inputs
	
	// LABLAXIS
	string labl_s = "exp(";
	labl_s += QuGetAttrText( "LABLAXIS", val1 );
	labl_s += ")";
	QuSetTxtAttr( "LABLAXIS", labl_s.c_str(), result );



 	// Frame
	QuSetFrame(result, "scalar" );
	
	out[0]->setValue(result, true);
}


void QEvaluatorTrace::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdObject_var result;
	string attrPtr;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	QdRMatrix_var mobj = QdRMatrix_var:: narrow(val1);
	QdRMatrixSeq_var msobj = QdRMatrixSeq_var:: narrow(val1);
  
	if(!mobj.is_nil())
	{
		if(mobj->rank() != 2) 
		{
			message=QString("Operation is only valid for 2D matrix");
			ok=false;
			return;
		}
		
		if( (mobj->dims())[0] != (mobj->dims())[1] )
		{
			message=QString("Operation is only valid for square matrix");
			ok=false;
			return;
		}

		QdRScalar_var Trace = new QdRScalar();
		try
		{
			*Trace = (*mobj)[0][0];
			for(int i=1; i<(mobj->dims())[0]; i++)
			{
				*Trace += (*mobj)[i][i];
			}
			result = Trace;
		}
		catch(Exception &e)
		{    
			message=QString(e.msg().c_str());
			ok=false;
			return;    
		}
	}
	else if (!msobj.is_nil()) 
	{
		if(msobj[0].rank() != 2) 
		{
			message=QString("Operation is only valid for 2D matrix");
			ok=false;
			return;
		}
		
		if( (msobj[0].dims())[0] != (msobj[0].dims())[1] )
		{
			message=QString("Operation is only valid for square matrix");
			ok=false;
			return;
		}
		QdRScalarSeq_var Trace = new QdRScalarSeq(msobj->sequence_size());
		try
		{
			for(int j=0; j<msobj->sequence_size(); j++)
			{
				Trace[j] = (msobj[j])[0][0];
				for(int i=1; i<(msobj[i].dims())[0]; i++)
				{
					Trace[j] += (msobj[j])[i][i];
				}
			}
			result = Trace;
		}
		catch(Exception &e)
		{     
			message=QString(e.msg().c_str());
			ok=false;
			return;
		}

	}
	else
	{   
			message=QString("Input not a matrix");
			ok=false;
			return;	
	}


	if ( result.is_nil() )
	{
		message=QString( "Trace returned empty object" );
		ok=false;
		return;
	}


	// Add timeline (or DEPEND_0) 
	QdObject_var qd0 = get_Depend_0( val1 );
	if ( !qd0.is_nil() ) result->change_xref(DEPEND_0 , qd0 );

  	// set xrefs
	
	if(val1->xref_exists("UNITS") ){
		QdObject_var xref = val1->get_xref("UNITS");
		result->change_xref("UNITS", xref);
	}
	if(val1->xref_exists("SI_CONVERSION") ){
		QdObject_var xref = val1->get_xref("SI_CONVERSION");
		result->change_xref("SI_CONVERSION", xref);
	}
	if(val1->xref_exists("FIELDNAM") ){
		QdObject_var xref = val1->get_xref("FIELDNAM");
		QdString_var sxref = QdString_var::narrow(xref);
		if(!sxref.is_nil()){
			*sxref += " trace";
			result->change_xref("FIELDNAM", sxref);
		}
	}
	
	// LABLAXIS
	string labl_s = "trace(";
	labl_s += QuGetAttrText( "LABLAXIS", val1 );
	labl_s += ")";
	QuSetTxtAttr( "LABLAXIS", labl_s.c_str(), result );



 	// Frame
	QuSetFrame(result, "scalar" );
	
	out[0]->setValue(result, true);
}

void QEvaluatorTTtoSec::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	
	ok=true;
	QdObject_var result;
	string attrPtr;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	QdTimeSeq_var tt = get_timetags(val1);
	
	QdObject_var val2=in[1]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[1]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	QdTime_var startTime = QdTime_var::narrow(val2);
	
	if(tt.is_nil() ||  tt->sequence_size() <= 0){
		message=QString("Could not get valid time tags from input");
		QplugAppendTextDisplay("Timetags to Sec Operator: timetags not found");
		ok=false;
		return;
	}
	if(startTime.is_nil() ){
		message=QString("2nd Object did not resolve to a time");
		QplugAppendTextDisplay("Timetags to Sec Operator: input not Time object. \nTry adding Extractor.");
		ok=false;
		return;
	}
	
	QdRScalarSeq_var ss = new QdRScalarSeq(tt->sequence_size());
	for(int i=0; i<tt->sequence_size(); i++){
		ss[i] = tt[i] - *startTime;
	}
	QdString_var unitStr = new QdString("sec");
	ss->change_xref(UNITS, unitStr);
	QdString_var SI_Str = new QdString("1>s");
	ss->change_xref(SI_CONVERSION, SI_Str);
	QdString_var NameStr = new QdString("Sec after ");
	*NameStr += startTime->getISOstring(0);
	
	ss->change_xref(FIELDNAM, NameStr);
	
	result = Factory::clone(val1);
	result->copy_xrefs_from(val1);
	result->change_xref(DEPEND_0, ss);
			 
	out[0]->setValue(result, true);

}

void QEvaluatorTIVLtoSec::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdObject_var result;
	string attrPtr;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	QdTimeInterval_var ti = QdTimeInterval_var::narrow(val1);
		
	if(ti.is_nil() )
	{
		message=QString("Object did not resolve to a time interval");
		QplugAppendTextDisplay("Time Interval to Sec Operator: input not Time interval. \nTry adding Extractor.");
		ok=false;
		return;
	}
		
	QdRScalar_var val = new QdRScalar();
	*val = ti->duration();
			
	QdString_var SI_Str = new QdString("1>s");
	val->change_xref(SI_CONVERSION, SI_Str);
			
	QdString_var unit_Str = new QdString("sec");
	val->change_xref(UNITS, unit_Str);

	QdString_var name_Str = new QdString("Seconds duration");
	val->change_xref(FIELDNAM, name_Str);

	result = val;
	out[0]->setValue(result, true);

}

void QEvaluatorTT2_TT1toSec::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	
	ok=true;
	QdObject_var result;
	string attrPtr;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	QdTimeSeq_var tt1 = get_timetags(val1);
	
	QdObject_var val2=in[1]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[1]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	QdTimeSeq_var tt2 = get_timetags(val2);
	
	if(tt1.is_nil() ||  tt1->sequence_size() <= 0){
		message=QString("Could not get valid time tags from input 1");
		QplugAppendTextDisplay("Timetags Diff Operator: timetags not found");
		ok=false;
		return;
	}
	if(tt2.is_nil() ||  tt2->sequence_size() <= 0){
		message=QString("Could not get valid time tags from input 2");
		QplugAppendTextDisplay("Timetags Diff Operator: timetags not found");
		ok=false;
		return;
	}
		
	if( tt1->sequence_size() != tt2->sequence_size())
	{
		message=QString("Timetag sequences of different lengths");
		QplugAppendTextDisplay("Timetags Diff Operator: sequences not same length");
		ok=false;
		return;

	}
	
	QdRScalarSeq_var val = new QdRScalarSeq(tt2->sequence_size());
	for(int i=0; i<tt2->sequence_size(); i++){
		val[i] = tt2[i] - tt1[i];
	}

	// get xrefs from inputs if requested
	result->use_xrefs_from(tt1, tt2);

			
	QdString_var SI_Str = new QdString("1>s");
	val->change_xref(SI_CONVERSION, SI_Str);
			
	QdString_var unit_Str = new QdString("sec");
	val->change_xref(UNITS, unit_Str);

	QdString_var name_Str = new QdString("Time Diffs");
	val->change_xref(FIELDNAM, name_Str);
	
	result = val;
			 
	out[0]->setValue(result, true);

}

void QEvaluatorT2_T1toSec::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	
	ok=true;
	QdObject_var result;
	string attrPtr;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	QdTime_var t1 = QdTime_var::narrow(val1);
	
	QdObject_var val2=in[1]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[1]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	QdTime_var t2 = QdTime_var::narrow(val2);
	
	if(t1.is_nil() ){
		message=QString("Could not get valid time from input 1");
		QplugAppendTextDisplay("Time Diff Operator: time input not found");
		ok=false;
		return;
	}
	if(t2.is_nil() ){
		message=QString("Could not get valid time from input 2");
		QplugAppendTextDisplay("Timetags Operator: time input not found");
		ok=false;
		return;
	}
		
	QdRScalar_var val = new QdRScalar();
	*val = *t2 - *t1;

	// get xrefs from inputs if requested
	result->use_xrefs_from(t1, t2);
			
	QdString_var SI_Str = new QdString("1>s");
	val->change_xref(SI_CONVERSION, SI_Str);
			
	QdString_var unit_Str = new QdString("sec");
	val->change_xref(UNITS, unit_Str);

	QdString_var name_Str = new QdString("Time Diff");
	val->change_xref(FIELDNAM, name_Str);

	
	result = val;
			 
	out[0]->setValue(result, true);

}

// ----------------------

void QEvaluatorLinReg::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdObject_var result;
	string attrPtr;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	QdRScalarSeq_var fitLine = QuRegression(val1);
		
	if(fitLine.is_nil() )
	{
		message=QString("Object did not resolve to a scalar sequence");
		QplugAppendTextDisplay("Linear Regression: input not Scalar Sequence. \nTry adding Extractor.");
		ok=false;
		return;
	}
		
	if(val1->xref_exists("UNITS") ){
		QdObject_var xref = val1->get_xref("UNITS");
		fitLine->change_xref("UNITS", xref);
	}
	if(val1->xref_exists("SI_CONVERSION") ){
		QdObject_var xref = val1->get_xref("SI_CONVERSION");
		fitLine->change_xref("SI_CONVERSION", xref);
	}
	if(val1->xref_exists("FIELDNAM") ){
		QdObject_var xref = val1->get_xref("FIELDNAM");
		QdString_var sxref = QdString_var::narrow(xref);
		if(!sxref.is_nil()){
			*sxref += " linear fit";
			fitLine->change_xref("FIELDNAM", sxref);
		}
	}

	result = fitLine;
	out[0]->setValue(result, true);

}

