#include "qcalcEvaluator.h"


void QEvaluatorFills::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdRScalar_var newVal_ptr;
	QdObject_var out_ptr;
	const char* attrPtr     = NULL;
	char* stringBuf   = NULL;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	QdObject_var dobj=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	QdObject_var userValue=in[1]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[1]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	newVal_ptr = QdRScalar_var::narrow(userValue);
	QdObject_var fill_xref = dobj->get_xref(FILL_VALUE);
	if( fill_xref.is_nil())
	{
		message=QString("Input needs FILLVAL for this filter ");
		ok=false;
		return;
	}
	  // create output object

	double fill;
	bool hasD0 = false;
	QdObject_var obj_out;
  
	double newValue = (double) *newVal_ptr;
	QdRScalar_var fill_out = new QdRScalar( newValue );

	fill = -10.e+30; // in case xref not set
    
	QdObject_var D0_in = get_Depend_0( dobj );
	hasD0 = !(D0_in.is_nil());

    // is sequence a matrix sequence
	QdRMatrixSeq_var ms_in=QdRMatrixSeq_var::narrow(dobj);
	if(!ms_in.is_nil())
	{
		if(ms_in->empty())
		{
			message=QString("Empty input object");
			ok=false;
			return;
		}

      // if so, do we have a single fill-value or a matrix of different fill
      // values for each dimension

		QdRMatrix_var fill_m=QdRMatrix_var::narrow(fill_xref);

      // ...either way create matrix of fill values
      // this assumes the MatrixSeq is homogeneous i.e. each matrix
      // element has the same no. and sizes of dimensions
		if(fill_m.is_nil())
		{
			QdRScalar_var fill_v=QdRScalar_var::narrow(fill_xref);
			if(fill_v.is_nil())
				fill_v = new QdRScalar(fill);

			fill_m=new QdRMatrix(ms_in[0].dimdata(),fill_v->scalar());

		}
		
      // initialise matrix output sequence and reserve max space
		QdRMatrixSeq_var ms_out = Factory::clone(ms_in);
	 
      // get underlying data for fill matrix
		valarray<double> v_fill(fill_m->data()[fill_m->getgslice()]);

		for(register int i=0;i<ms_out->size();i++)
		{ 
	      // create valarray<double> of results for assigning into
           // matrix (gslice_array)
			bool update=false;       
			valarray<double> v_out(ms_out[i].data()[ms_out[i].getgslice()]);
			for(register int j=0;j<v_out.size();j++)
			{
				const double& dfill = v_fill[j%v_fill.size()];
				if( QuIsMatch( v_out[j], dfill) ) {
					update=true;
					v_out[j] = newValue;  // newValue is a double
				}
			}
           // assign results into gslice array (only if need to)
			if(update)
				ms_out[i].data()[ms_out[i].getgslice()]=v_out;
		}

		ms_out->copy_xrefs_from(ms_in);
		ms_out->change_xref(FILL_VALUE, fill_out);
	 

		if( hasD0){
			ms_out->change_xref(DEPEND_0 , D0_in );
		}
		out[0]->setValue(ms_out, true);
		return;
	}

    // scalar sequence case

	QdRScalarSeq_var ss_in=QdRScalarSeq_var::narrow(dobj);
	if(!ss_in.is_nil())
	{
		if(ss_in->empty())
		{
			message=QString("Empty input object");
			ok=false;
			return;
		}

		QdRScalar_var fill_v=QdRScalar_var::narrow(fill_xref);
		if( !fill_v.is_nil()) fill = (double) *fill_v;

		QdRScalarSeq_var ss_out = Factory::clone(ss_in);
	 
		for(int i=0;i<ss_in->size();i++){
			if( QuIsMatch(ss_out[i], fill) ) ss_out[i] = newValue;
		}

		ss_out->copy_xrefs_from(ss_in);
		ss_out->change_xref(FILL_VALUE, fill_out);

		if( hasD0 ){
			ss_out->change_xref(DEPEND_0 , D0_in );
		}
		out[0]->setValue(ss_out, true);
		return;
	}


	QdRMatrix_var m_in=QdRMatrix_var::narrow(dobj);
	if(!m_in.is_nil())
	{

      // if so, do we have a single fill-value or a matrix of different fill
      // values for each dimension

		QdRMatrix_var fill_m=QdRMatrix_var::narrow(fill_xref);

      // ...either way create matrix of fill values
      // this assumes the MatrixSeq is homogeneous i.e. each matrix
      // element has the same no. and sizes of dimensions

		if(fill_m.is_nil())
		{
			QdRScalar_var fill_v=QdRScalar_var::narrow(fill_xref);
			if(fill_v.is_nil()) fill_v = new QdRScalar(fill);

			fill_m=new QdRMatrix(m_in->dimdata(),fill_v->scalar());

		}


      // iterator for fill values
		RMatrix::iterator fitr=fill_m->begin();
      
		QdRMatrix_var m_out = Factory::clone(m_in);
		m_out->copy_xrefs_from(m_in);
		m_out->change_xref(FILL_VALUE, fill_out);

		if( hasD0){
			m_out->change_xref(DEPEND_0 , D0_in );
		}
	 
      // iterator for matrix
		RMatrix::iterator itr=m_out->begin();
		int j=0;
		while(itr!=m_out->end() )
		{
			double& d=*itr++;
			if( QuIsMatch(d, *(fitr+j++) ) ) d = newValue;
		}

		out[0]->setValue(m_out, true);
		return;

	}

	QdRScalar_var s_in=QdRScalar_var::narrow(dobj);
	if(!s_in.is_nil())
	{
		QdRScalar_var fill_v=QdRScalar_var::narrow(fill_xref);
		if( !fill_v.is_nil()) fill = (double) *fill_v;
	 
		QdRScalar_var s_out = Factory::clone(s_in);
		s_out->copy_xrefs_from(s_in);
		s_out->change_xref(FILL_VALUE, fill_out);

		if( hasD0){
			s_out->change_xref(DEPEND_0 , D0_in );
		}

		if( QuIsMatch( *s_out, fill)) {
			*s_out = newValue;
		}

		out[0]->setValue(s_out, true);
		return;
	}

   // string and other input types not cleaned - return input object

	message=QString(" Fillval Conversion not handled for this data type\n" );
	ok=false;
	return;
}
