#include "qcalcEvaluator.h"


void QEvaluatorMod::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdObject_var result;
	string attrPtr;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	// do arithmetic...
	try
	{
		result = mod(val1);
	}
	catch(Exception &e)
	{
		message=QString(e.msg().c_str());
		message+=QString("The operation Mod is not yet supported for this input type");
		ok=false;
		return;
	}
	
	if(result.is_nil())
	{
		message=QString("Arithmetic>> Mod Operation is not supported for this input type");
		ok=false;
		return;
	}
	
	// get xrefs from inputs if requested
	result->use_xrefs_from(val1);

	// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
	QdObject_var qd0 = get_Depend_0( val1 );
	if ( !qd0.is_nil() ) result->change_xref(DEPEND_0 , qd0 );
	
	// Set QSAS essential attributes
	// SI_conversion
	attrPtr = QuGetAttrText( "SI_conversion", val1 );
	if( !attrPtr.empty() )
	{
		QuSetTxtAttr( "SI_conversion", attrPtr.c_str(), result );
	}
	
	// Frame
	// output is always a scalar
	QuSetFrame(result , "scalar" );

	// UNITS
	attrPtr = QuGetAttrText( "UNITS", val1 );
	if( !attrPtr.empty() )
	{
		QuSetTxtAttr( "UNITS", attrPtr.c_str(), result );
	}
	
	// LABLAXIS
	string labl_s = "|";
	labl_s += QuGetAttrText( "LABLAXIS", val1 );
	labl_s += "|";
	QuSetTxtAttr( "LABLAXIS", labl_s.c_str(), result );


	out[0]->setValue(result, true);
}

