#include "qcalcEvaluator.h"


void QEvaluatorOpp::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdObject_var result;
	string  attrPtr;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	// do arithmetic...
	try
	{
		result = - val1;
	}
	catch( Exception &e )
	{
		message=QString(e.msg().c_str());
		message+=QString(" This Operation Change Sign is not yet supported between these input types");
		ok=false;
		return;
	}

	if (result.is_nil())
	{
		message=QString("This Operation change sign is not yet supported for this input type" );
		ok=false;
		return;
	}

	// get xrefs from inputs if requested
	result->use_xrefs_from(val1);

	// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
	QdObject_var qd0 = get_Depend_0( val1 );
	if ( !qd0.is_nil() ) result->change_xref(DEPEND_0 , qd0 );


	// Set QSAS essential attributes

	// SI_conversion
	attrPtr = QuGetAttrText( "SI_conversion", val1);
	if( !attrPtr.empty() )
	{
		QuSetTxtAttr( "SI_conversion", attrPtr.c_str(), result );
	}

	// Frame
	attrPtr = QuGetFrameAttr(val1);
	if( !attrPtr.empty() )
	{
		QuSetFrame(result, attrPtr.c_str() );
	}

	// UNITS
	attrPtr = QuGetAttrText( "UNITS", val1 );
	if( !attrPtr.empty() )
	{
		QuSetTxtAttr( "UNITS", attrPtr.c_str(), result );
	}

	// DEPEND_i
	if (val1->xref_exists(DEPEND_1)) 
	{
		QdObject_var d1_xref = val1->get_xref(DEPEND_1);
		result->change_xref( DEPEND_1, d1_xref  );
	}
   
	if (val1->xref_exists(DEPEND_2)) 
	{
		QdObject_var d1_xref = val1->get_xref(DEPEND_2);
		result->change_xref( DEPEND_2, d1_xref  );
	}

	if (val1->xref_exists(DEPEND_3)) 
	{
		QdObject_var d1_xref = val1->get_xref(DEPEND_3);
		result->change_xref( DEPEND_3, d1_xref  );
	}
	
	out[0]->setValue(result, true);
}
