#include "qcalcEvaluator.h"


void QEvaluatorPiecewise::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdObject_var result;
	string attrPtr;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	QdObject_var val2=in[1]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[1]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	// Test validity of inputs...
	int nQarStatus =  QarTestConformal( val1, val2 );
	if ( nQarStatus == QAR_ERR )
	{
		message=QString("Sequence size or join error: Notify csc_support");
		ok=false;
		return;
	}
	
	
	// do arithmetic...
	try
	{
		// Not provided by qdos so handle explicitly all cases
		QdRMatrix_var mout;
		QdRMatrixSeq_var msout;
		
		QdRMatrix_var m2 = QdRMatrix_var::narrow(val2);
		QdRMatrixSeq_var ms2 = QdRMatrixSeq_var::narrow(val2);

		// Already tested that dims match
		Dim_var dim_ptr;
		QdRMatrix_var m1 = QdRMatrix_var::narrow(val1);
		if(!m1.is_nil())
		{
			// obj 1 is matrix obj
			dim_ptr = m1->dimvar();
			
			if(!m2.is_nil())
			{
				mout = Factory::clone(*m2);
				result = mout;
				RMatrix::iterator itr=mout->begin();
				RMatrix::iterator itr1=m1->begin();
				while(itr!=mout->end() )
				{
					*itr *= *itr1;
					++itr;
					++itr1;
				}

			}
			else if (!ms2.is_nil())
			{
				msout = Factory::clone(*ms2);
				result = msout;
				for(int j=0; j<ms2->sequence_size(); j++)
				{
					RMatrix::iterator itr=msout[j].begin();
					RMatrix::iterator itr1=m1->begin();
					while(itr!=msout[j].end() )
					{
						*itr *= *itr1;
						++itr;
						++itr1;
					}
				}
			}
			else
			{
				ok=false;
				return;
			}
		}
		else
		{
			// obj 1 is matrix sequence
			QdRMatrixSeq_var ms1 = QdRMatrixSeq_var::narrow(val1);
			if(ms1.is_nil() || ms1->sequence_size() == 0)
			{
				ok=false;
				return;
			}
			dim_ptr = ms1[0].dimvar();
			if(!m2.is_nil())
			{
				msout = Factory::clone(*ms1);
				result = msout;
				for(int j=0; j<ms1->sequence_size(); j++)
				{
					// iterator for matrix
					RMatrix::iterator itr=msout[j].begin();
					RMatrix::iterator itr2=m2->begin();
					while(itr!=msout[j].end() )
					{
						*itr *= *itr2;
						++itr;
						++itr2;
					}
				}			
			}
			else if (!ms2.is_nil())
			{
				msout = Factory::clone(*ms1);
				result = msout;
				for(int j=0; j<ms1->sequence_size(); j++)
				{
					// iterator for matrix
					RMatrix::iterator itr=msout[j].begin();
					RMatrix::iterator itr2=ms2[j].begin();
					while(itr!=msout[j].end() )
					{
						*itr *= *itr2;
						++itr;
						++itr2;
					}
				}
			}
			else
			{
				ok=false;
				return;
			}
			
		}
		
		
	}
	catch( Exception &e )
	{
		message=QString(e.msg().c_str());
		ok=false;
		return;
	}
	
	if ( result.is_nil() )
	{
		message="Arithmetic>> This Operation is not supported between these input types" ;
		ok=false;
		return;
	}
	
	if(val1->xref_exists(DEPEND_1) && val1->xref_exists(DEPEND_0)) result->copy_xrefs_from(val1);
	else if(val2->xref_exists(DEPEND_1) && val2->xref_exists(DEPEND_0)) result->copy_xrefs_from(val2);
	else 
	{
		if(val1->xref_exists(DEPEND_1) ) result->copy_xrefs_from(val1);
		else if(val2->xref_exists(DEPEND_1) ) result->copy_xrefs_from(val2);
		
		// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
		QdObject_var qd0 = get_Depend_0( val1 );
		if ( qd0.is_nil() ) qd0 = get_Depend_0( val2 );
		if ( !qd0.is_nil() ) result->change_xref(DEPEND_0 , qd0 );
	
	}
	
	// Fix some attributes
	
	// SI_conversion
	attrPtr = QarSIConv_product( val1, val2 );
	if ( !attrPtr.empty() )
	{
		QuSetTxtAttr( "SI_conversion", attrPtr.c_str(), result );
	}
	

	// UNITS
	attrPtr = QarUNITS_product( val1, val2 );
	if ( !attrPtr.empty() )
	{
		QuSetTxtAttr("UNITS", attrPtr.c_str(), result);
	}
	
	out[0]->setValue(result, true);
	
	ok=true;
}

// --------------------------------------------------------------------

void QEvaluatorPwDiv::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdObject_var result;
	string attrPtr;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	QdObject_var val2=in[1]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[1]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	// Test validity of inputs...
	int nQarStatus =  QarTestConformal( val1, val2 );
	if ( nQarStatus == QAR_ERR )
	{
		message=QString("Sequence size or join error: Notify csc_support");
		ok=false;
		return;
	}
	
	
	// do arithmetic...
	try
	{
		// Not provided by qdos so handle explicitly all cases
		QdRMatrix_var mout;
		QdRMatrixSeq_var msout;
		
		QdRMatrix_var m2 = QdRMatrix_var::narrow(val2);
		QdRMatrixSeq_var ms2 = QdRMatrixSeq_var::narrow(val2);

		// Already tested that dims match
		Dim_var dim_ptr;
		QdRMatrix_var m1 = QdRMatrix_var::narrow(val1);
		if(!m1.is_nil())
		{
			// obj 1 is matrix obj
			dim_ptr = m1->dimvar();
			
			if(!m2.is_nil())
			{
				mout = Factory::clone(*m1);
				result = mout;
				RMatrix::iterator itr1=mout->begin();
				RMatrix::iterator itr2=m2->begin();
				while(itr1!=mout->end() )
				{
					*itr1 /= *itr2;
					++itr2;
					++itr1;
				}

			}
			else if (!ms2.is_nil())
			{
				msout = Factory::clone(*ms2);
				result = msout;
				for(int j=0; j<ms2->sequence_size(); j++)
				{
					RMatrix::iterator itr=msout[j].begin();
					RMatrix::iterator itr1=m1->begin();
					while(itr!=msout[j].end() )
					{
						*itr = (*itr1)/(*itr);
						++itr;
						++itr1;
					}
				}
			}
			else
			{
				ok=false;
				return;
			}
		}
		else
		{
			// obj 1 is matrix sequence
			QdRMatrixSeq_var ms1 = QdRMatrixSeq_var::narrow(val1);
			if(ms1.is_nil() || ms1->sequence_size() == 0)
			{
				ok=false;
				return;
			}
			dim_ptr = ms1[0].dimvar();
			if(!m2.is_nil())
			{
				msout = Factory::clone(*ms1);
				result = msout;
				for(int j=0; j<ms1->sequence_size(); j++)
				{
					// iterator for matrix
					RMatrix::iterator itr=msout[j].begin();
					RMatrix::iterator itr2=m2->begin();
					while(itr!=msout[j].end() )
					{
						*itr /= *itr2;
						++itr;
						++itr2;
					}
				}			
			}
			else if (!ms2.is_nil())
			{
				msout = Factory::clone(*ms1);
				result = msout;
				for(int j=0; j<ms1->sequence_size(); j++)
				{
					// iterator for matrix
					RMatrix::iterator itr=msout[j].begin();
					RMatrix::iterator itr2=ms2[j].begin();
					while(itr!=msout[j].end() )
					{
						*itr /= *itr2;
						++itr;
						++itr2;
					}
				}
			}
			else
			{
				ok=false;
				return;
			}
			
		}
		
		
	}
	catch( Exception &e )
	{
		message=QString(e.msg().c_str());
		ok=false;
		return;
	}
	
	if ( result.is_nil() )
	{
		message="Arithmetic>> This Operation is not supported between these input types" ;
		ok=false;
		return;
	}
	if(val1->xref_exists(DEPEND_1) && val1->xref_exists(DEPEND_0)) result->copy_xrefs_from(val1);
	else if(val2->xref_exists(DEPEND_1) && val2->xref_exists(DEPEND_0)) result->copy_xrefs_from(val2);
	else 
	{
		if(val1->xref_exists(DEPEND_1) ) result->copy_xrefs_from(val1);
		else if(val2->xref_exists(DEPEND_1) ) result->copy_xrefs_from(val2);
		
		// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
		QdObject_var qd0 = get_Depend_0( val1 );
		if ( qd0.is_nil() ) qd0 = get_Depend_0( val2 );
		if ( !qd0.is_nil() ) result->change_xref(DEPEND_0 , qd0 );	
	}
	
	// Fix some attributes
	
	// SI_conversion
	attrPtr = QarSIConv_inverse( val2 );
	if ( !attrPtr.empty() )
	{
		// set to inverse of 2nd operator
		QuSetTxtAttr( "SI_conversion", attrPtr.c_str(), result );
	}	
	attrPtr = QarSIConv_product( val1, result );
	if ( !attrPtr.empty() )
	{
		// Then replace with correct ratio
		QuSetTxtAttr( "SI_conversion", attrPtr.c_str(), result );
	}
	

	// UNITS
	attrPtr = QuGetAttrText("UNITS", val2);
	if ( !attrPtr.empty() )
	{
		string string_buf = "(";
		string_buf += attrPtr;
		string_buf += ")^-1";
		QuSetTxtAttr("UNITS", string_buf.c_str(), result);
	}
	attrPtr = QarUNITS_product( val1, result );
	if ( !attrPtr.empty() )
	{
		QuSetTxtAttr("UNITS", attrPtr.c_str(), result);
	}

	
	out[0]->setValue(result, true);
	
	ok=true;
}
