#include "qcalcEvaluator.h"

const double PI_  = 4.0 * atan(1.0);

enum    UniteEnum { DEG_ID = 0, RAD_ID = 1, DEFAULT_ID = 2 };

typedef QdObject_var ( *QdObjectFnPtr )( QdObject_var qdOperand1, QString* pstrErrorMsg);

QdObject_var QdAcosLocalCalc2(     QdObject_var  qdOperand1, QString* pstrErrorMsg )
{
	QdRScalar_var one = new QdRScalar( 1.0 );
	QdObject_var  tmpObj;
	QdObject_var  qdResult;

	tmpObj = abs( qdOperand1 );
	if ( tmpObj > ( QdObject_var )one )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute acos of |data|>1." );
		return ( qdResult );
	}

	qdResult =  acos( qdOperand1 ) ;
	return ( qdResult );
}

QdObject_var QdAsinLocalCalc2(     QdObject_var  qdOperand1, QString* pstrErrorMsg )
{
	QdObject_var  qdResult;
	QdRScalar_var one = new QdRScalar( 1.0 );
	if ( one.is_nil() )
	{
		return ( qdResult );
	}
	QdObject_var  tmpObj;


	tmpObj = abs( qdOperand1 );
	if ( tmpObj > ( QdObject_var )one )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute acos of |data|>1." );
		return ( qdResult );
	}

	qdResult =  asin( qdOperand1 ) ;
	return ( qdResult );
}

QdObject_var QdAtanLocalCalc2(     QdObject_var  qdOperand1, QString* pstrErrorMsg )
{
	QdObject_var  qdResult =  atan( qdOperand1 ) ;
	return ( qdResult );
}


QdObject_var QdAcotLocalCalc2(     QdObject_var  qdOperand1, QString* pstrErrorMsg )
{
	QdObject_var  qdResult;
	QdRScalar_var one = new QdRScalar( 1.0 );
	if ( one.is_nil() )
	{
		return ( qdResult );
	}
	qdResult =  atan( ( QdObject_var )one / qdOperand1 ) ;
	return ( qdResult );
}

QdObject_var QdAcoshLocalCalc2(    QdObject_var  qdOperand1, QString* pstrErrorMsg )
{
	QdObject_var  qdResult;
	QdRScalar_var one = new QdRScalar( 1.0 );
	if ( one.is_nil() )
	{
		return ( qdResult );
	}

	if ( qdOperand1 < ( QdObject_var )one )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute acosh of data<1." );
		return ( qdResult );
	}

	qdResult =  log( qdOperand1 + sqrt( qdOperand1*qdOperand1 - ( QdObject_var )one ) ) ;
	return ( qdResult );
}

QdObject_var QdAsinhLocalCalc2(    QdObject_var  qdOperand1, QString* pstrErrorMsg )
{
	QdObject_var  qdResult;
	QdRScalar_var one = new QdRScalar( 1.0 );
	if ( one.is_nil() )
	{
		return ( qdResult );
	}
	qdResult =  log( qdOperand1 + sqrt( qdOperand1 * qdOperand1 + ( QdObject_var )one ) ) ;
	return ( qdResult );
}

QdObject_var QdAtanhLocalCalc2(    QdObject_var  qdOperand1, QString* pstrErrorMsg )
{
	QdObject_var  qdResult;
	QdRScalar_var one = new QdRScalar( 1.0 );
	if ( one.is_nil() )
	{
		return ( qdResult );
	}
	QdObject_var  tmpObj;
	QdRScalar_var half = new QdRScalar( 0.5 );
	if ( half.is_nil() )
	{
		return ( qdResult );
	}

	tmpObj = abs( qdOperand1 );
	if ( tmpObj >= ( QdObject_var )one )
	{
		*pstrErrorMsg = QString( "Error>> Cannot compute atanh of |data|>= 1" );
		return ( qdResult );
	}

	qdResult =  ( QdObject_var )half * log( ( ( QdObject_var )one + qdOperand1 ) / ( ( QdObject_var )one - qdOperand1 ) ) ;
	return ( qdResult );
}
  
QdObject_var QdAcothLocalCalc2(    QdObject_var  qdOperand1, QString* pstrErrorMsg )
{
	QdObject_var  qdResult;
	QdRScalar_var one  = new QdRScalar( 1.0 );
	if ( one.is_nil() )
	{
		return ( qdResult );
	}

	QdRScalar_var half = new QdRScalar( 0.5 );
	if ( half.is_nil() )
	{
		return ( qdResult );
	}

	qdResult =  ( QdObject_var )half * log( ( ( QdObject_var )one + qdOperand1 ) / ( qdOperand1 - ( QdObject_var )one ) ) ;
	return ( qdResult );
}

QdObject_var QdTangentCalc2(   QdObject_var  qdOperand1, QString* pstrErrorMsg )
{
	QdObject_var qdResult =  sin( qdOperand1 ) / cos( qdOperand1 ) ;
	return ( qdResult );
}


QdObject_var QdCTangentCalc2(  QdObject_var  qdOperand1, QString* pstrErrorMsg )
{
	QdObject_var  qdResult =  cos( qdOperand1 ) / sin( qdOperand1 ) ;
	return ( qdResult );
}


QdObject_var QdTangentHCalc2(  QdObject_var  qdOperand1, QString* pstrErrorMsg )
{
	QdObject_var  qdResult =  sinh( qdOperand1 ) / cosh( qdOperand1 ) ;
	return ( qdResult );
}


QdObject_var QdCTangentHCalc2( QdObject_var  qdOperand1, QString* pstrErrorMsg )
{
	QdObject_var  qdResult =  cosh( qdOperand1 ) / sinh( qdOperand1 ) ;
	return ( qdResult );
}


QdObject_var QdcosLocalCalc2(       QdObject_var  qdOperand1, QString* pstrErrorMsg )
{
	return ( cos( qdOperand1 ) );
}

QdObject_var QdsinLocalCalc2(       QdObject_var  qdOperand1, QString* pstrErrorMsg )
{
	return ( sin( qdOperand1 ) );
}

QdObject_var QdcoshLocalCalc2(      QdObject_var  qdOperand1, QString* pstrErrorMsg )
{
	return ( cosh( qdOperand1 ) );
}

QdObject_var QdsinhLocalCalc2(      QdObject_var  qdOperand1, QString* pstrErrorMsg )
{
	return ( sinh( qdOperand1 ) );
}




QdObject_var QdTrigFunction2 ( QdObjectFnPtr pFn,
			      QdObject_var  qdOperand1,
	 QdObject_var  qdOperand2,
  QString&      rqstrErrorMsg,
  QdString_var  funct,
  bool          bInverse,
  bool          bHyperbolic=false)
{
	QdObject_var qdResult;

	if ( ( qdOperand1.is_nil() )/* || ( qdOperand2.is_nil() ) */)
	{
        // The object will be empty - this should never happen
	    // The qdResult object is NULL at this point
		return ( qdResult );
	}

	if ( funct.is_nil() )
	{
		return ( qdResult );
	}

	if ( pFn == NULL )
	{
		return ( qdResult );
	}

	char*        winMsg           = new char[ 100 ];
	if ( winMsg == NULL )
	{
		return ( qdResult );
	}

	QdRScalar_var unit        = new QdRScalar( 1.0 );
	if ( unit.is_nil() )
	{
		return ( qdResult );
	}

	QdObject_var  DegToRadObj = new QdRScalar( PI_ / 180.0 );
	if ( DegToRadObj.is_nil() )
	{
		return ( qdResult );
	}

	QdObject_var  tmpObj      = qdOperand1;
	QdObject_var  anyObj      = qdOperand1;

	if ( qdOperand1.is_nil() )
	{
// 		QcalcAppendTextDisplay( "Error>> 1st input neither const nor sequence." );
		rqstrErrorMsg="Error>> 1st input neither const nor sequence.";
		return ( qdResult );
	}

	QdRScalar_var qdDegRad = QdRScalar_var::narrow( qdOperand2 );
	if ( qdDegRad.is_nil() && !bHyperbolic)
	{

// 		QcalcAppendTextDisplay( "Error>> Deg/Rad unspecified." );
		rqstrErrorMsg="Error>> Deg/Rad unspecified.";
		return ( qdResult );
	}

	int nDegRadId = (qdDegRad.is_nil() ? 0 : ( int )( *qdDegRad ));

	//-----check and read what's in the SI_conversion ----------

	string SIconvr   =  QuGetAttrText( "SI_conversion", qdOperand1 );

	string rbuf;
	if ( SIconvr.empty() )
	{
		SIconvr = "1 > unk";
		sprintf( winMsg, "Data SI_conversion: %s \n", SIconvr.c_str() );
	}

	size_t sepPtr = SIconvr.find(">");
	
	if( sepPtr != string::npos )
	{
		sepPtr++;
		size_t endPtr = SIconvr.size();
		endPtr--;
		
		unit = new QdRScalar( atof(SIconvr.c_str() ) );
		
		//pick up the RHS of the SI_conversion and put it into rbuf
		// remove white space from start
		while(SIconvr[sepPtr] == ' ' && sepPtr < endPtr) sepPtr++;
		
		//remove white space from end
		while(SIconvr[endPtr] == ' ' && sepPtr < endPtr) endPtr--;

		rbuf = SIconvr.substr(sepPtr, endPtr - sepPtr);
		
	}
	else
	{
		//  unit already set to 1.0
		
		// rhs
		rbuf ="unk";
	}


	if ( bInverse )
	{
		if ( ( nDegRadId == DEG_ID ) || ( nDegRadId == RAD_ID ) )
		{
			if ( bHyperbolic )
			{
				rqstrErrorMsg = QString( "Error>> output in degree/rad for non-hyperbolic functions only" );
				return ( qdResult );
			}
		}

		if ( ( nDegRadId == DEFAULT_ID ) && ( !bHyperbolic ) )
		{
			rqstrErrorMsg = QString( "Error>> must specify degree or rad for non-hyperbolic functions." );
			return ( qdResult );
		}
	}
	else if ( !bHyperbolic )
	{
		//if nDegRadId is degree or rad than ignore SI_conversion as it probably isn't a valid angle : pulldown is set in GUI
		switch ( nDegRadId )
		{
			case DEG_ID:
				if ( rbuf.find("deg" ) == string::npos )
				{
// 					QcalcAppendTextDisplay( "WARNING>> Ignoring invalid SI_conversion" ); //TODO add warnings
					rbuf += " as deg";
				}
				else
				{
					//if there's a match, then get the right unit
					if ( *unit != 1.0 )
					{
						tmpObj  = ( ( QdObject_var) unit ) * qdOperand1;
					}
				}
				anyObj = DegToRadObj * tmpObj;//transform to rad
				break;

			case RAD_ID:
				if ( rbuf.find("rad") == string::npos )
				{
// 					QcalcAppendTextDisplay( "WARNING>> Ignoring invalid SI_conversion" ); //TODO add warnings
					rbuf += " as rad";
				}
				else
				{
					if ( *unit != 1.0 )
					{
						anyObj = ( ( QdObject_var) unit ) * qdOperand1;
					}
				}
				break;

			case DEFAULT_ID:
				if ( rbuf.find("rad" ) != string::npos )
				{
					if ( *unit != 1.0 )
					{
						anyObj  = ( ( QdObject_var )unit ) * qdOperand1;
					}
				}
				else if	( rbuf.find("deg" ) != string::npos )
				{
					if ( *unit != 1.0 )
					{
						tmpObj  = ( ( QdObject_var )unit )*qdOperand1;
					}

					anyObj = DegToRadObj * tmpObj;//transform to rad
				}
				else 
				{
					//if not hyperbolic function
			
					rqstrErrorMsg = QString( "Error>> Input is neither in rad nor in degree. Please specify" );
					return ( qdResult );
				}
				break;

			default:
				rqstrErrorMsg = QString( "ERROR>> radian/degree flag choice not set. Notify csc support" );
				return ( qdResult );
				break;

		}
	}

	// DO TRIG OPERATION
	try
	{
		if ( bInverse )
		{
			qdResult = ( *pFn )( anyObj, &rqstrErrorMsg );
			if ( !rqstrErrorMsg.isEmpty() )
			{
				return ( qdResult );
			}
		}
		else
		{
			qdResult = ( *pFn )( anyObj, NULL );
		}
	}
	catch( Exception & e)
	{
		rqstrErrorMsg = QString( e.msg().c_str() );
		rqstrErrorMsg+= QString( " Error>> Nil Exception..." );
		return ( qdResult );
	}


	if ( ( qdResult.is_nil() ) || ( is_empty( qdResult ) ) )
	{
		rqstrErrorMsg =  QString( "Error>> invalid input to the trigonometry function. Empty object returned." );
		return ( qdResult );
	}

	if ( bInverse )
	{
		QdRScalar_var qdRadToDeg = new QdRScalar( 180.0 / PI_ );
		if ( nDegRadId == DEG_ID ) //then transform to output to degree
		{
			qdResult = ( ( QdObject_var )qdRadToDeg ) * qdResult;
		}
	}

	// Set xrefs
	
	QdString_var newSI = new QdString( "1.0 >" );
	if ( newSI.is_nil() )
	{
		return ( qdResult );
	}

	if ( bInverse && !bHyperbolic )
	{
		if (nDegRadId == RAD_ID) *newSI += "radian";
		else if(nDegRadId == DEG_ID) *newSI += "degree";
		else *newSI += "(unknown)";
	}
	else 
	{
		*newSI += "(";
		*newSI += *funct;
		*newSI += rbuf.c_str();
		*newSI += "))";
	}
	

	qdResult->change_xref( "SI_conversion", newSI );

	QdString_var unit_s = new QdString( *funct );
	
	if ( bInverse && !bHyperbolic )
	{
		if (nDegRadId == RAD_ID) *unit_s = "rad";
		else if(nDegRadId == DEG_ID) *unit_s = "deg";
		else {
			*unit_s += QuGetAttrText( "UNITS", qdOperand1 ).c_str() ;
			*unit_s += ")";
		}
	}
	else 
	{
		*unit_s += QuGetAttrText( "UNITS", qdOperand1 ).c_str();
		*unit_s += ")";
	}
		
	qdResult->change_xref( "UNITS", unit_s);

	QdString_var labl_s = new QdString( *funct );

	*labl_s += QuGetAttrText( "LABLAXIS", qdOperand1 ).c_str();
	*labl_s += ")";


	qdResult->change_xref( "LABLAXIS", labl_s );

	if(qdOperand1->xref_exists("FILLVAL")){
		qdResult->change_xref("FILLVAL", qdOperand1->get_xref("FILLVAL"));
	}
	if(qdOperand1->xref_exists("SCALEMAX")){
		qdResult->change_xref("SCALEMAX", qdOperand1->get_xref("SCALEMAX"));
	}
	if(qdOperand1->xref_exists("SCALEMIN")){
		qdResult->change_xref("SCALEMIN", qdOperand1->get_xref("SCALEMIN"));
	}
	if(qdOperand1->xref_exists("SCALETYP")){
		qdResult->change_xref("SCALETYP", qdOperand1->get_xref("SCALETYP"));
	}


	// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
	QdObject_var qd0 = get_Depend_0( qdOperand1 );
	if ( !qd0.is_nil() ) qdResult->change_xref(DEPEND_0 , qd0 );


	return ( qdResult );
}

void QEvaluatorTrig::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdObject_var result;

// 	QEvaluator::evaluate(in, out, ok, message);
// 	if(!ok)
// 		return;
	
	if(in[0]->connectedSlot()==NULL)
	{
		ok=false;
		message=QString("Missing input 1");
		return;
	}
	
	if(op==QCALC_ATAN2 && in[1]->connectedSlot()==NULL)
	{
		ok=false;
		message=QString("Missing input 2");
		return;
	}
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	QdObject_var val2;

	if(op==QCALC_COS || op==QCALC_SIN || op==QCALC_TAN || op==QCALC_COT || op==QCALC_ACOS || op==QCALC_ASIN || op==QCALC_ATAN || op==QCALC_ACOT)
	{
		if(in[1]->connectedSlot()!=NULL)
		{
			val2=in[1]->connectedSlot()->getValue(ok);
			if(!ok)
			{
				message=QString("Input ")+in[1]->getName()+QString(" invalid");
				ok=false;
				return;
			}
			QdString_var val2str=QdString_var::narrow(val2);
			if(val2str!=NULL)
			{
				if(val2str->compare("deg")==0 || val2str->compare("Deg")==0 || val2str->compare("DEG")==0)
				{
					val2=new QdRScalar(DEG_ID);
				}
				else if(val2str->compare("rad")==0 || val2str->compare("Rad")==0 || val2str->compare("RAD")==0)
				{
					val2=new QdRScalar(RAD_ID);
				}
				else
				{
					message=QString("Input 2 must be deg or rad");
					ok=false;
					return;
				}
			}
			else
			{
				message=QString("Input 2 must be deg or rad");
				ok=false;
				return;
			}
			
		}
		else
		{
			val2=new QdRScalar(DEG_ID);
		}
	}
	
	switch(op)
	{
		case QCALC_SIN:
			result=QdTrigFunction2( &QdsinLocalCalc2, val1, val2, message, new QdString(  "sin(" ), false);
			break;
		case QCALC_TAN:
			result=QdTrigFunction2( &QdTangentCalc2, val1, val2, message, new QdString(  "tan(" ), false);
			break;
		case QCALC_COT:
			result=QdTrigFunction2( &QdCTangentCalc2, val1, val2, message, new QdString(  "cot(" ), false);
			break;
		case QCALC_COSH:
			result=QdTrigFunction2( &QdcoshLocalCalc2, val1, val2, message, new QdString(  "cosh(" ), false, true);
			break;
		case QCALC_SINH:
			result=QdTrigFunction2( &QdsinhLocalCalc2, val1, val2, message, new QdString(  "sinh(" ), false, true);
			break;
		case QCALC_TANH:
			result=QdTrigFunction2( &QdTangentHCalc2, val1, val2, message, new QdString(  "tanh(" ), false, true);
			break;
		case QCALC_COTH:
			result=QdTrigFunction2( &QdCTangentHCalc2, val1, val2, message, new QdString(  "coth(" ), false, true);
			break;
		case QCALC_ACOS:
			result=QdTrigFunction2( &QdAcosLocalCalc2, val1, val2, message, new QdString(  "acos(" ), true, false);
			break;
		case QCALC_ASIN:
			result=QdTrigFunction2( &QdAsinLocalCalc2, val1, val2, message, new QdString(  "asin(" ), true, false);
			break;
		case QCALC_ATAN:
			result=QdTrigFunction2( &QdAtanLocalCalc2, val1, val2, message, new QdString(  "atan(" ), true, false);
			break;
		case QCALC_ATAN2:
			{		
				QdObject_var val2=in[1]->connectedSlot()->getValue(ok);
				if(!ok)
				{
					message=QString("Input ")+in[1]->getName()+QString(" invalid");
					ok=false;
					return;
				}
				if( !val1.is_nil() && !val2.is_nil()) result = atan2(val1, val2);

				string SIconvr = "1 > radian" ;


				if ( ( result.is_nil() ) || ( is_empty( result ) ) )
				{
					message =  QString( "Error>> invalid input to the trigonometry function. Empty object returned." );
					ok=false;
					return;
				}

				// Set xrefs
	
				QuSetTxtAttr( "SI_conversion", SIconvr.c_str(), result );

				QuSetTxtAttr( "UNITS", "rad", result );

				string labl_txt1 = QuGetAttrText( "LABLAXIS", val1 );
				string labl_txt2 = QuGetAttrText( "LABLAXIS", val2 );
				QdString_var labl_s = new QdString( "atan2(" );


				*labl_s += labl_txt1.c_str();
				*labl_s += ", ";
				*labl_s += labl_txt2.c_str();
				*labl_s += ")";

				result->change_xref( "LABLAXIS", labl_s );

				if(val1->xref_exists("FILLVAL")){
					result->change_xref("FILLVAL", val1->get_xref("FILLVAL"));
				}				

				// Add timeline (or DEPEND_0) : Either timeline is safe if it exists
				QdObject_var qd0 = get_Depend_0( val1 );
				if ( qd0.is_nil() ) qd0 = get_Depend_0( val2 );
				if ( !qd0.is_nil() ) result->change_xref(DEPEND_0 , qd0 );

			}
			break;
		case QCALC_ACOT:
			result=QdTrigFunction2( &QdAcotLocalCalc2, val1, val2, message, new QdString(  "acot(" ), true, false);
			break;
		case QCALC_ACOSH:
			result=QdTrigFunction2( &QdAcoshLocalCalc2, val1, val2, message, new QdString(  "acosh(" ), true, true);
			break;
		case QCALC_ASINH:
			result=QdTrigFunction2( &QdAsinhLocalCalc2, val1, val2, message, new QdString(  "asinh(" ), true, true);
			break;
		case QCALC_ATANH:
			result=QdTrigFunction2( &QdAtanhLocalCalc2, val1, val2, message, new QdString(  "atanh(" ), true, true);
			break;
		case QCALC_ACOTH:
			result=QdTrigFunction2( &QdAcothLocalCalc2, val1, val2, message, new QdString(  "acoth(" ), true, true);
			break;
		case QCALC_COS:
		default:
			result=QdTrigFunction2( &QdcosLocalCalc2, val1, val2, message, new QdString(  "cos(" ), false);
			break;
	}
	
	if(!message.isEmpty() && result.is_nil())
	{
		ok=false;
		return;
	}
	
	// get xrefs from inputs if requested
	result->use_xrefs_from(val1, val2);

	out[0]->setValue(result, true);
}

// --------------------------------

void QEvaluatorAngle360::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdObject_var result;

 	QEvaluator::evaluate(in, out, ok, message);
 	if(!ok)
 		return;
	
	if(in[0]->connectedSlot()==NULL)
	{
		ok=false;
		message=QString("Missing input 1");
		return;
	}
	
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	

	QdObject_var newD0;
	
	QdRMatrix_var M_var;
	QdRMatrixSeq_var MS_var;
	  
	// copy object
	QdRMatrix_var Mnarrow = QdRMatrix_var::narrow(val1);
	QdRMatrixSeq_var MSnarrow = QdRMatrixSeq_var::narrow(val1);
	newD0 = getcopy_Depend_0(val1);
	if( newD0.is_nil() ) {
		message=QString("Input ")+in[0]->getName()+QString(" has no Depend_0");
		ok=false;
		return;
	}
	
	int rank;
	if( !Mnarrow.is_nil() ){
		
		M_var = Factory::clone(Mnarrow);
		val1->copy_xrefs_to(M_var);
		M_var->change_xref(DEPEND_0, newD0);
		result = M_var;
		rank = M_var->rank();
	}
	else if( !MSnarrow.is_nil() ){
		
		MS_var = Factory::clone(MSnarrow);
		val1->copy_xrefs_to(MS_var);
		MS_var->change_xref(DEPEND_0, newD0);
		result = MS_var;
		rank = MS_var[0].rank();
	}
	else
	{
		message=QString("Object not matrix or matrix sequence");
		ok=false;
		return;
	}
	
	// get xrefs from inputs if requested
	result->use_xrefs_from(val1);

		
	// find each DEPEND_i
	for(int i=1; i <= rank; i++){
		QString dep = QString("DEPEND_%1").arg(i);
		if( !result->xref_exists(dep.toLatin1().data()) ) continue;
		QdObject_var xref = result->get_xref(dep.toLatin1().data());
			
		// If angle DEPEND modify
		string angle = QuAngleUnits(xref);
		if( angle.empty() ) continue;
			
		QdObject_var newXref = QuAngleMod(xref, 360, angle.c_str());
		result->change_xref(dep.toLatin1().data(), newXref);
	}

	if(!message.isEmpty() && result.is_nil())
	{
		ok=false;
		return;
	}
	out[0]->setValue(result, true);
}

// --------------------------------

void QEvaluatorAngle180::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdObject_var result;

 	QEvaluator::evaluate(in, out, ok, message);
 	if(!ok)
 		return;
	
	if(in[0]->connectedSlot()==NULL)
	{
		ok=false;
		message=QString("Missing input 1");
		return;
	}
	
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	

	QdObject_var newD0;
	
	QdRMatrix_var M_var;
	QdRMatrixSeq_var MS_var;
	  
	// copy object
	QdRMatrix_var Mnarrow = QdRMatrix_var::narrow(val1);
	QdRMatrixSeq_var MSnarrow = QdRMatrixSeq_var::narrow(val1);
	newD0 = getcopy_Depend_0(val1);
	if( newD0.is_nil() ) {
		message=QString("Input ")+in[0]->getName()+QString(" has no Depend_0");
		ok=false;
		return;
	}
	
	int rank;
	if( !Mnarrow.is_nil() ){
		
		M_var = Factory::clone(Mnarrow);
		val1->copy_xrefs_to(M_var);
		M_var->change_xref(DEPEND_0, newD0);
		result = M_var;
		rank = M_var->rank();
	}
	else if( !MSnarrow.is_nil() ){
		
		MS_var = Factory::clone(MSnarrow);
		val1->copy_xrefs_to(MS_var);
		MS_var->change_xref(DEPEND_0, newD0);
		result = MS_var;
		rank = MS_var[0].rank();
	}
	else
	{
		message=QString("Object not matrix or matrix sequence");
		ok=false;
		return;
	}

	// get xrefs from inputs if requested
	result->use_xrefs_from(val1);
		
	// find each DEPEND_i
	for(int i=1; i <= rank; i++){
		QString dep = QString("DEPEND_%1").arg(i);
		if( !result->xref_exists(dep.toLatin1().data()) ) continue;
		QdObject_var xref = result->get_xref(dep.toLatin1().data());
			
		// If angle DEPEND modify
		string angle = QuAngleUnits(xref);
		if( angle.empty() ) continue;
			
		QdObject_var newXref = QuAngleMod(xref, 180, angle.c_str());
		result->change_xref(dep.toLatin1().data(), newXref);
	}

	if(!message.isEmpty() && result.is_nil())
	{
		ok=false;
		return;
	}
	out[0]->setValue(result, true);
}

