#include "qcalcEvaluator.h"


void QEvaluatorUnits::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	QdObject_var result;
	string attrPtr;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	QdObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	QdObject_var val2=in[1]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[1]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	if(!QarHasUnits(val1))
	{
		message=QString("Input needs SI_conversion for this filter ");
		ok=false;
		return;
	}
	
	if(!QarHasUnits(val2))
	{
		QdString_var strObj = QdString_var::narrow(val2);
		if(strObj.is_nil() || strstr(strObj->c_str(), ">") == NULL )
		{
			message=QString("SI_conversion slot must be text or object with SI_conversion" );
			ok=false;
			return;
		}
	}
	
	string SIcTarget;
	QdString_var objStr=QdString_var::narrow(val2);
	if(objStr.is_nil())
	{
		SIcTarget =  QuGetAttrText("SI_conversion", val2);
	}
	else
	{
		SIcTarget = objStr->c_str();
	}

	if(SIcTarget.empty())
	{
		message=QString("Cannot get SI_conversion target for Unit Conversion\n" );
		ok=false;
		return;
	}


// create output object
  
	try{
		result = QarConvertUnitsTo(val1, SIcTarget.c_str()); 
    // this creates new converted object, copies xrefs across and sets SI_conversion
	}
	catch(Exception &e)
	{
		message=QString(e.msg().c_str() );
		ok=false;
		return;
	}
 
	if ( result.is_nil() )
	{
		message=QString("Cannot convert object to chosen target units\n RHS of target SI_conversion should be of form ...\n");
		attrPtr = QuGetAttrText("SI_conversion", val1);
		size_t ptr = attrPtr.find(">");
		message+=QString(attrPtr.substr(ptr+1).c_str());
		ok=false;
		return;
	}
 
	// get xrefs from inputs if requested
	result->use_xrefs_from(val1, val2);
 
 	// UNITS
	string targetUnits = QuGetAttrText("UNITS", val2);
	if(!targetUnits.empty() )
	{
		QuSetTxtAttr("UNITS", targetUnits.c_str(), result);    
	}
	else{  // create Units from SI_conversion
		if( !SIcTarget.empty() ){
			size_t ptr = SIcTarget.find(">");
			if(ptr != string::npos) {
				string newUnits = SIcTarget.substr(ptr+1);
				if(atof(SIcTarget.c_str()) != 1){
					newUnits += " / ";
					newUnits += SIcTarget.substr(0, ptr);
				}
				QuSetTxtAttr("UNITS", newUnits.c_str(), result);
			}
		}
	}
	
	out[0]->setValue(result, true);
}
