#include "qcalcExtractionItem.h"
#include <QGraphicsProxyWidget>
#include <QTimer>

QVector<VExtractionItem*> ExtractionItems;


VExtractionItem::VExtractionItem():
		VCalcItem()
{
	VInSlot* slot=new VInSlot(this);
	inputs.push_back(slot);
	VOutSlot* slot2=new VOutSlot(this);
	outputs.push_back(slot2);
    gui=NULL;
	extraction_id=counter++;
		name=QString("Extractor #%1").arg(extraction_id);
	objName=QString("Temp/Extraction_%1").arg(extraction_id);
	valueText=objName;
#ifdef GRAPH_DEBUG
	std::cout << "Creating VExtractionItem " << this << std::endl;
#endif
	askUser=true;
	Suffix="";
	InputObjName="";
}

VExtractionItem::~VExtractionItem()
{
#ifdef GRAPH_DEBUG
	std::cout << "Deleting VExtractionItem " << this << std::endl;
#endif
	if(gui!=NULL)
	{
		QGraphicsScene * scene=gui->scene();
		if(scene!=NULL)
		{
			scene->removeItem(gui);
		}
		gui->parentItem=NULL; // thus gui knows it is being deleted
		
		delete gui;
	}
}

ExtractionItem* VExtractionItem::getGUI()
{
	if(gui==NULL)
	{
		gui=new ExtractionItem(this, inputs[0], outputs[0]);
	}
	return gui;
}

void VExtractionItem::removeGUI()
{
	if(gui!=NULL)
	{
		gui->scene()->removeItem(gui);
		delete gui;
		gui=NULL;
	}
}

void VExtractionItem::setComp()
{
	if(gui != NULL) {
		valueText=objName+" "+Suffix.trimmed();
		gui->edit->setText(valueText);
	}
}

void VExtractionItem::reevaluateOutSlots()
{
	
	bool ok;
	if(inputs[0]->connectedSlot()==NULL)
	{
		ok=false;
		if(gui!=NULL) gui->setErrMessage(true, "Missing input");
		return;
	}
	QdObject_var tempObj=inputs[0]->connectedSlot()->getValue(ok);

	if(!ok)
	{
		if(gui!=NULL) gui->setErrMessage(true, QString("Input ")+inputs[0]->getName()+QString(" invalid"));
		return;
	}
		
	QwuiPutOnWL(tempObj, objName, true);

	QdObject_var result;
	if(gui != NULL)
	{
		if(askUser ) {
				gui->waitForValidation();
				askUser = false;
		}
		else{
		 	valueText=objName+" "+Suffix.trimmed();
			gui->edit->setText(valueText);
		}
		result = gui->edit->getSlotObject();
	}
	else{
		valueText=objName+" "+Suffix.trimmed();
		result=QuGetObjByNameAndComp(valueText);
	}

	outputs[0]->setValue(result, true);

	
	if(gui!=NULL) gui->setErrMessage(false, "");
	
}

QDomElement VExtractionItem::toXMLElement(QDomDocument& document) const
{
	QDomElement tag=document.createElement("Extractor");
    if(gui!=NULL)
    {
		tag.appendChild(gui->toXMLElement(document));
    }
	tag.appendChild(createTextXMLNode(document, "ID", calc_id));
	tag.appendChild(createTextXMLNode(document, "Data", valueText));
	tag.appendChild(createTextXMLNode(document, "Page", page));
	tag.appendChild(createTextXMLNode(document, "XPosnInBlackBox", positionInBlackBox.x()));
	tag.appendChild(createTextXMLNode(document, "YPosnInBlackBox", positionInBlackBox.y()));
	
	tag.appendChild(createTextXMLNode(document, "Ask_User", askUser));
	tag.appendChild(createTextXMLNode(document, "Suffix", Suffix));
    
    return tag;
}

void VExtractionItem::fromXMLElement(QDomElement& el, QGraphicsScene* scene)
{
	VCalcItem::fromXMLElement(el, scene);
	QDomElement guiElement=el.firstChildElement("GUI");
	valueText=objName;
	QString val;

	if(getXMLNodeText(el, "XPosnInBlackBox", val))
	{
		positionInBlackBox.setX(val.toDouble());
	}
	if(getXMLNodeText(el, "YPosnInBlackBox", val))
	{
		positionInBlackBox.setY(val.toDouble());
	}
	
	if(getXMLNodeText(el, "Data", val))
	{
		valueText=val;
	}
	if(getXMLNodeText(el, "Page", val))
	{
		page=val.toInt();
	}
	if(getXMLNodeText(el, "Ask_User", val))
	{
		askUser=val.toInt();
	}
	if(getXMLNodeText(el, "Suffix", val))
	{
		Suffix=val;
	}

	if(!guiElement.isNull())
	{
		ExtractionItem* item=this->getGUI();
        scene->addItem(item);
        item->fromXMLElement(guiElement);
		item->edit->setText(valueText);
	}
}

void VExtractionItem::updateSuffix(){

	Suffix = gui->edit->text();
	int pos = Suffix.indexOf(" [");
	Suffix.remove(0,pos);


}
// -----------------

ExtractionItem::ExtractionItem(VExtractionItem* parentItem, VInSlot* input, VOutSlot* output, QGraphicsItem* parent):
		CalcItem(parent)
{
	this->parentItem=parentItem;

	outlineColor=QColor(50, 75, 50);
	color1=QColor(169, 226, 149);
	color2=QColor(155, 207, 136);
		
	title=parentItem->getName();
	
	width=BASE_WIDTH;
	height=BASE_HEIGHT;
	setRect(-width/2., -height/2., width, height);
	
	edit=new QguWLinput(NULL, "");
	edit->setMaximumWidth((int)(BASE_WIDTH-6));
	edit->setMinimumWidth((int)(BASE_WIDTH-6));
	edit->setMaximumHeight((int)(HEADER_HEIGHT-2));
	edit->setMinimumHeight((int)(HEADER_HEIGHT-2));
	edit->lineBox->setEnabled(false);
    edit->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_ANY, QS_ANY));
	
	QGraphicsProxyWidget * editProxy=new QGraphicsProxyWidget(this);
	editProxy->setWidget(edit);
	editProxy->moveBy(-BASE_WIDTH/2.+3, 0.);
		
	connect(edit, SIGNAL (componentSelected(const char *, const char *)), this, SLOT(buttonPressed()));

	
	InSlot* slot=new InSlot(this, input);
	slot->setPos(0., -height/2.);
	inSlots.push_back(slot);
	
	OutSlot* slot2=new OutSlot(this, output);
	slot2->setPos(0., height/2.);
	outSlots.push_back(slot2);
	
	rzButton->setPos(-width/2.+1.5*HEADER_HEIGHT, -height/2.+HEADER_HEIGHT/2.);
	closeButton->setPos(width/2.-HEADER_HEIGHT/2., -height/2.+HEADER_HEIGHT/2.);
	errButton->setPos(width/2.-1.5*HEADER_HEIGHT, -height/2.+HEADER_HEIGHT/2.);

	connect(edit, SIGNAL(edited_OK()), this, SLOT(setValue()));

	setAcceptDrops(false);
	edit->setAcceptDrops(false);
	edit->setText(parentItem->objName);
	
	calctooltip = NULL;
	
#ifdef GRAPH_DEBUG
	std::cout << "Creating ExtractionItem " << this << " for VExtractionItem "<< parentItem << std::endl;
#endif
}

extern qwuiWindow* qsasWindow;
ExtractionItem::~ExtractionItem()
{
#ifdef GRAPH_DEBUG
	std::cout << "Deleting ExtractionItem " << this << std::endl;
#endif
	foreach(InSlot* slot, inSlots)
	{
		delete slot;
	}
	foreach(OutSlot* slot, outSlots)
	{
		delete slot;
	}
    if(qsasWindow!=NULL)
    {
        QString objName=QString("Temp/Extraction_%1").arg(parentItem->extraction_id);
        qsasWindow->wlview->deleteObjectFromPath(objName);
    }
	if(calctooltip != NULL) delete calctooltip;
}

QVariant ExtractionItem::itemChange(GraphicsItemChange change, const QVariant& value)
{
	if (change == ItemPositionChange && scene()) {
		QPointF newPos = value.toPointF();
		
		updateConnectorsPositions();
		
		return newPos;
	}
	else if(change==ItemSelectedChange && scene())
	{
		if(value.toBool())
		{
			setZValue(1.);
		}
		else setZValue(0.);
		parentItem->setSelected(value.toBool());
	}
	return QGraphicsItem::itemChange(change, value);
}

void ExtractionItem::keyPressEvent(QKeyEvent * e )
{
	if(e->key() == Qt::Key_Escape) okPressed = true;

}

void ExtractionItem::waitForValidation()
{
	calctooltip=new CalcToolTip(this);

	okPressed=false;
	calctooltip->setPos(width/2.-5+calctooltip->boundingRect().width()/2., -height/2.+70 - calctooltip->boundingRect().height()/2.);
	
	QplugAppendTextDisplay("Extractor waiting: Select Slot in Extractor and hit Esc to drop through");
	
	// time out after 2 minutes and carry on
	QTimer::singleShot(120000, this, SLOT(buttonPressed()));
	do
	{
		QCoreApplication::processEvents(QEventLoop::AllEvents, 100);
	} while(!okPressed);


	delete calctooltip;
	calctooltip = NULL;
}

void ExtractionItem::buttonPressed()
{
	
	okPressed=true;
	parentItem->updateSuffix();

}

void ExtractionItem::setValue()
{
	parentItem->valueText=edit->text();
}

void ExtractionItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
{
	CalcItem::paint(painter, option, widget);
}

QDomElement ExtractionItem::toXMLElement(QDomDocument& doc) const
{
	return CalcItem::toXMLElement(doc);
}

void ExtractionItem::fromXMLElement(QDomElement& el)
{
	CalcItem::fromXMLElement(el);
}
