#ifndef QCALC_JOIN_ITEM_H
#define QCALC_JOIN_ITEM_H

#include "qcalcCommon.h"
#include "qcalcCalcItem.h"

// Virtual join item.
class VJoinItem: public VCalcItem
{
	public:
		VJoinItem();
		
		// Also removes the GUI
		virtual ~VJoinItem();
		
		// Returns the associated GUI, if it exists, or NULL
		virtual JoinItem* getGUI();
		
		// if there is a GUI associated, removes it from the scene, and deletes it
		void removeGUI();
		
		// Recomputes the values of the outputs
		virtual void reevaluateOutSlots();
		
        virtual QDomElement toXMLElement(QDomDocument&) const;

        virtual void fromXMLElement(QDomElement& el, QGraphicsScene* scene);
		void setJoinTags(QdObject_var tags){this->tags=tags;}
		
		bool getNoJoin() const {return joinSettings->noJoin;}
		int getAlgorithm() const {return joinSettings->algorithm;}
		double getWidth() const {return joinSettings->width;}
		int getGap() const {return joinSettings->gap;}
		double getGapSize() const {return joinSettings->gapSize;}
		
		void setNoJoin(bool b){joinSettings->noJoin=b; flagOutSlotsForReevaluation();}
		void setAlgorithm(int i){joinSettings->algorithm=i;flagOutSlotsForReevaluation(); }
		void setWidth(double d) {joinSettings->width=d;flagOutSlotsForReevaluation();}
		void setGap(int i) {joinSettings->gap=i;flagOutSlotsForReevaluation();}
		void setGapSize(double d) {joinSettings->gapSize=d;flagOutSlotsForReevaluation();}
		
		static int counter;
		int join_id;
		virtual bool hasGui();
		
	protected:
		void setErrMessage(bool err, const QString& message);
		JoinItem* gui;
		JoinSettings* joinSettings;
		QdObject_var tags;
};

// GUI item associated to a VJoinItem.
class JoinItem: public CalcItem
{
	public:
		JoinItem(VJoinItem* parentItem, VInSlot* input, VOutSlot* output, QGraphicsItem* parent=NULL);
		
		// deletes the slots
		virtual ~JoinItem();
		
		
		virtual void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget=NULL);
		
		inline virtual int type() const { return Type;}
		enum { Type = TYPE_JOINITEM };
		
		VJoinItem* parentItem;
		
		virtual QDomElement toXMLElement(QDomDocument& doc) const;

        virtual void fromXMLElement(QDomElement& el);
		
		virtual int page(){return parentItem->page;}
	protected:
		virtual QVariant itemChange(GraphicsItemChange change, const QVariant& value);
        
};

#endif
