#include "qcalcScene.h"
#include "qcalcOperatorItem.h"
#include "qcalcSlots.h"
#include "qcalcEvaluator.h"
#include "qcalcInputItem.h"
#include "qcalcOutputItem.h"
#include "qcalcJoinItem.h"

extern qwuiWindow* qsasWindow;

VOperatorItem::VOperatorItem(const QCalcOperation& op, const QString& pluginName):
		VCalcItem()
{
// 	for(int i=0; i<2; ++i)
// 	{
// 		VInSlot* slot=new VInSlot(this);
// 		inputs.push_back(slot);
// 	}
// 	for(int i=0; i<1; ++i)
// 	{
// 		VOutSlot* slot=new VOutSlot(this);
// 		outputs.push_back(slot);
// 	}
	operator_id=counter++;
	gui=NULL;
	evaluator=NULL;
	setOperation(op, pluginName);
// 	isBlackBox=false;
	container=NULL;

#ifdef GRAPH_DEBUG
	std::cout << "Creating VOperatorItem " << this << std::endl;
#endif
}

VOperatorItem::~VOperatorItem()
{
#ifdef GRAPH_DEBUG
	std::cout << "Deleting VOperatorItem " << this << std::endl;
#endif
	
	if(isBlackBox)
	{
		foreach(VCalcItem* item, blackBoxList)
		{
			if(CalcScene::currentItems.contains(item)) delete item;
		}
		blackBoxList.clear();
	}
	
	if(gui!=NULL)
	{
		QGraphicsScene * scene=gui->scene();
		if(scene!=NULL)
		{
			scene->removeItem(gui);
		}
		gui->parentItem=NULL; // thus gui knows it is being deleted

		delete gui;
	}
	if(evaluator!=NULL)
		delete evaluator;
}

QDomElement VOperatorItem::toXMLElement(QDomDocument& document) const
{
	QDomElement tag=document.createElement("Operator");
    if(gui!=NULL)
    {
		tag.appendChild(gui->toXMLElement(document));
//         gui->updateDomElement(document, tag);
    }

	tag.appendChild(createTextXMLNode(document, "XPosnInBlackBox", positionInBlackBox.x()));
	tag.appendChild(createTextXMLNode(document, "YPosnInBlackBox", positionInBlackBox.y()));

	tag.appendChild(createTextXMLNode(document, "ID", calc_id));
	tag.appendChild(createTextXMLNode(document, "Type", operation));
	tag.appendChild(createTextXMLNode(document, "Page", page));
	tag.appendChild(createTextXMLNode(document, "Name", name));
	if(operation==QCALC_PLUGIN)
	{
		QString savePluginName = pluginName;
		savePluginName.replace(qsasWindow->qsas_home, "$QSAS_HOME");
		
		tag.appendChild(createTextXMLNode(document, "Plugin_Name", savePluginName));
	}
	else if(operation==QCALC_BLACKBOX)
	{
		QDomElement bb=document.createElement("Black_Box_Items");
		tag.appendChild(bb);
		foreach(VCalcItem* item, blackBoxList)
		{
			bb.appendChild(createTextXMLNode(document, "ID", item->calc_id));
		}
	}
	QDomElement inputSlots=document.createElement("Input_Slots");
	tag.appendChild(inputSlots);
	for(int i=0; i<inputs.size(); ++i)
	{
		QDomElement s=document.createElement("Inslot");
		s.setAttribute("Index", i);
		s.setAttribute("Name", inputs[i]->getName());
		s.setAttribute("Description", inputs[i]->getDescription());
		inputSlots.appendChild(s);
	}
	QDomElement outputSlots=document.createElement("Output_Slots");
	tag.appendChild(outputSlots);
	for(int i=0; i<outputs.size(); ++i)
	{
		QDomElement s=document.createElement("Outslot");
		s.setAttribute("Index", i);
		s.setAttribute("Name", outputs[i]->getName());
		s.setAttribute("Description", outputs[i]->getDescription());
		s.setAttribute("open", (int) outputs[i]->openSlot);
		outputSlots.appendChild(s);
	}
    
    return tag;
}

OperatorItem* VOperatorItem::getGUI()
{
	if(gui==NULL)
	{
		gui=new OperatorItem(this, inputs, outputs);
	}
	return gui;
}

void VOperatorItem::removeGUI()
{
	if(gui!=NULL)
	{
		gui->scene()->removeItem(gui);
		delete gui;
		gui=NULL;
	}
}

void VOperatorItem::setErrMessage(bool err, const QString& message)
{
	if(gui) gui->setErrMessage(err, message);
	else if(container!=NULL)
	{
		container->setErrMessage(err, message);
	}
}

void VOperatorItem::reevaluateOutSlots()
{
	bool ok;
	QString message;
	evaluator->evaluate(inputs, outputs, ok, message);
	if(!ok)
	{
		setErrMessage(true, message);
		for(int i=0; i<outputs.size(); ++i)
		{
			outputs[i]->setValue(QdObject_var(), false);
		}
	} 
	else 
	{
		QuMsgBar(" ", false);
		setErrMessage(false, "");
	}
}

void VOperatorItem::setOperation(QCalcOperation operation, const QString& pluginName)
{
	this->operation=operation;
    this->pluginName=pluginName;
	
	if(evaluator!=NULL)
		delete evaluator;
	
	switch(operation)
	{
		case QCALC_SUB:
			evaluator=new QEvaluatorSubtract;
			setNumSlots(2, 1);
			inputs[0]->setName("-: 1st Operand"); inputs[0]->setDescription("\"A\" in \"A-B\"");
			inputs[1]->setName("-: 2nd Operand"); inputs[1]->setDescription("\"B\" in \"A-B\"");
			setName("A-B");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_MULT:
			evaluator=new QEvaluatorMultiply;
			setNumSlots(2, 1);
			inputs[0]->setName("*: 1st Operand"); inputs[0]->setDescription("\"A\" in \"A*B\"");
			inputs[1]->setName("*: 2nd Operand"); inputs[1]->setDescription("\"B\" in \"A*B\"");
			setName("A*B");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_DIV:
			evaluator=new QEvaluatorDivide;
			setNumSlots(2, 1);
			inputs[0]->setName("/: 1st Operand"); inputs[0]->setDescription("\"A\" in \"A/B\"");
			inputs[1]->setName("/: 2nd Operand"); inputs[1]->setDescription("\"B\" in \"A/B\"");
			setName("A/B");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_MOD:
			evaluator=new QEvaluatorMod;
			setNumSlots(1, 1);
			inputs[0]->setName("Mod: operand"); inputs[0]->setDescription("\"A\" in \"mod(A)\"");
			setName("mod(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_POWER:
			evaluator=new QEvaluatorPow;
			setNumSlots(2, 1);
			inputs[0]->setName("Power: 1st Operand"); inputs[0]->setDescription("\"A\" in \"A^B\"");
			inputs[1]->setName("Power: 2nd Operand"); inputs[1]->setDescription("\"B\" in \"A^B\"");
			setName("A^B");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_OPP:
			evaluator=new QEvaluatorOpp;
			setNumSlots(1, 1);
			inputs[0]->setName("Opposite: operand"); inputs[0]->setDescription("\"A\" in \"-A\"");
			setName("-A");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_REMAINDER:
			evaluator=new QEvaluatorRemainder;
			setNumSlots(2, 1);
			inputs[0]->setName("Remainder: 1st operand"); inputs[0]->setDescription("\"A\" in \"A\%B\"");
			inputs[1]->setName("Remainder: 2nd operand"); inputs[1]->setDescription("\"B\" in \"A\%B\"");
			setName("A\%B");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_ABS:
			evaluator=new QEvaluatorAbs;
			setNumSlots(1, 1);
			inputs[0]->setName("Absolute value: operand"); inputs[0]->setDescription("\"A\" in \"|A|\"");
			setName("|A|");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_LOG:
			evaluator=new QEvaluatorLog;
			setNumSlots(1, 1);
			inputs[0]->setName("ln: operand"); inputs[0]->setDescription("\"A\" in \"ln(A)\"");
			setName("ln(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_LOG10:
			evaluator=new QEvaluatorLog10;
			setNumSlots(1, 1);
			inputs[0]->setName("log: operand"); inputs[0]->setDescription("\"A\" in \"log(A)\"");
			setName("log(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_INVERSE:
			evaluator=new QEvaluatorInverse;
			setNumSlots(1, 1);
			inputs[0]->setName("inverse: operand"); inputs[0]->setDescription("\"A\" in \"1/A\"");
			setName("1/A");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_SQRT:
			evaluator=new QEvaluatorSqrt;
			setNumSlots(1, 1);
			inputs[0]->setName("qsrt: operand"); inputs[0]->setDescription("\"A\" in \"sqrt(A)\"");
			setName("sqrt(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_EXP:
			evaluator=new QEvaluatorExp;
			setNumSlots(1, 1);
			inputs[0]->setName("exp: operand"); inputs[0]->setDescription("\"A\" in \"exp(A)\"");
			setName("exp(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_TRACE:
			evaluator=new QEvaluatorTrace;
			setNumSlots(1, 1);
			inputs[0]->setName("trace: operand"); inputs[0]->setDescription("\"sq matrix A\" for \"trace(A)\"");
			setName("trace(A)");
			outputs[0]->setName("Trace");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_VECTPROD:
			evaluator=new QEvaluatorVectProd;
			setNumSlots(2, 1);
			inputs[0]->setName("x: 1st Operand"); inputs[0]->setDescription("\"A\" in \"AxB\"");
			inputs[1]->setName("x: 2nd Operand"); inputs[1]->setDescription("\"B\" in \"AxB\"");
			setName("AxB");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_DOTPROD:
			evaluator=new QEvaluatorDotProd;
			setNumSlots(2, 1);
			inputs[0]->setName(".: 1st Operand"); inputs[0]->setDescription("\"A\" in \"A.B\"");
			inputs[1]->setName(".: 2nd Operand"); inputs[1]->setDescription("\"B\" in \"A.B\"");
			setName("A.B");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_DIFF3:
			evaluator=new QEvaluatorDiff3;
			setNumSlots(2, 1);
			inputs[0]->setName(".: 1st Operand"); inputs[0]->setDescription("\"A\" in \"dA/dt\" or \"dA/ds\"");
			inputs[1]->setName(".: 2nd Operand"); inputs[1]->setDescription("Gap threshold (sec)");
			setName("3pt estimate, dA/dt");
			outputs[0]->setName("3pt derivative");
			outputs[0]->setDescription(" A wrt Depend_0\n (f(x+h) - f(x-g) / (h+g)");
			break;
		case QCALC_DIFF5:
			evaluator=new QEvaluatorDiff5;
			setNumSlots(1, 1);
			inputs[0]->setName(".: 1st Operand"); inputs[0]->setDescription("\"A\" in \"dA/dt\" or \"dA/ds\"");
			setName("5pt estimate, dA/dt");
			outputs[0]->setName("5pt derivative");
			outputs[0]->setDescription(" A wrt Depend_0\n (-f(x+2h) + 8f(x+h) - 8f(x-h) + f(x-2h) / 12h");
			break;
		case QCALC_INT_DEF:
			evaluator=new QEvaluatorIntDef;
			setNumSlots(2, 1);
			inputs[0]->setName(".: 1st Operand"); inputs[0]->setDescription("\"A\" in \"def integral(A)dx\" ");
			inputs[1]->setName(".: 2nd Operand"); inputs[1]->setDescription("NN=0, linear=1, exclude gaps=2");
			setName("Definite Integral");
			outputs[0]->setName("Integral");
			outputs[0]->setDescription(" definite integral A wrt Depend_0");
			break;
		case QCALC_INT_INDEF:
			evaluator=new QEvaluatorIntIndef;
			setNumSlots(2, 1);
			inputs[0]->setName(".: 1st Operand"); inputs[0]->setDescription("\"A\" in \"integral(A)dx\" ");
			inputs[1]->setName(".: 2nd Operand"); inputs[1]->setDescription("NN=0, linear=1, exclude gaps=2");
			setName("Indefinite Integral");
			outputs[0]->setName("Integral");
			outputs[0]->setDescription(" integral A wrt Depend_0");
			break;
		case QCALC_PIECEWISE:
			evaluator=new QEvaluatorPiecewise;
			setNumSlots(2, 1);
			inputs[0]->setName("x: 1st Operand"); inputs[0]->setDescription("\"A\" in \"AxB\"");
			inputs[1]->setName("x: 2nd Operand"); inputs[1]->setDescription("\"B\" in \"AxB\"");
			setName("A(i,j)*B(i,j)->C(i,j)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_PW_DIV:
			evaluator=new QEvaluatorPwDiv;
			setNumSlots(2, 1);
			inputs[0]->setName("x: 1st Operand"); inputs[0]->setDescription("\"A\" in \"A/B\"");
			inputs[1]->setName("x: 2nd Operand"); inputs[1]->setDescription("\"B\" in \"A/B\"");
			setName("A(i,j)/B(i,j)->C(i,j)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_UNITS:
			evaluator=new QEvaluatorUnits;
			setNumSlots(2, 1);
			inputs[0]->setName("Object"); inputs[0]->setDescription("Object");
			inputs[1]->setName("SIconv"); inputs[1]->setDescription("Siconv");
			setName("Convert Units");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_FILLS:
			evaluator=new QEvaluatorFills;
			setNumSlots(2, 1);
			inputs[0]->setName("Object"); inputs[0]->setDescription("Object");
			inputs[1]->setName("Fill"); inputs[1]->setDescription("Fill");
			setName("Convert Fills");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_ANGLE360:
			evaluator=new QEvaluatorAngle360;
			setNumSlots(1, 1);
			inputs[0]->setName("Object"); inputs[0]->setDescription("Object");
			setName("Shift angle depends");
			outputs[0]->setName("DependsTo360");
			outputs[0]->setDescription("Data with depend angles in (0,360)");
			break;
		case QCALC_ANGLE180:
			evaluator=new QEvaluatorAngle180;
			setNumSlots(1, 1);
			inputs[0]->setName("Object"); inputs[0]->setDescription("Object");
			setName("Shift angle depends");
			outputs[0]->setName("DependsTo180");
			outputs[0]->setDescription("Object with depend angles in (-180,180)");
			break;
		case QCALC_TT_TO_SEC:
			evaluator=new QEvaluatorTTtoSec;
			setNumSlots(2, 1);
			inputs[0]->setName("Object"); inputs[0]->setDescription("Data Object");
			inputs[1]->setName("Reference Time"); inputs[1]->setDescription("Start Time for offset");
			setName("Timetags to seconds offset");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Obj with Depend_0 in sec from Reference");
			break;
		case QCALC_TIVL_TO_SEC:
			evaluator=new QEvaluatorTIVLtoSec;
			setNumSlots(1, 1);
			inputs[0]->setName("Time Interval"); inputs[0]->setDescription("Data Object");
			setName("Time Interval to seconds");
			outputs[0]->setName("Duration");
			outputs[0]->setDescription("Duration of Interval (sec)");
			break;
		case QCALC_TT2_TT1:
			evaluator=new QEvaluatorTT2_TT1toSec;
			setNumSlots(2, 1);
			inputs[0]->setName("Timetags 1"); inputs[0]->setDescription("Data Object");
			inputs[1]->setName("Timetags 2"); inputs[1]->setDescription("Data Object");
			setName("Timetags diff in seconds");
			outputs[0]->setName("TimeDiffs");
			outputs[0]->setDescription("Timetag differences (sec)");
			break;
		case QCALC_T2_T1:
			evaluator=new QEvaluatorT2_T1toSec;
			setNumSlots(2, 1);
			inputs[0]->setName("Time 1"); inputs[0]->setDescription("Data Object");
			inputs[1]->setName("Time 2"); inputs[1]->setDescription("Data Object");
			setName("Time diff in seconds");
			outputs[0]->setName("TimeDiff");
			outputs[0]->setDescription("Time difference (sec)");
			break;
        case QCALC_LIN_REG:
			evaluator=new QEvaluatorLinReg();
			setNumSlots(1, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			setName("Linear Regression");
			outputs[0]->setName("LinearFit");
			outputs[0]->setDescription("Linear least squares fit to input sequence");
			break;
		case QCALC_COS:
			evaluator=new QEvaluatorTrig(QCALC_COS);
			setNumSlots(2, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			inputs[1]->setName("Deg/Rad"); inputs[1]->setDescription("(Optional, if not provided in A) deg or rad");
			setName("cos(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_SIN:
			evaluator=new QEvaluatorTrig(QCALC_SIN);
			setNumSlots(2, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			inputs[1]->setName("Deg/Rad"); inputs[1]->setDescription("(Optional, if not provided in A) deg or rad");
			setName("sin(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_TAN:
			evaluator=new QEvaluatorTrig(QCALC_TAN);
			setNumSlots(2, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			inputs[1]->setName("Deg/Rad"); inputs[1]->setDescription("(Optional, if not provided in A) deg or rad");
			setName("tan(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_COT:
			evaluator=new QEvaluatorTrig(QCALC_COT);
			setNumSlots(2, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			inputs[1]->setName("Deg/Rad"); inputs[1]->setDescription("(Optional, if not provided in A) deg or rad");
			setName("cot(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_ACOS:
			evaluator=new QEvaluatorTrig(QCALC_ACOS);
			setNumSlots(2, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			inputs[1]->setName("Deg/Rad"); inputs[1]->setDescription("(Optional, if not provided in A) deg or rad");
			setName("acos(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_ASIN:
			evaluator=new QEvaluatorTrig(QCALC_ASIN);
			setNumSlots(2, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			inputs[1]->setName("Deg/Rad"); inputs[1]->setDescription("(Optional, if not provided in A) deg or rad");
			setName("asin(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_ATAN:
			evaluator=new QEvaluatorTrig(QCALC_ATAN);
			setNumSlots(2, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			inputs[1]->setName("Deg/Rad"); inputs[1]->setDescription("(Optional, if not provided in A) deg or rad");
			setName("atan(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_ATAN2:
			evaluator=new QEvaluatorTrig(QCALC_ATAN2);
			setNumSlots(2, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			inputs[1]->setName("B"); inputs[1]->setDescription("B");
			setName("atan2(A,B)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_ACOT:
			evaluator=new QEvaluatorTrig(QCALC_ACOT);
			setNumSlots(2, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			inputs[1]->setName("Deg/Rad"); inputs[1]->setDescription("(Optional, if not provided in A) deg or rad");
			setName("acot(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_COSH:
			evaluator=new QEvaluatorTrig(QCALC_COSH);
			setNumSlots(1, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			setName("cosh(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_SINH:
			evaluator=new QEvaluatorTrig(QCALC_SINH);
			setNumSlots(1, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			setName("sinh(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_TANH:
			evaluator=new QEvaluatorTrig(QCALC_TANH);
			setNumSlots(1, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			setName("tanh(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_COTH:
			evaluator=new QEvaluatorTrig(QCALC_COTH);
			setNumSlots(1, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			setName("coth(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_ACOSH:
			evaluator=new QEvaluatorTrig(QCALC_ACOSH);
			setNumSlots(1, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			setName("acosh(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_ASINH:
			evaluator=new QEvaluatorTrig(QCALC_ASINH);
			setNumSlots(1, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			setName("asinh(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_ATANH:
			evaluator=new QEvaluatorTrig(QCALC_ATANH);
			setNumSlots(1, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			setName("atanh(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
		case QCALC_ACOTH:
			evaluator=new QEvaluatorTrig(QCALC_ACOTH);
			setNumSlots(1, 1);
			inputs[0]->setName("A"); inputs[0]->setDescription("A");
			setName("acoth(A)");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
         case QCALC_PLUGIN:
            evaluator=new QEvaluatorPlugin(pluginName);
            setUpPlugin(pluginName);
            break;
		case QCALC_ADD:
		default:
			evaluator=new QEvaluatorAdd;
			setNumSlots(2, 1);
			inputs[0]->setName("+: 1st Operand"); inputs[0]->setDescription("\"A\" in \"A+B\"");
			inputs[1]->setName("+: 2nd Operand"); inputs[1]->setDescription("\"B\" in \"A+B\"");
			setName("+");
			outputs[0]->setName("Output");
			outputs[0]->setDescription("Result of computation");
			break;
	}
	
	if(gui!=NULL)
	{
		gui->setTitle(name);
	}
	
	flagOutSlotsForReevaluation();
}

void VOperatorItem::setUpPlugin(const QString& pluginName)
{
    QEvaluatorPlugin* eval=(QEvaluatorPlugin*)evaluator;
    QStringList pathToPlugin=eval->pluginName.split("/");
    setName(pathToPlugin.last());
//     this->pluginName=pluginName;

    if (eval->qtplFile->openedOK) {
        eval->num_in=0;
        eval->num_out=0;
        for(int i=0; i<eval->qtplFile->qtpl_vars.size(); ++i)
        {
            // 1st pass to count the number of inputs and outputs
            QtplParserVar *pVar = eval->qtplFile->qtpl_vars[i];
            if(pVar->ioType == "INPUT")
                ++eval->num_in;
            else if(pVar->ioType=="OUTPUT" || pVar->ioType=="CALC_OUTPUT")
                ++eval->num_out;
        }
        setNumSlots(eval->num_in, eval->num_out);

        int index_in=0;
        int index_out=0;
        for(int i=0; i<eval->qtplFile->qtpl_vars.size(); ++i)
        {
            QtplParserVar *pVar = eval->qtplFile->qtpl_vars[i];
            if(pVar->ioType == "INPUT")
            {
                inputs[index_in]->setName(pVar->public_name); inputs[index_in]->setDescription(pVar->description);
                ++index_in;
            }
            else if(pVar->ioType=="OUTPUT" || pVar->ioType=="CALC_OUTPUT")
            {
                outputs[index_out]->setName(pVar->public_name); outputs[index_out]->setDescription(pVar->description);
                ++index_out;
            }
        }
    }
    else
    {
        setName("Failed opening plugin");
        setNumSlots(0,0);
    }
}

bool VOperatorItem::isPlugin()
{
    return dynamic_cast<QEvaluatorPlugin*>(evaluator)!=NULL;
}

void VOperatorItem::getPluginInfo(QString& io, QString& help, QString& html)
{
    io="";
    help="";
    html="";
    QEvaluatorPlugin* eval=(QEvaluatorPlugin*)evaluator;
    if(eval==NULL)
    {
        return;
    }
    QStringList pathToPlugin=eval->pluginName.split("/");
    setName(pathToPlugin.last());

    
    if (eval->qtplFile->openedOK)
    {
        // IO
        QString inputs;
        QString outputs;

        inputs="<font size=\"2\"><center><h2>INPUTS</h2></center><br/>";
        outputs="<br/><center><h2>OUTPUTS</h2></center><br/>";

        for(int i=0; i<eval->qtplFile->qtpl_vars.size(); ++i)
        {
            QtplParserVar *pVar = eval->qtplFile->qtpl_vars[i];
            if(pVar->ioType == "INPUT")
            {
                inputs+="<h3>"+pVar->public_name+"</h3><br/>";
                inputs+=pVar->description+"<br/><br/>";
            }
            else if(pVar->ioType=="OUTPUT"|| pVar->ioType=="CALC_OUTPUT")
            {
                outputs+="<h3>"+pVar->public_name+"</h3><br/>";
                outputs+=pVar->description+"<br/><br/>";
            }
        }

        io=inputs+"\n"+outputs+"</font>";

        // HELP

        help=eval->qtplFile->description;

        // HTML
        html = pluginName;
        int posn = html.lastIndexOf("/");
        html.remove(0, posn+1);
        html.prepend("doc/html/PlugIns/");
        html.append(".html");
        const char * home = getenv("QSAS_HOME");
        if(home!=NULL) html.prepend(home);

    }

}

void VOperatorItem::setNumSlots(int nIn, int nOut)
{
	// Save existing connections
	QList<VOutSlot*> inputConnections;
	foreach(VInSlot* inSlot, inputs)
	{
		inputConnections.push_back(inSlot->connectedSlot());
	}

	QList<QList<VInSlot*> > outputConnections;
	foreach(VOutSlot* outSlot, outputs)
	{
		outputConnections.push_back(outSlot->connectedSlots());
	}

	// tell gui item to delete its slots
	if(gui!=NULL) gui->clearSlots();

	//Copy of the existing input and output lists
	
	QList<VInSlot*> oldInputs;
	
	qDeleteAll(inputs);
	qDeleteAll(outputs);
	inputs.clear();
	outputs.clear();
	for(int i=0; i<nIn; ++i)
	{
		VInSlot* slot=new VInSlot(this);
		inputs.push_back(slot);
	}
	for(int i=0; i<nOut; ++i)
	{
		VOutSlot* slot=new VOutSlot(this);
		outputs.push_back(slot);
	}
	// tell gui item to create new slots
	if(gui!=NULL)
	{
		bool exp=gui->isExpanded();
		gui->resetSlots(inputs, outputs);
		if(exp) gui->expand();
	}
	
	// Restore saved connections
	int max=nIn;
	if(max>inputConnections.size())max=inputConnections.size();
	for(int i=0; i<max; ++i)
	{
		if(inputConnections[i]!=NULL)
		{
			if(inputConnections[i]->getGUI() == NULL)continue; // skip empty from objects as constructor rejects them
			Connector* connector=new Connector(inputConnections[i]->getGUI(), inputs[i]->getGUI());
			inputConnections[i]->connectTo(inputs[i]);
			inputConnections[i]->getGUI()->scene()->addItem(connector);
			inputConnections[i]->getGUI()->updateConnectorsPositions();

		}
	}
	max=nOut;
	if(max>outputConnections.size()) max=outputConnections.size();
	for(int i=0; i<max; ++i)
	{
		if(!outputConnections[i].isEmpty())
		{
			foreach(VInSlot* inSlot, outputConnections[i])
			{
				outputs[i]->connectTo(inSlot);
				if(outputs[i]->getGUI() == NULL)continue; // skip empty from objects as constructor rejects them
				Connector* connector=new Connector(outputs[i]->getGUI(), inSlot->getGUI());
				inSlot->getGUI()->scene()->addItem(connector);
				outputs[i]->getGUI()->updateConnectorsPositions();
			}
		}
	}
}

// This is the hard part of the calculator :)
void VOperatorItem::setAsBlackBox(const QList<VCalcItem*>& bb, const QString& newName)
{
	// tell gui item to delete its slots
	if(gui!=NULL) gui->clearSlots();

	blackBoxList=bb;
	isBlackBox=true;
	operation=QCALC_BLACKBOX;
	
	qDeleteAll(inputs);
	qDeleteAll(outputs);
	inputs.clear();
	outputs.clear();
	
	QVector<VInSlot*> savedInSlots; // the connections are lost with all the creations/deletions of objects. We save them here
	QVector<VOutSlot*> savedOutSlots;

	// Saving the slots
	foreach(VCalcItem* item, bb)
	{
		
		foreach(VInSlot* slot, item->inputs) // we use this slot only if it is free or connected to something outside the black box
		{
			VCalcItem* conn_item=slot->connectedItem();
			if(conn_item==NULL) // the slot is not connected. Added to the black box
			{
				inputs.push_back(slot);
			}
			else
			{
				 // The slot is connected. Is is connected within the box or to something outside?
				if(!bb.contains(conn_item)) // Not connected to anythin in bb. But if bb contains a black box, could be connected to an item within this black box
				{
					// check that conn_item is not within a black box that would be in bb
					if(conn_item->container!=NULL)
					{
						if(!bb.contains(conn_item->container))
						{
							inputs.push_back(slot);
						}
					}
					else 
					{
						inputs.push_back(slot);
					}
				}
			}
			if(slot->connectedSlot()!=NULL)
			{
				savedInSlots.push_back(slot);
				savedOutSlots.push_back(slot->connectedSlot());
			}
		}
		foreach(VOutSlot* slot, item->outputs)// we use this slot only if it is free or connected to something outside the black box
		{
			QList<VCalcItem*> conn_items=slot->connectedItems();
			bool somethingConnectedOutsideBB=false;
			for(QList<VCalcItem*>::iterator iter=conn_items.begin(); iter!=conn_items.end() && !somethingConnectedOutsideBB; ++iter)
			{
				somethingConnectedOutsideBB=!(bb.contains(*iter));
			}

			if(conn_items.isEmpty() || somethingConnectedOutsideBB || slot->openSlot)
			{
				outputs.push_back(slot);
				slot->openSlot = true; // set for this test on BB restore
			}
			
			foreach(VInSlot* inslot, slot->connectedSlots())
			{
				savedInSlots.push_back(inslot);
				savedOutSlots.push_back(slot);
			}
		}
		item->container=this;
	}
	
	// Now that we have saved everything, delete the GUIs for the items we include in the box

	foreach(VCalcItem* item, bb)
	{
		VOperatorItem* opItem=dynamic_cast<VOperatorItem*>(item);
		VInputItem* inItem=dynamic_cast<VInputItem*>(item);
		VOutputItem* outItem=dynamic_cast<VOutputItem*>(item);
		VExtractionItem* extItem=dynamic_cast<VExtractionItem*>(item);
		VJoinItem* joinItem=dynamic_cast<VJoinItem*>(item);
		if(opItem!=NULL) opItem->removeGUI();
		if(inItem!=NULL) inItem->removeGUI();
		if(outItem!=NULL) outItem->removeGUI();
		if(extItem!=NULL) extItem->removeGUI();
		if(joinItem!=NULL) joinItem->removeGUI();
	}
	
	// Restoring the connections
	
	for(int i=0; i<savedOutSlots.size(); ++i)
	{
		savedOutSlots[i]->connectTo(savedInSlots[i]);
	}
	
	setName(newName);
	// tell gui item to create new slots
	if(gui!=NULL)
	{
		bool exp=gui->isExpanded();
		gui->resetSlots(inputs, outputs);
		gui->resetConnectors();
		if(exp) gui->expand();
		gui->setTitle(name);
	}
	

}

bool VOperatorItem::hasGui()
{
	return gui!=NULL;
}


void VOperatorItem::setSelected(bool sel)
{
	selected=sel;
	foreach(VCalcItem* item, blackBoxList)
	{
		item->setSelected(sel);
	}
}

void VOperatorItem::fromXMLElement(QDomElement& el, QGraphicsScene* scene)
{
	VCalcItem::fromXMLElement(el, scene);
	QDomElement guiElement=el.firstChildElement("GUI");

	QString val;
	QCalcOperation op;
	QString plugin="";
	if(getXMLNodeText(el, "Type", val))
	{
		op=(QCalcOperation)val.toInt();
	}
	if(getXMLNodeText(el, "Page", val))
	{
		page=val.toInt();
	}
	
	if(getXMLNodeText(el, "XPosnInBlackBox", val))
	{
		positionInBlackBox.setX(val.toDouble());
	}
	if(getXMLNodeText(el, "YPosnInBlackBox", val))
	{
		positionInBlackBox.setY(val.toDouble());
	}


	if(op==QCALC_PLUGIN)
	{
		if(getXMLNodeText(el, "Plugin_Name", val))
		{
			val.replace("$QSAS_HOME", qsasWindow->qsas_home);
			plugin=val;
		}
	}
	if(op==QCALC_BLACKBOX)
	{
		blackBoxIDList.clear();
		QDomElement items=el.firstChildElement("Black_Box_Items");
		if(!items.isNull())
		{
			QDomElement item=items.firstChildElement("ID");
			while(!item.isNull())
			{
				int temp_id=item.firstChild().nodeValue().toInt();
				blackBoxIDList.push_back(temp_id);
				item=item.nextSiblingElement("ID");
			}
		}
	}

	setOperation(op, plugin);
	QDomElement inputsElement=el.firstChildElement("Input_Slots");
	if(!inputsElement.isNull())
	{
		QDomElement inputSlot=inputsElement.firstChildElement("Inslot");
		while(!inputSlot.isNull())
		{
			int index=inputSlot.attribute("Index", "-1").toInt();
			QString name=inputSlot.attribute("Name", "");
			QString description=inputSlot.attribute("Description", "");
			if(index>=0 && index< inputs.size())
			{
				inputs[index]->setName(name);
				inputs[index]->setDescription(description);
			}
			inputSlot=inputSlot.nextSiblingElement("Inslot");
		}
	}
	
	QDomElement outputsElement=el.firstChildElement("Output_Slots");
	if(!outputsElement.isNull())
	{
		QDomElement outputSlot=outputsElement.firstChildElement("Outslot");
		while(!outputSlot.isNull())
		{
			int index=outputSlot.attribute("Index", "-1").toInt();
			QString name=outputSlot.attribute("Name", "");
			QString description=outputSlot.attribute("Description", "");
			int isOpenSlot=outputSlot.attribute("open", "0").toInt();
			if(index>=0 && index< outputs.size())
			{
				outputs[index]->setName(name);
				outputs[index]->setDescription(description);
				outputs[index]->openSlot=(bool)isOpenSlot;
			}
			outputSlot=outputSlot.nextSiblingElement("Outslot");
		}
	}

    if(getXMLNodeText(el, "Name", val))
    {
        name=val;
    }
    printf("Name %s\n", name.toLatin1().data());
	
	if(!guiElement.isNull())
	{
		OperatorItem* item=this->getGUI();
        scene->addItem(item);
        item->fromXMLElement(guiElement);
		item->setTitle(name);
	}
}

bool VOperatorItem::unwrap(QGraphicsScene* scene, const QPointF& pos)
{
	if(operation == QCALC_BLACKBOX)
	{
		foreach(VCalcItem* item, blackBoxList)
		{
			VInputItem* vin=dynamic_cast<VInputItem*>(item);
			VOutputItem* vout=dynamic_cast<VOutputItem*>(item);
			VOperatorItem* vop=dynamic_cast<VOperatorItem*>(item);
			VJoinItem* vjoin=dynamic_cast<VJoinItem*>(item);
			VExtractionItem* vext=dynamic_cast<VExtractionItem*>(item);
			if(vin!=NULL)
			{
				InputItem* in=vin->getGUI();
				scene->addItem(in);
				in->setPos(pos+vin->positionInBlackBox);
				in->setValue(vin->valueText);				
			}
			else if(vout!=NULL)
			{
				OutputItem* out=vout->getGUI();
				scene->addItem(out);
				out->setPos(pos+vout->positionInBlackBox);
				out->setValue(vout->outputName);
			}
			else if(vop!=NULL)
			{
				OperatorItem* op=vop->getGUI();
				scene->addItem(op);
				op->setPos(pos+vop->positionInBlackBox);
				op->setTitle(vop->name);
			}
			else if(vjoin!=NULL)
			{
				JoinItem* join=vjoin->getGUI();
				scene->addItem(join);
				join->setPos(pos+vjoin->positionInBlackBox);
			}
			else if(vext!=NULL)
			{
				ExtractionItem* ext=vext->getGUI();
				scene->addItem(ext);
				ext->setPos(pos+vext->positionInBlackBox);
				ext->edit->setText(vext->valueText);
			}
			item->container = NULL;
		}
		
		blackBoxList.clear();
		return true;
	}
	
	return false;
}

OperatorItem::OperatorItem(VOperatorItem* parentItem, const QVector<VInSlot*>& inputs, const QVector<VOutSlot*>& outputs, QGraphicsItem* parent):
		CalcItem(parent)
{
#ifdef GRAPH_DEBUG
	std::cout << "Creating OperatorItem " << this << " for VOperatorItem " << parentItem << std::endl;
#endif
	this->parentItem=parentItem;
	
	outlineColor=QColor(70, 70, 70);
	color1=QColor(240, 240, 240);
	color2=QColor(220, 220, 220);
	
	title=parentItem->getName();
	
	width=BASE_WIDTH;
	height=BASE_HEIGHT;
	setRect(-width/2., -height/2., width, height);
	
	for(int i=0; i<inputs.size(); ++i)
	{
		InSlot* slot=new InSlot(this, inputs[i]);
		inputs[i]->setGUI(slot);
		inputs[i]->resetToolTips();
		slot->setPos(-width/2.+width/(2.*inputs.size())*(2.*i+1.), -height/2.);
		inSlots.push_back(slot);
	}
	for(int i=0; i<outputs.size(); ++i)
	{
		OutSlot* slot=new OutSlot(this, outputs[i]);
		outputs[i]->setGUI(slot);
		outputs[i]->resetToolTips();
		slot->setPos(-width/2.+width/(2.*outputs.size())*(2.*i+1.), height/2.);
		outSlots.push_back(slot);
	}
	
	expandButton=new ExpandButton(this);
	expandButton->setPos(-width/2.+0.5*HEADER_HEIGHT, -height/2.+HEADER_HEIGHT/2.);
	rzButton->setPos(-width/2.+1.5*HEADER_HEIGHT, -height/2.+HEADER_HEIGHT/2.);
	closeButton->setPos(width/2.-HEADER_HEIGHT/2., -height/2.+HEADER_HEIGHT/2.);
	errButton->setPos(width/2.-1.5*HEADER_HEIGHT, -height/2.+HEADER_HEIGHT/2.);
	
	displayedString="";
	expanded=false;
}

OperatorItem::~OperatorItem()
{
#ifdef GRAPH_DEBUG
	std::cout << "Deleting OperatorItem " << this << std::endl;
#endif
	foreach(InSlot* slot, inSlots)
	{
		delete slot;
	}
	foreach(OutSlot* slot, outSlots)
	{
		delete slot;
	}
}

QVariant OperatorItem::itemChange(GraphicsItemChange change, const QVariant& value)
{
	if (change == ItemPositionChange && scene()) {
		QPointF newPos = value.toPointF();
		
		updateConnectorsPositions();
		
		return newPos;
	}
	else if(change==ItemSelectedChange && scene())
	{
		if(value.toBool())
		{
			setZValue(1.);
		}
		else setZValue(0.);
		parentItem->setSelected(value.toBool());
	}
	return QGraphicsItem::itemChange(change, value);
}

void OperatorItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
{
	CalcItem::paint(painter, option, widget);
	
	if(expanded)
	{
		paintExpanded(painter);
	}
	else
	{
		paintCollapsed(painter);
	}
}

void OperatorItem::paintExpanded(QPainter* painter) const
{
	int numSlots=inSlots.size();
	if(numSlots<outSlots.size()) numSlots=outSlots.size();
	
	QPen p(outlineColor);
	p.setWidthF(0.5);
	QBrush b(Qt::NoBrush);
	painter->setPen(p);
	painter->setBrush(b);
	
	for(int i=0; i<inSlots.size(); ++i)
	{
		double h=EXPANDED_SLOT_HEIGHT*numSlots/inSlots.size();
		QRectF box(-width/2., -height/2.+i*h+HEADER_HEIGHT, width/2., h);
		painter->drawRect(box);
		box.adjust(2., 2., -2., -2.);
		QString text=inSlots[i]->slot->getName()+" - "+inSlots[i]->slot->getDescription();
		painter->drawText(box, Qt::AlignTop|Qt::AlignLeft|Qt::TextWordWrap, text);
	}
	
	for(int i=0; i<outSlots.size(); ++i)
	{
		double h=EXPANDED_SLOT_HEIGHT*numSlots/outSlots.size();
		QRectF box(0., -height/2.+i*h+HEADER_HEIGHT, width/2., h);
		painter->drawRect(box);
		box.adjust(2., 2., -2., -2.);
		QString text=outSlots[i]->slot->getName()+" - "+outSlots[i]->slot->getDescription();
		painter->drawText(box, Qt::AlignTop|Qt::AlignLeft|Qt::TextWordWrap, text);
	}
	
}

void OperatorItem::paintCollapsed(QPainter* painter) const
{
	QRectF box(QPointF(-width/2., -height/2.+HEADER_HEIGHT), QPointF(width/2., height/2.));
	painter->drawText(box, Qt::AlignCenter, displayedString);
}

void OperatorItem::expand()
{
	expanded=true;
	matSave=matrix();
	QMatrix m;
	m.reset();
	setMatrix(m);
	prepareGeometryChange();
	
	int numSlots=inSlots.size();
	if(numSlots<outSlots.size()) numSlots=outSlots.size();
	
	width=BASE_WIDTH*1.5;
	height=HEADER_HEIGHT+numSlots*EXPANDED_SLOT_HEIGHT;
	setRect(-width/2., -height/2., width, height);
	
	for(int i=0; i<inSlots.size(); ++i)
	{
		double h=EXPANDED_SLOT_HEIGHT*numSlots/inSlots.size();
		inSlots[i]->setPos(-width/2., -height/2.+(i+0.5)*h+HEADER_HEIGHT);
	}
	
	for(int i=0; i<outSlots.size(); ++i)
	{
		double h=EXPANDED_SLOT_HEIGHT*numSlots/outSlots.size();
		OutSlot* slot=outSlots[i];
		slot->setPos(width/2., -height/2.+(i+0.5)*h+HEADER_HEIGHT);
	}
	
	expandButton->setPos(-width/2.+0.5*HEADER_HEIGHT, -height/2.+HEADER_HEIGHT/2.);
	rzButton->setVisible(false);
	closeButton->setPos(width/2.-HEADER_HEIGHT/2., -height/2.+HEADER_HEIGHT/2.);
	errButton->setPos(width/2.-1.5*HEADER_HEIGHT, -height/2.+HEADER_HEIGHT/2.);
	
	updateConnectorsPositions();
	
	update();
}

void OperatorItem::collapse()
{
	expanded=false;
	setMatrix(matSave);
	prepareGeometryChange();
	
	width=BASE_WIDTH;
	height=BASE_HEIGHT;
	setRect(-width/2., -height/2., width, height);
	
	for(int i=0; i<inSlots.size(); ++i)
	{
		inSlots[i]->setPos(-width/2.+width/(2.*inSlots.size())*(2.*i+1.), -height/2.);
	}
	for(int i=0; i<outSlots.size(); ++i)
	{
		outSlots[i]->setPos(-width/2.+width/(2.*outSlots.size())*(2.*i+1.), height/2.);
	}

	expandButton->setPos(-width/2.+0.5*HEADER_HEIGHT, -height/2.+HEADER_HEIGHT/2.);
	rzButton->setVisible(true);
	rzButton->setPos(-width/2.+1.5*HEADER_HEIGHT, -height/2.+HEADER_HEIGHT/2.);
	closeButton->setPos(width/2.-HEADER_HEIGHT/2., -height/2.+HEADER_HEIGHT/2.);
	errButton->setPos(width/2.-1.5*HEADER_HEIGHT, -height/2.+HEADER_HEIGHT/2.);
	
	updateConnectorsPositions();
	
	update();
}

void OperatorItem::setOperation(QCalcOperation op, const QString& pluginName)
{
	parentItem->setOperation(op, pluginName);
}

void OperatorItem::clearSlots()
{
	qDeleteAll(inSlots);
	qDeleteAll(outSlots);
	inSlots.clear();
	outSlots.clear();
}

void OperatorItem::resetSlots(const QVector<VInSlot*>& inputs, const QVector<VOutSlot*>& outputs)
{
	for(int i=0; i<inputs.size(); ++i)
	{
		InSlot* slot=new InSlot(this, inputs[i]);
		inputs[i]->setGUI(slot);
		inputs[i]->resetToolTips();
		slot->setPos(-width/2.+width/(2.*inputs.size())*(2.*i+1.), -height/2.);
		inSlots.push_back(slot);
	}
	for(int i=0; i<outputs.size(); ++i)
	{
		OutSlot* slot=new OutSlot(this, outputs[i]);
		outputs[i]->setGUI(slot);
		outputs[i]->resetToolTips();
		slot->setPos(-width/2.+width/(2.*outputs.size())*(2.*i+1.), height/2.);
		outSlots.push_back(slot);
	}
}

void OperatorItem::resetConnectors()
{
	foreach(VInSlot* inslot, parentItem->inputs)
	{
		VOutSlot* outslot=inslot->connectedSlot();
		if(outslot!=NULL)
		{
			if(outslot->getGUI()!=NULL && inslot->getGUI()!=NULL)
			{
				Connector* conn=new Connector(outslot->getGUI(), inslot->getGUI());
				scene()->addItem(conn);
				conn->updatePosition();
			}
		}
	}
	
	foreach(VOutSlot* outslot, parentItem->outputs)
	{
		QList<VInSlot*> inslots=outslot->connectedSlots();
		foreach(VInSlot* inslot, inslots)
		{
			if(inslot->getGUI()!=NULL && outslot->getGUI()!=NULL)
			{
				Connector* conn=new Connector(outslot->getGUI(), inslot->getGUI());
				scene()->addItem(conn);
				conn->updatePosition();
			}
		}
	}

}

QDomElement OperatorItem::toXMLElement(QDomDocument& doc) const
{
	QDomElement el=CalcItem::toXMLElement(doc);
	el.setAttribute("Expanded", QString("%1").arg(expanded));
	return el;
}

void OperatorItem::fromXMLElement(QDomElement& el)
{
		bool isEx=(el.attribute("Expanded", "0").toInt());

	if(isEx) expand();
	else collapse();
	CalcItem::fromXMLElement(el);

}

bool OperatorItem::unwrap()
{
	return parentItem->unwrap(scene(), pos());
}
