#ifndef QCALC_VIEW_H
#define QCALC_VIEW_H

#include <QGraphicsView>
#include <QWheelEvent>
#include "qcalcScene.h"

class CalcView: public QGraphicsView
{
	Q_OBJECT
	
	public:
		CalcView(QGraphicsScene* scene);
		
		virtual ~CalcView();
		
		void displayMenu(const QPoint& position);
		QList<VOutputItem*> listSelectedOutputItems();

		
	protected:
		virtual void wheelEvent(QWheelEvent * event);
		void mousePressEvent(QMouseEvent* mouseEvent);

		void dragEnterEvent(QDragEnterEvent* e);
		void dropEvent(QDropEvent* e);
		QMenu* RButtonMenu;
		QPointF position;
		QPointF autoPosition;
		QPointF centrePosition;
		
	signals:
		void updateBlackBoxMenu();
		void componentSelected(const char *, const char *);
		
	public slots:
		void horizontalAspect();
		void verticalAspect();
		void normalSize();
		void halfSize();
		void doubleSize();
		void resetAspect();
		
		void addInputItem();
		void addOutputItem();
		void addExtractionItem();
		void addJoinItem();

		void addBB(QAction*);
        void addPlugin(QAction*);
        void changePlugin(QAction*);

        void addPluginMenu(QMenu* menu, const char* folder, bool isAdd);
		void addBlackBoxMenu(QMenu* menu);
		
		void addPlugInBrowse();
		void changePlugInBrowse();
		
		void saveBlackBoxToLibrary();
		
		inline void addTemplateAdd(){addBinaryTemplate(QCALC_ADD);}
		inline void addTemplateSub(){addBinaryTemplate(QCALC_SUB);}
		inline void addTemplateMul(){addBinaryTemplate(QCALC_MULT);}
		inline void addTemplateDiv(){addBinaryTemplate(QCALC_DIV);}
		inline void addTemplateOpp(){addUnaryTemplate(QCALC_OPP);}
		
		inline void addOpAdd(){addOp(QCALC_ADD);}
		inline void addOpSub(){addOp(QCALC_SUB);}
		inline void addOpMul(){addOp(QCALC_MULT);}
		inline void addOpDiv(){addOp(QCALC_DIV);}
		inline void addOpMod(){addOp(QCALC_MOD);}
		inline void addOpPow(){addOp(QCALC_POWER);}
		inline void addOpOpp(){addOp(QCALC_OPP);}
		inline void addOpPiecewise(){addOp(QCALC_PIECEWISE);}
		inline void addOpPwDiv(){addOp(QCALC_PW_DIV);}
		
		inline void addOpRemainder(){addOp(QCALC_REMAINDER);}
		inline void addOpAbs(){addOp(QCALC_ABS);}
		inline void addOpLog(){addOp(QCALC_LOG);}
		inline void addOpLog10(){addOp(QCALC_LOG10);}
		inline void addOpInverse(){addOp(QCALC_INVERSE);}
		inline void addOpSqrt(){addOp(QCALC_SQRT);}
		inline void addOpExp(){addOp(QCALC_EXP);}
		inline void addOpVectProd(){addOp(QCALC_VECTPROD);}
		inline void addOpDotProd(){addOp(QCALC_DOTPROD);}
		inline void addOpUnits(){addOp(QCALC_UNITS);}
		inline void addOpFills(){addOp(QCALC_FILLS);}
		inline void addOpTrace(){addOp(QCALC_TRACE);}
		inline void addOpDiff3(){addOp(QCALC_DIFF3);}
		inline void addOpDiff5(){addOp(QCALC_DIFF5);}
		inline void addOpIntDef(){addOp(QCALC_INT_DEF);}
		inline void addOpIntIndef(){addOp(QCALC_INT_INDEF);}
		
		inline void addOpCos(){addOp(QCALC_COS);}
		inline void addOpSin(){addOp(QCALC_SIN);}
		inline void addOpTan(){addOp(QCALC_TAN);}
		inline void addOpCot(){addOp(QCALC_COT);}
		inline void addOpAcos(){addOp(QCALC_ACOS);}
		inline void addOpAsin(){addOp(QCALC_ASIN);}
		inline void addOpAtan(){addOp(QCALC_ATAN);}
		inline void addOpAtan2(){addOp(QCALC_ATAN2);}
		inline void addOpAcot(){addOp(QCALC_ACOT);}
		inline void addOpCosh(){addOp(QCALC_COSH);}
		inline void addOpSinh(){addOp(QCALC_SINH);}
		inline void addOpTanh(){addOp(QCALC_TANH);}
		inline void addOpCoth(){addOp(QCALC_COTH);}
		inline void addOpAcosh(){addOp(QCALC_ACOSH);}
		inline void addOpAsinh(){addOp(QCALC_ASINH);}
		inline void addOpAtanh(){addOp(QCALC_ATANH);}
		inline void addOpAcoth(){addOp(QCALC_ACOTH);}
		inline void addOpAngle360(){addOp(QCALC_ANGLE360);}
		inline void addOpAngle180(){addOp(QCALC_ANGLE180);}
		inline void addOpTTtoS(){addOp(QCALC_TT_TO_SEC);}
		inline void addOpTT2_TT1(){addOp(QCALC_TT2_TT1);}
		inline void addOpT2_T1(){addOp(QCALC_T2_T1);}
		inline void addOpTIVLtoS(){addOp(QCALC_TIVL_TO_SEC);}
		inline void addOpLinReg(){addOp(QCALC_LIN_REG);}
		
		inline void changeOpAdd(){changeOp(QCALC_ADD);}
		inline void changeOpSub(){changeOp(QCALC_SUB);}
		inline void changeOpMul(){changeOp(QCALC_MULT);}
		inline void changeOpDiv(){changeOp(QCALC_DIV);}
		inline void changeOpMod(){changeOp(QCALC_MOD);}
		inline void changeOpPow(){changeOp(QCALC_POWER);}
		inline void changeOpOpp(){changeOp(QCALC_OPP);}
		inline void changeOpPiecewise(){changeOp(QCALC_PIECEWISE);}
		inline void changeOpPwDiv(){changeOp(QCALC_PW_DIV);}
		
		inline void changeOpRemainder(){changeOp(QCALC_REMAINDER);}
		inline void changeOpAbs(){changeOp(QCALC_ABS);}
		inline void changeOpLog(){changeOp(QCALC_LOG);}
		inline void changeOpLog10(){changeOp(QCALC_LOG10);}
		inline void changeOpInverse(){changeOp(QCALC_INVERSE);}
		inline void changeOpSqrt(){changeOp(QCALC_SQRT);}
		inline void changeOpExp(){changeOp(QCALC_EXP);}
		
		inline void changeOpVectProd(){changeOp(QCALC_VECTPROD);}
		inline void changeOpDotProd(){changeOp(QCALC_DOTPROD);}
		inline void changeOpUnits(){changeOp(QCALC_UNITS);}
		inline void changeOpFills(){changeOp(QCALC_FILLS);}
		inline void changeOpTrace(){changeOp(QCALC_TRACE);}
		inline void changeOpDiff3(){changeOp(QCALC_DIFF3);}
		inline void changeOpDiff5(){changeOp(QCALC_DIFF5);}
		
		inline void changeOpCos(){changeOp(QCALC_COS);}
		inline void changeOpSin(){changeOp(QCALC_SIN);}
		inline void changeOpTan(){changeOp(QCALC_TAN);}
		inline void changeOpCot(){changeOp(QCALC_COT);}
		inline void changeOpAcos(){changeOp(QCALC_ACOS);}
		inline void changeOpAsin(){changeOp(QCALC_ASIN);}
		inline void changeOpAtan(){changeOp(QCALC_ATAN);}
		inline void changeOpAtan2(){changeOp(QCALC_ATAN2);}
		inline void changeOpAcot(){changeOp(QCALC_ACOT);}
		inline void changeOpCosh(){changeOp(QCALC_COSH);}
		inline void changeOpSinh(){changeOp(QCALC_SINH);}
		inline void changeOpTanh(){changeOp(QCALC_TANH);}
		inline void changeOpCoth(){changeOp(QCALC_COTH);}
		inline void changeOpAcosh(){changeOp(QCALC_ACOSH);}
		inline void changeOpAsinh(){changeOp(QCALC_ASINH);}
		inline void changeOpAtanh(){changeOp(QCALC_ATANH);}
		inline void changeOpAcoth(){changeOp(QCALC_ACOTH);}
		inline void changeOpAngle360(){changeOp(QCALC_ANGLE360);}
		inline void changeOpAngle180(){changeOp(QCALC_ANGLE180);}
		inline void changeOpTTtoS(){changeOp(QCALC_TT_TO_SEC);}
		inline void changeOpTT2_TT1(){changeOp(QCALC_TT2_TT1);}
		inline void changeOpT2_T1(){changeOp(QCALC_T2_T1);}
		inline void changeOpTIVLtoS(){changeOp(QCALC_TIVL_TO_SEC);}
		inline void changeOpLinReg(){changeOp(QCALC_LIN_REG);}
		
		void autoSelectPosition();
		void changeForBlackBox();
		void unwrapBlackBox();
		void positionCrosshair();
		
	protected:
		void changeOp(const QCalcOperation& op);
		void addOp(const QCalcOperation& op);
		void addUnaryTemplate(const QCalcOperation& op);
		void addBinaryTemplate(const QCalcOperation& op);
		bool positionSet;
		bool positionOK;
		
};


#endif
