//  qjui.cc
//
// Join Widget
//
// Mar 2002: Tony Allen
//

#include "qjui.h"
#include "qexuiW.h"
#include "qdutil.h"
#include "qwui.h"
#include "qdutil.h"

#include <qpalette.h>
#include <qgroupbox.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qtooltip.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <stdio.h>


#include <stdlib.h>

using namespace QSAS;

extern qwuiWindow* qsasWindow;
extern int qsas_restore_release;

// JOIN Window
//---------------

QjuiWindow::QjuiWindow(QWidget* parent, const char * name)
	:QguWindow(parent)
{
	this->setPalette(qApp->palette());
	setAcceptDrops(true);
	setFocusPolicy(Qt::WheelFocus);
	
	verbose=false;

	//// Body of the window
	
	_top_widget=new QWidget(this);
	
	QBoxLayout * main_layout=new QBoxLayout(QBoxLayout::TopToBottom, _top_widget);
	main_layout->setSpacing(4);
	main_layout->setMargin(4);
	
	//// SPLITTER
	
	_top_splitter=new QSplitter(_top_widget);
	
	//// LIST VIEW

	_join_widget=new QjuiWidget(_top_splitter, this);
	_top_splitter->addWidget(_join_widget);

	
	//// Options panel
	
	QWidget * opts_widget=new QWidget(_top_splitter);

	QBoxLayout * opts_layout=new QBoxLayout(QBoxLayout::TopToBottom, opts_widget);

	//// Join option
	
	QGroupBox * join_widget=new QGroupBox("Join Settings for Selected Items", opts_widget);
	
	QGridLayout * join_layout=new QGridLayout(join_widget);
	join_layout->setSpacing(4);
	
	QLabel* algorithm_label = new QLabel("Join Algorithm ",join_widget);
	join_layout->addWidget( algorithm_label, 0, 0, Qt::AlignRight );
		
	_algorithm = new QComboBox(join_widget);
	_algorithm->addItem(LINEAR );
	_algorithm->addItem(BOXCAR );
	join_layout->addWidget( _algorithm, 0, 1, Qt::AlignLeft );
// 	connect(_algorithm, SIGNAL(activated(int)), _join_listview, SLOT(setAlgorithm()));
	connect(_algorithm, SIGNAL(activated(int)), _join_widget, SLOT(setAlgorithm()));
			
	QLabel* width_label = new QLabel("Ave width ",join_widget);
	join_layout->addWidget( width_label, 1, 0, Qt::AlignRight );
		
	_width_field = new QguWLinput( join_widget, "width_field" );  
	_width_field->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	connect(_width_field, SIGNAL(edited_OK()), _join_widget, SLOT(setWidthField()));
	_width_field->setToolTip("width of the boxcar (for time series this is in seconds)");
	join_layout->addWidget( _width_field, 1, 1 );
		
	QLabel* blank_label = new QLabel(" ",join_widget);
	join_layout->addWidget( blank_label, 2, 0, Qt::AlignRight );
		
	QLabel* blank2_label = new QLabel(" ",join_widget);
	join_layout->addWidget( blank2_label, 2, 1, Qt::AlignRight );
		
	QLabel* gap_label = new QLabel("Gap Handling ",join_widget);
	join_layout->addWidget( gap_label, 3, 0, Qt::AlignRight );
		
	_gap = new QComboBox(join_widget);
	_gap->addItem(LINEAR_INTERP_GAP );
	_gap->addItem(ZERO_FILL_GAP );
	_gap->addItem(FILL_GAP );
	_gap->addItem(NEAREST_NEIGHBOUR_GAP );
	_gap->addItem(REMOVE_GAP );
	join_layout->addWidget( _gap, 3, 1, Qt::AlignLeft );
	connect(_gap, SIGNAL(activated(int)), _join_widget, SLOT(setGap()));
		
	QLabel* gapSize_label = new QLabel("Gap threshold ",join_widget);
	join_layout->addWidget( gapSize_label, 4, 0, Qt::AlignRight );
		
	_gap_field = new QguWLinput( join_widget, "gap_field" );  
	_gap_field->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	connect(_gap_field, SIGNAL(edited_OK()), _join_widget, SLOT(setGapField()));
	_gap_field->setToolTip("minimum interval to be treated as a gap if empty (for time series this is in seconds)");
	join_layout->addWidget( _gap_field, 4, 1 );

	opts_layout->addWidget(join_widget);
	
	//// Time option
	
	QGroupBox * time_widget=new QGroupBox("Join Target", opts_widget);
	
	QGridLayout * time_layout=new QGridLayout(time_widget);
	time_layout->setSpacing(4);
	
	 QLabel* TimeInterval_label = new QLabel("Interval ",time_widget);
	time_layout->addWidget( TimeInterval_label, 0, 0, Qt::AlignRight );
		
	_TimeIvlSelect = new QComboBox(time_widget);
	_TimeIvlSelect->insertItem(0, "from Data" );
	_TimeIvlSelect->insertItem(1, "from Object" );
	time_layout->addWidget( _TimeIvlSelect, 0, 1, Qt::AlignLeft );
	connect(_TimeIvlSelect, SIGNAL(activated(int)), this, SLOT(setTTivl(int)));
		
	IvlSource_label = new QLabel("  ",time_widget);
	time_layout->addWidget( IvlSource_label, 1, 0, Qt::AlignRight );
		
	_IvlSource = new QguWLinput( time_widget, "IvlSource" );  
	_IvlSource->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_NOT_SEQ, QS_ANY));
    _IvlSource->active = true;
		
	_IvlSource->setToolTip("not active");
	time_layout->addWidget( _IvlSource, 1, 1, Qt::AlignLeft );
	
	connect(_IvlSource, SIGNAL(edited_OK()), this, SLOT(setIntervalOption()));

	time_layout->addWidget(new QLabel("", time_widget), 2, 0); //space
	
	QLabel* TimeTags_label = new QLabel("Tags ",time_widget);
	time_layout->addWidget( TimeTags_label, 3, 0, Qt::AlignRight );
		
	_TimeTagSelect = new QComboBox(time_widget);
	_TimeTagSelect->insertItem(0, "Create Regular");
	_TimeTagSelect->insertItem(1, "from Object");
	time_layout->addWidget( _TimeTagSelect, 3, 1, Qt::AlignLeft );
	connect(_TimeTagSelect, SIGNAL(activated(int)), this, SLOT(setTTsource(int)));
		
	TagSource_label = new QLabel("Target Tag Object ",time_widget);
	time_layout->addWidget( TagSource_label, 4, 0, Qt::AlignRight );
		
	_TagSource = new QguWLinput( time_widget, "TagSource" );  
	_TagSource->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_SEQ, QS_ANY));
	connect(_TagSource, SIGNAL(edited_OK()), this, SLOT(setTagsOption()));
	
	_TagSource->setToolTip("Uses DEPEND_0 of this object as target time or scalar sequence to join onto");
	time_layout->addWidget( _TagSource, 4, 1, Qt::AlignLeft );

	opts_layout->addWidget(time_widget);

	//// Link option
	
	opts_widget->adjustSize();
	
	_top_splitter->addWidget(opts_widget);
	_top_splitter->adjustSize();
	
	main_layout->addWidget(_top_splitter);
	
	//// FEEDBACK
	
	QGroupBox * feedback_widget=new QGroupBox("Sequence Dependency (Time or Scalar Sequence) ", _top_widget);
	QGridLayout * feedback_layout=new QGridLayout(feedback_widget);
	feedback_layout->setSpacing(4);
	feedback_layout->setColumnStretch(1, 10);
	
	QLabel* Start_label = new QLabel("Start: ",feedback_widget);
	feedback_layout->addWidget( Start_label, 0, 0, Qt::AlignRight );

	Start_field = new QLabel("<Not yet set>",feedback_widget);
	feedback_layout->addWidget( Start_field, 0, 1, Qt::AlignLeft );

	QLabel* End_label = new QLabel("End: ",feedback_widget);
	feedback_layout->addWidget( End_label, 1, 0, Qt::AlignRight );

	End_field = new QLabel("<Not yet set>",feedback_widget);
	feedback_layout->addWidget( End_field, 1, 1, Qt::AlignLeft );

	QLabel* Ntags_label = new QLabel("     Number of Tags in Interval: ",feedback_widget);
	feedback_layout->addWidget( Ntags_label, 0, 2, Qt::AlignRight );

	Ntags_field = new QLabel("<Not yet set>",feedback_widget);
	feedback_layout->addWidget( Ntags_field, 0, 3, Qt::AlignLeft );

	QLabel* spacing_label = new QLabel("Tag Spacing: ",feedback_widget);
	feedback_layout->addWidget( spacing_label, 1, 2, Qt::AlignRight );

	spacing_field = new QLabel("<Not yet set>",feedback_widget);
	feedback_layout->addWidget( spacing_field, 1, 3, Qt::AlignLeft );

	main_layout->addWidget(feedback_widget);
	//// BUTTONS
	
	_buttons_widget=new QWidget(_top_widget);
	buttons_layout=new QBoxLayout(QBoxLayout::LeftToRight, _buttons_widget);

	buttons_layout->setSpacing(4);
	
	//  buttons added in JOIN or EXPORT create windows  
	
	main_layout->addWidget(_buttons_widget);
	
	setCentralWidget(_top_widget);
	
		//// Menus
	
	_menu_bar->clear();
	
	QMenu* file_menu = _menu_bar->addMenu("File");
	QMenu* edit_menu = _menu_bar->addMenu("Edit");
	options_menu = _menu_bar->addMenu("Options");
	_menu_bar->addSeparator();
	QMenu * help = _menu_bar->addMenu(" Help");

		
	file_menu->clear();
	file_menu->addAction("&Save", this, SLOT(save()), Qt::CTRL+Qt::Key_S);
	file_menu->addAction("Save As...", this, SLOT(saveAs()), Qt::CTRL+Qt::SHIFT+Qt::Key_S);
    file_menu->addAction("Save As XML...", this, SLOT(saveAsXML()));
	file_menu->addSeparator();
	file_menu->addAction("Include...", this, SLOT(include()));
	file_menu->addAction("&Restore", this, SLOT(restore()), Qt::CTRL+Qt::Key_R);
    file_menu->addAction("Restore XML...", this, SLOT(restoreXML()));
	file_menu->addSeparator();
	file_menu->addAction("Use Default Settings", this, SLOT(defaultAll()));
		
	edit_menu->clear();
	edit_menu->addAction("Re-Name Item", _join_widget, SLOT( renameSelectedItem()));
	edit_menu->addSeparator();
	edit_menu->addAction("&Copy", _join_widget, SLOT( copy() ), Qt::CTRL+Qt::Key_C );
	edit_menu->addAction("Paste", _join_widget, SLOT(paste()), Qt::CTRL+Qt::Key_V);
	edit_menu->addAction("select &All", _join_widget, SLOT( selectAll() ), Qt::CTRL+Qt::Key_A );
	edit_menu->addSeparator();
	edit_menu->addAction("Copy Settings <-",this,SLOT(keepChoices()), Qt::CTRL+Qt::Key_Left);
	edit_menu->addAction("Paste Settings ->",this,SLOT(applyChoices()), Qt::CTRL+Qt::Key_Right);
	edit_menu->addSeparator();
	edit_menu->addAction("Delete Item", _join_widget, SLOT(deleteListItems()), Qt::CTRL+Qt::Key_X);
	edit_menu->addAction("Clear List", this, SLOT(deleteAllItems()));

	options_menu->clear();
	verboseAction = options_menu->addAction("Verbose", this, SLOT(verboseToggle()));
	verboseAction->setCheckable(true);


	help->addAction("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	help->addSeparator();
	help->addAction("...Data Export", this, SLOT( launchManExport() ));
	help->addAction("...Join", this, SLOT( launchManJoin() ));
	help->addSeparator();
	help->addAction("Module Version", this, SLOT( launchVersion() ));
	
	
	adjustSize();
	
	status(" ");
}

void QjuiWindow::launchManual(){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::launchManual %ld\n", (long)this);
	fflush(stdout);
 }
 
  
   qsasWindow->help_window->manEntry("index.html");
}

void QjuiWindow::launchManExport(){

 if(QuSlotDebug())
 {
	printf("QjuiWindow::launchManExport %ld\n", (long)this);
	fflush(stdout);
 }
 
  
   qsasWindow->help_window->manEntry("QSAS_data_handling.html");
}

void QjuiWindow::launchManJoin(){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::launchManJoin %ld\n", (long)this);
	fflush(stdout);
 }
 
  
   qsasWindow->help_window->manEntry("QSAS_analysis.html");
}

void QjuiWindow::launchVersion(){

 if(QuSlotDebug())
 {
	printf("DirectImportWindow::launchVersion %ld\n", (long)this);
	fflush(stdout);
 }
 
  
  QString text;
  text  = QString("Version Numbers...\n"); 
  text += QString(QSAS_VERSION_ID);
  text += "\n Join/Export  ";
  text += QString(QJUI_VERSION);
  QMessageBox::information( this, "QSAS information", text.toLatin1().data(), "OK" );

}


void QJUImakeWindow( QWidget *parent, const char * Title )
{
  if(QJUI_widget==NULL)
  {
     QJUI_widget = QJUICreatePopup(parent, Title);
     QJUI_widget->setWindowTitle(Title);

     QJUI_widget->JoinBtn = new QPushButton( "Join", QJUI_widget->_buttons_widget);
//      QJUI_widget->JoinBtn->setMinimumSize(QJUI_widget->JoinBtn->sizeHint());
     QJUI_widget->buttons_layout->addWidget(QJUI_widget->JoinBtn);
     QJUI_widget->connect( QJUI_widget->JoinBtn, SIGNAL(clicked()), QJUI_widget, SLOT(DoJoin()) );
     //QJUI_widget->buttons_layout->addSpace(0);
     QJUI_widget->dismissBtn = new QPushButton( "Dismiss", QJUI_widget->_buttons_widget);
//      QJUI_widget->dismissBtn->setMinimumSize(QJUI_widget->dismissBtn->sizeHint());
     QJUI_widget->buttons_layout->addWidget(QJUI_widget->dismissBtn);
     QJUI_widget->connect( QJUI_widget->dismissBtn, SIGNAL(clicked()), QJUI_widget, SLOT(close()) );
  
     QJUI_widget->_filename += QString("/saveSet");
     QJUI_widget->SAVE_EXTENSION = QString(".qx_join");
     QJUI_widget->_filename += QJUI_widget->SAVE_EXTENSION;
     QJUI_widget->_TimeTagSelect->setCurrentIndex(1);
     QJUI_widget->_TimeTagSelect_Def = 1;
     QJUI_widget->_TimeTagSelect_cpy = 1;
	QJUI_widget->_saveset_filter=QString("Join savesets (*.qx_join)");
	QJUI_widget->adjustSize();

  }
 return;
 
}


void QEXPmakeWindow( QWidget *parent, const char * Title )
{
  if(QJEX_widget==NULL)
  {
     QJEX_widget = QJUICreatePopup(parent, Title);
     QJEX_widget->setWindowTitle(Title);
     QJEX_widget->WriteBtn = new QPushButton( "Export...", QJEX_widget->_buttons_widget);
     QJEX_widget->WriteBtn->setMinimumSize(QJEX_widget->WriteBtn->sizeHint());
     QJEX_widget->connect( QJEX_widget->WriteBtn, SIGNAL(clicked()),  QJEX_widget, SLOT(writeFile()) );
     QJEX_widget->buttons_layout->addWidget(QJEX_widget->WriteBtn);
     //QJEX_widget->buttons_layout->addSpace(0);
     QJEX_widget->dismissBtn = new QPushButton( "Dismiss", QJEX_widget->_buttons_widget);
     QJEX_widget->dismissBtn->setMinimumSize(QJEX_widget->dismissBtn->sizeHint());
     QJEX_widget->connect( QJEX_widget->dismissBtn, SIGNAL(clicked()), QJEX_widget, SLOT(close()) );
     QJEX_widget->buttons_layout->addWidget(QJEX_widget->dismissBtn);

     QJEX_widget->_filename += QString("/saveSet");
     QJEX_widget->SAVE_EXTENSION = QString(".qx_exp");
     QJEX_widget->_filename += QJEX_widget->SAVE_EXTENSION;
     QJEX_widget->_TimeTagSelect->setCurrentIndex(1);
     QJEX_widget->_TimeTagSelect_Def = 1;     
     QJEX_widget->_TimeTagSelect_cpy = 1;
//     QJEX_widget->_TagSource->setText("");
	QJEX_widget->_saveset_filter=QString("Export savesets (*.qx_exp)");
	QJEX_widget->adjustSize();
  }

  return;
}


void QjuiShowExport(){
  QSize qsasGUISize = qsasWindow->frameSize();
  QJEX_widget->move(qsasGUISize.width(),45);
  QJEX_widget->show();
}

void QjuiShowJoin(){  
  QSize qsasGUISize = qsasWindow->frameSize();
  QJUI_widget->move(qsasGUISize.width(),60);
  QJUI_widget->show();
}

QjuiWindow * QJUICreatePopup(QWidget *parent, const char * Title)
{
 QjuiWindow * Window = new QjuiWindow(parent,Title);
 Window->move(50,50);
 Window->setDefaults();
 Window->defaultAll();
 
 Window->connect(parent, SIGNAL(saveSessionRequest(QString)),  Window, SLOT(saveSession(QString)) );
 Window->connect(parent, SIGNAL(restoreSessionRequest(QString)),   Window, SLOT(restoreSession(QString)) );
 Window->connect(parent, SIGNAL(clearListRequest()),   Window, SLOT(deleteAllItems()) );
 return Window;
}

 QjuiWindow::~QjuiWindow()
{ 
     delete _IvlSource;
     delete _TagSource;
}

 void QjuiWindow::dragEnterEvent(QDragEnterEvent *event)
{ 
 if(QuSlotDebug())
 {
	printf("QjuiWindow::dragEnterEvent %ld\n", (long)this);
	fflush(stdout);
 }
 
      this->activateWindow();
      this->raise();
}


void QjuiWindow::save(Vostream& v_os)
{
	string guiID;
	guiID=QSAS_VERSION_ID;

	v_os << guiID;
	
   v_os << _algorithm->currentIndex();

   _width_field->with = 0;
   v_os << *_width_field;

   v_os << _gap->currentIndex();

   _gap_field->with = with;
   v_os << *_gap_field;

   v_os << _TimeIvlSelect->currentIndex();


   _IvlSource->with = with;

   v_os << *_IvlSource;

   v_os << _TimeTagSelect->currentIndex();

   _TagSource->with = with;
   v_os << *_TagSource;

   v_os << *_join_widget;
}


void QjuiWindow::saveXML()
{
 	ofstream file;
	file.open(_filename.toLatin1().data(),ios::out);

	if(file.is_open())
	{
		QDomDocument doc("QSAS_join_save");

		QDomElement root=doc.createElement("QSAS_Join");
		doc.appendChild(root);
		string guiID;
		guiID=QSAS_VERSION_ID;

		QDomElement version=createTextXMLNode(doc, "QSAS_Version", guiID);
		root.appendChild(version);

		QDomElement join_options=doc.createElement("Join_Options");
		join_options.appendChild(createTextXMLNode(doc, "Algorithm", _algorithm->currentIndex()));
		join_options.appendChild(createTextXMLNode(doc, "Width", _width_field->text()));
		join_options.appendChild(createTextXMLNode(doc, "Gap_Handling", _gap->currentIndex()));
		join_options.appendChild(createTextXMLNode(doc, "Gap_Threshold", _gap_field->text()));
		root.appendChild(join_options);

		QDomElement time_source=doc.createElement("Time_Source");
		time_source.appendChild(createTextXMLNode(doc, "Interval", _TimeIvlSelect->currentIndex()));
		time_source.appendChild(createTextXMLNode(doc, "Object", _IvlSource->text()));
		time_source.appendChild(createTextXMLNode(doc, "Time_Tags", _TimeTagSelect->currentIndex()));
		time_source.appendChild(createTextXMLNode(doc, "Tags_Source", _TagSource->text()));
		root.appendChild(time_source);

		root.appendChild(_join_widget->toXMLElement(doc));
            
		QString xml = doc.toString();
		std::cout << xml.toLatin1().data() << std::endl;
		file << xml.toLatin1().data();
		file.close();
	}
}

void QjuiWindow::saveAsXML()
{
    QString path=QFileDialog::getSaveFileName(this, "Save as...", QString(), _saveset_filter);

    if(!path.isEmpty())
    {
		if(!path.contains(SAVE_EXTENSION) && !path.contains(".xml")){
			path += SAVE_EXTENSION;
		}
		
		_filename=path;

		saveXML();
	}
}


bool QjuiWindow::restoreXML(QString path, bool replace)
{
	if( !QFile::exists(path) ) return false;

	QDomDocument doc;
	QFile myFile(path);
	doc.setContent(&myFile);
	myFile.close();
        
	QDomElement docElem = doc.documentElement();
	if(!docElem.isNull())
	{
		if(replace) deleteAllItems();

		string guiID="0.0.00";
		QDomElement el= docElem.firstChildElement();

		QString s;
		if(getXMLNodeText(el, "QSAS_Version", s))
		{
			guiID=s.toLatin1().data();
		}

		QDomElement join_options=docElem.firstChildElement("Join_Options");
		if(!join_options.isNull())
		{
			if(getXMLNodeText(join_options, "Algorithm", s))
			{
				_algorithm->setCurrentIndex(s.toInt());
			}
			if(getXMLNodeText(join_options, "Width", s))
			{
				_width_field->setText(s);
			}
			if(getXMLNodeText(join_options, "Gap_Handling", s))
			{
				_gap->setCurrentIndex(s.toInt());
			}
			if(getXMLNodeText(join_options, "Gap_Threshold", s))
			{
				_gap_field->setText(s);
			}
				
		}

		QDomElement time_source=docElem.firstChildElement("Time_Source");
		if(!time_source.isNull())
		{
			if(getXMLNodeText(time_source, "Interval", s))
			{
				_TimeIvlSelect->setCurrentIndex(s.toInt());
			}
			if(getXMLNodeText(time_source, "Object", s))
			{
				_IvlSource->setText(s);
			}
			if(getXMLNodeText(time_source, "Time_Tags", s))
			{
				_TimeTagSelect->setCurrentIndex(s.toInt());
			}
			if(getXMLNodeText(time_source, "Tags_Source", s))
			{
				_TagSource->setText(s);
			}
		}

		QDomElement join_list=docElem.firstChildElement("List");
		if(!join_list.isNull())
		{
			_join_widget->fromXMLElement(join_list);
		}
	}
	return true;
}

void QjuiWindow::restoreXML(bool replace)
{

    QString path=QFileDialog::getOpenFileName(this, "Open...", QString(), "QSAS join save (*.qx_join);; XML files (*.xml);; All files (*)");
    if(!path.isEmpty())
    {
		restoreXML(path, replace);
	}
}


void QjuiWindow::restore(Vistream& v_is, bool replace)
{
	string guiID;
	v_is >> guiID;
	char *ptr;

	if( QuSlotDebug() ) printf("QjuiWindow::restore set ID =  %s\n---------------------------------\n", guiID.c_str());

	if( strstr(guiID.c_str(), "qjui") == NULL)
	{
		// changed to QSAS_VERSION_ID at 2.3.04
		setRestoreVersion(guiID.c_str());
	}
	else
	{
		setRestoreVersion("QSAS 2.3.03"); // old saveset so ensure old Time class is used
	}

  if(replace) deleteAllItems();


   v_is >> _algorithm_cpy;
   _algorithm->setCurrentIndex(_algorithm_cpy);

   v_is >> *_width_field;

   v_is >> _gap_cpy;
   _gap->setCurrentIndex(_gap_cpy);

   v_is >> *_gap_field;

   v_is >> _TimeIvlSelect_cpy;
   _TimeIvlSelect->setCurrentIndex(_TimeIvlSelect_cpy);
   setTTivl(_TimeIvlSelect_cpy);

   v_is >> *_IvlSource;

   v_is >> _TimeTagSelect_cpy;
   _TimeTagSelect->setCurrentIndex(_TimeTagSelect_cpy);
 
   v_is >> *_TagSource;

   v_is >> *_join_widget;

  setTTsource(_TimeTagSelect_cpy);
//   _TagSource->reValidate();


}


 void QjuiWindow::saveAs()
 {
 
 if(QuSlotDebug())
 {
	printf("QjuiWindow::saveAs %ld\n", (long)this);
	fflush(stdout);
 }
 

   QString _filter = QString("save (*");
   QString extn = SAVE_EXTENSION;
   _filter += extn;
   QString fn=QFileDialog::getSaveFileName(this, "Select file - Save Join Settings", _filename, _filter);
   if(!fn.isEmpty())
   {
     int pos = fn.lastIndexOf(extn);
     if (pos < 0) {
       pos = fn.lastIndexOf(".");
       if (pos > 0) fn.truncate(pos);
       fn += extn;
     }
     _filename=fn;
     save();
   }
   else
     status("saving aborted ",10000);
 }
            
 void QjuiWindow::save()
 { 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::save %ld\n", (long)this);
	fflush(stdout);
 }

    if(_filename.isEmpty())
    {
       saveAs();
       return;
    }

    try
    {
      VXDRostream xdr_os(_filename.toLatin1().data());
      save(xdr_os);
    }
    catch(Exception &e)
    {
       status(e.msg().c_str(),10000);
       return;
    }
    status(_filename+" saved",10000);
    return;
 }


void QjuiWindow::saveSession(QString folder){

 if(QuSlotDebug())
 {
	printf("QjuiWindow::saveSession %ld\n", (long)this);
	fflush(stdout);
 }
 

  if(folder.isEmpty()) return;
  // keep original name
  QString fileN = QString(_filename.toLatin1().data());
  
  // set session folder and save
	this->_filename = QString(folder.toLatin1().data());
	this->_filename += QString("/saveSet");
   	QString extn = SAVE_EXTENSION;
	this->_filename += extn;
    saveXML();

  // put file name back
  _filename = QString(fileN.toLatin1().data());
}


 void QjuiWindow::deleteAllItems(){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::deleteAllItems %ld\n", (long)this);
	fflush(stdout);
 }
 
 
   _join_widget->selectAll();
   _join_widget->deleteListItems();
}

 void QjuiWindow::verboseToggle(){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::verboseToggle %ld\n", (long)this);
	fflush(stdout);
 }
	 
	verbose = !verbose;
	verboseAction->setChecked(verbose);

}
 
 void QjuiWindow::DoJoin(){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::DoJoin %ld\n", (long)this);
	fflush(stdout);
 }
 
 
  // keep settings
  keepChoices();
  
  QdObjectSeq_var JoinedList = new QdObjectSeq();
  vector<KVDataBase_var> OptionList; 

  if(alreadyJoined()){
       status("No target time tags or already joined",50000);
       return;
  }
  
  vector <string> nameList;
  
  if( joinObjects(JoinedList, nameList) ){

     status("join finished...",10000);
    
     // put resulting objects on the Working List
     for (int jcount=0; jcount<JoinedList->size(); jcount++)  {

       // skip empty and failed slots quietly
    
       QdObject_var out_obj = JoinedList[jcount];
     
       if( out_obj.is_nil() ) continue;
     
      // Note Multijoin sets Xrefs and time tags on output objects
	
     
        // put joined obj on W List
       try{
         QString joined_name = QString(nameList[jcount].c_str());
         joined_name += "_J";
		 status("putting "+joined_name+" on working list",10000);
         QwuiPutOnWL(out_obj, joined_name);

       }
       catch(Exception &e){
         status( QString(e.msg().c_str()), 10000 );
         return;
       }
    
      }  // end of: for jcount putting back on Working List
      status("JOIN Finished", 10000);
   }
  
  
} 

 bool QjuiWindow::alreadyJoined()
{
	int numRows=_join_widget->count();
	if(numRows==0) return true;

	int startIndex=0;
	QdObject_var target_D0;
	target_D0 = getTimeTags();
	if(target_D0.is_nil() )
	{
		// no target set, test against first item to allow export to work without joining
		QdObject_var firstObj = QuGetObjByName(_join_widget->item(startIndex)->text());
		target_D0 = get_Depend_0(firstObj);
		if(target_D0.is_nil()) return false;  // no Dep0, give up, not joined
		++startIndex;
	}
	


	for(int i=startIndex; i<numRows; ++i)
	{
		QjuiWidgetItem* it=(QjuiWidgetItem*)(_join_widget->item(i));
		QdObject_var obj = QuGetObjByName(it->text());
		if(obj.is_nil())
		{
			return false;
		}
		if(it->I_algorithm == 1 || !QuIsJoined(obj, target_D0)) return false;  // not joined or boxcar method
	}
	return true;

}


 bool QjuiWindow::joinObjects(QdObjectSeq_var JoinedList, vector <string> &nameList)
{

 if(QuSlotDebug())
 {
	printf("QjuiWindow::joinObjects %ld\n", (long)this);
	fflush(stdout);
 }
 
  JoinedList->clear();
  QdObject_var target_D0;
  QdObjectSeq_var ToJoinList = new QdObjectSeq();
  
//   QjuiListItem * next;
  

  // Get Time tags
     
  target_D0 = getTimeTags();
  if(target_D0.is_nil() ){
    status("No Target Tags",50000);
    return false;
  }
     
  // if already
  
  // for each item on Join List, set choices
  int numRows=_join_widget->count();
  if(numRows==0)
	  return false;

  vector <double> 	gap_or_width(numRows);
  QdStringSeq_var	gap_method = new QdStringSeq(numRows);
  QdStringSeq_var	join_method = new QdStringSeq(numRows);
  
     QjuiWidgetItem* item;

	for(int i=0; i<numRows; ++i)
	{
		item=(QjuiWidgetItem*)(_join_widget->item(i));
		bool OK;

		QdObject_var obj = QuGetObjByName(item->WLcontext);
		if (obj.is_nil() )
		{
			status("object no longer exists",10000);
			return false;
		}

		if(disjointSeries(obj))
		{
			(void)QMessageBox::warning(this,"Join","disjoint series - ");
			qApp->processEvents();
		}

      
      // gap tolerance in seconds
      
		double gap=item->I_gap_field.toDouble(&OK);
		if(gap < 1.e-20) gap = -1.0; // tell multijoin to work it out
		gap_or_width[i] = gap;
		
      
      // boxcar width in seconds
      
		double boxcar_width=item->I_width_field.toDouble(&OK);
      
      // interpolation method
      
		QdString_var interp_ptr = new QdString( (_algorithm->itemText(item->I_algorithm)).toLatin1().data() );
		join_method[i] = *interp_ptr;
		
		if(*interp_ptr == string(BOXCAR) ){
			gap_or_width[i] = boxcar_width;
		}
		
     // gap handling option
     
		QdString_var gap_opt_ptr = new QdString( (_gap->itemText(item->I_gap)).toLatin1().data() );
		gap_method[i] = *gap_opt_ptr;
           
		string new_obj_name(item->WLcontext.toLatin1().data());
		nameList.push_back(new_obj_name);   
                  
		ToJoinList->push_back(obj);

	}
  // Do actual join
     
  try{
    status("performing join...", 10000);
	
	QuMultiJoin(ToJoinList, target_D0, JoinedList, gap_or_width, gap_method, join_method);

    return true;
  }
  catch(Exception &e){
    status( QString(e.msg().c_str()), 10000);
    return false;
  }


}

// ---------------------------------------------------

 void QjuiWindow::writeFile(){ 

 if(QuSlotDebug())
  {
	printf("QjuiWindow::writeFile %ld\n", (long)this);
	fflush(stdout);
 }
  
   // keep settings for default behaviour
   keepChoices();
   
  QdObjectSeq_var JoinedList = new QdObjectSeq();
  vector <string> nameList; 
  bool OK;
  
   // test if already joined first
  if( alreadyJoined() ){

     // copy over inputs to outputs and set  name in options list
        
    JoinedList->clear();

    int numRows=_join_widget->count();
    if(numRows==0)
	    return;
  
    QjuiWidgetItem* item;

    for(int i=0; i<numRows; ++i)
    {
	    item=(QjuiWidgetItem*)(_join_widget->item(i));
	    QdObject_var obj = QuGetObjByName(item->WLcontext);
	    if (obj.is_nil() )
	    {
		    status("object no longer exists",10000);
		    return;
	    }
        JoinedList->push_back(obj);
	    
        QdString new_obj_name(item->WLcontext.toLatin1().data());
        
        nameList.push_back(new_obj_name);
    }

  }
  else if(joinObjects(JoinedList, nameList)) {     // try to multi-join first
    // join was successful, nothing more to do before assembling export QIE structure
    
  }
  else{
        // DEPEND_0 not set: copy over inputs to outputs
        // for each item on Join List, set new name in options list
    JoinedList->clear();
    int sequence_length = 1;

    int numRows=_join_widget->count();
    if(numRows==0)
	    return;
  
    QjuiWidgetItem* item;

    for(int i=0; i<numRows; ++i)
    {
	    item=(QjuiWidgetItem*)(_join_widget->item(i));
	    QdObject_var obj = QuGetObjByName(item->WLcontext);
	    if (obj.is_nil() )
	    {
		    status("object no longer exists",10000);
		    return;
	    }
	    QdObject_var D0 = get_Depend_0(obj);
		if(!D0.is_nil()) {
	  		// should only be in this loop for data series or join failed
	  		// since tags are present, join must have failed
          	status("Unable to join to target time tags", 50000);
	  	return;
		}

		if(sequence_length == 1) sequence_length = obj->sequence_size();
		else if (sequence_length != obj->sequence_size() && obj->sequence_size() != 1){
          	status("Data sequences of differing lengths.", 50000);
	 	 	return;
		}


        QdString new_obj_name(item->WLcontext.toLatin1().data());
        
        nameList.push_back(new_obj_name);
    }
  
  }
  if(nameList.size() == 0) return;
  if(JoinedList->size() == 0) return;

   //  Build QIE Export file object from QJUI join object and Write file

  QexBuildExportQIEObj (JoinedList, nameList );

  OK = QexWriteFile (this);

    return;

}


 void QjuiWindow::defaultAll(){
 
  _width_field->setText(_width_field_Def);
  _gap->setCurrentIndex(_gap_Def);
  _gap_field->setText(_gap_field_Def);
  _algorithm->setCurrentIndex(_algorithm_Def);
  _TimeIvlSelect->setCurrentIndex(_TimeIvlSelect_Def);
  _IvlSource->setText(_IvlSource_Def);
  _TimeTagSelect->setCurrentIndex(_TimeTagSelect_Def);
  _TagSource->setText(_TagSource_Def);
  _join_widget->setItemChoices(-1);

 
}

 void QjuiWindow::setDefaults()
{
  _width_field_Def = QString("60");
  _width_field_cpy = _width_field_Def;
  
  _gap_Def = 0;
  _gap_cpy = _gap_Def;
  
  _gap_field_Def = QString("6");
  _gap_field_cpy = _gap_field_Def;
  
  _algorithm_Def = 0;
  _algorithm_cpy = _algorithm_Def;
  
  _IvlSource_Def = QString(" ");
  _IvlSource_cpy = _IvlSource_Def;
  
  _TimeIvlSelect_Def = 0;
  _TimeIvlSelect_cpy = _TimeIvlSelect_Def;
  
  _TagSource_Def = QString(" ");
  _TagSource_cpy = _TagSource_Def;
  
  // re-set in join or export, but here to allow safe initialisation
  _TimeTagSelect_Def = 1;
  _TimeTagSelect_cpy = _TimeTagSelect_Def;
}

 void QjuiWindow::keepChoices(){

 if(QuSlotDebug())
 {
	printf("QjuiWindow::keepChoices %ld\n", (long)this);
	fflush(stdout);
 }
 
 
  _width_field_cpy = _width_field->text();
  _gap_cpy = _gap->currentIndex();
  _gap_field_cpy = _gap_field->text();
  _algorithm_cpy = _algorithm->currentIndex();
  _IvlSource_cpy = _IvlSource->text();
  _TimeIvlSelect_cpy = _TimeIvlSelect->currentIndex();
  _TagSource_cpy = _TagSource->text();
  _TimeTagSelect_cpy = _TimeTagSelect->currentIndex();
 
}

 void QjuiWindow::applyChoices(){

 if(QuSlotDebug())
 {
	printf("QjuiWindow::applyChoices %ld\n", (long)this);
	fflush(stdout);
 }
 
 
  _width_field->setText(_width_field_cpy);
  _gap->setCurrentIndex(_gap_cpy);
  _gap_field->setText(_gap_field_cpy);
  _algorithm->setCurrentIndex(_algorithm_cpy);
  _TimeIvlSelect->setCurrentIndex(_TimeIvlSelect_cpy);
  _IvlSource->setText(_IvlSource_cpy);
  _TimeTagSelect->setCurrentIndex(_TimeTagSelect_cpy);
  _TagSource->setText(_TagSource_cpy);
  _join_widget->setItemChoices(-1);
 
}

 void QjuiWindow::setTTivl(int item){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::setTTivl %ld, %d\n", (long)this, item);
	fflush(stdout);
 }
 
 
  if(item == 0) {
    // from data
    IvlSource_label->setText(" ");
     _IvlSource->setToolTip("not active");
    _IvlSource->setText(" "); // force signal so that colour updates
  }
  else if(item == 1){
    IvlSource_label->setText("Interval Object");
     _IvlSource->setToolTip("drag and drop object here");
    _IvlSource->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_NOT_SEQ, QS_ANY));
    _IvlSource->addAcceptable(Qcondition(QS_NARROW, QS_ANY, QS_ANY, QS_SEQ, QS_ANY));
    _IvlSource->setText("");
    _IvlSource->reValidate(true);
  }
}


 void QjuiWindow::setTTsource(int item){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::setTTsource %ld, %d\n", (long)this, item);
	fflush(stdout);
 }
 
 
  if(item == 0) {
    // create regular
    TagSource_label->setText("Spacing");
    _TagSource->setToolTip("enter spacing here (time is in seconds)");
    _TagSource->setText(" ");
	_TagSource->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
  }
  else if(item == 1){
    TagSource_label->setText("Target Tag Object");
    _TagSource->setToolTip("drag and drop object here");
    _TagSource->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ANY, QS_SEQ, QS_ANY));
    _TagSource->addAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_SEQ, QS_ANY));
  }
  _TagSource->reValidate(true); // updates T display if valid
}

// ---------------------------------

 void QjuiWindow::setIntervalOption(){ 
 	_TimeIvlSelect->setCurrentIndex(1);
	setTimeFeedback();
 }

// ---------------------------------

 void QjuiWindow::setTagsOption(){ 
 	
	setTimeFeedback();
 }

// ---------------------------------
 
 void QjuiWindow::setTimeFeedback(){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::setTimeFeedback %ld\n", (long)this);
	fflush(stdout);
 }
 
 
  QdTimeInterval_var ivl;
  int Ntags=-1;
  double Spacing=-1.;
  double Tinterval;
  
  // set interval

  ivl = QjuiGetInterval();
 
  if ( ivl.is_nil() ) {
  	double start;
	double end;
  	if(QjuiGetScalarInterval(start, end)){
		Tinterval = end - start;
    	Start_field->setText(QString("%1").arg(start));
    	End_field->setText(QString("%1").arg(end));
	}
	else{
		Ntags_field->setText("??");
    	spacing_field->setText("??");
    	Start_field->setText("??");
    	End_field->setText("??");
    	Tinterval = -1.0;
	}
  }
  else{
    Start_field->setText(QString(ivl->start().getISOstring().c_str()));
    End_field->setText(QString(ivl->end().getISOstring().c_str()));
    Tinterval = ivl->duration_abs();
  }
  
  // set tags
  
  if( _TimeTagSelect->currentIndex() == 0)
  {
    // create regular tags
    bool OK;
    Spacing = _TagSource->text().toDouble(&OK);
    if(!OK || Spacing < 1.e-20) {
      Spacing = -1.0;
      Ntags = -1;
    }
    else
      Ntags = (int) (Tinterval/Spacing + 1.0);
    
  }
  else
  {
    // tags from object
    QdTimeSeq_var ts = getTimeTagsFromObj();
	if(ts.is_nil()){
		QdRScalarSeq_var ss = getScalarTagsFromObj();
		Spacing = get_scalar_spacing(ss);
	}
    else{
    	Spacing = get_time_spacing(ts);
	}
    if(Spacing > 0) Ntags = (int) (fabs(Tinterval/Spacing) + 1.0);
  }
  
  QString txt;
  if(Ntags > 0) Ntags_field->setText(txt.setNum(Ntags));
  else Ntags_field->setText("??");
  
  if(Spacing > 0) spacing_field->setText(txt.setNum(Spacing));
  else spacing_field->setText("??");
  
}

// --------------------------------

bool QjuiWindow::QjuiGetScalarInterval(double &start, double &end){

  if( _TimeIvlSelect->currentIndex() == 0)
  {
    // interval from data
    return QjuiGetScalarIntervalFromData( start, end);

  }
  else
  {
    // interval from object
    QdObject_var ivl_obj = _IvlSource->getSlotObject();
    if ( !ivl_obj.is_nil() ) {
      
        QdObject_var D0=get_Depend_0(ivl_obj);
        if(D0.is_nil()) return false;
		
		QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(D0);
		if(ss.is_nil() || ss->sequence_size() <=1) return false;
     	
		start = ss[0];
		end = ss[ss->sequence_size()-1];
		return true;
    }

  } 
  return false; 
}

// --------------------------------

QdTimeInterval_var QjuiWindow::QjuiGetInterval(){

  QdTimeInterval_var ivl;
  if( _TimeIvlSelect->currentIndex() == 0)
  {
    // interval from data
    ivl = QjuiGetIntervalFromData();

  }
  else
  {
    // interval from object
    QdObject_var ivl_obj = _IvlSource->getSlotObject();
    if ( !ivl_obj.is_nil() ) {
      ivl = QdTimeInterval_var::narrow(ivl_obj);
      if(ivl.is_nil()){
        QdTimeSeq_var ts=get_timetags(ivl_obj);
        if(!ts.is_nil())
            ivl=new QdTimeInterval(ts->get_interval_bounds());
      }
    }

  } 
  return ivl; 
}

QdTimeInterval_var QjuiWindow::QjuiGetIntervalFromData()
{
  QdTimeInterval_var ivl;
  QdTimeInterval_var ivlUnion;

  int numRows=_join_widget->count();

  QjuiWidgetItem * item;
  for(int i=0; i<numRows; ++i)
  {
	item=(QjuiWidgetItem*)_join_widget->item(i);
	QdObject_var ivl_obj = QuGetObjByName(item->WLcontext);
	if ( !ivl_obj.is_nil() )
	{
		ivl = QdTimeInterval_var::narrow(ivl_obj);
		if(ivl.is_nil())
		{
			QdTimeSeq_var ts=get_timetags(ivl_obj);
			if(!ts.is_nil())
			ivl=new QdTimeInterval(ts->get_interval_bounds());
		}
      	}

	if(ivlUnion.is_nil() ) ivlUnion = ivl;
      else if ( !ivl.is_nil())
      {
        QdTimeInterval_var ivlNew = new QdTimeInterval(ivlUnion->int_union(*ivl));
        ivlUnion = ivlNew;
      }
	
  }
  
  return ivlUnion;
}

bool QjuiWindow::QjuiGetScalarIntervalFromData(double &start, double &end)
{
	bool OK = false;
	double obj_start, obj_end;
	int numRows=_join_widget->count();
	if(numRows < 1) return false;
	
	QjuiWidgetItem * item;
	for(int i=0; i<numRows; ++i)
	{
		item=(QjuiWidgetItem*)_join_widget->item(i);
		QdObject_var ivl_obj = QuGetObjByName(item->WLcontext);
		if( get_ScalarInterval(ivl_obj, obj_start, obj_end ) ){
			if(OK){
				if(start < end){
					if( start > obj_start ) start = obj_start;
					if( end < obj_end ) end = obj_end;
				}
				else{
					if( start < obj_start ) start = obj_start;
					if( end > obj_end ) end = obj_end;
				}
			}
			else{
				// first valid data
				start = obj_start;
				end = obj_end;
			}
		}
		OK=true;	
	}
  
	return OK;
}


QdTimeSeq_var QjuiWindow::getTimeTagsFromObj(){
    
	
	// tags from object
    QdObject_var tag_obj = _TagSource->getSlotObject();
	if(tag_obj.is_nil()){
		// use first object in list
		QjuiWidgetItem* item=(QjuiWidgetItem*)(_join_widget->item(0));
		if(item != 0) tag_obj = QuGetObjByName(item->WLcontext);		
	}
    QdTimeSeq_var ts=get_timetags(tag_obj);

	return ts;
}

QdRScalarSeq_var QjuiWindow::getScalarTagsFromObj(){
    
	QdRScalarSeq_var ss;
	// tags from object
    QdObject_var tag_obj = _TagSource->getSlotObject();
	if(tag_obj.is_nil()){
		// use first object in list
		QjuiWidgetItem* item=(QjuiWidgetItem*)(_join_widget->item(0));
		if(item != 0) tag_obj = QuGetObjByName(item->WLcontext);		
	}
    
	// get DEPEND_0 if it exists or try object itself if not
	QdObject_var D0 = get_Depend_0(tag_obj);
	if(D0.is_nil()) ss = QdRScalarSeq_var::narrow(tag_obj);
	else ss = QdRScalarSeq_var::narrow(D0);
	
	return ss;
}


QdObject_var QjuiWindow::getTimeTags(){

	QdObject_var tags;
	QdTimeInterval_var ivl;
	double Spacing;
	double Tinterval;
	double start, end;

	// set interval

	ivl = QjuiGetInterval();
	if ( ivl.is_nil() ) {
	
		// look for scalar Depend_0
		
  		if(QjuiGetScalarInterval(start, end) == false) return tags; // tags will be nil
		Tinterval = end - start;
		// set scalar tags

  		if( _TimeTagSelect->currentIndex() == 0)
  		{
    		// create regular tags
    		bool OK;
    		Spacing = _TagSource->text().toDouble(&OK);
    		if(!OK || fabs(Spacing) < 1.e-20) return tags;

			if(Tinterval < 0) Spacing = -fabs(Spacing); // ensure spacing -ve if decreasing sequence
			
    		int Ntags = (int) (fabs(Tinterval/Spacing) + 1.0);
	
			QdRScalarSeq_var stags = new QdRScalarSeq();
			stags->assign_regular(Ntags, start, Spacing );
			tags = stags;
		}
  		else
  		{
    		// tags from object
    		QdRScalarSeq_var ss = getScalarTagsFromObj();
    		if(ss.is_nil())  return tags;    // will be .is_nil()

    		// truncate to interval if necessary
			QdRScalarSeq_var stags;
    		if ( start < end && (ss[0] < start || ss[ss->sequence_size()-1] > end) ) {

      			SequenceMask msk(ss->get_mask(QdRScalarSeq::InRange(start,end)));
      			stags = new QdRScalarSeq();
      			ss->get_subsequence(*stags, msk);
    		}
    		else if ( start > end && (ss[0] > start || ss[ss->sequence_size()-1] < end) ) {

      			SequenceMask msk(ss->get_mask(QdRScalarSeq::InRange(start,end)));
      			stags = new QdRScalarSeq();
      			ss->get_subsequence(*stags, msk);
    		}
    		else stags = new QdRScalarSeq(*ss);
			tags = stags;
		}
  	}
  	else {
		Tinterval = ivl->duration_abs();
  		// set tags

  		if( _TimeTagSelect->currentIndex() == 0)
  		{
    		// create regular tags
    		bool OK;
    		Spacing = _TagSource->text().toDouble(&OK);
    		if(!OK || Spacing < 1.e-20) return tags;

    		int Ntags = (int) (Tinterval/Spacing + 1.0);
	
			QdTimeSeq_var ttags = new QdTimeSeq();
			ttags->assign_regular(Ntags, ivl->start(), Spacing );
			tags = ttags;
		}
  		else
  		{
    		// tags from object
    		QdTimeSeq_var ts = getTimeTagsFromObj();
    		if(ts.is_nil())  return tags;    // will be .is_nil()

    		// truncate to interval if necessary

    		if (( ivl->start() != ts->get_interval_bounds().start()) ||
        		( ivl->end()   != ts->get_interval_bounds().end())) {

      			SequenceMask msk(ts->get_mask(QdTimeSeq::InRange(ivl->start(),ivl->end())));
      			QdTimeSeq_var ttags = new QdTimeSeq();
      			ts->get_subsequence(*ttags, msk);
				tags = ttags;
    		}
    		else {
				QdTimeSeq_var ttags = new QdTimeSeq(*ts);
				tags = ttags;
			}
		}
	}

	return tags;

}

bool QjuiWindow::disjointSeries(QdObject_var obj)
{
// checks whether obj is disjoint Time Series.


  if(obj.is_nil())
    return false;

  String objName;

  if(obj->xref_exists(OBJECT_NAME))
  {
    QdString_var objStr=QdString_var::narrow(obj->get_xref(OBJECT_NAME));
    if(!objStr.is_nil())
      objName=*objStr;
  }

#ifdef DIAG
  if(objName!="")
    cout << "checking object " << objName << endl;
#endif

  // fetch time interval associated with this object
  QdTimeInterval_var obj_ivl=get_timeinterval(obj);
  if(obj_ivl.is_nil())
    return false;

  bool result=true;

  // iterate thru' join listview and compare time intervals of each join object
  // if no object is found such that the intersection of the time intervals of
  // both objects is not empty, then the series is disjoint - so return true

  
  int n_series=0;
  int numRows=_join_widget->count();

  for(int i=0; i<numRows; ++i)
  {
	  QjuiWidgetItem* join_item=(QjuiWidgetItem*)(_join_widget->item(i));

        QdObject_var join_obj = QuGetObjByName(join_item->WLcontext);

	String joinObjName;

	if(join_obj->xref_exists(OBJECT_NAME))
        {
          QdString_var objStr=QdString_var::narrow(join_obj->get_xref(OBJECT_NAME));
          if(!objStr.is_nil())
            joinObjName=*objStr;
        }

#ifdef DIAG
	if(joinObjName!="")
	  cout << "about to compare with " << joinObjName << endl;
#endif

	// ensure we are looking at different objects
	if(join_obj!=obj)
	{
	  n_series++;
          QdTimeInterval_var join_ivl=get_timeinterval(join_obj);
	  if(!join_ivl.is_nil())
	  {
            if(!obj_ivl->int_intersect(*join_ivl).empty())
              return false;
	  }
	}

  }

  if(n_series==0)
    return false;
  else
    return result;
}

void QjuiWindow::messageHandler(QString txt){

 if(QuSlotDebug())
 {
	printf("QjuiWindow::messageHandler %ld\n", (long)this);
	fflush(stdout);
 }


  this->status(txt, 10000);
}

QDomElement QjuiWidgetItem::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("Join_Item");
    xml.appendChild(createTextXMLNode(doc, "Object", WLcontext));
    xml.appendChild(createTextXMLNode(doc, "Algorithm", I_algorithm));
    xml.appendChild(createTextXMLNode(doc, "Width", I_width_field));
    xml.appendChild(createTextXMLNode(doc, "Gap_Handling", I_gap));
    xml.appendChild(createTextXMLNode(doc, "Gap_Threshold", I_gap_field));

    return xml;
}

void QjuiWidgetItem::fromXMLElement(QDomElement& el)
{
    if(!el.isNull())
    {
		QString s;
		if(getXMLNodeText(el, "Object", s))
		{
			setText(s);
			WLcontext = s;
		}
//         QDomElement object=el.firstChildElement("Object");
//         if(!object.isNull())
//         {
//             setText(object.firstChild().nodeValue());
//             WLcontext = object.firstChild().nodeValue();
//         }

		if(getXMLNodeText(el, "Algorithm", s))
		{
			I_algorithm=s.toInt();
		}
		if(getXMLNodeText(el, "Width", s))
		{
			I_width_field=s;
		}
		if(getXMLNodeText(el, "Gap_Handling", s))
		{
			I_gap=s.toInt();
		}
		if(getXMLNodeText(el, "Gap_Threshold", s))
		{
			I_gap_field=s;
		}
//         QDomElement algo=el.firstChildElement("Algorithm");
//         if(!algo.isNull()) I_algorithm=algo.firstChild().nodeValue().toInt();
// 
//         QDomElement width=el.firstChildElement("Width");
//         if(!width.isNull()) I_width_field=width.firstChild().nodeValue();
// 
//         QDomElement gap=el.firstChildElement("Gap_Handling");
//         if(!gap.isNull()) I_gap=gap.firstChild().nodeValue().toInt();
// 
//         QDomElement gapT=el.firstChildElement("Gap_Threshold");
//         if(!gapT.isNull()) I_gap_field=gapT.firstChild().nodeValue();
    }
}

Vostream& operator<<(Vostream& v_os,QjuiWidgetItem& item)
{
	// QguListItem part (compatibility with old list)

	v_os << (int)0;//item.with;
	v_os << std::string(item.text().toLatin1().data());
	v_os << false;//item.Lock;
	v_os << ListTypeEntry;//item.entryType;
	
	// end QguListItem part

     v_os << std::string(item.I_width_field.toLatin1().data());
     v_os << std::string(item.I_gap_field.toLatin1().data());
     v_os << item.I_algorithm;
     v_os << item.I_gap;
     v_os << std::string(item.WLcontext.toLatin1().data());

   v_os << 0;
   
   return v_os;
}

Vistream& operator>>(Vistream& v_is, QjuiWidgetItem& item)
{
	std::string text;
	std::string WLcontext;
	
	// QguListItem part (compatibility with old list)

	int with;
	bool lock;
	int entryType;
	v_is >> with; // not used any more
	v_is >> text;
	if( qsas_restore_release < 20400) cleanWLtext(text);
	item.setText(text.c_str());
	v_is >> lock;
	v_is >> entryType;

	// QguListItem part done
	if(entryType == ListTypeEntry)
	{
		string width_str;
		v_is >> width_str;
		item.I_width_field=width_str.c_str();
		string gap_str;
		v_is >> gap_str;
		item.I_gap_field=gap_str.c_str();
		v_is >> item.I_algorithm;
		v_is >> item.I_gap;
		string contxt;
		v_is >> contxt;
		item.WLcontext = contxt.c_str();
	}
	else
	{
		item.toBeDeleted=true;
	}
	
	int cnt;
	v_is >> cnt;
	for(int i=0;i<cnt;i++)
	{
		QjuiWidgetItem *newItem=new QjuiWidgetItem("", item.listWidget());
		v_is >> *newItem;
		if(newItem->toBeDeleted)
			delete newItem;
	}
	return v_is;
}

QDomElement QjuiWidget::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("List");
    for(int i=0; i<count(); ++i)
    {
        xml.appendChild(((QjuiWidgetItem*)item(i))->toXMLElement(doc));
    }
    return xml;
}

void QjuiWidget::fromXMLElement(QDomElement& el)
{
    if(!el.isNull())
    {
        QDomElement item=el.firstChildElement("Join_Item");
        while(!item.isNull())
        {
            QjuiWidgetItem *i=new QjuiWidgetItem("", this);
            i->fromXMLElement(item);
            item=item.nextSiblingElement("Join_Item");
        }
    }
}

Vostream& operator<<(Vostream& v_os,QjuiWidget& lv)
{
	int cnt=lv.count();


  v_os << cnt;
  QjuiWidgetItem * item;
  for(int i=0; i<cnt; ++i)
  {
	  item=(QjuiWidgetItem*)lv.item(i);
	  v_os << *item;
  }

  return v_os;
}

Vistream& operator>>(Vistream& v_is, QjuiWidget& lv)
{
	int cnt;
	v_is >> cnt;

	for(int i=0;i<cnt;i++)
	{
		QjuiWidgetItem *item=new QjuiWidgetItem("", &lv);
		v_is >> *item;
		if(item->toBeDeleted)
			delete item;
	}
// 	lv.listModified();

	return v_is;
}

