//
// Data Slice Widget
//
// Nov 2012: Tony Allen
//

#include <stdlib.h>

#include <qpushbutton.h>
#include <qlayout.h>
#include "qops.h"
#include "qwui.h"
#include "qar.h"
extern qwuiWindow* qsasWindow;

using namespace QSAS;

MergeTSWindow::MergeTSWindow(QWidget *parent) 
	: QMainWindow(/*parent*/) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{
	if(QuSlotDebug())
	{
		printf("MergeTSWindow::MergeTSWindow \n");
		fflush(stdout);
	}
  	this->setWindowTitle("Merge Time Series tool");
  
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slots
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QVBoxLayout * slotLayout=new QVBoxLayout;
	slotPanel->setTitle("TS Data Objects to Merge");
	
	objField1 = new QguWLinput(slotPanel,"objField1");
	objField1->setAcceptable(Qcondition(QS_NARROW, QS_ANY, QS_ANY, QS_SEQ, QS_TT));
	objField1->setEnabled(true);
	objField1->setMinimumWidth(char_width*55);
	slotLayout->addWidget(objField1);
	
	objField2 = new QguWLinput(slotPanel,"objField2");
	objField2->setAcceptable(Qcondition(QS_NARROW, QS_ANY, QS_ANY, QS_SEQ, QS_TT));
	objField2->setEnabled(true);
	objField2->setMinimumWidth(char_width*55);
	slotLayout->addWidget(objField2);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setText("MergedTS");
	name_field->setMinimumWidth(char_width*50);
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	doBtn = new QPushButton( "Merge Objects ");
	doBtn->setMinimumSize(doBtn->sizeHint());
	connect( doBtn, SIGNAL(clicked()), this, SLOT(merge()) );
	doBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(doBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	dismissBtn->setDefault(true);
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);
		
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}


void MergeTSWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("MergeTSWindow::setObject \n");
	fflush(stdout);
 }
  
   objField1->setText(WLctxt);
   objField1->reValidate();
   
}


void MergeTSWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("MergeTSWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( doBtn->isDefault()  ) {
		   doBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   doBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(doBtn->isDefault()) merge();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void MergeTSWindow::merge(){
      
	if(QuSlotDebug())
	{
		printf("MergeTSWindow::merge \n");
		fflush(stdout);
	}

	QString obj1String = objField1->text();
	QdObject_var dobj1 = QuGetObjByNameAndComp(obj1String);

	QString obj2String = objField2->text();
	QdObject_var dobj2 = QuGetObjByNameAndComp(obj2String);

	// this may be the original untouched object, so duplicate to be safe...
 
	QdObject_var resObj = QuMergeObjects(dobj1, dobj2);

	if( !resObj.is_nil()) // ensures error message is not overwritten by empty obj rejected
	{
		QString newName = name_field->text();
		QwuiPutOnWL(resObj, QString("Data/")+newName);
	}
	
	this->close();
	return;
}

// ----------------------------------------------

ArrayTraceWindow::ArrayTraceWindow(QWidget *parent) 
	: QMainWindow(0) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{

	if(QuSlotDebug())
	{
		printf("ArrayTraceWindow::ArrayTraceWindow \n");
		fflush(stdout);
	}
    
	this->setWindowTitle("");

	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	slotPanel->setTitle("Square 2D Array");
	
	objField = new QguWLinput(slotPanel,"objField");
	objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_2D, QS_ANY, QS_ANY));
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	connect(objField, SIGNAL(edited_OK()), this, SLOT(setNewName()));
	slotLayout->addWidget(objField);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	userSet = false;
	connect( name_field,  SIGNAL( edited_OK() ), this, SLOT( setUserInput() ) );
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Save ");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptSlice()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	dismissBtn->setDefault(true);
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}

void ArrayTraceWindow::setUserInput(){
  if(name_field->lineBox->hasFocus()){
	
     if(name_field->lineBox->text().isEmpty() ) userSet = false; // allow refresh from drop to object slot
	 else userSet = true; // user typed input, disable update from data slot
  }
  // else ignore
}



void ArrayTraceWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("ArrayTraceWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   objField->reValidate();
   
}

void ArrayTraceWindow::setNewName(){
     
 if(QuSlotDebug())
 {
	printf("ArrayTraceWindow::setNewName \n");
	fflush(stdout);
 }

   if (userSet) return;
   QString objName = QString(this->objField->textObject().toLatin1().data());
   
   if(!objName.isEmpty()){
       QString newName = QString(objName);
       
     int ptr = newName.lastIndexOf("/");
     if (ptr >= 0) newName.remove(0, ptr+1);
	 ptr = newName.indexOf(" [");
	 if (ptr > 0) newName.truncate(ptr);
     newName += "_Tr";
     this->name_field->setText(newName);
   }
   

}

void ArrayTraceWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("ArrayTraceWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptSlice();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void ArrayTraceWindow::acceptSlice(){
      
	if(QuSlotDebug())
	{
		printf("ArrayTraceWindow::acceptSlice \n");
		fflush(stdout);
	}
  
	QdObject_var dobj = objField->getSlotObject();
	QdObject_var retObj;
	
	QdRMatrix_var mobj = QdRMatrix_var:: narrow(dobj);
	QdRMatrixSeq_var msobj = QdRMatrixSeq_var:: narrow(dobj);
  
	if(!mobj.is_nil())
	{
		if(mobj->rank() != 2) 
		{
			QuMsgBar("Input matrix not 2D", true);
			return;
		}
		
		if( (mobj->dims())[0] != (mobj->dims())[1] )
		{
			QuMsgBar("Input matrix not square", true);
			return;
		}

		QdRScalar_var Trace = new QdRScalar();
		try
		{
			*Trace = (*mobj)[0][0];
			for(int i=1; i<(mobj->dims())[0]; i++)
			{
				*Trace += (*mobj)[i][i];
			}
			retObj = Trace;
		}
		catch(Exception &e)
		{        
			QuMsgBar( e.msg().c_str(), true);
			return;
		}
	}
	else if (!msobj.is_nil()) 
	{
		if(msobj[0].rank() != 2) 
		{
			QuMsgBar("Input matrices not 2D", true);
			return;
		}
		
		if( (msobj[0].dims())[0] != (msobj[0].dims())[1] )
		{
			QuMsgBar("Input matrices not square", true);
			return;
		}
		QdRScalarSeq_var Trace = new QdRScalarSeq(msobj->sequence_size());
		try
		{
			for(int j=0; j<msobj->sequence_size(); j++)
			{
				Trace[j] = (msobj[j])[0][0];
				for(int i=1; i<(msobj[i].dims())[0]; i++)
				{
					Trace[j] += (msobj[j])[i][i];
				}
			}
			retObj = Trace;
		}
		catch(Exception &e)
		{        
			QuMsgBar( e.msg().c_str(), true);
			return;
		}

	}
	else
	{
		QuMsgBar("Input not a matrix", true);
		return;
	}
  
  	// set xrefs
	
	if(dobj->xref_exists("DEPEND_0") ){
		QdObject_var xref = dobj->get_xref("DEPEND_0");
		retObj->change_xref("DEPEND_0", xref);
	}
	if(dobj->xref_exists("UNITS") ){
		QdObject_var xref = dobj->get_xref("UNITS");
		retObj->change_xref("UNITS", xref);
	}
	if(dobj->xref_exists("SI_CONVERSION") ){
		QdObject_var xref = dobj->get_xref("SI_CONVERSION");
		retObj->change_xref("SI_CONVERSION", xref);
	}
	if(dobj->xref_exists("FIELDNAM") ){
		QdObject_var xref = dobj->get_xref("FIELDNAM");
		QdString_var sxref = QdString_var::narrow(xref);
		if(!sxref.is_nil()){
			*sxref += " trace";
			retObj->change_xref("FIELDNAM", sxref);
		}
	}

	QString newName = name_field->text();
	if( !retObj.is_nil() ) QwuiPutOnWL(retObj, QString("Data/")+newName);
	else QuMsgBar("NULL object from array Trace", true);

	this->close();
	return;
}

// ----------------------------------------------

AngleDepWindow::AngleDepWindow(int angTop) 
	: QMainWindow() 
{

	if(QuSlotDebug())
	{
		printf("AngleDepWindow::AngleDepWindow \n");
		fflush(stdout);
	}
    top = angTop;
	this->setWindowTitle(" ");

	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	if(angTop == 360) this->setWindowTitle("Convert Angle Depends to use 0 to 360 deg");
	else this->setWindowTitle("Convert Angle Depends to use -180 to 180 deg");
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	slotPanel->setTitle("Data Object");
	
	objField = new QguWLinput(slotPanel,"objField");
	objField->setAcceptable(Qcondition());
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	connect(objField, SIGNAL(edited_OK()), this, SLOT(setNewName()));
	connect(objField, SIGNAL(hadDropEvent()), this, SLOT(setAccept()));
	slotLayout->addWidget(objField);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	userSet = false;
	connect( name_field,  SIGNAL( edited_OK() ), this, SLOT( setUserInput() ) );
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Save ");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptOperation()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	dismissBtn->setDefault(true);
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);
	
	//Done
	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}

void AngleDepWindow::setUserInput(){
  if(name_field->lineBox->hasFocus()){
	
     if(name_field->lineBox->text().isEmpty() ) userSet = false; // allow refresh from drop to object slot
	 else userSet = true; // user typed input, disable update from data slot
  }
  // else ignore
}



void AngleDepWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("DataSliceWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   setAccept(); // ensure correct drop down if opened with preselected object
   objField->reValidate();
   
}

void AngleDepWindow::setAccept(){
    
	if(QuSlotDebug())
	{
		printf("AngleDepWindow::setAccept \n");
		fflush(stdout);
	}
  
	QdObject_var dobj = objField->getSlotObject();

	// special case of event table
	QdTimeIntervalSeq_var tis;
	if(!dobj.is_nil()) tis = QdTimeIntervalSeq_var::narrow(dobj);
	if(!tis.is_nil()) 
	{
		objField->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_ANY, QS_ANY));
		objField->reValidate();
	}
   else objField->setAcceptable(Qcondition());
   
}

void AngleDepWindow::setNewName(){
     
 if(QuSlotDebug())
 {
	printf("AngleDepWindow::setNewName \n");
	fflush(stdout);
 }

   if (userSet) return;
   QString objName = QString(this->objField->textObject().toLatin1().data());
   
   if(!objName.isEmpty()){
       QString newName = QString(objName);
       
     int ptr = newName.lastIndexOf("/");
     if (ptr >= 0) newName.remove(0, ptr+1);
	 ptr = newName.indexOf(" [");
	 if (ptr > 0) newName.truncate(ptr);
     newName += QString("_M");
     this->name_field->setText(newName);
   }
   

}

void AngleDepWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("AngleDepWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptOperation();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void AngleDepWindow::acceptOperation(){
      
	if(QuSlotDebug())
	{
		printf("AngleDepWindow::acceptSlice \n");
		fflush(stdout);
	}
  
	QdObject_var dobj = objField->getSlotObject();
	QdObject_var newObject;
	QdObject_var newD0;
	QString newName = name_field->text();
	
	QdRMatrix_var M_var;
	QdRMatrixSeq_var MS_var;
	if( !(dobj.is_nil() ) ) 
	{   
		// copy object
		QdRMatrix_var Mnarrow = QdRMatrix_var::narrow(dobj);
		QdRMatrixSeq_var MSnarrow = QdRMatrixSeq_var::narrow(dobj);
		newD0 = getcopy_Depend_0(dobj);
		
		int rank;
		if( !Mnarrow.is_nil() ){
		
			M_var = Factory::clone(Mnarrow);
			dobj->copy_xrefs_to(M_var);
			if( ! newD0.is_nil() ) M_var->change_xref(DEPEND_0, newD0);
			newObject = M_var;
			rank = M_var->rank();
		}
		else if( !MSnarrow.is_nil() ){
		
			MS_var = Factory::clone(MSnarrow);
			dobj->copy_xrefs_to(MS_var);
			if( ! newD0.is_nil() ) MS_var->change_xref(DEPEND_0, newD0);
			newObject = MS_var;
			
			rank = MS_var[0].rank();
		}
		else
		{
			QuMsgBar("Data object not recognised", true);
			return;
		}
		
		// find each DEPEND_i
		for(int i=1; i <= rank; i++){
			QString dep = QString("DEPEND_%1").arg(i);
			if( !newObject->xref_exists(dep.toLatin1().data()) ) continue;
			QdObject_var xref = newObject->get_xref(dep.toLatin1().data());
			
			// If angle DEPEND modify
			string angle = QuAngleUnits(xref);
			if( angle.empty() ) continue;
			
			QdObject_var newXref = QuAngleMod(xref, top, angle.c_str());
			newObject->change_xref(dep.toLatin1().data(), newXref);
		}
		
		QwuiPutOnWL(newObject, QString("Data/")+newName);
			

	}
	else QuMsgBar("NULL input object", true);
	
	this->close();
	return;
}

void AngleDepWindow::dismiss(){
      
	if(QuSlotDebug())
	{
		printf("AngleDepWindow::dismiss \n");
		fflush(stdout);
	}
	
	this->close();
	return;
}



// -------------Derivative Window class ---------------------------------

DerivativeWindow::DerivativeWindow() 
	: QMainWindow() 
{

	if(QuSlotDebug())
	{
		printf("DerivativeWindow::DerivativeWindow \n");
		fflush(stdout);
	}
	this->setWindowTitle("Derivative Estimator");
	
  	QMenu * help = new QMenu( this );
	this->menuBar()->addMenu(help );
	help->addAction("Differentiation in QSAS", this, SLOT( launchHelp() ));
	help->setTitle(" Help ");
	
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
     
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	slotPanel->setTitle("Differentiate...");
	
	objField = new QguWLinput(slotPanel,"objField");
	objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ANY, QS_ANY, QS_ANY));
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	connect(objField, SIGNAL(edited_OK()), this, SLOT(setNewName()));
	slotLayout->addWidget(objField);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// Method panel
	
	QGroupBox * method_panel=new QGroupBox(mainWidget);
	QVBoxLayout * method_layout=new QVBoxLayout;
	
	derivMethod = new QComboBox(method_panel);
	derivMethod->addItem("3-point estimate");
	derivMethod->addItem("5-point estimate" );
	method_layout->addWidget( derivMethod, 0,Qt::AlignLeft );
	
	
	methodLabl = new QLabel("(f(x+h) - f(x-g) ) / (h + g) \nTwo point estimate used at edges and gaps");
	method_layout->addWidget( methodLabl, 1, Qt::AlignLeft );

	connect( derivMethod,  SIGNAL( currentIndexChanged(const QString &) ), this, SLOT( setMethod(const QString &) ) );
	
	method_panel->setLayout(method_layout);
	mainLayout->addWidget(method_panel);

	// Gap panel
	
	QGroupBox * gap_panel=new QGroupBox(mainWidget);
	QHBoxLayout * gap_layout=new QHBoxLayout;
	
	QLabel *gapLabl = new QLabel("Gap Threshold: ");
	gap_layout->addWidget( gapLabl, 0, Qt::AlignLeft );

	gapSizeField = new QguWLinput(gap_panel,"gapField");	
	gapSizeField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	gapSizeField->setToolTip("Treat spacing above this size as a gap (time gaps are in seconds)");
	gapSizeField->setDouble(6.0);

	gap_layout->addWidget( gapSizeField, 1, Qt::AlignLeft );
		
	gap_panel->setLayout(gap_layout);
	mainLayout->addWidget(gap_panel);

// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	userSet = false;
	connect( name_field,  SIGNAL( edited_OK() ), this, SLOT( setUserInput() ) );
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Do Derivative ");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(doDerivative()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	dismissBtn->setDefault(true);
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}

DerivativeWindow::~DerivativeWindow() 
{
}
// -------------

void DerivativeWindow::launchHelp() 
{
	 if(QuSlotDebug())
 	{
		printf("DerivativeWindow::launchHelp %ld\n", (long)this);
		fflush(stdout);
	 }
 
  	qsasWindow->help_window->manEntry("QSAS_analysis.html","#differentiation");

}
// -------------

void DerivativeWindow::setUserInput(){
  if(name_field->lineBox->hasFocus()){
	
     if(name_field->lineBox->text().isEmpty() ) userSet = false; // allow refresh from drop to object slot
	 else userSet = true; // user typed input, disable update from data slot
  }
  // else ignore
}

// -------------

void DerivativeWindow::setMethod(const QString & text){
    
	if( text.contains("3") ) {
		methodLabl->setText("(f(x+h) - f(x-g) ) / (h + g) \nTwo point estimate used at edges and gaps");
		gapSizeField->setEnabled(true);
	}
	else 	{
		methodLabl->setText("(-f(x+2h) + 8f(x+h) - 8f(x-h) + f(x-2h) ) / 12h \nWill join onto regularly spaced sequence first");
		gapSizeField->setEnabled(false);
	}
}


// -------------

void DerivativeWindow::setNewName(){
     
	if(QuSlotDebug())
	{
		printf("DerivativeWindow::setNewName \n");
		fflush(stdout);
	}
	
	// first set default gap
	QdObject_var dobj = objField->getSlotObject();

	QdTimeSeq_var tt;
	QdRScalarSeq_var ss;
	QdObject_var d0;
	if(dobj->xref_exists(DEPEND_0) ){
		d0 = dobj->get_xref(DEPEND_0);
		tt = QdTimeSeq_var::narrow(d0);
		ss = QdRScalarSeq_var::narrow(d0);
	}
	
	double spacing=0;
	if(!tt.is_nil() && tt->sequence_size() > 1) spacing = tt[1] - tt[0];
	else if(!ss.is_nil() && ss->sequence_size() > 1) spacing = ss[1] - ss[0];
	
	if(spacing > 0) gapSizeField->setDouble(spacing *1.5);
	
	// set output name default
   if (userSet) return;
   QString objName = QString(this->objField->textObject().toLatin1().data());
   
   if(!objName.isEmpty()){
		QString newName = QString(objName);
       
		int ptr = newName.lastIndexOf("/");
		if (ptr >= 0) newName.remove(0, ptr+1);
		ptr = newName.indexOf(" [");
		if (ptr > 0) newName.truncate(ptr);

		this->name_field->setText("d_"+newName);
   }

}

// -------------

void DerivativeWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("DerivativeWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) doDerivative();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

// -------------

void DerivativeWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("DerivativeWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   objField->reValidate(true);
   
}

// -------------

void DerivativeWindow::doDerivative(){
      
	if(QuSlotDebug())
	{
		printf("DerivativeWindow::doDerivative \n");
		fflush(stdout);
	}
	qApp->processEvents();
	
	double gapSize = gapSizeField->value();
	
	QdObject_var dobj = objField->getSlotObject();
	
	if(dobj->sequence_size() < 2){
		QuMsgBar("Input sequence too short", true);
		return;
	}
	QdTimeSeq_var tt;
	QdRScalarSeq_var ss;
	QdObject_var d0;
	QdRScalarSeq_var h = new QdRScalarSeq(dobj->sequence_size());
	
	if(dobj->xref_exists(DEPEND_0) ){
		d0 = dobj->get_xref(DEPEND_0);
		tt = QdTimeSeq_var::narrow(d0);
		ss = QdRScalarSeq_var::narrow(d0);
	}
	if(tt.is_nil() && ss.is_nil()){
		QuMsgBar("Input sequence has no useable DEPEND_0", true);
		return;
	}
	
	if(!tt.is_nil()){
		h[0] = 0.0;
		for(int i=1; i<dobj->sequence_size(); i++){
			h[i] = tt[i] - tt[i-1];
		}
	}
	else if(!ss.is_nil()){
		h[0] = 0.0;
		for(int i=1; i<dobj->sequence_size(); i++){
			h[i] = ss[i] - ss[i-1];
		}
	}
	
	QdObject_var retObj;
	QdRMatrixSeq_var msobj = QdRMatrixSeq_var:: narrow(dobj);
	QdRScalarSeq_var ssobj = QdRScalarSeq_var:: narrow(dobj);
	
	if(derivMethod->currentIndex() == 0){
	
		// 3 point estimate	

		if (!ssobj.is_nil()) 
		{

			QdRScalarSeq_var retS = new QdRScalarSeq(ssobj->sequence_size());
			// do first entry as 2-point estimate
			retS[0] = (ssobj[1] - ssobj[0])	/ h[1];
			for(int i=1; i<ssobj->sequence_size()-1; i++){
				// use 2-point measure if gap
				if(h[i+1] > gapSize) {
					retS[i] = (ssobj[i] - ssobj[i-1]) / h[i];
					i++;
					// and set start of next block with 2-point estimate
					if(i<ssobj->sequence_size()-1) retS[i] = (ssobj[i+1] - ssobj[i]) / h[i+1]; 

				}
				else retS[i] = (ssobj[i+1] - ssobj[i-1]) / (h[i] + h[i+1]);
			}
			// do last entry as 2-point estimate
			retS[ssobj->sequence_size()-1] = (ssobj[ssobj->sequence_size()-1] - ssobj[ssobj->sequence_size()-2]) / h[ssobj->sequence_size()-1];
			
			retObj = retS;
			retObj->copy_xrefs_from(ssobj);
			
		}
  		else if (!msobj.is_nil()) 
		{

			QdRMatrixSeq_var retM = Factory::clone(msobj);
			
			// do first entry as 2-point estimate
			retM[0] = (msobj[1] - msobj[0])	/ h[1];
			for(int i=1; i<msobj->sequence_size()-1; i++){
				// ensure no gaps 
				if(h[i+1] > gapSize) {
					retM[i] = (msobj[i] - msobj[i-1]) / h[i];
					i++;
					// set start of next block with 2-point estimate
					if(i<msobj->sequence_size()-1) retM[i] = (msobj[i+1] - msobj[i]) / h[i+1]; 

				}
				else retM[i] = (msobj[i+1] - msobj[i-1]) / (h[i] + h[i+1]); // 3-point estimate
			}
			// do last entry as 2-point estimate
			retM[msobj->sequence_size()-1] = (msobj[msobj->sequence_size()-1] - msobj[msobj->sequence_size()-2]) / h[msobj->sequence_size()-1];
			
			retObj = retM;
			retObj->copy_xrefs_from(msobj);

		}
		else
		{
			QuMsgBar("Input not a recognised sequence", true);
			return;
		}
	}
	else {
		// 5 point estimate

		// First test for regular data
		double hReg;
		bool isReg;
		if(!tt.is_nil()) isReg = QuIsTSRegular(tt, &hReg);
		else  isReg = QuIsSSRegular(ss, &hReg);
		
		double h12 = 12.* hReg;
		double h2 = 2.* hReg;
		gapSize = 1.5 * hReg;
		
		if(!isReg){
			// make tt or ss regular and multijoin
			if(tt.is_nil() ){
				// scalar sequence
				double range = ss[ss->sequence_size()-1] - ss[0];
				int Ntags = (int) (range/hReg + 1.0);
    			QdRScalarSeq_var stags = new QdRScalarSeq();
    			stags->assign_regular(Ntags, ss[0], hReg );
				
				if(!ssobj.is_nil()){
					QdRScalarSeq_var newSS = new QdRScalarSeq(Ntags);
					newSS[0] = ssobj[0];
					int j = 0;
					for(int i=1; i<Ntags; i++){
						while(ss[j] < stags[i] ) j++;
						if(j >= ssobj->sequence_size() ) newSS[i] = ssobj[ssobj->sequence_size() - 1]; // catch last entry
						else newSS[i] = (ssobj[j-1] * (ss[j] - stags[i]) + ssobj[j] * (stags[i] - ss[j-1]) ) / (ss[j] - ss[j-1]); 
					}
					newSS->copy_xrefs_from(ssobj);
					ssobj = newSS;
					ssobj->change_xref(DEPEND_0, stags);
				}
				else if(!msobj.is_nil()){
					QdRMatrixSeq_var newMS = new QdRMatrixSeq(Ntags, msobj[0]);
					newMS[0] = msobj[0];
					int j = 0;
					for(int i=1; i<Ntags; i++){
						while(ss[j] < stags[i] ) j++;
						if(j >= msobj->sequence_size() ) newMS[i] = msobj[msobj->sequence_size() - 1]; // catch last entry
						else newMS[i] = (msobj[j-1] * (ss[j] - stags[i]) + msobj[j] * (stags[i] - ss[j-1]) ) / (ss[j] - ss[j-1]); 
					}
					newMS->copy_xrefs_from(msobj);
					msobj = newMS;
					msobj->change_xref(DEPEND_0, stags);
				}
				
			}
			else{
				// time sequence
				double Tinterval = tt->get_interval_bounds().duration_abs();
				int Ntags = (int) (Tinterval/hReg + 1.0);

    			QdTimeSeq_var ttags = new QdTimeSeq();
    			ttags->assign_regular(Ntags, tt[0], hReg );
			
  				if(!ssobj.is_nil()){
					QdRScalarSeq_var newSS = new QdRScalarSeq(Ntags);
					newSS[0] = ssobj[0];
					int j = 0;
					for(int i=1; i<Ntags; i++){
						while(tt[j] < ttags[i] && j < ssobj->sequence_size()) j++;
						if(j >= ssobj->sequence_size() ) newSS[i] = ssobj[ssobj->sequence_size() - 1]; // catch last entry
						else newSS[i] = (ssobj[j-1] * (tt[j] - ttags[i]) + ssobj[j] * (ttags[i] - tt[j-1]) ) / (tt[j] - tt[j-1]); 
					}
					newSS->copy_xrefs_from(ssobj);
					ssobj = newSS;
					ssobj->change_xref(DEPEND_0, ttags);
				}
				else if(!msobj.is_nil()){
					QdRMatrixSeq_var newMS = new QdRMatrixSeq(Ntags, msobj[0]);
					newMS[0] = msobj[0];
					int j = 0;

					for(int i=1; i<Ntags; i++){

						while(tt[j] < ttags[i] && j < msobj->sequence_size()) j++;
						if(j >= msobj->sequence_size() ) newMS[i] = msobj[msobj->sequence_size() - 1]; // catch last entry
						else newMS[i] = (msobj[j-1] * (tt[j] - ttags[i]) + msobj[j] * (ttags[i] - tt[j-1]) ) / (tt[j] - tt[j-1]); 
					}
					newMS->copy_xrefs_from(msobj);
					msobj = newMS;
					msobj->change_xref(DEPEND_0, ttags);
				}

			}
		}

		// Do derivative
		
		if (!ssobj.is_nil()) 
		{

			QdRScalarSeq_var retS = new QdRScalarSeq(ssobj->sequence_size());
			
			// do first entry as 2-point estimate
			retS[0] = (ssobj[1] - ssobj[0])	/ hReg;
			// do second entry as 3-point estimate
			retS[1] = (ssobj[2] - ssobj[0])	/ h2;
			
			for(int i=2; i<ssobj->sequence_size()-2; i++){
				retS[i] = (-ssobj[i+2] + 8.*ssobj[i+1] - 8.*ssobj[i-1] + ssobj[i-2]) / h12; // 5-point estimate
			}
			// do last entries as 2-point and 3-point estimate
			retS[ssobj->sequence_size()-2] = (ssobj[ssobj->sequence_size()-1] - ssobj[ssobj->sequence_size()-3]) / h2;
			retS[ssobj->sequence_size()-1] = (ssobj[ssobj->sequence_size()-1] - ssobj[ssobj->sequence_size()-2]) / hReg;
			
			retObj = retS;
			retObj->copy_xrefs_from(ssobj);
			
		}
  		else if (!msobj.is_nil()) 
		{

			QdRMatrixSeq_var retM = new QdRMatrixSeq(msobj->sequence_size(), msobj[0]);
			
			// do first entry as 2-point estimate
			retM[0] = (msobj[1] - msobj[0])	/ hReg;
			// do second entry as 3-point estimate
			retM[1] = (msobj[2] - msobj[0])	/ h2;
			
			for(int i=2; i<msobj->sequence_size()-2; i++){
				retM[i] = (-msobj[i+2] + 8.*msobj[i+1] - 8.*msobj[i-1] + msobj[i-2]) / h12; // 5-point estimate
			}
			// do last entries as 2-point and 3-point estimate
			retM[msobj->sequence_size()-2] = (msobj[msobj->sequence_size()-1] - msobj[msobj->sequence_size()-3]) / h2;
			retM[msobj->sequence_size()-1] = (msobj[msobj->sequence_size()-1] - msobj[msobj->sequence_size()-2]) / hReg;
			
			retObj = retM;
			retObj->copy_xrefs_from(msobj);

		}
		else
		{
			QuMsgBar("Input not a recognised sequence", true);
			return;
		}
	
	}
	if( retObj.is_nil() )  {
		QuMsgBar("NULL object from derivative", true);
		return;
	}

  	// set xrefs
	
	// Units
	
	string attr_ptr = QuGetAttrText(UNITS, d0);
	string string_buf="";
	if( !attr_ptr.empty() ){
     	string_buf = " (";
     	if(tt.is_nil() ) string_buf += attr_ptr;
		else string_buf += "s"; // QSAS does time arithmetic in seconds whatever the metadata says
     	string_buf += ")^-1";
	
		attr_ptr = QuGetAttrText(UNITS, dobj);
		if( !attr_ptr.empty() ){
	
			QdString_var unitS = new QdString(attr_ptr.c_str());
			*unitS += QdString(string_buf.c_str());
			retObj->change_xref(UNITS, unitS);
		}
	} 
	
	// SI_conversion
    attr_ptr = QuSIConv_inverse(d0);

    QdRScalar_var inv_ptr = new QdRScalar(1.0); // this is irrelevant as we only need metadata
   
    if( !attr_ptr.empty() ){
		if(tt.is_nil() ) QuSetTxtAttr(SI_CONVERSION, attr_ptr.c_str(), inv_ptr);
		else QuSetTxtAttr(SI_CONVERSION, "1.0 > s^-1", inv_ptr);
		
		attr_ptr = QuSIConv_product(dobj, inv_ptr);
		if ( !attr_ptr.empty() ) {
			QuSetTxtAttr(SI_CONVERSION, attr_ptr.c_str(), retObj);
 		}
	}
	
	if(dobj->xref_exists("FIELDNAM") ){
		QdObject_var xref = dobj->get_xref("FIELDNAM");
		QdString_var sxref = QdString_var::narrow(xref);
		if(!sxref.is_nil()){
			QdString_var fName = new QdString("d ");
			*fName +=  *sxref;
			if(!tt.is_nil() ) *fName += "/ dt";
			else *fName += "/ ds";
			retObj->change_xref("FIELDNAM", fName);
		}
	}

	if(derivMethod->currentIndex() == 0) QuSetTxtAttr("Method", "Derivative (3-point estimate)", retObj);
	else QuSetTxtAttr("Method", "Derivative (5-point estimate)", retObj);

	QString newName = name_field->text();
	QwuiPutOnWL(retObj, QString("Data/")+newName);

	qApp->processEvents();
	
	this->close();

}



// -------------Regression Window class ---------------------------------

RegressionWindow::RegressionWindow() 
	: QMainWindow() 
{

	if(QuSlotDebug())
	{
		printf("RegressionWindow::RegressionWindow \n");
		fflush(stdout);
	}
   
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	slotPanel->setTitle("Linear Regression");
	
	objField = new QguWLinput(slotPanel,"objField");
	objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_SEQ, QS_ANY));
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	connect(objField, SIGNAL(edited_OK()), this, SLOT(setNewName()));
	slotLayout->addWidget(objField);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save best fit line as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	userSet = false;
	connect( name_field,  SIGNAL( edited_OK() ), this, SLOT( setUserInput() ) );
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Save ");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptOp()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	dismissBtn->setDefault(true);
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}

void RegressionWindow::setUserInput(){
  if(name_field->lineBox->hasFocus()){
	
     if(name_field->lineBox->text().isEmpty() ) userSet = false; // allow refresh from drop to object slot
	 else userSet = true; // user typed input, disable update from data slot
  }
  // else ignore
}



void RegressionWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("ArrayTraceWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   objField->reValidate();
   
}

void RegressionWindow::setNewName(){
     
 if(QuSlotDebug())
 {
	printf("ArrayTraceWindow::setNewName \n");
	fflush(stdout);
 }

   if (userSet) return;
   QString objName = QString(this->objField->textObject().toLatin1().data());
   
   if(!objName.isEmpty()){
	 QString newName = QString(objName);
       
     int ptr = newName.lastIndexOf("/");
     if (ptr >= 0) newName.remove(0, ptr+1);
	 ptr = newName.indexOf(" [");
	 if (ptr > 0) newName.truncate(ptr);
     newName += "_LinFit";
     this->name_field->setText(newName);
   }
   

}

void RegressionWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("RegressionWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptOp();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void RegressionWindow::acceptOp(){
      
	if(QuSlotDebug())
	{
		printf("ArrayTraceWindow::acceptSlice \n");
		fflush(stdout);
	}
  
	QdObject_var dobj = objField->getSlotObject();
	
	QdRScalarSeq_var fitLine = QuRegression(dobj);

	if(fitLine.is_nil()) {
		this->close();
		return;
	}
	
	// set xrefs
	
	if(dobj->xref_exists("UNITS") ){
		QdObject_var xref = dobj->get_xref("UNITS");
		fitLine->change_xref("UNITS", xref);
	}
	if(dobj->xref_exists("SI_CONVERSION") ){
		QdObject_var xref = dobj->get_xref("SI_CONVERSION");
		fitLine->change_xref("SI_CONVERSION", xref);
	}
	if(dobj->xref_exists("FIELDNAM") ){
		QdObject_var xref = dobj->get_xref("FIELDNAM");
		QdString_var sxref = QdString_var::narrow(xref);
		if(!sxref.is_nil()){
			*sxref += " linear fit";
			fitLine->change_xref("FIELDNAM", sxref);
		}
	}

	QString newName = name_field->text();
	if( !fitLine.is_nil() ) QwuiPutOnWL((QdObject_var) fitLine, QString("Data/")+newName);
	else QuMsgBar("NULL object from Linear Fit", true);

	this->close();
	return;
}




// ----------------------------------------------
//-----------------------------------------------


T1T2ToSecWindow::T1T2ToSecWindow(QWidget *parent) 
	: QMainWindow(/*parent*/) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{
	// converts DEPEND_0 timetags to scalar offset (default from ttag start time)
	// and attaches as xref (default is DEPEND_0)
	
	if(QuSlotDebug())
	{
		printf("T1T2ToSecWindow::T1T2ToSecWindow \n");
		fflush(stdout);
	}
  
  	this->setWindowTitle("Convert Time Difference to seconds Tend - Tstart");
	
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * startPanel=new QGroupBox(mainWidget);
	QHBoxLayout * startLayout=new QHBoxLayout;
	startPanel->setTitle("Start Time Object");
	
	startField = new QguWLinput(startPanel,"startField");
	startField->setToolTip("Accepts Time (can extract from interval or time tags)");
	startField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
	startField->setEnabled(true);
	startField->setMinimumWidth(char_width*55);
	startLayout->addWidget(startField);
	
	startPanel->setLayout(startLayout);
	mainLayout->addWidget(startPanel);

	
	QGroupBox * endPanel=new QGroupBox(mainWidget);
	QHBoxLayout * endLayout=new QHBoxLayout;
	endPanel->setTitle("End Time Object");
	
	endField = new QguWLinput(endPanel,"endField");
	endField->setToolTip("Accepts Time (can extract from interval or time tags)");
	endField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
	endField->setEnabled(true);
	endField->setMinimumWidth(char_width*55);
	endLayout->addWidget(endField);
	
	endPanel->setLayout(endLayout);
	mainLayout->addWidget(endPanel);
	
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	name_field->setText("difference");
	userSet = false;
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Calculate");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptOperation()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	dismissBtn->setDefault(true);
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);
	
	//Done
	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}



void T1T2ToSecWindow::setObjectStart(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("T1T2ToSecWindow::setObject \n");
	fflush(stdout);
 }
  
   startField->setText(WLctxt);
   startField->reValidate(true);
   
}

void T1T2ToSecWindow::setObjectEnd(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("T1T2ToSecWindow::setObject \n");
	fflush(stdout);
 }
  
   endField->setText(WLctxt);
   endField->reValidate(true);
   
}


void T1T2ToSecWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("T1T2ToSecWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptOperation();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void T1T2ToSecWindow::acceptOperation(){
      
	if(QuSlotDebug())
	{
		printf("T1T2ToSecWindow::acceptOperation \n");
		fflush(stdout);
	}
	
	QString name = name_field->text();
	if (name.isEmpty() ) name = "diffSec";
	    
	QdObject_var obj1 = startField->getSlotObject();
  	QdObject_var obj2 = endField->getSlotObject();
	
	if(!obj2.is_nil() && !obj1.is_nil()){

		QdTime_var t1 = QdTime_var::narrow(obj1);
		QdTime_var t2 = QdTime_var::narrow(obj2);
		
		if(!t1.is_nil() && !t2.is_nil())
		{
			QdRScalar_var val = new QdRScalar();
			*val = *t2 - *t1;
			
			QdString_var SI_Str = new QdString("1>s");
			val->change_xref(SI_CONVERSION, SI_Str);
			
			QdString_var unit_Str = new QdString("sec");
			val->change_xref(UNITS, unit_Str);

			QdString_var name_Str = new QdString(name.toLatin1().data());
			val->change_xref(FIELDNAM, name_Str);

			QwuiPutOnWL(QdObject_var(val), QString("Data/")+name);
			return;
		}
	}
	
	this->close();
	return;
}

void T1T2ToSecWindow::dismiss(){
      
	if(QuSlotDebug())
	{
		printf("T1T2ToSecWindow::dismiss \n");
		fflush(stdout);
	}

	this->close();
	return;
}

//-----------------------------------------------


TT1TT2ToSecWindow::TT1TT2ToSecWindow(QWidget *parent) 
	: QMainWindow(/*parent*/) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{
	// converts DEPEND_0 timetags to scalar offset (default from ttag start time)
	// and attaches as xref (default is DEPEND_0)
	
	if(QuSlotDebug())
	{
		printf("TT1TT2ToSecWindow::TT1TT2ToSecWindow \n");
		fflush(stdout);
	}
  
  	this->setWindowTitle("Convert Time Sequence Difference to scalar sequence in seconds (TTend - TTstart)");
	
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * startPanel=new QGroupBox(mainWidget);
	QHBoxLayout * startLayout=new QHBoxLayout;
	startPanel->setTitle("Start Time Object");
	
	startField = new QguWLinput(startPanel,"startField");
	startField->setToolTip("Accepts Time Sequence (can extract from time tags)");
	startField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_SEQ, QS_ANY));
	startField->setEnabled(true);
	startField->setMinimumWidth(char_width*55);
	startLayout->addWidget(startField);
	
	startPanel->setLayout(startLayout);
	mainLayout->addWidget(startPanel);

	
	QGroupBox * endPanel=new QGroupBox(mainWidget);
	QHBoxLayout * endLayout=new QHBoxLayout;
	endPanel->setTitle("End Time Object");
	
	endField = new QguWLinput(endPanel,"endField");
	endField->setToolTip("Accepts Time sequence(can extract from time tags)");
	endField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_SEQ, QS_ANY));
	endField->setEnabled(true);
	endField->setMinimumWidth(char_width*55);
	endLayout->addWidget(endField);
	
	endPanel->setLayout(endLayout);
	mainLayout->addWidget(endPanel);
	
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	name_field->setText("difference");
	userSet = false;
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Calculate");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptOperation()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	dismissBtn->setDefault(true);
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);
	
	//Done
	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}



void TT1TT2ToSecWindow::setObjectStart(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("TT1TT2ToSecWindow::setObject \n");
	fflush(stdout);
 }
  
   startField->setText(WLctxt);
   startField->reValidate(true);
   
}

void TT1TT2ToSecWindow::setObjectEnd(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("TT1TT2ToSecWindow::setObject \n");
	fflush(stdout);
 }
  
   endField->setText(WLctxt);
   endField->reValidate(true);
   
}


void TT1TT2ToSecWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("TT1TT2ToSecWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptOperation();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void TT1TT2ToSecWindow::acceptOperation(){
      
	if(QuSlotDebug())
	{
		printf("TT1TT2ToSecWindow::acceptOperation \n");
		fflush(stdout);
	}
	
	QString name = name_field->text();
	if (name.isEmpty() ) name = "diffSecSeq";
	    
	QdObject_var obj1 = startField->getSlotObject();
  	QdObject_var obj2 = endField->getSlotObject();
	
	if(!obj2.is_nil() && !obj1.is_nil()){

		QdTimeSeq_var t1 = QdTimeSeq_var::narrow(obj1);
		QdTimeSeq_var t2 = QdTimeSeq_var::narrow(obj2);
		
		if(!t1.is_nil() && !t2.is_nil() && t1->sequence_size() == t2->sequence_size())
		{
			QdRScalarSeq_var val = new QdRScalarSeq(t2->sequence_size());
			for(int i=0; i<t2->sequence_size(); i++){
				val[i] = t2[i] - t1[i];
			}
			
			QdString_var SI_Str = new QdString("1>s");
			val->change_xref(SI_CONVERSION, SI_Str);
			
			QdString_var unit_Str = new QdString("sec");
			val->change_xref(UNITS, unit_Str);

			QdString_var name_Str = new QdString(name.toLatin1().data());
			val->change_xref(FIELDNAM, name_Str);

			QwuiPutOnWL(QdObject_var(val), QString("Data/")+name);
			return;
		}
	}
		
	this->close();
	return;
}

void TT1TT2ToSecWindow::dismiss(){
      
	if(QuSlotDebug())
	{
		printf("TT1TT2ToSecWindow::dismiss \n");
		fflush(stdout);
	}

	this->close();
	return;
}


//-----------------------------------------------


TIToSecWindow::TIToSecWindow(QWidget *parent) 
	: QMainWindow(/*parent*/) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{
	// converts DEPEND_0 timetags to scalar offset (default from ttag start time)
	// and attaches as xref (default is DEPEND_0)
	
	if(QuSlotDebug())
	{
		printf("TIToSecWindow::TIToSecWindow \n");
		fflush(stdout);
	}
  
  	this->setWindowTitle("Convert Time Difference to seconds Tend - Tstart");
	
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	slotPanel->setTitle("Data Object");
	
	objField = new QguWLinput(slotPanel,"startField");
	objField->setToolTip("Accepts Time (can extract from interval or time tags)");
	objField->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_NOT_SEQ, QS_ANY));
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	slotLayout->addWidget(objField);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	name_field->setText("duration");
	userSet = false;
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Calculate");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptOperation()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	dismissBtn->setDefault(true);
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);
	
	//Done
	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}



void TIToSecWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("TIToSecWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   objField->reValidate(true);
   
}


void TIToSecWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("TIToSecWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptOperation();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void TIToSecWindow::acceptOperation(){
      
	if(QuSlotDebug())
	{
		printf("TIToSecWindow::acceptOperation \n");
		fflush(stdout);
	}
	
	QString name = name_field->text();
	if (name.isEmpty() ) name = "duration";
	    
	QdObject_var dobj = objField->getSlotObject();
	
	if(!dobj.is_nil() ){

		QdTimeInterval_var ti = QdTimeInterval_var::narrow(dobj);
		
		if(!ti.is_nil() )
		{
			QdRScalar_var val = new QdRScalar();
			*val = ti->duration();
			
			QdString_var SI_Str = new QdString("1>s");
			val->change_xref(SI_CONVERSION, SI_Str);
			
			QdString_var unit_Str = new QdString("sec");
			val->change_xref(UNITS, unit_Str);

			QdString_var name_Str = new QdString(name.toLatin1().data());
			val->change_xref(FIELDNAM, name_Str);

			QwuiPutOnWL(QdObject_var(val), QString("Data/")+name);
			return;
		}
	}
		
	this->close();
	return;
}

void TIToSecWindow::dismiss(){
      
	if(QuSlotDebug())
	{
		printf("TIToSecWindow::dismiss \n");
		fflush(stdout);
	}

	this->close();
	return;
}


//-----------------------------------------------


TTagToScalarWindow::TTagToScalarWindow(QWidget *parent) 
	: QMainWindow(/*parent*/) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{
	// converts DEPEND_0 timetags to scalar offset (default from ttag start time)
	// and attaches as xref (default is DEPEND_0)
	
	if(QuSlotDebug())
	{
		printf("TTagToScalarWindow::TTagToScalarWindow \n");
		fflush(stdout);
	}
  
  	this->setWindowTitle("Convert Time Tags to seconds from Start Time");
	
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	slotPanel->setTitle("Data Object");
	
	objField = new QguWLinput(slotPanel,"objField");
	objField->setAcceptable(Qcondition());
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	connect(objField, SIGNAL(edited_OK()), this, SLOT(setNewOffset()));
	slotLayout->addWidget(objField);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// Offset slot
	
	QGroupBox * offsetPanel=new QGroupBox(mainWidget);
	QHBoxLayout * offsetLayout=new QHBoxLayout;
	offsetPanel->setTitle("Start Time");
	
	offsetField = new QguWLinput(offsetPanel,"objField");
	offsetField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
	offsetField->setEnabled(true);
	offsetField->setMinimumWidth(char_width*55);
	offsetLayout->addWidget(offsetField);
	connect( offsetField,  SIGNAL( edited_OK() ), this, SLOT( setUserOffset() ) );
	
	offsetPanel->setLayout(offsetLayout);
	mainLayout->addWidget(offsetPanel);
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save metadata as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	name_field->setText(DEPEND_0);
	userSet = false;
	connect( name_field,  SIGNAL( edited_OK() ), this, SLOT( setUserName() ) );
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Set as new DEPEND_0 ");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptOperation()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	dismissBtn->setDefault(true);
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);
	
	//Done
	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}

void TTagToScalarWindow::setUserName(){

  if(name_field->lineBox->text() == QString(DEPEND_0)) saveBtn->setText("Set as new DEPEND_0 ");
  else saveBtn->setText("Save new metadata ");
  
}


void TTagToScalarWindow::setUserOffset(){
     
 if(QuSlotDebug())
 {
	printf("TTagToScalarWindow::setUserOffset \n");
	fflush(stdout);
 }

  if(offsetField->lineBox->hasFocus()){
	
     if(offsetField->lineBox->text().isEmpty() ) userSet = false; // allow refresh from drop to object slot
	 else userSet = true; // user typed input, disable update from data slot
  }

}

void TTagToScalarWindow::setNewOffset(){
     
	if(QuSlotDebug())
	{
		printf("TTagToScalarWindow::setNewOffset \n");
		fflush(stdout);
	}

	if (userSet) return;
   
	QdObject_var dobj = objField->getSlotObject();
   
	if(!dobj.is_nil()){

		QdTimeSeq_var tt = get_timetags(dobj);
		if(!tt.is_nil() && tt->sequence_size() > 0){
			QdTime_var startTime = new QdTime(tt[0]);
			offsetField->setText(startTime->getISOstring(0).c_str());
		}
	}

}


void TTagToScalarWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("TTagToScalarWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   objField->reValidate(true);
   
}


void TTagToScalarWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("TTagToScalarWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptOperation();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void TTagToScalarWindow::acceptOperation(){
      
	if(QuSlotDebug())
	{
		printf("TTagToScalarWindow::acceptOperation \n");
		fflush(stdout);
	}
  
	QdObject_var dobj = objField->getSlotObject();
  	QdObject_var offset = offsetField->getSlotObject();
	
	if(!dobj.is_nil() && !offset.is_nil()){

		QdTimeSeq_var tt = get_timetags(dobj);
		QdTime_var startTime = QdTime_var::narrow(offset);
		
		if(!tt.is_nil() && !offset.is_nil() && tt->sequence_size() > 0){
			
			QdRScalarSeq_var ss = new QdRScalarSeq(tt->sequence_size());
			for(int i=0; i<tt->sequence_size(); i++){
			 	ss[i] = tt[i] - *startTime;
			}
			QdString_var unitStr = new QdString("sec");
			ss->change_xref(UNITS, unitStr);
			QdString_var SI_Str = new QdString("1>s");
			ss->change_xref(SI_CONVERSION, SI_Str);
			QdString_var NameStr = new QdString("Sec after ");
			*NameStr += startTime->getISOstring(0);
			ss->change_xref(FIELDNAM, NameStr);
			dobj->change_xref(name_field->lineBox->text().toLatin1().data(), ss);
			 
			// clear cache to force use of new DEPEND_0		
			vector<string> sl;
			dobj->list_xref_names(sl);
			for(int i=0; i<sl.size(); i++)
			{
				if( sl[i][0] == '_') dobj->delete_xref(sl[i].c_str());
			}		

		}
	}
	
	this->close();
	return;
}

void TTagToScalarWindow::dismiss(){
      
	if(QuSlotDebug())
	{
		printf("TTagToScalarWindow::dismiss \n");
		fflush(stdout);
	}

	this->close();
	return;
}


// -------------Integral Window class ---------------------------------

IntegralWindow::IntegralWindow() 
	: QMainWindow() 
{

	if(QuSlotDebug())
	{
		printf("IntegralWindow::IntegralWindow \n");
		fflush(stdout);
	}
	this->setWindowTitle("Integral Estimator");
	
  	QMenu * help = new QMenu( this );
	this->menuBar()->addMenu(help );
	help->addAction("Integration in QSAS", this, SLOT( launchHelp() ));
	help->setTitle(" Help ");
	
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
     
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	slotPanel->setTitle("Integrate...");
	
	objField = new QguWLinput(slotPanel,"objField");
	objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ANY, QS_ANY, QS_ANY));
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	connect(objField, SIGNAL(edited_OK()), this, SLOT(setNewName()));
	slotLayout->addWidget(objField);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// Method panel
	
	QGroupBox * method_panel=new QGroupBox(mainWidget);
	QVBoxLayout * method_layout=new QVBoxLayout;
	method_panel->setTitle("Integration form");
	
	integralMethod = new QComboBox(method_panel);
	integralMethod->addItem("Definite Integral");
	integralMethod->addItem("Indefinite Intergal" );
	method_layout->addWidget( integralMethod, 0,Qt::AlignLeft );
	
	
	methodLabl = new QLabel("Total Sum (f(x)*h) over range");
	method_layout->addWidget( methodLabl, 1, Qt::AlignLeft );

	connect( integralMethod,  SIGNAL( currentIndexChanged(const QString &) ), this, SLOT( setMethod(const QString &) ) );
	
	method_panel->setLayout(method_layout);
	mainLayout->addWidget(method_panel);

	// Gap panel
	
	QGroupBox * gap_panel=new QGroupBox(mainWidget);
	QVBoxLayout *gap_layout=new QVBoxLayout;
	gap_panel->setTitle("Interpolation");
	
	gapMethod = new QComboBox(gap_panel);	
	gapMethod->addItem("Nearest Neighbour (histogram)" );
	gapMethod->addItem("Linear interp (trapezoidal)" );
	gapMethod->addItem("Ignore gaps" );
	connect( gapMethod,  SIGNAL( currentIndexChanged(const QString &) ), this, SLOT( setGapMethod(const QString &) ) );
	

	gap_layout->addWidget( gapMethod, 2, Qt::AlignLeft );
	gapMethodLabl = new QLabel("Divide gap into two bins using Nearest Neighbour value");
	gap_layout->addWidget( gapMethodLabl, 3, Qt::AlignLeft );

	
	gap_panel->setLayout(gap_layout);
	mainLayout->addWidget(gap_panel);


// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	userSet = false;
	connect( name_field,  SIGNAL( edited_OK() ), this, SLOT( setUserInput() ) );
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Do Integral ");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(doIntegral()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	dismissBtn->setDefault(true);
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}

IntegralWindow::~IntegralWindow() 
{
}
// -------------

void IntegralWindow::launchHelp() 
{
	 if(QuSlotDebug())
 	{
		printf("IntegralWindow::launchHelp %ld\n", (long)this);
		fflush(stdout);
	 }
 
  	qsasWindow->help_window->manEntry("QSAS_analysis.html","#integration");

}
// -------------

void IntegralWindow::setUserInput(){
  if(name_field->lineBox->hasFocus()){
	
     if(name_field->lineBox->text().isEmpty() ) userSet = false; // allow refresh from drop to object slot
	 else userSet = true; // user typed input, disable update from data slot
  }
  // else ignore
}

// -------------

void IntegralWindow::setMethod(const QString & text){
    
	if( text.contains("Definite") ) {
		methodLabl->setText("Total Sum (f(t)*h), over range");
	}
	else  {
		methodLabl->setText("Cumulative Sum (f(t)*h), as function of Depend_0 (time or scalar) ");
	}
	
	
}

// -------------

void IntegralWindow::setGapMethod(const QString & text){
    
	if( text.contains("Nearest") ) {
		gapMethodLabl->setText("Histogram using Nearest Neighbour value");
	}
	else if( text.contains("Linear") ) {
		gapMethodLabl->setText("Trapeziodal fit between data points ");
	}
	else {
		gapMethodLabl->setText("Ignore gaps in sums (use delta_plus/minus) - integral incomplete ");
	}
	
}


// -------------

void IntegralWindow::setNewName(){
     
	if(QuSlotDebug())
	{
		printf("IntegralWindow::setNewName \n");
		fflush(stdout);
	}
	
	// first set default gap
	QdObject_var dobj = objField->getSlotObject();

	
	// set output name default
   if (userSet) return;
   QString objName = QString(this->objField->textObject().toLatin1().data());
   
   if(!objName.isEmpty()){
		QString newName = QString(objName);
       
		int ptr = newName.lastIndexOf("/");
		if (ptr >= 0) newName.remove(0, ptr+1);
		ptr = newName.indexOf(" [");
		if (ptr > 0) newName.truncate(ptr);

		this->name_field->setText("I_"+newName);
   }

}

// -------------

void IntegralWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("IntegralWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) doIntegral();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

// -------------

void IntegralWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("IntegralWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   objField->reValidate(true);
   
}

// -------------

void IntegralWindow::doIntegral(){
      
	if(QuSlotDebug())
	{
		printf("IntegralWindow::doIntegral \n");
		fflush(stdout);
	}
	qApp->processEvents();
		
	QdObject_var dobj = objField->getSlotObject();
	
	if(dobj->sequence_size() < 2){
		QuMsgBar("Input sequence too short", true);
		return;
	}
	QdTimeSeq_var tt;
	QdRScalarSeq_var ss;
	QdObject_var d0;
	QdRScalarSeq_var x = new QdRScalarSeq(dobj->sequence_size(), 0.0);
	QdRScalarSeq_var xm = new QdRScalarSeq(dobj->sequence_size(), 0.0);
	QdRScalarSeq_var xp = new QdRScalarSeq(dobj->sequence_size(), 0.0);
	double deltaTop = 0.0;
	
	if(dobj->xref_exists(DEPEND_0) ){
		d0 = dobj->get_xref(DEPEND_0);
		if(!d0.is_nil()) {
			tt = QdTimeSeq_var::narrow(d0);
			ss = QdRScalarSeq_var::narrow(d0);
		}
	}
	if(tt.is_nil() && ss.is_nil()){
		QuMsgBar("Input sequence has no useable DEPEND_0", true);
		return;
	}
	
	bool deltasFound=false;
	if(d0->xref_exists(DELTA_PLUS) && d0->xref_exists(DELTA_MINUS)){

		// deltas exist and gaps ignored (so use bin width from deltas)
		QdObject_var dp = tt->get_xref(DELTA_PLUS);
		QdObject_var dm = tt->get_xref(DELTA_MINUS);
		QdRScalar_var dps = QdRScalar_var::narrow(dp);
		QdRScalarSeq_var dpss;
		QdRScalar_var dms = QdRScalar_var::narrow(dm);
		QdRScalarSeq_var dmss;
		
		if(!dps.is_nil()){
			dpss = new QdRScalarSeq(dobj->sequence_size(), *dps);
			dpss->copy_xrefs_from(dps);
		}
		else dpss = QdRScalarSeq_var::narrow(dp);
		
		if(!dms.is_nil()){
			dmss = new QdRScalarSeq(dobj->sequence_size(), *dms);
			dmss->copy_xrefs_from(dms);
		}
		else dmss = QdRScalarSeq_var::narrow(dm);
		
		if(!dpss.is_nil() && !dmss.is_nil()){
			deltasFound=true;
			// we now have deltas as sequences
			if(!tt.is_nil()){
				QdRScalarSeq_var dmsec = Factory::clone(dmss);
				QdRScalarSeq_var dpsec = Factory::clone(dpss);
				if(dmss->xref_exists(SI_CONVERSION) && dpss->xref_exists(SI_CONVERSION) ){
					QdObject_var attr = dmss->get_xref(SI_CONVERSION);
					QdString_var SIc = QdString_var::narrow(attr);
					if( !SIc.is_nil()) {
						double conv = DvUnitConversion(SIc->c_str(), "1.0>s");
						*dmsec *= conv;
					}
					attr = dpss->get_xref(SI_CONVERSION);
					SIc = QdString_var::narrow(attr);
					if( !SIc.is_nil()) {
						double conv = DvUnitConversion(SIc->c_str(), "1.0>s");
						*dpsec *= conv;
					}
				//	dmsec = QdRScalarSeq_var::narrow(QarConvertUnitsTo(dmss, "1.0>s")); // Windows can't get Qar
				//	dpsec = QdRScalarSeq_var::narrow(QarConvertUnitsTo(dpss, "1.0>s"));
				}
				else{
					QuMsgBar("Timetag deltas have no SI_CONVERSION, result magnitude assumed sec", true);
					
				}
				xm[0] = 0.0;
				x[0] = dmsec[0];
				xp[0] = x[0] + dpsec[0];
				for(int i=1; i<dobj->sequence_size(); i++){
					// done this way to get scalar from time
					x[i] = tt[i] - tt[i-1] + x[i-1];
					xm[i] = x[i] - dmsec[i];
					xp[i] = x[i] + dpsec[i];
				}
				deltaTop = dpsec[dobj->sequence_size()-1];
			}
			else {
				xm[0] = 0.0;
				x[0] = dmss[0];
				xp[0] = x[0] + dpss[0];
				for(int i=1; i<dobj->sequence_size(); i++){
					// done this way to get scalar from time
					x[i] = ss[i] - ss[i-1] + x[i-1];
					xm[i] = x[i] - dmss[i];
					xp[i] = x[i] + dpss[i];
				}
				deltaTop = dpss[dobj->sequence_size()-1];

			} // end if on tt vs ss
		} // end if on deltas found ok
	}

	if(gapMethod->currentIndex() != 2 || !deltasFound){
	
		// not using deltas except at ends if possible (fill gaps)
		if(!tt.is_nil()){
			// replacing x, xm and xp, so use ends before overwritten
			if(!deltasFound) x[0] = 0.0;
			// else is already incremented by delta_minus
			
			xm[0] = 0.0;
			for(int i=1; i<dobj->sequence_size(); i++){
				// done this way to get scalar from time
				xm[i] = x[i-1] + 0.5*(tt[i] - tt[i-1]);
				x[i] = x[i-1] + (double)(tt[i] - tt[i-1]) ;
				xp[i-1] = xm[i];
			}
			xp[dobj->sequence_size()-1] = x[dobj->sequence_size()-1] + deltaTop;
		}
		else {
			if(deltasFound) x[0] = xm[0];
			else  x[0] = 0.0;
			xm[0] = 0.0;
			for(int i=1; i<dobj->sequence_size(); i++){
				// done this way to get scalar from time
				xm[i] = x[i-1] + 0.5*(ss[i] - ss[i-1]);
				x[i] = x[i-1] + ss[i] - ss[i-1];
				xp[i-1] = xm[i];
			}
			xp[dobj->sequence_size()-1] = x[dobj->sequence_size()-1] + deltaTop;
	
		}
	}
	
	QdObject_var retObj;
	QdRMatrixSeq_var msobj = QdRMatrixSeq_var:: narrow(dobj);
	QdRScalarSeq_var ssobj = QdRScalarSeq_var:: narrow(dobj);
	
	// Note step lengths may not be constant, so Trapezium, Simpson's rule etc don't work.
	
	if(gapMethod->currentIndex() == 0 || gapMethod->currentIndex() == 2){
		// histogram
		// same method for ignore gaps or histogram as difference is only in xm and xp bin boundaries
		if(integralMethod->currentIndex() == 0){
	
			// definite integral
			if(!ssobj.is_nil()){
				QdRScalar_var Iobj = new QdRScalar(0.);
				for(int i=0; i<dobj->sequence_size(); i++){
					*Iobj += ssobj[i]*(xp[i] - xm[i]);
				}
				retObj = Iobj;
			}
			if(!msobj.is_nil()){
				QdRMatrix_var Iobj = new QdRMatrix(msobj[0].dims(), 0.0);
				for(int i=0; i<dobj->sequence_size(); i++){
					*Iobj += msobj[i]*(xp[i] - xm[i]);
				}
				retObj = Iobj;
			}
			
		}
		else{
			// indefinite integral
			if(!ssobj.is_nil()){
				QdRScalarSeq_var Iobj = new QdRScalarSeq(dobj->sequence_size(), 0.0);
				Iobj[0] = (xp[0] - xm[0]) * ssobj[0];
				for(int i=1; i<dobj->sequence_size(); i++){
					Iobj[i] = Iobj[i-1] + ssobj[i]*(xp[i] - xm[i]);
				}
				retObj = Iobj;
			}
			if(!msobj.is_nil()){
				QdRMatrixSeq_var Iobj = new QdRMatrixSeq(dobj->sequence_size(), msobj[0].dims());
				Iobj[0] = msobj[0]*(xp[0] - xm[0]);
				for(int i=1; i<dobj->sequence_size(); i++){
					Iobj[i] = Iobj[i-1] + msobj[i]*(xp[i] - xm[i]);
				}
				retObj = Iobj;
			}
		}
	}
	else if(gapMethod->currentIndex() == 1){
		// linear
	
		if(integralMethod->currentIndex() == 0){
	
			// definite integral
			if(!ssobj.is_nil()){
				QdRScalar_var Iobj = new QdRScalar(0.);
				double h = xp[0] - xm[0];
				*Iobj = (double)ssobj[0] * h;
				for(int i=1; i<dobj->sequence_size()-1; i++){
					// does gap filling linearly
					double h = xp[i] - xm[i];
					double hm = 0.5 * h * (xm[i] - x[i]) / (x[i-1] - x[i]);
					double hp = 0.5 * h * (xp[i] - x[i]) / (x[i+1] - x[i]) ;
					h = h - hp - hm;
					*Iobj += ssobj[i]*h + ssobj[i+1]*hp + ssobj[i-1]*hm  ;
				}
				h = xp[dobj->sequence_size()-1] - xm[dobj->sequence_size()-1];
				*Iobj = (double) ssobj[dobj->sequence_size()-1] * h;

				retObj = Iobj;
			}
			if(!msobj.is_nil()){
				QdRMatrix_var Iobj = new QdRMatrix(msobj[0].dims(), 0.0);
				*Iobj = (xp[0] - xm[0]) * msobj[0];
				for(int i=1; i<dobj->sequence_size()-1; i++){
					// does gap filling linearly
					double h = xp[i] - xm[i];
					double hm = 0.5 * h * (xm[i] - x[i]) / (x[i-1] - x[i]);
					double hp = 0.5 * h * (xp[i] - x[i]) / (x[i+1] - x[i]) ;
					h = h - hp - hm;
					*Iobj += msobj[i]*h + msobj[i+1]*hp + msobj[i-1]*hm  ;
				}
				*Iobj = (xp[dobj->sequence_size()-1] - xm[dobj->sequence_size()-1]) * msobj[dobj->sequence_size()-1];

				retObj = Iobj;
			}
			
		}
		else{
			// indefinite integral
			if(!ssobj.is_nil()){
				QdRScalarSeq_var Iobj = new QdRScalarSeq(dobj->sequence_size(), 0.0);
				double h = xp[0] - xm[0];
				Iobj[0] = ssobj[0] * h;
				for(int i=1; i<dobj->sequence_size()-1; i++){
					double h = xp[i] - xm[i];
					double hm = 0.5 * h * (xm[i] - x[i]) / (x[i-1] - x[i]);
					double hp = 0.5 * h * (xp[i] - x[i]) / (x[i+1] - x[i]) ;
					h = h - hp - hm;
					Iobj[i] = Iobj[i-1] + ssobj[i]*h + ssobj[i+1]*hp + ssobj[i-1]*hm  ;
				}
				Iobj[dobj->sequence_size()-1] = Iobj[dobj->sequence_size()-2] + (xp[dobj->sequence_size()-1] - xm[dobj->sequence_size()-1]) * ssobj[dobj->sequence_size()-1];
				retObj = Iobj;
			}
			if(!msobj.is_nil()){
				QdRMatrixSeq_var Iobj = new QdRMatrixSeq(dobj->sequence_size(), msobj[0].dims());
				Iobj[0] = (xp[0] - xm[0]) * msobj[0];
				for(int i=1; i<dobj->sequence_size()-1; i++){
					double h = xp[i] - xm[i];
					double hm = 0.5 * h * (xm[i] - x[i]) / (x[i-1] - x[i]);
					double hp = 0.5 * h * (xp[i] - x[i]) / (x[i+1] - x[i]) ;
					h = h - hp - hm;
					Iobj[i] = Iobj[i-1] + msobj[i]*h + msobj[i+1]*hp + msobj[i-1]*hm  ;
				}
				Iobj[dobj->sequence_size()-1] = Iobj[dobj->sequence_size()-2] + (xp[dobj->sequence_size()-1] - xm[dobj->sequence_size()-1]) * msobj[dobj->sequence_size()-1];
				retObj = Iobj;
			}
		}

	}

	if( retObj.is_nil() ) {
		QuMsgBar("NULL object from integral", true);
		return;
	}


  	// set xrefs
	
	retObj->copy_xrefs_from(dobj);
	
	if(integralMethod->currentIndex() == 0) retObj->delete_xref(DEPEND_0);
	
	// Units
	
	string attr_ptr = QuGetAttrText(UNITS, d0);
	string string_buf=" ";
	if( !attr_ptr.empty() ){
     	if(tt.is_nil() ) string_buf += attr_ptr;
		else string_buf += "s"; // QSAS does time arithmetic in seconds whatever the metadata says
	
		attr_ptr = QuGetAttrText(UNITS, dobj);
		if( !attr_ptr.empty() ){
	
			QdString_var unitS = new QdString(attr_ptr.c_str());
			*unitS += QdString(string_buf.c_str());
			retObj->change_xref(UNITS, unitS);
		}
	} 
	
	// SI_conversion
    attr_ptr = QuGetAttrText(SI_CONVERSION, d0);
    QdRScalar_var dummy_ptr = new QdRScalar(0.0); // this is irrelevant as we only need metadata
   
    if( !attr_ptr.empty() ){
		if(tt.is_nil() ) QuSetTxtAttr(SI_CONVERSION, attr_ptr.c_str(), dummy_ptr);
		else QuSetTxtAttr(SI_CONVERSION, "1.0 > s", dummy_ptr);// QSAS does time arithmetic in seconds
		
		attr_ptr = QuSIConv_product(dobj, dummy_ptr);
		if ( !attr_ptr.empty() ) {
			QuSetTxtAttr(SI_CONVERSION, attr_ptr.c_str(), retObj);
 		}
	}
	
	if(dobj->xref_exists("FIELDNAM") ){
		QdObject_var xref = dobj->get_xref("FIELDNAM");
		QdString_var sxref = QdString_var::narrow(xref);
		if(!sxref.is_nil()){
			QdString_var fName = new QdString("Integral(");
			*fName +=  *sxref;
			if(!tt.is_nil() ) *fName += ")dt";
			else *fName += ") ds";
			retObj->change_xref("FIELDNAM", fName);
		}
	}

	if(gapMethod->currentIndex() == 0) QuSetTxtAttr("Method", "Histogram sum (gaps filled)", retObj);
	else if(gapMethod->currentIndex() == 1) QuSetTxtAttr("Method", "Linear fit (gaps filled)", retObj);
	else QuSetTxtAttr("Method", "Histogram sum (gaps excluded)", retObj);

	QString newName = name_field->text();
	QwuiPutOnWL(retObj, QString("Data/")+newName);

	qApp->processEvents();
	
	this->close();

}

// Class MakeVectorWindow
//------------------------

MakeVectorWindow::MakeVectorWindow(QWidget *parent) 
	: QMainWindow(/*parent*/NULL) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{

	if(QuSlotDebug())
	{
		printf("MakeVectorWindow::MakeVectorWindow \n");
		fflush(stdout);
	}
  
	this->setPalette(qApp->palette());
	this->setWindowTitle("Add vector metadata to 3 element Data Array ");

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
		
	// Status bar
	
	msgBar = statusBar();
	
	// Main widget
	
	QWidget * mainWidget=new QWidget(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Slot widget
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	
	slotPanel->setTitle(" Data Array Object ");
	
	objField = new QguWLinput(slotPanel,"objField");
	objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_ANY, QS_ANY));
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	connect(objField, SIGNAL(edited_OK()), this, SLOT(testArray()));
	slotLayout->addWidget(objField);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// Frame panel
	
	QGroupBox * frame_panel=new QGroupBox(mainWidget);
	QHBoxLayout * frame_layout=new QHBoxLayout;
	
	QLabel *frame_label = new QLabel("Frame    ", frame_panel);
	frame_layout->addWidget(frame_label);
	
	frame_field = new QguWLinput(frame_panel, "frame_field");
	frame_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	frame_field->setEnabled(true);
	frame_field->setMinimumWidth(char_width*25);
	frame_field->setText("gse");
	frame_layout->addWidget(frame_field);
	
	frame_panel->setLayout(frame_layout);
	mainLayout->addWidget(frame_panel);
	
	// Representation panel
	
	QGroupBox * rep_panel=new QGroupBox(mainWidget);
	QHBoxLayout * rep_layout=new QHBoxLayout;
	
	QLabel *rep_label = new QLabel("Representation ", rep_panel);
	rep_layout->addWidget(rep_label);
	
	rep_field = new QComboBox(rep_panel);
	rep_field->setEnabled(true);
	rep_field->addItem("xyz");
	rep_field->addItem("rlp (rad)");
	rep_field->addItem("rlp (deg)");
	rep_field->addItem("rtp (rad)");
	rep_field->addItem("rtp (deg)");
	rep_layout->addWidget(rep_field);
	
	rep_panel->setLayout(rep_layout);
	mainLayout->addWidget(rep_panel);
	
	// Buttons layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	doBtn = new QPushButton( "Convert into Vector");
	doBtn->setMinimumSize(doBtn->sizeHint());
	connect( doBtn, SIGNAL(clicked()), this, SLOT(makeVector()) );
	doBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(doBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	dismissBtn->setDefault(true);
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);
	
	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
}


void MakeVectorWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("MakeVectorWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   objField->reValidate();

}


void MakeVectorWindow::testArray(){
    
 if(QuSlotDebug())
 {
	printf("MakeVectorWindow::setObject \n");
	fflush(stdout);
 }
  
   // if input not 3 comp array disable doBtn based on component subset
   QdObject_var dobj = QuGetObjByName(objField->text());
   QdRMatrix_var rmatrix=QdRMatrix_var::narrow(dobj);
   if(rmatrix.is_nil())
   {
     QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(dobj);
	 if(rmatrixseq.is_nil()){
	   doBtn->setEnabled(false);
	   return; // not a matrix object, cannot convert
	 }
   }
   doBtn->setEnabled(true);
}


void MakeVectorWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("MakeVectorWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( doBtn->isDefault()  ) {
		doBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   doBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(doBtn->isDefault()) makeVector();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void MakeVectorWindow::makeVector(){
      
 if(QuSlotDebug())
 {
	printf("MakeVectorWindow::makeVector \n");
	fflush(stdout);
 }

 
  QString objString = objField->text();
  
  QdObject_var newObject = QuGetObjByNameAndComp(objString);
  
  // attach xrefs here
  QString repTxt = QString(rep_field->currentText().toLatin1().data());

  if( repTxt.contains("rlp", Qt::CaseInsensitive) > 0){
	   QString SI_conv = QString(QuGetAttrText("SI_CONVERSION", newObject).c_str());
       if( repTxt.contains("deg", Qt::CaseInsensitive) > 0){
	     int ptr = SI_conv.indexOf(",");
		 if(ptr > -1) SI_conv.truncate(ptr);
		 SI_conv += ", deg, deg";
       }
	   else{
	     int ptr = SI_conv.indexOf(",");
		 if(ptr > -1) SI_conv.truncate(ptr);
		 SI_conv += ", rad, rad";	   
	   }
	   QuSetTxtAttr("SI_CONVERSION", SI_conv.toLatin1().data(),  newObject);
       QuSetFrame( newObject, "vector", frame_field->text().toLatin1().data(), "rlp");
  }
  else if(repTxt.contains("rtp",Qt::CaseInsensitive) > 0){
	   QString SI_conv = QString(QuGetAttrText("SI_CONVERSION", newObject).c_str());
       if( repTxt.contains("deg",Qt::CaseInsensitive) > 0){
	     int ptr = SI_conv.indexOf(",");
		 if(ptr > -1) SI_conv.truncate(ptr);
		 SI_conv += ", deg, deg";
       }
	   else{
	     int ptr = SI_conv.indexOf(",");
		 if(ptr > -1) SI_conv.truncate(ptr);
		 SI_conv += ", rad, rad";	   
	   }  
	   QuSetTxtAttr("SI_CONVERSION", SI_conv.toLatin1().data(),  newObject);
       QuSetFrame( newObject, "vector", frame_field->text().toLatin1().data(), "rtp");
  }

  else{
    QuSetFrame( newObject, "vector", frame_field->text().toLatin1().data(), "xyz");
  }

  if( !QuIsThreeVector(newObject) ) {
    QuMsgBar("VECTOR FAILED: Input array needs 3 components", true);
	return;
  }
  
  if( !(newObject.is_nil() ) ) QwuiPutOnWL(newObject, objString.toLatin1().data(), true);
  else QuMsgBar("NULL object after conversion to vector", true);

  this->close();
  return;
}

// -----------------------------

// ----------------------------------------------

DVOS_TestWindow::DVOS_TestWindow(QWidget *parent) 
	: QMainWindow(0) 
{

	if(QuSlotDebug())
	{
		printf("DVOS_TestWindow::DVOS_TestWindow \n");
		fflush(stdout);
	}
    
	this->setWindowTitle("DVOS Test");

	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	slotPanel->setTitle("Input");
	
	objField = new QguWLinput(slotPanel,"objField 1");
	objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ANY, QS_ANY, QS_ANY));
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	slotLayout->addWidget(objField);
	
	objField2 = new QguWLinput(slotPanel,"objField 2");
	objField2->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ANY, QS_ANY, QS_ANY));
	objField2->setEnabled(true);
	objField2->setMinimumWidth(char_width*55);
	slotLayout->addWidget(objField2);
	

	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	

	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Test ");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptSlice()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	dismissBtn->setDefault(true);
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}

void DVOS_TestWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("DVOS_TestWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);

}



void DVOS_TestWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("ArrayTraceWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptSlice();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void DVOS_TestWindow::acceptSlice(){
      
	if(QuSlotDebug())
	{
		printf("DVOS_TestWindow::acceptSlice \n");
		fflush(stdout);
	}
  
	QdObject_var obj = objField->getSlotObject();
	QdObject_var obj2 = objField2->getSlotObject();
	
	QuNow("1st QDOS to DVOS");
	
	DvObject_var dobj = toDVOS(obj);
	QuNow("1st QDOS to DVOS done");
	
	QuNow("2nd QDOS to DVOS");fflush(stdout);
	DvObject_var dobj2 = toDVOS(obj2);
	QuNow("2nd QDOS to DVOS done");
	
	
	QuNow("start minusStart ");fflush(stdout);

	dobj = dobj->minusStart();
	
	QuNow("end minusStart");

	dobj2 = dobj2->minusStart();
	
	QuNow("end minusStart 2");

	QdObject_var retObj = toQDOS(dobj);
	QdObject_var retObj2 = toQDOS(dobj2);

	QuNow("DVOS to QDOS done");

	if( !retObj.is_nil() ) QwuiPutOnWL(retObj, QString("Data/DVOS"));
	else QuMsgBar("NULL object from DVOS test", true);

	if( !retObj2.is_nil() ) QwuiPutOnWL(retObj2, QString("Data/DVOS2"));
	else QuMsgBar("NULL object from DVOS test", true);


	this->close();
	return;
}


