/* calcBShock.cc Jeremy Mitchell 05/08/2009
Uses qbowshock (sjs) routines 
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "qbowshock.h"

#include "qplug_if.h"
#include "qdutil.h"
#include "Qdos.h"
#include "qar.h"
#include "Xrefs.h"

extern "C" QplugReturnStatus calcBShock(QplugArgList *);

using namespace QSAS;

QplugReturnStatus calcBShock(QplugArgList *call_list) {
  QdObject_var sw_vel_in = (* call_list)[0];
  QdObject_var model_in = (* call_list)[1];
  QdObject_var scale_in = (* call_list)[2];
  QdObject_var pos_in = (* call_list)[3];
  QdObject_var plane_in = (* call_list)[4];
  
  QdRScalar_var sw_vel_qd = QdRScalar_var::narrow(sw_vel_in);
  if (sw_vel_qd.is_nil()) {
    QplugAppendTextDisplay("calcBShock Error: Unable to unpack SW vel\n");
    return QPLUG_FAILURE;
  }
  float vel_sw = (float)(* sw_vel_qd);
  if (vel_sw < 0) {
    QplugAppendTextDisplay("calcBShock Error: SW vel < 0\n");
    return QPLUG_FAILURE;
  }
  
  QdRScalar_var model_qd = QdRScalar_var::narrow(model_in);
  if (model_qd.is_nil()) {
    QplugAppendTextDisplay("calcBShock Error: Unable to unpack model\n");
    return QPLUG_FAILURE;
  }
  int model = (int)(* model_qd);
  if ((model < 0) || (model > 13)) {
    QplugAppendTextDisplay("calcBShock Error: Must have 0 <= model <= 13\n");
    return QPLUG_FAILURE;
  }
  
  QdRScalar_var scale_qd = QdRScalar_var::narrow(scale_in);
  if (scale_qd.is_nil()) {
    QplugAppendTextDisplay("calcBShock Error: Unable to unpack scale\n");
    return QPLUG_FAILURE;
  }
  double scale = (double)(* scale_qd);
  
  bool useOrigin = false;
  
  QdRScalar_var pos_scl_qd = QdRScalar_var::narrow(pos_in);
  QdRMatrix_var pos_qd = QdRMatrix_var::narrow(pos_in);
  if (!pos_scl_qd.is_nil()) {
    if ( (int)(*pos_scl_qd) != 0 ) {
      QplugAppendTextDisplay("calcBShock Error: position must be either 0 or a vector\n");
      return QPLUG_FAILURE;
    }
    
    useOrigin = true;
  } else if (pos_qd.is_nil()) {
    QplugAppendTextDisplay("calcBShock Error: position must be either 0 or a vector\n");
    return QPLUG_FAILURE;
  }
  
  QdRScalar_var plane_qd = QdRScalar_var::narrow(plane_in);
  if (plane_qd.is_nil()) {
    QplugAppendTextDisplay("calcBShock Error: Unable to unpack plane\n");
    return QPLUG_FAILURE;
  }
  float plane = (float)(* plane_qd);
  if ((plane < 0.) || (plane > 360.)) {
    QplugAppendTextDisplay("calcBShock Error: Must have 0 <= plane <= 360\n");
    return QPLUG_FAILURE;
  }
  
  /************************ Units ****************************/
   
  // convert plane to radians, unless already done
  string units = QuGetAttrText(UNITS, plane_in);
  if (QuToUpper(units).find("RAD") == string::npos)
    plane /= 57.295779506;
  else QplugAppendTextDisplay("Plane in radians\n");
    
  /****************************************************/
  
  int nmax = 20;
  
  QdRMatrixSeq_var shock = new QdRMatrixSeq(nmax, Dim(3));
  QdRScalarSeq_var shock_x = new QdRScalarSeq(nmax);
  
  float min = 1.e30;
  float max = -1.e30;
  
  for (int i=0; i < nmax; i++) {
    float alpha = ( 1.5 * 3.14159 * (i+1) )/ (nmax + 1) - 0.25 * 3.14159;
    float rin[3], pos0[3];
    if (useOrigin) {
      pos0[0] = 0.0;
      pos0[1] = 0.0;
      pos0[2] = 0.0;
    } else {
      pos0[0] = (* pos_qd)[0];
      pos0[1] = (* pos_qd)[1];
      pos0[2] = (* pos_qd)[2];
    }
    
    rin[0] = -sin(alpha);
    rin[1] = -cos(alpha)*cos(plane);
    rin[2] = -cos(alpha)*sin(plane);
    
    QBSMP_models model_arg = (QBSMP_models) model;
    
    float dist, r_out[3];
    
    QBSPosDistanceAlongVec(pos0,vel_sw,model_arg,scale,rin,&dist,r_out);
    
    if (r_out[0] < min) min = r_out[0]; if (r_out[1] < min) min = r_out[1]; if (r_out[2] < min) min = r_out[2];
    if (r_out[0] > max) max = r_out[0]; if (r_out[1] > max) max = r_out[1]; if (r_out[2] > max) max = r_out[2];
    
    shock[i][0] = r_out[0];
    shock[i][1] = r_out[1];
    shock[i][2] = r_out[2];
    shock_x[i] = shock[i][0];
  }
  
  /********************** Set Attributes *************************/
  
  QdObject_var shock_out = (QdObject_var)shock;
  QdObject_var shock_x_out = (QdObject_var)shock_x;
  
  /*QuSetTxtAttr("TENSOR_FRAME", "gse", shock_x_out);
  QuSetTxtAttr(SI_CONVERSION, "1>m", shock_x_out);
  if (model <= 5)  {
    QuSetTxtAttr(LABLAXIS, "Bowshock", shock_x_out);
    QuSetTxtAttr(FIELDNAM, "Bowshock", shock_x_out);
  } else {
    QuSetTxtAttr(LABLAXIS, "Magnetopause", shock_x_out);
    QuSetTxtAttr(FIELDNAM, "Magnetopause", shock_x_out);
  }*/
  
  QuSetTxtAttr(FRAME, "vector>gse_xyz", shock_out);
  QuSetTxtAttr("TENSOR_FRAME", "gse", shock_out);
  QuSetTxtAttr(SI_CONVERSION, "6.3172e6>m", shock_out);
  QuSetTxtAttr(UNITS, "Re", shock_out);
  QdRScalar_var torder = new QdRScalar(1);
  shock_out->set_xref(TENSOR_ORDER, (QdObject_var) torder);
  if (model <= 5)  {
    QuSetTxtAttr(LABLAXIS, "Bowshock", shock_out);
    QuSetTxtAttr(FIELDNAM, "Bowshock", shock_out);
  } else {
    QuSetTxtAttr(LABLAXIS, "Magnetopause", shock_out);
    QuSetTxtAttr(FIELDNAM, "Magnetopause", shock_out);
  }
  QdSMatrix_var rep = new QdSMatrix( Dim(3) );
  (* rep)[0] = "x"; (* rep)[1] = "y"; (* rep)[2] = "z";
  shock_out->set_xref(REPRESENTATION_1, (QdObject_var) rep);
  shock_out->set_xref(LABEL_1, (QdObject_var) rep);
  
  /*QuSetTxtAttr(SCALETYP,  "lin", shock_out);
  
  shock_x_out->copy_xrefs_from(shock_out);
  QuSetTxtAttr(FRAME, "scalar>na", shock_x_out);
  shock_out->set_xref("DEPEND_0", shock_x_out);*/
 
  call_list->push_back( shock_out );
  
  return QPLUG_SUCCESS;
  
}
