/**********************************************************************
* SCCS id: @(#)CorCoefPlug_utils.c	1.1 14:32:45 12/17/98
************************************************************************
* PROJECT:     Cluster UK CDHF
* COMPONENT:   Correlation Coefficient Plugin 
* MODULE:      CorCoefPlug_utils.c
* LANGUAGE:	   C
* FUNCTION:
* PURPOSE:     C utilities for CorCoefPlugin 
* DESCRIPTION: Test Timetags are Joined, Test Regular Timetags and compute dt
* ARGUMENTS:
* RETURN:
* FUNCTIONS_USED: 
* ALGORITHM: 
* HISTORY:	   Steve Schwartz  Dec 98
***********************************************************************/

#include "CorCoefPlug_utils.h"

/***********************************************************************
*********************************************** Test_join
***********************************************************************/

CCutilsSuccessorFailure Test_join (double *t1, int n1, 
                            double *t2, int n2, 
                            double tol){

/* test 2 time series have same timetags to within tol */
/* all times in ms  */

int    i;

tol=fabs(tol);
 
if (n1!=n2) return CCUTILSFAILURE;
if (n1<=0) return CCUTILSFAILURE;

for(i=0; i<n1; i++){
  if( fabs(t1[i] - t2[i] ) > tol ) return CCUTILSFAILURE;
}
return CCUTILSSUCCESS;
}     // end Test_join



/***********************************************************************
***********************************************      Test_regular
***********************************************************************/

CCutilsSuccessorFailure Test_regular (double *t, int n, double *halfdtp){

/* crude test(s) for regular timetags. Tag half-spacing returned in halfdt
   All times in ms   */


double  dt;
double tol;
CCutilsSuccessorFailure  sof=CCUTILSSUCCESS;


if (n<2) return CCUTILSFAILURE;                 /* need at least 2 pts */

dt = (fabs(t[n-1] - t[0]))/(n-1) ;            /* take avg dt over all tags */
tol =  (dt > 100) ? 10 : 0.1*dt;  /* use min of 10ms or 10%dt */

while (sof==CCUTILSSUCCESS && n > 1){
	sof = (fabs(t[n-1]-t[n-2])-dt) <= tol ? 
			CCUTILSSUCCESS : CCUTILSFAILURE ;
	n--;
}

*halfdtp = dt/2.0;
return sof;
}

