/****************************************************************************
							LICENSE

  Copyright (C) 2010 Eugen Sorbalo

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  A copy of the GNU General Public License is available on the internet at:

  http://www.gnu.org/copyleft/gpl.html

  or you can write to:

  The Free Software Foundation, Inc.
  675 Mass Ave
  Cambridge, MA 02139, USA
****************************************************************************/


/**
 * \file ClusterCommons.cpp
 * \author: eugenatjacobs
 * 
 * \brief 
 * 
 */

#include "qplug_if.h"
#include "qdutil.h"
#include "qar.h"
#include "Qdos.h"
#include "ClusterCommons.h"

namespace cluster {

using namespace std;

// --------------------------------------------------------

int strcmp_ignorecase(const char* str1, const char* str2)
{
	size_t n = strlen(str1)+1;
	char *str1_lower = (char*)malloc(n);
	for (size_t i = 0; i < n; ++i) str1_lower[i] = static_cast<char>(tolower(str1[i]));
	n = strlen(str2)+1;
	char *str2_lower = (char*)malloc(n);
	for (size_t i = 0; i < n; ++i) str2_lower[i] = static_cast<char>(tolower(str2[i]));
	int result = strcmp(str1_lower, str2_lower);
	free(str1_lower);
	free(str2_lower);
	return result;
}


// ------------  class Status  ---------------------------

Status::Status()
{
	message_ = "";
}

Status::~Status()
{
}

string Status::message() const
{
	string result = "NablaEstimation: ";
	result += message_;
	return result;
}

void Status::PrintMessage(bool to_user, const char fmt[], ...)
{
    va_list args;
    char buffer[2048];
    va_start(args, fmt);
    vsprintf(buffer, fmt, args);
    va_end(args);
    if (to_user)
    	QplugAppendTextDisplay(buffer);
    printf("%s", buffer);
    fflush(stdout);
}

// ---------  class ClusterAnalysis  ------------------------

ClusterAnalysis::ClusterAnalysis() : deltat_(0)
	, nr_satellites_(0)
	, nr_coordinates_(0)
	, nr_measurements_(0)
	, use_dopler_correction_(false)
	, do_detrend_(false)
{
	field_TS_ = NULL;
	position_TS_ = NULL;
	grad_magn_TS_ = NULL;
	grad_TS_ = NULL;
	curl_TS_ = NULL;
	div_TS_ = NULL;
	vol_TS_ = NULL;
	bary_vector_TS_ = NULL;
	bary_scalar_TS_ = NULL;
	normal_TS_ = NULL;
	curl_TS_MVA_ = NULL;
	bary_TS_MVA_ = NULL;
	velo_TS_MVA_ = NULL;
	rotation_matrix_ = NULL;

	csd_parameters_.nr_points_overlap_between_subintervals = 0;
	csd_parameters_.nr_points_subinterval = 0;
	csd_parameters_.window_type = TCrossSpectralDensityParameters::kHanning;
}

ClusterAnalysis::~ClusterAnalysis()
{
}

}  //namespace cluster
