/**
 * \file MyMatrixEig.h
 * 
 * The file was originally imported from QSAS and then the class MatrixEig
 *	 	was slightly modified so that it can find eigenvalues of complex matrices as well.
 *	 	A small bug was also corrected.
 * 
 */

//
//  MyMatrixEig.h
//
//  Qdos2 - Class for calculating eigenvalues/vectors of a symmetric numeric
//  matrix
//
//  Janet Barnes
//  Sept 1999
//

#ifndef QDOS_MYEIG_H
#define QDOS_MYEIG_H

#include <algorithm>
#include "Matrix.h"

using namespace std;

namespace QDOSS {
	void eigen(double * a, double * v, int n);
	void eigen(std::vector<std::complex<double> > &array, std::vector<double> &eigvalues, unsigned int size);
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ class MyMatrixEig
// Class for calculating eigenvalues/egigenvectors for a rank 2 square
// array
// -----------------------------------------------------------------------------
	class MyMatrixEig
	{
		protected:
			size_t _n;                    // no. rows/cols
			QDOS::ScalarSeq<double> _eigvals;   // _n elt sequence of _rank 0 matrices
			QDOS::MatrixSeq<double> _eigvecs;   // _n elt sequence of rank 1 matrices

		public:
     // --- Constructors
     // Empty constructor
			MyMatrixEig(){;}
     // Construct eigenvalues/eigenvectors from Matrix
			template<class T>
					MyMatrixEig(const QDOS::Matrix<T>& m):_n(m.dims()[0]),\
					_eigvecs(QDOS::MatrixSeq<double>(m.dims()[0])),\
							_eigvals(QDOS::ScalarSeq<double>(m.dims()[0]))
							{
								try
								{
									int res;
             // verify that matrix argument is rank 2
									if (m.rank()!=2)
										throw QDOS::MatrixRankError();
             //  verify that matrix is symmetric
									if (!m.is_symmetric())
									{
										throw QDOS::MatrixDomainError(QDOS::ExceptionMsg()<<\
										"Matrix not symmetric");
									}
									char jobz='V'; // Compute eigenvalues AND eigenvectors
									char uplo='L';
									long int n=_n;
									QDOS::Matrix<double> eigvecs(QDOS::Dim(_n,_n,QDOS::ROW_MAJOR_ORDER));
									eigvecs.copy(m);
									long int lda=_n;
									long int info;
									QDOS::Matrix<double> eigvals(QDOS::Dim(_n,QDOS::ROW_MAJOR_ORDER));
             // call CLAPACK routine
#ifdef DIAG
									cout << "calling CLAPACK eigenvalue/eigenvector routine" << endl;
#endif
									eigen(eigvecs.ptr(), eigvals.ptr(), n);
             // convert to MatrixSeq
									for(int i=0;i<n;i++)
									{
										_eigvecs[i]=eigvecs.col(i);
										_eigvals[i]=eigvals.row(i);
									}

								}
								catch(QDOS::Exception& e)
								{
									e.msg() << "[MyMatrixEig<T>]";
									throw e;
								}
							}

     // --- Misc functions
							QDOS::ScalarSeq<double>& eigenvalues()
							{
								return _eigvals;
							}

							QDOS::MatrixSeq<double>& eigenvectors()
							{
								return _eigvecs;
							}

	};

}  // end namespace QDOS
#endif
