/* makeArrow.cc Jeremy Mitchell 06/08/2009 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "qplug_if.h"
#include "qdutil.h"
#include "Qdos.h"
#include "qar.h"
#include "Xrefs.h"

extern "C" QplugReturnStatus makeArrow(QplugArgList *);

using namespace QSAS;

QplugReturnStatus makeArrow(QplugArgList *call_list) {
  // Unpack input data
  
  QdObject_var pos1_in   = (* call_list)[0];
  QdObject_var deltax_in = (* call_list)[1];
  QdObject_var scale_in  = (* call_list)[2];
  QdObject_var head_in   = (* call_list)[3];
  QdObject_var hScale_in   = (* call_list)[4];
  
  //*********************************************************
  
  //***************
  // Test Inputs:
  //***************
  
  // Narrow type to a matrix
  QdRMatrix_var pos1_qd = QdRMatrix_var::narrow(pos1_in);
  if (pos1_qd.is_nil()) {
    QdRMatrixSeq_var pos1_tmp = QdRMatrixSeq_var::narrow(pos1_in);
    if (pos1_tmp.is_nil()) {
      QplugAppendTextDisplay("makeArrow error: Position 1 is not a Matrix\n");
      return QPLUG_FAILURE;
    } else {
      pos1_qd = new QdRMatrix(Dim(3));
      (* pos1_qd)[0] = pos1_tmp[0][0];
      (* pos1_qd)[1] = pos1_tmp[0][1];
      (* pos1_qd)[2] = pos1_tmp[0][2];
    }
  }
  if (!pos1_in->xref_exists(FRAME)) {
    QplugAppendTextDisplay("makeArrow error: Position 1 is not a Vector\n");
    return QPLUG_FAILURE;
  }
  
  /*if (deltax_in->xref_exists(DEPEND_0)) {
    deltax_in->delete_xref(DEPEND_0);
    QplugAppendTextDisplay("Deleting deltax DEPEND_0\n");
  }*/
  QdRMatrix_var deltax_qd = QdRMatrix_var::narrow(deltax_in);
  if (deltax_qd.is_nil()) {
    QdRMatrixSeq_var deltax_tmp = QdRMatrixSeq_var::narrow(deltax_in);
    if (deltax_tmp.is_nil()) {
      QplugAppendTextDisplay("makeArrow error: delta x is not a Matrix\n");
      return QPLUG_FAILURE;
    } else {
      deltax_qd = new QdRMatrix(Dim(3));
      (* deltax_qd)[0] = deltax_tmp[0][0];
      (* deltax_qd)[1] = deltax_tmp[0][1];
      (* deltax_qd)[2] = deltax_tmp[0][2];
    }
  }
  if (!deltax_in->xref_exists(FRAME)) {
    QplugAppendTextDisplay("makeArrow error:delta x is not a Vector\n");
    return QPLUG_FAILURE;
  }
  
  QdRScalar_var scale_qd = QdRScalar_var::narrow(scale_in);
  if (scale_qd.is_nil()) {
    QplugAppendTextDisplay("makeArrow error: Scale is not a Scalar\n");
    return QPLUG_FAILURE;
  }
  
  QdRScalar_var head_qd = QdRScalar_var::narrow(head_in);
  if (head_qd.is_nil()) {
    QplugAppendTextDisplay("makeArrow error: Draw Head is not a Scalar\n");
    return QPLUG_FAILURE;
  }
  int head = (int)(* head_qd);
  if ((head < 0) || (head > 3)) {
    QplugAppendTextDisplay("makeArrow error: Draw Head must be between 0 and 3\n");
    return QPLUG_FAILURE;
  }
  
  QdRScalar_var hScale_qd = QdRScalar_var::narrow(hScale_in);
  if (hScale_qd.is_nil()) {
    QplugAppendTextDisplay("makeArrow error: Width is not a Scalar\n");
    return QPLUG_FAILURE;
  }
  float hScale = (float)(* hScale_qd);
  if ((hScale < 0)) {
    QplugAppendTextDisplay("makeArrow WARNING: Width is less than 0\n\tArrow may point wrong way!");
  }
  
 //******************** calculate *************************
  
  QdRMatrixSeq_var arrow;
  if (head == 0)
    arrow = new QdRMatrixSeq(2, Dim(3) );
  else
    arrow = new QdRMatrixSeq(5, Dim(3) );
  QdRMatrix_var pos2_qd = new QdRMatrix();
  
  pos2_qd = deltax_qd;
  
  arrow[0][0] = (* pos1_qd)[0];
  arrow[0][1] = (* pos1_qd)[1];
  arrow[0][2] = (* pos1_qd)[2];
  float delx = (float)(* deltax_qd)[0] * (float)(* scale_qd);
  float dely = (float)(* deltax_qd)[1] * (float)(* scale_qd);
  float delz = (float)(* deltax_qd)[2] * (float)(* scale_qd);
  arrow[1][0] = (float)(* pos1_qd)[0] + delx;
  arrow[1][1] = (float)(* pos1_qd)[1] + dely;
  arrow[1][2] = (float)(* pos1_qd)[2] + delz;
  
  QdRScalarSeq_var dep0;
  
  if (head != 0) {
    float phi, xarr, yarr, zarr, width;
    
    switch (head) {
      case 1:
        width = hScale*sqrt(delx*delx + dely*dely);
        phi = atan2(dely, delx);
        xarr = -sin(phi)*width;
        yarr = cos(phi)*width;
	zarr = 0;
	break;
      case 2:
        width = hScale*sqrt(delx*delx + delz*delz);
        phi = atan2(delz, delx);
        xarr = -sin(phi)*width;
        yarr = 0;
	zarr = cos(phi)*width;
	break;
      case 3:
        width = hScale*sqrt(dely*dely + delz*delz);
        phi = atan2(dely, delz);
        xarr = 0;
        yarr = cos(phi)*width;
	zarr = -sin(phi)*width;
	break;
    }
    
    float onemw = 1. - hScale;
    arrow[2][0] = (float)(* pos1_qd)[0] + onemw*delx + xarr;
    arrow[2][1] = (float)(* pos1_qd)[1] + onemw*dely + yarr;
    arrow[2][2] = (float)(* pos1_qd)[2] + onemw*delz + zarr;
    
    arrow[3][0] = arrow[1][0];
    arrow[3][1] = arrow[1][1];
    arrow[3][2] = arrow[1][2];
    
    arrow[4][0] = (float)(* pos1_qd)[0] + onemw*delx - xarr;
    arrow[4][1] = (float)(* pos1_qd)[1] + onemw*dely - yarr;
    arrow[4][2] = (float)(* pos1_qd)[2] + onemw*delz - zarr;
    
    dep0 = new QdRScalarSeq(5);
    int d0=0;
    if (head == 3)
      d0 = 1;
    for (int i=0; i<5; i++) dep0[i] = arrow[i][d0];
    
  } else {
    dep0 = new QdRScalarSeq(2);
    dep0[0] = arrow[0][0]; dep0[1] = arrow[1][0];
  }
  
  // Add attributes:
  
  QdObject_var arrow_out = (QdObject_var) arrow;
  QdObject_var dep0_out = (QdObject_var) dep0;
  
  arrow_out->copy_xrefs_from(pos1_in);
  
  dep0_out->copy_xrefs_from(pos1_in);
  // Narrow type to a scalar
  if (dep0_out->xref_exists(DEPEND_0)) {
    dep0_out->delete_xref(DEPEND_0);
    arrow_out->delete_xref(DEPEND_0);
    QplugAppendTextDisplay("Deleting arrow DEPEND_0\n");
  }
  if (dep0_out->xref_exists(DEPEND_1)) {
    dep0_out->delete_xref(DEPEND_1);
    arrow_out->delete_xref(DEPEND_1);
    QplugAppendTextDisplay("Deleting arrow DEPEND_1\n");
  }
  QuSetTxtAttr(FRAME, "scalar>na", dep0_out);
  arrow_out->set_xref("DEPEND_1", dep0_out);
    
  if (dep0_out->xref_exists(DEPEND_0)) {
    dep0_out->delete_xref(DEPEND_0);
    arrow_out->delete_xref(DEPEND_0);
   QplugAppendTextDisplay("Deleting arrow DEPEND_0\n");
  }
    
  //***********************************************************
  
  // Assign data to output:
  
  call_list->push_back( arrow_out );
  
  QplugAppendTextDisplay ("makeArrow>> finished...\n");
  
  return QPLUG_SUCCESS;
}
