/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

/**********************************************************************
*
*  QSAS Plugin to shift timetags of a time series
*
*
*  Inputs:  Time Series
*           dt          time (secs) to ADD to each timetag
*
*  Output:  Cloned Time series with shifted tags and all xrefs
*
*  History:  sjs September 2001
***********************************************************************/

#include "qdutil.h"
#include "qplug_if.h"
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "Qdos.h"

using namespace QSAS;

// local prototype
extern "C" QplugReturnStatus MakeMonotonic (QplugArgList *);

QplugReturnStatus MakeMonotonic( QplugArgList *call_list )
{

// call-list arguments

QdObject_var tsin_obj;
QdObject_var ts_obj;
QdObject_var tsout_obj;
QdRScalar_var tol_ptr;
double tolerance;

//	Unpack the input data objects
//
//	..first input item is  time series

//  set pointer to objects and check objects

tsin_obj = QdObject_var::narrow((*call_list)[0]);

if (tsin_obj.is_nil()){
   QplugAppendTextDisplay("TimeShift> Input TS is nil\n");
   return QPLUG_FAILURE;
}


// re-order time tags
ts_obj = QuMakeMonotonic(tsin_obj);

tol_ptr = QdRScalar_var::narrow((*call_list)[1]);

if (tol_ptr.is_nil()){
   QplugAppendTextDisplay("Tolerence not set: Keeping all time tags\n");
   tsout_obj = ts_obj;
}
else
{
	// remove duplicate time tags (closer than tolerance)
	tolerance = *tol_ptr;
	if(tolerance > 1.e-10) tsout_obj = QuStripDuplicates(ts_obj,  tolerance);
	else tsout_obj = ts_obj;
}


call_list->push_back(tsout_obj);

QplugAppendTextDisplay ("MakeMonotonic: Done\n");
  
return QPLUG_SUCCESS;

}

