/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include "qplug_if.h"
#include "Qdos.h"

#include "qrtn.h"

extern "C" QplugReturnStatus   CoordRotation (QplugArgList *);


QplugReturnStatus   CoordRotation  ( QplugArgList *call_list )
{

QplugReturnStatus	status;

QrVector vec;

QdRMatrix_var m_in;
QdRMatrixSeq_var ts_in;
QdObject_var fr_ptr;
QdRScalar_var new_frame_ptr;

long check;
char *attr_value;

status = QPLUG_SUCCESS;

// identify plug-in version

QplugAppendTextDisplay (
        "MAH Coord Rotation: qrtn_plugin.cc (AJA)  8 Aug 1999, 18 Aug 2000, Feb 2006\n");


//	...first, unpack the input data objects

 ts_in = QdRMatrixSeq_var::narrow((*call_list)[0]);

if ( ts_in.is_nil()  ) {	
   m_in = QdRMatrix_var::narrow((*call_list)[0]);
   if( m_in.is_nil() ){
      QplugAppendTextDisplay ("CoordRotation>> ERROR - input object not valid\n");
	  return QPLUG_FAILURE; 
   }
   ts_in = new QdRMatrixSeq(1);
   ts_in[0] = *m_in;
   ts_in->copy_xrefs_from(m_in);
}      
if (  !QuIsThreeVector(ts_in) ) {	
   QplugAppendTextDisplay ("CoordRotation>> ERROR - input not a Vector!\n");
	return QPLUG_FAILURE; 
}        


 vec.n_recs = ts_in->sequence_size();

//       next item in list is int identifying target frame
//       acronyms are set to integer values by #define in qrtn.h
//       0  GSE,  Geocentric Solar Ecliptic
//       1  GSM,  Geocentric Solar Magnetic
//       2  GEOC, Geographic (geocentric)
//       3  SM,   Solar Magnetic
//       4  GEI,  Geocentric Inertial
//       5  MAGD,  Geomagnetic (dipole)
//       6  HAE,  Heliospheric Aries Ecliptic
//       7  HEE,  Heliospheric Earth Ecliptic
//       8  HEEQ, Heliospheric Earth Equatorial
//       9  GSEQ, Geocentric Solar Equatorial


 fr_ptr = (*call_list)[1];

 new_frame_ptr = QdRScalar_var::narrow(fr_ptr);
 
 if ( new_frame_ptr.is_nil() ) { 
    QplugAppendTextDisplay (
        "CoordRotation>> ERROR - argument not an integer flag!\n");
    return QPLUG_FAILURE; 
 }  

 vec.out_frame = (int)(*new_frame_ptr);

 // Find original frame
 
 vec.in_frame = vec.QrGetFrameFromAttr(ts_in);

// Find if this is a position vector


    vec.IsPositionVector = vec.QrIsPositionVector(ts_in);
    if(vec.IsPositionVector == 1){
      vec.IsPositionVector = 0; /* issue warning & continue, delta r rotates */
      
      if(vec.QrOriginsDiffer(vec.in_frame, vec.out_frame)) {
        QplugAppendTextDisplay 
          ("WARNING: CoordRotation plug-in provides frame rotation only.\n");
        QplugAppendTextDisplay 
          ("Transformation is between geocentric and heliocentric frames.\n");
        QplugAppendTextDisplay 
          ("Position vector coordinate transformations will be WRONG.\n");
        QplugAppendTextDisplay 
          ("Note: Spacecraft separations will rotate OK.\n");
      }
    }
 
QdRMatrixSeq_var ts_out = (QdRMatrixSeq_var) Factory::clone(ts_in);
ts_out->copy_xrefs_from(ts_in);
vec.ptr = ts_out;

vec.t = get_timetags(ts_in);
if( (vec.t).is_nil() ) {
  QplugAppendTextDisplay ("Cannot find time tags, some frames are time dependent\n");
  return QPLUG_FAILURE;
}


//  End of Plug-in wrapper, call C function to do rotation
 
 check = vec.QrDoRotation();

if (check == QRTN_OK || check == SAME_FRAME) {

 
  QdTimeSeq_var new_tt = Factory::clone(vec.t);
  
  if(new_tt.is_nil()){
    QplugAppendTextDisplay ("Cannot create new time tags\n");
    return QPLUG_FAILURE;
  }
  set_timetags((QdObject_var)vec.ptr, new_tt);     


  //	..add  the output series to the list
  
  call_list->push_back((QdObject_var)vec.ptr);

  if(check == SAME_FRAME){
    QplugAppendTextDisplay ("CoordRotation: Same frame, null operation\n");
  }
  else{
    QplugAppendTextDisplay ("CoordRotation: vector rotated \n");
  }
}
else{
    QplugAppendTextDisplay (vec.QrErrorMessage(check));
    status = QPLUG_FAILURE;
} // end if on check


return status;

}

