#include "IntervalWidgets.h"
#include "qwui.h"
#include "qbu.h"

TimeIntervalWidget::TimeIntervalWidget(QWidget* parent, int opt):
		QWidget(parent)
{
	currentInterval=QmjdTimeInterval(QmjdTime(2001, 9, 4), QmjdTime(2001, 9, 5));
	events.clear();
	deletedEvent = QmjdTimeInterval(QmjdTime(2001, 9, 4), QmjdTime(2001, 9, 4));
	
	InitialStart = currentInterval.start();
	InitialEnd = currentInterval.end();
	
	QGridLayout* layout=new QGridLayout(this);
	layout->setSpacing(2);

	
	slider=new TimeSlider(this, &currentInterval, &events);


	QHBoxLayout* buttonsLayout=new QHBoxLayout;
	buttonsLayout->setSpacing(1);
	previousEventButton=new QPushButton(style()->standardIcon(QStyle::SP_ArrowLeft, 0, this), "Previous event");
	buttonsLayout->addWidget(previousEventButton, 10);
	buttonsLayout->addSpacing(25);
	eventLab = new QLabel(" Event");
	buttonsLayout->addWidget(eventLab);
	eventNum = new QLineEdit(this);
	eventNum->setText("0");
	eventNum->setMaximumWidth(50);
	buttonsLayout->addWidget(eventNum);
	sliderTitle = new QLabel(" in (0, 0) ");
	buttonsLayout->addWidget(sliderTitle);
	buttonsLayout->addSpacing(15);
	nextEventButton=new QPushButton(style()->standardIcon(QStyle::SP_ArrowRight, 0, this), "Next event");
	buttonsLayout->addWidget(nextEventButton, 10);
	
	nextEventButton->setMaximumHeight(slider->height()/2);
	previousEventButton->setMaximumHeight(slider->height()/2);
	

	QHBoxLayout* eventLayout=new QHBoxLayout;
	eventLayout->setSpacing(1);

	delButton=new QPushButton( "Delete Event");
	eventLayout->addWidget(delButton, 10);
	eventLayout->addSpacing(20);

	resetButton=new QPushButton( "Reset Event");
	eventLayout->addWidget(resetButton, 10);
	eventLayout->addSpacing(20);
	QMenu* resetMenu=new QMenu;
	resetMenu->addAction("Reset interval to this Event", this, SLOT(resetWidget()));
	resetMenu->addAction("Undo Delete Event", this, SLOT(undoDelete()));
	resetButton->setMenu(resetMenu);

	nameLab = new QLabel("Event Name ");	
	eventLayout->addWidget(nameLab);
	nameFld = new QLineEdit(this);
	nameFld->setMaximumWidth(200);
	nameFld->setMinimumWidth(150);
	eventLayout->addWidget(nameFld, 20);
	eventLayout->addSpacing(5);
	
	sortButton=new QPushButton( "Sort Events");
	eventLayout->addWidget(sortButton, 10);
	eventLayout->addSpacing(5);
		

	QGroupBox *intervalBox=new QGroupBox(this);
	intervalBox->setTitle("Time Interval");
	QBoxLayout *intervaLayout=new QBoxLayout(QBoxLayout::TopToBottom);
	intervaLayout->setSpacing(1);
	
	intervalBox->setLayout(intervaLayout);
	
	QHBoxLayout* fromtoLayout=new QHBoxLayout;
	
	fromtoLayout->addWidget(new QLabel(" From", this), 0, Qt::AlignRight);
	fromWidget=new QbDateTimeWidget(this);
	fromtoLayout->addWidget(fromWidget, 1);

	fromtoLayout->addWidget(new QLabel(" To", this), 2, Qt::AlignRight);
	toWidget=new QbDateTimeWidget(this);
	fromtoLayout->addWidget(toWidget, 3);

	fromtoLayout->addSpacing(10);

	expandButton=new QPushButton("Collapse");
	fromtoLayout->addWidget(expandButton, 4);
	
	extrasLayout=new QHBoxLayout;

	centreLabel=new QLabel(" Centre", this);
	extrasLayout->addWidget(centreLabel, 0, Qt::AlignRight);
	centreWidget=new QbDateTimeWidget(this);
	extrasLayout->addWidget(centreWidget, 1);

	durationLabel=new QLabel(" Duration", this);
	extrasLayout->addWidget(durationLabel, 2, Qt::AlignRight);
	durationWidget=new QbDurationWidget(this);
	extrasLayout->addWidget(durationWidget, 3);
	extrasLayout->addSpacing(10);

	saveButton=new QPushButton("Save Interval");
	extrasLayout->addWidget(saveButton, 4);
	QMenu* saveMenu=new QMenu;
	saveMenu->addAction("Set current event", this, SLOT(changeEventSlot()));
	saveMenu->addAction("Add interval to events table", this, SLOT(saveIntervalToList()));
	saveMenu->addAction("Save interval to WL...", this, SLOT(saveInterval()));
	saveButton->setMenu(saveMenu);


	QHBoxLayout* viewLayout=new QHBoxLayout;
	viewLayout->setSpacing(1);
		
	viewButton =new QPushButton("Set View/Interval");
	QMenu* viewMenu=new QMenu;
	viewMenu->addAction("Reset Interval range", this, SLOT(resetInterval()));
	viewMenu->addSeparator();
	viewMenu->addAction("Reset view range", slider, SLOT(resetView()));
	viewMenu->addAction("Change view range...", slider, SLOT(changeViewRangeSlot()));
	viewButton->setMenu(viewMenu);

	viewLayout->addWidget(viewButton, 10, Qt::AlignLeft);

	QLabel *dropTarget = new QLabel(" ->  Drop interval or event list here  <- ");
	viewLayout->addWidget(dropTarget, 10, Qt::AlignHCenter);
	
	saveEventsButton =new QPushButton("Save Event Table");

	QMenu* saveEventsMenu=new QMenu;
	saveEventsMenu->addAction("Save", this, SLOT(saveEventTable()));
	saveEventsMenu->addAction("Save events table as...", this, SLOT(saveNewEventTable()));
	saveEventsButton->setMenu(saveEventsMenu);


	viewLayout->addWidget(saveEventsButton, 10, Qt::AlignRight);

    layout->setColumnStretch(0, 1);
    layout->setColumnStretch(1, 5);
    layout->setColumnStretch(2, 1);
    layout->setColumnStretch(3, 5);
    layout->setColumnStretch(4, 1);

	intervaLayout->addLayout(fromtoLayout);
	intervaLayout->addLayout(extrasLayout);

	
	layout->addWidget(intervalBox, 0, 0, 3, 5);
	layout->addLayout(eventLayout, 4, 0, 1, 5);
	QLabel *pad = new QLabel("    ");
	layout->addWidget(pad, 5, 0, 1, 5);
	layout->addWidget(slider, 6, 0, 2, 5);
	layout->addLayout(buttonsLayout, 8, 0, 1, 5);
	layout->addLayout(viewLayout, 9, 0, 1, 5);
	
	setLayout(layout);

	lastEventNum=0;
	current=1;
	previous=2;
	updateSelection();

	connect(fromWidget, SIGNAL(gotFocus()), this, SLOT(startFocus()));
	connect(fromWidget, SIGNAL(valueChanged()), this , SLOT(updateFields()));
	connect(toWidget, SIGNAL(gotFocus()), this, SLOT(endFocus()));
	connect(toWidget, SIGNAL(valueChanged()), this , SLOT(updateFields()));
	connect(centreWidget, SIGNAL(gotFocus()), this, SLOT(centreFocus()));
	connect(centreWidget, SIGNAL(valueChanged()), this , SLOT(updateFields()));
	connect(durationWidget, SIGNAL(gotFocus()), this, SLOT(durationFocus()));
	connect(durationWidget, SIGNAL(textChanged(const QString &)), this, SLOT(updateFields()));

	connect(slider, SIGNAL(eventDoubleClick(int)), this, SLOT(updateEvent(int)));
	connect(slider, SIGNAL(intervalChanged()), this, SLOT(updateAllFields()));

	connect(previousEventButton, SIGNAL(clicked()), this, SLOT(previousEvent()));
	connect(nextEventButton, SIGNAL(clicked()), this, SLOT(nextEvent()));
	connect(eventNum, SIGNAL(textEdited(const QString&)), this, SLOT(setEvent(const QString&)));
	connect(expandButton, SIGNAL(clicked()), this, SLOT(expand_collapse_fields()));
	connect(nameFld, SIGNAL(textEdited(const QString&)), this, SLOT(setEventName(const QString&)));
	connect(delButton, SIGNAL(clicked()), this, SLOT(deleteEvent()));
	connect(sortButton, SIGNAL(clicked()), this, SLOT(sortEvents()));
	
	setAcceptDrops(true);
	
	if(opt == 0) {
		collapse();
		showEvents(false);
	}
	else if(opt==1) {
		expand();
		showEvents(false);
	}
	else if(opt==2) {
		showEvents(true);
		collapse();
	}
}

TimeIntervalWidget::~TimeIntervalWidget()
{
}

QmjdTime TimeIntervalWidget::getStart() const
{
	return currentInterval.start();
}

QmjdTime TimeIntervalWidget::getEnd() const
{
	return currentInterval.end();
}

QmjdTimeInterval TimeIntervalWidget::getInterval() const
{
	return currentInterval;
}

double TimeIntervalWidget::getDuration() const
{
	return currentInterval.duration();
}

void TimeIntervalWidget::setStart(const QmjdTime& start)
{
	currentInterval.set_start(start);
	updateDisplay();
}

void TimeIntervalWidget::setEnd(const QmjdTime& end)
{
	currentInterval.set_end(end);
	updateDisplay();
}

void TimeIntervalWidget::setEvent(const QmjdTimeInterval& interval)
{	
	currentInterval=interval;
	
	updateDisplay();
    slider->updateDisplayedString();
	slider->update();
	EnableName(true);


}

void TimeIntervalWidget::setInterval(const QmjdTime& start, const QmjdTime& end, bool setViewRangeToo)
{
	currentInterval.set(start, end);
	if(setViewRangeToo){
		slider->setViewRange(QmjdTimeInterval(currentInterval));
		InitialStart = start;
		InitialEnd = end;
		
	}
	updateDisplay();
    slider->updateDisplayedString();
	slider->update();
	EnableName(false);

}

void TimeIntervalWidget::setInterval(const QmjdTimeInterval& interval, bool setViewRangeToo)
{
	currentInterval=interval;
	if(setViewRangeToo){
		slider->setViewRange(QmjdTimeInterval(currentInterval));
		InitialStart = interval.start();
		InitialEnd = interval.end();
	}
	updateDisplay();
    slider->updateDisplayedString();
	slider->update();
	EnableName(false);
	
}

void TimeIntervalWidget::setInterval(const QdObject_var& obj, bool setViewRangeToo)
{
	if(obj.is_nil())
		return;
		
	QdTimeInterval_var int_var=QdTimeInterval_var::narrow(obj);
	if(!int_var.is_nil())
	{
		setInterval(*int_var, setViewRangeToo);
		return;
	}

	QdTimeIntervalSeq_var tis_var=QdTimeIntervalSeq_var::narrow(obj);
	if(!tis_var.is_nil())
	{
		if(tis_var->sequence_size()>0)
		{
			QmjdTime min=tis_var->elt(0).start();
			QmjdTime max=min;
			for(int i=0; i<tis_var->sequence_size(); ++i)
			{
				if(tis_var->elt(i).start()<min)
					min=tis_var->elt(i).start();
				if(tis_var->elt(i).start()>max)
					max=tis_var->elt(i).start();
				if(tis_var->elt(i).end()<min)
					min=tis_var->elt(i).end();
				if(tis_var->elt(i).end()>max)
					max=tis_var->elt(i).end();
			}
			setInterval(min, max, setViewRangeToo);
			
			setEvent(tis_var[0]);
			
		}
		return;
	}

	QdTimeSeq_var ts_var=get_timetags(obj);
	
	if(!ts_var.is_nil())
	{
		setInterval(ts_var->elt(0), ts_var->elt(ts_var->size()-1), setViewRangeToo);
		return;
	}
}

void TimeIntervalWidget::setViewRange(const QmjdTimeInterval& tintv)
{
    slider->setViewRange(tintv);
}

QmjdTimeInterval TimeIntervalWidget::getViewRange() const
{
    return slider->getViewRange();
}

void TimeIntervalWidget::resetView()
{
	slider->resetView();
}


void TimeIntervalWidget::resetWidget()
{
	
	if(events.size() > 0)  setEvent(events.elt(lastEventNum));

}

void TimeIntervalWidget::resetInterval()
{
	
	setInterval(InitialStart, InitialEnd, false);

}

void TimeIntervalWidget::undoDelete()
{
	
	if(deletedEvent.duration() > 1.e-10 || deletedEvent.duration() < -1.e-10)  {
		setEvent(deletedEvent);
		appendIntervalSlot();
		// set event flag to new event
		bool OK;
		lastEventNum = eventNum->text().toInt(&OK);
		if(!OK) lastEventNum = 0;
		// turn off undo
		deletedEvent = QmjdTimeInterval(QmjdTime(2001, 9, 4), QmjdTime(2001, 9, 4));

	}

}

void TimeIntervalWidget::clearEventTable()
{
	events.clear();
	showEvents(false);	
}

void TimeIntervalWidget::setEventTable(QdTimeIntervalSeq_var et)
{
	events.assign(*et);
	events.copy_xrefs_from(et);
	
    expand();
	
	if( !(events.xref_exists("EVENT_NAME")) ) {
		QdStringSeq_var eventNames = new QdStringSeq(events.size(), QdString(""));
		events.change_xref("EVENT_NAME", eventNames);
	}
	else{
		if(events.xref_exists("EVENT_NAME") ){
			QdStringSeq_var names = QdStringSeq_var::narrow(events.get_xref("EVENT_NAME"));
			if( !names.is_nil() && names->size() == events.size()) nameFld->setText(names[0].c_str());
			else{
				QdStringSeq_var eventNames = new QdStringSeq(events.size(), QdString(""));
				events.change_xref("EVENT_NAME", eventNames);
			}
		}
		
	}
	
	sliderTitle->setText(QString(" in (0,%1) ").arg(et->size()-1) );

	slider->update();
}

QdTimeIntervalSeq_var TimeIntervalWidget::getEventTable()
{
	QdTimeIntervalSeq_var events_var = new QdTimeIntervalSeq(events);
	events.copy_xrefs_to(events_var);
	return events_var;
}

void TimeIntervalWidget::setAvailabilityRange(const QmjdTime& start, const QmjdTime& end)
{
	setAvailabilityRange(QmjdTimeInterval(start, end));
}

void TimeIntervalWidget::setAvailabilityRange(const QmjdTimeInterval& interval)
{
	slider->setAvailabilityRange(interval);
}

void TimeIntervalWidget::setAltPalette(bool alt)
{
	fromWidget->setAltPalette(alt);
	toWidget->setAltPalette(alt);
	centreWidget->setAltPalette(alt);
	durationWidget->setAltPalette(alt);
}

void TimeIntervalWidget::expand_collapse()
{
	if(centreLabel->isVisible() &&  slider->isVisible())
	{
		collapse();
	}
	else
	{
		expand();
	}
}


void TimeIntervalWidget::expand_collapse_fields()
{
    if(centreLabel->isVisible())
    {
        collapse();
    }
    else
    {
        expand();
    }
}

void TimeIntervalWidget::expand()
{
	centreLabel->setVisible(true);
	centreWidget->setVisible(true);
	durationLabel->setVisible(true);
	durationWidget->setVisible(true);
	saveButton->setVisible(true);
	expandButton->setText("Collapse");

}

void TimeIntervalWidget::collapse()
{
	centreLabel->setVisible(false);
	centreWidget->setVisible(false);
	durationLabel->setVisible(false);
	durationWidget->setVisible(false);
	saveButton->setVisible(false);
	expandButton->setText("Expand");
	current=1;
	previous=2;
	updateSelection();
}

void TimeIntervalWidget::showEvents(bool show)
{
	if(show){
	//	slider->setVisible(true);
		previousEventButton->setVisible(true);
		nextEventButton->setVisible(true);
		sliderTitle->setVisible(true);
		eventNum->setVisible(true);
		eventLab->setVisible(true);
		
		sortButton->setVisible(true);
		delButton->setVisible(true);
		resetButton->setVisible(true);
	//	viewButton->setVisible(true);
		saveEventsButton->setVisible(true);
		nameFld->setVisible(true);
		nameLab->setVisible(true);
	}
	else{
	//	slider->setVisible(false);
		previousEventButton->setVisible(false);
		nextEventButton->setVisible(false);
		sliderTitle->setVisible(false);
		eventNum->setVisible(false);
		eventLab->setVisible(false);
		
		sortButton->setVisible(false);
		delButton->setVisible(false);
		resetButton->setVisible(false);
	//	viewButton->setVisible(false);
		saveEventsButton->setVisible(false);
		nameFld->setVisible(false);
		nameLab->setVisible(false);
	}
}

void TimeIntervalWidget::updateDisplay()
{
	int p(previous), c(current);
	previous=0;
	current=0;
	fromWidget->setTime(currentInterval.start());
	toWidget->setTime(currentInterval.end());
	updateCentreDuration();
	previous=p;
	current=c;
}

void TimeIntervalWidget::updateStartEnd()
{
	QmjdTime centre=centreWidget->time();
	double duration;
	if(durationWidget->getDuration(duration))
	{
		QmjdTime start=centre-duration/2.;
		QmjdTime end=centre+duration/2.;
		fromWidget->setTime(start);
		toWidget->setTime(end);
	}
}

void TimeIntervalWidget::updateStartCentre()
{
	QmjdTime end=toWidget->time();
	double duration;
	if(durationWidget->getDuration(duration))
	{
		QmjdTime start=end-duration;
		QmjdTime centre=end-duration/2.;
		fromWidget->setTime(start);
		centreWidget->setTime(centre);
	}
}

void TimeIntervalWidget::updateStartDuration()
{
	QmjdTime end=toWidget->time();
	QmjdTime centre=centreWidget->time();
	double duration=(end.getDiffSecs(centre))*2.;
	fromWidget->setTime(end-duration);
	durationWidget->setDuration(duration);
}

void TimeIntervalWidget::updateEndCentre()
{
	QmjdTime start=fromWidget->time();
	double duration;
	if(durationWidget->getDuration(duration))
	{
		QmjdTime end=start+duration;
		QmjdTime centre=start+duration/2.;
		toWidget->setTime(end);
		centreWidget->setTime(centre);
	}
}

void TimeIntervalWidget::updateEndDuration()
{
	QmjdTime start=fromWidget->time();
	QmjdTime centre=centreWidget->time();
	double duration=(centre.getDiffSecs(start))*2.;
	toWidget->setTime(start+2.*duration);
	durationWidget->setDuration(duration);
}

void TimeIntervalWidget::updateCentreDuration()
{
	QmjdTime start=fromWidget->time();
	QmjdTime end=toWidget->time();
	double duration=end.getDiffSecs(start);
	centreWidget->setTime(start+duration/2.);
	durationWidget->setDuration(duration);
}

void TimeIntervalWidget::startFocus()
{
	if(current!=1)
	{
		previous=current;
		current=1;
		updateSelection();
	}
}

void TimeIntervalWidget::endFocus()
{
	if(current!=2)
	{
		previous=current;
		current=2;
		updateSelection();
	}
}

void TimeIntervalWidget::centreFocus()
{
	if(current!=3)
	{
		previous=4; // select centre also selects duration
		current=3;
		updateSelection();
	}
}

void TimeIntervalWidget::durationFocus()
{
	if(current!=4)
	{
		previous=current;
		current=4;
		updateSelection();
	}
}

void TimeIntervalWidget::updateSelection()
{
	if(current !=0 && previous !=0)
	{
		fromWidget->setActive(current==1 || previous==1);
		toWidget->setActive(current==2 || previous==2);
		centreWidget->setActive(current==3 || previous==3);
		durationWidget->setActive(current==4 || previous==4);
	}
}

void TimeIntervalWidget::updateFields()
{
	if(current==0 || previous==0) return; // Changed by drop
	int c(current), p(previous);
	current=0;
	previous=0;

	if(c==1 || p==1) // One of the parameters is start
	{
		if(c==2 || p==2)
			updateCentreDuration();
		else if(c==3 || p==3)
			updateEndDuration();
		else updateEndCentre();
	}
	else if(c==2 || p==2) // One of the params is end, the other is not start
	{
		if(c==3 || p==3)
			updateStartDuration();
		else updateStartCentre();
	}
	else updateStartEnd();

	currentInterval.set(fromWidget->time(), toWidget->time());
	slider->update();
	
	previous=p;
	current=c;
	emit intervalUpdated();
}

bool TimeIntervalWidget::hasFocus() const
{
	return (QWidget::hasFocus()||slider->hasFocus()||previousEventButton->hasFocus() ||nextEventButton->hasFocus()||fromWidget->hasFocus()||toWidget->hasFocus()||centreWidget->hasFocus()||durationWidget->hasFocus());
}

void TimeIntervalWidget::updateEvent(int index)
{
	eventNum->setText( QString("%1").arg(index) );
	if(events.xref_exists("EVENT_NAME") ){
		QdStringSeq_var names = QdStringSeq_var::narrow(events.get_xref("EVENT_NAME"));
		if( !names.is_nil() && names->size() == events.size()) nameFld->setText(names[index].c_str());
	}
	lastEventNum = index;
	EnableName(true);
}

void TimeIntervalWidget::updateAllFields()
{
	int c(current), p(previous);
	current=0;
	previous=0;

	fromWidget->setTime(currentInterval.start());
	toWidget->setTime(currentInterval.end());
	double duration=currentInterval.duration();
	centreWidget->setTime(currentInterval.start()+duration/2.);
	durationWidget->setDuration(duration);
	
	previous=p;
	current=c;
	
	EnableName(false);
	emit intervalUpdated();
}

void TimeIntervalWidget::dropEvent(QDropEvent* event)
{
	QdObject_var xref_obj = QuGetObjByName(event->mimeData()->text());

	QdTimeInterval_var var_int=QdTimeInterval_var::narrow(xref_obj);
	QdTimeIntervalSeq_var seq_int=QdTimeIntervalSeq_var::narrow(xref_obj);
	
	if(!var_int.is_nil())
	{
		setInterval(var_int, false);
		if(events.size() > 0) appendIntervalSlot();
		
		QmjdTimeInterval range = slider->getViewRange();
		
		slider->setViewRange(range.int_union(currentInterval));
				
		emit intervalUpdated();
		return;
	}
	
	if(!seq_int.is_nil())
	{
		setEventTable(seq_int);
		dobjPath = QString(event->mimeData()->text());
		slider->setViewRange(*seq_int);
		setEvent(seq_int->elt(0));
		showEvents(true);
		
		emit intervalUpdated();
		return;
	}
	
	QdTimeSeq_var ts_var=get_timetags(xref_obj);
	
	if(!ts_var.is_nil())
	{
		setInterval(ts_var->elt(0), ts_var->elt(ts_var->size()-1), false);
		emit intervalUpdated();
		return;
	}
}

void TimeIntervalWidget::dragEnterEvent(QDragEnterEvent* event)
{
	this->activateWindow();
	this->raise();
	if (event->mimeData()->hasText()) event->acceptProposedAction();
}

void TimeIntervalWidget::appendIntervalSlot()
{
	events.push_back(currentInterval);
    showEvents(true);
	slider->update();

	eventNum->setText(QString("%1").arg(events.size()-1));
	sliderTitle->setText(QString(" in (0,%1) ").arg(events.size()-1) );

	if(events.xref_exists("EVENT_NAME") ){
		QdStringSeq_var names = QdStringSeq_var::narrow(events.get_xref("EVENT_NAME"));
		if( !names.is_nil() ) names->push_back(QdString(nameFld->text().toLatin1().data()));
	}
				
	emit intervalUpdated();
}

void TimeIntervalWidget::changeEventSlot()
{
	
	if(events.sequence_size()<1)
	{
        appendIntervalSlot();
		return;
	}
	
	int max=events.sequence_size();
	
	bool ok;
	int chosen = eventNum->text().toInt(&ok, 10);
		
	if(ok && chosen < events.size())
	{
		(events)[chosen]=currentInterval;
		slider->update();
		emit intervalUpdated();
	}
}


void TimeIntervalWidget::EnableName(bool enable)
{

	nameFld->setEnabled(enable);
	nameLab->setEnabled(enable);
}

void TimeIntervalWidget::saveInterval()
{
	QdTimeInterval_var newObj=new QdTimeInterval(currentInterval);
	if(!newObj.is_nil())
    {
		bool ok=false;
		QString eventName = nameFld->text();
		if(eventName.isEmpty()) eventName = "interval";
		
		QString text = QInputDialog::getText(this, tr("         New Time Interval"),
                                          tr("     Time interval name:       "), QLineEdit::Normal,
                                          eventName, &ok);
		if(!ok) return;
		
	    QString path = "Time Intervals/"+text;
		QwuiPutOnWL(newObj, path);
			
    }
	
}

void TimeIntervalWidget::saveIntervalToList()
{
	
	bool ok=false;
	QString defName = QString("event %1").arg(events.size());
	
	QString text = QInputDialog::getText(this, tr("         New Event"),
                                          tr("     Event name:       "), QLineEdit::Normal,
                                          defName, &ok);
	if(!ok) return;
		

	nameFld->setText(text);
	EnableName(true);
	
	qApp->processEvents(); // ensure name is changed on slot before continuing
	
	appendIntervalSlot();
}

void TimeIntervalWidget::saveEventTable()
{
    if(events.sequence_size()==0)
        return;

	if(dobjPath.isEmpty() ) {
		saveNewEventTable();
		return;
	}
	
	QdTimeIntervalSeq_var newObj=new QdTimeIntervalSeq(events);
	events.copy_xrefs_to(newObj);
		
	
	QwuiPutOnWL(newObj, dobjPath, true);

}

void TimeIntervalWidget::saveNewEventTable()
{
    if(events.sequence_size()==0)
        return;
    
	QdTimeIntervalSeq_var newObj=new QdTimeIntervalSeq(events);
	events.copy_xrefs_to(newObj);
	
    if(!newObj.is_nil())
    {
		bool ok=false;
	 
		QString text = QInputDialog::getText(this, tr("         New Event Table"),
                                          tr("      Event table name:        "), QLineEdit::Normal,
                                          tr("table"), &ok);
		if(!ok) return;
		
	    QString path = "Time Intervals/"+text;
		QwuiPutOnWL(newObj, path);
			
    }
}

void TimeIntervalWidget::sortEvents()
{
	if(events.sequence_size()<1) return;
	
	bool ok;
	int index = eventNum->text().toInt(&ok);
	
	if(!ok || index <0 || index >= events.sequence_size()) index = 0;
		
	vector <int> order(events.sequence_size());
	
	// sort assuming order is almost correct
	
	for(int i=0; i<events.sequence_size(); i++) order[i] = i;
	
	for(int i=1; i<events.sequence_size(); i++)
	{
		if(events[order[i-1]] > events[order[i]]) 
		{
			order[i] = order[i-1];
			order[i-1] = i;
			// put in correct place
			for(int j=i-1; j>0; j--)
			{
				if( events[order[j-1]] > events[i]) 
				{	
					int hold = order[j];
					order[j] = order[j-1];
					order[j-1] = hold;
				}
				else break;
			}
		}
	}
	
	QdTimeIntervalSeq_var newEvents = Factory::clone(events);
	QdStringSeq_var newNames = new QdStringSeq(events.sequence_size());
	QdStringSeq_var names;
	
	if(events.xref_exists("EVENT_NAME") ){
		names = QdStringSeq_var::narrow(events.get_xref("EVENT_NAME"));
	}
	
	for(int i=0; i<events.sequence_size(); ++i)
	{
		newEvents->elt(i) = events.elt(order[i]);
		if( !names.is_nil() && names->size() == events.size()) newNames[i] = names[order[i]];
	}
	
	events.clear();
	events.assign(*newEvents);
	events.change_xref("EVENT_NAME", newNames);
	
	eventNum->setText( QString("%1").arg(order[index]) );
	setEvent(eventNum->text());
	
	slider->update();

	lastEventNum = order[index];
}



void TimeIntervalWidget::nextEvent()
{
	if(events.sequence_size()<1) return;

	bool ok;
	int index = eventNum->text().toInt(&ok,10);
	index++;
	
	if(ok && index < events.size()) {
		setEvent(events.elt(index));
		slider->update();
		eventNum->setText( QString("%1").arg(index) );
		
		if(events.xref_exists("EVENT_NAME") ){
			QdStringSeq_var names = QdStringSeq_var::narrow(events.get_xref("EVENT_NAME"));
			if( !names.is_nil() && names->size() == events.size()) nameFld->setText(names[index].c_str());
		}

	}
	lastEventNum = index;
	emit intervalUpdated();

}

void TimeIntervalWidget::previousEvent()
{
	if(events.sequence_size()<1) return;
	// events are ordered by start time. Events with common start times are ordered by end time
	
	bool ok;
	int index = eventNum->text().toInt(&ok,10);
	index--;
	
	if(ok && index < events.size()) {
		setEvent(events.elt(index));
		slider->update();
		eventNum->setText( QString("%1").arg(index) );
		
		if(events.xref_exists("EVENT_NAME") ){
			QdStringSeq_var names = QdStringSeq_var::narrow(events.get_xref("EVENT_NAME"));
			if( !names.is_nil() && names->size() == events.size()) nameFld->setText(names[index].c_str());
		}
	}
	lastEventNum = index;
 	emit intervalUpdated();

}

void TimeIntervalWidget::setEvent(const QString &txt)
{
	bool ok;
	int index = txt.toInt(&ok,10);
	
	if(index >= events.size() || !ok) {
		index = events.size() - 1;
		if(index < 0) {
			eventNum->setText( QString(""));
			return;
		}
		else eventNum->setText( QString("%1").arg(index) );
	}

	setEvent(events.elt(index));
		
	if(events.xref_exists("EVENT_NAME") ){
		QdStringSeq_var names = QdStringSeq_var::narrow(events.get_xref("EVENT_NAME"));
		if( !names.is_nil() && names->size() == events.size()) nameFld->setText(names[index].c_str());
	}

	slider->update();
	
	lastEventNum = index;
	

}



void TimeIntervalWidget::setEventName(const QString &txt)
{
	bool ok;
	int index = eventNum->text().toInt(&ok,10);
	
	if(index < events.size() && ok) {
		if(events.xref_exists("EVENT_NAME") ){
			QdStringSeq_var names = QdStringSeq_var::narrow(events.get_xref("EVENT_NAME"));
			if( !names.is_nil() && names->size() == events.size()) names[index] = txt.toLatin1().data();
		}
	}

	emit intervalUpdated();

}

void TimeIntervalWidget::deleteEvent()
{
	bool ok;
	int index = eventNum->text().toInt(&ok,10);
	
	vector <bool> msk(events.size(), false);

	if(index < events.size() && ok) {
		deletedEvent = events.elt(index);
		msk[index] = true;
		if(events.xref_exists("EVENT_NAME") ){
			QdStringSeq_var names = QdStringSeq_var::narrow(events.get_xref("EVENT_NAME"));
			if( !names.is_nil() && names->size() == events.size()) {
				names->erase(msk);
			}
		}
		events.erase(msk);
	}
	
	if( events.size() <= 0 ) 
	{
		eventNum->setText(QString(""));
		slider->update();
	}
	else {
		if(index >= events.size()  ) eventNum->setText(QString("%1").arg(events.size()-1));
	
		setEvent(eventNum->text()); // will update slider
	}
	emit intervalUpdated();

}

void TimeIntervalWidget::setReadOnly(bool readOnly)
{
	
	fromWidget->setReadOnly(readOnly);
	toWidget->setReadOnly(readOnly);
	centreWidget->setReadOnly(readOnly);
	durationWidget->setReadOnly(readOnly);
	
}

//------------------------------------------------------------------

//------------------------------------------------------------------

ScalarIntervalWidget::ScalarIntervalWidget(QWidget* parent):
		QWidget(parent)
{
	currentStart=0.;
	currentEnd=1.;
	
	QGridLayout* layout=new QGridLayout(this);

	
	layout->addWidget(new QLabel("From", this), 0, 0, Qt::AlignRight);
	fromWidget=new QguWLinput(this, "");
	fromWidget->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	layout->addWidget(fromWidget, 0, 1);

	layout->addWidget(new QLabel("To", this), 0, 2, Qt::AlignRight);
	toWidget=new QguWLinput(this, "");
	toWidget->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	layout->addWidget(toWidget, 0, 3);

	expandButton=new QPushButton("Expand");
	layout->addWidget(expandButton, 0, 4);

	centreLabel=new QLabel("Centre", this);
	layout->addWidget(centreLabel, 1, 0, Qt::AlignRight);
	centreWidget=new QguWLinput(this, "");
	centreWidget->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	layout->addWidget(centreWidget, 1, 1);

	lengthLabel=new QLabel("Duration", this);
	layout->addWidget(lengthLabel, 1, 2, Qt::AlignRight);
	lengthWidget=new QguWLinput(this, "");
	lengthWidget->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	layout->addWidget(lengthWidget, 1, 3);

	saveButton=new QPushButton("Save Interval");
	QMenu* saveMenu=new QMenu;
	saveMenu->addAction("Save", this, SLOT(saveInterval()));
	saveMenu->addAction("Save As...", this, SLOT(saveNewInterval()));
	saveButton->setMenu(saveMenu);
	layout->addWidget(saveButton, 1, 4);

	slider=new ScalarIntervalSlider(this, &currentStart, &currentEnd);

	layout->addWidget(slider, 2, 0, 1, 5);

    layout->setColumnStretch(0, 1);
    layout->setColumnStretch(1, 5);
    layout->setColumnStretch(2, 1);
    layout->setColumnStretch(3, 5);
    layout->setColumnStretch(4, 1);
	
//	menu->addAction("Reset view range", slider, SLOT(resetView()));
//	menu->addAction("Change view range...", slider, SLOT(changeViewRangeSlot()));
	
	setLayout(layout);
	current=1;
	previous=2;
	updateSelection();

	connect(fromWidget->lineBox, SIGNAL(gotFocus()), this, SLOT(startFocus()));
	connect(fromWidget, SIGNAL(edited_OK()), this , SLOT(updateFields()));
	connect(toWidget->lineBox, SIGNAL(gotFocus()), this, SLOT(endFocus()));
	connect(toWidget, SIGNAL(edited_OK()), this , SLOT(updateFields()));
	connect(centreWidget->lineBox, SIGNAL(gotFocus()), this, SLOT(centreFocus()));
	connect(centreWidget, SIGNAL(edited_OK()), this , SLOT(updateFields()));
	connect(lengthWidget->lineBox, SIGNAL(gotFocus()), this, SLOT(lengthFocus()));
	connect(lengthWidget, SIGNAL(edited_OK()), this, SLOT(updateFields()));
	connect(expandButton, SIGNAL(clicked()), this, SLOT(expand_collapse()));

	connect(slider, SIGNAL(intervalChanged()), this, SLOT(updateAllFields()));

	altPalette=false;

	setAcceptDrops(true);
	
	collapse();
}

ScalarIntervalWidget::~ScalarIntervalWidget()
{}

double ScalarIntervalWidget::getStart() const
{
	return fromWidget->text().toDouble();
}

double ScalarIntervalWidget::getEnd() const
{
	return toWidget->text().toDouble();
}

double ScalarIntervalWidget::getLength() const
{
	return getEnd()-getStart();
}

void ScalarIntervalWidget::setRangeName(QString Name)
{
	rangeName = QString("Constants/") + Name;
}

void ScalarIntervalWidget::setStart(const double start)
{
	currentStart=start;
	
	int p(previous), c(current);
	previous=0;
	current=0;
	fromWidget->setText(QString("%1").arg(start));
	updateCentreLength();
	previous=p;
	current=c;
	emit intervalUpdated();
}

void ScalarIntervalWidget::setEnd(const double end)
{
	currentEnd=end;
	
	int p(previous), c(current);
	previous=0;
	current=0;
	toWidget->setText(QString("%1").arg(end));
	updateCentreLength();
	previous=p;
	current=c;
	emit intervalUpdated();
}

void ScalarIntervalWidget::setInterval(double start, double end, bool setViewRangeToo)
{
	// trap inf and NaN which may be created in data
	if( !isfinite(start) ) start = -1.e30;
	if( !isfinite(end) ) end = 1.e30;
	
	currentStart=start;
	currentEnd=end;
	
	int p(previous), c(current);
	previous=0;
	current=0;
	fromWidget->setText(QString("%1").arg(start));
	toWidget->setText(QString("%1").arg(end));
	updateCentreLength();
	previous=p;
	current=c;
	
	if(setViewRangeToo) slider->setViewRange(2*start-end, 2*end-start);
	
	emit intervalUpdated();
	slider->update();
}
/*
void ScalarIntervalWidget::setViewRange(double start, double end)
{
    slider->setViewRange(start, end);
}*/

void ScalarIntervalWidget::setAvailabilityRange(double start, double end)
{
	slider->setAvailabilityRange(start, end);
	slider->setViewRange(start, end);
}

void ScalarIntervalWidget::expand_collapse()
{
	if(slider->isVisible() )
	{
		collapse();
	}
	else
	{
		expand();
	}
}

void ScalarIntervalWidget::expand()
{
	centreLabel->setVisible(true);
	centreWidget->setVisible(true);
	lengthLabel->setVisible(true);
	lengthWidget->setVisible(true);
	saveButton->setVisible(true);
	expandButton->setText("Collapse");
	slider->setVisible(true);

}

void ScalarIntervalWidget::collapse()
{
	centreLabel->setVisible(false);
	centreWidget->setVisible(false);
	lengthLabel->setVisible(false);
	lengthWidget->setVisible(false);
	saveButton->setVisible(false);
	expandButton->setText("Expand");
	slider->setVisible(false);

	current=1;
	previous=2;
	updateSelection();
}

void ScalarIntervalWidget::saveInterval()
{
	// create an object to store axis ranges
	QdRMatrix_var x_mat=new QdRMatrix(Dim(2));
	(*x_mat)[0] = getStart();
	(*x_mat)[1] = getEnd();
	QdObject_var x_obj = x_mat;
	
	if(rangeName.isEmpty()) {
		saveNewInterval();
		return;
	}
	
	// 	Save to WL
	QwuiPutOnWL(x_obj, rangeName, true);
	
}

void ScalarIntervalWidget::saveNewInterval()
{
	// create an object to store axis ranges
	QdRMatrix_var x_mat=new QdRMatrix(Dim(2));
	(*x_mat)[0] = getStart();
	(*x_mat)[1] = getEnd();
	QdObject_var x_obj = x_mat;
	
	// 	Save to WL

    {
		bool ok=false;
		if(rangeName.isEmpty()) setRangeName("range");
		
		QString text = QInputDialog::getText(this, tr("        New Scalar range"),
                                          tr("     Range name:       "), QLineEdit::Normal,
                                          rangeName, &ok);
		if(!ok) return;
		
		rangeName = QwuiPutOnWL(x_obj, text);
			
    }
	
}


void ScalarIntervalWidget::updateDisplay()
{
	int p(previous), c(current);
	previous=0;
	current=0;
	fromWidget->setText(QString("%1").arg(currentStart));
	toWidget->setText(QString("%1").arg(currentEnd));
	updateCentreLength();
	previous=p;
	current=c;
}

void ScalarIntervalWidget::updateStartEnd()
{
	double centre=centreWidget->text().toDouble();
	double length=lengthWidget->text().toDouble();
	double start=centre-length/2.;
	double end=centre+length/2.;
	fromWidget->setText(QString("%1").arg(start));
	toWidget->setText(QString("%1").arg(end));
}

void ScalarIntervalWidget::updateStartCentre()
{
	double end=toWidget->text().toDouble();
	double length=lengthWidget->text().toDouble();
	double start=end-length;
	double centre=end-length/2.;
	fromWidget->setText(QString("%1").arg(start));
	centreWidget->setText(QString("%1").arg(centre));

}

void ScalarIntervalWidget::updateStartLength()
{
	double end=toWidget->text().toDouble();
	double centre=centreWidget->text().toDouble();
	double length=(end-centre)*2.;
	toWidget->setText(QString("%1").arg(end-length));
	lengthWidget->setText(QString("%1").arg(length));
}

void ScalarIntervalWidget::updateEndCentre()
{
	double start=fromWidget->text().toDouble();
	double length=lengthWidget->text().toDouble();
	double end=start+length;
	double centre=start+length/2.;
	toWidget->setText(QString("%1").arg(end));
	centreWidget->setText(QString("%1").arg(centre));
}

void ScalarIntervalWidget::updateEndLength()
{
	double start=fromWidget->text().toDouble();
	double centre=centreWidget->text().toDouble();
	double length=(centre-start)*2.;
	toWidget->setText(QString("%1").arg(start+2.*length));
	lengthWidget->setText(QString("%1").arg(length));
}

void ScalarIntervalWidget::updateCentreLength()
{
	double start=fromWidget->text().toDouble();
	double end=toWidget->text().toDouble();
	double length=end-start;
	centreWidget->setText(QString("%1").arg(start+length/2.));
	lengthWidget->setText(QString("%1").arg(length));
}

void ScalarIntervalWidget::startFocus()
{
	if(current!=1)
	{
		previous=current;
		current=1;
		updateSelection();
	}
}

void ScalarIntervalWidget::endFocus()
{
	if(current!=2)
	{
		previous=current;
		current=2;
		updateSelection();
	}
}

void ScalarIntervalWidget::centreFocus()
{
	if(current!=3)
	{
		previous=current;
		current=3;
		updateSelection();
	}
}

void ScalarIntervalWidget::lengthFocus()
{
	if(current!=4)
	{
		previous=current;
		current=4;
		updateSelection();
	}
}

void ScalarIntervalWidget::updateSelection()
{
	if(current !=0 && previous !=0)
	{
		fromWidget->lineBox->setActive(current==1 || previous==1);
		toWidget->lineBox->setActive(current==2 || previous==2);
		centreWidget->lineBox->setActive(current==3 || previous==3);
		lengthWidget->lineBox->setActive(current==4 || previous==4);
	}
}

void ScalarIntervalWidget::updateFields()
{
	if(current==0 || previous==0) return; // Changed by drop
	int c(current), p(previous);
	current=0;
	previous=0;

	if(c==1 || p==1) // One of the parameters is start
	{
		if(c==2 || p==2)
			updateCentreLength();
		else if(c==3 || p==3)
			updateEndLength();
		else updateEndCentre();
	}
	else if(c==2 || p==2) // One of the params is end, the other is not start
	{
		if(c==3 || p==3)
			updateStartLength();
		else updateStartCentre();
	}
	else updateStartEnd();

	currentStart=fromWidget->text().toDouble();
	currentEnd=toWidget->text().toDouble();

	slider->update();
	
	previous=p;
	current=c;
	
	emit intervalUpdated();
}

void ScalarIntervalWidget::updateAllFields()
{
	int c(current), p(previous);
	current=0;
	previous=0;

	fromWidget->setText(QString("%1").arg(currentStart));
	toWidget->setText(QString("%1").arg(currentEnd));
	centreWidget->setText(QString("%1").arg((currentStart+currentEnd)/2.));
	lengthWidget->setText(QString("%1").arg(currentEnd-currentStart));
	
	previous=p;
	current=c;
	
	emit intervalUpdated();
}

bool ScalarIntervalWidget::hasFocus() const
{
	return (QWidget::hasFocus()||slider->hasFocus()||fromWidget->hasFocus()||toWidget->hasFocus()||centreWidget->hasFocus()||lengthWidget->hasFocus());
}

void ScalarIntervalWidget::setAltPalette(bool alt)
{
	if(altPalette!=alt)
	{
		altPalette=alt;
		QColor active=(alt ? QApplication::palette().color(QPalette::Active, QPalette::Midlight) : QApplication::palette().color(QPalette::Active, QPalette::Light));
		QColor inactive=(alt ? QApplication::palette().color(QPalette::Active, QPalette::Button) : QApplication::palette().color(QPalette::Active, QPalette::Midlight));

		if(fromWidget->lineBox->active)
		{
			QPalette palette;
			palette.setColor(fromWidget->lineBox->backgroundRole(), active);
			fromWidget->lineBox->setPalette(palette);
		}
		else
		{
			QPalette palette;
			palette.setColor(toWidget->lineBox->backgroundRole(), inactive);
			toWidget->lineBox->setPalette(palette);
		}

		if(toWidget->lineBox->active)
		{
			QPalette palette;
			palette.setColor(toWidget->lineBox->backgroundRole(), active);
			toWidget->lineBox->setPalette(palette);
		}
		else
		{
			QPalette palette;
			palette.setColor(toWidget->lineBox->backgroundRole(), inactive);
			toWidget->lineBox->setPalette(palette);
		}
		
		if(centreWidget->lineBox->active)
		{
			QPalette palette;
			palette.setColor(centreWidget->lineBox->backgroundRole(), active);
			centreWidget->lineBox->setPalette(palette);
		}
		else
		{
			QPalette palette;
			palette.setColor(centreWidget->lineBox->backgroundRole(), inactive);
			centreWidget->lineBox->setPalette(palette);
		}

		if(lengthWidget->lineBox->active)
		{
			QPalette palette;
			palette.setColor(lengthWidget->lineBox->backgroundRole(), active);
			lengthWidget->lineBox->setPalette(palette);
		}
		else
		{
			QPalette palette;
			palette.setColor(lengthWidget->lineBox->backgroundRole(), inactive);
			lengthWidget->lineBox->setPalette(palette);
		}
	}
}


void ScalarIntervalWidget::dropEvent(QDropEvent* event)
{
	QdObject_var xref_obj = QuGetObjByName(event->mimeData()->text());

	QdRMatrix_var var_range=QdRMatrix_var::narrow(xref_obj);
	
	if(!var_range.is_nil() && var_range->size() >=2 )
	{
		rangeName = event->mimeData()->text();

		RMatrix::iterator itr = var_range->begin();
		double from = *itr;
		itr++;
		double to = *itr;
		setInterval(from, to);		
				
		emit intervalUpdated();
		return;
	}
		
}

void ScalarIntervalWidget::dragEnterEvent(QDragEnterEvent* event)
{
	this->activateWindow();
	this->raise();
	if (event->mimeData()->hasText()) event->acceptProposedAction();
}

