#include "QbEditors.h"

#include "qdutil.h"

#include "QbTimeWidgets.h"


QbEditor::QbEditor(QdObject_var dobj, QWidget *parent, const char *name ) 
	: QGroupBox(parent)
{
	
	this->setPalette(qApp->palette());
	QFontMetrics qbu_font_mt(qApp->font());
	int char_width = qbu_font_mt.width("n");

	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	valueBox = new QWidget(this);
	QHBoxLayout * valueLayout=new QHBoxLayout;
	valueLayout->setSpacing(0);
#if QT_VERSION < 0x040300
	valueLayout->setMargin(0);
#else
	valueLayout->setContentsMargins(0,0,0,0);
#endif

	valueLabel = new QLabel("  Value ");
	valueLayout->addWidget(valueLabel);
	
	valueField = new QguWLinput(valueBox,"value");
	valueField->setMinimumWidth(char_width*23);
	valueField->setEnabled(true);
	valueLayout->addWidget(valueField);
	
	valueBox->setLayout(valueLayout);
// 	valueBox->setMaximumHeight(30);
	mainLayout->addWidget(valueBox);
    
	dep0Box = new QWidget(this);
	QHBoxLayout * timeLayout=new QHBoxLayout;
	timeLayout->setSpacing(0);
#if QT_VERSION < 0x040300
	timeLayout->setMargin(0);
	timeField = new QbDateTimeLineEdit(dep0Box);
#else
	timeLayout->setContentsMargins(0,0,0,0);
	timeField = new QbDateTimeWidget(dep0Box);
#endif

	SdepField = new QguWLinput(dep0Box,"dep0");
	SdepField->setMinimumWidth(char_width*23);
	SdepField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));

	QdObject_var D0 = get_Depend_0(dobj);
	QdTimeSeq_var tt = QdTimeSeq_var::narrow(D0);
	QdTime_var ttag = QdTime_var::narrow(D0);
	QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(D0);
	QdRScalar_var stag = QdRScalar_var::narrow(D0);


	
	if(!tt.is_nil() || !ttag.is_nil()) {
		QLabel * timeLabel = new QLabel("at Time ");
		timeLayout->addWidget(timeLabel, Qt::AlignRight);
		timeLayout->addWidget(timeField);
		timeField->setEnabled(true);
		SdepField->setEnabled(false);
		SdepField->setVisible(false);
	}
	else if(!ss.is_nil() || !stag.is_nil()) {
		QLabel * atLabel = new QLabel("at ");
		timeLayout->addWidget(atLabel, Qt::AlignRight);
		timeLayout->addWidget(SdepField);
		SdepField->setEnabled(true);
		timeField->setEnabled(false);
		timeField->setVisible(false);
	}
	
	dep0Box->setLayout(timeLayout);
// 	dep0Box->setMaximumHeight(30);
	mainLayout->addWidget(dep0Box);
  
// 	mainLayout->addStretch(1);
	
	this->setLayout(mainLayout);
	
	maxRec = 0;
	record = 0;
}


QbEditor::~QbEditor(){
}


void QbEditor::setReadOnly(bool readOnly){

	valueField->setReadOnly(readOnly);
	timeField->setReadOnly(readOnly);

}

void QbEditor::showValueField(bool OKtoShow){
   
	if ( OKtoShow ){
		valueBox->show();
	}
	else{
		valueBox->hide();
	}
   
	return;
}

void QbEditor::showDep0Field(bool OKtoShow){
   
	if ( OKtoShow ){
		dep0Box->show();
	}
	else{
		dep0Box->hide();
	}
   
	return;
}



//-----------------------------------------------------------------------
QbScalarEditor::QbScalarEditor(QdObject_var dobj, QWidget *parent, const char *name ) 
	: QbEditor(dobj, parent,name)
{
	setTitle("Constant Scalar");
  
	valueField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	connect(valueField, SIGNAL(edited_OK()), this, SLOT(valueChanged())  );
	connect(timeField,  SIGNAL(valueChanged()), this, SLOT(timeValueChanged()) );
	connect(SdepField,  SIGNAL(edited_OK()), this, SLOT(SdepValueChanged()) );

	setData(dobj);
	editorType = "QbScalarEditor";

}


void QbScalarEditor::setData(QdObject_var dobj){
	QString txt;
	if( dobj.is_nil() ) return;
	QdRScalar_var Snarrow = QdRScalar_var::narrow(dobj);
   
	if( !Snarrow.is_nil() ){
		S_var = Snarrow;
		maxRec = 0;
		record = 0;
		double value = (double) *Snarrow;
		txt.setNum(value);
		valueField->setText(txt);
		          
		D0 = get_Depend_0(Snarrow);
		ttags = QdTimeSeq_var::narrow(D0);
		stags = QdRScalarSeq_var::narrow(D0);

		
		if ( D0.is_nil() ){
			showDep0Field(false);
		}
		else{
			showDep0Field(true);
			if ( D0->sequence_size() == 1 )
			{
				if(!ttags.is_nil()) timeField->setTime(ttags[0]);
				else if(!stags.is_nil()) SdepField->setDouble(stags[0]);
			}
			else
				QuMsgBar("Wrong number of Time tags", true);
		}
	}
	else
		QuMsgBar("Data object not recognised", true);

	((QHBoxLayout*)layout())->addStretch(1);
}

void QbScalarEditor::makeCopy(QdObject_var dobj){

	if( dobj.is_nil() ) return;
	QdRScalar_var Snarrow = QdRScalar_var::narrow(dobj);
   
	if( !Snarrow.is_nil() ){
		S_var = Factory::clone(Snarrow);
		S_var->copy_xrefs_from(dobj);
          
		QdObject_var D0_orig = get_Depend_0(S_var);
		if ( !D0_orig.is_nil() ){
			
			D0 = Factory::clone(D0_orig); // ensure we don't edit tags shared with another object
			D0->copy_xrefs_from(D0_orig);
			if( !D0.is_nil() ) S_var->change_xref(DEPEND_0, D0);
			ttags = QdTimeSeq_var::narrow(D0);
			stags = QdRScalarSeq_var::narrow(D0);
		}
		edited_var = S_var;

	}
	else
		QuMsgBar("Data object not recognised", true);

}


void QbScalarEditor::valueChanged(){
  
	bool OK;
	double val;
  
	if(QuSlotDebug())
	{
		printf("QbScalarEditor::valueChanged %s\n", valueField->text().toLatin1().data());
		fflush(stdout);
	}
 
	val = valueField->text().toDouble(&OK);
	if (OK){
		*S_var = val;
	}
}

void QbScalarEditor::timeValueChanged(){
 
	QDOS::QmjdTime tag=timeField->time();
	if(!ttags.is_nil() && ttags->sequence_size() > record) {
		ttags[0] = tag;
	}

}

void QbScalarEditor::SdepValueChanged(){
 
	double val = SdepField->value();
	if(!stags.is_nil() && stags->sequence_size() > record) {
		stags[0] = val;
	}

}

//-----------------------------------------------------------------------

QbScalarSeqEditor::QbScalarSeqEditor(QdObject_var dobj, QWidget *parent, const char *name) 
	: QbEditor(dobj, parent,name)
{
	setTitle("Scalar Sequence");
  
	valueField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_ANY, QS_ANY));
	connect(valueField, SIGNAL(edited_OK()), this, SLOT(valueChanged())  );
	connect(timeField,  SIGNAL(valueChanged()), this, SLOT(timeValueChanged()) );
	connect(SdepField,  SIGNAL(edited_OK()), this, SLOT(SdepValueChanged()) );

	setData(dobj);
	editorType = "QbScalarSeqEditor";

}

void QbScalarSeqEditor::updateForRec(){
	QString txt;
	double value = (double) Sseq_var[record];
	txt.setNum(value);
	valueField->setText(txt);
          
	if ( D0.is_nil() ){
		showDep0Field(false);
	}
	else{
		showDep0Field(true);
		if ( D0->sequence_size() > record )
		{
			if(!ttags.is_nil()) timeField->setTime(ttags[record]);
			else if(!stags.is_nil()) SdepField->setDouble(stags[record]);
		}
	}
}


void QbScalarSeqEditor::setData(QdObject_var dobj){
	QString txt;

	if( dobj.is_nil() ) return;
	QdRScalarSeq_var Sseqnarrow = QdRScalarSeq_var::narrow(dobj);
   
	if( !Sseqnarrow.is_nil() ){
		Sseq_var = Sseqnarrow;
		maxRec = Sseqnarrow->sequence_size() - 1;
		record = 0;
		double value = (double) Sseqnarrow[record];
		txt.setNum(value);
		valueField->setText(txt);
          
		          
		D0 = get_Depend_0(Sseqnarrow);
		ttags = QdTimeSeq_var::narrow(D0);
		stags = QdRScalarSeq_var::narrow(D0);
		
		
		if ( D0.is_nil() ){
			showDep0Field(false);
		}
		else{
			showDep0Field(true);
			if ( D0->sequence_size() == dobj->sequence_size() )
			{
				if(!ttags.is_nil()) timeField->setTime(ttags[record]);
				else if(!stags.is_nil()) SdepField->setDouble(stags[record]);
			}

			else
				QuMsgBar("Wrong number of timetags/Depend_0", true);
		}
	}
	else
		QuMsgBar("Data object not recognised", true);
   
	((QHBoxLayout*)layout())->addStretch(1);
}


void QbScalarSeqEditor::makeCopy(QdObject_var dobj){

	if( dobj.is_nil() ) return;
	QdRScalarSeq_var Sseqnarrow = QdRScalarSeq_var::narrow(dobj);
   
	if( !Sseqnarrow.is_nil() ){

		Sseq_var = Factory::clone(Sseqnarrow);
		Sseq_var->copy_xrefs_from(dobj);
          
		QdObject_var D0_orig = get_Depend_0(Sseq_var);
          
		if ( !D0_orig.is_nil() ){
			D0 = Factory::clone(D0_orig); // ensure we don't edit tags shared with another object
			D0->copy_xrefs_from(D0_orig);
			if( ! D0.is_nil() ) Sseq_var->change_xref(DEPEND_0, D0);
		}
		edited_var = Sseq_var;

	}
	else
		QuMsgBar("Data object not recognised", true);
   
}

void QbScalarSeqEditor::valueChanged(){

	bool OK;
	double val;
  
	if(QuSlotDebug())
	{
		printf("QbScalarSeqEditor::valueChanged %s\n", valueField->text().toLatin1().data());
		fflush(stdout);
	}

	val = valueField->text().toDouble(&OK);
	if (OK){
		Sseq_var[record] = val;
	}
}

void QbScalarSeqEditor::timeValueChanged(){

	QDOS::QmjdTime tag=timeField->time();
	if(!ttags.is_nil() && ttags->sequence_size() > record) {
		ttags[record] = tag;
	}


}


void QbScalarSeqEditor::SdepValueChanged(){
 
	double val = SdepField->value();
	if(!stags.is_nil() && stags->sequence_size() > record) {
		stags[0] = val;
	}

}

//-----------------------------------------------------------------------

QbTtagEditor::QbTtagEditor(QdObject_var dobj, QWidget *parent, const char *name ) 
	: QbEditor(dobj, parent,name)
{
	setTitle("Time Tags");
  
	valueField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
	connect(valueField, SIGNAL(edited_OK()), this, SLOT(valueChanged())  );
	setData(dobj);
	editorType = "QbTtagEditor";

}

void QbTtagEditor::updateForRec(){
	QString txt;
     
	valueField->setText(Ttag_var[record].getISOstring(0).c_str());
}


void QbTtagEditor::setData(QdObject_var dobj){

	if( dobj.is_nil() ) return;
	QdTimeSeq_var TTnarrow = QdTimeSeq_var::narrow(dobj);

	if( !TTnarrow.is_nil() ){
		Ttag_var = TTnarrow;
		maxRec = TTnarrow->sequence_size() - 1;
		record = 0;
		valueField->setText(TTnarrow[record].getISOstring(0).c_str());
     
		showDep0Field(false);
	}
	else
		QuMsgBar("Data object not recognised", true);
   
	((QHBoxLayout*)layout())->addStretch(1);
}


void QbTtagEditor::makeCopy(QdObject_var dobj){

	if( dobj.is_nil() ) return;
	QdTimeSeq_var TTnarrow = QdTimeSeq_var::narrow(dobj);
   
	if( !TTnarrow.is_nil() ){
		
		Ttag_var = Factory::clone(TTnarrow);
		Ttag_var->copy_xrefs_from(dobj);
     	edited_var = Ttag_var;

	}
	else
		QuMsgBar("Data object not recognised", true);
   
}

void QbTtagEditor::valueChanged(){

	QDOS::QmjdTime tag;

	if( tag.setFromISOstring(valueField->text().toLatin1().data() ) ){
		if(!Ttag_var.is_nil() && Ttag_var->sequence_size() > 0) {
			Ttag_var[record] = tag;
		}
	}
   
}

//-----------------------------------------------------------------------

QbTimeObjEditor::QbTimeObjEditor(QdObject_var dobj, QWidget *parent, const char *name ) 
	: QbEditor(dobj, parent,name)
{
	setTitle("Time Object");
  
	valueField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
	valueField->setMinimumWidth(250);
	valueField->adjustSize();
	connect(valueField, SIGNAL(edited_OK()), this, SLOT(valueChanged())  );
	connect(valueField, SIGNAL(TimeChosen()), this, SLOT(setValue())  );
	setData(dobj);
	editorType = "QbTimeObjEditor";

}

void QbTimeObjEditor::setData(QdObject_var dobj){

	if( dobj.is_nil() ) return;
	QdTime_var Tnarrow = QdTime_var::narrow(dobj);
   
	if( !Tnarrow.is_nil() ){
		T_var = Tnarrow;
		valueField->setText(Tnarrow->getISOstring(0).c_str());
     
		showDep0Field(false);
	}
	else
		QuMsgBar("Data object not recognised", true);
   
	((QHBoxLayout*)layout())->addStretch(1);
}

void QbTimeObjEditor::makeCopy(QdObject_var dobj){

	if( dobj.is_nil() ) return;
	QdTime_var Tnarrow = QdTime_var::narrow(dobj);
   
	if( !Tnarrow.is_nil() ){
		T_var = Factory::clone(Tnarrow);
		edited_var = T_var;

	}
	else
		QuMsgBar("Data object not recognised", true);
   
}

void QbTimeObjEditor::valueChanged(){

	QDOS::QmjdTime tag;

	if( tag.setFromISOstring( valueField->text().toLatin1().data() ) ){
		T_var = new QdTime (tag);
	}
    edited_var = T_var;
}

void QbTimeObjEditor::setValue(){

	QdTime_var tobj = QdTime_var::narrow(valueField->getSlotObject());
	if( !tobj.is_nil() ){

		T_var = Factory::clone(tobj);
		valueField->setText(T_var->getISOstring(0).c_str());
		edited_var = T_var;
	}
   
}

//-----------------------------------------------------------------------

QbMatrixEditor::QbMatrixEditor(QdObject_var dobj, QWidget *parent, const char *name ) 
	: QbEditor(dobj, parent,name)
{
	setTitle("Constant Matrix");
  
	connect(timeField,  SIGNAL(valueChanged()), this, SLOT(timeValueChanged()) );
	connect(SdepField,  SIGNAL(edited_OK()), this, SLOT(SdepValueChanged()) );

	editorType = "QbMatrixEditor";
  
	QdRMatrix_var Mnarrow = QdRMatrix_var::narrow(dobj);
	QdSMatrix_var SMnarrow = QdSMatrix_var::narrow(dobj);
   
	if(!Mnarrow.is_nil())
	{
		setTitle("Constant Numeric Matrix");

		// set condition for element slot
		valueField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));

		connect(valueField, SIGNAL(edited_OK()), this, SLOT(valueChanged())  );
	}
	else if(!SMnarrow.is_nil())
	{
		setTitle("Constant String Matrix");
		valueField->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
		connect(valueField, SIGNAL(edited_OK()), this, SLOT(valueChanged())  );
	}
  
	setData(dobj);
}


void QbMatrixEditor::setData(QdObject_var dobj){
	QString txt;

	if( dobj.is_nil() ) return;
   
	D0 = get_Depend_0(dobj);
	ttags = QdTimeSeq_var::narrow(D0);
	stags = QdRScalarSeq_var::narrow(D0);
	
	if ( D0.is_nil() ){
		showDep0Field(false);
	}
	else{
		showDep0Field(true);
		if ( D0->sequence_size() == 1 )
		{
			if(!ttags.is_nil()) timeField->setTime(ttags[0]);
			else if(!stags.is_nil()) SdepField->setDouble(stags[0]);
		}
		else
			QuMsgBar("Wrong number of Time tags", true);
	}

	QdRMatrix_var Mnarrow = QdRMatrix_var::narrow(dobj);
	QdSMatrix_var SMnarrow = QdSMatrix_var::narrow(dobj);
  
	if( !Mnarrow.is_nil() ){
		M_var = Mnarrow;
		dimSize = (Mnarrow->dims()).size();
		index.clear();
		for (int i=0; i<dimSize; i++){
			int maxN = (Mnarrow->dims())[i];
			int nextInd = 0;
			index.push_back(nextInd);
			argSlider * argS = new argSlider(tr("index %1").arg(i+1), i, maxN, this);
			connect(argS, SIGNAL(QbEchanged(int, int)), this, SLOT(changeElement(int, int)) );

			layout()->addWidget(argS);
		}

		double value = (double) Mnarrow->matrix_indexelt(index);
		txt.setNum(value);
		valueField->setText(txt);

	}
	else if( !SMnarrow.is_nil() ){
		SM_var = SMnarrow;
		dimSize = (SMnarrow->dims()).size();
		index.clear();
		for (int i=0; i<dimSize; i++){
			int maxN = (SMnarrow->dims())[i];
			int nextInd = 0;
			index.push_back(nextInd);
			argSlider * argS = new argSlider(tr("index %1").arg(i+1), i, maxN, this);
			connect(argS, SIGNAL(QbEchanged(int, int)), this, SLOT(changeElement(int, int)) );
			layout()->addWidget(argS);
			((QHBoxLayout*)layout())->addStretch(1);
		}

		string txtValue = SMnarrow->matrix_indexelt(index);
		valueField->setText(txtValue.c_str());
	}
	else
	{
		QuMsgBar("Data object not recognised", true);
		return;
	}
   
	((QHBoxLayout*)layout())->addStretch(1);
  
}

void QbMatrixEditor::makeCopy(QdObject_var dobj){

	if( dobj.is_nil() ) return;
   
		QdObject_var D0_orig = get_Depend_0(dobj);
		if ( !D0_orig.is_nil() ){
			
			D0 = Factory::clone(D0_orig); // ensure we don't edit tags shared with another object
			D0->copy_xrefs_from(D0_orig);
			ttags = QdTimeSeq_var::narrow(D0);
			stags = QdRScalarSeq_var::narrow(D0);
		}


	QdRMatrix_var Mnarrow = QdRMatrix_var::narrow(dobj);
	QdSMatrix_var SMnarrow = QdSMatrix_var::narrow(dobj);
  
	if( !Mnarrow.is_nil() ){
		
		M_var = Factory::clone(Mnarrow);
		M_var->copy_xrefs_from(dobj);
		if( !D0.is_nil() ) M_var->change_xref(DEPEND_0, D0);
		edited_var = M_var;

	}
	else if( !SMnarrow.is_nil() ){
		
		SM_var = Factory::clone(SMnarrow);
		SM_var->copy_xrefs_from(dobj);
		if( !D0.is_nil() ) SM_var->change_xref(DEPEND_0, D0);
		edited_var = SM_var;

	}
	else
	{
		QuMsgBar("Data object not recognised", true);
		return;
	}
     
}


void QbMatrixEditor::valueChanged(){

	if(!M_var.is_nil())
	{
		bool OK;
		double val;
   
		if(QuSlotDebug())
		{
			printf("QbMatrixEditor::valueChanged %s\n", valueField->text().toLatin1().data());
			fflush(stdout);
		}
 
		val = valueField->text().toDouble(&OK);
		if (OK){
			M_var->matrix_indexelt(index) = val;
		}
	}
	else if(!SM_var.is_nil())
	{
		string val=valueField->text().toLatin1().data();
		SM_var->matrix_indexelt(index) = val;
	}
}

void QbMatrixEditor::timeValueChanged(){
 
	QDOS::QmjdTime tag;
	tag=timeField->time();
	if(!ttags.is_nil() && ttags->sequence_size() > 0) {
		ttags[0] = tag;
	}
}

void QbMatrixEditor::SdepValueChanged(){
 
	double val = SdepField->value();
	if(!stags.is_nil() && stags->sequence_size() > record) {
		stags[0] = val;
	}

}


void QbMatrixEditor::changeElement(int id, int idx){
	QString txt;
   
	index[id] = idx;
   
	if( !M_var.is_nil() ){

		double value = (double) M_var->matrix_indexelt(index);
		txt.setNum(value);
		valueField->setText(txt);
	}
	else if( !SM_var.is_nil() ){

		string textValue = SM_var->matrix_indexelt(index);
		valueField->setText(textValue.c_str());
	}
	else
	{
		QuMsgBar("Data object not recognised", true);
		return;
	}
   
}


//-----------------------------------------------------------------------

QbMatrixSeqEditor::QbMatrixSeqEditor(QdObject_var dobj, QWidget *parent, const char *name ) 
	: QbEditor(dobj, parent,name)
{

	setTitle("Sequence of Arrays");
	QdRMatrixSeq_var Mseqnarrow = QdRMatrixSeq_var::narrow(dobj);
	QdSMatrixSeq_var SMseqnarrow = QdSMatrixSeq_var::narrow(dobj);
  
	if(!Mseqnarrow.is_nil())
	{
		valueField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_ANY, QS_ANY));
		connect(valueField, SIGNAL(edited_OK()), this, SLOT(valueChanged())  );
	}
	else if(!SMseqnarrow.is_nil())
	{
		valueField->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_ANY, QS_ANY));
		connect(valueField, SIGNAL(edited_OK()), this, SLOT(valueChanged())  );
	}
    
	connect(timeField,  SIGNAL(valueChanged()), this, SLOT(timeValueChanged()) );
	connect(SdepField,  SIGNAL(edited_OK()), this, SLOT(SdepValueChanged()) );

	editorType = "QbMatrixSeqEditor";
  
	setData(dobj);

}

void QbMatrixSeqEditor::updateForRec(){

	if(QuSlotDebug())
	{
		printf("QbMatrixSeqEditor::updateForRec %ld \n", (long)this);
		fflush(stdout);
	}

	QString txt;
	if(!Mseq_var.is_nil())
	{
		double value = (double) Mseq_var[record].matrix_indexelt(index);
		txt.setNum(value);
		valueField->setText(txt);
	}
	else if(!SMseq_var.is_nil())
	{
		string textValue=SMseq_var[record].matrix_indexelt(index);
		valueField->setText(textValue.c_str());
	}
        
	if ( D0.is_nil() ){
		showDep0Field(false);
	}
	else{
		showDep0Field(true);
		if ( D0->sequence_size() > record )
		{
			if(!ttags.is_nil()) timeField->setTime(ttags[record]);
			else if(!stags.is_nil()) SdepField->setDouble(stags[record]);
		}
	}
}


void QbMatrixSeqEditor::setData(QdObject_var dobj){
	QString txt;

	if( dobj.is_nil() ) return;
   
	D0 = get_Depend_0(dobj);
	ttags = QdTimeSeq_var::narrow(D0);
	stags = QdRScalarSeq_var::narrow(D0);
		
	if ( D0.is_nil() ){
		showDep0Field(false);
	}
	else{
		showDep0Field(true);
		if ( D0->sequence_size() == dobj->sequence_size() )
		{
			if(!ttags.is_nil()) timeField->setTime(ttags[record]);
			else if(!stags.is_nil()) SdepField->setDouble(stags[record]);
		}

		else
			QuMsgBar("Wrong number of timetags/Depend_0", true);
	}
	
	QdRMatrixSeq_var Mseqnarrow = QdRMatrixSeq_var::narrow(dobj);
	QdSMatrixSeq_var SMseqnarrow = QdSMatrixSeq_var::narrow(dobj);
        
	if( !Mseqnarrow.is_nil() ){
		Mseq_var = Mseqnarrow;
		dimSize = (Mseqnarrow[0].dims()).size();
		index.clear();
		for (int i=0; i<dimSize; i++){
			int maxN = (Mseqnarrow[0].dims())[i];
			int nextInd = 0;
			index.push_back(nextInd);
			argSlider *argS = new argSlider(tr("index %1").arg(i+1), i, maxN, this);
			connect(argS, SIGNAL(QbEchanged(int, int)), this, SLOT(changeElement(int,int)) );
			layout()->addWidget(argS);
		}
     
		maxRec = Mseqnarrow->sequence_size() - 1;
		record = 0;
		double value = (double) Mseqnarrow[record].matrix_indexelt(index);
		txt.setNum(value);
		valueField->setText(txt);
	}
	else if( !SMseqnarrow.is_nil() ){
		SMseq_var = SMseqnarrow;
		dimSize = (SMseqnarrow[0].dims()).size();
		index.clear();
		for (int i=0; i<dimSize; i++){
			int maxN = (SMseqnarrow[0].dims())[i];
			int nextInd = 0;
			index.push_back(nextInd);
			argSlider * argS = new argSlider(tr("index %1").arg(i+1), i, maxN, this);
			connect(argS, SIGNAL(QbEchanged(int,int)), this, SLOT(changeElement(int,int)) );
			layout()->addWidget(argS);
		}
    

		maxRec = SMseqnarrow->sequence_size() - 1;
		record = 0;
		string textValue = SMseqnarrow[record].matrix_indexelt(index);
		valueField->setText(textValue.c_str());
	}
	else
	{
		QuMsgBar("Data object not recognised", true);
		return;
	}
	((QHBoxLayout*)layout())->addStretch(1);
}


void QbMatrixSeqEditor::makeCopy(QdObject_var dobj){

	if( dobj.is_nil() ) return;
   
	QdObject_var D0_orig = get_Depend_0(dobj);
          
	if ( !D0_orig.is_nil() ){
		D0 = Factory::clone(D0_orig); // ensure we don't edit tags shared with another object
		D0->copy_xrefs_from(D0_orig);
	}

	QdRMatrixSeq_var Mseqnarrow = QdRMatrixSeq_var::narrow(dobj);
	QdSMatrixSeq_var SMseqnarrow = QdSMatrixSeq_var::narrow(dobj);
        
	if( !Mseqnarrow.is_nil() ){
    
		Mseq_var = Factory::clone(Mseqnarrow);

		Mseq_var->copy_xrefs_from(dobj);
		if( !D0.is_nil() ) Mseq_var->change_xref(DEPEND_0, D0);
		edited_var = Mseq_var;

	}
	else if( !SMseqnarrow.is_nil() ){
  
		SMseq_var = Factory::clone(SMseqnarrow);
		
		SMseq_var->copy_xrefs_from(dobj);
		if( !D0.is_nil() ) SMseq_var->change_xref(DEPEND_0, D0);
		edited_var = SMseq_var;
	}
	else
	{
		QuMsgBar("Data object not recognised", true);
		return;
	}

}

void QbMatrixSeqEditor::changeElement(int id, int idx){
	QString txt;
      
	index[id] = idx;
   
	if( !Mseq_var.is_nil() ){
          
		double value = (double) Mseq_var[record].matrix_indexelt(index);
		txt.setNum(value);
		valueField->setText(txt);
	}
	else if( !SMseq_var.is_nil() ){

		string textValue = SMseq_var[record].matrix_indexelt(index);
		valueField->setText(textValue.c_str());
	}
	else
	{
		QuMsgBar("Data object not recognised", true);
		return;
	}
   
}

void QbMatrixSeqEditor::valueChanged(){
    
	if(QuSlotDebug())
	{
		printf("QbMatrixSeqEditor::valueChanged %s\n", valueField->text().toLatin1().data());
		fflush(stdout);
	}
  
	if(!Mseq_var.is_nil())
	{
		bool OK;
		double val;
		val = valueField->text().toDouble(&OK);
		if (OK){
			Mseq_var[record].matrix_indexelt(index) = val;
		}
	}
	else if(!SMseq_var.is_nil())
	{
		string textValue;
		textValue = valueField->text().toLatin1().data();
		SMseq_var[record].matrix_indexelt(index) = textValue;
	}
}

void QbMatrixSeqEditor::timeValueChanged(){
 
	QDOS::QmjdTime tag;
	tag=timeField->time();
	if(!ttags.is_nil() && ttags->sequence_size() > record) {
		ttags[record] = tag;
        // object has DEPEND_0 tt_orig unless tt get changed
	}

}

void QbMatrixSeqEditor::SdepValueChanged(){
 
	double val = SdepField->value();
	if(!stags.is_nil() && stags->sequence_size() > record) {
		stags[0] = val;
	}

}

// text editor
//----------------

QbTextEditor::QbTextEditor(QdObject_var dobj, QWidget *parent, const char *name)
	: QbEditor(dobj, parent,name)
{
	setTitle("Text");
	valueLabel->hide();
	valueField->setMinimumWidth(250);
  
	valueField->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));

	connect(valueField, SIGNAL(edited_OK()), this, SLOT(valueChanged())  );

	connect(timeField,  SIGNAL(valueChanged()), this, SLOT(timeValueChanged()) );
	connect(SdepField,  SIGNAL(edited_OK()), this, SLOT(SdepValueChanged()) );

	editorType = "QbTextEditor";
	setData(dobj);

}


void QbTextEditor::setData(QdObject_var dobj){

	if( dobj.is_nil() ) return;
	QdString_var Strnarrow = QdString_var::narrow(dobj);
   
	if( !Strnarrow.is_nil() ){
		Str_var = Strnarrow;
		maxRec = 0;
		record = 0;
		valueField->setText(QString(Strnarrow->c_str()));
     
		int txt_len = valueField->text().length() * 7 + 20;
		if(txt_len < 450) valueField->setMinimumWidth(txt_len);
		else valueField->setMinimumWidth(450);
		valueField->setCursorPosition(0);
               
		D0 = get_Depend_0(dobj);
		ttags = QdTimeSeq_var::narrow(D0);
		stags = QdRScalarSeq_var::narrow(D0);
	
		if ( D0.is_nil() ){
			showDep0Field(false);
		}
		else{
			showDep0Field(true);
			if ( D0->sequence_size() == 1 )
			{
				if(!ttags.is_nil()) timeField->setTime(ttags[0]);
				else if(!stags.is_nil()) SdepField->setDouble(stags[0]);
			}
			else QuMsgBar("Wrong number of Time tags", true);
		}
	}
	else
		QuMsgBar("Data object not recognised", true);
        
	((QHBoxLayout*)layout())->addStretch(1);

}

void QbTextEditor::makeCopy(QdObject_var dobj){

	if( dobj.is_nil() ) return;
	QdString_var Strnarrow = QdString_var::narrow(dobj);
   
	if( !Strnarrow.is_nil() ){
		
		Str_var = Factory::clone(Strnarrow);
     
		Str_var->copy_xrefs_from(dobj);
          
		QdObject_var D0_orig = get_Depend_0(dobj);
		if ( !D0_orig.is_nil() ){
			
			D0 = Factory::clone(D0_orig); // ensure we don't edit tags shared with another object
			D0->copy_xrefs_from(D0_orig);
			ttags = QdTimeSeq_var::narrow(D0);
			stags = QdRScalarSeq_var::narrow(D0);
		}
		if( !D0.is_nil() ) Str_var->change_xref(DEPEND_0, D0);
		edited_var = Str_var;

	}
	else
		QuMsgBar("Data object not recognised", true);
        
}

void QbTextEditor::valueChanged(){
  
	bool OK;
	QdString_var new_str;
  
	new_str = new QdString(valueField->text().toLatin1().data());
	new_str->copy_xrefs_from(Str_var);
	edited_var = new_str;
  
}

void QbTextEditor::timeValueChanged(){
 
	QDOS::QmjdTime tag;
	tag=timeField->time();
	if(!ttags.is_nil() && ttags->sequence_size() > 0) {
		ttags[0] = tag;
	}

}

void QbTextEditor::SdepValueChanged(){
 
	double val = SdepField->value();
	if(!stags.is_nil() && stags->sequence_size() > record) {
		stags[0] = val;
	}

}

// text sequence editor
//----------------

QbTextSeqEditor::QbTextSeqEditor(QdObject_var dobj, QWidget *parent, const char *name ) 
	: QbEditor(dobj, parent,name)
{
	setTitle("Text Sequence");
	valueLabel->hide();
	valueField->setMinimumWidth(300);
	valueField->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_ANY, QS_ANY));

	connect(valueField, SIGNAL(edited_OK()), this, SLOT(valueChanged())  );
	connect(timeField,  SIGNAL(valueChanged()), this, SLOT(timeValueChanged()) );
	connect(SdepField,  SIGNAL(edited_OK()), this, SLOT(SdepValueChanged()) );

	editorType = "QbTextSeqEditor";
	setData(dobj);

}


void QbTextSeqEditor::setData(QdObject_var dobj){

	if( dobj.is_nil() ) return;
	QdStringSeq_var Strseqnarrow = QdStringSeq_var::narrow(dobj);
   
	if( !Strseqnarrow.is_nil() ){
		Strseq_var = Strseqnarrow;
		maxRec = Strseqnarrow->sequence_size() - 1;
		record = 0;
		valueField->setText(QString(Strseqnarrow[record].c_str()));

		int txt_len = valueField->text().length() * 7 + 20;
     
		if(txt_len < 450) valueField->setMinimumWidth(txt_len);
		else valueField->setMinimumWidth(450);
		valueField->setCursorPosition(0);
               
		D0 = get_Depend_0(dobj);
		ttags = QdTimeSeq_var::narrow(D0);
		stags = QdRScalarSeq_var::narrow(D0);
		
		if ( D0.is_nil() ){
			showDep0Field(false);
		}
		else{
			showDep0Field(true);
			if ( D0->sequence_size() == dobj->sequence_size() )
			{
				if(!ttags.is_nil()) timeField->setTime(ttags[record]);
				else if(!stags.is_nil()) SdepField->setDouble(stags[record]);
			}

			else
				QuMsgBar("Wrong number of timetags/Depend_0", true);
		}
	}
	else
		QuMsgBar("Data object not recognised", true);
        
	((QHBoxLayout*)layout())->addStretch(1);

}


void QbTextSeqEditor::makeCopy(QdObject_var dobj){

	if( dobj.is_nil() ) return;
	QdStringSeq_var Strseqnarrow = QdStringSeq_var::narrow(dobj);
   
	if( !Strseqnarrow.is_nil() ){
	
		Strseq_var = Factory::clone(Strseqnarrow);
     
		Strseq_var->copy_xrefs_from(dobj);
          
		QdObject_var D0_orig = get_Depend_0(dobj);
          
		if ( !D0_orig.is_nil() ){
			D0 = Factory::clone(D0_orig); // ensure we don't edit tags shared with another object
			D0->copy_xrefs_from(D0_orig);
			if( ! D0.is_nil() ) Strseq_var->change_xref(DEPEND_0, D0);
		}
		edited_var = Strseq_var;

	}
	else
		QuMsgBar("Data object not recognised", true);
        

}

void QbTextSeqEditor::updateForRec(){

	QString txt;


	txt = QString(Strseq_var[record].c_str());
	valueField->setText(txt);

	int txt_len = valueField->text().length() * 7 + 20;
	if(txt_len < 450) valueField->setMinimumWidth(txt_len);
	else valueField->setMinimumWidth(450);
	valueField->setCursorPosition(0);
                    
	if ( D0.is_nil() ){
		showDep0Field(false);
	}
	else{
		showDep0Field(true);
			if ( D0->sequence_size() > record )
			{
				if(!ttags.is_nil()) timeField->setTime(ttags[record]);
				else if(!stags.is_nil()) SdepField->setDouble(stags[record]);
			}
	}
}


void QbTextSeqEditor::valueChanged(){
  
	bool OK;
	QdString_var new_str;
  
	new_str = new QdString(valueField->text().toLatin1().data());
	Strseq_var[record] = *new_str;
  
}

void QbTextSeqEditor::timeValueChanged(){
 
	QDOS::QmjdTime tag;
	tag=timeField->time();
	if(!ttags.is_nil() &&  ttags->sequence_size() > record) {
		ttags[record] = tag;
	}


}

void QbTextSeqEditor::SdepValueChanged(){
 
	double val = SdepField->value();
	if(!stags.is_nil() && stags->sequence_size() > record) {
		stags[0] = val;
	}

}


// class argSlider

 argSlider::argSlider( QString title, int indx, int maxN, QWidget *parent, char * name)
	: QWidget(parent)
{

	maxNum = maxN-1;
	num = 0;
	id = indx;
   
	QGridLayout * layout=new QGridLayout(this);
	layout->setSpacing(0);
	layout->setMargin(0);
   
	QToolButton * leftSlide = new QToolButton(this);
	leftSlide->setArrowType(Qt::LeftArrow);
	leftSlide->setFixedSize(18, 20);
	layout->addWidget(leftSlide, 0,0);
   
	numField = new QguWLinput(this, "numField");
	numField->setAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));
	numField->setEnabled(true);
	numField->setText("1");
	numField->setFixedSize(40, 20);
	numField->setAlignment(Qt::AlignHCenter);
	layout->addWidget(numField, 0,1);
   
	QToolButton * rightSlide = new QToolButton(this);
	rightSlide->setArrowType(Qt::RightArrow);
	rightSlide->setFixedSize(18, 20);
	layout->addWidget(rightSlide, 0,2);
   
	QLabel * Spacer=new QLabel(this);
	Spacer->setFixedSize(18, 20);
	layout->addWidget(Spacer, 0,3);
   
	QLabel *sliderLabel = new QLabel(tr("in range [1,%1]").arg(maxNum+1), this);
	layout->addWidget(sliderLabel, 0, 4);
   
	layout->setColumnStretch(4, 1);
   
	connect( rightSlide, SIGNAL(clicked()), this, SLOT(incRec()) );
	connect( leftSlide,  SIGNAL(clicked()), this, SLOT(decRec()) );
	connect( numField,   SIGNAL(edited_OK()), this, SLOT(adjustRec()) );
   
}

argSlider::~argSlider(){
	delete numField;
}

void argSlider::incRec(){
	if(QuSlotDebug())
	{
		printf("argSlider::incRec %ld\n", (long)this);
		fflush(stdout);
	}
   
	num += 1;
	numField->setText(tr("%1").arg(num+1));
	adjustRec();  // done explicitly as textChanged() is not emitted if slot does not have focus
}

void argSlider::decRec(){
	if(QuSlotDebug())
	{
		printf("argSlider::decRec %ld\n", (long)this);
		fflush(stdout);
	}
    
	num -= 1;
	numField->setText(tr("%1").arg(num+1));
	adjustRec(); // done explicitly as textChanged() is not emitted if slot does not have focus
}

void argSlider::adjustRec(){
	if(QuSlotDebug())
	{
		printf("argSlider::adjustRec %ld\n", (long)this);
		fflush(stdout);
	}

	bool OK;
	num = numField->text().toInt(&OK);
	num -=1; // as feedback goes 1-N while code goes 0-N-1
	if(OK){
		if(num < 0) num = 0;
		if(num > maxNum) num = maxNum;
		numField->setText(tr("%1").arg(num+1));
		emit QbEchanged(id, num);
	}
	else
		numField->setText("1");
	emit QbEchanged(id, num);
}
