//  qbu.cc
//
// Data Browser Widget
//
// July 2002: Tony Allen
//

#include <stdlib.h>
#include "qt_version.h"

#include "qt_version.h"

#include "qgu.h"
#include "qbu.h"

#include "qdutil.h"
#include "qwui.h"

extern qwuiWindow* qsasWindow;
extern void QdaLaunchWindow(QString,QWidget*);


XrefNameEditor::XrefNameEditor(QObject *parent):
	QItemDelegate(parent)
{
	viewer = dynamic_cast<QbObjViewer*>(parent);
}

QWidget * XrefNameEditor::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
	QLineEdit *e=new QLineEdit(parent);
	e->setFrame(false);
	return e;
}

void XrefNameEditor::setEditorData(QWidget *editor, const QModelIndex &index) const
{
	QString value = index.model()->data(index, Qt::DisplayRole).toString();

	QLineEdit *edit = static_cast<QLineEdit*>(editor);
	edit->setText(value);
	
	// get object and current name for saving in setModelData()
	QdObject_var xrefObj;
	try{
		xrefObj = viewer->activeEditor->edited_var->get_xref(value.toLatin1().data());
		viewer->activeEditor->edited_var->change_xref("nameBeingChanged", xrefObj);
		viewer->activeEditor->edited_var->delete_xref(value.toLatin1().data());
	}
	catch(Exception &e)
	{
	std::cout << "exception in setEditorData" << std::endl;
		return;
	}

}
		
void XrefNameEditor::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
{

	QLineEdit *edit = static_cast<QLineEdit*>(editor);
	QString value = edit->text();

	model->setData(index, value, Qt::EditRole);	

	QdObject_var xrefObj;
	try{
		xrefObj = viewer->activeEditor->edited_var->get_xref("nameBeingChanged");
		viewer->activeEditor->edited_var->change_xref(value.toLatin1().data(), xrefObj);
		viewer->activeEditor->edited_var->delete_xref("nameBeingChanged");
	}
	catch(Exception &e)
	{
	std::cout << "exception in setModelData" << std::endl;
		return;
	}
	
	viewer->refreshList();
}

void XrefNameEditor::updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &/*index*/) const
{
	editor->setGeometry(option.rect);
}


XrefListItem::XrefListItem(XrefListView* parent, const QString& text):
	QListWidgetItem(text, parent)
{
	setFlags (flags () | Qt::ItemIsEditable);
}


XrefListView::XrefListView(QWidget* parent):
	QListWidget(parent)
{
	setDragEnabled(true);
	viewport()->setAcceptDrops(true);
	setDropIndicatorShown(true);

	//setDragDropMode(QAbstractItemView::InternalMove);
	setDragDropMode(QAbstractItemView::DragDrop);
	setSelectionMode(QAbstractItemView::ExtendedSelection);
		
	delegate=new XrefNameEditor(parent);
	setItemDelegate(delegate);

	#if QT_VERSION >= 0x040300
	setEditTriggers(QAbstractItemView::NoEditTriggers);
	#endif

}

QMimeData* XrefListView::mimeData ( const QList<QListWidgetItem *> items ) const 
{
	QMimeData *mimeData = new QMimeData();
	
	QStringList idList;

	foreach (QListWidgetItem* item, items) {
		idList << item->text();
	}
	
	QString data=idList.join("//");

	mimeData->setData("application/xrefitems", data.toLatin1().data());
	mimeData->setText(items.first()->text());
	return mimeData;

}
 QStringList XrefListView::mimeTypes() const
{
	QStringList types;
	types << "application/xrefitems";
	return types;
}


void XrefListView::dragEnterEvent(QDragEnterEvent* event)
{
	if (event->mimeData()->hasFormat("application/wlitems"))
	{
		event->acceptProposedAction();
		activateWindow();
	}
}

void XrefListView::dropEvent(QDropEvent* event)
{
	if(!event->mimeData()->hasFormat("application/wlitems"))
		return;

	QStringList itemList=QString(event->mimeData()->data("application/wlitems")).split("//");

	foreach(QString item, itemList)
	{
		QStringList name_decomp=item.split("/");
		QString name=name_decomp[name_decomp.size()-1];

		if( name != viewer->objName() ) // stop drop of obj on its own xref list
		{
			QdObject_var xref_obj = QuGetObjByName(item);
			if(!xref_obj.is_nil())
			{
				try {
					viewer->activeEditor->edited_var->change_xref(name.toLatin1().data(), xref_obj);
				}
				catch(Exception &e){
					QuMsgBar(e.msg().c_str(), false);
					std::cout << e.msg().c_str() << std::endl;
				}
			}
		}
	}

	viewer->refreshList();
}


DataBrowseWindow::DataBrowseWindow(QdObject_var dobj, QString WLcontext,
				   QWidget *parent, const char *name )
	: QMainWindow(0)
{

	QSizePolicy fixed(QSizePolicy::Fixed,QSizePolicy::Fixed);

	this->setPalette(qApp->palette());

	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
	
	browser =0;
	browserVB=new QWidget(this);

	QVBoxLayout * browserVBLayout=new QVBoxLayout;
	
	// Name panel
	
	name_panel=new QGroupBox;
	QHBoxLayout * name_panel_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Name", name_panel);
	name_panel_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->lineBox->setReadOnly(true);
	name_panel_layout->addWidget(name_field);
	
	extractBtn = new QPushButton( "Extract...");
	connect( extractBtn, SIGNAL(clicked()), this, SLOT(extract()) );
	extractBtn->setEnabled(true);
	
	name_panel_layout->addWidget(extractBtn);

	
	name_panel->setLayout(name_panel_layout);
	browserVBLayout->addWidget(name_panel);
	

	// Record number panel
	
	recSlider=new QGroupBox;
	QHBoxLayout * recSliderLayout=new QHBoxLayout;
	
	QToolButton * leftEnd = new QToolButton;
	leftEnd->setText("|");
	leftEnd->setFixedSize(20,21);
	recSliderLayout->addWidget(leftEnd);
	
	QToolButton * leftSlide = new QToolButton( recSlider);
	leftSlide->setArrowType(Qt::LeftArrow);
	leftSlide->setFixedSize(20,21);
	recSliderLayout->addWidget(leftSlide);
	
	recBar = new QSlider(Qt::Horizontal, recSlider);
	recSliderLayout->addWidget(recBar);
	
	QToolButton * rightSlide = new QToolButton( recSlider);
	rightSlide->setArrowType(Qt::RightArrow);
	rightSlide->setFixedSize(20,21);
	recSliderLayout->addWidget(rightSlide);
	
	QToolButton * rightEnd = new QToolButton( recSlider);
	rightEnd->setText("|");
	rightEnd->setFixedSize(20,21);
	recSliderLayout->addWidget(rightEnd);
	
	recField = new QguWLinput(recSlider,"recField");
	recField->setAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));
	recField->setEnabled(true);
	recField->setFixedWidth(60);
	recField->setAlignment(Qt::AlignHCenter);
	recSliderLayout->addWidget(recField);
	
	connect( rightSlide, SIGNAL(clicked()), this, SLOT(incRec()) );
	connect( leftSlide,  SIGNAL(clicked()), this, SLOT(decRec()) );
	connect( rightEnd, SIGNAL(clicked()), this, SLOT(lastRec()) );
	connect( leftEnd,  SIGNAL(clicked()), this, SLOT(firstRec()) );
	connect( recBar,   SIGNAL(valueChanged(int)), this, SLOT(adjustRec(int)) );
	connect( recField, SIGNAL(edited_OK()), this, SLOT(inputRec()) );
	
	recSlider->setLayout(recSliderLayout);
	browserVBLayout->addWidget(recSlider);
	
	
	// Object view widget
	

	// Buttons layout
	
	buttons_groupbox=new QGroupBox;
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	editBtn = new QPushButton( "Allow Editing");
	editBtn->setMinimumSize(editBtn->sizeHint());
	connect( editBtn, SIGNAL(clicked()), this, SLOT(allowEdit()) );
	editBtn->setEnabled(true);
	editBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(editBtn, 5);
	editBtn->setToolTip( "Copies data object and enables editing");
	
	
	saveEditsBtn = new QPushButton( "Save Edits");
	saveEditsBtn->setMinimumSize(saveEditsBtn->sizeHint());
	connect( saveEditsBtn, SIGNAL(clicked()), this, SLOT(saveEdits()) );
	saveEditsBtn->setEnabled(false);
	saveEditsBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveEditsBtn, 5);
	saveEditsBtn->setToolTip("Saves result of editing (will over-write object of same name)");
	
	pad = new QLabel("                     ");
	buttons_layout->addWidget(pad, 5);
	pad->setVisible(false); // used only in time interval and event table editors
	
	dismissBtn = new QPushButton( "    Dismiss    ");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	dismissBtn->setDefault(true);
	buttons_layout->addWidget(dismissBtn, 3, Qt::AlignRight);
	
	buttons_groupbox->setLayout(buttons_layout);
	
	setObject(dobj, WLcontext);
	
	//Menu
	
	editMenu = new QMenu( this );
	menuBar()->addMenu(editMenu );
	editMenu->setTitle("  Edit ");
	editMenu->addAction("New Metadata", browser, SLOT( newXref() ) );
	editMenu->addAction("Attach Metadata", browser, SLOT( attachXref() ) );
	editMenu->addAction("Add time tags", browser, SLOT( addTimeTags() ));
	editMenu->addSeparator();
	editMenu->addAction("Delete Metadata", browser, SLOT( deleteXref() ), Qt::CTRL+Qt::Key_X );
   
   
	menuBar()->addSeparator();
	QMenu * help = new QMenu( this );
	menuBar()->addMenu( help );
	help->setTitle("  Help ");
	help->addAction("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	help->addSeparator();
	help->addAction("...Data Import", this, SLOT( launchManBrowser() ));
	help->addSeparator();
	help->addAction("Module Version", this, SLOT( launchVersion() ));
	
	browserVBLayout->addWidget(browser);
	browserVBLayout->addWidget(buttons_groupbox);
	browserVB->setLayout(browserVBLayout);
	
	setCentralWidget(browserVB);

	item=qsasWindow->wlview->wlmodel()->itemFromPath(WLcontext);
	
}

void DataBrowseWindow::keyPressEvent(QKeyEvent * e )
{
  // use escape key to toggle default buttons
	if(e->key() == Qt::Key_Escape){
		if( editBtn->isDefault()  ) {
			if( editBtn->isEnabled()) {
				editBtn->setDefault(false);
				saveEditsBtn->setDefault(true);
			}
			else{
				editBtn->setDefault(false);
				dismissBtn->setDefault(true);
			}
		}
		else if(saveEditsBtn->isDefault() ) {
			saveEditsBtn->setDefault(false);
			dismissBtn->setDefault(true);
		}
		else if(dismissBtn->isDefault()  )  {
			if(editBtn->isEnabled()){
				dismissBtn->setDefault(false);
				editBtn->setDefault(true);
			}
			else if(saveEditsBtn->isEnabled()){
				dismissBtn->setDefault(false);
				saveEditsBtn->setDefault(true);
			}
		}
		return;
	}
  // use return or enter key to perform default
	if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
		if(editBtn->isDefault()) allowEdit();
		else if (saveEditsBtn->isDefault()) saveEdits();
		else if (dismissBtn->isDefault()) dismiss();
		return;
	}

  // else pass on to list view 
	e->ignore(); // let selection change

}


DataBrowseWindow::~DataBrowseWindow(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::~DataBrowseWindow %ld\n", (long)this);
		fflush(stdout);
	}
 
 
//  delete recField;
	if(browser != 0) delete browser;
}

void DataBrowseWindow::dismiss(){
	dismissBtn->setEnabled(false);
	close();
}

void DataBrowseWindow::launchManual(){ 

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::launchManual %ld\n", (long)this);
		fflush(stdout);
	}
 
	qsasWindow->help_window->manEntry("index.html");
}

void DataBrowseWindow::launchManBrowser(){ 

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::launchManBrowser %ld\n", (long)this);
		fflush(stdout);
	}
 
	qsasWindow->help_window->manEntry("QSAS_data_handling.html");
}


void DataBrowseWindow::launchVersion(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::launchVersion %ld\n", (long)this);
		fflush(stdout);
	}
 
  
	QString text;
	text  = QString("Version Numbers...\n");
	text += QString(QSAS_VERSION_ID);
	text += "\n Data Object Browser  ";
	text += QString(QBU_VERSION);
	QMessageBox::information( this, "QSAS information", text.toLatin1().data(), "OK" );

}

void DataBrowseWindow::incRec(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::incRec %ld\n", (long)this);
		fflush(stdout);
	}
 
	if(browser->activeEditor->record < browser->activeEditor->maxRec){
		browser->activeEditor->record +=1;
	}
	else{
		browser->activeEditor->record = browser->activeEditor->maxRec;
	}
	recBar->setValue(browser->activeEditor->record);

}

void DataBrowseWindow::decRec(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::decRec %ld\n", (long)this);
		fflush(stdout);
	}
 
	if(browser->activeEditor->record > 0){
		browser->activeEditor->record -= 1;
	}
	else{
		browser->activeEditor->record = 0;
	}
	recBar->setValue(browser->activeEditor->record);
}

void DataBrowseWindow::lastRec(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::lastRec %ld\n", (long)this);
		fflush(stdout);
	}
 
 
	browser->activeEditor->record = browser->activeEditor->maxRec;
	recBar->setValue(browser->activeEditor->record);

}

void DataBrowseWindow::firstRec(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::firstRec %ld\n", (long)this);
		fflush(stdout);
	}
 
 
	browser->activeEditor->record = 0;
	recBar->setValue(browser->activeEditor->record);
}

void DataBrowseWindow::inputRec(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::inputRec %ld\n", (long)this);
		fflush(stdout);
	}
 
	bool ok;
	int rec = recField->text().toInt(&ok);
	if(ok){
		if(rec < 0) rec = 0;
		if(rec > browser->activeEditor->maxRec)  rec = browser->activeEditor->maxRec;
		recField->setText(tr("%1").arg(rec)); // reset field in case changed, does not cause re-validation
		browser->activeEditor->record = rec;
		recBar->setValue(rec);
		browser->activeEditor->updateForRec();
	}
}
 
void DataBrowseWindow::adjustRec(int rec){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::adjustRec %ld %d\n", (long)this, rec);
		fflush(stdout);
	}
 
 
	browser->activeEditor->record = rec;
	recField->setText(tr("%1").arg(rec));
	browser->activeEditor->updateForRec();
  
}

void DataBrowseWindow::extract(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::extract %ld\n", (long)this);
		fflush(stdout);
	}
	
	// ask user for selection
	DataSliceWindow *SliceWindow = new DataSliceWindow(0);
	if(parentBrowser)
	{
		QdObject_var x_obj;
		try
		{
			x_obj = parentObject->get_xref(editObjectName.toLatin1().data());
		}
		catch(Exception &e)
		{
			msgBar->showMessage(e.msg().c_str(), 10000);
			return;
		}
		QString tempName = QString("Temp/")+editObjectName;
		QwuiPutOnWL(x_obj, tempName);
		SliceWindow->setObject(tempName.toLatin1().data());
	}
	else
	{
			SliceWindow->setObject(editObjectName.toLatin1().data());
	}
	SliceWindow->show();

}

void DataBrowseWindow::allowSave(bool allow){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::allowSave %ld\n", (long)this);
		fflush(stdout);
	}
  
	if(browser == 0 || browser->activeEditor == 0) return;
 
	if(allow)
	{
		// enable save buttons

		this->saveEditsBtn->setEnabled(true);
		this->extractBtn->setEnabled(false);
		this->dismissBtn->setDefault(false);
		this->saveEditsBtn->setDefault(true);
	
		// enable slot edits
		this->name_field->setReadOnly(false);
		this->name_field->lineBox->selectAll();
		// give name slot focus
		this->name_field->lineBox->setFocus(Qt::OtherFocusReason);
		this->browser->activeEditor->setReadOnly(false);
	}
	else
	{
	    // dis-enable save buttons

		this->saveEditsBtn->setEnabled(false);
		this->extractBtn->setEnabled(true);
		this->dismissBtn->setDefault(true);
		this->saveEditsBtn->setDefault(false);
	
		// dis-enable slot edits
		this->name_field->setReadOnly(true);
		this->name_field->lineBox->deselect();
	
		this->browser->activeEditor->setReadOnly(true);
	}
}



void DataBrowseWindow::dragEnterEvent(QDragEnterEvent *event)
{
	if(QuSlotDebug()){ printf("DataBrowseWindow::dragEnterEvent %ld\n", (long)this); fflush(stdout); }
	this->activateWindow();
	this->raise();
}

void DataBrowseWindow::saveEdits(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::saveEdits %ld\n", (long)this);
		fflush(stdout);
	}
	
	if(name_field->text().isEmpty()) {
		msgBar->showMessage("Name field is empty");
		return;
	}
	
	QdObject_var new_obj = browser->getUpdate();
	QString newName = name_field->text();

	if( parentObject.is_nil() )
	{
		// editing object from working list
		if(originalName != name_field->text() || item == 0)
		{
			// save as new name
			QdTimeInterval_var t_out = QdTimeInterval_var::narrow(new_obj);
			QdTimeIntervalSeq_var evttbl_out=QdTimeIntervalSeq_var::narrow(new_obj);
     
			if ( !t_out.is_nil() || !evttbl_out.is_nil()){
				newName=QString("Time Intervals/")+newName;
			}
			else if( new_obj->sequence_size() <= 1 ){
				newName=QString("Constants/")+newName;
			}
			else{
				newName=QString("Data/")+newName;
			}
		
			QwuiPutOnWL(new_obj, newName);
		
			this->dismiss();

			//originalName = name_field->text();
			//editObjectName = newName;
		}
		else
		{
		// replace dobj
			QuReplaceObject(editObjectName, new_obj);
			emit objChanged(editObjectName);
			qsasWindow->wlview->update();
			
			this->dismiss();

		}
		
		
	}
	else {

		// xref object
		parentObject->change_xref(name_field->text().toLatin1().data(), new_obj);
		if(parentBrowser)
		{
			parentBrowser->browser->refreshList();
			emit objChanged(parentBrowser->editObjectName);
		}
		
		// close since Xref being edited
		this->dismiss();
		return;
	}
	
	// unless TI or event list editor, reset edit off
	this->disallowEdit();
	
	return;
}


void DataBrowseWindow::allowEdit()
{
	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::allowEdit %ld\n", (long)this);
		fflush(stdout);
	}
	
	if(browser == 0 || browser->activeEditor == 0) return;
	
	// enable save button and slots

	this->allowSave(true);
	
	// copy object
	
	browser->activeEditor->makeCopy(browser->activeEditor->edited_var);
	
	// set button inactive until save made
	this->editBtn->setEnabled(false);
	

}

void DataBrowseWindow::disallowEdit()
{
	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::disallowEdit %ld\n", (long)this);
		fflush(stdout);
	}
	
	// disable save button and slots

	this->allowSave(false);
	
	// set button active 
	this->editBtn->setEnabled(true);
	
}

void DataBrowseWindow::setObject(QdObject_var dobj, QString WLcontext){

	editObjectName = WLcontext;
  
	browser = new QbObjViewer(this, dobj, browserVB, WLcontext);
	if( browser == 0 || browser->activeEditor == 0){
		return;
	}
  
  // ask qsas main window to ask qda to open a file (for Windows port)
	connect(browser, SIGNAL(launchQdaWindow(const char *)), qsasWindow, SIGNAL(openQdaWindow(const char *)));
    
	msgBar->showMessage(WLcontext);
	QuMsgBar("", false);
  
  
  // initialise slider to first record
	browser->activeEditor->record = 0;
	recField->setText("0");
  
	recBar->setRange(0, browser->activeEditor->maxRec);
	int step = browser->activeEditor->maxRec / 10;
	if (step < 1) step = 1;
	recBar->setPageStep(step);

	QdTimeIntervalSeq_var events = QdTimeIntervalSeq_var::narrow(dobj);
	QdTimeInterval_var ti = QdTimeInterval_var::narrow(dobj);
	
	if(events.is_nil() && browser->activeEditor->maxRec > 0)  recSlider->setEnabled(true);
	else {
	
		// single object or event list with own bar
		recSlider->setEnabled(false);
		recSlider->setVisible(false);
		if( !events.is_nil()){
			// time interval or evant table (have own widgets)
			name_panel->setVisible(false);
			editBtn->setVisible(false);
			saveEditsBtn->setVisible(false);
			pad->setVisible(true);
		}
	}
	
	
  // set Xrefs
	vector<string> xrefs;
	dobj->list_xref_names(xrefs);
	for(int i=0; i<xrefs.size(); ++i)
	{
		new XrefListItem(browser->XrefList, QString(xrefs[i].c_str()));
	}

	browser->activeEditor->setReadOnly(true);

}



// generic viewer widget class

QbObjViewer::QbObjViewer(DataBrowseWindow * pWindow, QdObject_var dobj, QWidget *parent, QString name)
	: QGroupBox(parent)
{
	QdObject_var nilObj;
	activeEditor = 0;
	parentBrowser = pWindow;
	
	QHBoxLayout * viewerLayout=new QHBoxLayout;

	QdTimeInterval_var TI_in = QdTimeInterval_var::narrow(dobj);
	QdTimeIntervalSeq_var TIS_in = QdTimeIntervalSeq_var::narrow(dobj);
	QdTimeSeq_var Ttag_in =  QdTimeSeq_var::narrow (dobj);
	QdRScalar_var S_in = QdRScalar_var::narrow(dobj);
	QdTime_var T_in = QdTime_var::narrow(dobj);
	QdRScalarSeq_var Sseq_in = QdRScalarSeq_var::narrow(dobj);
	QdRMatrix_var M_in = QdRMatrix_var::narrow(dobj);
	QdRMatrixSeq_var Mseq_in = QdRMatrixSeq_var::narrow(dobj);
	QdString_var Str_in = QdString_var::narrow(dobj);
	QdStringSeq_var Strseq_in = QdStringSeq_var::narrow(dobj);
	QdSMatrix_var SM_in = QdSMatrix_var::narrow(dobj);
	QdSMatrixSeq_var SMseq_in = QdSMatrixSeq_var::narrow(dobj);
  
	// is this a time interval object?
	if( !TI_in.is_nil() ){
		TI_window = new QbTimeIntervalEditor(TI_in, this, "Time Interval");
		TI_window->adjustSize();
		TI_window->show();
		activeEditor = TI_window;
		viewerLayout->addWidget(TI_window);
	//	activeEditor->edited_var=TI_window->_time_interval_var;
		
		// allow edits and disable edit button (i.e. click on edit button now)
		pWindow->allowEdit();
	}
  
	//  time interval sequence object?  
	else if( !TIS_in.is_nil() ){
		TIS_window = new QbEventTableEditor(TIS_in, this, name);
		TIS_window->adjustSize();
		TIS_window->show();
		activeEditor = TIS_window;
		viewerLayout->addWidget(TIS_window);
	//	activeEditor->edited_var=TIS_window->_event_table_var;
	}
  
	else if(!Ttag_in.is_nil()){
		Ttag_window = new QbTtagEditor(Ttag_in, this, "Time Line");
		Ttag_window->show();
		activeEditor = Ttag_window;
		viewerLayout->addWidget(Ttag_window);
	}
  
	// is this a time const object?
	else if( !T_in.is_nil() ){
		T_window = new QbTimeObjEditor(T_in, this, "Constant");

		T_window->show();
		activeEditor = T_window;
		viewerLayout->addWidget(T_window);
	}
  
	// is this a scalar sequence object?
	else if( !Sseq_in.is_nil() ){
		Sseq_window = new QbScalarSeqEditor(Sseq_in, this, "Sequence");

		Sseq_window->show();
		activeEditor = Sseq_window;
		viewerLayout->addWidget(Sseq_window);

	}

	// is this a scalar const object?
	else if( !S_in.is_nil() ){
		S_window = new QbScalarEditor(S_in, this, "Constant");
		S_window->show();
		activeEditor = S_window;
		viewerLayout->addWidget(S_window);
	}

	// is this a Matrix sequence object?
	else if( !Mseq_in.is_nil() ){
		Mseq_window = new QbMatrixSeqEditor(Mseq_in, this, "MSequence");
    
		Mseq_window->show();
		activeEditor = Mseq_window;
		viewerLayout->addWidget(Mseq_window);
	}
  
	// is this a Matrix object?
	else if( !M_in.is_nil() ){
		M_window = new QbMatrixEditor(M_in, this, "Matrix");

		M_window->show();
		activeEditor = M_window;
		viewerLayout->addWidget(M_window);
	}
  
	// is this a text sequence object?
	else if( !Strseq_in.is_nil() ){
		Strseq_window = new QbTextSeqEditor(Strseq_in, this, "TextSequence");
    
		Strseq_window->show();
		activeEditor = Strseq_window;
		viewerLayout->addWidget(Strseq_window);
	}
  
	// is this a text const object?
	else if( !Str_in.is_nil() ){
		Str_window = new QbTextEditor(Str_in, this, "Text");
    
		Str_window->show();
		activeEditor = Str_window;
		viewerLayout->addWidget(Str_window);
	}
 
	// is this a String Matrix sequence object?
	else if( !SMseq_in.is_nil() ){
		SMseq_window = new QbMatrixSeqEditor(SMseq_in, this, "StringMatrixSequence");
    
		SMseq_window->show();
		activeEditor = SMseq_window;
		viewerLayout->addWidget(SMseq_window);
	}
  
	// Is this a String Matrix?
	else if( !SM_in.is_nil() ){
		SM_window = new QbMatrixEditor(SM_in, this, "StringMatrix");

		SM_window->show();
		activeEditor = SM_window;
		viewerLayout->addWidget(SM_window);
	}
	  
	if (activeEditor == 0) {
		QuMsgBar("object type not supported", true);
		return;
	}
	activeEditor->edited_var = dobj;
	
	QWidget *Attribute_box = new QWidget(this);
	QVBoxLayout * Attribute_box_layout=new QVBoxLayout;
	Attribute_box_layout->setSpacing(2);

	XrefList=new XrefListView(this);
	XrefList->setMinimumHeight(200);
	XrefList->setMinimumWidth(160);
	XrefList->viewer = this;
	connect(XrefList, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(xrefEdit(QListWidgetItem*)));
	connect(qsasWindow->wlview->wlmodel(), SIGNAL(xrefDropDetected(const QString&)), this, SLOT(copyXrefToWL(const QString &)));
	connect(qsasWindow->wlview->wlmodel(), SIGNAL(refreshBrowser()), this, SLOT(refreshList()));


	parentsBtn = new QPushButton( " View Source Files... ", Attribute_box );
	parentsBtn->setMinimumSize(parentsBtn->sizeHint());
	connect( parentsBtn, SIGNAL(clicked()), this, SLOT(openParentFile()) );

	renameBtn = new QPushButton( " Rename Attribute ", Attribute_box);
	renameBtn->setMinimumSize(renameBtn->sizeHint());
	connect( renameBtn, SIGNAL(clicked()), this, SLOT(renameAttribute()) );

	QLabel* attributesLabel=new QLabel("Attributes", this);
	attributesLabel->setAlignment(Qt::AlignCenter);
	QFont f=attributesLabel->font();
	f.setWeight(QFont::Bold);
	attributesLabel->setFont(f);
	Attribute_box_layout->addWidget(attributesLabel, Qt::AlignCenter);
	Attribute_box_layout->addWidget(XrefList);
	Attribute_box_layout->addWidget(renameBtn);
	Attribute_box_layout->addWidget(parentsBtn);
	Attribute_box->setLayout(Attribute_box_layout);
    
	viewerLayout->addWidget(Attribute_box);
	
	if(!TI_in.is_nil() || !TIS_in.is_nil())
	{
		Attribute_box->hide();
	}
    
	fileList = new QMenu(this);

	setLayout(viewerLayout);
	adjustSize();
}


void QbObjViewer::copyXrefToWL(const QString &attrName){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::copyXrefToWL '%s'\n", attrName.toLatin1().data());
		fflush(stdout);
	}
	if(activeEditor->edited_var.is_nil()) return;
	QdObject_var xref_obj;
	try{
		xref_obj = activeEditor->edited_var->get_xref(attrName.toLatin1().data());
	}
	catch(Exception &e)
	{
        QuMsgBar( e.msg().c_str(), attrName.toLatin1().data(), true);
		return;
	}
	QdObject_var new_obj = Factory::clone(xref_obj);
	new_obj->copy_xrefs_from(xref_obj);
	
	// save to WL
	QdTimeInterval_var t_out = QdTimeInterval_var::narrow(new_obj);
	QdTimeIntervalSeq_var evttbl_out=QdTimeIntervalSeq_var::narrow(new_obj);
	
	QString newName = attrName;
	if(newName.startsWith('_'))
	{
		// WL objects must not contain reserved chars [,],{,}
		newName=QString("Cache_Obj");
	}
	
	if ( !t_out.is_nil() || ! evttbl_out.is_nil()){
		newName=QString("Time Intervals/")+newName;
	}
	else if( new_obj->sequence_size() <= 1 ){
		newName=QString("Constants/")+newName;
	}
	else{
		newName=QString("Data/")+newName;
	}
		
	QwuiPutOnWL(new_obj, newName);
	
}

void QbObjViewer::refreshList(){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::refreshList %ld\n", (long)this);
		fflush(stdout);
	}

	XrefList->clear();
	if( activeEditor == NULL || activeEditor->edited_var.is_nil() ) return;

   // reset Xrefs
	vector<string> xrefs;
	activeEditor->edited_var->list_xref_names(xrefs);

	for(int i=0; i<xrefs.size(); ++i)
	{
		new XrefListItem(XrefList, QString(xrefs[i].c_str()));
	}
   
}



QdObject_var QbObjViewer::getUpdate(){
  
	return activeEditor->edited_var;
  
}

void QbObjViewer::xrefEdit ( QListWidgetItem *item )
{

	XrefListItem* xitem=(XrefListItem*)(item);
	if(QuSlotDebug())
	{
		printf("QbObjViewer::xrefEdit %ld, %ld\n", (long)this, (long)item);
		fflush(stdout);
	}
 

	DataBrowseWindow *QBROWSER_widget_xref=0;
    
	if(item == 0) return;
	
	QdObject_var xref_obj=activeEditor->edited_var->get_xref(xitem->text().toLatin1().data());
	if(xref_obj.is_nil())
	{
		QuMsgBar("Unknown attribute ", xitem->text().toLatin1().data(), true);
		return;
	}
  
	QBROWSER_widget_xref = QbuLaunchBrowse(activeEditor->edited_var,
					       parentBrowser, xitem->text().toLatin1().data(),
							       xref_obj,
									       QString(item->text().toLatin1().data()));

  // browser is not modal, so code falls through to here immediately
  // and connect is available before "save" is pressed
     
	if (QBROWSER_widget_xref == 0 ) return;
  
	connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this,  SLOT(refreshList()) );
		    
}

void QbObjViewer::addTimeTags(){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::addTimeTags %ld\n", (long)this);
		fflush(stdout);
	}
 
 	parentBrowser->allowEdit();
  
	if(activeEditor == 0 || activeEditor->edited_var.is_nil() ) {
		QuMsgBar("Object does not exist", true);
		return;
	}

	if(activeEditor->edited_var->xref_exists(DEPEND_0) ){
		QuMsgBar("Time Tags already exist", true);
		return;
	}
  
	int n_recs = activeEditor->edited_var->sequence_size();
  
	QdTimeSeq_var T_var = new QdTimeSeq(n_recs, QmjdTime(2001, 9, 4));
    
	activeEditor->dep0Box->show();

	activeEditor->timeField->setTime( T_var[0] );

	activeEditor->edited_var->change_xref(DEPEND_0, T_var);
	
	parentBrowser->saveEdits();

}

void QbObjViewer::renameAttribute(){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::renameAttribute %ld\n", (long)this);
		fflush(stdout);
	}

	XrefList->setCurrentIndex(XrefList->currentIndex());
	XrefList->edit(XrefList->currentIndex());

}

void QbObjViewer::openParentFile(){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::openParentFile %ld\n", (long)this);
		fflush(stdout);
	}
 
  
	QdStringSeq_var parentsXref;
	if(activeEditor == 0) return;
  
	if( (activeEditor->edited_var).is_nil() ) {
		QuMsgBar("No variable edited", true);
		return;
	}
	try{
		parentsXref = QdStringSeq_var::narrow(activeEditor->edited_var->get_xref("Parents"));
	}
	catch(Exception &e){
		QuMsgBar( e.msg().c_str(), true );
		return;
	}
  
	if ( parentsXref.is_nil() ) return;
  
	fileList->clear();
	int listSize = parentsXref->sequence_size();
	for (int i=0; i<listSize; i++){
		fileList->addAction( QString(parentsXref[i].c_str()));
	}
	fileList->addSeparator();
	fileList->addAction( QString("Cancel") );
	
	QAction* act=fileList->exec(QCursor::pos());
	if(act==NULL) return;
	if(act->text()==QString("Cancel")) return;
	
	emit launchQdaWindow(act->text().toLatin1().data());
}


void QbObjViewer::attachXref()
{

	if(QuSlotDebug())
	{
		printf("QbObjViewer::attachXref %ld\n", (long)this);
		fflush(stdout);
	}
	
  	parentBrowser->allowEdit();

	QbuAttachWindow * attachWindow = new QbuAttachWindow(this, "attachXrefChooser");
	attachWindow->dobj = activeEditor->edited_var;
	attachWindow->thisViewer = this;
	attachWindow->show();
}


void QbObjViewer::deleteXref()
{

	if(QuSlotDebug())
	{
		printf("QbObjViewer::deleteXref %ld\n", (long)this);
		fflush(stdout);
	}
  
  	parentBrowser->allowEdit();

	QList<QListWidgetItem *> items=XrefList->selectedItems();
	if(items.isEmpty()) return;
	foreach(QListWidgetItem* item, items)
	{
		try
		{
			activeEditor->edited_var->delete_xref(item->text().toLatin1().data() );
		}
		catch(Exception &e){
			QuMsgBar(e.msg().c_str(), false);
			return;
		}
	}
  
	refreshList();
 
}

void QbObjViewer::newXref()
{

	if(QuSlotDebug())
	{
		printf("QbObjViewer::newXref %ld\n", (long)this);
		fflush(stdout);
	}
 // allow edit would result in user having to click for save attr and save dobj
//	parentBrowser->allowEdit();
	
	static QPushButton *Sbtn;
	static QPushButton *Mbtn;
	static QPushButton *Vbtn;
	static QPushButton *tbtn;
	static QPushButton *Tbtn;
	static QPushButton *Txtbtn;
	static QPushButton *TxtArrbtn;
	static QWidget * typeBox;
   
	objTypeSelector = new QDialog(0);
	typeBox = new QWidget(objTypeSelector);

	QVBoxLayout * typeBoxLayout=new QVBoxLayout;
	typeBoxLayout->setMargin(QSAS_profiles->current()->GUImargins);
	typeBoxLayout->setSpacing(QSAS_profiles->current()->GUImargins);

	Txtbtn = new QPushButton(" Text ", typeBox); typeBoxLayout->addWidget(Txtbtn);
	TxtArrbtn = new QPushButton(" Text Array", typeBox); typeBoxLayout->addWidget(TxtArrbtn);
	Tbtn = new QPushButton(" Time Object ", typeBox); typeBoxLayout->addWidget(Tbtn);
	tbtn = new QPushButton(" Time Interval  ", typeBox);     typeBoxLayout->addWidget(tbtn);
	Sbtn = new QPushButton(" Scalar Constant  ", typeBox); typeBoxLayout->addWidget(Sbtn);
	Mbtn = new QPushButton(" Matrix Constant  ", typeBox); typeBoxLayout->addWidget(Mbtn);
	Vbtn = new QPushButton(" Vector Constant  ", typeBox); typeBoxLayout->addWidget(Vbtn);
	connect(Txtbtn,SIGNAL(clicked()),this,SLOT(createText()));
	connect(TxtArrbtn,SIGNAL(clicked()),this,SLOT(createTextArr()));
	connect(Tbtn,SIGNAL(clicked()),this,SLOT(createTime()));
	connect(tbtn,SIGNAL(clicked()),this,SLOT(createTInterval()));
	connect(Sbtn,SIGNAL(clicked()),this,SLOT(createConstant()));
	connect(Mbtn,SIGNAL(clicked()),this,SLOT(createArray()));
	connect(Vbtn,SIGNAL(clicked()),this,SLOT(createVConstant()));
	typeBox->setLayout(typeBoxLayout);
	typeBox->adjustSize();
     
	objTypeSelector->show();
}


void QbObjViewer::createText ( ){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::createText %ld\n", (long)this);
		fflush(stdout);
	}
 
 
	DataBrowseWindow *QBROWSER_widget_xref;
	QdObject_var dobj;
	QdString_var sobj = new QdString("");
  
	dobj = sobj;

	objTypeSelector->close();

	QBROWSER_widget_xref = QbuLaunchBrowse(activeEditor->edited_var,
					       parentBrowser, "new", dobj, "new");

  // browser is not modal, so code falls through to here immediately
  // and connect is available before "save" is pressed
  
	if(QBROWSER_widget_xref == 0 ) return;
  
	QBROWSER_widget_xref->allowEdit();
	connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this, SLOT(refreshList()) );

}


void QbObjViewer::createTime(){ 

	if(QuSlotDebug())
	{
		printf("QbObjViewer::createTime %ld\n", (long)this);
		fflush(stdout);
	}
 
	QdTime_var dobj;
	DataBrowseWindow *QBROWSER_widget_xref;
  
	dobj = new QdTime("2001-09-05 00:00:00.000");
	objTypeSelector->close();
  
	QdString_var SI_ptr = new QdString("1>s");
	dobj->change_xref(SI_CONVERSION, SI_ptr);
	QBROWSER_widget_xref = QbuLaunchBrowse(activeEditor->edited_var,
					       parentBrowser, "newTime", dobj, "newTime");

  // browser is not modal, so code falls through to here immediately
  // and connect is available before "save" is pressed
  
	if(QBROWSER_widget_xref == 0 ) return;
  
	QBROWSER_widget_xref->allowEdit();
	connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this, SLOT(refreshList()) );
  
}


void QbObjViewer::createTInterval ( ){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::createTInterval %ld\n", (long)this);
		fflush(stdout);
	}
 
 
	printf("type not yet supported\n");
	fflush(stdout);
	objTypeSelector->close();

}


void QbObjViewer::createVConstant ( ){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::createVConstant %ld\n", (long)this);
		fflush(stdout);
	}
 
 
	Dim mDim = Dim(3);
  
	QdRMatrix_var dobj = new QdRMatrix(mDim, 0);
	QdString_var Frame_ptr = new QdString("vector>gse_xyz");
	dobj->change_xref(FRAME, Frame_ptr);
  
	DataBrowseWindow *QBROWSER_widget_xref = QbuLaunchBrowse(activeEditor->edited_var,
			parentBrowser, "newVector", dobj, "newVector");

  // browser is not modal, so code falls through to here immediately
  // and connect is available before "save" is pressed
  
	if(QBROWSER_widget_xref == 0 ) return;
  
	QBROWSER_widget_xref->allowEdit();
	connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this, SLOT(refreshList()) );
 
	objTypeSelector->close();

}

void QbObjViewer::createArray(){ 

	if(QuSlotDebug())
	{
		printf("QbObjViewer::createArray %ld\n", (long)this);
		fflush(stdout);
	}
   
	objTypeSelector->close();
  
	arrTypeSelector = new arrDialog(this, "arrayDimChooser", this, true);


	arrTypeSelector->arrType = QB_DOUBLE;
	arrTypeSelector->show();
}



const char * QbObjViewer::objName(){ 
   
	return parentBrowser->name_field->text().toLatin1().data();
  
}


void QbObjViewer::createTextArr(){ 

	if(QuSlotDebug())
	{
		printf("QbObjViewer::createTextArr %ld\n", (long)this);
		fflush(stdout);
	}
   
	objTypeSelector->close();
  
	arrTypeSelector = new arrDialog(this, "arrayDimChooser", this, true);


	arrTypeSelector->arrType = QB_STRING;
	arrTypeSelector->show();
}


void QbObjViewer::createConstant ( ){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::createConstant %ld\n", (long)this);
		fflush(stdout);
	}
 
 
	DataBrowseWindow *QBROWSER_widget_xref;
	QdRScalar_var dobj;
	dobj = new QdRScalar(0.0);

	objTypeSelector->close();

	QBROWSER_widget_xref = QbuLaunchBrowse(activeEditor->edited_var,
					       parentBrowser, "new",dobj, "new");

  // browser is not modal, so code falls through to here immediately
  // and connect is available before "save" is pressed
  
	if (QBROWSER_widget_xref == 0) return;
  
	QBROWSER_widget_xref->allowEdit();
	connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this, SLOT(refreshList()) );

}


DataBrowseWindow * QbuLaunchBrowse(QdObject_var source,  QWidget *parent, const char * objectName, QdObject_var dobj, QString WLcontext, bool edit)
{
	DataBrowseWindow *QBROWSER_widget;
      
	QBROWSER_widget = new DataBrowseWindow(dobj, WLcontext, parent, "editor");

	QBROWSER_widget->originalName = objectName;
	QBROWSER_widget->name_field->setText(objectName);
	if( QBROWSER_widget->browser == 0 || QBROWSER_widget->browser->activeEditor == 0) {
		QBROWSER_widget->close();
		return 0;
	}

	 QBROWSER_widget->parentObject = source;


	if( source.is_nil() ){
		QBROWSER_widget->setWindowTitle("Object Editor");
		QBROWSER_widget->browser->parentsBtn->setEnabled(true);
		QBROWSER_widget->move(30,30);
		QBROWSER_widget->parentBrowser = 0;
	}
	else{
     // source means launched from a xref on another edit window
		QBROWSER_widget->setWindowTitle("Metadata Editor");
		QBROWSER_widget->browser->parentsBtn->setEnabled(false);
		QBROWSER_widget->move(30,250);
		QBROWSER_widget->parentBrowser = dynamic_cast <DataBrowseWindow*> (parent);
	}
  
  	if(edit) QBROWSER_widget->allowEdit();

	QBROWSER_widget->show();
 
	return QBROWSER_widget;
 
}

QbuAttachWindow::QbuAttachWindow(QbObjViewer *parent, const char *name)
	:  QDialog(parent)
{

	QVBoxLayout * centreBox;
	QGroupBox * typeBox;
	QWidget * btnBox;
	QLabel *slotLabel;
	QLabel *slotSpace;
	QLabel *ObjSlotLabel;
	QLabel *infoSlot;
	QPushButton *cancelBtn;
	QPushButton *acceptBtn;
  
	centreBox = new QVBoxLayout(this);
	typeBox = new QGroupBox(" Choose Name and Object " , this);
	QVBoxLayout * typeBoxLayout=new QVBoxLayout;
	typeBox->setMinimumWidth(250);
	typeBoxLayout->setMargin(4);
       
  	ObjSlotLabel = new QLabel(typeBox);
	ObjSlotLabel->setText(" Drop object to attach ");
	typeBoxLayout->addWidget(ObjSlotLabel);
  
	object_field  = new QguWLinput(typeBox, "object_field");
	object_field->setAcceptable(Qcondition(QS_WL_OBJ));
	object_field->setMinimumWidth(150);
	typeBoxLayout->addWidget(object_field);
  
	slotSpace = new QLabel(typeBox);
	slotSpace->setText("  ");
	typeBoxLayout->addWidget(slotSpace);
  
	slotLabel = new QLabel(typeBox);
	slotLabel->setText(" Metadata Name ");
	typeBoxLayout->addWidget(slotLabel);
  
	name_field  = new QguWLinput(typeBox, "nameField");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setText("new");
	name_field->setMinimumWidth(150);
	typeBoxLayout->addWidget(name_field);
  
	infoSlot = new QLabel(typeBox);
	infoSlot->setText("( Drag straight to metadata List attaches with same name as on WL)");
	typeBoxLayout->addWidget(infoSlot);
  
	connect(object_field,SIGNAL(edited_OK()),this, SLOT(updateName()));

	btnBox = new QWidget(this);
	QHBoxLayout * btnLayout=new QHBoxLayout(btnBox);
	btnLayout->setSpacing(QSAS_profiles->current()->GUImargins);

	btnLayout->setContentsMargins(0,0,0,0);

	cancelBtn = new QPushButton("Cancel", btnBox);
	acceptBtn = new QPushButton("Create...", btnBox);

	btnBox->layout()->addWidget(cancelBtn);
	btnBox->layout()->addWidget(acceptBtn);

	connect(cancelBtn,SIGNAL(clicked()),this,SLOT(close()));
	connect(acceptBtn,SIGNAL(clicked()),this,SLOT(attachObject()));
	centreBox->addWidget(typeBox);
	centreBox->addWidget(btnBox);
	
	typeBox->setLayout(typeBoxLayout);
	typeBox->adjustSize();
  
}
  
void QbuAttachWindow::updateName(){
	  
	QString newText = QString(object_field->textObject().toLatin1().data());
	int ptr = newText.lastIndexOf( "/");
	if (ptr >= 0) newText.remove(0, ptr+1);
     
	// disallow drop of itself into list
	if (thisViewer->objName() == newText ) {
		QuMsgBar(" drop of object as its own xref", true);
		object_field->setText("self-link");
		object_field->reValidate();
	} // stop drop of obj on its own xref list
	else{
		name_field->setText(newText);
		name_field->reValidate();
		QuMsgBar(" ", false);
	}
}
  
void  QbuAttachWindow::attachObject(){
	try{
		dobj->change_xref(name_field->text().toLatin1().data(), object_field->getSlotObject() );
	}
	catch (Exception &e){
		QuMsgBar(e.msg().c_str(), true);
	}

	thisViewer->refreshList();
	this->close();

}

  arrDialog::arrDialog(QWidget *parent, const char *name, QbObjViewer* browser, bool modal)
	:  QDialog(parent)
{
	this->setModal(modal);
	parentViewer = browser;
	dimSlot = 0;
	QVBoxLayout * centreBox;
	QGroupBox * dimBox;

	folderPath=QString("Constants");
	
	QWidget * btnBox;

	QLabel *slotLabel;
	QPushButton *cancelBtn;
	QPushButton *acceptBtn;

	centreBox = new QVBoxLayout(this);
	dimBox = new QGroupBox(" Choose Array Dimensions ", this);
	QVBoxLayout * dimBoxLayout=new QVBoxLayout;
	dimBox->setMinimumWidth(250);
	dimBoxLayout->setMargin(QSAS_profiles->current()->GUImargins);
	
	dimSlot = new QguWLinput(dimBox, "dimSlot");
	dimSlot->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	dimSlot->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ARRAY, QS_ANY, QS_ANY));
	dimBoxLayout->addWidget(dimSlot);

	slotLabel = new QLabel(dimBox);
	slotLabel->setText("drop object with dimensions \n    or \ntype in array sizes, e.g. 3, 16, 36  or  3, 3");
	dimBoxLayout->addWidget(slotLabel);
	
	btnBox = new QWidget(this);
	QHBoxLayout * btnLayout=new QHBoxLayout(btnBox);
#if QT_VERSION < 0x040300
	btnLayout->setMargin(0);
#else
	btnLayout->setContentsMargins(0,0,0,0);
#endif
	cancelBtn = new QPushButton("Cancel", btnBox);
	acceptBtn = new QPushButton("Create...", btnBox);
	btnLayout->addWidget(cancelBtn);
	btnLayout->addWidget(acceptBtn);
	acceptBtn->setDefault(true);

	connect(cancelBtn,SIGNAL(clicked()), this, SLOT(close()));
	connect(acceptBtn,SIGNAL(clicked()), this, SLOT(editArray()));
	
	centreBox->addWidget(dimBox);
	centreBox->addWidget(btnBox);
	dimBox->setLayout(dimBoxLayout);
	dimBox->adjustSize();
}



// ---------------------------------------------------

void arrDialog::editArray(){ 

	if(QuSlotDebug())
	{
		printf("arrDialog::editArray %ld\n", (long)this);
		fflush(stdout);
	}
 
	QdObject_var target = dimSlot->getSlotObject();
	if( target.is_nil() ){
		QuMsgBar("Bad Dimension Input", true);
		return;
	}
  
	QdRMatrix_var m_obj = QdRMatrix_var::narrow(target);
	QdRMatrixSeq_var ms_obj = QdRMatrixSeq_var::narrow(target);
	QdString_var str_obj = QdString_var::narrow(target);
	Dim mDim;
	if( !m_obj.is_nil() ) {
		mDim = m_obj->dimdata();
	}
	else if (!ms_obj.is_nil()){
		mDim = ms_obj[0].dimdata();
    
	}
	else if( !str_obj.is_nil()) {
		QString dimStr = QString( str_obj->c_str()) ;
		int rankM = 0;
		vector <int> dimList;
		int ptr = dimStr.indexOf(",");
		QString token = QString(dimStr.toLatin1().data());
		if( ptr > 0) { // last entry has np comma
			token.truncate(ptr);
			dimStr.remove(0, ptr+1);
		}
		else dimStr = QString("stop");
	
		bool OK = true;
		while (OK){
	   // handle 5 dims for now
			int sizeN = token.toInt(&OK);
			if (!OK) break;
			dimList.push_back(sizeN);
			rankM++;
			ptr = dimStr.indexOf(",");
			token = QString(dimStr.toLatin1().data());
			if( ptr > 0) {	// last entry has no comma
				token.truncate(ptr);
				dimStr.remove(0, ptr+1);
			}
			else dimStr = QString("stop");
		}
	
		if(rankM == 0){
			QuMsgBar("Bad Dimension Input string", true);
			return;
		}
	
		mDim = Dim(dimList);

	}
	else{
		QuMsgBar("Input not matrix type or text", true);
		return;
	}

	DataBrowseWindow *QBROWSER_widget=0;
	DataBrowseWindow *parentBrowser=0;
	if (arrType == QB_DOUBLE){
  
		QdRMatrix_var dobj = new QdRMatrix(mDim, 0);
		//QString context("Constants/newArray");
		QdObject_var parentObj;
		if( parentViewer != 0 )
		{
			parentObj = parentViewer->activeEditor->edited_var;
			parentBrowser = parentViewer->parentBrowser;
		}
     
		QdString_var SI_ptr = new QdString("1>?");
		dobj->change_xref(SI_CONVERSION, SI_ptr);
  
		QBROWSER_widget = QbuLaunchBrowse(parentObj, parentBrowser, "", dobj, "");
		QBROWSER_widget->name_field->setText("newArray"); // set here so that it doesn't think object exists on WL
	}
	else if (arrType == QB_STRING){
  
		QdSMatrix_var dobj = new QdSMatrix(mDim, " ");

	//	QString context("Constants/newTextArray");

		QdObject_var parentObj;
		if( parentViewer != 0 ) 
		{
			parentObj = parentViewer->activeEditor->edited_var;
			parentBrowser = parentViewer->parentBrowser;			
		}
  
		QBROWSER_widget = QbuLaunchBrowse(parentObj, parentBrowser, "", dobj, "");
		QBROWSER_widget->name_field->setText("newTextArray"); // set here so that it doesn't think object exists on WL
	}
	
    if (QBROWSER_widget != 0) 
	{
		if(parentViewer != 0){
		
			connect(QBROWSER_widget, SIGNAL(objChanged(const QString&)), parentViewer,  SLOT(refreshList()) );
  
		}
		QBROWSER_widget->allowEdit();
		this->hide();
		QBROWSER_widget->activateWindow();
		this->setParent(QBROWSER_widget); //so deleting browser deletes this, (close() would set main window as active)
	}
	else this->close();

}

