#include "PlotConfig.h"


PlotConfig::PlotConfig()
{
	QGridLayout * mainLayout=new QGridLayout(this);
	
	QGroupBox * dropGroup=new QGroupBox(this);
	dropGroup->setTitle("Drag and drop default behaviour");
	QGridLayout * dropLayout=new QGridLayout(dropGroup);
	
	QLabel * tempLabel=new QLabel("Default when dropping a 3-vector:", dropGroup);

	tempLabel->setWordWrap(true); 

	dropLayout->addWidget(tempLabel, 0, 0, 1, 2);
	
	vectorCombo=new QComboBox(dropGroup);

	vectorCombo->addItem("1 panel: [R]");
	vectorCombo->addItem("1 panel: [R,X,Y,Z]");
	vectorCombo->addItem("1 panel: [X,Y,Z]");
	vectorCombo->addItem("2 panels: [R] [X,Y,Z]");
	vectorCombo->addItem("2 panels: [R] [Lat,Phi]");
	vectorCombo->addItem("2 panels: [R] [Theta,Phi]");
	vectorCombo->addItem("3 panels: [X] [Y] [Z]");
	vectorCombo->addItem("3 panels: [R] [Lat] [Phi]");
	vectorCombo->addItem("3 panels: [R] [Theta] [Phi]");
	vectorCombo->addItem("4 panels: [R] [X] [Y] [Z]");

	
	dropLayout->addWidget(vectorCombo, 1, 0);
		
	tempLabel=new QLabel("Default reducing 3D array scalar (eg trace):", dropGroup);
	tempLabel->setWordWrap(true);
	dropLayout->addWidget(tempLabel, 3, 0, 1, 2);
	
	arrayCombo0D=new QComboBox(dropGroup);

	arrayCombo0D->addItem("Sum of components");
	arrayCombo0D->addItem("Average of components");
	arrayCombo0D->addItem("Other/No default behaviour");

	dropLayout->addWidget(arrayCombo0D, 4, 0);

	connect(arrayCombo0D, SIGNAL(currentIndexChanged(int)), this, SLOT(arrayToScalarChanged(int)));

	arrayLine0D=new QLineEdit(dropGroup);
	arrayLine0D->setToolTip("Either SSS (sum), AAA (averaging)\nor an explicit string");
	dropLayout->addWidget(arrayLine0D, 4, 1);

	tempLabel=new QLabel("Default reducing 3D array to 1D (eg spectrum or XY trace):", dropGroup);
	tempLabel->setWordWrap(true);
	dropLayout->addWidget(tempLabel, 5, 0, 1, 2);
	
	arrayCombo1D=new QComboBox(dropGroup);

	arrayCombo1D->addItem("Sum over components 1 & 2");
	arrayCombo1D->addItem("Sum over components 1 & 3");
	arrayCombo1D->addItem("Sum over components 2 & 3");
	arrayCombo1D->addItem("Preset: PEACE 3DRL data (sum)");
	arrayCombo1D->addItem("Preset: PEACE 3DRL data (averaging)");
	arrayCombo1D->addItem("Preset: CIS-CODIF energies (sum)");
	arrayCombo1D->addItem("Other/No default behaviour");

	dropLayout->addWidget(arrayCombo1D, 6, 0);

	connect(arrayCombo1D, SIGNAL(currentIndexChanged(int)), this, SLOT(arrayChanged(int)));

	
	arrayLine1D=new QLineEdit(dropGroup);
	arrayLine1D->setToolTip("Either a summation over two components (SSi, SiS or iSS)\nor an explicit string - some presets are provided");
	dropLayout->addWidget(arrayLine1D, 6, 1);
	

	
	tempLabel=new QLabel("Default reducing 3D array to 2D (e.g. XY spectrum):", dropGroup);
	tempLabel->setWordWrap(true);
	dropLayout->addWidget(tempLabel, 7, 0, 1, 2);
	
	arrayCombo2D=new QComboBox(dropGroup);

	arrayCombo2D->addItem("Sum over component 1");
	arrayCombo2D->addItem("Sum over component 2");
	arrayCombo2D->addItem("Sum over component 3");
	arrayCombo2D->addItem("Preset: PEACE 3DR energies (sum)");
	arrayCombo2D->addItem("Preset: PEACE 3DR energies (averaging)");
	arrayCombo2D->addItem("Preset: CIS-CODIF energies (sum)");
	arrayCombo2D->addItem("Other/No default behaviour");

	dropLayout->addWidget(arrayCombo2D, 8, 0);

	connect(arrayCombo2D, SIGNAL(currentIndexChanged(int)), this, SLOT(arrayChangedXY(int)));

	
	arrayLine2D=new QLineEdit(dropGroup);
	arrayLine2D->setToolTip("Either a summation/Average over a component (iiS, iiA, Sii etc) \nor an explicit string - some presets are provided\n Always uses record 0");
	dropLayout->addWidget(arrayLine2D, 8, 1);
	

	mainLayout->addWidget(dropGroup, 0, 0);
	
	QGroupBox * plotGroup=new QGroupBox(this);
	plotGroup->setTitle("Default plot settings");
	QGridLayout * plotLayout=new QGridLayout(plotGroup);
	
	plotLayout->addWidget(new QLabel("Time labels format:", plotGroup), 0, 0);
	timeRepCombo=new QComboBox(plotGroup);

	timeRepCombo->addItem("Colon-separated: H:M:S");
	timeRepCombo->addItem("Hour/Min/Sec: HhMmSs");

	plotLayout->addWidget(timeRepCombo, 0, 1);
	
	mainLayout->addWidget(plotGroup, 1, 0);
	
	QGroupBox * pageGroup=new QGroupBox(this);
	pageGroup->setTitle("Default page settings");
	QGridLayout * pageLayout=new QGridLayout(pageGroup);
	
	pageLayout->addWidget(new QLabel("Size:", pageGroup), 0, 0, Qt::AlignRight);
	
	pageSizeCombo=new QComboBox(pageGroup);

	pageSizeCombo->addItem("A4");
	pageSizeCombo->addItem("Custom");
	pageSizeCombo->addItem("Executive");
	pageSizeCombo->addItem("Legal");
	pageSizeCombo->addItem("Letter");
	pageSizeCombo->addItem("Default");

	pageLayout->addWidget(pageSizeCombo, 0, 1);

	connect(pageSizeCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(pageSizeChanged(int)));

	
	pageLayout->addWidget(new QLabel("Orientation:", pageGroup), 0, 3, Qt::AlignRight);
	
	orientationCombo=new QComboBox(pageGroup);

	orientationCombo->addItem("Portrait");
	orientationCombo->addItem("Landscape");

	pageLayout->addWidget(orientationCombo, 0, 4);
	
	pageLayout->addWidget(new QLabel("Resolution:", pageGroup), 0, 5, Qt::AlignRight);
	
	pageDPICombo=new QComboBox(pageGroup);

	pageDPICombo->addItem("72 dpi");
	pageDPICombo->addItem("150 dpi");
	pageDPICombo->addItem("300 dpi");
	pageDPICombo->addItem("600 dpi");

	pageLayout->addWidget(pageDPICombo, 0, 6);
	
	connect(pageDPICombo, SIGNAL(currentIndexChanged(int)), this, SLOT(pageDPIChanged(int)));
	
	pageLayout->addWidget(new QLabel("Width:", pageGroup), 1, 0, Qt::AlignRight);
	
	widthLine=new QLineEdit(pageGroup);
	widthLine->setValidator(new QDoubleValidator(widthLine));
	pageLayout->addWidget(widthLine, 1, 1);
		
	pageLayout->addWidget(new QLabel("Height:", pageGroup), 1, 2, Qt::AlignRight);
	
	heightLine=new QLineEdit(pageGroup);
	heightLine->setValidator(new QDoubleValidator(heightLine));
	pageLayout->addWidget(heightLine, 1, 3);
	
	unitsCombo=new QComboBox(pageGroup);

	unitsCombo->addItem("pixels");
	unitsCombo->addItem("inches");
	unitsCombo->addItem("cm");
	unitsCombo->addItem("mm");
	unitsCombo->addItem("PS points");

	pageLayout->addWidget(unitsCombo, 1, 4);
	
	connect(unitsCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(unitsChanged(int)));

	kprinterCheck=new QCheckBox("Print manager: use kprinter rather than lpr (Linux only):", pageGroup);

	pageLayout->addWidget(kprinterCheck, 2, 0, 1, 4);

#if defined(WINDOWS) || defined(Q_OS_MACX)
	kprinterCheck->setEnabled(false);
#endif
		
	mainLayout->addWidget(pageGroup, 2, 0);
	
	previous_units=0;
	previous_dpi=72.;
}

void PlotConfig::updateProfile(QProfile * p)
{

	switch(vectorCombo->currentIndex())
	{
		case 1:
			p->Vector3D="RXYZ";
			break;
		case 2:
			p->Vector3D="XYZ";
			break;
		case 3:
			p->Vector3D="R_XYZ";
			break;
		case 4:
			p->Vector3D="R_LP";
			break;
		case 5:
			p->Vector3D="R_TP";
			break;
		case 6:
			p->Vector3D="X_Y_Z";
			break;
		case 7:
			p->Vector3D="R_L_P";
			break;
		case 8:
			p->Vector3D="R_T_P";
			break;
		case 9:
			p->Vector3D="R_X_Y_Z";
			break;
		case 0:
		default:
			p->Vector3D="R";
			break;
	}
	
	p->Array3Dto1D=arrayLine1D->text();
	
	p->Array3Dto0D=arrayLine0D->text();
	
	p->Array3Dto2D=arrayLine2D->text();
	
	
	switch(timeRepCombo->currentIndex())
	{
		case 1:
			p->TimeRep="HMS";
			break;
		case 0:
		default:
			p->TimeRep="Colon";
			break;
	}

	switch(pageSizeCombo->currentIndex())
	{
		case 1:
			p->PageSize="Custom";
			break;
		case 2:
			p->PageSize="Executive";
			break;
		case 3:
			p->PageSize="Legal";
			break;
		case 4:
			p->PageSize="Letter";
			break;
		case 5:
			p->PageSize="Default";
			break;
		case 0:
		default:
			p->PageSize="A4";
			break;
	}
	
	p->PageDPI=(int)getCurrentDPI();
	
	switch(orientationCombo->currentIndex())
	{
		case 0:
			p->PageOrientation="Portrait";
			break;
		case 1:
		default:
			p->PageOrientation="Landscape";
			break;
	}
		double _conversion_factors[5]={getCurrentDPI(),1,2.54,25.4,getCurrentDPI()};
	p->PageWidth=widthLine->text().toDouble()/_conversion_factors[unitsCombo->currentIndex()];
	p->PageHeight=heightLine->text().toDouble()/_conversion_factors[unitsCombo->currentIndex()];
	
	p->UseKPrinter=kprinterCheck->isChecked();
}

void PlotConfig::setSelectedProfile(QProfile * p)
{

	QString strVector3D=p->Vector3D;
	if(strVector3D=="RXYZ") vectorCombo->setCurrentIndex(1);
	else if(strVector3D=="XYZ") vectorCombo->setCurrentIndex(2);
	else if(strVector3D=="R_XYZ") vectorCombo->setCurrentIndex(3);
	else if(strVector3D=="R_LP") vectorCombo->setCurrentIndex(4);
	else if(strVector3D=="R_TP") vectorCombo->setCurrentIndex(5);
	else if(strVector3D=="X_Y_Z") vectorCombo->setCurrentIndex(6);
	else if(strVector3D=="R_L_P") vectorCombo->setCurrentIndex(7);
	else if(strVector3D=="R_T_P") vectorCombo->setCurrentIndex(8);
	else if(strVector3D=="R_X_Y_Z") vectorCombo->setCurrentIndex(9);
	else vectorCombo->setCurrentIndex(0);
	
	QString strArray3Dto1D=p->Array3Dto1D;
	if(strArray3Dto1D=="SSi") arrayCombo1D->setCurrentIndex(0);
	else if(strArray3Dto1D=="SiS") arrayCombo1D->setCurrentIndex(1);
	else if(strArray3Dto1D=="iSS") arrayCombo1D->setCurrentIndex(2);
	else if(strArray3Dto1D==ARRAY_PEACE_SUM) arrayCombo1D->setCurrentIndex(3);
	else if(strArray3Dto1D==ARRAY_PEACE_AVE) arrayCombo1D->setCurrentIndex(4);
	else if(strArray3Dto1D==ARRAY_CODIF_SUM) arrayCombo1D->setCurrentIndex(5);
	else
	{
		arrayCombo1D->setCurrentIndex(6);
	}
	arrayLine1D->setText(strArray3Dto1D);

	QString strArray3Dto0D=p->Array3Dto0D;
	if(strArray3Dto0D=="SSS") arrayCombo0D->setCurrentIndex(0);
	else if(strArray3Dto0D=="AAA") arrayCombo0D->setCurrentIndex(1);
	else
	{
		arrayCombo0D->setCurrentIndex(2);
	}
	arrayLine0D->setText(strArray3Dto0D);
	
	QString strArray3Dto2D=p->Array3Dto2D;
	if(strArray3Dto2D=="Sii") arrayCombo2D->setCurrentIndex(0);
	else if(strArray3Dto2D=="iSi") arrayCombo2D->setCurrentIndex(1);
	else if(strArray3Dto2D=="iiS") arrayCombo2D->setCurrentIndex(2);
	else if(strArray3Dto2D==ARRAY_PEACE_SUM_XY) arrayCombo2D->setCurrentIndex(3);
	else if(strArray3Dto2D==ARRAY_PEACE_AVE_XY) arrayCombo2D->setCurrentIndex(4);
	else if(strArray3Dto2D==ARRAY_CODIF_SUM_XY) arrayCombo2D->setCurrentIndex(5);
	else
	{
		arrayCombo2D->setCurrentIndex(6);
	}
	arrayLine2D->setText(strArray3Dto2D);
		
	QString strTimeRep=p->TimeRep;
	if(strTimeRep=="HMS") timeRepCombo->setCurrentIndex(1);
	else timeRepCombo->setCurrentIndex(0);
	
	QString strPageSize=p->PageSize;
	if(strPageSize=="A4") {pageSizeCombo->setCurrentIndex(0); widthLine->setEnabled(false); heightLine->setEnabled(false);}
	else if(strPageSize=="Executive") {pageSizeCombo->setCurrentIndex(2); widthLine->setEnabled(false); heightLine->setEnabled(false);}
	else if(strPageSize=="Legal") {pageSizeCombo->setCurrentIndex(3); widthLine->setEnabled(false); heightLine->setEnabled(false);}
	else if(strPageSize=="Letter") {pageSizeCombo->setCurrentIndex(4); widthLine->setEnabled(false); heightLine->setEnabled(false);}
	else if(strPageSize=="Default") {pageSizeCombo->setCurrentIndex(5); widthLine->setEnabled(false); heightLine->setEnabled(false);}
	else  {pageSizeCombo->setCurrentIndex(1);  widthLine->setEnabled(true); heightLine->setEnabled(true);} //Custom
	
	int pageDPI=p->PageDPI;
	switch(pageDPI)
	{
		case 150: pageDPICombo->setCurrentIndex(1); break;
		case 300: pageDPICombo->setCurrentIndex(2); break;
		case 600: pageDPICombo->setCurrentIndex(3); break;
		default: pageDPICombo->setCurrentIndex(0); break;
	}
	
	QString strPageOrientation=p->PageOrientation;
	if(strPageOrientation=="Portrait") orientationCombo->setCurrentIndex(0);
	else orientationCombo->setCurrentIndex(1);
	
	double _conversion_factors[5]={getCurrentDPI(),1,2.54,25.4,getCurrentDPI()};
	widthLine->setText(QString().setNum(p->PageWidth*_conversion_factors[unitsCombo->currentIndex()]));
	heightLine->setText(QString().setNum(p->PageHeight*_conversion_factors[unitsCombo->currentIndex()]));
	
	kprinterCheck->setChecked(p->UseKPrinter);
}


void PlotConfig::arrayChanged(int index)
{
	switch(index)
	{
		case 0:
			arrayLine1D->setText("SSi");
			break;
		case 1:
			arrayLine1D->setText("SiS");
			break;
		case 2:
			arrayLine1D->setText("iSS");
			break;
		case 3:
			arrayLine1D->setText(ARRAY_PEACE_SUM);
			break;
		case 4:
			arrayLine1D->setText(ARRAY_PEACE_AVE);
			break;
		case 5:
			arrayLine1D->setText(ARRAY_CODIF_SUM);
			break;
		default:
			arrayLine1D->setText("");
			break;
	}
}

void PlotConfig::arrayToScalarChanged(int index)
{
	switch(index)
	{
		case 0:
			arrayLine0D->setText("SSS");
			break;
		case 1:
			arrayLine0D->setText("AAA");
			break;
		default:
			arrayLine0D->setText("");
			break;
	}
}

void PlotConfig::arrayChangedXY(int index)
{
	switch(index)
	{
		case 0:
			arrayLine2D->setText("Sii");
			break;
		case 1:
			arrayLine2D->setText("iSi");
			break;
		case 2:
			arrayLine2D->setText("iiS");
			break;
		case 3:
			arrayLine2D->setText(ARRAY_PEACE_SUM_XY);
			break;
		case 4:
			arrayLine2D->setText(ARRAY_PEACE_AVE_XY);
			break;
		case 5:
			arrayLine2D->setText(ARRAY_CODIF_SUM_XY);
			break;
		default:
			arrayLine2D->setText("");
			break;
	}
}


void PlotConfig::pageSizeChanged(int index)
{
	double _conversion_factors[5]={getCurrentDPI(),1,2.54,25.4,getCurrentDPI()};
	double _width_in_inches[6]={8.3,8.5,7.5,8.5,8.5,0.};
	double _height_in_inches[6]={11.694,9,10.,14,11,0.}; //TODO keep synched with qpui_page.cc
	
	double width=_width_in_inches[pageSizeCombo->currentIndex()];
	double height=_height_in_inches[pageSizeCombo->currentIndex()];
	double conv=_conversion_factors[unitsCombo->currentIndex()];

	width=width*conv;
	height=height*conv;
	widthLine->setText(QString().setNum(width));
	heightLine->setText(QString().setNum(height));
	
	if(index==1) //custom
	{
		widthLine->setEnabled(true);
		heightLine->setEnabled(true);
	}
	else
	{
		widthLine->setEnabled(false);
		heightLine->setEnabled(false);
	}
}

void PlotConfig::unitsChanged(int index)
{
	double _conversion_factors[5]={getCurrentDPI(),1,2.54,25.4,getCurrentDPI()};
	double _width_in_inches[6]={8.3,8.5,7.5,8.5,8.5,0.};
	double _height_in_inches[6]={11.694,9,10.,14,11,0.}; //TODO keep synched with qpui_page.cc
	
	double width=widthLine->text().toDouble();
	double height=heightLine->text().toDouble();
	
	//conversion into inches
	double conv=_conversion_factors[previous_units];
	width=width/conv;
	height=height/conv;
	
	//conversion into new units
	conv=_conversion_factors[index];
	width=width*conv;
	height=height*conv;
	
	widthLine->setText(QString().setNum(width));
	heightLine->setText(QString().setNum(height));
	
	previous_units=index;
}

void PlotConfig::pageDPIChanged(int index)
{
	double _conversion_factors[5]={getCurrentDPI(),1,2.54,25.4,getCurrentDPI()};
	double new_dpi=_conversion_factors[unitsCombo->currentIndex()];
	
	double ratio=new_dpi/previous_dpi;
	
	if(ratio!=1. && (unitsCombo->currentIndex()==0 || unitsCombo->currentIndex()==4))
	{
		double width=widthLine->text().toDouble();
		double height=heightLine->text().toDouble();
		widthLine->setText(QString().setNum(width*ratio));
		heightLine->setText(QString().setNum(height*ratio));
	}
	
	previous_dpi=new_dpi;
}

double PlotConfig::getCurrentDPI()
{
	double dpi;
	switch(pageDPICombo->currentIndex())
	{
		case 1: dpi=150.; break;
		case 2: dpi=300.; break;
		case 3: dpi=600.; break;
		default: dpi=72.; break;
	}
	return dpi;
}

