//  qbu.cc
//
// Data Slice Widget
//
// April 2006: Tony Allen
//

#include <stdlib.h>

#include <qpushbutton.h>
#include <qlayout.h>
#include "qslot.h"
#include "qwui.h"
#include "qar.h"
extern qwuiWindow* qsasWindow;

using namespace QSAS;


// ----------------------------------------------

DataSliceWindow::DataSliceWindow(QWidget *parent) 
	: QMainWindow(0) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{

	if(QuSlotDebug())
	{
		printf("DataSliceWindow::DataSliceWindow \n");
		fflush(stdout);
	}
  
    this->setWindowTitle(" ");

	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QVBoxLayout * slotLayout=new QVBoxLayout;
	slotPanel->setTitle("Data Object");
	
	objField = new QguWLinput(slotPanel,"objField");
	objField->setAcceptable(Qcondition());
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	connect(objField, SIGNAL(edited_OK()), this, SLOT(setNewName()));
	connect(objField, SIGNAL(hadDropEvent()), this, SLOT(setAccept()));
	slotLayout->addWidget(objField);
	
	typeField = new QComboBox();
	typeField->setToolTip("Optional control over output types available. All Metadata is ALWAYS available");

	typeField->addItem("Change Output Type (optional)");
	typeField->addItem("Time");
	typeField->addItem("Time Interval");
	typeField->addItem("Time Tags");
	typeField->addItem("Scalar Series");
	typeField->addItem("Array Series");
	typeField->addItem("Single Vector");
	typeField->addItem("Scalar Value");
	typeField->addItem("Array Object");
	slotLayout->addWidget(typeField);
	connect(typeField, SIGNAL(activated(int)), this, SLOT(setType(int)) );

	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	userSet = false;
	connect( name_field,  SIGNAL( edited_OK() ), this, SLOT( setUserInput() ) );
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Save ");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptSlice()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	dismissBtn->setDefault(true);
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);
	
	//Done
	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}

void DataSliceWindow::setType(int type)
{ 
	switch(type){
		default: case 0:
			objField->setAcceptable(Qcondition());
			typeField->setItemText(0, "Default");
			break;
		case 1:
			objField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
			break;
		case 2: 
			objField->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_NOT_SEQ, QS_ANY));
			break;
		case 3:
			objField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_SEQ, QS_ANY));
			break;
		case 4:
			objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_SEQ, QS_ANY));
			break;
		case 5:
			objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ARRAY, QS_SEQ, QS_ANY));
			break;
		case 6:
			objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_VECTOR, QS_NOT_SEQ, QS_ANY));
			break;
		case 7:
			objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
			break;
		case 8:
			objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ARRAY, QS_NOT_SEQ, QS_ANY));
			break;  
	}
	objField->reValidate();
}


void DataSliceWindow::setUserInput(){
  if(name_field->lineBox->hasFocus()){
	
     if(name_field->lineBox->text().isEmpty() ) userSet = false; // allow refresh from drop to object slot
	 else userSet = true; // user typed input, disable update from data slot
  }
  // else ignore
}



void DataSliceWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("DataSliceWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   setAccept(); // ensure correct drop down if opened with preselected object
   objField->reValidate();
   
}

void DataSliceWindow::setAccept(){
    
	if(QuSlotDebug())
	{
		printf("DataSliceWindow::setAccept \n");
		fflush(stdout);
	}
  
	QdObject_var dobj = objField->getSlotObject();

	// special case of event table
	QdTimeIntervalSeq_var tis;
	if(!dobj.is_nil()) tis = QdTimeIntervalSeq_var::narrow(dobj);
	if(!tis.is_nil()) 
	{
		objField->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_ANY, QS_ANY));
		objField->reValidate();
	}
   else objField->setAcceptable(Qcondition());
   
}

void DataSliceWindow::setNewName(){
     
 if(QuSlotDebug())
 {
	printf("DataSliceWindow::setNewName \n");
	fflush(stdout);
 }

   if (userSet) return;
   QString objName = QString(this->objField->textObject().toLatin1().data());
   
   if(!objName.isEmpty()){
       QString newName = QString(objName);
       
     int ptr = newName.lastIndexOf("/");
     if (ptr >= 0) newName.remove(0, ptr+1);
	 ptr = newName.indexOf(" [");
	 if (ptr > 0) newName.truncate(ptr);
     newName += "_S";
     this->name_field->setText(newName);
   }
   

}

void DataSliceWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("DataSliceWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptSlice();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void DataSliceWindow::acceptSlice(){
      
	if(QuSlotDebug())
	{
		printf("DataSliceWindow::acceptSlice \n");
		fflush(stdout);
	}

	QString objString = objField->text();
  
	QdObject_var slicedObject = objField->getSlotObject();
  
	QString newName = name_field->text();
	if( !(slicedObject.is_nil() ) ) 
	{       
		QdTimeIntervalSeq_var test_tis;
		QdObject_var obj = QuGetObjByName(objField->textObject().toLatin1().data());
		if(!obj.is_nil())  test_tis=QdTimeIntervalSeq_var::narrow(obj);

		if(test_tis.is_nil()) QwuiPutOnWL(slicedObject, QString("Data/")+newName);
		else QwuiPutOnWL(slicedObject, QString("Time Intervals/")+newName);
	}
	else QuMsgBar("NULL object after slice", true);
	
	if(objString.startsWith("Temp/")) QuDeleteObject(baseObjectText(objString));
	this->close();
	return;
}

void DataSliceWindow::dismiss(){
      
	if(QuSlotDebug())
	{
		printf("DataSliceWindow::dismiss \n");
		fflush(stdout);
	}

	QString objString = objField->text();
	
	if(objString.startsWith("Temp/")) QuDeleteObject(baseObjectText(objString));
	this->close();
	return;
}


// class TimeSliceWindow
// ------------------------

TimeSliceWindow::TimeSliceWindow(QString baseName, const char * source, QWidget *parent, const char *name, 
                                     bool acceptTSubset, bool acceptTSlice)
  : QDialog(0)
{  
      
	if(QuSlotDebug())
	{
		printf("TimeSliceWindow::TimeSliceWindow \n");
		fflush(stdout);
	}

	QdObject_var dobj = QuGetObjByName(baseName);

	setAttribute(Qt::WA_DeleteOnClose);

	this->setWindowTitle(baseName);
	QString feedbackMsg;
	
	QVBoxLayout * mainLayout=new QVBoxLayout;
	mainLayout->setSpacing(4);
	
	QGroupBox * componentBox;
	if ( strcmp(source, "T") == 0 )
	{
		feedbackMsg = QString("Subset or slice a time sequence");
		vectorChoice = 0;  
	}
	else if ( strcmp(source, "V") == 0 )
	{
	
		componentBox = new QGroupBox( "Select Component", this);
		QVBoxLayout * componentLayout=new QVBoxLayout;
		
		vectorChoice = new QComboBox(componentBox); 
		vectorChoice->addItem("This Object");	
		vectorChoice->addItem(" r (mag) ");
		vectorChoice->addItem(" x ");
		vectorChoice->addItem(" y ");
		vectorChoice->addItem(" z ");
		vectorChoice->addItem(" Lat (deg) ");
		vectorChoice->addItem(" theta (deg) ");
		vectorChoice->addItem(" phi (deg) ");
		vectorChoice->addItem(" Lat (rad) ");
		vectorChoice->addItem(" theta (rad) ");
		vectorChoice->addItem(" phi (rad) ");
		vectorChoice->adjustSize();
		componentLayout->addWidget(vectorChoice);
		
		feedbackMsg = QString("Input is a vector sequence"); 

		QFontMetrics slice_font_mt(qApp->font());
		int char_height = slice_font_mt.height();
		
		componentBox->setLayout(componentLayout);
		mainLayout->addWidget(componentBox);
	}
	else {
		feedbackMsg = QString("Input is a scalar sequence");
		vectorChoice = 0;
	}
	
	if( !dobj.is_nil() &&(acceptTSlice || acceptTSubset) ) {
		slicer = new timeSlider(dobj, acceptTSlice, acceptTSubset, this);
		mainLayout->addWidget(slicer);
	}
	else slicer = 0;
	
	buttons_groupbox=new QGroupBox(this);
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	QPushButton *acceptBtn = new QPushButton( "Accept");
	acceptBtn->setMinimumSize(acceptBtn->sizeHint());
	connect( acceptBtn, SIGNAL(clicked()), this, SLOT(acceptSelection()) );
	acceptBtn->setFocusPolicy(Qt::StrongFocus);
	acceptBtn->setDefault(true);
	buttons_layout->addWidget(acceptBtn);

	QPushButton *setDefaultBtn = new QPushButton( "Reset");
	setDefaultBtn->setMinimumSize(setDefaultBtn->sizeHint());
	connect( setDefaultBtn, SIGNAL(clicked()), this, SLOT(setDefaults()) );
	buttons_layout->addWidget(setDefaultBtn);

	QPushButton *dismissBtn = new QPushButton( "Cancel");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	buttons_layout->addWidget(dismissBtn);
	
	buttons_groupbox->setAlignment(Qt::AlignHCenter);
	buttons_groupbox->setTitle(feedbackMsg);
	buttons_groupbox->setMinimumSize(buttons_layout->sizeHint());
	
	buttons_groupbox->setLayout(buttons_layout);
	mainLayout->addWidget(buttons_groupbox);
	
	setLayout(mainLayout);
	
}


void TimeSliceWindow::updateFeedback(){
  if(QuSlotDebug())
  {
	printf("TimeSliceWindow::updateFeedback %ld\n", (long)this);
	fflush(stdout);
  }

  QString newTitle;
  if( vectorChoice != 0)  {
     newTitle =  QString("  Output is %1 component of a vector sequence").arg(vectorChoice->currentText());
  }
  else  newTitle =  QString("  Output is a scalar sequence") ;
   
  if (slicer != 0){
     if(slicer->sliceChoice->currentIndex() == slicer->Isubset){
	   int nrec = slicer->toSlider->record - slicer->fromSlider->record + 1;
	    newTitle += QString(", subset of %1 records  ").arg(nrec);
	 }
     if(slicer->sliceChoice->currentIndex() == slicer->Iaverage){
	   int nrec = slicer->toSlider->record - slicer->fromSlider->record + 1;
	    newTitle += QString(", averaged over %1 records to single value  ").arg(nrec);
	 }
	 if(slicer->sliceChoice->currentIndex() == slicer->Islice){
	    newTitle += QString(", value at record %1  ").arg(slicer->atSlider->record);
	 }
  }
  else  newTitle += "  ";
   
  buttons_groupbox->setTitle( newTitle);

}

void TimeSliceWindow::dismiss(){
  close();
}

void TimeSliceWindow::setDefaults(){
    
 if(QuSlotDebug())
 {
	printf("TimeSliceWindow::setDefaults \n");
	fflush(stdout);
  }

   if( vectorChoice != 0)  vectorChoice->setCurrentIndex(0);
   
   if (slicer != 0){
      if(slicer->sliceChoice->currentIndex() == slicer->Isubset){
         slicer->fromSlider->adjustRec(0);
         slicer->toSlider->adjustRec(slicer->toSlider->maxRecord);
	  }
      else if(slicer->sliceChoice->currentIndex() == slicer->Iaverage){
         slicer->fromSlider->adjustRec(0);
         slicer->toSlider->adjustRec(slicer->toSlider->maxRecord);
	  }
	  else if(slicer->sliceChoice->currentIndex() == slicer->Islice){
         slicer->atSlider->adjustRec(0);
	  }
   }

}

void TimeSliceWindow::setToLastChoice(){
  
 if(QuSlotDebug())
 {
	printf("TimeSliceWindow::setToLastChoice \n");
	fflush(stdout);
 }

  if(vectorChoice != 0) {
     QguComp_e component = parentSlot->getComponent();
	 if ( component >= 0 && component <= 10){
	     if(vectorChoice->itemText(0) != "This Object" ) vectorChoice->setCurrentIndex(component -1);
	     else vectorChoice->setCurrentIndex(component);
	}
  }
  
   dimTriple sliceTriple = parentSlot->parseTimeSlice();
   if(sliceTriple.size() == 3 && slicer != 0){
      if(sliceTriple[0] == 0 ) {
	     slicer->sliceChoice->setCurrentIndex(slicer->Isubset);
         slicer->fromSlider->recBar->setValue(sliceTriple[1]);
         slicer->toSlider->recBar->setValue(sliceTriple[2]);
	  }
	  else if(sliceTriple[0] == 1) {
	     slicer->sliceChoice->setCurrentIndex(slicer->Iaverage);
		 slicer->fromSlider->recBar->setValue(sliceTriple[1]);
		 slicer->toSlider->recBar->setValue(sliceTriple[2]);
	  }
	  else if(sliceTriple[0] == 2) {
	     slicer->sliceChoice->setCurrentIndex(slicer->Islice);
		 slicer->atSlider->recBar->setValue(sliceTriple[1]);
		 slicer->updateTimeSlider(slicer->Islice);
	  }
   }

}


void TimeSliceWindow::acceptSelection(){
  
 if(QuSlotDebug())
 {
	printf("TimeSliceWindow::acceptSelection \n");
	fflush(stdout);
 }

  if(slicer != 0 && slicer->sliceChoice->currentIndex() != slicer->Islice){
  	if(slicer->fromSlider->record > slicer->toSlider->record) {
		( void )QMessageBox::warning( this, "Time Subset", "Start time after End Time, action cancelled" );
		return;
	}
  }

  QString newText = parentSlot->textObject();
  QString cmpText = "";
  
  if ( vectorChoice != 0 && vectorChoice->currentText() != QString("This Object"))  {
    cmpText += " [";
	cmpText += vectorChoice->currentText();
    cmpText += "]";
  }
  
  if(slicer != 0 && !(slicer->isDefault()) ){
    
     cmpText += " < ";

     // Time slice/subset available
	 if(slicer->sliceChoice->currentIndex() == slicer->Isubset){
		QString start = slicer->fromSlider->recField->text();      
		QString end   = slicer->toSlider->recField->text();
	 	cmpText += start;
	 	cmpText += ",";
	 	cmpText += end;
	 }
	 else if(slicer->sliceChoice->currentIndex() == slicer->Iaverage){
		QString start = slicer->fromSlider->recField->text();      
		QString end   = slicer->toSlider->recField->text();
	 	cmpText += "Ave ";
	 	cmpText += start;
	 	cmpText += ",";
	 	cmpText += end;
	 }
	 else if(slicer->sliceChoice->currentIndex() == slicer->Islice){
		QString atT = slicer->atSlider->recField->text();      
	 	cmpText += atT;
	 }
	 
     cmpText += " >";
  }


  newText += cmpText;
  parentSlot->setText(newText);
  parentSlot->reValidate(true);
  parentSlot->emitCmpSelected(cmpText.toLatin1().data(), "SLICE");
  
  close();
}

// class SelectTivlWindow
// ------------------------

SelectTivlWindow::SelectTivlWindow(QString baseName, QWidget *slot, const char *name, bool modal )
    : QDialog(0), record(0)
{  
      
    if(QuSlotDebug())
    {
		printf("SelectTivlWindow::SelectTivlWindow \n");
		fflush(stdout);
	}
	setModal(modal);
	
    QdObject_var dobj = QuGetObjByName(baseName);
    
    obj=QdTimeIntervalSeq_var::narrow(dobj);
    
    if(obj.is_nil()) max_rec = 0;
	else max_rec=obj->sequence_size()-1;

    setAttribute(Qt::WA_DeleteOnClose);
    
    QVBoxLayout * main_layout=new QVBoxLayout(this);
    main_layout->setSpacing(4);
    
    slider=new TimeIntervalSlider("", this, "slider");
    slider->maxRecord=max_rec;
    slider->recBar->setMaximum(max_rec);
    slider->obj=obj;
    slider->updateTitle();
    main_layout->addWidget(slider);
	slider->setMinimumSize(slider->sizeHint());
   
	QWidget *buttons_widget = new QWidget;
    QHBoxLayout * buttons_layout=new QHBoxLayout;
    validateBtn=new QPushButton("OK", this);
    validateBtn->setEnabled(true);
    validateBtn->setFocusPolicy(Qt::StrongFocus);
    
    cancelBtn=new QPushButton("Cancel", this);
    cancelBtn->setEnabled(true);
    cancelBtn->setFocusPolicy(Qt::StrongFocus);
    
    buttons_layout->addWidget(validateBtn);
    buttons_layout->addWidget(cancelBtn);

    connect(validateBtn, SIGNAL(clicked()), this, SLOT(validated()));
    connect(cancelBtn, SIGNAL(clicked()), this, SLOT(cancelled()));
    
	buttons_widget->setLayout(buttons_layout);
	main_layout->addWidget(buttons_widget);
    this->setMinimumSize(this->sizeHint());

}

void SelectTivlWindow::validated()
{
    if(QuSlotDebug())
    {
		printf("SelectTivlWindow::validated \n");
		fflush(stdout);
	}

    QString newText = parentSlot->textObject();
    QString cmpText = "";
    
    cmpText += " < ";
    cmpText += tr("%1").arg(slider->record);
    cmpText += " >";

    newText += cmpText;
    parentSlot->setText(newText);
    parentSlot->reValidate(true);
    parentSlot->emitCmpSelected(cmpText.toLatin1().data(), "IVL");
    close();
}

void SelectTivlWindow::cancelled()
{
    if(QuSlotDebug())
    {
		printf("TimeSliceWindow::acceptSelection \n");
		fflush(stdout);
	}

    QString newText = parentSlot->textObject();
    parentSlot->setText(newText);
    parentSlot->reValidate();
  
    close();
}

// class SelectETWindow
// ------------------------

SelectETWindow::SelectETWindow(QString baseName, QWidget *parent, const char *name, bool modal )
    : QDialog(0), record(0)
{  
      
    if(QuSlotDebug())
    {
		printf("SelectETWindow::SelectETWindow \n");
		fflush(stdout);
	}
	setModal(modal);
	
    QdObject_var dobj = QuGetObjByName(baseName);
    
    obj=QdTimeIntervalSeq_var::narrow(dobj);
    
	
    if(obj.is_nil()) max_rec = 0;
	else max_rec=obj->sequence_size()-1;

    setAttribute(Qt::WA_DeleteOnClose);
    
    QVBoxLayout * main_layout=new QVBoxLayout(this);
    main_layout->setSpacing(4);
    
	pullDown  = new QComboBox();
	pullDown->addItem("Start Time");
	pullDown->addItem("Centre Time");
	pullDown->addItem("End Time");
    main_layout->addWidget(pullDown);
	
	
    slider=new TimeIntervalSlider("", this, "slider");
    slider->maxRecord=max_rec;
    slider->recBar->setMaximum(max_rec);
    slider->obj=obj;
    slider->updateTitle();
    main_layout->addWidget(slider);
    
	QWidget *buttons_widget = new QWidget;
    QHBoxLayout * buttons_layout=new QHBoxLayout;
    validateBtn=new QPushButton("OK", this);
    validateBtn->setEnabled(true);
    validateBtn->setFocusPolicy(Qt::StrongFocus);
    
    cancelBtn=new QPushButton("Cancel", this);
    cancelBtn->setEnabled(true);
    cancelBtn->setFocusPolicy(Qt::StrongFocus);
    
    buttons_layout->addWidget(validateBtn);
    buttons_layout->addWidget(cancelBtn);

    connect(validateBtn, SIGNAL(clicked()), this, SLOT(validated()));
    connect(cancelBtn, SIGNAL(clicked()), this, SLOT(cancelled()));
    
	buttons_widget->setLayout(buttons_layout);
	main_layout->addWidget(buttons_widget);
    this->setMinimumSize(this->sizeHint());

}

void SelectETWindow::validated()
{
    if(QuSlotDebug())
    {
		printf("SelectETWindow::validated \n");
		fflush(stdout);
	}

    QString newText = parentSlot->textObject();
    QString cmpText = "";
    
    cmpText += " < ";
    cmpText += tr("%1").arg(slider->record);
    cmpText += " >";

	cmpText += " [ ";
	cmpText += pullDown->currentText();
	cmpText += " ]";
	
    newText += cmpText;
    parentSlot->setText(newText);
    parentSlot->reValidate(true);
    parentSlot->emitCmpSelected(cmpText.toLatin1().data(), "IVL");
    close();
}

void SelectETWindow::cancelled()
{
    if(QuSlotDebug())
    {
		printf("SelectETWindow::acceptSelection \n");
		fflush(stdout);
	}

    QString newText = parentSlot->textObject();
    parentSlot->setText(newText);
    parentSlot->reValidate();
  
    close();
}

// class SelectTivlSubsetWindow
// ------------------------

SelectTivlSubsetWindow::SelectTivlSubsetWindow(QString baseName, QWidget *parent, const char *name, bool modal )
    : QDialog(0), record_start(0)
{  
      
    if(QuSlotDebug())
    {
		printf("SelectTivlSubsetWindow::SelectTivlSubsetWindow \n");
		fflush(stdout);
	}

	this->setModal(modal);
	
    QdObject_var dobj = QuGetObjByName(baseName);
    
    obj=QdTimeIntervalSeq_var::narrow(dobj);
    
    max_rec=obj->sequence_size()-1;
    record_end=max_rec;

    setAttribute(Qt::WA_DeleteOnClose);
    
    QVBoxLayout * main_layout=new QVBoxLayout(this);
    main_layout->setSpacing(4);

    slider1=new TimeIntervalSlider("From", this);
    slider1->maxRecord=max_rec;
    slider1->recBar->setMaximum(max_rec);
    slider1->obj=obj;
    slider1->updateTitle();
    main_layout->addWidget(slider1);
    
    slider2=new TimeIntervalSlider("To", this);
    slider2->maxRecord=max_rec;
    slider2->record=max_rec;
    slider2->recBar->setMaximum(max_rec);
    slider2->recBar->setValue(max_rec);
    slider2->obj=obj;
    slider2->updateTitle();
    main_layout->addWidget(slider2);

	QWidget *buttons_widget = new QWidget;
    QHBoxLayout * buttons_layout=new QHBoxLayout();
    validateBtn=new QPushButton("OK", this);
    validateBtn->setEnabled(true);
    validateBtn->setFocusPolicy(Qt::StrongFocus);
    
    cancelBtn=new QPushButton("Cancel", this);
    cancelBtn->setEnabled(true);
    cancelBtn->setFocusPolicy(Qt::StrongFocus);
    
    buttons_layout->addWidget(validateBtn);
    buttons_layout->addWidget(cancelBtn);

    connect(validateBtn, SIGNAL(clicked()), this, SLOT(validated()));
    connect(cancelBtn, SIGNAL(clicked()), this, SLOT(cancelled()));
    
	buttons_widget->setLayout(buttons_layout);
	main_layout->addWidget(buttons_widget);

    this->setMinimumSize(this->sizeHint());

}

void SelectTivlSubsetWindow::validated()
{
    if(QuSlotDebug())
    {
		printf("SelectTivlSubsetWindow::validated \n");
		fflush(stdout);
	}
    
    QString newText = parentSlot->textObject();
    QString cmpText = "";
    
    int record1=slider1->record;
    int record2=slider2->record;
    if(record1>record2)
    {
        int temp=record1;
        record1=record2;
        record2=record1;
    }

        cmpText += " < ";
        cmpText += tr("%1").arg(record1)+ ","+tr("%1").arg(record2);
        cmpText += " >";

    newText += cmpText;
    parentSlot->setText(newText);
    parentSlot->reValidate(true);
    parentSlot->emitCmpSelected(cmpText.toLatin1().data(), "IVL");
    
    close();
}

void SelectTivlSubsetWindow::cancelled()
{
    if(QuSlotDebug())
    {
		printf("TimeSliceWindow::acceptSelection \n");
		fflush(stdout);
	}

    QString newText = parentSlot->textObject();
    parentSlot->setText(newText);
    parentSlot->reValidate();
  
    close();
}


