#include "WLModel.h"
#include "WLView.h"
#include "qwui.h"

WLModel::WLModel(WLView* view):
	QAbstractItemModel(view)
{
// 	connect(this, SIGNAL(pathChanged(const QString&, const QString&)), this, SLOT(displayPathChanged(const QString&, const QString&)));
// 	
// 	connect(this, SIGNAL(deleted(const QString&)), this, SLOT(displayDeleted(const QString&)));
// 	
// 	connect(this, SIGNAL(created(const QString&)), this , SLOT(displayCreated(const QString&)));
	this->view=view;
	_root=new WLItem("", NULL, true);
	_data=new WLItem("Data", _root, true);
	_time=new WLItem("Time Intervals", _root, true);
	_constants=new WLItem("Constants", _root, true);
    _temp=new WLItem("Temp", _root, true);
	
	if(QSAS_profiles->current()->dbgWL)
	{
		index_ptr=&WLModel::index_dbg;
		rowCount_ptr=&WLModel::rowCount_dbg;
	}
	else
	{
		index_ptr=&WLModel::index_std;
		rowCount_ptr=&WLModel::rowCount_std;
	}

//	view->collapseAll();
		

}

WLModel::~WLModel()
{
	delete _root;
}

// Minimum set of functions



QModelIndex WLModel::index_std(int row, int column, const QModelIndex& parent) const
{
	if (!hasIndex(row, column, parent))
		return QModelIndex();

	WLItem *parentItem;

	if (!parent.isValid())
		parentItem = _root;
	else
		parentItem = itemFromIndex(parent);
	WLItem *childItem = parentItem->getChildByNumber(row);
	if (childItem  && childItem!=_temp)
		return createIndex(row, column, childItem);
	else
		return QModelIndex();
}

QModelIndex WLModel::index_dbg(int row, int column, const QModelIndex& parent) const
{
	if (!hasIndex(row, column, parent))
		return QModelIndex();

	WLItem *parentItem;

	if (!parent.isValid())
		parentItem = _root;
	else
		parentItem = itemFromIndex(parent);
	WLItem *childItem = parentItem->getChildByNumber(row);
	if (childItem)
		return createIndex(row, column, childItem);
	else
		return QModelIndex();
}

QModelIndex WLModel::parent(const QModelIndex& index) const
{
	if (!index.isValid())
		return QModelIndex();

	WLItem *childItem = itemFromIndex(index);
	WLItem *parentItem = childItem->parent();

	if (parentItem == _root)
		return QModelIndex();

	return createIndex(parentItem->childNumber(), 0, parentItem);
}


int WLModel::rowCount_std(const QModelIndex& parent) const
{
	WLItem *parentItem;
	if (parent.column() > 0)
		return 0;


	if (!parent.isValid())
		parentItem = _root;
	else
		parentItem = itemFromIndex(parent);
		
	if (parentItem==_root)
		return 3;
	return parentItem->numChildren();
}

int WLModel::rowCount_dbg(const QModelIndex& parent) const
{
	WLItem *parentItem;
	if (parent.column() > 0)
		return 0;


	if (!parent.isValid())
		parentItem = _root;
	else
		parentItem = itemFromIndex(parent);
	
	return parentItem->numChildren();
}

int WLModel::columnCount(const QModelIndex&) const
{
	return 1;
}

QVariant WLModel::data(const QModelIndex& index, int role) const
{
	if (!index.isValid())
		return QVariant();

	if (role == Qt::DisplayRole)
	{
		WLItem *item = itemFromIndex(index);
		return item->getName();
	}
	else if(role==Qt::ToolTipRole)
	{
		WLItem *item = itemFromIndex(index);
		if(item->isFolder())
			return QVariant();
		
		QdObject_var tmp=QuGetObjByName(item->getPath());
		if(tmp.is_nil()) return QString(item->getName());
		if(!tmp->xref_exists("FIELDNAM")) return QVariant();
		QdString_var fieldnam=QdString_var::narrow(tmp->get_xref("FIELDNAM"));
		if(fieldnam.is_nil())
		{
			return item->getName();
		}
		
		return QString((*fieldnam).c_str());
	}
	else if(role==Qt::DecorationRole)
	{
		WLItem* item=itemFromIndex(index);
		if(item->isFolder() && item!=_data && item!=_time && item!=_constants&& item!=_temp)
		{
			return QVariant(QApplication::style()->standardIcon(QStyle::SP_DirOpenIcon));
		}
		return QVariant(item->getName());
	}
	
	return QVariant();

}

// Editable

 Qt::ItemFlags WLModel::flags(const QModelIndex &index) const
{
	if (!index.isValid())
		return 0;
	WLItem* item=itemFromIndex(index);
	
	if(item==NULL || item==_root)
		return 0;
	
	if(item==_data || item==_time || item==_constants || item==_temp   ) 
	{
		return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDropEnabled;
	}
	
// 	if(item->isFolder())
		return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsEditable | Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled;
	
// 	return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsEditable | Qt::ItemIsDragEnabled;

}

bool WLModel::setData(const QModelIndex& index, const QVariant& value, int role)
{
	if (index.isValid() && role == Qt::EditRole)
	{
		WLItem* item=itemFromIndex(index);
		if(item)
		{
			if(item!=_data && item!=_time && item!=_constants)
			{
				item->setName(value.toString().trimmed());
				emit dataChanged(index, index);
				return true;
			}
		}
	}
	return false;
}

// Add/Remove rows/cols

bool WLModel::insertRows(int position, int rows, const QModelIndex& parent)
{
	if(!parent.isValid())
		return false;
	
	WLItem* parentItem=itemFromIndex(parent);
	
	if(parentItem==NULL || parentItem==_root || !parentItem->isFolder())
		return false;
	
	beginInsertRows(parent, position, position+rows-1);

	for (int row = 0; row < rows; ++row)
	{
		WLItem* newItem=new WLItem("", NULL);
		setUpConnections(newItem);
		newItem->reparent(parentItem, position);
	}

	endInsertRows();
	return true;

}
		
bool WLModel::deleteItem(WLItem* item)
{
	if(item==NULL || item==_root || item==_data || item==_time || item==_constants) return false;

	QString path=item->getPath();
		
	WLItem* parentItem=item->parent();
	if(parentItem==NULL) return false;
	
	QModelIndex parent = indexFromItem(parentItem);
	
	view->setCurrentItem(_data);
	
	emit layoutAboutToBeChanged();

	int row=item->childNumber();

	if(parentItem==NULL || parentItem==_root) return false;
	
	beginRemoveRows(parent, row, row);
	

	parentItem->removeChild(item, true);

	endRemoveRows();
		
	emit layoutChanged();

	view->setCurrentItem(_data);

	emit(deleted(path));
	return true;
}

bool WLModel::removeRows(int position, int rows, const QModelIndex& parent)
{
	if(!parent.isValid())	return false;
	
	WLItem* parentItem=itemFromIndex(parent);

	if(parentItem==NULL || parentItem==_root)
		return false;
	
	beginRemoveRows(parent, position, position+rows-1);

	for (int row = 0; row < rows; ++row)
	{
		parentItem->removeChild(position, true);
	}

	endRemoveRows();
	return true;
	
}

void WLModel::clear(QModelIndex& parent)
{
	if(!parent.isValid()) return;
	
	WLItem* item=itemFromIndex(parent);
	
	if(item!=NULL && item!=_root && item->numChildren()>0)
	{
		removeRows(0, item->numChildren(), parent);
	}
}


bool WLModel::insertColumns(int, int, const QModelIndex&)
{
	return false;
}

bool WLModel::removeColumns(int, int, const QModelIndex&)
{
	return false;
}

QModelIndex WLModel::indexFromItem(WLItem* i)
{
	if(i==NULL) return QModelIndex();
	if(i==_root) return view->rootIndex();
	return createIndex(i->childNumber(), 0, i);
}

// Drag and drop tools

 QStringList WLModel::mimeTypes() const
{
	QStringList types;
	types << "application/wlitems";
	types << "application/xrefitems";
	types << "text/plain";
	return types;
}

QMimeData* WLModel::mimeData(const QModelIndexList& indexes) const
{
	QMimeData *mimeData = new QMimeData();
	
	QStringList idList;

	foreach (QModelIndex index, indexes) {
		if (index.isValid()) {
			idList << itemFromIndex(index)->getPath();
		}
	}
	
	QString data=idList.join("//");

	mimeData->setData("application/wlitems", data.toLatin1().data());
	mimeData->setText(itemFromIndex(indexes.first())->getPath());
	return mimeData;
}

bool WLModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent)
{

	if (action == Qt::IgnoreAction)
		return true;

	// handle drag from attribute list in browsers
	if(data->hasFormat("application/xrefitems"))
	{
		QStringList attrList=QString(data->data("application/xrefitems")).split("//");
		
		foreach(QString attrString, attrList)
		{
			emit this->xrefDropDetected(attrString);
		}
		
		qApp->processEvents();
		emit this->refreshBrowser();
		return true;
	}
	
	// handle drag from slot
	if(data->hasFormat("text/plain") && !data->hasFormat("application/wlitems") )
	{
		QString name(data->data("text/plain"));
		
		QdObject_var dobj = QuGetObjByNameAndComp(name.toLatin1().data());
		
		// remove path
		int ptr = name.lastIndexOf("/");
		if(ptr >= 0) name.remove(0, ptr+1);
		
		// fix comp text
		bool comp=false;
		// metadata
		if((ptr = name.indexOf("[=>")) >= 0) 
		{
			comp = true;
			name.replace(ptr, 3, "::");
		}

		// component
		while((ptr = name.indexOf("[")) >= 0) 
		{
			comp = true;
			name.replace(ptr, 1, "_");
		}
				
		while((ptr = name.indexOf("]")) >= 0) 
		{
			name.remove(ptr, 1);
		}
		
		// record slice
		if((ptr = name.indexOf("<")) >= 0) 
		{
			comp = true;
			name.replace(ptr, 1, "{");
		}
		
		if((ptr = name.indexOf(">")) >= 0) 
		{
			name.replace(ptr, 1, "}");
		}
				
		// srip out white spaces
		int nSpace;
		while( (nSpace = name.indexOf(" ")) != -1) name.remove(nSpace, 1);
		
		if(comp) {
			QwuiPutOnWL(dobj, name.toLatin1().data());
		}
		else 
		{
			QdObject_var dobjCopy = Factory::clone(dobj);
			dobjCopy->copy_xrefs_from(dobj);
			name += "_copy";
			QwuiPutOnWL(dobjCopy, name.toLatin1().data());
		}
		return true;
	}
	
	if (!data->hasFormat("application/wlitems"))
		return false;

	if (column > 0)
		return false;
	
	if(!parent.isValid())
		return false;
	
	WLItem* parentItem=itemFromIndex(parent);
	
	if(parentItem==NULL)
		return false;
	
	int actualRow=row;
	if(!parentItem->isFolder())
	{
		actualRow=parentItem->childNumber();
		parentItem=parentItem->parent();
	}
	
	QStringList itemList=QString(data->data("application/wlitems")).split("//");
	
	bool res=true;
	WLItem* currentItem;
	
	emit layoutAboutToBeChanged();
	
	QList<WLItem*> list;
	
	foreach(QString itemString, itemList)
	{
		currentItem=_root->getChildByPath(itemString);
		if(currentItem!=NULL)
		{
			res=res && currentItem->reparent(parentItem, actualRow);
			list.push_back(currentItem);
		}
	}
	
	emit layoutChanged();
	
	QList<QModelIndex> list2;
	foreach(WLItem* i, list)
	{
		list2.push_back(createIndex(i->childNumber(), 0, i));
	}
	
	emit newSelectedIndexes(list2);
	myReset();
	return res;
}

// Utils

void WLModel::appendStatus(QLinkedList<bool>& status, QModelIndex& in)
{
	status.push_back(view->isExpanded(in));
	if(hasChildren(in))
	{
		int num=rowCount(in);
		for(int i=0; i<num; ++i)
		{
			QModelIndex temp=index(i, 0, in);
			appendStatus(status, temp);
		}
	}
}

void WLModel::setStatus(QLinkedList<bool>& status, QModelIndex& in)
{
	bool isExpanded=status.first();
	status.removeFirst();
	view->setExpanded(in, isExpanded);
	if(hasChildren(in))
	{
		int num=rowCount(in);
		for(int i=0; i<num; ++i)
		{
			QModelIndex temp=index(i, 0, in);
			setStatus(status, temp);
		}
	}
}

void WLModel::myReset()
{
	// Stores the expanded/collapsed status of every item, resets, and restores the status
	
	QLinkedList<bool> status;
	QModelIndex rootIndex=indexFromItem(_root);
	appendStatus(status, rootIndex);

	rootIndex=indexFromItem(_root);
	setStatus(status, rootIndex);
}

QModelIndex WLModel::moveUp(QModelIndex& index)
{
	if(!index.isValid())
		return QModelIndex();
	
	WLItem* item=itemFromIndex(index);
	
	if(item!=NULL && item!=_root && item!=_data && item!=_time && item!=_constants)
	{
		int row=item->childNumber();
		if(row>0)
		{
			emit layoutAboutToBeChanged();
			
			item->reparent(item->parent(), row-1);
			
			emit layoutChanged();
		}
	}
	
	return createIndex(item->childNumber(), 0, item);
}

QModelIndex WLModel::moveDown(QModelIndex& index)
{
	if(!index.isValid())
		return QModelIndex();
	
	WLItem* item=itemFromIndex(index);
	
	if(item!=NULL && item!=_root && item!=_data && item!=_time && item!=_constants)
	{
		int row=item->childNumber();
		if(row!=item->numChildren()-1)
		{
			emit layoutAboutToBeChanged();
			
			item->reparent(item->parent(), row+1);
			
			emit layoutChanged();
		}
	}
	
	return createIndex(item->childNumber(), 0, item);
}

void WLModel::sortWLSelection(QList <WLItem*> &itemList, string &sortOn, bool reverse)
{
	if(itemList.size() < 2) return;
	
	vector <WLItem*> newList;
	vector<WLItem*>::iterator it;
	
	WLItem *item = itemList[0];
	newList.push_back(item);
	int row=item->childNumber();

	// find new order
	if(sortOn == string("name")){
		for(int i=1; i<itemList.size(); i++){
			item = itemList[i];

			if(row > item->childNumber()) row=item->childNumber(); // find earliest entry

			bool inserted = false;
			it=newList.begin();
			while ( it<newList.end()){
				if( item->getName() < (*it)->getName() ){
					newList.insert(it, item);
					inserted = true;
					break; 
				}
				it++;
			}
			if( !inserted) newList.push_back(item);
		}
	}
	else if(sortOn == string("time")){
		for(int i=1; i<itemList.size(); i++){
			item = itemList[i];

			if(row > item->childNumber()) row=item->childNumber(); // find earliest entry

			bool inserted = false;
			it=newList.begin();
			while ( it<newList.end()){
				if( item->getTime() < (*it)->getTime() ){
					newList.insert(it, item);
					inserted = true;
					break; 
				}
				it++;
			}
			if( !inserted) newList.push_back(item);
		}	
	}
	else if(sortOn == string("object")){
		for(int i=1; i<itemList.size(); i++){
			item = itemList[i];

			if(row > item->childNumber()) row=item->childNumber(); // find earliest entry

			bool inserted = false;
			it=newList.begin();
			while ( it<newList.end()){
				if( item->getType() < (*it)->getType() ){
					newList.insert(it, item);
					inserted = true;
					break; 
				}
				it++;
			}
			if( !inserted) newList.push_back(item);
		}	
	}
	
	// place in new order on WL
	for(int i=0; i<newList.size(); i++){
		if(reverse) item = newList[newList.size() - i - 1];
		else		item = newList[i];

		if(item!=NULL && item!=_root && item!=_data && item!=_time && item!=_constants)
		{
			if(row!=item->numChildren()-1)
			{
				emit layoutAboutToBeChanged();
			
				item->reparent(item->parent(), row);
			
				emit layoutChanged();
			}
			row++;
		}
	}
	
	return;
}



WLItem* WLModel::createItem(const QString& path)
{
	QStringList pathExpl=path.split("/", QString::SkipEmptyParts);
	
	if(pathExpl.isEmpty()) return NULL;
	
	QString currentPath=pathExpl.at(0);
	
	WLItem* currentItem=itemFromPath(currentPath);
	if(currentItem==NULL) // We don't add anything to the root
		return NULL;
	
	WLItem* currentParent;
	emit layoutAboutToBeChanged();
	for(int i=1; i<pathExpl.size(); ++i)
	{
		currentPath.append("/").append(pathExpl.at(i));
		currentParent=currentItem;
		currentItem=itemFromPath(currentPath);
		if(currentItem==NULL && i!=pathExpl.size()-1)
		{
			currentItem=new WLItem(pathExpl.at(i), currentParent);
			currentItem->setFolder(true);
			
		}
		if(i==pathExpl.size()-1) // We always create something in the last step: if something with the same name exists, the name will get "A" appended
		{
			currentItem=new WLItem(pathExpl.at(i), currentParent);
			setUpConnections(currentItem);
		}
	}
	emit layoutChanged();
	
	currentItem->setFolder(false);
	return currentItem;
}

QModelIndex WLModel::newFolder(const QModelIndexList& indexes)
{
	if(indexes.isEmpty())
	{
		WLItem* folder=createItem("Data/Group");
		folder->setFolder();
		return createIndex(folder->childNumber(), 0, folder);
	}
	else
	{
		// Place the new folder at the same level as the first item in selection
		WLItem* firstItem=itemFromIndex(indexes.first());
		
		if(firstItem==NULL) return QModelIndex();
		
		QString folderName=firstItem->parent()->getPath()+"/Group";
		
		// Store pointers to moved items now, as the layout changes just after and the indexes may change
		QList<WLItem*> items;
		foreach(QModelIndex index, indexes)
		{
			WLItem* item=itemFromIndex(index);
			if(item!=NULL && item!=_root && item!=_data && item!=_time && item!=_constants && item!=_temp)
			{
				items.push_back(itemFromIndex(index));
			}
		}
		
		WLItem* folder=createItem(folderName);
		folder->setFolder();
		
		return createIndex(folder->childNumber(), 0, folder);
	}
}

Qt::DropActions WLModel::supportedDropActions() const
{
	return Qt::CopyAction|Qt::MoveAction;
}


// Streaming I/O classes

Vostream& operator<<(Vostream& v_os, WLModel& model)
{
	int cnt;
	cnt=model._data->numChildren();
	
	v_os << cnt;
	
	for(int i=0; i<cnt; ++i)
	{
		v_os << *(model._data->getChildByNumber(i));
	}
	cnt=model._time->numChildren();
	
	v_os << cnt;
	
	for(int i=0; i<cnt; ++i)
	{
		v_os << *(model._time->getChildByNumber(i));
	}
	
	cnt=model._constants->numChildren();
	
	v_os << cnt;
	
	for(int i=0; i<cnt; ++i)
	{
		v_os << *(model._constants->getChildByNumber(i));
	}
	
	return v_os;
}

Vistream& operator>>(Vistream& v_is, WLModel& model)
{
	int cnt;
	
	v_is >> cnt;

	if(cnt!=0)
	{
		model.BIR(model._data, 0, cnt);
		for(int i=0; i<cnt; ++i)
		{
			WLItem* item=new WLItem("", model._data);
			v_is >> *item;
			model.setUpConnectionsRec(item);
		}
		model.EIR();
	}

	// cnt objects in Time intervals
	
	v_is >> cnt;
	if(cnt!=0)
	{
		model.BIR(model._time, 0, cnt);
		for(int i=0; i<cnt; ++i)
		{
			WLItem* item=new WLItem("", model._time);
			v_is >> *item;
			model.setUpConnectionsRec(item);
		}
		model.EIR();
	}
	
	// cnt objects in Constants
	
	v_is >> cnt;
	if(cnt!=0)
	{
		model.BIR(model._constants, 0, cnt);
		for(int i=0; i<cnt; ++i)
		{
			WLItem* item=new WLItem("", model._constants);
			v_is >> *item;
			model.setUpConnectionsRec(item);
		}
		model.EIR();
	}
	
	return v_is;
}
