/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/


#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>  
#include <math.h>
#include <ctype.h>
#include <sys/param.h>

#include <qfiledialog.h> 
#include <qglobal.h>
#include <qfont.h>
#include <qwidget.h>

#include "qt_version.h"
#include "qwui.h"
#include "qslot.h"
#include "qdutil.h"
#include "QCCWindow.h"
#include "cdf.h"
#include "DvObj.h"

QCCWindow * qccWindow;

QFont defaultFont;

extern qwuiWindow* qsasWindow;


 using namespace QSAS;

 
// QSAS main window is a class on its own

QWidget *qwuiWindow::toggleBar;

qwuiWindow::qwuiWindow(QWidget *parent, const char *name) 
   :  QguWindow(parent,name)
{
	double qsas_height = QSAS_profiles->current()->Height; 
	if(qsas_height == 0) qsas_height = 48.; // default if no env set or is bad

	// ensure QSAS_HOME env is set
	char * qsas_home_env = getenv("QSAS_HOME"); 
	if( qsas_home_env != NULL) {
        qsas_home = QString(qsas_home_env);
		qsas_home=qsas_home.remove(QString("\""));
	}
	else{   
		bool ok;
		QString user_choice = QuTextInputDialog("Environment QSAS_HOME not set", 
                                   "Please provide the full path to QSAS_HOME Directory",
				   "/usr/local/QSAS_2", &ok, this);
		if( !ok || user_choice.isEmpty() ) user_choice = QString(".");
		qsas_home = QString(user_choice.toLatin1().data());   // deep copy
	}

	QguWindow::_saveset_filter=QString("WL savesets (*.qs_WL)");
	setAcceptDrops(true);
	help_window = new QswHelpWindow();
	
	// Menu insertion moved to qsas_main to avoid circular dependencies
	
	QFontMetrics qsas_font_mt(qApp->font());
	int char_width = qsas_font_mt.width("M");
	int char_height = qsas_font_mt.height();
   
	resize( char_width*48, (int)(char_height*qsas_height) );

	centralW = new QWidget(this);
	centralBox = new QVBoxLayout(centralW);
	
	toggleBar = new QWidget(centralW);
	QHBoxLayout * toggleLayout=new QHBoxLayout(toggleBar);

	centralBox->addWidget(toggleBar);

	StripBadData = checkBadData(toggleBar);
	StripBadData->setChecked(FALSE);
	StripBadData->setMaximumHeight(18);
   
	StripFills = checkFills( toggleBar);
	StripFills->setChecked(FALSE);
	StripFills->setMaximumHeight(18);
   
	overWrite = new QCheckBox("Replace", toggleBar);
	overWrite->setChecked(FALSE);
	overWrite->setMaximumHeight(18);
	
	toggleLayout->addWidget(StripBadData);
	toggleLayout->addWidget(StripFills);
	toggleLayout->addWidget(overWrite);
	
#if QT_VERSION < 0x040300
	toggleLayout->setMargin(0);
#else
	toggleLayout->setContentsMargins(0,0,0,0);
#endif
   
	this->_sessionFolder = QString(this->_filename.toLatin1().data());
	#if defined(Q_OS_MACX)
		this->_sessionFolder += QString("/SampleSession.qss");
	#endif
	this->_filename += QString("/saveSet");
	this->SAVE_EXTENSION = QString(".qx_WL");
	this->_filename += this->SAVE_EXTENSION;
	this->fromSession = false;

	wlview=new WLView(centralW);
	WLModel* model=new WLModel(wlview);
	wlview->setModel(model);
	connect(model, SIGNAL(newSelectedIndexes(const QModelIndexList&)), wlview, SLOT(newSelectedIndexes(const QModelIndexList&)));
//	wlview->expandAll();
	
	centralBox->addWidget(wlview);
	
	setCentralWidget( centralW );
}


void qwuiWindow::makeWindows()
{
	wlview->wlmodel()->blockSignals(true);
	
	setStandardConstants( );

	wlview->wlmodel()->blockSignals(false);
	/* create some test time interval objects to put in the list */
	QdTimeInterval_var ti_1 = new QdTimeInterval;  
	ti_1->set( QmjdTime(2001, 9, 4), QmjdTime(2001, 9, 5) );
	QwuiPutOnWL((QdObject_var)ti_1, "Time Intervals/SampleTI");
	
	wlview->collapse(wlview->wlmodel()->indexFromItem(wlview->wlmodel()->_constants));

// create window for qtet plugin to access QApplication (static QT linkage)
	qccWindow = new QCCWindow(); 

}
 
void qwuiWindow::save(Vostream & v_os){

	if(QuSlotDebug())
	{
		printf("qwuiWindow::saveWith Vostream %ld\n", (long)this);
		fflush(stdout);
	}

	string guiID;
	guiID=QSAS_VERSION_ID;
	v_os << guiID;   

	int with = 1/*WorkingList->with*/;
      
	v_os << StripBadData->isChecked();
	v_os << StripFills->isChecked();

	v_os << *(wlview->wlmodel());
  
}

void qwuiWindow::restore(Vistream & v_is, bool replace)
{
	string guiID; 
	v_is >> guiID;
	char *ptr;
    
	if(QuSlotDebug())
	{
		printf("QwuiWindow::restore ------- set ID =  %s ------\n   --------------------------------------\n ", guiID.c_str());
		fflush(stdout);
	}

	wlview->wlmodel()->blockSignals(true);

    if(replace)
    {
		wlview->deleteAllDataItems();
		wlview->deleteAllTimeIntervalItems();
		wlview->deleteAllConstantItems();
    }

	if( strstr(guiID.c_str(), "qwui") == NULL)
	{
		// changed to QSAS_VERSION_ID at 2.3.04
		setRestoreVersion(guiID.c_str());
	}
	else
	{
		setRestoreVersion("QSAS 2.3.03"); // old saveset so ensure old Time class is used
	}

	bool sbd;
	v_is >> sbd;
	StripBadData->setChecked(sbd);
    
	bool sf;
	v_is >> sf;
	StripFills->setChecked(sf);
   
	v_is >> *(wlview->wlmodel());
	
	wlview->expandAll();
	wlview->collapse(wlview->wlmodel()->indexFromItem(wlview->wlmodel()->_constants));

	wlview->wlmodel()->blockSignals(false);
	
	

}

void qwuiWindow::saveSession(){
 
 if(QuSlotDebug())
 {
	printf("qwuiWindow::saveSession %ld\n", (long)this);
	fflush(stdout);
 }
 
  QuMsgBar(" ", false);

// ensure default is set
   if(_sessionFolder.isEmpty())
  {
     saveSessionAs();
     return;
  }

// ensure default has right filter
  int posn = _sessionFolder.lastIndexOf(".qss");
  if (posn < 0 ){
     saveSessionAs();
     return;  
  }

// ensure default exists still
// user might delete it

  QDir sessionDir = QDir(_sessionFolder);
  if( !sessionDir.exists() ){
    bool OK = sessionDir.mkdir(_sessionFolder);
    if (!OK){
      QuMsgBar("Failed to create directory", _sessionFolder.toLatin1().data(), true);
      return;
    }
  }
  // remove trailing '/'

  if (_sessionFolder[_sessionFolder.length()-1] == '/') _sessionFolder.truncate(_sessionFolder.length()-1);
 
  emit saveSessionRequest(_sessionFolder);
  
    // do save of WL
  QString fileN = QString(_filename.toLatin1().data());
  
  // set session folder and save
  this->_filename = QString(_sessionFolder.toLatin1().data());
  this->_filename += QString("/saveSet");
  this->_filename += this->SAVE_EXTENSION;
  saveWL();
 
  // put file name back
  _filename = QString(fileN.toLatin1().data());
}
  
void qwuiWindow::saveSessionAs(){
 
  if(QuSlotDebug())
  {
	printf("qwuiWindow::saveSessionAs %ld\n", (long)this);
	fflush(stdout);
  }
 
  QString fn;
  QuMsgBar(" ", false);
  
      // use path only
  int posn = _sessionFolder.lastIndexOf(".qss");
  
  if (posn > 0) {
     _sessionFolder.truncate(posn);
    posn = _sessionFolder.lastIndexOf("/");
    if (posn > 0) _sessionFolder.truncate(posn+1);
  }

  
  SessionFolderDialog *dialog = new SessionFolderDialog(this,"FolderSession");
  dialog->setDirectory(_sessionFolder);
  if(dialog->exec() == QDialog::Accepted){
    fn = dialog->getSessionFolder();
  
    if(fn.isEmpty())
    {
       QuMsgBar("Not .qss directory", true);
    }
    else
    {
       _sessionFolder=fn;
       if (_sessionFolder[_sessionFolder.length()-1] == '/') _sessionFolder.truncate(_sessionFolder.length()-1);
       saveSession();
	 
	  // set QSAS main window title
	  QString capt = QString(fn.toLatin1().data());
	  int pos_end = capt.lastIndexOf(".qss");
	  capt.truncate(pos_end);
	  int pos_strt = capt.lastIndexOf("/");
	  capt.remove(0,pos_strt+1);

	  this->setWindowTitle(" '" + capt + "'   " + QString(QSAS_VERSION_ID));

    }
  }
 
}
 
  
void qwuiWindow::restoreSession(){

 if(QuSlotDebug())
 {
	printf("qwuiWindow::restoreSession %ld\n--------------------------------\n\n", (long)this);
	fflush(stdout);
 }

	// use path only
	int posn = _sessionFolder.lastIndexOf(".qss");
  
	if (posn > 0) {
		_sessionFolder.truncate(posn);
		posn = _sessionFolder.lastIndexOf("/");
		if (posn > 0) _sessionFolder.truncate(posn+1);
	}

	QuMsgBar(" ", false);
    QString fn=QFileDialog::getExistingDirectory(this, "Select session folder to restore", _sessionFolder);
	if(!fn.isEmpty())
	{
	   QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );

	   qApp->processEvents();

	  if (fn[fn.length()-1] == '/')  fn.truncate(fn.length()-1);

      _sessionFolder = QString(fn.toLatin1().data());

      // restore WL

     // set session folder and restore
     this->_filename = QString(fn.toLatin1().data());
	 this->_filename += QString("/saveSet");
     this->_filename += this->SAVE_EXTENSION;
     this->fromSession = true;

	// tell plot and calc GUIs to clear their listviews to minimise wasted revalidation
	emit clearListRequest();

	// empty the working list done in restore(true)
	
	qApp->processEvents();

	QguWindow::restore(true);

	// restore each GUI
	emit restoreSessionRequest(fn);
  
	qApp->processEvents();
	  	 
		
     status(fn+" restored",10000);

     // set QSAS main window title
     QString capt = QString(_sessionFolder.toLatin1().data());
     int pos_end = capt.lastIndexOf(".qss");
     capt.truncate(pos_end);
     int pos_strt = capt.lastIndexOf("/");
     capt.remove(0,pos_strt+1);

     this->setWindowTitle(QString(QSAS_VERSION_ID) + "   '" + capt + "'"  );
     QApplication::restoreOverrideCursor();

   }
   else
     status("Restore session aborted",10000);
}

void qwuiWindow::saveWL(){

	if(QuSlotDebug())
	{
		printf("qwuiWindow::saveWL \n");
		fflush(stdout);
	}
	
	_filename.replace(".qs_", ".qx_"); // ensure XML save 

	if(_filename.isEmpty())
	{
		saveWLAs();
		return;
	}
    
	std::ofstream ofp;
 
	ofp.open(_filename.toLatin1().data(), std::fstream::out );

	ofp << "<QSAS_ID>";
	ofp << QSAS_VERSION_ID;
	ofp << "</QSAS_ID>\n";	
	
	ofp << "<STRIP_BAD_DATA_TOGGLE>";
	ofp << (int) StripBadData->isChecked();
	ofp << "</STRIP_BAD_DATA_TOGGLE>\n";

	ofp << "<STRIP_FILLS_TOGGLE>";
	ofp << (int) StripFills->isChecked();
	ofp << "</STRIP_FILLS_TOGGLE>\n";

	ofp << "<OVER_WRITE_TOGGLE>";
	ofp << (int) overWrite->isChecked();
	ofp << "</OVER_WRITE_TOGGLE>\n";      

	ofp <<  " <WORKING_LIST>\n";

	int cnt=wlview->wlmodel()->_data->numChildren();
	for(int i=cnt-1; i>=0; i--)
	{
		WLItem* item = wlview->wlmodel()->_data->getChildByNumber(i);
		if(item->isFolder()){
            
            saveFolder(item, ofp);
			
		}
		else{
			QdObject_var qobj = QuGetObjByName( item->getPath() );
			DvObject_var dobj = toDVOS( qobj);
			DvNode *node = new DvNode(item->getPath().toLatin1().data(), dobj);
			node->toXML(ofp);
		}
	}
	
	cnt=wlview->wlmodel()->_time->numChildren();
	for(int i=cnt-1; i>=0; i--)
	{
		WLItem* item = wlview->wlmodel()->_time->getChildByNumber(i);
		if(item->isFolder()){
			int k=item->numChildren();
			for(int i=k-1; i>=0; i--)
			{
				WLItem* kitem = item->getChildByNumber(i);
				QdObject_var qobj = QuGetObjByName( kitem->getPath() );
				DvObject_var dobj = toDVOS( qobj);
				DvNode *node = new DvNode(kitem->getPath().toLatin1().data(), dobj);
				node->toXML(ofp);
			}
			
		}
		else{
			QdObject_var qobj = QuGetObjByName( item->getPath() );
			DvObject_var dobj = toDVOS( qobj);
			DvNode *node = new DvNode(item->getPath().toLatin1().data(), dobj);
			node->toXML(ofp);
		}
	}
	
	cnt=wlview->wlmodel()->_constants->numChildren();	
	for(int i=cnt-1; i>=0; i--)
	{
		WLItem* item = wlview->wlmodel()->_constants->getChildByNumber(i);
		QdObject_var qobj = QuGetObjByName( item->getPath() );
		DvObject_var dobj = toDVOS(qobj);
		DvNode *node = new DvNode(item->getPath().toLatin1().data(), dobj);
		node->toXML(ofp);
	}
		
	ofp <<  " </WORKING_LIST>\n";
	
	ofp.close();

  	status(_filename+" saved",10000);

	// Also save old style for QSAS 2.4
	QString fileNameCopy = _filename;
	fileNameCopy.replace(".qx_", ".qs_"); // now do old XDR save as well

try
	{
    	VXDRostream xdr_os(fileNameCopy.toLatin1().data());
    	save(xdr_os);
  	}
  	catch(Exception &e)
  	{
		status(e.msg().c_str(),10000);   
     	return;
  	}
	
	return;   
}

void qwuiWindow::saveFolder(WLItem* item, std::ofstream &ofp){

    int k=item->numChildren();
    for(int i=k-1; i>=0; i--)
    {
        WLItem* kitem = item->getChildByNumber(i);
        if(kitem->isFolder()){
                
            saveFolder(kitem, ofp);
                
        }
        else{
            QdObject_var qobj = QuGetObjByName( kitem->getPath() );
            DvObject_var dobj = toDVOS( qobj);
            DvNode *node = new DvNode(kitem->getPath().toLatin1().data(), dobj);
            node->toXML(ofp);
        }
    }
}

void qwuiWindow::saveWLAs(){

 if(QuSlotDebug())
 {
	printf("qwuiWindow::saveWLAs \n");
	fflush(stdout);
 }
  
  QString _filter = QString("save (*");
  _filter += SAVE_EXTENSION;
  _filter += QString(")");
  QString fn=QFileDialog::getSaveFileName(this, "Select file - Save WL Settings", _filename, _filter);
  if(!fn.isEmpty())
  {
    int pos = fn.lastIndexOf(SAVE_EXTENSION);
    if (pos < 0) {
      pos = fn.lastIndexOf(".");
      if (pos > 0) fn.truncate(pos);
      fn += SAVE_EXTENSION;
    }
    _filename=fn;
    saveWL();
  }
  else
    status("saving aborted ",10000); 
}



void qwuiWindow::setPreferences(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::setPreferences %ld\n", (long)this);
	fflush(stdout);
 }
 
  
  QuMsgBar("TBD: Window to set user preferences\n global settings", false);
  
}


void qwuiWindow::launchManual(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchManual %ld\n", (long)this);
	fflush(stdout);
 }
 
   qsasWindow->help_window->manEntry("index.html");
}

void qwuiWindow::launchManMainW(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchManMainW %ld\n", (long)this);
	fflush(stdout);
 }
 
  qsasWindow->help_window->manEntry("QSAS_main.html");
}

void qwuiWindow::launchManSampleW(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchManSampleW %ld\n", (long)this);
	fflush(stdout);
 }
 
  qsasWindow->help_window->manEntry("QSAS_intro.html");
}

void qwuiWindow::launchManDataH(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchManDataH %ld\n", (long)this);
	fflush(stdout);
 }
 
  qsasWindow->help_window->manEntry("QSAS_data_handling.html");
}

void qwuiWindow::launchManSaveRestore(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchManSaveRestore %ld\n", (long)this);
	fflush(stdout);
 }
 
  qsasWindow->help_window->manEntry("QSAS_save_restore.html");
}

void qwuiWindow::launchVersion(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchVersion %ld\n", (long)this);
	fflush(stdout);
 }
 
  QString text;
  text  = QString("Version Numbers...\n"); 
  text += QString(QSAS_VERSION_ID);
  text += "\n";
  text += QString(QSAS_VERSION_DATE);
  // QT
  text += "\nQT version detected:  ";
  text += QString(QT_VERSION_STR);
  // cdf 
  CDFid id;
  long version, release, increment;
  long numDims=0;
  long dimSizes[1] = {0};
  CDFsetFileBackward(BACKWARDFILEoff);
  QString file_path=QString(getenv("QSAS_HOME"));
  file_path+="/empty";
  CDFstatus status = CDFlib(CREATE_, CDF_, file_path.toLatin1().data(), numDims, dimSizes, &id, NULL_);
  char message[CDF_STATUSTEXT_LEN+1];
  CDFerror(status, message);
   if(QuSlotDebug())  printf("%s\n",message);
  if(status == CDF_OK) {
  	CDFgetVersion (id, &version, &release, &increment);
	CDFdelete(id);
  	text += "\nCDF version detected:  ";
  	QString cdf_version_str = QString("CDF %1.%2.%3").arg(version).arg(release).arg(increment);
  	text += cdf_version_str;
  }
  Qwui_msgBox(text.toLatin1().data());

}

void qwuiWindow::launchContact(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchContact %ld\n", (long)this);
	fflush(stdout);
 }

  qsasWindow->help_window->manEntry("ContactUs.html");


}


void qwuiWindow::closeEvent( QCloseEvent* ce)
{     
	char * fastClose = (char *) getenv ("EXIT_MODE");
	if(fastClose != NULL && strcmp(fastClose, "QSAS_FAST_EXIT") == 0) 
	{
		qApp->exit(0);
	}
	else {
		
		switch( QMessageBox::information( this, "Qwui Window",
                                          "Exit Qsas Main Window?",
                                          "Exit", "Save Session", "Cancel",
                                          0, 1 ) ) 
    	{
        	case 0:
			{
				qApp->exit(0);
				break;
			}
        	case 1:
			{
				saveSession();
            	ce->ignore();	    
            	break;
			}
        	case 2:
        	default: 
        	{
		   	 	ce->ignore();
            	break;
			}
    	}
	}
}
 void qwuiWindow::dragEnterEvent(QDragEnterEvent *event)
{   
       this->activateWindow();
       this->raise();
}


void qwuiWindow::Qwui_msgBox( const char *  text)
{
   QMessageBox::information( this, "QSAS information", text, "OK" ); 
   return;
}

  

SessionFolderDialog::SessionFolderDialog( QWidget *parent, const char *name ) 
    : QFileDialog(parent) 
{
    this->setWindowTitle("Select a .qss folder for this Save Session");
    this->setFileMode(QFileDialog::Directory);
    setOption(QFileDialog::ShowDirsOnly, true);
    QString qss(".qss");
    setDefaultSuffix(qss);
    this->setNameFilter("session dir (*.qss)");

    
//    this->setWindowTitle("Select a .qss folder for this Save Session");
//    this->setFileMode(QFileDialog::DirectoryOnly);
//    setAcceptMode(QFileDialog::AcceptSave);
//    this->setFilter("session dir (*.qss)");
//    this->layout()->addWidget(new QWidget());//blank
//    this->layout()->addWidget(new QLabel("Select an existing .qss folder and click OK - Will overwrite contents of this folder.\n (or Create New Folder from menu bar first, hit return then select as above)", this));
}

QString SessionFolderDialog::getSessionFolder(){
	QStringList files = this->selectedFiles();
  	QString dirName;
  	if (!files.isEmpty()) dirName = files[0];
  
  	if( !dirName.isEmpty()){
//     int pos = dirName.findRev(".qss");
// 
//     if (pos < 0 )  dirName = QString::null;
//     if( pos != dirName.length()-4 ) {
//       // some versions of QT add / to directories
// 	 pos = dirName.findRev(".qss/");
//       if( pos != dirName.length()-5 ) dirName = QString::null;
//     }
      if(!dirName.endsWith(".qss") && !dirName.endsWith(".qss/") )
      {
          dirName.append(".qss");
          QuMsgBar(".qss appended to folder name", true);
      }
  } 
  return dirName;
}


//---------------------------------------------------------


QString QwuiPutOnWL(QdObject_var obj, const QString& path, bool replace)
{

	bool ok = true;
	WLModel* model=qsasWindow->wlview->wlmodel();
	WLItem* item=model->itemFromPath(path);

	if(qsasWindow->overWrite->isChecked()) replace = true;
	
	QString newPath=path;
	
	// stop Data etc folders being deleted
	if(item && item->parent() == model->_root) {
		newPath.prepend("Data/");  
		item=model->itemFromPath(newPath);
	}
	
	newPath.replace("[", "(");
	newPath.replace("]", ")");
  // remove and '[' or ']' from WL name to prevent calculator problems
	
	if(item==NULL)
	{
		item=model->createItem(newPath);
		if(item==NULL)
		{
			item=model->createItem(QString("Data/")+newPath);
			if(item!=NULL)
			{
				newPath=item->getPath();
			}
		}
		
		ok = QuSetObjByName(obj, newPath);
	}
	else
	{

		if(replace)
		{
			ok = QuSetObjByName(obj, newPath);
		}
		else
		{
			while(model->itemFromPath(newPath)!=NULL)
			{
				newPath.append("A");
			}
			item=model->createItem(newPath);
			
			ok = QuSetObjByName(obj, newPath);
		}
	}

	if(item==NULL) return QString();

	if( !ok) 
	{
		model->deleteItem(item);
		return QString();
	}
	
//	model->emitCreated(item->getPath());
	qsasWindow->wlview->setCurrentIndex(model->indexFromItem(item));
	
	QString context = item->getPath();
	
	model->emitObjChanged(context);
	
	return context;
	
}

//---------------------------------------------------------

QSize qsasGUISize(){ 
   return qsasWindow->frameSize();
}

