#include "qie_VOtable.h"
#include <QDomDocument>


QDOS::QdTimeIntervalSeq_var QiLoadVOTable(QString& fileName)
{
    QDOS::QdTimeIntervalSeq_var res=new QDOS::QdTimeIntervalSeq;
    
    QDomDocument doc("Table");
    QFile myFile(fileName);
    doc.setContent(&myFile);
    myFile.close();

    QDomElement docElem = doc.documentElement();
    
    // The table entries are in RESOURCE/TABLE/DATA/TABLEDATA/TR

    QDomElement resource=docElem.firstChildElement("RESOURCE");
    if(resource.isNull())
    {
        std::cout << "RESOURCE is null" << std::endl;
        return res;
    }

    QDomElement table=resource.firstChildElement("TABLE");
    if(table.isNull())
    {
        std::cout << "TABLE is null" << std::endl;
        return res;
    }

    int indexStart=-1;
    int indexEnd=-1;

    int index=0;
    QDomElement field=table.firstChildElement("FIELD");
    while(!field.isNull() && (indexStart==-1 || indexEnd==-1))
    {
        if(field.attribute("ucd")=="time.start")
        {
            indexStart=index;
        }
        else if(field.attribute("ucd")=="time.stop")
        {
            indexEnd=index;
        }

        ++index;
        field=field.nextSiblingElement("FIELD");
    }

    if(indexStart==-1 || indexEnd==-1)
    {
        std::cout << "Indexes for start and/or end times not found" << std::endl;
        return res;
    }
    
    QDomElement data=table.firstChildElement("DATA");
    if(data.isNull())
    {
        std::cout << "DATA is null" << std::endl;
        return res;
    }

    QDomElement tableData=data.firstChildElement("TABLEDATA");
    if(tableData.isNull())
    {
        std::cout << "TABLEDATA is null" << std::endl;
        return res;
    }
    
    QDomElement tr=tableData.firstChildElement("TR");
    QDomElement td;
    QDomElement td2;
    while(!tr.isNull())
    {
        QDomNodeList list=tr.childNodes();
        if(list.size()>=indexStart || list.size()>=indexEnd)
        {
            td=list.at(indexStart).toElement();
            td2=list.at(indexEnd).toElement();
            if(!td.isNull() && !td2.isNull())
            {
                res->push_back(QDOS::QdTimeInterval(QDOS::QmjdTime(td.text().toLatin1().data()), QDOS::QmjdTime(td2.text().toLatin1().data())));
            }
        }
        tr=tr.nextSiblingElement("TR");
    }
    
    return res;
}

bool QiSaveVOTable(QDOS::QdTimeIntervalSeq_var& table, QString& fileName)
{
    std::ofstream file(fileName.toLatin1().data());
    
    if(!file.is_open()) return false;
    
    time_t now;
    time(&now);
    
    file << "<?xml version=\"1.0\"?>" << std::endl;
    file << "<VOTABLE version=\"1.1\"><DESCRIPTION>" << std::endl;
    file << "Table generated by QSAS;" << std::endl;
    file << "Creation Date: " << ctime(&now);
    file << "</DESCRIPTION><RESOURCE><DESCRIPTION>" << std::endl;
    file << "        QSAS" << std::endl;
    file << "</DESCRIPTION><TABLE><FIELD datatype=\"char\" name=\"Start Time\" ID=\"TimeIntervalStart\" ucd=\"time.start\"><DESCRIPTION>time tag for beginning of interval</DESCRIPTION></FIELD><FIELD datatype=\"char\" name=\"Stop Time\" ID=\"TimeIntervalStop\" ucd=\"time.stop\"><DESCRIPTION>time tag for end of interval</DESCRIPTION></FIELD><DATA><TABLEDATA>" << std::endl;
    
    for(int i=0; i<table->sequence_size(); i++)
    {
        file << "<TR><TD>" << table[i].start().getISOstring(1) << "</TD><TD>" << table[i].end().getISOstring(1) << "</TD></TR>" << std::endl;
    }
    
    file << "</TABLEDATA></DATA></TABLE></RESOURCE></VOTABLE>" << std::endl;
    
    file.close();
    
    return true;
}

