
#include <qstring.h>
#include <qfiledialog.h> 

#include "qsasMenus.h"
#include "qplug_if.h"
#include "qie_VOtable.h"
#include "qpsd.h"

 extern qwuiWindow* qsasWindow;
#include "qcalcWindow.h"
extern CalcWindow *QCALC_Proto_widget;
 
// for fortran dependent plugins under cygwin
extern "C" QplugReturnStatus  Tsyganenko(  QplugArgList *);
extern "C" QplugReturnStatus  WaveletDenoise(  QplugArgList *);

 QplugArgList* qsasPlugArgs;

void AddQwuiMenus(){

	static QplugArgList staticArgList;
   
	// these are here because cygwin/windows does not allow forward declaration
   
	static mainMenu * file = new mainMenu( qsasWindow );
	qsasWindow->_menu_bar->addMenu(file );
	file->insertFileItems();
	file->setTitle( " File ");
   
	static mainMenu * edit = new mainMenu( qsasWindow );
	qsasWindow->_menu_bar->addMenu(edit );
	edit->insertEditItems();
	edit->setTitle(" Edit ");
  

	// Add to test the Object Calculator
	static mainMenu *eCalculate = new mainMenu( qsasWindow );
	qsasWindow->_menu_bar->addMenu( eCalculate );
	eCalculate->insertCalcItems();
	eCalculate->setTitle(" Analysis ");
   
	static mainMenu *plugins = new mainMenu( qsasWindow );
	qsasWindow->_menu_bar->addMenu(plugins );
	plugins->insertPluginMenu();
	plugins->setTitle(" Plug-Ins ");
     
	static mainMenu * plot = new mainMenu( qsasWindow );
	qsasWindow->_menu_bar->addMenu(plot );
	plot->insertPlotItems();
	plot->setTitle(" Plot    ");

	static mainMenu * profiles = new mainMenu(qsasWindow);
	qsasWindow->_menu_bar->addMenu(profiles );
	profiles->insertProfilesItems();
	profiles->setTitle(" Profiles ");
   
	qsasWindow->_menu_bar->addSeparator();   
   
	static mainMenu * help = new mainMenu( qsasWindow );
	qsasWindow->_menu_bar->addMenu(help );
	help->insertHelpItems();
	help->setTitle(" Help ");
   
	// populate time and maths menus
	qsasWindow->mathOps->addFolderToMenu("Arithmetic");
   

	qsasWindow->timeOps->addFolderToMenu("TimeOperations");
	qsasWindow->filterOps->addFolderToMenu("Filtering");
	qsasWindow->statOps->addFolderToMenu("Statistics");
	qsasWindow->unitOps->addFolderToMenu("UnitOperations");
	qsasWindow->eventTableOps->addFolderToMenu("EventTables");
   
	// populate plugin menu
	plugins->refreshPlugins(true);
   
	// for static plugins (cygwin missing dynamic libg2c.dll) 
   
	qsasPlugArgs = &staticArgList;
   
 }

// class mainMenu
// -----------------------

mainMenu::mainMenu(QWidget *parent, const char *name)
    :  QMenu(parent)
{        
   // initialise NULL pointer    
    objTypeSelector = NULL;
    setTearOffEnabled(true);
}

// -----------------------

void mainMenu::menuOp(QAction *act)
{ 

  QString pluginName; 
  
  if( act->text() == QString("Browse") ){
	pluginName = QFileDialog::getOpenFileName(this, "Select qtpl template", ".", "*.qtpl");
    QtplOpenByName(pluginName, qsasWindow);
    return;
  }
  else if( act->text() == QString("Refresh Menu") ){
    this->refreshPlugins(false);
    return;
  }
  
}


void mainMenu::refreshPlugins(bool first)
{ 
  for(int i=0; i<qsasWindow->pluginFolders.size(); i++){
    qsasWindow->pluginFolders[i]->clear();
    if(first) {
	
      this->addMenu(qsasWindow->pluginFolders[i]);
    }
    qsasWindow->pluginFolders[i]->addFolderToMenu(qsasWindow->pluginFolders[i]->title().toLatin1().data(), first);
  }

  if(first){
    this->addSeparator();
    this->addAction("Browse" );
    this->addAction("Refresh Menu" );
  }
  QuMsgBar("Plugin Menus rebuilt", false);
}

void mainMenu::insertFileItems(){
//	this->addAction("Test DVOS", this, SLOT( DVOS_Test() ));
	this->addAction("Data Selector", this, SLOT( openDS() ));
	this->addAction(style()->standardIcon(QStyle::SP_DialogOpenButton, 0, this), "&Open Data File", this, SLOT( openCAA() ), Qt::CTRL+Qt::Key_O );
	this->addAction(style()->standardIcon(QStyle::SP_DialogSaveButton, 0, this), "&Write Data File", this, SLOT( openExport() ), Qt::CTRL+Qt::Key_W );
	this->addSeparator();
	this->addAction("Open Data File (old interface)", this, SLOT( openQIE() ) );
	this->addAction("Load Event Table", this, SLOT(openEvtTable()));
	this->addAction("Export Event Table", this, SLOT(exportET()));
	this->addSeparator();
	this->addAction("Save Working List", qsasWindow, SLOT( saveWL() ) );
	this->addAction("Save Working List As...", qsasWindow, SLOT( saveWLAs() ) );
	this->addAction("Restore Working List...", qsasWindow, SLOT( restore() ) );
	this->addAction("Include in WL...", qsasWindow, SLOT( include() ) );
	this->addSeparator();
	this->addAction("&Save Session", qsasWindow, SLOT( saveSession() ), Qt::CTRL+Qt::Key_S );
	this->addAction("Save Session As...", qsasWindow, SLOT( saveSessionAs() ), Qt::SHIFT+Qt::CTRL+Qt::Key_S );
	this->addAction("&Restore Session", qsasWindow, SLOT( restoreSession() ), Qt::CTRL+Qt::Key_R );
	this->addSeparator();
	this->addAction(style()->standardIcon(QStyle::SP_DialogCloseButton, 0, this), "&Quit", qApp, SLOT( closeAllWindows() ), Qt::CTRL+Qt::Key_Q );
	connect(qApp, SIGNAL(lastWindowClosed() ), qApp, SLOT(quit()));
	connect(qsasWindow, SIGNAL(openQdaWindow(const char *  )), this, SLOT(openDataFile(const char * )));
	
	connect(qsasWindow->wlview, SIGNAL(exportET()), this, SLOT(exportET())); // see comments on mainMenu::exportET()
}

void mainMenu::insertEditItems(){

	this->addAction("new &Object", qsasWindow->wlview, SLOT( newObject() ) );
   this->addAction("new &Folder", qsasWindow->wlview, SLOT( newFolder() ), Qt::CTRL+Qt::Key_F );
   this->addSeparator();

   this->addAction("&Duplicate Object", qsasWindow->wlview, SLOT( duplicateObj() ), Qt::CTRL+Qt::Key_D );
   this->addAction("Subsample/Extract", qsasWindow->wlview, SLOT( sliceObj() ), Qt::CTRL+Qt::ALT+Qt::Key_G );
   this->addAction("&Browse Object",qsasWindow->wlview, SLOT(browseObject()), Qt::CTRL+Qt::Key_B );
   this->addAction("&Edit Object",qsasWindow->wlview, SLOT(editObject()), Qt::CTRL+Qt::Key_E );
   this->addAction("re-&Name Object", qsasWindow->wlview, SLOT( rename() ), Qt::CTRL+Qt::Key_N );
   this->addAction("select &All Data", qsasWindow->wlview, SLOT(selectAllData()), Qt::CTRL+Qt::Key_A );
   this->addSeparator();

   this->addAction(style()->standardIcon(QStyle::SP_TrashIcon, 0, this), "Delete Object", qsasWindow->wlview, SLOT( deleteObject() ), Qt::CTRL+Qt::Key_X );
   this->addAction("Remove object's time tags", qsasWindow->wlview, SLOT( removeTimeTags() ));
   this->addSeparator();
   this->addAction("Clear Working List", qsasWindow->wlview, SLOT( deleteAllDataItems() ));
   this->addAction("Clear Time Interval List", qsasWindow->wlview, SLOT( deleteAllTimeIntervalItems() ));
   this->addSeparator();
   this->addAction("Clear Selected Cache", qsasWindow->wlview, SLOT( clearCacheSelected() ));
   this->addAction("Clear Cache", qsasWindow->wlview, SLOT( clearCache() ));
   this->addSeparator();
   this->addAction("Move Up", qsasWindow->wlview, SLOT(moveUp()), Qt::CTRL+Qt::Key_Up );
   this->addAction("Move Down", qsasWindow->wlview, SLOT(moveDown()), Qt::CTRL+Qt::Key_Down );
   this->addSeparator();
   this->addAction("Sort selection", qsasWindow->wlview, SLOT(sortWLSelection()) );
   this->addAction("Sort Working List", qsasWindow->wlview, SLOT(sortWL()) );
   this->addAction("Set Sort Prefs...", qsasWindow->wlview, SLOT(sortWLPrefs()) );

   

  }
  
void mainMenu::insertHelpItems(){

	this->addAction(style()->standardIcon(QStyle::SP_MessageBoxInformation, 0, this), "QSAS &Manual", qsasWindow, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	this->addSeparator();
	this->addAction("...Getting Started", qsasWindow, SLOT( launchManSampleW() ));
	this->addAction("...Main Window", qsasWindow, SLOT( launchManMainW() ));
	this->addAction("...Data Handling", qsasWindow, SLOT( launchManDataH() ));
	this->addAction("...Save and Restore", qsasWindow, SLOT( launchManSaveRestore() ));
	this->addSeparator();
	this->addAction("Module Version", qsasWindow, SLOT( launchVersion() ));
	this->addAction("Contact QSAS Team", qsasWindow, SLOT( launchContact() ));
   
}

void mainMenu::insertCalcItems(){


	this->addAction("&Calculator", this, SLOT( launchCalculatorPrototype()), Qt::CTRL+Qt::Key_C );
	this->addAction("Calculator (old interface)", this, SLOT( launchCalculator()));
	this->addAction("Particle Tools (PSD)", this, SLOT( launchPSD() ));



	// End Calculator
   
   // old Arithmetic plugins
   qsasWindow->mathOps = new qtplMenu( qsasWindow, "Simple Maths" );
   this->addMenu(qsasWindow->mathOps );
   connect(qsasWindow->mathOps, SIGNAL(triggered(QAction *)), qsasWindow->mathOps,  SLOT(qtplOpen(QAction *)) );

   // Unit conversion plugins
   qsasWindow->unitOps = new qtplMenu( qsasWindow, "Change Units" );
   this->addMenu(qsasWindow->unitOps );
   connect(qsasWindow->unitOps, SIGNAL(triggered(QAction *)), qsasWindow->unitOps,  SLOT(qtplOpen(QAction *)) );

   qsasWindow->timeOps = new qtplMenu( qsasWindow, "Time Ops..." );
   qsasWindow->timeOps->addAction("&Join", this, SLOT( launchJoin() ), Qt::CTRL+Qt::Key_J );
   qsasWindow->timeOps->addAction("Timetags to seconds offset", this, SLOT( launchTTOffset() ) );
   qsasWindow->timeOps->addAction("Timetags TT2 - TT1 in seconds", this, SLOT( launchTT1TT2toSec() ) );
   qsasWindow->timeOps->addAction("Time Interval to seconds", this, SLOT( launchTItoSec() ) );
   qsasWindow->timeOps->addAction("Time T2 - T1 in seconds", this, SLOT( launchT1T2toSec() ) );
   
   this->addMenu(qsasWindow->timeOps);
   connect(qsasWindow->timeOps, SIGNAL(triggered(QAction *)), qsasWindow->timeOps,  SLOT(qtplOpen(QAction *)) );

   qsasWindow->filterOps = new qtplMenu( qsasWindow, "Filtering" );
   this->addMenu(qsasWindow->filterOps );
   connect(qsasWindow->filterOps, SIGNAL(triggered(QAction *)), qsasWindow->filterOps,  SLOT(qtplOpen(QAction *)) );

   qsasWindow->statOps = new qtplMenu( qsasWindow, "Statistics" );
   qsasWindow->statOps->addAction("Linear Regression", this, SLOT( regression() ) );
   this->addMenu(qsasWindow->statOps );
   connect(qsasWindow->statOps, SIGNAL(triggered(QAction *)), qsasWindow->statOps,  SLOT(qtplOpen(QAction *)) );
   
   qsasWindow->eventTableOps = new qtplMenu( qsasWindow, "Event Tables" );
   this->addMenu(qsasWindow->eventTableOps );
   connect(qsasWindow->eventTableOps, SIGNAL(triggered(QAction *)), qsasWindow->eventTableOps,  SLOT(qtplOpen(QAction *)) );
   
   QMenu* angle_menu = this->addMenu("Angle Depends");
   angle_menu->addAction("Convert Angle Depends to (0,360) deg", this, SLOT(angleDepTo360()));
   angle_menu->addAction("Convert Angle Depends to (-180,180) deg", this, SLOT(angleDepTo180()));
      
   this->addAction("Subsample/Extract", qsasWindow->wlview, SLOT( sliceObj() ) );

   this->addAction("Trace of Array", this, SLOT( arrayTrace() ) );

   this->addAction("Derivative", this, SLOT( derivative() ) );

   this->addAction("Integral", this, SLOT( integral() ) );
   
   this->addAction("Array to Vector", this, SLOT(vect2array()) );

   this->addAction("Merge TS Objects", this, SLOT(mergeTS()) );
   
}

void mainMenu::insertPlotItems(){

    this->addAction("&Plot Layout", this, SLOT( launchPlot() ), Qt::CTRL+Qt::Key_P );
    this->addAction("2D Polar View", this, SLOT( launchPolarPlot() ) );
    this->addAction("Surface View", this, SLOT( launchSurfacePlot() ) );
    this->addAction("3D View", this, SLOT( launch3DPlot() ) );
}

void mainMenu::insertPluginMenu(){
   static qtplMenu *geoMenu = new qtplMenu(qsasWindow, "Geophysics");
   qsasWindow->pluginFolders.push_back(geoMenu);
   
   static qtplMenu *analysisMenu = new qtplMenu(qsasWindow, "Analysis"); 
   qsasWindow->pluginFolders.push_back(analysisMenu);
   
   connect(this, SIGNAL(triggered(QAction *)),this,  SLOT(menuOp(QAction *)) );
   connect(geoMenu, SIGNAL(triggered(QAction *)),geoMenu,  SLOT(qtplOpen(QAction *)) );
   connect(analysisMenu, SIGNAL(triggered(QAction *)),analysisMenu,  SLOT(qtplOpen(QAction *)) );
}

void mainMenu::insertProfilesItems()
{
	this->addAction("Edit profiles...", this, SLOT( launchEditProfile() ));
	this->addSeparator();

	QString currentName=QSAS_profiles->current()->getName();
	std::vector<QString> names;
	QSAS_profiles->names(names);
	for(int i=0; i<names.size(); ++i)
	{
		QAction* thisAction=this->addAction(names[i]);			
		connect(this, SIGNAL(triggered(QAction*)), this, SLOT(launchSelectProfile(QAction*)) );
		thisAction->setCheckable(true);

		if(names[i]==currentName)
		{
			checkedProfile = thisAction;
			checkedProfile->setChecked(true);
		}
		else
		{
			thisAction->setChecked(false);
		}
	}
}

// Why put this here as it's not part of the main menu?
// Answer: it should be in WLView, BUT as the Windows build cannot make dlls with undefined symbols, and as QiSaveVOTable is defined in qie, which must be compiled after the GUI dlls, we have no other choice than put it somewhere around here...
void mainMenu::exportET()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::exportET %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=qsasWindow->wlview->currentIndex();
	WLItem* item=qsasWindow->wlview->wlmodel()->itemFromIndex(index);
	if(item==NULL) return;
	if(item->isFolder()) return;
	
	QdObject_var keep_object=QuGetObjByName(item->getPath());
    
	QdTimeIntervalSeq_var table=QdTimeIntervalSeq_var::narrow(keep_object);
	if(table.is_nil()){
		QdTimeInterval_var TI = QdTimeInterval_var::narrow(keep_object);
		if(TI.is_nil()){
			QdTimeSeq_var tt = get_timetags(keep_object);
			if(!tt.is_nil()) TI = new QdTimeInterval( tt->get_interval_bounds());
		}
		if(!TI.is_nil()) table = new QdTimeIntervalSeq(1, *TI);
	}
	    
	if(!table.is_nil())
	{
		QString fileName=item->getName();
		fileName.remove(" (evt table)").append(".xml");
		QString fn=QFileDialog::getSaveFileName(this, "Select file - Save WL Settings", fileName, "*.xml");
		if(!fn.isEmpty())
		{
			bool res=QiSaveVOTable(table, fn);
			if(!res) QuMsgBar("ERROR, Export failed\n", true);
		}
		else
		{
			QuMsgBar("ERROR, Failed selecting the file\n", true);
		}
	}
	else
	{
		QuMsgBar("ERROR, Wrong object type\n", true);
	}
}

void mainMenu::openDS(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::openDS %ld\n", (long)this);
  fflush(stdout);
  }
 
 
  QdsShow();

}

void mainMenu::openQIE(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::openQIE %ld\n", (long)this);
  fflush(stdout);
 }
 
	if(def_dir.isEmpty()) 
	{
		def_dir = QSAS_profiles->current()->DataPath.toLatin1().data();
		if(def_dir.isEmpty())
		{
#ifdef WINDOWS
			def_dir = getenv("USERPROFILE");
#else
			def_dir = getenv("PWD"); // getenv returns const char * so is used to deep copy to string
#endif
		}
	}
	
	QString file;
  
	// get user file choice
	file = QFileDialog::getOpenFileName(this, "Select Data File", def_dir, "All QSAS data files ( *.cdf *.cef *.qft *.qfd);;*.cdf;;*.cef;;*.qft;;*.qfd;;Try any file (*)");
 
	if ( !file.isEmpty() ) {
  
		// keep chosen directory for next time
		int posn = file.lastIndexOf("/");
		if (posn >= 0){
			def_dir = QString(file.toLatin1().data()); // force deep copy
			def_dir.truncate(posn); 
		} 
		QdiLaunchWindow(file, qsasWindow);
	}
}


void mainMenu::openCAA(){ 

	if(QuSlotDebug())
	{
		printf("mainMenu::openCAA %ld\n", (long)this);
		fflush(stdout);
	}
 
	if(def_dir.isEmpty()) 
	{
		def_dir = QSAS_profiles->current()->DataPath.toLatin1().data();
		if(def_dir.isEmpty())
		{
#ifdef WINDOWS
			def_dir = getenv("USERPROFILE");
#else
			def_dir = getenv("PWD"); // getenv returns const char * so is used to deep copy to string
#endif
		}
	}
	
	QString file;
  
	// get user file choice
	file = QFileDialog::getOpenFileName(this, "Select Data File", def_dir, "CAA syntax CEF and CDF data files ( *.cef *.cef.gz *.cdf );;*.cef;;*.cef.gz;;*.cdf");
 
	if ( !file.isEmpty() ) {
  
		// keep chosen directory for next time
		int posn = file.lastIndexOf("/");
		if (posn >= 0){
			def_dir = QString(file.toLatin1().data()); // force deep copy
			def_dir.truncate(posn); 
			} 
		QdaLaunchWindow(file, this);
	}
}

void mainMenu::openExport(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::openExport %ld\n", (long)this);
  fflush(stdout);
  }
 
 
   QjuiShowExport();
}

void mainMenu::openEvtTable(){ 
	if(QuSlotDebug())
	{
		printf("mainMenu::openEvtTable %ld\n", (long)this);
		fflush(stdout);
	}
	
	QString file;
	static QString dir=".";
	
	//       get user file choice
	file = QFileDialog::getOpenFileName(this, "Select Data File", dir, "VOTables ( *.xml);;*.*" );
	
	if ( !file.isEmpty() )
	{
		QdTimeIntervalSeq_var obj=QiLoadVOTable(file);
		if(!obj.is_nil())
		{
			file=file.remove(".xml");
			int posn = file.lastIndexOf("/");
			if (posn >= 0)
			{
				dir = QString(file.toLatin1().data()); // force deep copy
				dir.truncate(posn);
				file=file.remove(dir);
			}
			
			file=file.append(" (evt table)");
			file=QString("Time Intervals")+file;
		
			QwuiPutOnWL((QdObject_var) obj, file);
		}
	}

}
void mainMenu::launchPSD()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::launchPSD %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=qsasWindow->wlview->currentIndex();
	WLItem* item=qsasWindow->wlview->wlmodel()->itemFromIndex(index);

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	static PSD_Window *psdWindow = NULL;
	
	if(psdWindow == NULL) {
		psdWindow = new PSD_Window();
		psdWindow->setObject(path.toLatin1().data());
	}
	psdWindow->move(qsasGUISize().width(),30);
	psdWindow->show();
}


void mainMenu::launchPlot(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::launchPlot %ld\n", (long)this);
  fflush(stdout);
  }
 
  
  QpuiShow();
  
}

void mainMenu::launchPolarPlot(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::launchPolarPlot %ld\n", (long)this);
  fflush(stdout);
  }
 
  
  QpolarShow();
  
}


void mainMenu::launchSurfacePlot(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::launchSurfacePlot %ld\n", (long)this);
  fflush(stdout);
  }
 
  
  QsurfaceShow();
  
}

void mainMenu::launch3DPlot(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::launch3DPlot %ld\n", (long)this);
  fflush(stdout);
  }
 
  
  Q3dviewShow();
  
}

void mainMenu::launchJoin()
{ 
   QjuiShowJoin();
}


void mainMenu::launchCalculator()
{
   QcalcShowCalc();
}

void mainMenu::launchCalculatorPrototype()
{
	QcalcShowCalcProto();
}

void mainMenu::openDataFile(const char *file){
   
   QdaLaunchWindow(QString(file), 0);
   
}

void mainMenu::launchEditProfile()
{
	QProfileEditor * edit=new QProfileEditor;
	if(QPUI_widget!=NULL) edit->setQpuiWidget(QPUI_widget);
	connect(edit, SIGNAL(changedNames()), this, SLOT(updateProfileList()));
	if(QPUI_widget!=NULL)
	{
		connect(edit, SIGNAL(refreshColourTableButtons()), QPUI_widget, SIGNAL(refreshColourTableButtonsSignal()));
	}
}

void mainMenu::updateProfileList()
{
	clear();
	insertProfilesItems();
	QSAS_profiles->save();
}

void mainMenu::launchSelectProfile(QAction *act)
{
	if(QSAS_profiles->setCurrent(act->text()))
	{
		checkedProfile->setChecked(false);
		checkedProfile=act;
		act->setChecked(true);
		
		if(QSAS_profiles->current()->fontSize>=5)
		{
			QFont f;
			f.setPixelSize(QSAS_profiles->current()->fontSize);
			qApp->setFont(f);
		}
		else
		{
			qApp->setFont(defaultFont);
		}
		
	}

    if(QPUI_widget!=NULL)
    {
        QPUI_widget->refreshColourTableButtons();
    }
	
}

void mainMenu::angleDepTo360()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::angleDepTo360 %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=qsasWindow->wlview->currentIndex();
	WLItem* item=qsasWindow->wlview->wlmodel()->itemFromIndex(index);

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	AngleDepWindow *angleWindow = new AngleDepWindow(360);
	angleWindow->setObject(path.toLatin1().data());
	angleWindow->move(qsasGUISize().width(),30);
	angleWindow->show();
}

void mainMenu::angleDepTo180()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::angleDepTo180 %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=qsasWindow->wlview->currentIndex();
	WLItem* item=qsasWindow->wlview->wlmodel()->itemFromIndex(index);

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	AngleDepWindow *angleWindow = new AngleDepWindow(180);
	angleWindow->setObject(path.toLatin1().data());
	angleWindow->move(qsasGUISize().width(),30);
	angleWindow->show();
}

void mainMenu::arrayTrace()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::launchTrace %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=qsasWindow->wlview->currentIndex();
	WLItem* item=qsasWindow->wlview->wlmodel()->itemFromIndex(index);

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	ArrayTraceWindow *TraceWindow = new ArrayTraceWindow(0);
	TraceWindow->setObject(path.toLatin1().data());
	TraceWindow->move(qsasGUISize().width(),30);
	TraceWindow->show();
}

void mainMenu::DVOS_Test()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::launchDVOS_Test %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=qsasWindow->wlview->currentIndex();
	WLItem* item=qsasWindow->wlview->wlmodel()->itemFromIndex(index);

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	DVOS_TestWindow *testWindow = new DVOS_TestWindow(0);
	testWindow->setObject(path.toLatin1().data());
	testWindow->move(qsasGUISize().width(),30);
	testWindow->show();
}

void mainMenu::regression()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::regression %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=qsasWindow->wlview->currentIndex();
	WLItem* item=qsasWindow->wlview->wlmodel()->itemFromIndex(index);

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	RegressionWindow *regressionWindow = new RegressionWindow();
	regressionWindow->setObject(path.toLatin1().data());
	regressionWindow->move(qsasGUISize().width(),30);
	regressionWindow->show();
}

void mainMenu::derivative()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::derivative %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=qsasWindow->wlview->currentIndex();
	WLItem* item=qsasWindow->wlview->wlmodel()->itemFromIndex(index);

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	DerivativeWindow *derivativeWindow = new DerivativeWindow();
	derivativeWindow->setObject(path.toLatin1().data());
	derivativeWindow->move(qsasGUISize().width(),30);
	derivativeWindow->show();
}

void mainMenu::integral()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::integral %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=qsasWindow->wlview->currentIndex();
	WLItem* item=qsasWindow->wlview->wlmodel()->itemFromIndex(index);

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	IntegralWindow *integralWindow = new IntegralWindow();
	integralWindow->setObject(path.toLatin1().data());
	integralWindow->move(qsasGUISize().width(),30);
	integralWindow->show();
}


void mainMenu::launchTTOffset()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::launchTTOffset %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=qsasWindow->wlview->currentIndex();
	WLItem* item=qsasWindow->wlview->wlmodel()->itemFromIndex(index);

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	TTagToScalarWindow *ttWindow = new TTagToScalarWindow();
	ttWindow->setObject(path.toLatin1().data());
	ttWindow->move(qsasGUISize().width(),30);
	ttWindow->show();
	
}

void mainMenu::launchTItoSec()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::launchTItoSec %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=qsasWindow->wlview->currentIndex();
	WLItem* item=qsasWindow->wlview->wlmodel()->itemFromIndex(index);

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	TIToSecWindow *ttWindow = new TIToSecWindow();
	ttWindow->setObject(path.toLatin1().data());
	ttWindow->move(qsasGUISize().width(),30);
	ttWindow->show();
	
}

void mainMenu::launchT1T2toSec()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::launchT1T2toSec %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=qsasWindow->wlview->currentIndex();
	WLItem* item=qsasWindow->wlview->wlmodel()->itemFromIndex(index);

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	T1T2ToSecWindow *ttWindow = new T1T2ToSecWindow();
	ttWindow->setObjectStart(path.toLatin1().data());
	ttWindow->setObjectEnd(path.toLatin1().data());
	ttWindow->move(qsasGUISize().width(),30);
	ttWindow->show();
	
}

void mainMenu::launchTT1TT2toSec()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::launchT1T2toSec %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=qsasWindow->wlview->currentIndex();
	WLItem* item=qsasWindow->wlview->wlmodel()->itemFromIndex(index);

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	TT1TT2ToSecWindow *ttWindow = new TT1TT2ToSecWindow();
	ttWindow->setObjectStart(path.toLatin1().data());
	ttWindow->move(qsasGUISize().width(),30);
	ttWindow->show();
	
}


void mainMenu::mergeTS()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::mergeTS %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=qsasWindow->wlview->currentIndex();
	WLItem* item=qsasWindow->wlview->wlmodel()->itemFromIndex(index);

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	MergeTSWindow *toMergeWindow = new MergeTSWindow(0);
	toMergeWindow->setObject(path.toLatin1().data());
	toMergeWindow->move(qsasGUISize().width(),30);
	toMergeWindow->show();
}

void mainMenu::vect2array()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::vec2array %ld\n", (long)this);
		fflush(stdout);
	}
	
	QModelIndex index=qsasWindow->wlview->currentIndex();
	WLItem* item=qsasWindow->wlview->wlmodel()->itemFromIndex(index);

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();

	// ask user for selection 
	MakeVectorWindow *toVectorWindow = new MakeVectorWindow(0);
	toVectorWindow->setObject(path.toLatin1().data());
	toVectorWindow->move(qsasGUISize().width(),30);
	toVectorWindow->show();  

}
