#include "qpui.h"
#include "qdutil.h"
#include "qbu.h"

extern QPalette QPUI_normal_palette;

PlotEditor::PlotEditor(QObject *parent):
		QItemDelegate(parent)
{
	
}

QWidget * PlotEditor::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
	QLineEdit *e=new QLineEdit(parent);
	e->setFrame(false);
	return e;
}

void PlotEditor::setEditorData(QWidget *editor, const QModelIndex &index) const
{
	QString value = index.model()->data(index, Qt::DisplayRole).toString();

	QLineEdit *edit = static_cast<QLineEdit*>(editor);
	edit->setText(value);
}
		
void PlotEditor::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
{
	QLineEdit *edit = static_cast<QLineEdit*>(editor);
	QString value = edit->text();

	model->setData(index, value, Qt::EditRole);
}

void PlotEditor::updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &/*index*/) const
{
	editor->setGeometry(option.rect);
}


PlotView::PlotView(QWidget* parent):
	QTreeView(parent)
{
	setDragEnabled(true);
	setAcceptDrops(true);
	setDropIndicatorShown(true);
	setUniformRowHeights(true);
	setSelectionMode(QAbstractItemView::ExtendedSelection);
 
 	copyDoc = new QDomDocument("plotCopy");

	header()->setStretchLastSection(false);
//	header()->setResizeMode(QHeaderView::ResizeToContents);
	
	isLink = false; // global available to every DataTab for use in linking
	
	delegate=new PlotEditor(parent);
	setItemDelegate(delegate);
	
// 	// Set Right Button Menu
// 	
	RButtonMenu=new QMenu(this);
// 	
#if QT_VERSION >= 0x040300
	setEditTriggers(QAbstractItemView::NoEditTriggers);
#endif

	connect(this, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(openObjectEditor(const QModelIndex&)));
	
	I_LastTab = QP_IGNORE_TAB;
	dropRow = 0;
}

void PlotView::dragEnterEvent(QDragEnterEvent *event)
{
	if(QuSlotDebug()) printf("QpListView::dragEnterEvent %ld\n", (long)this);

	event->acceptProposedAction();
	emit raiseWindow();

}
void PlotView::dragMoveEvent(QDragMoveEvent *event)
{
	QModelIndex idx = indexAt(event->pos());

	dropRow = idx.row();
	
}


void PlotView::displayMenu(const QPoint& p)
{
	RButtonMenu->clear();
	
	QMenu* subMenu=RButtonMenu->addMenu("New");
	subMenu->addAction("Trace", this, SLOT(makeTraceItem()));
	subMenu->addAction("Spectrogram", this, SLOT(makeSpectrumItem()));
	subMenu->addSeparator();
	subMenu->addAction("Constant X or T Trace", this, SLOT(makeConstXTraceItem()));
	subMenu->addAction("Constant Y Trace", this, SLOT(makeConstYTraceItem()));
	subMenu->addAction("Regression Line", this, SLOT(makeRegLineTraceItem()));
	subMenu->addSeparator();
	subMenu->addAction("Event Label", this, SLOT(makeEventItem()));
	subMenu->addSeparator();
	subMenu->addAction("Panel", this, SLOT(makePanelItem()));
	subMenu->addSeparator();
	subMenu->addAction("Time Series Frame", this, SLOT(makeTSFrameItem()));
	subMenu->addAction("XY Frame", this, SLOT(makeXYFrameItem()));
	subMenu->addSeparator();
	subMenu->addAction("Page", this, SLOT(makePageItem()));
	
	RButtonMenu->addSeparator();
	RButtonMenu->addAction("Rename", this, SLOT(renameSelectedItem()));
	RButtonMenu->addAction("Cut", this, SLOT(cut()));
	RButtonMenu->addAction("Copy", this, SLOT(copy()));
	RButtonMenu->addAction("Paste", this, SLOT(paste()));
	RButtonMenu->addSeparator();
	RButtonMenu->addAction("Move Up", this, SLOT(moveItemUp()));
	RButtonMenu->addAction("Move Down", this, SLOT(moveItemDown()));
	RButtonMenu->addSeparator();
	RButtonMenu->addAction("Plot Selected Pages", this, SLOT(plotSelectedItems()));
	RButtonMenu->addSeparator();
	RButtonMenu->addAction("Use Default Settings", this, SLOT(setDefaultSelectedItems()));
	RButtonMenu->addSeparator();
	RButtonMenu->addAction("Delete", this, SLOT(deleteSelectedItems()));
	RButtonMenu->addSeparator();
	RButtonMenu->addAction("Link Selected Traces", this, SLOT(linkPlots()));
	RButtonMenu->addAction("Unlink Selected Traces", this, SLOT(unlinkPlots()));
	
	
	
	RButtonMenu->exec(p);
}

void PlotView::mousePressEvent(QMouseEvent* event)
{
	QTreeView::mousePressEvent(event);
	if(event->button()==Qt::RightButton)
	{
		displayMenu(event->globalPos());
	}
}

PageItem* PlotView::makePage()
{
	clearSelection();
	PageItem* pageItem = plotModel()->makePage();

	return pageItem;
}

void PlotView::currentChanged(const QModelIndex& current, const QModelIndex& previous)
{
	QTreeView::currentChanged(current, previous);
	updateOnSelection();
}

void PlotView::setCurrentItem(QpuiItem* i)
{
	QModelIndex index=plotModel()->indexFromItem(i);
	if(index.isValid())
	{
 		selectionModel()->clear();
		selectionModel()->setCurrentIndex(/*currentIndex()*/index, QItemSelectionModel::SelectCurrent);
	}
}

void PlotView::updateOnSelection()
{
	QWidget * oldPage = this->tabWidget->currentWidget();
	if(oldPage)
	{
		QpTab * oldTab = dynamic_cast <QpTab*>(oldPage);
		if(this->I_LastTab == QP_IGNORE_TAB) this->I_LastTab = QP_DATA_TAB;
		else this->I_LastTab = oldTab->TabType;
	}

	// sets tabs appropriate when selection is changed
	QpuiItem *item = dynamic_cast<QpuiItem*>(this->currentItem());
	if(QuSlotDebug())
	{
		printf("PlotView::updateOnSelection %ld %ld\n", (long)this, (long)item);
		fflush(stdout);
	}

	// show tabs for different item types via virtual method
	if(item) item->showTabs(); 
	this->tabWidget->adjustSize();
}

void PlotView::setDefaultSelectedItems()
{
	//QModelIndexList list=selectionModel()->selectedIndexes();
// 	QList<QPersistentModelIndex> list=currentSelection();
// 	foreach(QModelIndex index, list)
// 	{
// 		QpuiItem* item=plotModel()->itemFromIndex(index);
// 		if(item!=NULL) item->setDefaultOptions();
// 	}
	QList<QpuiItem*> list=selectedItems();
	foreach(QpuiItem* item, list) item->setDefaultOptions();
}

void PlotView::plotSelectedItems()
{
	_plot_page_list.clear();
	
	QList<QpuiItem*> list=selectedItems();
	
	if(list.isEmpty()) return;
	
	foreach(QpuiItem* item, list)
	{
		PageItem* pageItem=NULL;
		if(dynamic_cast<PlotItem*>(item)!=NULL) pageItem=((PlotItem*)item)->page();
		if(dynamic_cast<PanelItem*>(item)!=NULL) pageItem=((PanelItem*)item)->page();
		if(dynamic_cast<PanelItem*>(item)!=NULL) pageItem=((PanelItem*)item)->page();
		if(dynamic_cast<PageItem*>(item)!=NULL) pageItem=(PageItem*)item;
		
		if(pageItem && !_plot_page_list.contains(pageItem))
		{
			_plot_page_list.push_back(pageItem);
		}
	}
	
	
	if(!_plot_page_list.isEmpty()) plot();
}

void PlotView::plot()
{
	PageItem* page_item;
	
	if(_plot_page_list.isEmpty())
	{
		(void)QMessageBox::warning(this,"Plot",("No Pages Selected..."));
		return;
	}
		
	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
	qApp->processEvents();
	
	try{
		for(uint i=0; i<_plot_page_list.size(); i++)
		{
			page_item = _plot_page_list[i];
			if(!page_item) continue;
			
			double width, height;
			double conv;
			
			width = page_item->PageTab->_width_fld->text().toDouble();
			height =page_item->PageTab->_height_fld->text().toDouble();
			
			int _units = page_item->PageTab->_units_fld->currentIndex();
			conv=page_item->_conversion_factors[_units];
			width=width/conv; //conversion to inches
			height=height/conv; //conversion to inches
			
			conv=page_item->_conversion_factors[0]; // conv factor to pixels
			width=width*conv; //px
			height=height*conv; //px
			
			if(width<=0. || height<=0.)
			{
				width=800.;
				height=600.;
			}
			
		
			if((page_item->PageTab->_orientation_fld->currentIndex() == QPUI_LANDSCAPE && width<height) || (page_item->PageTab->_orientation_fld->currentIndex() == QPUI_PORTRAIT && width>height) )
			{
				double temp=width;
				width=height;
				height=temp;
			}
			
			PlotWindow* win;
			int plot_id=page_item->PageTab->_xwin_num_fld->text().toInt();
			QPrinter::Orientation orientation=(page_item->PageTab->_orientation_fld->currentIndex() == QPUI_LANDSCAPE ? QPrinter::Landscape : QPrinter::Portrait);
			QPrinter::PageSize pageSize=QPrinter::A4;
			int resolution=72;
			switch(page_item->PageTab->_page_size_fld->currentIndex())
			{
				case QPUI_EXECUTIVE:
					pageSize=QPrinter::Executive;
					break;
				case QPUI_LEGAL:
					pageSize=QPrinter::Legal;
					break;
				case QPUI_LETTER:
					pageSize=QPrinter::Letter;
					break;
				case QPUI_CUSTOM:
					pageSize=QPrinter::Custom;
					break;
				default: break;
			}
			
			switch(page_item->PageTab->_dpi_fld->currentIndex())
			{
				case QPUI_150_DPI: resolution=150; break;
				case QPUI_300_DPI: resolution=300; break;
				case QPUI_600_DPI: resolution=600; break;
				case QPUI_1200_DPI: resolution=1200; break;
				default: resolution=72; break;
			}
			
			if(plot_id < 0) // create new window
			{
				win=page_item->newWindow(width, height, &(plot_id), resolution, pageSize, orientation);
				page_item->PageTab->setXwinNum(QString::number(plot_id));
				page_item->PageTab->_xwin_num_fld->setText(QString::number(plot_id));
			}
			else
			{
				win=page_item->focusWindow(plot_id, width, height, resolution, pageSize, orientation);
			}
			
			win->disconnect(SIGNAL(clicked(double, double, int, int))); // disconnect possibly pre-existing connections
			
#ifdef Q_OS_MACX
			connect(win, SIGNAL(doubleClicked(double, double, int, int)), page_item, SLOT(displayIntervalWidget(double, double, int, int)));
#else
			connect(win, SIGNAL(clicked(double, double, int, int)), page_item, SLOT(displayIntervalWidget(double, double, int, int)));
#endif

			qApp->processEvents();

			
			if(QPUI_FAILURE == page_item->plot(win))
			{
				(void)QMessageBox::warning(this,"Plot", (QString("Problem plotting page %1").arg(page_item->getName())));
			}


// fix Mac & Windows problem with spectra not showing until focus change
			else 
			{
				
				emit raiseWindow();
				qApp->processEvents();

				win->activateWindow();
  				win->raise();
				
			}

		}
	}
	catch(...)
	{;}
	
	
	QApplication::restoreOverrideCursor();
	qApp->processEvents();
		
}

QList<QPersistentModelIndex> PlotView::selectedIndexes()
{
	QModelIndexList indexes=selectionModel()->selectedRows(); // Currently selected indexes
	
	QList<QPersistentModelIndex> res;
	res.clear();
	// We add indexes for all the selected indexes and the current one, relatively to column 0 only, and only once each.
	
	if(!indexes.contains(currentIndex())) indexes.push_back(currentIndex());
	
	foreach(QModelIndex index, indexes)
	{
		QpuiItem* item=plotModel()->itemFromIndex(index);
		if(item!=NULL)
		{
			QPersistentModelIndex i(plotModel()->indexFromItem(item)); // indexFromItem sets the column to 0
			res.push_back(i);
		}
	}
	return res;
}

/*
void PlotView::copy()
{
// 	QModelIndexList indexes=selectionModel()->selectedIndexes();
	QList<QPersistentModelIndex> indexes=selectedIndexes();
	if(!indexes.isEmpty())
	{
		QModelIndexList list;
		foreach(QModelIndex index, indexes) list.push_back(index);
		QMimeData* data=plotModel()->mimeData(list);
		QApplication::clipboard()->setMimeData(data);
	}
}

void PlotView::paste()
{
	QModelIndex parent=currentIndex();
	if(!parent.isValid()) return;
	
	plotModel()->dropMimeData(QApplication::clipboard()->mimeData(), Qt::MoveAction, parent.row(), 0, parent);
}
*/
void PlotView::cut()
{  
	copyDoc->clear();
	QDomElement docElem=copyDoc->createElement("copyItems");
	copyDoc->appendChild(docElem);

	QList<QPersistentModelIndex> indexes = selectedIndexes();
	
	docElem.setAttribute("size", QString::number(indexes.size()));
	if(!indexes.isEmpty())
	{
		foreach(QModelIndex index, indexes){
			docElem.appendChild(plotModel()->itemFromIndex(index)->toXMLElement(*copyDoc));
		}
	}
	deleteSelectedItems();

}

void PlotView::copy()
{  
	copyDoc->clear();
	QDomElement docElem=copyDoc->createElement("copyItems");
	copyDoc->appendChild(docElem);

	QList<QPersistentModelIndex> indexes = selectedIndexes();
	
	docElem.setAttribute("size", QString::number(indexes.size()));
	if(!indexes.isEmpty())
	{
		foreach(QModelIndex index, indexes){
			docElem.appendChild(plotModel()->itemFromIndex(index)->toXMLElement(*copyDoc));
		}
	}
}


void PlotView::paste()
{  
	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );

	plotModel()->_replace=false;
	
	bool newFrame=false;
	
	QDomElement docElem = copyDoc->documentElement();
	if(!docElem.isNull())
	{
	    int cnt=docElem.attribute("size", "0").toInt();
		
		QDomElement pageElement=docElem.firstChildElement("Page_Item");
		if(!pageElement.isNull())
		{
			while (!pageElement.isNull()){
				PageItem* page = plotModel()->makePage();
				blockSignals(true);
				page->fromXMLElement(pageElement);
				blockSignals(false);
	
				pageElement=pageElement.nextSiblingElement("Page_Item");
			}
			QApplication::restoreOverrideCursor();
			return;
		}

        plotModel()->emitLayoutAboutToBeChanged();
		
		QpuiItem *target = currentItem();
		if(!target) {
			cout << "No item selected as paste target in plot" << endl;
			return;
		}
		
		blockSignals(true);


		// Page target
		PageItem *page = dynamic_cast <PageItem*>(target); 
		if(page){
			QDomElement element=docElem.firstChildElement("Frame_Item");
			if(element.isNull())  QuMsgBar("Can only paste frame into page", FALSE);
			while(!element.isNull())
			{
				newFrame = true;
				FrameItem *frameItem = new FrameItem("", page);
				frameItem->fromXMLElement(element);
	
				element=element.nextSiblingElement("Frame_Item");
			}
			if(newFrame) {
				page->LayoutTab->updateRowCol(page->numChildren());
				page->setLayout();
			}
		}
		
		// Frame target
		FrameItem *frame = dynamic_cast <FrameItem*>(target); 
		if(frame){
			QDomElement panelElement=docElem.firstChildElement("Panel_Item");
			QDomElement frameElement=docElem.firstChildElement("Frame_Item");
			
			if(panelElement.isNull() && frameElement.isNull())  QuMsgBar("Cannot paste this into frame", FALSE);
			while(!panelElement.isNull())
			{
				PanelItem *panelItem = new PanelItem("", frame);
				panelItem->fromXMLElement(panelElement);
	
				panelElement=panelElement.nextSiblingElement("Panel_Item");
			}
			while(!frameElement.isNull())
			{
				newFrame = true;
				FrameItem *frameItem = new FrameItem("", frame->page());
				frameItem->fromXMLElement(frameElement);
	
				frameElement=frameElement.nextSiblingElement("Frame_Item");
			}
			if(newFrame) {
				frame->page()->LayoutTab->updateRowCol(frame->page()->numChildren());
				frame->page()->setLayout();
			}

		}

		// Panel target
		PanelItem *panel = dynamic_cast <PanelItem*>(target); 
		PlotItem *plotItem = dynamic_cast <PlotItem*>(target);
		if(plotItem) panel = plotItem->panel();
		
		if(panel){
			QDomElement plotElement=docElem.firstChildElement("Plot_Item");
			QDomElement panelElement=docElem.firstChildElement("Panel_Item");
			QDomElement frameElement=docElem.firstChildElement("Frame_Item");
			if(plotElement.isNull() && panelElement.isNull() && frameElement.isNull())  QuMsgBar("Cannot paste this into panel", FALSE);

			while(!plotElement.isNull())
			{
        		QString type=plotElement.attribute("Type", "");
        		if(type=="Trace")
        		{
					TraceItem *traceItem = new TraceItem("", panel, "", QP_TRACE);
            		traceItem->fromXMLElement(plotElement);
        		}
        		else if(type=="Spectrogram")
        		{
            		SpectrumItem *spectrumItem = new SpectrumItem("", panel);
					spectrumItem->fromXMLElement(plotElement);
				}
				else if(type=="Const_Y")
				{
					TraceItem *constYItem = new TraceItem("", panel, "", QP_CONST_Y);
					constYItem->fromXMLElement(plotElement);
				}
				else if(type=="Const_XT")
				{
 					TraceItem *constXTItem = new TraceItem("", panel, "", QP_CONST_XT);
					constXTItem->fromXMLElement(plotElement);
				}
				else if(type=="Reg_Line")
				{
 					TraceItem *regLineItem = new TraceItem("", panel, "", QP_REG_LINE);
					regLineItem->fromXMLElement(plotElement);
				}
				else if(type=="Event")
				{
					EventItem *eventItem = new EventItem("", panel);
					eventItem->fromXMLElement(plotElement);
				}
				plotElement=plotElement.nextSiblingElement("Plot_Item");
			}
			while(!panelElement.isNull())
			{
				PanelItem *panelItem = new PanelItem("", panel->frame());
				panelItem->fromXMLElement(panelElement);
	
				panelElement=panelElement.nextSiblingElement("Panel_Item");
			}
			while(!frameElement.isNull())
			{
				newFrame=true;
				FrameItem *frameItem = new FrameItem("", panel->page());
				frameItem->fromXMLElement(frameElement);
	
				frameElement=frameElement.nextSiblingElement("Frame_Item");
			}
			if(newFrame)  {
				panel->page()->LayoutTab->updateRowCol(panel->page()->numChildren());
				panel->page()->setLayout();
			}
		}
		blockSignals(false);
        plotModel()->emitLayoutChanged();
	}
	QApplication::restoreOverrideCursor();

}

void PlotView::setPageOption(int option, int iChoice, float fChoice, const QString& sChoice, bool bChoice, const QColor& cChoice)
{
	if(QuSlotDebug()) cout << "PlotView::setPageOption" << endl;

	QList<QpuiItem*> list=selectedItems();
	foreach(QpuiItem* item, list)
	{
		PageItem* page_item=dynamic_cast<PageItem*>(item);
    
		if(!page_item) continue;
		
		// don't set Page of visible tab - user just did this - avoid infinite loop
		if(page_item->PageTab->isVisible()) continue;
		if(page_item->LayoutTab->isVisible()) continue;
		
		switch(option)
		{
		
			// Layout Tab
			case AUTOLAYOUT:
				page_item->LayoutTab->_autolayout_fld->setChecked(bChoice);
				page_item->setFramesAuto(bChoice);

				break;
			case NUM_ROWS:
				page_item->LayoutTab->_row_num_fld->setValue(iChoice);
				break;
			case NUM_COLS:
				page_item->LayoutTab->_col_num_fld->setValue(iChoice);
				break;
			case HGAP:
				page_item->LayoutTab->_hgap_fld->setValue(fChoice);
				break;
			case VGAP:
				page_item->LayoutTab->_vgap_fld->setValue(fChoice);
				break;
			case LMARGIN:
				page_item->LayoutTab->_lmargin_fld->setValue(fChoice);
				break;
			case RMARGIN:
				page_item->LayoutTab->_rmargin_fld->setValue(fChoice);
				break;
			case TMARGIN:
				page_item->LayoutTab->_tmargin_fld->setValue(fChoice);
				break;
			case BMARGIN:
				page_item->LayoutTab->_bmargin_fld->setValue(fChoice);
				break;
				
			// Page Tab
			case PAGE_SIZE:
				page_item->PageTab->_page_size_fld->setCurrentIndex(iChoice);
                page_item->updateIsotropicFrames();
				break;
			case PAGE_UNITS:
				page_item->PageTab->_units_fld->setCurrentIndex(iChoice);
				break;
			case PAGE_DPI:
				page_item->PageTab->_dpi_fld->setCurrentIndex(iChoice);
				break;
			case PAGE_WIDTH:
				page_item->PageTab->_width_fld->setText(sChoice);
                page_item->updateIsotropicFrames();
				break;
			case PAGE_HEIGHT:
				page_item->PageTab->_height_fld->setText(sChoice);
                page_item->updateIsotropicFrames();
				break;
			case PAGE_ORIENTATION:
				page_item->PageTab->_orientation_fld->setCurrentIndex(iChoice);
                page_item->updateIsotropicFrames();
				break;
			case XWIN_NUM:
				page_item->PageTab->_xwin_num_fld->setText(tr("%1").arg(iChoice));
				break;
			case IDENT:
				page_item->PageTab->_show_ident_fld->setChecked(bChoice);
				break;
			default:
				break;
				
		}
		page_item->setLayout();

	} // end for over local list
}
		
void PlotView::setFrameOption(int option, int iChoice, float fChoice, const QString& sChoice, bool bChoice, const QColor& cChoice)
{
	if(QuSlotDebug()) cout << "PlotView::setFrameOption" << endl;

	// apply Panel option to all selected panels (that belonging to visible tab is always selected)

	QList<QpuiItem*> list=selectedItems();
    double page_ratio;
	foreach(QpuiItem* item, list)
	{
		FrameItem* frame_item=dynamic_cast<FrameItem*>(item);
    
		if( !frame_item) continue;
	  
		// don't set panel for visible tab - user just did this - avoid infinite loop
		if(frame_item->PropertiesTab->isVisible()) continue;
		if(frame_item->DisplayTab->isVisible()) continue;
		if(frame_item->TitleTab->isVisible()) continue;
	  
		switch(option)
		{
			// Display Tab
			case PANEL_LINE_WIDTH:
				frame_item->DisplayTab->_line_width_fld->setValue(fChoice);
				break;
			case PANEL_LINE_COLOUR:
			{
				QPalette line_palette = QPalette(QPUI_normal_palette);
				line_palette.setColor(QPalette::Active, QPalette::ButtonText, cChoice);
				line_palette.setColor(QPalette::Inactive, QPalette::ButtonText, cChoice);
				frame_item->DisplayTab->_line_colour_fld->setPalette(line_palette);
				frame_item->_line_colour = cChoice;
				break;
			}
			case TICKS_LENGTH:
				frame_item->DisplayTab->_ticks_length_fld->setValue(fChoice);
				break;
			case TOP_AXIS:
				frame_item->DisplayTab->_show_top_axis_fld->setChecked(bChoice);
				break;
			case BOTTOM_AXIS:
				frame_item->DisplayTab->_show_bottom_axis_fld->setChecked(bChoice);
				break;
			case LEFT_AXIS:
				frame_item->DisplayTab->_show_left_axis_fld->setChecked(bChoice);
				break;
			case RIGHT_AXIS:
				frame_item->DisplayTab->_show_right_axis_fld->setChecked(bChoice);
				break;
			case LR_ALTERNATING_AXIS:
				frame_item->DisplayTab->_show_lr_alternating_axis_fld->setChecked(bChoice);
				break;
			case RL_ALTERNATING_AXIS:
				frame_item->DisplayTab->_show_rl_alternating_axis_fld->setChecked(bChoice);
				break;
			case TOP_TICKS_LABELS:
				frame_item->DisplayTab->_show_top_ticks_labels_fld->setChecked(bChoice);
				break;
			case BOTTOM_TICKS_LABELS:
				frame_item->DisplayTab->_show_bottom_ticks_labels_fld->setChecked(bChoice);
				break;
			case LEFT_TICKS_LABELS:
				frame_item->DisplayTab->_show_left_ticks_labels_fld->setChecked(bChoice);
				break;
			case RIGHT_TICKS_LABELS:
				frame_item->DisplayTab->_show_right_ticks_labels_fld->setChecked(bChoice);
				break;
			case LR_ALTERNATING_TICKS_LABELS:
				frame_item->DisplayTab->_show_lr_alternating_ticks_labels_fld->setChecked(bChoice);
				break;
			case RL_ALTERNATING_TICKS_LABELS:
				frame_item->DisplayTab->_show_rl_alternating_ticks_labels_fld->setChecked(bChoice);
				break;
			case TOP_LABELS:
				frame_item->DisplayTab->_show_top_labels_fld->setChecked(bChoice);
				break;
			case BOTTOM_LABELS:
				frame_item->DisplayTab->_show_bottom_labels_fld->setChecked(bChoice);
				break;
			case LEFT_LABELS:
				frame_item->DisplayTab->_show_left_labels_fld->setChecked(bChoice);
				break;
			case RIGHT_LABELS:
				frame_item->DisplayTab->_show_right_labels_fld->setChecked(bChoice);
				break;
			case LR_ALTERNATING_LABELS:
				frame_item->DisplayTab->_show_lr_alternating_labels_fld->setChecked(bChoice);
				break;
			case RL_ALTERNATING_LABELS:
				frame_item->DisplayTab->_show_rl_alternating_labels_fld->setChecked(bChoice);
				break;
			case Y_IS_ZERO:
				frame_item->DisplayTab->_show_yiszero_fld->setChecked(bChoice);
				break;
			case X_IS_ZERO:
				frame_item->DisplayTab->_show_xiszero_fld->setChecked(bChoice);
				break;
			//  Frame Properties Tab
			case LOFFSET:
				frame_item->PropertiesTab->_loffset_fld->setValue(fChoice);
				break;
			case TOFFSET:
				frame_item->PropertiesTab->_toffset_fld->setValue(fChoice);
				break;
			case WIDTH:
				frame_item->PropertiesTab->_width_fld->setValue(fChoice);
                switch(frame_item->PropertiesTab->_aspect_ratio_fld->currentIndex())
                {
                    case QPUI_FIXED:
                        frame_item->PropertiesTab->_height_fld->setValue(fChoice*frame_item->PropertiesTab->_aspect_ratio);
                        break;

                    case QPUI_SQUARE_ASPECT:
                        page_ratio=frame_item->page()->aspectRatio();
                        frame_item->PropertiesTab->_height_fld->setValue(fChoice*page_ratio);
                        break;

                    case QPUI_ISOTROPIC:
                        frame_item->PropertiesTab->_height_fld->setValue(fChoice*frame_item->PropertiesTab->_aspect_ratio);
                        break;

                    default:
                        frame_item->PropertiesTab->_aspect_ratio = frame_item->PropertiesTab->_height_fld->value()/fChoice;
                        break;
                }
// 				if(frame_item->PropertiesTab->_aspect_ratio_fld->isChecked()) frame_item->PropertiesTab->_height_fld->setValue(fChoice*frame_item->PropertiesTab->_aspect_ratio);
// 				else frame_item->PropertiesTab->_aspect_ratio = frame_item->PropertiesTab->_height_fld->value()/fChoice;
				break;
			case HEIGHT:
				frame_item->PropertiesTab->_height_fld->setValue(fChoice);
                switch(frame_item->PropertiesTab->_aspect_ratio_fld->currentIndex())
                {
                    case QPUI_FIXED:
                        frame_item->PropertiesTab->_width_fld->setValue(fChoice/frame_item->PropertiesTab->_aspect_ratio);
                        break;

                    case QPUI_SQUARE_ASPECT:
                        page_ratio=frame_item->page()->aspectRatio();
                        frame_item->PropertiesTab->_width_fld->setValue(fChoice/page_ratio);
                        break;

                    case QPUI_ISOTROPIC:
                       frame_item->PropertiesTab->_width_fld->setValue(fChoice/frame_item->PropertiesTab->_aspect_ratio);
                        break;

                    default:
                        frame_item->PropertiesTab->_aspect_ratio = fChoice/frame_item->PropertiesTab->_width_fld->value();
                        break;
                }
// 				if(frame_item->PropertiesTab->_aspect_ratio_fld->isChecked()) frame_item->PropertiesTab->_width_fld->setValue(fChoice/frame_item->PropertiesTab->_aspect_ratio);
// 				else frame_item->PropertiesTab->_aspect_ratio = fChoice/frame_item->PropertiesTab->_width_fld->value();
				break;
			case ASPECT_RATIO:
				frame_item->PropertiesTab->_aspect_ratio_fld->setCurrentIndex(iChoice);
                if(iChoice==QPUI_SQUARE_ASPECT)
                {
                    page_ratio=frame_item->page()->aspectRatio();
                    frame_item->PropertiesTab->_width_fld->setValue(frame_item->PropertiesTab->_height_fld->value()/page_ratio);
                }
                if(iChoice==QPUI_ISOTROPIC)
                {
					double aspectRatio = frame_item->PropertiesTab->setIsotropicRatio();
					double newWidth = frame_item->PropertiesTab->_height_fld->value()/aspectRatio;
					double l_offset = frame_item->PropertiesTab->_loffset_fld->value();
					if (newWidth + l_offset > 0.99)
					{
						newWidth = 0.99 - l_offset;
						double newHeight = newWidth * aspectRatio;
                    	frame_item->PropertiesTab->_height_fld->setValue(newHeight);
					}

					frame_item->PropertiesTab->_width_fld->setValue(newWidth);
                }
				break;
			case FRAME_TYPE:
				frame_item->PropertiesTab->_frame_type_fld->setCurrentIndex(iChoice);
				break;
			//  Title Tab
			case AUTOTITLE:
				frame_item->TitleTab->_autotitle_fld->setChecked(bChoice);
				break;
			case TITLE:
				frame_item->TitleTab->_title_fld->setText(sChoice);
				frame_item->TitleTab->_autotitle_fld->setChecked(false);
				break;
			case TITLE_CHAR_SIZE:
				frame_item->TitleTab->_title_char_size_fld->setValue(fChoice);
				break;
			case TITLE_DISP:
				frame_item->TitleTab->_title_disp_fld->setValue(fChoice);
				break;
			case SUBTITLE:
				frame_item->TitleTab->_subtitle_fld->setText(sChoice);
				break;
			case SUBTITLE_CHAR_SIZE:
				frame_item->TitleTab->_subtitle_char_size_fld->setValue(fChoice);
				break;
			case SUBTITLE_DISP:
				frame_item->TitleTab->_subtitle_disp_fld->setValue(fChoice);
				break;
			case TITLE_JUST:
				frame_item->TitleTab->_title_just_fld->setCurrentIndex(iChoice);
				break;
			case TITLE_FONT_FAMILY:
				switch(iChoice)
				{
					case PL_FCI_SERIF:
						frame_item->TitleTab->_title_font_fld->setSerifFont();
						break;
					case PL_FCI_MONO:
						frame_item->TitleTab->_title_font_fld->setMonotypeFont();
						break;
					case PL_FCI_SCRIPT:
						frame_item->TitleTab->_title_font_fld->setDecorativeFont();
						break;
					default:
						frame_item->TitleTab->_title_font_fld->setSansFont();
						break;
				}
				break;
			case TITLE_FONT_STYLE:
				frame_item->TitleTab->_title_font_fld->setItalic(bChoice);
				break;
			case TITLE_FONT_WEIGHT:
				frame_item->TitleTab->_title_font_fld->setBold(bChoice);
				break;
			case TITLE_COLOUR:
			{
				QPalette line_palette = QPalette(QPUI_normal_palette);
				line_palette.setColor(QPalette::Active, QPalette::ButtonText, cChoice);
				line_palette.setColor(QPalette::Inactive, QPalette::ButtonText, cChoice);
				frame_item->TitleTab->_title_colour_fld->setPalette(line_palette);
				frame_item->_title_colour = cChoice;
				break;
			}
			case TICKS_CHAR_SIZE:
				frame_item->DisplayTab->_ticks_char_size_fld->setValue(fChoice);
				break;
			default:
				break;
				
		}
	} // end for over local list
}
		
void PlotView::setTraceOption(int option, int iChoice, float fChoice, const QString& sChoice, bool bChoice, const QColor& cChoice)
{
	if(QuSlotDebug()) cout << "QpListView::setTraceOption" << endl;

	// apply Trace option to all selected traces (that belonging to visible tab is always selected)
	QList<QPersistentModelIndex> indexes=selectedIndexes();
	
	foreach(QModelIndex index, indexes)
   	{
		TraceItem* tr_item=dynamic_cast<TraceItem*>(plotModel()->itemFromIndex(index));
	   
		if(tr_item==NULL) continue;
	   
		// don't set trace for visible tab - user just did this - avoid infinite loop
		if(index==currentIndex()) continue;
		if(tr_item->TraceTab->isVisible()) continue;
		if(tr_item->DataTab->isVisible()) continue;
		
		switch(option)
		{
			// Trace Tab
			case TRACE_LINE_STYLE:
			{
				tr_item->TraceTab->_line_style_fld->setCurrentIndex(iChoice);
				break;
			}
			case TRACE_LINE_WIDTH:
			{
				tr_item->TraceTab->_line_width_fld->setValue(fChoice);
				break;
			}
			case TRACE_CONNECT_STYLE: 
			{
				tr_item->TraceTab->_connect_style_fld->setCurrentIndex(iChoice);
				break;
			}
			case TRACE_LINE_SCALE:
			{
				tr_item->TraceTab->_line_scale_fld->setValue(fChoice);
				break;
			}
			case TRACE_LINE_COLOUR: 
			{
				QPalette line_palette = QPalette(QPUI_normal_palette);
				line_palette.setColor(QPalette::Active, QPalette::ButtonText, cChoice);
				line_palette.setColor(QPalette::Inactive, QPalette::ButtonText, cChoice);
				tr_item->TraceTab->_line_colour_fld->setPalette(line_palette);
				tr_item->_line_colour = cChoice;
				tr_item->userLineColour = true;
				break;
			}
			case TRACE_SYMBOL_SHAPE:
			{ 
				tr_item->TraceTab->_symbol_shape_fld->setCurrentIndex(iChoice);
				if(iChoice==20) // corresponds to "custom"
				{
					tr_item->TraceTab->_custom_symbol_fld->setEnabled(true);
				}
				else
				{
					tr_item->TraceTab->_custom_symbol_fld->setEnabled(false);
				}
				break;
			}
			case TRACE_CUSTOM_SYMBOL:
			{ 
				tr_item->TraceTab->_custom_symbol_fld->setText(sChoice);
				break;
			}
			case TRACE_SYMBOL_SIZE: 
			{
				tr_item->TraceTab->_symbol_size_fld->setValue(fChoice);
			}
			case TRACE_SYMBOL_COLOUR: 
			{
				QPalette line_palette = QPalette(QPUI_normal_palette);
				line_palette.setColor(QPalette::Active, QPalette::ButtonText, cChoice);
				line_palette.setColor(QPalette::Inactive, QPalette::ButtonText, cChoice);
				tr_item->TraceTab->_symbol_colour_fld->setPalette(line_palette);
				tr_item->_symbol_colour = cChoice;
				tr_item->userSymbolColour = true;
				break;
			}
			// Data Tab
			case TRACE_LEGEND_TEXT:
			{
				tr_item->DataTab->_legend_text_fld->setText(sChoice);
				break;
			}
			case TRACE_SHOW_LEGEND:
			{
				tr_item->DataTab->_show_legend_fld->setChecked(iChoice);
				break;
			}
			case TRACE_YDATA:
			{
				tr_item->replaceData(sChoice);
				break;
			}
			case TRACE_YCOMP:
			{
				tr_item->replaceComponent(sChoice);
				break;
			}
			case TRACE_TIDATA:
			{
				tr_item->replaceTIdata(sChoice);
				break;
			}
			case TRACE_TICOMP:
			{
				tr_item->replaceTIcomp(sChoice);
				break;
			}
			case TRACE_XDATA:
			{
				tr_item->replaceXData(sChoice);
				break;
			}
			case TRACE_XCOMP:
			{
				tr_item->replaceXComponent(sChoice);
				break;
			}
			case LEGEND_AUTO_TEXT:
			{
				tr_item->DataTab->_auto_legend_fld->setChecked(bChoice);
				break;
			}
            case TRACE_GAP_DETECTION:
            {
                tr_item->DataTab->_gap_detection_fld->setChecked(bChoice);
                break;
            }
            case TRACE_GAP_THRESHOLD:
            {
                tr_item->DataTab->_gap_threshold_fld->setValue(fChoice);
                break;
            }
            case TRACE_GAP_THRESHOLD_TYPE:
            {
                tr_item->DataTab->_gap_threshold_type_fld->setCurrentIndex(iChoice);
                break;
            }
            case TRACE_COL_COMP:
            {
				tr_item->replace_C_Component(sChoice);
                break;
            }
            case TRACE_COL_DATA:
            {
				tr_item->replaceCdata(sChoice);
                break;
            }
            case TRACE_COL_MIN:
            {
				tr_item->TraceTab->_c_min_fld->setText(sChoice);
                break;
            }
            case TRACE_COL_MAX:
            {
				tr_item->TraceTab->_c_max_fld->setText(sChoice);
                break;
            }
            case TRACE_SHOW_COL_BAR:
            {
				tr_item->TraceTab->_show_colour_bar->setChecked(bChoice);
                break;
            }
            case TRACE_AUTO_COL_BAR:
            {
				tr_item->TraceTab->c_range_auto->setChecked(bChoice);
                break;
            }
            case TRACE_LOG_COL_BAR:
            {
				tr_item->TraceTab->c_range_log->setChecked(bChoice);
                break;
            }
            case TRACE_SET_C_SIZE:
            {
				tr_item->TraceTab->_size_on->setChecked(bChoice);
                break;
            }
            case TRACE_SET_C_COL:
            {
				tr_item->TraceTab->_colour_on->setChecked(bChoice);
                break;
            }
            case TRACE_EXP_COL_BAR_X:
            {
				tr_item->TraceTab->_exp_label_offset_x->setValue(fChoice);
                break;
            }
            case TRACE_EXP_COL_BAR_Y:
            {
				tr_item->TraceTab->_exp_label_offset_y->setValue(fChoice);
                break;
            }
            case TRACE_SET_COL_TABLE:
            {
				tr_item->TraceTab->colourTable->setProfilePalette(iChoice);
                break;
            }
            case TRACE_USER_COL_LAB:
            {
				tr_item->TraceTab->colLabSlot->setText(sChoice);
                break;
            }
            case TRACE_COL_BAR_W:
            {
				tr_item->TraceTab->_colour_bar_width->setValue(fChoice);
                break;
            }
            case TRACE_COL_BAR_H:
            {
				tr_item->TraceTab->_colour_bar_height->setValue(fChoice);
                break;
            }
            case TRACE_COL_BAR_HOFF:
            {
				tr_item->TraceTab->_colour_bar_disp->setValue(fChoice);
                break;
            }
            case TRACE_COL_BAR_VOFF:
            {
				tr_item->TraceTab->_colour_bar_voffset->setValue(fChoice);
                break;
            }
            case TRACE_COL_BAR_TICK:
            {
				tr_item->TraceTab->_colourscale_ticklength->setValue(fChoice);
                break;
            }
			default:
				break;	  
		}
   }
   
}
		
void PlotView::setEventOption(int option, int iChoice, float fChoice, const QString& sChoice, bool bChoice, const QColor& cChoice)
{
	if(QuSlotDebug()) cout << "PlotView::setEventOption" << endl;

	// apply Event option to all selected events (that belonging to visible tab is always selected)

	QList<QpuiItem*> list=selectedItems();
	foreach(QpuiItem* item, list)
	{
		EventItem* ev_item=dynamic_cast<EventItem*>(item);
    
		if(!ev_item) continue;
	  
		// don't set event for visible tab - user just did this - avoid infinite loop
		if(ev_item->eventTab->isVisible()) continue;
	  
		switch(option)
		{
			case EVENT_Y_DATA:
				ev_item->eventTab->_y_data_fld->setText(sChoice);
				ev_item->eventTab->_y_data_fld->reValidate(true);
				break;
			case EVENT_X_DATA:
				ev_item->eventTab->_x_data_fld->setText(sChoice);
				ev_item->eventTab->_x_data_fld->reValidate(true);
				break;
			case EVENT_LABEL:
				ev_item->eventTab->_event_label_fld->setText(sChoice);
				break;
			case EVENT_LABEL_CHAR_SIZE: 
				ev_item->eventTab->_char_size_fld->setValue(fChoice);
                break;
            case EVENT_LABEL_JUST:
                ev_item->eventTab->_event_label_just_fld->setValue(fChoice);
                break;
			case EVENT_LABEL_FONT_FAMILY:
				switch(iChoice)
				{
					case PL_FCI_SERIF:
						ev_item->eventTab->_event_label_font_fld->setSerifFont();
						break;
					case PL_FCI_MONO:
						ev_item->eventTab->_event_label_font_fld->setMonotypeFont();
						break;
					case PL_FCI_SCRIPT:
						ev_item->eventTab->_event_label_font_fld->setDecorativeFont();
						break;
					default:
						ev_item->eventTab->_event_label_font_fld->setSansFont();
						break;
				}
				break;
			case EVENT_LABEL_FONT_STYLE:
				ev_item->eventTab->_event_label_font_fld->setItalic(bChoice);
				break;
			case EVENT_LABEL_FONT_WEIGHT:
				ev_item->eventTab->_event_label_font_fld->setBold(bChoice);
				break;
			case EVENT_LABEL_COLOUR: 
			{
				QPalette label_palette = QPalette(QPUI_normal_palette);
				label_palette.setColor(QPalette::Active, QPalette::ButtonText, cChoice);
				label_palette.setColor(QPalette::Inactive, QPalette::ButtonText, cChoice);
				ev_item->eventTab->_event_label_colour_fld->setPalette(label_palette);
				ev_item->_event_label_colour = cChoice;
				break;
			}
			default:
				break;
			  
		}
	}
}

void PlotView::setSpectrumOption(int option, int iChoice, float fChoice, const QString& sChoice, bool bChoice, const QColor& cChoice)
{
	if(QuSlotDebug()) cout << "QpListView::setSpectrumOption" << endl;

	// apply Spectrum option to all selected spectra (that belonging to visible tab is always selected)

	QList<QpuiItem*> list=selectedItems();
	foreach(QpuiItem* item, list)
	{
		SpectrumItem* sp_item=dynamic_cast<SpectrumItem*>(item);
		
		if( !sp_item) continue;
		
		// don't set spectrum for visible tab - user just did this - avoid infinite loop
		if(sp_item->DataTab->isVisible()) continue;
		if(sp_item->ColourscaleTab->isVisible()) continue;

		switch(option)
		{
			// Spectrum Data Tab
			case SPECTROGRAM_ZDATA:

				sp_item->replaceData(sChoice);

				break;
			case SPECTROGRAM_ZCOMP:

				sp_item->replaceComponent(sChoice);

				break;
			case SPECTROGRAM_TIDATA:
			{
				sp_item->replaceTIdata(sChoice);
				break;
			}
			case SPECTROGRAM_TICOMP:
			{
				sp_item->replaceTIcomp(sChoice);
				break;
			}
			case SPECTROGRAM_YDATA_START:
				// sChoice will be either component selection or object base name
				if(sChoice.contains("["))
				{
					// change component
					QString newName = baseObjectText( sp_item->DataTab->_y_start_data_fld->text() );
					newName += sChoice;
					sp_item->DataTab->_y_start_data_fld->setText(newName);
				}
				else
				{
					sp_item->DataTab->_y_start_data_fld->setTextKeepComp(sChoice);
				}		
				sp_item->DataTab->_y_start_data_fld->reValidate(true);
	
				break;
			case SPECTROGRAM_YDATA_END:
				/*
				// sChoice will be either component selection or object base name
				if(sChoice.contains("["))
				{
					// change component
					QString newName = baseObjectText( sp_item->DataTab->_y_end_data_fld->text() );
					newName += sChoice;
					sp_item->DataTab->_y_end_data_fld->setText(newName);
				}
				else
				{
					sp_item->DataTab->_y_end_data_fld->setTextKeepComp(sChoice);
				}
				sp_item->DataTab->_y_end_data_fld->reValidate(true);	
				*/
				break;
			case SPECTROGRAM_XDATA_START:
				// sChoice will be either component selection or object base name
				if(sChoice.contains("["))
				{
					// change component
					QString newName = baseObjectText( sp_item->DataTab->_x_start_data_fld->text() );
					newName += sChoice;
					sp_item->DataTab->_x_start_data_fld->setText(newName);
				}
				else
				{
					sp_item->DataTab->_x_start_data_fld->setTextKeepComp(sChoice);
				}			
				sp_item->DataTab->_x_start_data_fld->reValidate(true);
				break;
			case SPECTROGRAM_XY_SWAP:
				
				sp_item->DataTab->_XYtoggle->toggle();
				
				sp_item->DataTab->setXYswap();
				
				break;
			case SPECTROGRAM_XDATA_END:
				/*
				// sChoice will be either component selection or object base name
				if(sChoice.contains("["))
				{
					// change component
					QString newName = baseObjectText( sp_item->DataTab->_x_end_data_fld->text() );
					newName += sChoice;
					sp_item->DataTab->_x_end_data_fld->setText(newName);
				}
				else
				{
					sp_item->DataTab->_x_end_data_fld->setTextKeepComp(sChoice);
				}	
				sp_item->DataTab->_x_end_data_fld->reValidate(true);
				*/
				break;
			case COLOUR_SCALE_MIN:
				sp_item->ColourscaleTab->_colourscale_min_fld->setDouble(fChoice);
				sp_item->ColourscaleTab->_colourscale_auto_fld->setCurrentIndex(QP_FROM_USER);
				sp_item->ColourscaleTab->setSlotColour(true);
				sp_item->ColourscaleTab->_colourscale_min_fld->lineBox->setPalette(sp_item->ColourscaleTab->GUIpalette);
				sp_item->ColourscaleTab->_colourscale_max_fld->lineBox->setPalette(sp_item->ColourscaleTab->GUIpalette);
				break;
			case COLOUR_SCALE_MAX:
				sp_item->ColourscaleTab->_colourscale_max_fld->setDouble(fChoice);
				sp_item->ColourscaleTab->_colourscale_auto_fld->setCurrentIndex(QP_FROM_USER);
				sp_item->ColourscaleTab->setSlotColour(true);
				sp_item->ColourscaleTab->_colourscale_min_fld->lineBox->setPalette(sp_item->ColourscaleTab->GUIpalette);
				sp_item->ColourscaleTab->_colourscale_max_fld->lineBox->setPalette(sp_item->ColourscaleTab->GUIpalette);
				break;
			case COLOUR_SCALE_TICK_LENGTH:
				sp_item->ColourscaleTab->_colourscale_ticklength_fld->setValue(fChoice);
				break;
			case COLOUR_SCALE_LABEL:
				sp_item->ColourscaleTab->_colourscale_label_fld->setText(sChoice);
				sp_item->ColourscaleTab->_colourscale_autolabel_fld->setChecked(false);
				break;
			case COLOUR_SCALE_AUTOSCALE:
				sp_item->ColourscaleTab->_colourscale_auto_fld->setCurrentIndex(iChoice);
				sp_item->ColourscaleTab->setColourscaleMinMax();
				break;
			case COLOUR_SCALE_AUTOLABEL:
				sp_item->ColourscaleTab->_colourscale_autolabel_fld->setChecked(bChoice);
				sp_item->ColourscaleTab->setColourScaleAutoLabel();
				break;
			case COLOUR_SCALE_LINLOG:
				sp_item->ColourscaleTab->_colourscale_linlog_fld->setCurrentIndex(iChoice);
				sp_item->ColourscaleTab->_colourscale_auto_fld->setCurrentIndex(QP_FROM_USER);
				break;
			case COLOUR_SCALE_SHOW_COLOUR_BAR:
				sp_item->ColourscaleTab->_colourscale_show_colour_bar_fld->setChecked(bChoice);
				break;
			case COLOUR_SCALE_COLOUR_BAR_WIDTH:
				sp_item->ColourscaleTab->_colourscale_colour_bar_width_fld->setValue(fChoice);
				break;
			case COLOUR_SCALE_COLOUR_BAR_HEIGHT:
				sp_item->ColourscaleTab->_colourscale_colour_bar_height_fld->setValue(fChoice);
				break;
			case COLOUR_SCALE_COLOUR_BAR_VOFFSET:
				sp_item->ColourscaleTab->_colourscale_colour_bar_voffset_fld->setValue(fChoice);
				break;
			case COLOUR_SCALE_COLOUR_BAR_DISP:
				sp_item->ColourscaleTab->_colourscale_colour_bar_disp_fld->setValue(fChoice);
				break;
// 			case COLOUR_SCALE_TABLE:
// 				sp_item->ColourscaleTab->_colourscale_table_fld->setCurrentItem(iChoice);
// 				break;
            case COLOUR_SCALE_TABLE_STD:
                sp_item->ColourscaleTab->_colourtable_fld->setStdPalette(iChoice);
                break;
            case COLOUR_SCALE_TABLE_PROFILE:
                sp_item->ColourscaleTab->_colourtable_fld->setProfilePalette(iChoice);
                break;
            case COLOUR_SCALE_TABLE_SESSION:
                sp_item->ColourscaleTab->_colourtable_fld->setSessionPalette(iChoice);
                break;
            case COLOUR_SCALE_AXIS_EXP_LABEL_X:
                sp_item->ColourscaleTab->_exp_label_offset_x_fld->setValue(fChoice);
                break;
            case COLOUR_SCALE_AXIS_EXP_LABEL_Y:
                sp_item->ColourscaleTab->_exp_label_offset_y_fld->setValue(fChoice);
                break;
            case SPECTROGRAM_INTERLACE:
                sp_item->DataTab->interlace_toggle->setChecked(bChoice);
                break;
            case SPECTROGRAM_GAP:
                sp_item->DataTab->gap_toggle->setChecked(bChoice);
                break;
            case SPECTROGRAM_MIN_CELL_WIDTH:
                sp_item->DataTab->minCellWidth->setValue(fChoice);
                break;
			default:
				break;
				
		}
	} // end for over local list
}
		
void PlotView::setLegendOption(int option, int iChoice, float fChoice, const QString& sChoice, bool bChoice, const QColor& cChoice)
{
	if(QuSlotDebug()) cout << "QpListView::setLegendOption" << endl;
	// Slot called when value changed on legend tab

	// apply Legend option to all selected panels (that belonging to visible tab is always selected)

	QList<QpuiItem*> list=selectedItems();
	foreach(QpuiItem* item, list)
	{
		PanelItem* panel_item=dynamic_cast<PanelItem*>(item);
    
		if( !panel_item) continue;
		
		if(panel_item->LegendTab->isVisible()) continue;
	  
		switch(option)
		{
			case SHOW_LEGEND:
			{
				panel_item->LegendTab->_show_legend_fld->setChecked(bChoice);
				break;
			}
			case LEGEND_BORDER:
			{
				panel_item->LegendTab->_legend_border_fld->setCurrentIndex(iChoice);
				break;
			}
			case LEGEND_NUM_ROWS:
			{
				panel_item->LegendTab->_legend_num_rows_fld->setValue(iChoice);
				int num_traces = panel_item->numberTraces();
				int num_rows = iChoice;
				int num_cols = (int)ceil((float)num_traces / num_rows);
				panel_item->LegendTab->_legend_num_cols_fld->setValue(num_cols);
				break;
			}
			case LEGEND_NUM_COLS:
			{
				panel_item->LegendTab->_legend_num_cols_fld->setValue(iChoice);
				int num_traces = panel_item->numberTraces();
				int num_cols = iChoice;
				int num_rows = (int)ceil((float)num_traces / num_cols);
				panel_item->LegendTab->_legend_num_rows_fld->setValue(num_rows);
				break;
			}
			case LEGEND_CHAR_SIZE:
			{
				panel_item->LegendTab->_legend_char_size_fld->setValue(fChoice);
				break;
			}
			case LEGEND_MAX_CHAR_LENGTH:
			{
				panel_item->LegendTab->_legend_max_char_length_fld->setValue(iChoice);
				break;
			}
			case LEGEND_LINE_LENGTH:
			{
				panel_item->LegendTab->_legend_line_length_fld->setValue(fChoice);
				break;
			}
			case LEGEND_JUST:
			{
				panel_item->LegendTab->_legend_just_fld->setCurrentIndex(iChoice);
				break;
			}
			case LEGEND_FONT_FAMILY:
			{
				switch(iChoice)
				{
					case PL_FCI_SERIF:
						panel_item->LegendTab->_legend_font_fld->setSerifFont();
						break;
					case PL_FCI_MONO:
						panel_item->LegendTab->_legend_font_fld->setMonotypeFont();
						break;
					case PL_FCI_SCRIPT:
						panel_item->LegendTab->_legend_font_fld->setDecorativeFont();
						break;
					default:
						panel_item->LegendTab->_legend_font_fld->setSansFont();
						break;
				}
				break;
			}
			case LEGEND_FONT_STYLE:
				panel_item->LegendTab->_legend_font_fld->setItalic(bChoice);
				break;
			case LEGEND_FONT_WEIGHT:
				panel_item->LegendTab->_legend_font_fld->setBold(bChoice);
				break;
			case LEGEND_ALIGN:
			{
				panel_item->LegendTab->_legend_align_fld->setCurrentIndex(iChoice);
				break;
			}
			case LEGEND_L_OFFSET:
			{
				panel_item->LegendTab->_legend_loffset_fld->setValue(fChoice);
				break;
			}
			case LEGEND_T_OFFSET:
			{
				panel_item->LegendTab->_legend_toffset_fld->setValue(fChoice);
				break;
			}
			case LEGEND_BACKGROUND:
			{
				panel_item->LegendTab->_legend_background_fld->setCurrentIndex(iChoice);
				break;
			}
			case LEGEND_WIDTH:
			{
				panel_item->LegendTab->_legend_width_fld->setValue(fChoice);

				switch(panel_item->LegendTab->_legend_align_fld->currentIndex())	
				{
					case QPUI_INSIDE_TOP_RIGHT:
					case QPUI_INSIDE_BOTTOM_RIGHT:
						panel_item->LegendTab->_legend_loffset_fld->setValue(1.-fChoice);
						break;
						
					case QPUI_ABOVE_TOP_CENTRE:
						panel_item->LegendTab->_legend_loffset_fld->setValue(0.5-fChoice/2.);
						break;
					
					default:
						break;
				}
				break;
			}
			case LEGEND_HEIGHT:
			{
				panel_item->LegendTab->_legend_height_fld->setValue(fChoice);

				switch(panel_item->LegendTab->_legend_align_fld->currentIndex())	
				{
					case QPUI_INSIDE_BOTTOM_LEFT:
					case QPUI_INSIDE_BOTTOM_RIGHT:
						panel_item->LegendTab->_legend_toffset_fld->setValue(1.-fChoice);
						break;
					
					default:
						break;
				}
				break;
			}
		}
	} // end for over selected list
}
		
void PlotView::setYRangeAllSelected()
{
	double from = 1.e31;
	double to = -1.e31;
	
	// find min/max for selected panels
	QList<QpuiItem*> list=selectedItems();

	foreach(QpuiItem* item, list)
	{
		PanelItem * panel_item = dynamic_cast<PanelItem*>(item);
		if( !panel_item) continue;
			
		double panelFrom, panelTo;
		bool found = panel_item->getYRange(panelFrom, panelTo);

		if(found) {
			if(from > panelFrom) from = panelFrom;
			if(to < panelTo) to = panelTo;
		}
	}
	
	if( from > to) return;

	// set range on selected panels
	foreach(QpuiItem* item, list)
	{
		PanelItem * panel_item = dynamic_cast<PanelItem*>(item);
		if( !panel_item) continue;
		
		panel_item->yAxisTab->_axis_xy_fld->setInterval(from, to);
	}
	
	return;
	
}

void PlotView::setAxisOption(int option, int axis, int iChoice, float fChoice, const QString& sChoice, bool bChoice, const QColor& cChoice)
{
	// Slot called when value changed on axis option tabs
	// axis identifies which of X,Y or T is changed.
	
	if(QuSlotDebug()) cout << "QpListView::setAxisOption" << endl;

	QpAxisTab * axisTab = 0;
	// apply axis option to all selected frames or panels (that belonging to visible tab is always selected)

	QList<QpuiItem*> list=selectedItems();
	foreach(QpuiItem* item, list)
	{
		// if Y axis only modify panel items
		if(axis == Y_AXIS)
		{
			PanelItem * panel_item = dynamic_cast<PanelItem*>(item);
			if( !panel_item) continue;
			
			axisTab = panel_item->yAxisTab;
		}
		else if(axis == X_AXIS){
			FrameItem * frame_item = dynamic_cast<FrameItem*>(item);
			if( !frame_item) continue;

			axisTab = frame_item->xAxisTab;
		}
		else if(axis == T_AXIS){
			FrameItem * frame_item = dynamic_cast<FrameItem*>(item);
			if( !frame_item) continue;
			
			axisTab = frame_item->tAxisTab;
		}
		else break;
		
		// don't set axis for visible tab - user just did this - avoid infinite loop

		if(axisTab->isVisible()) continue;

		switch(option)
		{
			case AXIS_AUTOSCALE:	
			{
				axisTab->handleAutoScale(iChoice);
				axisTab->setAxisMinMax();
				break;
			}
			case AXIS_LINLOG:	
			{
				// only called for X or Y axis	
				axisTab->handleAutoScale(QP_FROM_USER);
				axisTab->_axis_linlog_fld->setCurrentIndex(iChoice);
				break;
			}
			case AXIS_MIN:
			{
				axisTab->handleAutoScale(QP_FROM_USER);
				if(axisTab->I_axis!=T_AXIS)
				{
					axisTab->_axis_xy_fld->setStart(sChoice.toDouble());

					axisTab->_axis_xy_fld->setAltPalette(false);
				}
				else
				{
					axisTab->_axis_time_fld->setStart(QmjdTime(sChoice.toLatin1().data()));

					axisTab->_axis_time_fld->setAltPalette(false);
					axisTab->setSlotColour(false);
					axisTab->_t_data_fld->lineBox->setPalette(axisTab->GUIpalette);
				}
				break;
			}
			case AXIS_MAX:
			{
				axisTab->handleAutoScale(QP_FROM_USER);
				if(axisTab->I_axis!=T_AXIS)
				{
					axisTab->_axis_xy_fld->setEnd(sChoice.toDouble());

					axisTab->_axis_xy_fld->setAltPalette(false);
				}
				else
				{
					axisTab->_axis_time_fld->setEnd(QmjdTime(sChoice.toLatin1().data()));

					axisTab->_axis_time_fld->setAltPalette(false);
					axisTab->setSlotColour(false);
					axisTab->_t_data_fld->lineBox->setPalette(axisTab->GUIpalette);
				}
				break;
			}
			case AXIS_NUM_MAJOR_TICKS:			
			{
				axisTab->_axis_major_ticks_num_fld->setValue(iChoice);
				break;
			}
			case AXIS_NUM_MINOR_TICKS:			
			{
				axisTab->_axis_minor_ticks_num_fld->setValue(iChoice);
				break;
			}
			case AXIS_MAJOR_TICKMARKS:			
			{
				axisTab->_axis_major_tickmarks_fld->setCurrentIndex(iChoice);
				break;
			}
			case AXIS_MINOR_TICKMARKS:			
			{
				axisTab->_axis_minor_tickmarks_fld->setCurrentIndex(iChoice);
				break;
			}
			case AXIS_LABEL:			
			{
				axisTab->_label_fld->setText(sChoice);
				break;
			}
			case AXIS_LABEL_CHAR_SIZE:			
			{
				axisTab->_label_char_size_fld->setValue(fChoice);
				break;
			}
			case AXIS_LABEL_DISP:			
			{
				axisTab->_label_disp_fld->setValue(fChoice);
				break;
			}
			case AXIS_AUTOLABEL:			
			{
				axisTab->_axis_autolabel_fld->setChecked(bChoice);
				axisTab->setAutoLabelText();
				break;
			}
			case AXIS_LABEL_ROTATION:			
			{
				axisTab->_label_rotation_fld->setValue(fChoice);
				break;
			}
			case AXIS_LABEL_FONT_FAMILY:			
			{
				switch(iChoice)
				{
					case PL_FCI_SERIF:
						axisTab->_label_font_fld->setSerifFont();
						break;
					case PL_FCI_MONO:
						axisTab->_label_font_fld->setMonotypeFont();
						break;
					case PL_FCI_SCRIPT:
						axisTab->_label_font_fld->setDecorativeFont();
						break;
					default:
						axisTab->_label_font_fld->setSansFont();
						break;
				}
				break;
			}
			case AXIS_LABEL_FONT_STYLE:
				axisTab->_label_font_fld->setItalic(bChoice);
				break;
			case AXIS_LABEL_FONT_WEIGHT:
				axisTab->_label_font_fld->setBold(bChoice);
				break;
			case AXIS_LABEL_COLOUR:	
			{
				QPalette label_palette = QPalette(QPUI_normal_palette);
				label_palette.setColor(QPalette::Active, QPalette::ButtonText, cChoice);
				label_palette.setColor(QPalette::Inactive, QPalette::ButtonText, cChoice);
				axisTab->_label_colour_fld->setPalette(label_palette);
				axisTab->_label_colour = cChoice;
				break;
			}
			case AXIS_T_DATA:	
			{
				// only called for T axis
				axisTab->_t_data_fld->setText(sChoice);
				axisTab->_t_data_fld->reValidate(true);
				axisTab->setTfromObj();
				break;
			}
			case AXIS_TICK_TIME_FORMAT:
			{
				// only called for T axis
				axisTab->tick_time_fmt_fld->setText(sChoice);
				break;
			}
			case AXIS_TICK_TIME_FORMAT_AUTO:
			{
				// only called for T axis
				axisTab->tick_time_fmt_auto_fld->setChecked(iChoice);
				axisTab->tick_time_fmt_fld->setReadOnly(iChoice);
				axisTab->tick_time_fmt_fld->setEnabled(!iChoice);
				break;
			}
			case AXIS_TIME_LABELS_FORMAT:
			{
				// only called for T axis
				axisTab->time_labels_fmt_fld->setText(sChoice);
				break;
			}
			case AXIS_TIME_LABELS_FORMAT_AUTO:
			{
				// only called for T axis
				axisTab->time_labels_fmt_auto_fld->setChecked(iChoice);
				axisTab->time_labels_fmt_fld->setReadOnly(iChoice);
				axisTab->time_labels_fmt_fld->setEnabled(!iChoice);
				break;
			}
            case AXIS_EXP_LABEL_Y:
            {
                axisTab->_exp_label_offset_y_fld->setValue(fChoice);
                break;
            }
            case AXIS_EXP_LABEL_X:
            {
                axisTab->_exp_label_offset_x_fld->setValue(fChoice);
                break;
            }
		}
	} // end for over local list
}

void PlotView::setPlotOption(int option, bool bChoice)
{
	if(QuSlotDebug()) cout << "PlotView::setPlotOption" << endl;

	// apply Trace option to all selected traces (that belonging to visible tab is always selected)

	QList<QpuiItem*> list=selectedItems();
	foreach(QpuiItem* item, list)
	{
		PlotItem* pl_item=dynamic_cast<PlotItem*>(item);
    
		if( !pl_item) continue;
	  
		// don't set option for visible tab - user just did this - avoid infinite loop
		if(pl_item->DataTab->isVisible()) continue;
	  
	  
		switch(option){
			// Data Tab
			default: case USE_TIME_AS_SEC:
			{
				pl_item->DataTab->timeAsSec_toggle->setChecked(bChoice);
				break;
			}
		}
	}
}

void PlotView::moveItemUp()
{
// 	QModelIndexList indexes=selectionModel()->selectedIndexes();
	QList<QPersistentModelIndex> indexes=selectedIndexes();
	QModelIndex temp;
	
	selectionModel()->clearSelection();
	
	foreach(QModelIndex index, indexes)
	{
		selectionModel()->select(index, QItemSelectionModel::Deselect);
		temp=plotModel()->moveUp(index);
		if(temp.isValid())
		{
			selectionModel()->select(temp, QItemSelectionModel::Select);
		}
	}
}

void PlotView::moveItemDown()
{
// 	QModelIndexList indexes=selectionModel()->selectedIndexes();
	QList<QPersistentModelIndex> indexes=selectedIndexes();
	QModelIndex temp;
	
	selectionModel()->clearSelection();
	
	foreach(QModelIndex index, indexes)
	{
		selectionModel()->select(index, QItemSelectionModel::Deselect);
		temp=plotModel()->moveDown(index);
		if(temp.isValid())
		{
			selectionModel()->select(temp, QItemSelectionModel::Select);
		}
	}
}

void PlotView::deleteSelectedItems()
{
	// deleting items affects the QModelIndex List, so collect items
	QModelIndexList indexes_temp=selectionModel()->selectedRows();
	QList <QpuiItem*> itemList;
	itemList.clear();
	
	foreach(QModelIndex index, indexes_temp)
	{
		if(index != rootIndex()){
			QpuiItem *item = plotModel()->itemFromIndex(index);
			itemList.push_back(item);
		}
	}
	clearSelection();
	
	// select current to something never deleted
	selectionModel()->setCurrentIndex(rootIndex(), QItemSelectionModel::NoUpdate );


	// now remove children of items on list
	int i=0;
	while(i<itemList.size())
	{
		QpuiItem* item=itemList[i];	
		// remove any item whose ancestor is also selected
		if( item->ancestorInList(itemList) )  itemList.removeAt(i);
		else i++;
	}
	
	// now finally safe to start deleting
	for(int i=0; i<itemList.size(); i++)
	{
		QpuiItem* item=itemList[i];	
		plotModel()->deleteItem(item);
	
	}

	expandChildren(plotModel()->indexFromItem(plotModel()->_root));

}

void PlotView::deleteAllItems()
{
	
	QpuiItem *rootItem = plotModel()->_root;
	if(rootItem == NULL) return;

		
	while(rootItem->_children.size() >0)
	{
		QpuiItem *item = rootItem->_children.at(0);
		PageItem* pageItem=dynamic_cast<PageItem*>(item);
		if(rootItem->_children.size() == 1 && pageItem) break; // avoid inf loop
		plotModel()->deleteItem(item);
	}
		
	// select current to something never deleted
	selectionModel()->setCurrentIndex(rootIndex(), QItemSelectionModel::NoUpdate );

}


void PlotView::renameSelectedItem()
{
	if(plotModel()->flags(currentIndex()) & Qt::ItemIsEditable) // Just to avoid seeing the "editing failed" message if not editable
	{	
		setCurrentIndex(currentIndex()); // See Qt doc for this precaution
		edit(currentIndex());
	}
}

void PlotView::linkPlots()
{
	QList<PlotItem*> list;
	QList<QpuiItem*> selectedList=selectedItems();
	
	foreach(QpuiItem* qpuiItem, selectedList)
	{
		PlotItem* item=dynamic_cast<PlotItem*>(qpuiItem);
		if(item!=NULL) list.push_back(item);
	}
	
	if(!list.empty())
		plotModel()->linkPlots(list);
}

void PlotView::unlinkPlots()
{
	
	QList<QpuiItem*> list=selectedItems();
	
	foreach(QpuiItem* qpuiItem, list)
	{
		PlotItem* item=dynamic_cast<PlotItem*>(qpuiItem);
		if(item!=NULL)
		{
			foreach(PlotItem* temp, item->linkedPlots)
			{
				temp->linkedPlots.removeAll(item);
			}
			item->linkedPlots.clear();
			item->setLinkText("");
		}
	}
}

void PlotView::setPlotType(QAction *act)
{
	QpItemType plotType = QP_TRACE;
	if(act->text() == QString("Spectrogram") )plotType = QP_SPECTROGRAM;
	else if(act->text() == QString("Const X or T Trace") )plotType = QP_CONST_XT;
	else if(act->text() == QString("Const Y Trace") )plotType = QP_CONST_Y;
	else if(act->text() == QString("Regression Line") )plotType = QP_REG_LINE;
	 
	if(QuSlotDebug()) cout << "PlotView::setPlotType" << endl;
	
	QList<QPersistentModelIndex> indexes=selectedIndexes();
	
	foreach(QModelIndex index, indexes)
	{
		plotModel()->changePlotType(index, plotType);
	}
}

void PlotView::setFrameType(QAction *act)
{
	if(QuSlotDebug()) cout << "PlotView::setFrameType" << endl;
	
	int frameType = QP_TS;
	if(act->text() == QString("XY Frame") ) frameType = QP_XY;
	
	QList<QpuiItem*> list=selectedItems();

	foreach(QpuiItem* item, list)
	{
		FrameItem* frameItem=dynamic_cast<FrameItem*>(item);
		
		PlotItem* plotItem=dynamic_cast<PlotItem*>(item);
		
		if(frameItem!=NULL) frameItem->changeFrame(frameType);
		if(plotItem!=NULL) plotItem->frame()->changeFrame(frameType);
		
	}
	updateOnSelection();

}


void PlotView::openObjectEditor(const QModelIndex& index)
{
	PlotItem* plotItem=dynamic_cast<PlotItem*>(plotModel()->itemFromIndex(index));
	
	if(plotItem!=NULL)
	{
		QString strippedName=plotItem->getName();
		int index=strippedName.indexOf(" ");
		if(index>0)
		{
			strippedName.truncate(index);
		}
		QdObject_var obj=QuGetObjByName(strippedName);
		
		QString name;
		QStringList sl=strippedName.split("//");
		name=sl.last();
		
		QdTimeInterval_var ti = QdTimeInterval_var::narrow(obj);
		QdTimeIntervalSeq_var tis = QdTimeIntervalSeq_var::narrow(obj);
		if(ti.is_nil() && tis.is_nil())
			QbuLaunchBrowse(QdObject_var(), this, name.toLatin1().data(), obj, plotItem->WLcontext, false);
		else
			QbuLaunchBrowse(QdObject_var(), this, name.toLatin1().data(), obj, plotItem->WLcontext, true);
	}
}

void PlotView::expandChildren(const QModelIndex& index)
{
	QpuiItem* item=plotModel()->itemFromIndex(index);
	if(item==NULL)
	{
		return;
	}
	
	expand(index);
	
	int numChildren=0;
	
	if(item!=NULL)
	{
		numChildren=item->numChildren();
	}
	for(int i=0; i<numChildren; ++i)
	{
		expandChildren(index.child(i, 0));
	}
}


QList<QpuiItem*> PlotView::selectedItems()
{
	QModelIndexList indexes=selectionModel()->selectedRows();
	
	QList<QpuiItem*> res;
	res.clear();
	
	foreach(QModelIndex index, indexes)
	{
		QpuiItem* item=plotModel()->itemFromIndex(index);
		if(item!=NULL && !res.contains(item))
		{
			res.push_back(item);
		}
	}

    QModelIndex current=currentIndex();
    QpuiItem* currentItem=plotModel()->itemFromIndex(current);
    if(!res.contains(currentItem))
        res.push_back(currentItem);
    
	return res;
}

Vistream& operator>>(Vistream& v_is, PlotView& view)
{
	view.blockSignals(true);
	v_is>> *(view.plotModel());
	view.blockSignals(false);
	
	PageItem* pageOne=dynamic_cast<PageItem*>(view.plotModel()->_root->getChildByNumber(0));
	if(pageOne)
	{
		view.setCurrentItem(pageOne);
//		view.updateOnSelection();
		view.expandAll();
	}
    return v_is;
}
