// qpui_event_options.cc
//
// Plot Interface Implementation - Event Options Tab
//
// Janet Barnes
// Sept 2006
//
// Tony Allen
// April 2008

#include "qpui.h"
#include "QdTime.h"
#include <qtooltip.h> 

extern QPalette QPUI_normal_palette;

//     QpEventTab CLASS
//     ----------------------

QpEventTab::QpEventTab(QTabWidget *tabWidget,  const QString& name, EventItem * _event):
		QpTab(tabWidget, name, QP_EVENT_TAB)
{
	// Constructor
	
	// keep track of item which owns this tab
	I_event = _event;

	create();
	
	connect(_y_data_fld, SIGNAL(edited_OK()), this, SLOT(setYData()));
	connect(_x_data_fld, SIGNAL(edited_OK()), this, SLOT(setXData()));
	connect( _event_label_fld->lineBox, SIGNAL(editingFinished()), this, SLOT(setEventLabel()) );
	connect(_char_size_fld, SIGNAL(valueChanged(double)),this, SLOT(setEventLabelCharSize(double)));
	connect( _event_label_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setEventLabelFamily(int)) );
	connect( _event_label_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setEventLabelStyle(bool)) );
	connect( _event_label_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setEventLabelWeight(bool)) );
	connect(_event_label_colour_fld, SIGNAL(clicked()), this, SLOT(setEventLabelColour()));
    connect(_event_label_just_fld, SIGNAL(valueChanged(double)), this, SLOT(setEventLabelJust(double)));
}

QpEventTab::QpEventTab(QpEventTab* tab, EventItem* _event):
		QpTab((QTabWidget*)(tab->parent()), "EventTab", QP_EVENT_TAB)
{
	I_event = _event;
	
	create();
	_char_size_fld->setValue(tab->_char_size_fld->value());
	PLINT family, style, weight;
	tab->_event_label_font_fld->getFontParams(family, style, weight);
	_event_label_font_fld->setFontParams(family, style, weight);

	_y_data_fld->setText(tab->_y_data_fld->text());
	_x_data_fld->setText(tab->_x_data_fld->text());
	
	QPalette line_palette = QPalette(QPUI_normal_palette);
	line_palette.setColor(QPalette::Active ,QPalette::ButtonText, I_event->_event_label_colour);
	line_palette.setColor(QPalette::Inactive, QPalette::ButtonText, I_event->_event_label_colour);
	this->_event_label_colour_fld->setPalette(line_palette);
	_event_label_fld->setText(tab->_event_label_fld->text());
    _event_label_just_fld->setValue(tab->_event_label_just_fld->value());
	
	connect(_y_data_fld, SIGNAL(edited_OK()), this, SLOT(setYData()));
	connect(_x_data_fld, SIGNAL(edited_OK()), this, SLOT(setXData()));
	connect( _event_label_fld->lineBox, SIGNAL(editingFinished()), this, SLOT(setEventLabel()) );
	connect(_char_size_fld, SIGNAL(valueChanged(double)),this, SLOT(setEventLabelCharSize(double)));
	connect( _event_label_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setEventLabelFamily(int)) );
	connect( _event_label_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setEventLabelStyle(bool)) );
	connect( _event_label_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setEventLabelWeight(bool)) );
	connect(_event_label_colour_fld, SIGNAL(clicked()), this, SLOT(setEventLabelColour()));
    connect(_event_label_just_fld, SIGNAL(valueChanged(double)), this, SLOT(setEventLabelJust(double)));
}

void QpEventTab::create()
{
	QVBoxLayout *event_top_layout = new QVBoxLayout(this);
	event_top_layout->setSpacing(4);

	QGroupBox *event_pos_groupbox = new QGroupBox(this);

	event_pos_groupbox->setTitle("Event Label Position");

	QGridLayout *event_pos_layout = new QGridLayout;
	event_pos_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	event_pos_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_y_label_fld = new QLabel("Y position: ");
	event_pos_layout->addWidget(_y_label_fld, 1, 0, Qt::AlignRight);
	_y_data_fld = new QguWLinput(event_pos_groupbox, "y_data");
	_y_data_fld->setMinimumWidth(300);
	event_pos_layout->addWidget(_y_data_fld, 1, 1, 1, 2, Qt::AlignLeft);
	_y_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));

	_x_label_fld = new QLabel("T position: ");
	if(I_event->frameType() == QP_XY) _x_label_fld->setText("X position: ");
		
	event_pos_layout->addWidget(_x_label_fld, 3, 0, Qt::AlignRight);
	_x_data_fld = new QguWLinput(event_pos_groupbox, "x_data");
	if(I_event->frameType()  == QP_TS)
	{
		_x_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
	}
	else
	{
		_x_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	}
	_x_data_fld->setMinimumWidth(300);
	event_pos_layout->addWidget(_x_data_fld, 3, 1, 1, 2, Qt::AlignLeft);

	QPushButton *get_values_pb = new QPushButton("Get Values...");
	get_values_pb->setFixedWidth(150);
	event_pos_layout->addWidget(get_values_pb, 3, 3, Qt::AlignLeft);
	get_values_pb->setEnabled(true);

	event_pos_groupbox->setLayout(event_pos_layout);
	
	event_top_layout->addWidget(event_pos_groupbox);
   
	QGroupBox *event_label_groupbox = new QGroupBox(this);

	event_label_groupbox->setTitle("Label");

	QGridLayout *event_label_layout = new QGridLayout;
	event_label_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	event_label_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* text_label = new QLabel("Text: ");
	event_label_layout->addWidget(text_label, 1, 0, Qt::AlignRight);
	_event_label_fld = new QguWLinput(this, "event_text");
	_event_label_fld->setMinimumWidth(300);
	event_label_layout->addWidget(_event_label_fld, 1, 1, 1, 4, Qt::AlignLeft);
	_event_label_fld->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));

	QLabel* event_label_char_size_label = new QLabel("Character size: ");
	event_label_layout->addWidget(event_label_char_size_label, 3, 0, Qt::AlignRight);
	_char_size_fld = new FloatSpinBox(0.01, 40., 1,.1, event_label_groupbox, "_char_size");
	_char_size_fld->setFixedWidth(100);
	event_label_layout->addWidget(_char_size_fld, 3, 1, Qt::AlignLeft);

	QLabel* font_label = new QLabel("Font: ");
	event_label_layout->addWidget(font_label, 5, 0, Qt::AlignRight);
	_event_label_font_fld = new FontButton;
	_event_label_font_fld->setToolTip("Select font for event label");
	event_label_layout->addWidget(_event_label_font_fld, 5, 1, Qt::AlignLeft);

	QLabel* colour_label = new QLabel("Colour: ");
	event_label_layout->addWidget(colour_label, 5, 2, Qt::AlignRight);
	_event_label_colour_fld = new QPushButton("Colour...");
	_event_label_colour_fld->setFixedWidth(100);
	_event_label_colour_fld->setFixedHeight(30);
	QFont f = _event_label_colour_fld->font();
	f.setBold(TRUE);
	_event_label_colour_fld->setFont(f);
	event_label_layout->addWidget(_event_label_colour_fld, 5, 3, Qt::AlignLeft);

    QLabel* just_label=new QLabel("Alignment: ");

    event_label_layout->addWidget(just_label, 7, 0, 1, 2, Qt::AlignRight);
    _event_label_just_fld=new FloatSpinBox(-1., 2., 2, 0.1);
    _event_label_just_fld->setFixedWidth(100);
    event_label_layout->addWidget(_event_label_just_fld, 7, 2, Qt::AlignLeft);
	_event_label_just_fld->setToolTip("In range (-1, 1): left=-1, centre=0, right=1");
    
    

	event_label_groupbox->setLayout(event_label_layout);

	event_top_layout->addWidget(event_label_groupbox);

	event_top_layout->addSpacing(4);
	event_top_layout->addStretch(10);
	connect(get_values_pb, SIGNAL(clicked()), this, SLOT(getValues()));
}

// -------------------------------------------

void QpEventTab::getValues()
{
	// slot invoked when Get Values... push button is clicked

	if(QuSlotDebug()) cout << "QpEventTab::getValues" << endl;
	double _x_from, _y_from;
	      // clear listview selection to avoid user accidentally applying changes
  // to the wrong panel. Selection is from the plot.
	PanelItem *panel_item = I_event->panel();
	FrameItem *frame_item = panel_item->frame();
	PageItem *page_item = frame_item->page();
    
      // check that the currently selected plot x window is open
	int plot_id=page_item->PageTab->_xwin_num_fld->text().toInt();
	if(plot_id<0)
	{
		(void)QMessageBox::warning( this,"Event Options", "Error! Device not open? Try replotting.");
		return;
	}
    
	PlotWindow* win=page_item->focusWindow(plot_id);
	if(win==NULL)
	{
		(void)QMessageBox::warning( this,"Event Options", "Error! Window id not found.");
		return;
	}
	double x_start, y_start, x_end, y_end;
	win->plot->captureMouse(&x_start, &y_start, &x_end, &y_end);

	win->setPLFocus(); // ensure subsequent actions are applied to this window
	
	y_start=1.-y_start;
	y_end=1.-y_end;
	
	// Get clicks coordinates relatively to the current pane;
	bool hasSpectrum = page_item->containsSpectrum();
	double factor=(hasSpectrum?0.9:1.);
	
	double x_offset = frame_item->PropertiesTab->_loffset_fld->value();
	double x_sz = frame_item->PropertiesTab->_width_fld->value()*factor;
	double y_sz = frame_item->PropertiesTab->_height_fld->value();
	y_sz /= frame_item->numChildren();
	double y_offset = 1. - frame_item->PropertiesTab->_toffset_fld->value() - ( panel_item->getStackNumber() ) * y_sz;

  
	// calculate new x axis ranges from plot selection coordinates 
	if(frame_item->frameType() == QP_XY)
	{
		double xmin = frame_item->xAxisTab->_axis_xy_fld->getStart();
		double xmax = frame_item->xAxisTab->_axis_xy_fld->getEnd();
	
    		// if logarithmic axis, convert to logs first
		if(frame_item->xAxisTab->_axis_linlog_fld->currentIndex() == QP_LOG)
		{
			xmin = SafeLog10(xmin);
			xmax = SafeLog10(xmax);
		}
	
		double x_range = xmax - xmin;
		_x_from = xmin + x_range * (x_start-x_offset)/x_sz;
		if(frame_item->xAxisTab->_axis_linlog_fld->currentIndex() == QP_LOG)
		{
			_x_from=pow(10., _x_from);
		}	    
		_x_data_fld->setText(QString("%1").arg(_x_from));
	}
	else if(frame_item->frameType() == QP_TS)
	{
		QmjdTime tmin= frame_item->tAxisTab->_axis_time_fld->getStart().getEpoch2000();
		double t_range=frame_item->tAxisTab->_axis_time_fld->getDuration();
		QmjdTime tFrom=tmin+t_range * (x_start-x_offset)/x_sz;
		_x_data_fld->setText(tFrom.getISOstring(0).c_str());

	} 
  
  // calculate new y axis ranges
	if(panel_item->yAxisTab)
	{
		double ymin = panel_item->yAxisTab->_axis_xy_fld->getStart();
		double ymax = panel_item->yAxisTab->_axis_xy_fld->getEnd();
    // if logarithmic axis convert to logs first
		if(panel_item->yAxisTab->_axis_linlog_fld->currentIndex() == QP_LOG)
		{
			ymin = SafeLog10(ymin);
			ymax = SafeLog10(ymax);
		}
		double y_range = ymax - ymin;
		_y_from = ymin + y_range*(y_start-y_offset)/y_sz;
		if(panel_item->yAxisTab->_axis_linlog_fld->currentIndex() == QP_LOG)
		{
			_y_from = pow(10., _y_from);
		}	    
		_y_data_fld->setText(QString("%1").arg(_y_from));
	}

} 

// -------------------------------------------------

void QpEventTab::setXData()
{
  // slot called when x data name field changed on widget
	// Only used to propagate multiple selection input
 
     if(this->isVisible() ) I_event->root()->I_PlotView->setEventOption(EVENT_X_DATA, 0, 0., this->_x_data_fld->text());
}

// -----------------------------

void QpEventTab::setYData()
{
	// slot called when y data name field changed on widget
	// Only used to propagate multiple selection input
  
    if(this->isVisible() ) I_event->root()->I_PlotView->setEventOption(EVENT_Y_DATA, 0, 0., this->_y_data_fld->text());
}

// -----------------------------

void QpEventTab::setEventLabel()
{
	// slot called when label text field changed on widget
	// Only used to propagate multiple selection input
  
    if(this->isVisible() ) I_event->root()->I_PlotView->setEventOption(EVENT_LABEL, 0, 0., this->_event_label_fld->text());
}

// -----------------------------

void QpEventTab::setEventLabelCharSize(double fValue)
{
	// slot called when cahracter size field changed on widget
	// Only used to propagate multiple selection input
  
    if(this->isVisible() ) I_event->root()->I_PlotView->setEventOption(EVENT_LABEL_CHAR_SIZE, 0, fValue);
}

void QpEventTab::setEventLabelJust(double fValue)
{
    // slot called when cahracter size field changed on widget
    // Only used to propagate multiple selection input
  
    if(this->isVisible() ) I_event->root()->I_PlotView->setEventOption(EVENT_LABEL_JUST, 0, fValue);
}

// -----------------------------

void QpEventTab::setEventLabelFamily(int iValue)
{
	// slot called when cahracter size field changed on widget
	// Only used to propagate multiple selection input
  
	if(this->isVisible() ) I_event->root()->I_PlotView->setEventOption(EVENT_LABEL_FONT_FAMILY, iValue);
}

void QpEventTab::setEventLabelStyle(bool bValue)
{
	// slot called when cahracter size field changed on widget
	// Only used to propagate multiple selection input
  
	if(this->isVisible() ) I_event->root()->I_PlotView->setEventOption(EVENT_LABEL_FONT_STYLE, 0, 0., "", bValue);
}

void QpEventTab::setEventLabelWeight(bool bValue)
{
	// slot called when cahracter size field changed on widget
	// Only used to propagate multiple selection input
  
	if(this->isVisible() ) I_event->root()->I_PlotView->setEventOption(EVENT_LABEL_FONT_WEIGHT, 0, 0., "", bValue);
}

// -----------------------------

void QpEventTab::setEventLabelColour()
{
   bool changed = false;
   // ask the user for colour choice
   QColor c = QuGetColour(I_event->_event_label_colour, &changed, this);
   if(changed)
   {
        QPalette line_palette = QPalette(QPUI_normal_palette);
        line_palette.setColor(QPalette::Active ,QPalette::ButtonText, c);
        line_palette.setColor(QPalette::Inactive, QPalette::ButtonText, c);
        this->_event_label_colour_fld->setPalette(line_palette);
        I_event->_event_label_colour = c;
   }

   // only tell list view to make changes to selected event items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_event->root()->I_PlotView->setEventOption(EVENT_LABEL_COLOUR, 0, 0., "", false, c);
}


QDomElement QpEventTab::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("Event_Tab");

    xml.appendChild(createTextXMLNode(doc, "X_Data", _x_data_fld->text()));
    xml.appendChild(createTextXMLNode(doc, "Y_Data", _y_data_fld->text()));
    xml.appendChild(createTextXMLNode(doc, "Label", _event_label_fld->text()));
    xml.appendChild(createTextXMLNode(doc, "Label_Char_Size", _char_size_fld->value()));

    xml.appendChild(createTextXMLNode(doc, "Justification", _event_label_just_fld->value()));

    
    
    PLINT family, style, weight;
    _event_label_font_fld->getFontParams(family, style, weight);
    xml.appendChild(createTextXMLNode(doc, "Label_Font_Family", family));
    xml.appendChild(createTextXMLNode(doc, "Label_Font_Style", style));
    xml.appendChild(createTextXMLNode(doc, "Label_Font_Weight", weight));


    return xml;
}

void QpEventTab::fromXMLElement(QDomElement& el)
{
    QString s;
    
    if(getXMLNodeText(el, "Y_Data", s))
    {
        _y_data_fld->setText(s);
    }
    if(getXMLNodeText(el, "X_Data", s))
    {
        _x_data_fld->setText(s);
    }
    if(getXMLNodeText(el, "Label", s))
    {
        _event_label_fld->setText(s);
    }
    if(getXMLNodeText(el, "Label_Char_Size", s))
    {
        _char_size_fld->setValue(s.toDouble());
    }

    PLINT family(PL_FCI_SANS), style(0), weight(0);
    if(getXMLNodeText(el, "Label_Font_Family", s))
    {
        family=s.toInt();
    }
    if(getXMLNodeText(el, "Label_Font_Style", s))
    {
        style=s.toInt();
    }
    if(getXMLNodeText(el, "Label_Font_Weight", s))
    {
        weight=s.toInt();
    }
    switch(family)
    {
        case PL_FCI_SERIF: _event_label_font_fld->setSerifFont(); break;
        case PL_FCI_MONO: _event_label_font_fld->setMonotypeFont(); break;
        case PL_FCI_SCRIPT: _event_label_font_fld->setDecorativeFont(); break;
        default: _event_label_font_fld->setSansFont(); break;
    }
    _event_label_font_fld->setItalic(style==PL_FCI_ITALIC);
    _event_label_font_fld->setBold(weight==PL_FCI_BOLD);


    if(getXMLNodeText(el, "Justification", s))
    {
        _event_label_just_fld->setValue(s.toDouble());
    }

}
