// qpui_spectrogram.cc
//
// Plot Interface Implementation - SpectroGram Object
//
// Janet Barnes
// July 2002
//
// Tony Allen
// Feb 2008

#include "qpui.h"
#include "qar.h"
#include "qdutil.h"
#include "qwui.h"

extern int qsas_restore_release;

//     SpectrumItem CLASS
//     ------------------


SpectrumItem::SpectrumItem(const QString& name, QpuiItem* parent, const QString& WLcontext, int position):
		PlotItem(name, parent, WLcontext, position)
{
	if(name.isEmpty()) _name=_name+QString(": Spect");
	I_Type = QP_SPECTROGRAM;

	ColourscaleTab = new QpColourscaleTab(tabWidget, "ColourscaleTab", this);
	ColourscaleTab->setVisible(false);
	
	DataTab = new QpDataTab(tabWidget, "DataTab", this, QP_SPECTROGRAM);
	DataTab->setVisible(false);

	if(frameType() == QP_TS){
		DataTab->_z_data_fld->set3DSubsetDef("r");
	}
	else{
		DataTab->_z_data_fld->set3DSubsetDef("rp", true);
	}
	
	DataTab->_z_data_fld->setText(WLcontext);
	DataTab->_z_data_fld->reValidate(true);
	
	_z_obj_id=0;
	_y_start_obj_id=0;
	_y_end_obj_id=0;
	_x_start_obj_id=0;
	_x_end_obj_id=0;
	
	y_data_start_str = "";
	y_data_end_str = "";
	x_data_start_str = "";
	x_data_end_str = "";

	// starts as not set (used by fill_X_Data and fill_Y_Data to find when toggle changes)
	XYtoggle_in_fill_x = false;
	XYtoggle_in_fill_y = false;
	xdo_id_getXRange = 0;
	ydo_id_getYRange = 0;
	zdo_id_getZRange = 0;
	
	x_min_data = 0;
	x_max_data = 0;
	y_min_data = 0;
	y_max_data = 0;
	z_min_data = 0;
	z_max_data = 0;
	
	PanelItem * panel = dynamic_cast<PanelItem*>(parent);
	if(panel && panel->isInterlaced()) DataTab->interlace_toggle->setChecked(true);
}

SpectrumItem::SpectrumItem(SpectrumItem* item, QpuiItem* parent, int position):
		PlotItem(item, parent, position)
{
	I_Type = QP_SPECTROGRAM;

	ColourscaleTab = new QpColourscaleTab(item->ColourscaleTab, this);
	ColourscaleTab->setVisible(false);
	
	DataTab = new QpDataTab(item->DataTab, this);
	DataTab->setVisible(false);

	if(frameType() == QP_TS){
		DataTab->_z_data_fld->set3DSubsetDef("r");
	}
	else{
		DataTab->_z_data_fld->set3DSubsetDef("rp", true);
	}

	DataTab->_z_data_fld->setText(item->DataTab->_z_data_fld->text());
	DataTab->_z_data_fld->reValidate(true);
	
	_z_obj_id=0;
	_y_start_obj_id=0;
	_y_end_obj_id=0;
	_x_start_obj_id=0;
	_x_end_obj_id=0;
	
	y_data_start_str = "";
	y_data_end_str = "";
	x_data_start_str = "";
	x_data_end_str = "";

	// starts as not set (used by fill_X_Data and fill_Y_Data to find when toggle changes)
	XYtoggle_in_fill_x = false;
	XYtoggle_in_fill_y = false;
	xdo_id_getXRange = 0;
	ydo_id_getYRange = 0;
	zdo_id_getZRange = 0;
	
	x_min_data = 0;
	x_max_data = 0;
	y_min_data = 0;
	y_max_data = 0;
	z_min_data = 0;
	z_max_data = 0;
	
	PanelItem * panel = dynamic_cast<PanelItem*>(parent);
	if(panel && panel->isInterlaced()) DataTab->interlace_toggle->setChecked(true);

}

SpectrumItem::~SpectrumItem()
{
	if(DataTab) delete DataTab;
	if(ColourscaleTab) delete ColourscaleTab;
	foreach(PlotItem* i, linkedPlots)
	{
		i->linkedPlots.removeAll(this);
	}
}

// ------------------------------------

void SpectrumItem::clearObj_id()
{
   // called when we need to force a range update
	_z_obj_id = 0;
	_y_start_obj_id = 0;
	_x_start_obj_id = 0;
}

// -------------------------------------------

void SpectrumItem::replaceData(QString obj_name)
{
	if(QuSlotDebug()) cout << "SpectrumItem::replaceData" << endl;
	if(obj_name.isEmpty()) return;
	
	QString oldCompText = compText( DataTab->_z_data_fld->text() );

	if(oldCompText.isEmpty())
	{
		if(this->frameType() == QP_TS)
		{
			DataTab->_z_data_fld->getDefaultComp(QS_1D, QS_SEQ); // set data slot
		}
		else
		{
			DataTab->_z_data_fld->getDefaultComp(QS_2D, QS_NOT_SEQ); // set data slot
		}
	}
	else
	{
		DataTab->_z_data_fld->setTextKeepComp(obj_name); // set data slot

	}
	if(!nameLock) this->setName(DataTab->_z_data_fld->text()); // set item name
	this->WLcontext = QString(obj_name.toLatin1().data());

}

// -------------------------------------------

void SpectrumItem::replaceComponent(QString comp)
{
	if(QuSlotDebug()) cout << "SpectrumItem::replaceComponent" << endl;
	if(comp.isEmpty()) return;
	
	QString newName = baseObjectText(DataTab->_z_data_fld->text().toLatin1().data());
	newName += comp;
	DataTab->_z_data_fld->setText(newName); // set data slot
	DataTab->_z_data_fld->reValidate(true);
	this->setName(newName);                      // set item name

}

// -------------------------------------------
void SpectrumItem::replaceTIdata(QString tivl)
{
	if(QuSlotDebug()) cout << "SpectrumItem::replaceTI with '" << tivl.toLatin1().data() << "'" << endl;
	if(tivl.isEmpty()) return;
	
	DataTab->_mask_data_fld->setText(tivl);  // set TI slot
	DataTab->_mask_data_fld->reValidate();


}

// -------------------------------------------
void SpectrumItem::replaceTIcomp(QString comp)
{
	if(QuSlotDebug()) cout << "SpectrumItem::replaceTI with '" << comp.toLatin1().data() << "'" << endl;
	if(comp.isEmpty()) return;
	
	QString newName = baseObjectText(DataTab->_mask_data_fld->text().toLatin1().data());
	newName += comp;
	DataTab->_mask_data_fld->setText(newName);  // set TI slot

	DataTab->_mask_data_fld->reValidate();



}

// -------------------------------------------

void SpectrumItem::refreshTabs()
{
	if(QuSlotDebug()) cout << "SpectrumItem::refreshTabs" << endl;
	DataTab->_z_data_fld->reValidate();

}

// -------------------------------------------

void SpectrumItem::showTabs()
{
	if(QuSlotDebug()) cout << "SpectrumItem::showTabs" << endl;

	// ensure all tabs are hidden
	int Ntabs = tabWidget->count();
	for(int i = Ntabs-1; i >= 0; i--)
	{
		if(tabWidget->widget(i))
		{
			tabWidget->widget(i)->setVisible(false);
			tabWidget->removeTab(i);
		}
	}

	tabWidget->addTab(DataTab, "Data");

	tabWidget->addTab(ColourscaleTab, "Colourscale");
	// value return is always shown
	tabWidget->addTab(page()->ValueReturnTab, "Value Return");
	
	if(this->root()->I_PlotView->I_LastTab == QP_COLOUR_TAB)  tabWidget->setCurrentIndex(tabWidget->indexOf(ColourscaleTab));
	else	tabWidget->setCurrentIndex(tabWidget->indexOf(DataTab));
}

// ------------------------------------------

void SpectrumItem::changeFrameType(int frameType)
{
	// called when Frame changes type XY <-> TS
	
	if(frameType == QP_TS)
	{
		DataTab->_x_start_label_fld->setVisible(false);
		DataTab->_x_start_data_fld->setVisible(false);
//		DataTab->_x_end_label_fld->setVisible(false);
//		DataTab->_x_end_data_fld->setVisible(false);
		DataTab->_XYtoggle->setVisible(false);
		DataTab->gap_toggle->setVisible(true);
		DataTab->interlace_toggle->setVisible(true);
		DataTab->timeAsSec_toggle->setVisible(false);
		DataTab->_z_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_SEQ, QS_TT));
		DataTab->_z_data_fld->set3DSubsetDef("r");
		QString baseTxt = DataTab->_z_data_fld->textObject();
		DataTab->_z_data_fld->setText(baseTxt);
		DataTab->_z_data_fld->reValidate(true);
	}
	else
	{
		DataTab->_x_start_label_fld->setVisible(true);
		DataTab->_x_start_data_fld->setVisible(true);
//		DataTab->_x_end_label_fld->setVisible(true);
//		DataTab->_x_end_data_fld->setVisible(true);
		DataTab->_XYtoggle->setVisible(true);
		DataTab->gap_toggle->setVisible(false);
		DataTab->interlace_toggle->setVisible(false);
		DataTab->timeAsSec_toggle->setVisible(true);
		DataTab->_z_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_SEQ, QS_ANY));
		DataTab->_z_data_fld->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_2D, QS_NOT_SEQ, QS_ANY));
		DataTab->_z_data_fld->set3DSubsetDef("rp", true);
		QString baseTxt = DataTab->_z_data_fld->textObject();
		DataTab->_z_data_fld->setText(baseTxt);
		DataTab->_z_data_fld->reValidate(true);
	}
	
	DataTab->setDataTypeTitle();
	
	// force axis update
	xdo_id_getXRange = 0;
	ydo_id_getYRange = 0;
	_z_obj_id = 0;
	DataTab->setZData();
}



// ---------------------------------------

void SpectrumItem::get_dobjs()
{
	if( QuSlotDebug() ) cout << "QpSpectrumItem::get_dobjs " <<  endl;

	_zdo = getZDO(); // sets _zdo
	if(_zdo.is_nil()) return;


	// Apply Time Interval Mask if set
	QdObject_var TImaskObj = DataTab->_mask_data_fld->getSlotObject();
	if( !TImaskObj.is_nil()) {
		QdTimeInterval_var TImask = QdTimeInterval_var::narrow(TImaskObj);
		if( !TImask.is_nil()) {
			QdObject_var zdo = QuGetSubset(_zdo, TImask);
			zdo->copy_xrefs_from(_zdo);
			_zdo = zdo;
		}
	}

 
	_nx_data = fill_X_Data(); // sets _xdo
	_ny_data = fill_Y_Data(); // sets _ydo
	

}

// ---------------------------------------

void SpectrumItem::getLabel(const QpAxes axis, string& label)
{
	if( QuSlotDebug() ) cout << "QpSpectrumItem::getLabel for axis " << (int)axis <<  endl;

	// fetch label for axis
  
	// test on is_nil() done after default label is set below
  
	QdObject_var obj;
	string default_label;
	switch(axis)
	{
		case X_AXIS:
			// if time series plot default label is Universal Time
			if(frameType() == QP_TS)
			{
				label="Universal Time";
				return;
			}
			if(DataTab->timeAsSec_toggle->isChecked())
			{
				label="Seconds";
				return;
			}
			else label="X Axis";

			obj=_xdo;
			break;

		case Y_AXIS:
			label="Y Axis";
			obj=_ydo;
			break;

		case Z_AXIS:
			label="Z Axis";

			obj=_zdo;
			break;
			
		case COL_AXIS:
			label="Colour Bar";

			obj=_zdo;
			break;

		default:
			return;
	}
	if(obj.is_nil()) return;

	if(axis == COL_AXIS)
	{
		string units=QuGetAttrText(UNITS, obj);
		if(units.size() && units!=" ")
		{
			QString sanitised = QString(units.c_str()).replace(" summed", "");
			label = string(sanitised.toLatin1().data());
		}
	}
	else
	{

		default_label=QuGetAttrText(LABLAXIS,obj);
		if(default_label.size()==0) default_label=QuGetAttrText(FIELDNAM, obj);
		if(default_label.size()==0) default_label=QuGetAttrText(OBJECT_NAME, obj);
		if(default_label.size()!=0) label=default_label;
	
		string units=QuGetAttrText(UNITS, obj);
		if(units.size() && units!=" ")
		{
			label.append(" (");
			label.append(units);
			label.append(")");
		}
		string scid=QuGetAttrText(SC_ID,obj);
		if(scid.size() && scid!=" ")
		{
			label.append(" SC=");
			label.append(scid);
		}
	}
	
	QString greek = QString(label.c_str());
	greek.replace("theta","#gh");
	greek.replace("phi","#gf");
	label = greek.toLatin1().data();
	
   
	return;
}

// -------------------------------------

int SpectrumItem::getXData(QdObject_var xdo, QdObject_var xdo_end){
	int nx_data;

	QdTimeSeq_var    timeseq_t    = QdTimeSeq_var::narrow(xdo);
	QdRScalarSeq_var rscalarseq_x = QdRScalarSeq_var::narrow(xdo);
	QdRMatrix_var    rmatrix_x    = QdRMatrix_var::narrow(xdo);
  
	QdTimeSeq_var    timeseq_t_end    = QdTimeSeq_var::narrow(xdo_end);
	QdRScalarSeq_var rscalarseq_x_end = QdRScalarSeq_var::narrow(xdo_end);
  
  // if we don't understand xdo_end object call alternative method
	if(timeseq_t_end.is_nil() && rscalarseq_x_end.is_nil()) return getXData( xdo);
  
	 
	if (!timeseq_t.is_nil() )
	{
		nx_data = timeseq_t->sequence_size();
		_x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);
	  
      // create full array
		for (int i=0; i< nx_data; i++)
		{
			_x_start_all[i] = timeseq_t[i].getEpoch2000();
		}
	}
	else if ( !rscalarseq_x.is_nil() )
	{
		nx_data = rscalarseq_x->sequence_size();
		_x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);
	 
		for (int i=0; i< nx_data; i++)
		{
			_x_start_all[i] = rscalarseq_x[i];
		}
	}
	else if ( !rmatrix_x.is_nil() )
	{
		if(rmatrix_x->rank() == 1)
		{
			nx_data = rmatrix_x->dimdata()[0];
			_x_start_all.resize(nx_data);
			_x_end_all.resize(nx_data);
		  
			for (int i=0; i< nx_data; i++)
			{
				_x_start_all[i] = (*rmatrix_x)[i];
			}
		}
		else
		{
			return 0;
		}
  
	}
	else
	{
		return 0;
	}
  
	if (!timeseq_t_end.is_nil() && timeseq_t_end->sequence_size() == nx_data)
	{
		for (int i=0; i< nx_data; i++)
		{
			_x_end_all[i] = timeseq_t_end[i].getEpoch2000();
		}
	  
	}
	else if ( !rscalarseq_x_end.is_nil() && rscalarseq_x_end->sequence_size() == nx_data)
	{
		for (int i=0; i< nx_data; i++)
		{
			_x_end_all[i] = rscalarseq_x_end[i];
		}

	}
	else
	{
		return getXData(xdo);
	}

	return nx_data;
}


// ----------------------------------------------

int SpectrumItem::getXData(QdObject_var xdo, QdObject_var xdo_plus, QdObject_var xdo_minus){
	int nx_data;
  
  	// remove PEACE white gaps 
	if(frameType() == QP_TS && DataTab->gap_toggle->isChecked() )
	{
		return getXData(xdo);
	}
	
  
	_x_start_all.clear();
	_x_end_all.clear();
	
	if(frameType() == QP_TS && DataTab->interlace_toggle->isChecked() )
	{
		// Make spectra plot interlaced
		int nSpectra = panel()->numberSpectra();
		int nThis = panel()->numberThisSpectrum(this);
		QdTimeSeq_var    timeseq_t    = QdTimeSeq_var::narrow(xdo);
		if( !timeseq_t.is_nil() ) return getXData(timeseq_t, nSpectra, nThis);
	}

    // Allow for data set with single value for delta_plus and delta_minus
	if(xdo_plus.is_nil() ) {
      // if no deltas call basic xdo only method
		if( xdo_minus.is_nil()) {
			cout << " X Object delta plus and minus are empty" << endl;
			return getXData(xdo);
		}
		xdo_plus = new QdRScalar(0.0);
	}
	else if(xdo_minus.is_nil() ) xdo_minus = new QdRScalar(0.0);
	QdRScalar_var m1 = QdRScalar_var::narrow(xdo_minus);
	QdRScalar_var m2 = QdRScalar_var::narrow(xdo_plus);

	QdTimeSeq_var    timeseq_t    = QdTimeSeq_var::narrow(xdo);
	QdRScalarSeq_var rscalarseq_x = QdRScalarSeq_var::narrow(xdo);
	QdRMatrix_var    rmatrix_x    = QdRMatrix_var::narrow(xdo);
  
	QdTimeSeq_var    timeseq_t_plus    = QdTimeSeq_var::narrow(xdo_plus);
	QdRScalarSeq_var rscalarseq_x_plus = QdRScalarSeq_var::narrow(xdo_plus);
	QdRScalar_var    rscalar_x_plus    = QdRScalar_var::narrow(xdo_plus);
	QdRMatrix_var    rmatrix_x_plus    = QdRMatrix_var::narrow(xdo_plus);
  
	QdTimeSeq_var    timeseq_t_minus    = QdTimeSeq_var::narrow(xdo_minus);
	QdRScalarSeq_var rscalarseq_x_minus = QdRScalarSeq_var::narrow(xdo_minus);
	QdRScalar_var    rscalar_x_minus    = QdRScalar_var::narrow(xdo_minus);
	QdRMatrix_var    rmatrix_x_minus    = QdRMatrix_var::narrow(xdo_minus);
  	
  // if we don't understand xdo_deltas objects call alternative method
	if(timeseq_t_plus.is_nil() && rscalarseq_x_plus.is_nil() && rmatrix_x_plus.is_nil() && rscalar_x_plus.is_nil()) {
		cout << " X Object delta plus not allowed data type" << endl;
		return getXData(xdo);
	}
	if(timeseq_t_minus.is_nil() && rscalarseq_x_minus.is_nil() && rmatrix_x_minus.is_nil() && rscalar_x_minus.is_nil()) {
		cout << " X Object delta minus not allowed data type" << endl;
		return getXData(xdo);
	}
	if (!timeseq_t.is_nil() )
	{
		nx_data = timeseq_t->sequence_size();

		QdRScalarSeq_var d_minus;
		if( !timeseq_t_minus.is_nil() && timeseq_t_minus->sequence_size() == nx_data)
		{
			d_minus = new QdRScalarSeq(nx_data);
			for(int i=0; i<nx_data; i++)
			{
				d_minus[i] = timeseq_t_minus[i].getEpoch2000();
			}
		}
		else if( !rscalarseq_x_minus.is_nil() && rscalarseq_x_minus->sequence_size() == nx_data)
		{
			if(xdo_minus->xref_exists(SI_CONVERSION) )
			{
				QdObject_var dm = QarConvertUnitsTo(rscalarseq_x_minus, "1>s" );
				if( dm.is_nil() ) d_minus = rscalarseq_x_minus; // assumes seconds
				else  d_minus = QdRScalarSeq_var::narrow(dm);
			}
			else  d_minus = rscalarseq_x_minus; // assumes seconds
		}
		else if( !rscalar_x_minus.is_nil() )
		{
			d_minus = new QdRScalarSeq(nx_data);
			QdObject_var  dm;
			if(xdo_minus->xref_exists(SI_CONVERSION) )
			{
				dm = QarConvertUnitsTo(rscalar_x_minus, "1>s" );
			}
			else dm = rscalar_x_minus; // assumes seconds
		 
			for(int i=0; i<nx_data; i++)
			{
				d_minus[i] = QdRScalar_var::narrow(dm)->scalar();
			}
		}
		else
		{
			cout << " X Object delta minus sequence length != time sequence length" << endl;
			return getXData(xdo);
		}
	 
		QdRScalarSeq_var d_plus;
		if( !timeseq_t_plus.is_nil() && timeseq_t_plus->sequence_size() == nx_data)
		{
			d_plus = new QdRScalarSeq(nx_data);
			for(int i=0; i<nx_data; i++)
			{
				d_plus[i] = timeseq_t_plus[i].getEpoch2000();
			}
		}
		else if( !rscalarseq_x_plus.is_nil() && rscalarseq_x_plus->sequence_size() == nx_data)
		{
			if(xdo_plus->xref_exists(SI_CONVERSION) )
			{
				QdObject_var dp =  QarConvertUnitsTo(rscalarseq_x_plus, "1>s" );
				if( dp.is_nil() ) d_plus = rscalarseq_x_plus; // assumes seconds
				else d_plus = QdRScalarSeq_var::narrow(dp);
			}
			else  d_plus = rscalarseq_x_plus; // assumes seconds
		}
		else if( !rscalar_x_plus.is_nil() )
		{
			d_plus = new QdRScalarSeq(nx_data);
			QdObject_var  dp;
			if(xdo_plus->xref_exists(SI_CONVERSION) )
			{
				dp = QarConvertUnitsTo(rscalar_x_plus, "1.0>s" );
			}
			else dp = rscalar_x_plus; // assumes seconds
		 
			for(int i=0; i<nx_data; i++)
			{
				d_plus[i] = QdRScalar_var::narrow(dp)->scalar();
			}
		}
		else
		{
			cout << " X Object delta plus sequence length != time sequence length" << endl;
			return getXData(xdo);
		}

      // create  arrays
		_x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);
		for (int i=0; i< nx_data; i++)
		{
			double value_ip = timeseq_t[i].getEpoch2000();
			double value_im = value_ip;
			value_ip += d_plus[i];
			value_im -= d_minus[i];

			_x_end_all[i] = value_ip;
			_x_start_all[i] = value_im;
		}

	}
	else if ( !rscalarseq_x.is_nil() )
	{
		nx_data = rscalarseq_x->sequence_size();
	  	  
		QdRScalarSeq_var ss_start;
		if( !rscalarseq_x_minus.is_nil() && rscalarseq_x_minus->sequence_size() == nx_data)
		{
			try{
				ss_start = QdRScalarSeq_var::narrow(rscalarseq_x - rscalarseq_x_minus);
			}
			catch(Exception &e){
				e.print_msg();
				return 0;
			}
		}
		else if( !rscalar_x_minus.is_nil() )
		{
			try{
				ss_start = QdRScalarSeq_var::narrow(rscalarseq_x - rscalar_x_minus);
			}
			catch(Exception &e){
				e.print_msg();
				return 0;
			}
		}
		else
		{
			cout << " X Object delta minus sequence length or type error" << endl;
			return getXData(xdo);
		}
	 
		QdRScalarSeq_var ss_end;
		if( !rscalarseq_x_plus.is_nil() && rscalarseq_x_plus->sequence_size() == nx_data)
		{
			try{
				ss_end = QdRScalarSeq_var::narrow(rscalarseq_x + rscalarseq_x_plus);
			}
			catch(Exception &e){
				e.print_msg();
				return 0;
			}
		}
		else if( !rscalar_x_plus.is_nil() )
		{
			try{
				ss_end = QdRScalarSeq_var::narrow(rscalarseq_x + rscalar_x_plus);
			}
			catch(Exception &e){
				e.print_msg();
				return 0;
			}
		}
		else
		{
			cout << " X Object delta plus sequence length or type error" << endl;
			return getXData(xdo);
		}
	 
		_x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);
		for (int i=0; i< nx_data; i++)
		{
			_x_start_all[i] = ss_start[i];
			_x_end_all[i] = ss_end[i];
		}

	}
	else if ( !rmatrix_x.is_nil() )
	{
		if(rmatrix_x->rank() == 1)
		{
			nx_data = rmatrix_x->dimdata()[0];
		 
			QdRMatrix_var m_start;
			if( !rmatrix_x_minus.is_nil() &&  rmatrix_x_minus->dimdata()[0] == nx_data)
			{
				try{
					m_start = QdRMatrix_var::narrow(rmatrix_x - rmatrix_x_minus);
				}
				catch(Exception &e){
					e.print_msg();
					return 0;
				}
			}
			else if( !rscalar_x_minus.is_nil() )
			{
				try{
					m_start = QdRMatrix_var::narrow(rmatrix_x - rscalar_x_minus);
				}
				catch(Exception &e){
					e.print_msg();
					return 0;
				}
			}
			else
			{
				cout << " X Object delta minus type or dimension error" << endl;
				return getXData(xdo);
			}
	 
			QdRMatrix_var m_end;
			if( !rmatrix_x_plus.is_nil() &&  rmatrix_x_plus->dimdata()[0] == nx_data)
			{
				try{
					m_end = QdRMatrix_var::narrow(rmatrix_x + rmatrix_x_plus);
				}
				catch(Exception &e){
					e.print_msg();
					return 0;
				}
			}
			else if( !rscalar_x_plus.is_nil() )
			{
				try{
					m_end = QdRMatrix_var::narrow(rmatrix_x + rscalar_x_plus);
				}
				catch(Exception &e){
					e.print_msg();
					return 0;
				}
			}
			else
			{
				cout << " X Object delta plus type or dimension error" << endl;
				return getXData(xdo);
			}

		
			_x_start_all.resize(nx_data);
			_x_end_all.resize(nx_data);
			for (int i=0; i< nx_data; i++)
			{
				_x_start_all[i] = (*m_start)[i];
				_x_end_all[i] = (*m_end)[i];
			}

		}
		else
		{
			cout << "X Object matrix must be 1-D" << endl;
			return 0;
		}
  
	}
	else
	{
		cout << " X Object not time or scalar sequence nor 1D array" << endl;
		return 0;
	}
  
  // in the case of monotonic decreasing X the deltas may be applied the wrong way
  // ensure if decreasing that start > end
  
	if (_x_start_all[0] > _x_start_all[nx_data-1])
	{
    // decreasing bins, ensure start > end for each bin
		for(int i=0; i<nx_data; i++)
		{
			if(_x_start_all[i] < _x_end_all[i] )
			{
				double hold = _x_end_all[i];
				_x_end_all[i] = _x_start_all[i];
				_x_start_all[i] = hold;
			}
		}
	}
	return nx_data;
}

// ----------------------------------------------

int SpectrumItem::getXData(QdTimeSeq_var tdo, int nSpectra, int nItem){
	_x_start_all.clear();
	_x_end_all.clear();

	int nx_data;
	double half_spacing=0.1; // to ensure something plots

	if (!tdo.is_nil() )
	{
		nx_data = tdo->sequence_size();
		_x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);

	  // create  array
		for (int i=0; i< nx_data; i++)
		{
			double value_i = tdo[i].getEpoch2000();
			_x_start_all[i] = value_i;
		}
	  // find bin widths
		double diff[nx_data + 1];
		 
		for(int i=1; i<nx_data; i++)
		{
			diff[i] = _x_start_all[i] - _x_start_all[i-1];
		}
	  // handle ends
		diff[0] = diff[1];
		diff[nx_data] = diff[nx_data-1];
		 
	  // set start end of bin using widths
		for(int i=0; i < nx_data; i++)
		{
			_x_end_all[i] = _x_start_all[i] + + diff[i] * (((double) (nItem + 1) / nSpectra) - 0.5) ;
			_x_start_all[i] = _x_start_all[i]  + diff[i] * (((double) nItem / nSpectra) - 0.5)  ;	  
		}

	}
  
	return nx_data;
}

// ----------------------------------------------

int SpectrumItem::getXData(QdObject_var xdo){
	_x_start_all.clear();
	_x_end_all.clear();

	int nx_data;
	double half_spacing=0.1; // to ensure something plots
 
	QdTimeSeq_var    timeseq_t    = QdTimeSeq_var::narrow(xdo);
	QdRScalarSeq_var rscalarseq_x = QdRScalarSeq_var::narrow(xdo);
	QdRMatrix_var    rmatrix_x    = QdRMatrix_var::narrow(xdo);

	if (!timeseq_t.is_nil() )
	{
		nx_data = timeseq_t->sequence_size();
		_x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);

	    QmjdTime spacing_t;
	  // tolerence is tol * spacing, so half_spacing = tol / 2
//         if(0==calc_tol_spacing(spacing_t, *timeseq_t)) half_spacing = (spacing_t.getEpoch2000()) / (QSAS_profiles->current()->gapTolerance*2.);
	  
	  // create  array
		for (int i=0; i< nx_data; i++)
		{
			double value_i = timeseq_t[i].getEpoch2000();
			_x_start_all[i] = value_i;
		}
	  // find bin widths
		double diff[nx_data + 1];
		 
		for(int i=1; i<nx_data; i++)
		{
			diff[i] = _x_start_all[i] - _x_start_all[i-1];
		}
	  // handle ends
		diff[0] = diff[1];
		diff[nx_data] = diff[nx_data-1];
		 
	  // set start end of bin using widths
		for(int i=0; i < nx_data; i++)
		{
			_x_end_all[i] = _x_start_all[i] + 0.5 * diff[i+1];
			_x_start_all[i] = _x_start_all[i] - 0.5 * diff[i];
	  
		}

	}
	else if ( !rscalarseq_x.is_nil() )
	{
		nx_data = rscalarseq_x->sequence_size();
		_x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);
		half_spacing = getSpacingTolerance(rscalarseq_x);
	  
		for (int i=0; i< nx_data; i++)
		{
			double value_i =  rscalarseq_x[i];
			_x_start_all[i] = value_i;
			_x_end_all[i] = value_i + 2* half_spacing;
		}
	}
	else if ( !rmatrix_x.is_nil() )
	{
		if(rmatrix_x->rank() == 1)
		{
			nx_data = rmatrix_x->dimdata()[0];
			_x_start_all.resize(nx_data);
			_x_end_all.resize(nx_data);
			for (int i=0; i< nx_data; i++)
			{
				_x_start_all[i] = (*rmatrix_x)[i];
			}
		 // find bin widths
			double diff[nx_data + 1];
		 
			for(int i=1; i<nx_data; i++)
			{
				diff[i] = _x_start_all[i] - _x_start_all[i-1];
			}
		 // handle ends
			diff[0] = diff[1];
			diff[nx_data] = diff[nx_data-1];
		 
		 // set start end of bin using widths
			for(int i=0; i < nx_data; i++)
			{
				_x_end_all[i] = _x_start_all[i] + 0.5 * diff[i+1];
				_x_start_all[i] = _x_start_all[i] - 0.5 * diff[i];
			}
		}
		else
		{
			cout << "X Object matrix must be 1-D" << endl;
			return 0;
		}
  
	}
	else
	{
		cout << " X Object not time or scalar sequence nor 1D array" << endl;
		return 0;
	}
  
	return nx_data;
}

// -------------------------------------------------

int SpectrumItem::getYData(QdObject_var ydo, QdObject_var ydo_end)
{
	_y_start_all.clear();
	_y_end_all.clear();
	_y_start_all.resize(_nx_data);
	_y_end_all.resize(_nx_data);
	
	QdRMatrix_var    rmatrix_y    = QdRMatrix_var::narrow(ydo);
	QdRMatrixSeq_var rmatrixseq_y = QdRMatrixSeq_var::narrow(ydo);
	if( !rmatrixseq_y.is_nil() && (rmatrixseq_y->sequence_size() == 1 || rmatrixseq_y->sequence_size() != _nx_data) )
	{
		rmatrix_y = new QdRMatrix(rmatrixseq_y[0]);
		rmatrixseq_y = 0;
	}
	
	QdRMatrix_var rmatrix_y_end = QdRMatrix_var::narrow(ydo_end);
	QdRMatrixSeq_var rmatrixseq_y_end = QdRMatrixSeq_var::narrow(ydo_end);
	if( !rmatrixseq_y_end.is_nil() && (rmatrixseq_y_end->sequence_size() == 1 || rmatrixseq_y_end->sequence_size() != _nx_data ))
	{
		rmatrix_y_end = new QdRMatrix(rmatrixseq_y_end[0]);
		rmatrixseq_y_end  = 0;
	}


	int ny_data=0;
  

	if(!rmatrixseq_y.is_nil() )
	{
		for(int nx=0; nx < _nx_data; ++nx){

			if(rmatrixseq_y->elt(nx).rank() == 1)
			{
				ny_data = rmatrixseq_y->elt(nx).dimdata()[0];
				_y_start_all[nx].resize(ny_data);

//          for(int j=0; j<ny_data; j++)
// 	     {
// 	         _y_start_all[nx][j] = rmatrixseq_y->elt(nx)[j];
// 	     }
				_y_start_all[nx] = rmatrixseq_y->elt(nx);
			}
			else{
				cout <<  "Y Object is sequence but not 1-D" << endl;
				return -1;
			}
		}
	}
	else if(!rmatrix_y.is_nil())
	{
		ny_data = rmatrix_y->dimdata()[0];
		for(int nx=0; nx < _nx_data; ++nx){
			if(rmatrix_y->rank() == 1)
			{
				_y_start_all[nx].resize(ny_data);
// 		   for(int j=0; j<ny_data; j++)
// 	       {
// 	          _y_start_all[nx][j] =  (*rmatrix_y)[j] ;
// 	       }
				_y_start_all[nx] =  (*rmatrix_y);
			}
			else
			{
				cout << "Y Object matrix must be 1-D" << endl;
				return -1;
			}
		}
	}
	else
	{
		plmtex("lv",-2.0,0.75,0.0, "Y Object not a matrix or matrix sequence");
		return -1;
	}
  
  // Handle y_end
  
	if( rmatrix_y_end.is_nil() && rmatrixseq_y_end.is_nil()) return getYData( ydo);
  
	if( !rmatrixseq_y_end.is_nil())
	{
		for(int nx=0; nx< _nx_data; ++nx){

			ny_data = rmatrixseq_y_end->elt(nx).dimdata()[0];
			if(_y_start_all[nx].size() != ny_data)
			{
				cout <<  "Y End Sequence [nx] dimension error" << endl;
				return -1;
			}
			_y_end_all[nx].resize(ny_data);
//         for(int j=0; j<ny_data; j++)
// 	    {
//            _y_end_all[nx][j] =  rmatrixseq_y_end->elt(nx)[j] ;
// 	    }
			_y_end_all[nx] =  rmatrixseq_y_end->elt(nx);
		}
	}
	else if( !rmatrix_y_end.is_nil())
	{
		ny_data = rmatrix_y_end->dimdata()[0];
		for(int nx=0; nx< _nx_data; ++nx){

			if(_y_start_all[nx].size() != ny_data)
			{
				cout << "Y End Object dimension error" << endl;
				return -1;
			}
			_y_end_all[nx].resize(ny_data);
//         for(int j=0; j<ny_data; j++)
// 	    {
//            _y_end_all[nx][j] =  (*rmatrix_y_end)[j] ;
// 	    }
			_y_end_all[nx]=(*rmatrix_y_end);

		}
	}
	return ny_data;
  
}


// -------------------------------------------------

int SpectrumItem::getYData(QdObject_var ydo, QdObject_var ydo_plus, QdObject_var ydo_minus)
{
	_y_start_all.clear();
	_y_end_all.clear();
	_y_start_all.resize(_nx_data);
	_y_end_all.resize(_nx_data);
	int ny_data=0;
	double value_i;

	bool isDeg=false; // not used
	QuIsAngle(ydo, isDeg) ;
	
	// Allow for data set with single value for delta_plus and delta_minus
	if(ydo_plus.is_nil() ) {
		// if no deltas call basic xdo only method
		if( ydo_minus.is_nil()) {
			cout << " Y Object delta plus and minus are empty" << endl;
			return getYData( ydo);
		}
		ydo_plus = new QdRScalar(0.0);
	}
	else if(ydo_minus.is_nil() ) ydo_minus = new QdRScalar(0.0);

	QdRMatrix_var    rmatrix_y    = QdRMatrix_var::narrow(ydo);
	QdRMatrixSeq_var rmatrixseq_y = QdRMatrixSeq_var::narrow(ydo);

	if(!rmatrixseq_y.is_nil() && ( rmatrixseq_y->sequence_size() == 1 || rmatrixseq_y->sequence_size() != _nx_data) )
	{
		rmatrix_y = new QdRMatrix(rmatrixseq_y[0]);
		rmatrixseq_y = 0;
	}

	QdRScalarSeq_var rscalarseq_y_plus = QdRScalarSeq_var::narrow(ydo_plus);
	QdRScalar_var    rscalar_y_plus    = QdRScalar_var::narrow(ydo_plus);
	QdRMatrix_var    rmatrix_y_plus    = QdRMatrix_var::narrow(ydo_plus);
	QdRMatrixSeq_var    rmatrixseq_y_plus    = QdRMatrixSeq_var::narrow(ydo_plus);

	if(!rmatrixseq_y_plus.is_nil() && ( rmatrixseq_y_plus->sequence_size() == 1 || rmatrixseq_y_plus->sequence_size() != _nx_data) )
	{
		rmatrix_y_plus = new QdRMatrix(rmatrixseq_y_plus[0]);
		rmatrixseq_y_plus = 0;
	}

	QdRScalarSeq_var rscalarseq_y_minus = QdRScalarSeq_var::narrow(ydo_minus);
	QdRScalar_var    rscalar_y_minus    = QdRScalar_var::narrow(ydo_minus);
	QdRMatrix_var    rmatrix_y_minus    = QdRMatrix_var::narrow(ydo_minus);
	QdRMatrixSeq_var    rmatrixseq_y_minus    = QdRMatrixSeq_var::narrow(ydo_minus);

	if(!rmatrixseq_y_minus.is_nil() && (rmatrixseq_y_minus->sequence_size() == 1 || rmatrixseq_y_minus->sequence_size() != _nx_data ) )
	{
		rmatrix_y_minus = new QdRMatrix(rmatrixseq_y_minus[0]);
		rmatrixseq_y_minus = 0;
	}


	if( rscalarseq_y_plus.is_nil() && rmatrix_y_plus.is_nil() && rscalar_y_plus.is_nil() && rmatrixseq_y_plus.is_nil()) {
		cout << " Y Object delta plus not allowed data type" << endl;
		return getYData( ydo);
	}
	if( rscalarseq_y_minus.is_nil() && rmatrix_y_minus.is_nil() && rscalar_y_minus.is_nil() && rmatrixseq_y_minus.is_nil()) {
		cout << " Y Object delta minus not allowed data type" << endl;
		return getYData(  ydo);
	}

	if(!rmatrixseq_y.is_nil())
	{
		if(rmatrixseq_y->elt(0).rank() == 1)
		{
			QdRMatrixSeq_var ms_start;
			QdRMatrixSeq_var ms_end;
	  
			if( !rscalarseq_y_minus.is_nil() )
			{
				try{
					ms_start = QdRMatrixSeq_var::narrow(rmatrixseq_y - rscalarseq_y_minus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else if(!rscalar_y_minus.is_nil() )
			{
				try{
					ms_start = QdRMatrixSeq_var::narrow(rmatrixseq_y - rscalar_y_minus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else if(!rmatrix_y_minus.is_nil() )
			{
				try{
					ms_start = QdRMatrixSeq_var::narrow(rmatrixseq_y - rmatrix_y_minus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else if(!rmatrixseq_y_minus.is_nil() )
			{
				if(rmatrixseq_y_minus->sequence_size() != _nx_data )
				{
					cout << "delta plus sequence of wrong length" << endl;
					return -1;
				}
				try{
					ms_start = new QdRMatrixSeq(_nx_data, rmatrixseq_y->elt(0).dimdata());
					for(int i=0; i<_nx_data; i++)
					{
						ms_start[i] = rmatrixseq_y[i] - rmatrixseq_y_minus[i];
					}
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
	  
			if( !rscalarseq_y_plus.is_nil() )
			{
				try{
					ms_end = QdRMatrixSeq_var::narrow(rmatrixseq_y + rscalarseq_y_plus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else if(!rscalar_y_plus.is_nil() )
			{
				try{
					ms_end = QdRMatrixSeq_var::narrow(rmatrixseq_y + rscalar_y_plus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else if(!rmatrix_y_plus.is_nil() )
			{
				try{
					ms_end = QdRMatrixSeq_var::narrow(rmatrixseq_y + rmatrix_y_plus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else if(!rmatrixseq_y_plus.is_nil() )
			{
				if(rmatrixseq_y_plus->sequence_size() != _nx_data )
				{
					cout << "delta plus sequence of wrong length" << endl;
					return -1;
				}
				try{
					ms_end = new QdRMatrixSeq(_nx_data, rmatrixseq_y->elt(0).dimdata());
					for(int i=0; i<_nx_data; ++i)
					{
						ms_end[i] = rmatrixseq_y[i] + rmatrixseq_y_plus[i];
					}
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
	  

			if(ms_start->sequence_size() != _nx_data)
			{
				cout << "delta minus sequence of wrong length" << endl;
				return -1;
			}
			if(ms_end->sequence_size() != _nx_data)
			{
				cout << "delta plus sequence of wrong length" << endl;
				return -1;
			}
	  
			for(int nx=0; nx< _nx_data; ++nx){

				ny_data = rmatrixseq_y->elt(nx).dimdata()[0];
				_y_start_all[nx].resize(ny_data);
				_y_end_all[nx].resize(ny_data);

// 				for(int j=0; j<ny_data; j++)
// 				{
// 					_y_start_all[nx][j] = ms_start->elt(nx)[j];
// 					_y_end_all[nx][j] = ms_end->elt(nx)[j];
// 				}
				_y_start_all[nx]=ms_start->elt(nx);
				_y_end_all[nx]=ms_end->elt(nx);

			}
		}
		else{
			cout << "Y Object is sequence but not 1-D" << endl;
			return -1;
		}
	}
	else if(!rmatrix_y.is_nil())
	{
		QdRMatrix_var m_start;
		QdRMatrix_var m_end;
		if(rmatrix_y->rank() == 1)
		{
			if(!rscalar_y_minus.is_nil() )
			{
				try{
					m_start = QdRMatrix_var::narrow(rmatrix_y - rscalar_y_minus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else if(!rmatrix_y_minus.is_nil() )
			{
				try{
					m_start = QdRMatrix_var::narrow(rmatrix_y - rmatrix_y_minus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else
			{
				cout << " Y Object delta plus is sequence " << endl;
				return getYData( ydo);
			}
	  
			if(!rscalar_y_plus.is_nil() )
			{
				try{
					m_end = QdRMatrix_var::narrow(rmatrix_y + rscalar_y_plus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else if(!rmatrix_y_plus.is_nil() )
			{
				try{
					m_end = QdRMatrix_var::narrow(rmatrix_y + rmatrix_y_plus);
				}
				catch(Exception &e){
					e.print_msg();
					return -1;
				}
			}
			else
			{
				cout << " Y Object delta minus is sequence " << endl;
				return getYData(ydo);
			}

			ny_data = rmatrix_y->dimdata()[0];
			for(int nx=0; nx< _nx_data; ++nx)
			{
				_y_start_all[nx].resize(ny_data);
				_y_end_all[nx].resize(ny_data);
	  
// 				for(int j=0; j<ny_data; j++)
// 				{
// 					_y_start_all[nx][j] = (*m_start)[j];
// 					_y_end_all[nx][j] = (*m_end)[j];
// 				}
				_y_start_all[nx]=(*m_start);
				_y_end_all[nx]=(*m_end);
			}

		}
		else
		{
			cout << "Y Object matrix must be 1-D" << endl;
			return -1;
		}
	}
	else
	{
		cout << "Y Object is not a matrix or matrix sequence" << endl;
		return -1;
	}

	// in the case of monotonic decreasing Y the deltas may be applied the wrong way
	// ensure if decreasing that start > end
  
	for(int j=0; j< _nx_data; j++)
	{
		ny_data = _y_start_all[j].size();
		if ( (_y_start_all[j])[0] > (_y_start_all[j])[ny_data-1])
		{
			// decreasing bins, ensure start > end for each bin
			for(int i=0; i<ny_data; i++)
			{
				if((_y_start_all[j])[i] < (_y_end_all[j])[i] )
				{
					double hold = (_y_end_all[j])[i];
					(_y_end_all[j])[i] = (_y_start_all[j])[i];
					(_y_start_all[j])[i] = hold;
				}
			}
		}
	}
	return ny_data;
}

// -------------------------------------------------

int SpectrumItem::getYData(QdObject_var ydo)
{
	if ( QuSlotDebug() )
	{
		printf( "SpectrumItem::getYData\n" );
		fflush(stdout);
	}

	_y_start_all.clear();
	_y_end_all.clear();
	_y_start_all.resize(_nx_data);
	_y_end_all.resize(_nx_data);

	QdRMatrix_var    rmatrix_y    = QdRMatrix_var::narrow(ydo);
	QdRMatrixSeq_var rmatrixseq_y = QdRMatrixSeq_var::narrow(ydo);
	if( !rmatrixseq_y.is_nil() && ( rmatrixseq_y->sequence_size() == 1 ||  rmatrixseq_y->sequence_size() != _nx_data ))
	{
		rmatrix_y = new QdRMatrix(rmatrixseq_y[0]);
		rmatrixseq_y = 0;
	}

	int ny_data=0;

	if(!rmatrixseq_y.is_nil())
	{
		for(int nx=0; nx< _nx_data; ++nx){
			if(rmatrixseq_y->elt(nx).rank() == 1)
			{
				ny_data = rmatrixseq_y->elt(nx).dimdata()[0];
				_y_start_all[nx].resize(ny_data);
				_y_end_all[nx].resize(ny_data);
// 				for(int j=0; j<ny_data-1; j++)
// 				{
// 					_y_start_all[nx][j] = rmatrixseq_y->elt(nx)[j];
// 					_y_end_all[nx][j] = rmatrixseq_y->elt(nx)[j+1];
// 				}
				_y_start_all[nx] = rmatrixseq_y->elt(nx)[std::gslice(0, valarray<size_t>(ny_data,1), valarray<size_t>(1,1))];
				_y_end_all[nx] = rmatrixseq_y->elt(nx)[std::gslice(1, valarray<size_t>(ny_data-1,1), valarray<size_t>(1,1))];
				//TODO seems to work. For safety, I've left the old version commented out.

			//	_y_start_all[nx][ny_data-1] = rmatrixseq_y->elt(nx)[ny_data-1];
				double lastVal = rmatrixseq_y->elt(nx)[ny_data-1];
				double lastValminus = rmatrixseq_y->elt(nx)[ny_data-2];
				_y_end_all[nx][ny_data-1] = ( 2.0*lastVal - lastValminus);
			}
			else{
				cout << "Y Object is sequence but not 1-D" << endl;
				return -1;
			}
		}
	}
	else if(!rmatrix_y.is_nil())
	{
		if(rmatrix_y->rank() != 1)
		{
			cout << "Y Object matrix must be 1-D" << endl;
			return -1;
		}

		for(int nx=0; nx< _nx_data; nx++){
	  
			ny_data = rmatrix_y->dimdata()[0];
			_y_start_all[nx].resize(ny_data);
			_y_end_all[nx].resize(ny_data);
			if(ny_data == 1){
				double value = (*rmatrix_y)[0];
				_y_start_all[nx][0] = value;
				_y_end_all[nx][0] = value+1.;
			}
			else
			{
// 				for(int j=0; j<ny_data-1; j++)
// 				{
// 					_y_start_all[nx][j] = (*rmatrix_y)[j];
// 					_y_end_all[nx][j] = (*rmatrix_y)[j+1];
// 				}
				_y_start_all[nx]=(*rmatrix_y)[std::gslice(0, valarray<size_t>(ny_data,1), valarray<size_t>(1,1))];
				_y_end_all[nx]=(*rmatrix_y)[std::gslice(1, valarray<size_t>(ny_data-1,1), valarray<size_t>(1,1))];
				//TODO seems to work. For safety, I've left the old version commented out.
				
				double value = (*rmatrix_y)[ny_data-1];
				value *= 2.0;
				value -= (*rmatrix_y)[ny_data-2];
				_y_end_all[nx][ny_data-1] = value;
			}
		}
	}
	else
	{
		cout << "Y Object is not a matrix or matrix sequence" << endl;
		return -1;
	}
	return ny_data;
  
}

// -------------------------------------------------

int SpectrumItem::isZConformal(int nx_len, int ny_len)
{
   // does something only for 2D matrix or 2D matrix sequence
	if ( QuSlotDebug() )
	{
		printf( "SpectrumItem::isZConformal nx=%d, ny=%d\n", nx_len , ny_len );
		fflush(stdout);
	}

	QdRMatrixSeq_var rmatrixseq_z = QdRMatrixSeq_var::narrow(_zdo);
	QdRMatrix_var    rmatrix_z    = QdRMatrix_var::narrow(_zdo);
	if( rmatrix_z.is_nil() && rmatrixseq_z.is_nil() ) return 1; // do nothing

	int n0, n1;
	if( !rmatrixseq_z.is_nil() )
	{
		if(  rmatrixseq_z->elt(0).rank() != 2) return 1; // do nothing, not 2D array
		n0 = rmatrixseq_z->elt(0).dimdata()[0];
		n1 = rmatrixseq_z->elt(0).dimdata()[1];
	}
	else
	{
		if(  rmatrix_z->rank() != 2) return 1; // do nothing, not 2D array
		n0 = rmatrix_z->dimdata()[0];
		n1 = rmatrix_z->dimdata()[1];
	}
	if (n0 == n1)
	{
      // square matrix, so take care
		if(_zdo->xref_exists(DEPEND_1) && _zdo->xref_exists(DEPEND_2) )
		{
			QdObject_var dep1 = _zdo->get_xref(DEPEND_1);
			int id_1 = dep1->get_id();
			QdObject_var dep2 = _zdo->get_xref(DEPEND_2);
			int id_2 = dep2->get_id();
			if(id_1 == _xdo->get_id() && id_2 == _ydo->get_id() ) return 0;
			else if (id_2 == _xdo->get_id() && id_1 == _ydo->get_id() ) return 1; // default behaviour
			else
			{
				cout << "Z data is square matrix don't know which axis is which, add DEPEND_1 and DEPEND_2";
				return -1;
			}
		}
		else return 1; // No idea, do nothing, 1st index is Y, 2nd is X
	}
	else if(n0 == nx_len && n1 == ny_len)
	{
		return 0;
	}
	else if(n0 == ny_len && n1 == nx_len)
	{
		return 1;
	}
	else return -1;
}

//----------------------------

void SpectrumItem::getSpectrogramName(string& name)
{
  // fetch name of spectrogram object (for display in listview)

	name = DataTab->_z_data_fld->text().toLatin1().data();
	if(name == "")
	{
		if(frameType() == QP_XY)
			name = "XY SpectroGram";
		else
			name = "TS SpectroGram";
	}
}

//----------------------------

string SpectrumItem::getTitle()
{
	// fetch title for current spectrogram object

	if(_zdo.is_nil())
	{
		return string("");
	}
	string title = QuGetAttrText(FIELDNAM, _zdo);
	if(title.size() == 0) title += DataTab->_z_data_fld->text().toLatin1().data();
  
	return title;
}
// --------------------------------

int SpectrumItem::getXmetaIsLog()
{
	if ( QuSlotDebug() )
	{
		printf( "QpSpectrumItem::getXmetaIsLog \n" );
		fflush(stdout);
	}
	
	int isLog = 0;
	
	// find SCALETYP
		
	if (!_xdo.is_nil() && _xdo->xref_exists(SCALETYP) )
	{
		QdString_var scaleType = QdString_var::narrow(_xdo->get_xref(SCALETYP));
		if(!scaleType.is_nil() )
		{
			if(QuToUpper(scaleType->c_str()) == "LOG") isLog = 1;
		}
	}
	return isLog;
}


// -----------------------------------------------------

bool SpectrumItem::getXmetaRange(double& xmin, double& xmax)
{
	// gets x min and max values for this spectrogram object
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::getXmetaRange" << endl;
    
	bool rangeFound = false;
	
	// reject if no Z object yet
	if(_zdo.is_nil()) return false;

	if ( _nx_data == 0 )   return false;
	
	if(frameType() == QP_TS)
	{
		QdTimeSeq_var tdo = get_timetags(_zdo);
		if(tdo.is_nil() || tdo->sequence_size() == 0) return false;
	
	// Change to use bins as Y range tests against bin edges which include delta_plus/minus
	//	xmin = tdo[0].getEpoch2000();
	//	xmax = tdo[tdo->sequence_size()-1].getEpoch2000();
		xmin = _x_start_all[0];
		xmax = _x_end_all[_nx_data-1];

		rangeFound = true;
	}
	else
	{
		if( DataTab->timeAsSec_toggle->isChecked() )
		{
			QdTimeSeq_var tdo = get_timetags(_zdo);
			if(tdo.is_nil() || tdo->sequence_size() == 0) return false;
		
		//	xmin = tdo[0].getEpoch2000();
		//	xmax = tdo[tdo->sequence_size()-1].getEpoch2000();
			xmin = _x_start_all[0];
			xmax = _x_end_all[_nx_data-1];
			rangeFound = true;
		}
		else
		{
			if (_xdo->xref_exists(SCALEMIN) && _xdo->xref_exists(SCALEMIN))
			{
				QdRScalar_var smin = QdRScalar_var::narrow(_xdo->get_xref(SCALEMIN));
				QdRScalar_var smax = QdRScalar_var::narrow(_xdo->get_xref(SCALEMAX));
				if(smin.is_nil() || smax.is_nil()) return false;
			
				xmin = *smin;
				xmax = *smax;
				rangeFound = true;
			}
		}
	}
	if(rangeFound) return rangeFound;
	else return getXRange( xmin,  xmax);

}
// -----------------------------------------------------

bool SpectrumItem::getXRange(double& xmin, double& xmax)
{
	// gets x min and max values for this spectrogram object
	if ( QuSlotDebug() ) cout << "SpectrumItem::getXRange" << endl;
    
	// reject if no Z object yet
	if(_zdo.is_nil()) return false;
  	
	xmin = DBL_MAX;
	xmax = -DBL_MAX;
	
	if ( _xdo.is_nil() )   return false;
	
	if(_xdo->get_id() == xdo_id_getXRange)
	{
		xmin = x_min_data;
		xmax = x_max_data;
		return true;
	}
	xdo_id_getXRange = _xdo->get_id();
  
	for(int i=0; i<_nx_data; i++)
	{
		double mini = _x_start_all[i];
		double maxi = _x_end_all[i];
		// may not be increasing bins
		if(mini < maxi)
		{
			if(mini < xmin) xmin = mini;
			if(maxi > xmax) xmax = maxi;
		}
		else
		{
			if(maxi < xmin) xmin = maxi;
			if(mini > xmax) xmax = mini;
		}
	}
   
   // store for reuse
	x_min_data = xmin;
	x_max_data = xmax;
    
	return true;
}

// --------------------------------

int SpectrumItem::getYmetaIsLog()
{
	if ( QuSlotDebug() )
	{
		printf( " QpSpectrumItem::getXmetaIsLog \n" );
		fflush(stdout);
	}
	 
	int isLog=0;
	// find SCALETYP
		
	if (!_ydo.is_nil() && _ydo->xref_exists(SCALETYP) )
	{
		QdString_var scaleType = QdString_var::narrow(_ydo->get_xref(SCALETYP));
		if(!scaleType.is_nil() )
		{
			if(QuToUpper(scaleType->c_str()) == "LOG")  isLog = 1;
		}
	}
	return isLog;
}

// ------------------------------------

bool SpectrumItem::getYmetaRange(double& ymin, double& ymax)
{
	// gets y min and max values for whole Y axis
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::getYmetaRange" << endl;

	ymin=0;
	ymax=0;
	bool rangeFound = false;
  
	if(_zdo.is_nil()) return false;
	
	// reject if no X object yet (also traps no z object)
	if ( _xdo.is_nil() )   return false;
  
	if ( _ydo.is_nil() ) return false;

	if (_ydo->xref_exists(SCALEMIN) && _ydo->xref_exists(SCALEMAX))
	{
		QdRScalar_var smin = QdRScalar_var::narrow(_ydo->get_xref(SCALEMIN));
		QdRScalar_var smax = QdRScalar_var::narrow(_ydo->get_xref(SCALEMAX));
		if(!smin.is_nil() && !smax.is_nil() ){
			ymin = *smin;
			ymax = *smax;
			rangeFound = true;
		}
	}
	
	if(rangeFound) return rangeFound;
	else	return getYRange(ymin, ymax);
	
}

// ---------------------------------------

bool SpectrumItem::getYRange(double& ymin, double& ymax)
{
	// gets y min and max values for all x values in range
	// xmin - xmax for the current spectrogram object
	if ( QuSlotDebug() ) cout << "SpectrumItem::getYRange" << endl;
	
	double xmin, xmax;
	
	if(frameType()==QP_TS)
	{
		xmin = frame()->tAxisTab->_axis_time_fld->getStart().getEpoch2000();
		xmax = frame()->tAxisTab->_axis_time_fld->getEnd().getEpoch2000();
	}
	else
	{
		xmin = frame()->xAxisTab->_axis_xy_fld->getStart();
		xmax = frame()->xAxisTab->_axis_xy_fld->getEnd();
	}
	
	if(xmin >= xmax) return false;
	
	ymin = DBL_MAX;
	ymax = -DBL_MAX;
	
	bool recalc = false;
	
	// reject if no Z objects yet
	if(_zdo.is_nil()) return false;
	if ( _xdo.is_nil() )   return false;
	if ( _ydo.is_nil() ) return false;
	
	if(x_range_min != xmin || x_range_max != xmax)
	{
		// keep x range used to calculate y range
		x_range_min = xmin;
		x_range_max = xmax;
		recalc = true;
	}
	
	if( _ydo->get_id() != ydo_id_getYRange) recalc = true;
		
	ydo_id_getYRange = _ydo->get_id();

	if(!recalc)
	{
		ymin = y_min_data;
		ymax = y_max_data;
		return true;
	}

	for(int nx=0; nx<_x_start_all.size(); nx++)
	{
		double dx1 =  _x_start_all[nx] ;
		double dx2 =  _x_end_all[nx];
      	
		// skip if out of range
		if(dx1 < dx2)
		{
			if(dx1 < xmin || dx2 > xmax) continue;
		}
		else
		{
			if(dx2 < xmin || dx1 > xmax) continue;
		}

		// find ymin and ymax for each x bin
		for(int j=0; j<_y_start_all[nx].size(); j++)
		{
			double ymini = (_y_start_all[nx])[j];
			double ymaxi = (_y_end_all[nx])[j];
			// may be decreasing bin
			if(ymini < ymaxi)
			{
				if(ymax < ymaxi) ymax = ymaxi;
				if(ymin > ymini) ymin = ymini;
			}
			else
			{
				// decreasing bin
				if(ymax < ymini) ymax = ymini;
				if(ymin > ymaxi) ymin = ymaxi;
			}
		}
	}
  	
	// store to reuse if nothing changed
	y_min_data = ymin;
	y_max_data = ymax;

	return true;

}

// -----------------------------------------------------

bool SpectrumItem::getZmetaRange(double& zmin,double& zmax)
{
	// gets z min and max values using metadata
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::getZmetaRange" << endl;
	
	bool rangeFound = false;
	
	if(_zdo.is_nil()) return false;

	if (_zdo->xref_exists(SCALEMIN) && _zdo->xref_exists(SCALEMIN))
	{
		QdRScalar_var smin = QdRScalar_var::narrow(_zdo->get_xref(SCALEMIN));
		if(smin.is_nil()) return false;
		zmin = *smin;
		
		QdRScalar_var smax = QdRScalar_var::narrow(_zdo->get_xref(SCALEMAX));
		if(smax.is_nil()) return false;
		zmax = *smax;
		rangeFound = true;
	}
				
	// set lin/log from metadata
	int scaleTypeFlag = QP_LOG;
	
	if (_zdo->xref_exists(SCALETYP) )
	{
		QdString_var scaleType = QdString_var::narrow(_zdo->get_xref(SCALETYP));
		if( !scaleType.is_nil() ) 
		{
			if (QuToUpper(scaleType->c_str()) == "LOG")
				scaleTypeFlag = QP_LOG;
			else
				scaleTypeFlag = QP_LINEAR;

		}
	}
	ColourscaleTab->_colourscale_linlog_fld->setCurrentIndex(scaleTypeFlag);
	
	return rangeFound;
}

// ----------------------------------

bool SpectrumItem::getZRange(double &zmin, double &zmax)
{
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::getZRange" << endl;

	double xmin, xmax;
	
	if(frameType()==QP_TS)
	{
		xmin = frame()->tAxisTab->_axis_time_fld->getStart().getEpoch2000();
		xmax = frame()->tAxisTab->_axis_time_fld->getEnd().getEpoch2000();
	}
	else
	{
		xmin = frame()->xAxisTab->_axis_xy_fld->getStart();
		xmax = frame()->xAxisTab->_axis_xy_fld->getEnd();
	}
	
	double ymin = panel()->yAxisTab->_axis_xy_fld->getStart();
	double ymax = panel()->yAxisTab->_axis_xy_fld->getEnd();

	// only try if x, y ranges available and valid
	if(xmin >= xmax || ymin >= ymax) return false;

	zmin = DBL_MAX;
	zmax = -DBL_MAX;
	bool recalc = false;

	 // fill X will reject if no Z object yet
	if(_zdo.is_nil()) return false;
	if( _zdo->get_id() != zdo_id_getZRange) recalc = true;
		
	zdo_id_getZRange = _zdo->get_id();

	if ( _xdo.is_nil() )   return false;
	if ( _ydo.is_nil() ) return false;
  
	QpAxisTab * axisTab;
	if(panel()->frameType() == QP_TS)  axisTab = frame()->tAxisTab;
	else  axisTab = frame()->xAxisTab;

	if(x_range_min_z != xmin || x_range_max_z != xmax )
	{
		x_range_min_z = xmin;
		x_range_max_z = xmax;
		recalc = true;
	}
	
	axisTab = panel()->yAxisTab;
	if(y_range_min_z != ymin || y_range_max_z != ymax )
	{
		y_range_min_z = ymin;
		y_range_max_z = ymax;
		recalc = true;
	}

	if(!recalc)
	{
		zmin = z_min_data;
		zmax = z_max_data;
		return true;
	}
	
	if(_nx_data != _zdo->sequence_size() && _zdo->sequence_size() != 1) return false;
  
	int XtoY = isZConformal(_nx_data, _ny_data);
	if( XtoY == -1)  return false;

	if( fill_Z_Data( XtoY,false) == -1)  return false;

	double zmin_for_log = DBL_MAX;

	for(int nx=0; nx<_x_start_all.size(); nx++)
	{
		double dx1 =  _x_start_all[nx] ;
		double dx2 =  _x_end_all[nx];
		// skip if out of range
		if(dx1 < dx2)
		{
			if(dx1 < xmin || dx2 > xmax) continue;
		}
		else
		{
			if(dx2 < xmin || dx1 > xmax) continue;
		}

		if (_z_data_all[nx].size() != _y_start_all[nx].size()) return false;
		for(int j=0; j<_y_start_all[nx].size();  j++)
		{
			double dy1 =  (_y_start_all[nx])[j] ;
			double dy2 =  (_y_end_all[nx])[j];
			// skip if out of range
			if(dy1 < dy2)
			{
				if(dy1 < ymin || dy2 > ymax) continue;
			}
			else
			{
				if(dy2 < ymin || dy1 > ymax) continue;
			}
			double value_j = (_z_data_all[nx])[j];
			if(value_j == missing_value) continue;
			 
			if(value_j < zmin_for_log && value_j != 0 ) zmin_for_log = value_j;
			
			if(value_j < zmin) 	zmin = value_j;
			if(value_j > zmax) zmax = value_j;
			
		}
	}
	 
	if(ColourscaleTab->_colourscale_linlog_fld->currentIndex() == QP_LOG)
	{
		if(zmin_for_log == DBL_MAX)
		{
			zmin = 1.0; // defensive just in case we found no non-zero values
			zmax = 10.0;
		}
		else zmin = zmin_for_log;
	}

	// keep for reuse if nothing changed   
	z_min_data = zmin;
	z_max_data = zmax;

	return true;
}



// ------------------------------------

QdObject_var SpectrumItem::getZDO()
{
	// get zdo object
	if ( QuSlotDebug() ) cout << "SpectrumItem::getZDO" << endl;

	QdObject_var zdo = DataTab->_z_data_fld->getSlotObject();

	if(!zdo.is_nil())
	{
		if(_z_obj_id != zdo->get_id())
		{
			_z_obj_id = zdo->get_id();
			
			// get z object fill value
			QdObject_var fillObj;
			if( zdo->xref_exists(FILL_VALUE)) fillObj = zdo->get_xref(FILL_VALUE);
			QdRScalar_var fill_S = QdRScalar_var::narrow(fillObj);
			if(fill_S.is_nil() ) missing_value = 0; // not handling array type fill
			else missing_value = fill_S->scalar();
		}
	 
	}
	else _z_obj_id = 0;
	
	return zdo;
}

// ------------------------------------

int SpectrumItem::fill_X_Data()
{
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::fill_X_Data" << endl;
	if (_zdo.is_nil() ) return 0; // cannot guarantee to create xdo so don't try till zdo exists

	QdObject_var xdo_end, xdo_delta_plus, xdo_delta_minus;
	
	// find x data object _xdo
	if(frameType() == QP_TS)
	{
		_xdo = get_timetags(_zdo);
		if(_xdo.is_nil())
		{
			cout << "No time tags for TS spectrogram";
			return 0;
		}
		
		if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
		if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
		if(_zdo->xref_exists(DEPEND_END_0))  xdo_end = _zdo->get_xref(DEPEND_END_0);

	}
	else
	{
		// X-Y Spectrogram
		
		if(DataTab->timeAsSec_toggle->isChecked())
		{
			QdTimeSeq_var tt = get_timetags(_zdo);
			QdTime_var tOffset = QdTime_var::narrow(DataTab->_x_start_data_fld->getSlotObject());
			if(tt.is_nil() || tOffset.is_nil()) return 0;
			
			QdRScalarSeq_var ss = new QdRScalarSeq(tt->sequence_size());
			for(int i=0; i<tt->sequence_size(); i++)
			{
				ss[i] = tt[i].getDiffSecs(*tOffset);
			}
			_xdo = ss;
			if(tt->xref_exists(DELTA_PLUS))  xdo_delta_plus = tt->get_xref(DELTA_PLUS);
			if(tt->xref_exists(DELTA_MINUS)) xdo_delta_minus = tt->get_xref(DELTA_MINUS);
		}
		else
		{
			// use X data slot object
		
			_xdo = DataTab->_x_start_data_fld->getSlotObject();

			if(!_xdo.is_nil())
			{
	//			xdo_end = DataTab->_x_end_data_fld->getSlotObject();

				if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
				if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
			}
			else
			{
				// Nothing in X slot
				if(_zdo->sequence_size() == 1){
					if( DataTab->_XYtoggle->isChecked() )
					{
						// swap X and Y is on
						if(_zdo->xref_exists(DEPEND_1)) _xdo = _zdo->get_xref(DEPEND_1);
						else  _xdo = makeIndexObject(_zdo, 1);

						if(_zdo->xref_exists(DEPEND_END_1)) xdo_end = _zdo->get_xref(DEPEND_END_1);
						else if(!_xdo.is_nil())
						{
							if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
							if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
						}
					}
					else
					{
						if(_zdo->xref_exists(DEPEND_2)) _xdo = _zdo->get_xref(DEPEND_2);
						else  _xdo = makeIndexObject(_zdo, 2);

						if(_zdo->xref_exists(DEPEND_END_2)) xdo_end = _zdo->get_xref(DEPEND_END_2);
						else if(!_xdo.is_nil())
						{
							if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
							if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
						}
					}
					if (XYtoggle_in_fill_x != DataTab->_XYtoggle->isChecked() )
					{
						XYtoggle_in_fill_x = DataTab->_XYtoggle->isChecked();
						_x_start_obj_id = 0; // force recalc as XY toggle changed
					}
				}
				else
				{
					if(DataTab->_XYtoggle->isChecked() )
					{
						// swap X and Y is on
						if(_zdo->xref_exists(DEPEND_1)) _xdo = _zdo->get_xref(DEPEND_1);
						else  _xdo = makeIndexObject(_zdo, 1);

						if(_zdo->xref_exists(DEPEND_END_1)) xdo_end = _zdo->get_xref(DEPEND_END_1);
						else if(!_xdo.is_nil())
						{
							if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
							if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
						}
					}
					else
					{
						// use dep 0
						if(_zdo->xref_exists(DEPEND_0))
							_xdo = _zdo->get_xref(DEPEND_0);
						else
						{
							_xdo = makeIndexObject(_zdo, 0);
						}
					}
					if(_zdo->xref_exists(DEPEND_END_0))
						xdo_end = _zdo->get_xref(DEPEND_END_0);
					else if(!_xdo.is_nil())
					{
						if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
						if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
					}
				}
			} // end else on X slot obj set
		} // end else on use T as Sec toggle
	} // end if-else on spectrum type

	// Apply Time Interval Mask if set
	QdObject_var TImaskObj = DataTab->_mask_data_fld->getSlotObject();
	if( !TImaskObj.is_nil()) {
		QdTimeInterval_var TImask = QdTimeInterval_var::narrow(TImaskObj);
		if( !TImask.is_nil()) {
			_xdo = QuGetSubset(_xdo, TImask);
			if(!xdo_end.is_nil()) xdo_end = QuGetSubset(xdo_end, TImask);
			if(!xdo_delta_plus.is_nil() && xdo_delta_plus->sequence_size() > 1) xdo_delta_plus = QuGetSubset(xdo_delta_plus, TImask);
			if(!xdo_delta_minus.is_nil() && xdo_delta_minus->sequence_size() > 1) xdo_delta_minus = QuGetSubset(xdo_delta_minus, TImask);
		}
	}

	if(_xdo.is_nil()) return 0;

	_x_start_obj_id = _xdo->get_id();
	if( !xdo_end.is_nil() ) _x_end_obj_id = xdo_end->get_id();

	int nx_data;
	
	if( !xdo_delta_plus.is_nil() || !xdo_delta_minus.is_nil())
		nx_data = getXData(_xdo, xdo_delta_plus, xdo_delta_minus);
	else if( !xdo_end.is_nil() )
		nx_data = getXData(_xdo, xdo_end);	
	else
		nx_data = getXData(_xdo);

	return nx_data;
  
}

void SpectrumItem::refreshX()
{
	_x_start_obj_id=0;
	_x_end_obj_id=0;
	//	get_dobjs();
	_nx_data = fill_X_Data();
}

// -----------------------------------------------------

int SpectrumItem::fill_Y_Data()
{
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::fill_Y_Data" << endl;
	
	if ( _zdo.is_nil() ) return -1; // cannot guarantee to create xdo so don't try till zdo exists
  	
	QdObject_var  ydo = DataTab->_y_start_data_fld->getSlotObject();
	QdObject_var ydo_end;
	QdObject_var ydo_delta_minus;
	QdObject_var ydo_delta_plus;
    
	if(!ydo.is_nil())
	{
	//	ydo_end = DataTab->_y_end_data_fld->getSlotObject();

	}
	else
	{
		// use metadata off 
		if( DataTab->_XYtoggle->isChecked() )
		{
			if(_zdo->sequence_size() != 1)
			{
				// Depend 0 is being used, so Y is Dep 1
				if(_zdo->xref_exists(DEPEND_1))
					ydo = _zdo->get_xref(DEPEND_1);
				else
					ydo = makeIndexObject(_zdo, 1);
			
				if(_zdo->xref_exists(DEPEND_END_1))
					ydo_end = _zdo->get_xref(DEPEND_END_1);

			}
			else
			{
				// no Y object but swap XY is on
				if(_zdo->xref_exists(DEPEND_2))
					ydo = _zdo->get_xref(DEPEND_2);
				else
				{
					ydo = makeIndexObject(_zdo, 2);
				}
		
				if(_zdo->xref_exists(DEPEND_END_2))
					ydo_end = _zdo->get_xref(DEPEND_END_2);
			}
		}
		else
		{
			if(_zdo->xref_exists(DEPEND_1))
			{
				ydo = _zdo->get_xref(DEPEND_1);
			}
			
			if(ydo.is_nil())
				ydo = makeIndexObject(_zdo, 1);
			
			if(_zdo->xref_exists(DEPEND_END_1))
				ydo_end = _zdo->get_xref(DEPEND_END_1);
			
		}
	
		if (XYtoggle_in_fill_y != DataTab->_XYtoggle->isChecked() )
		{
			XYtoggle_in_fill_y = DataTab->_XYtoggle->isChecked();
			_y_start_obj_id = 0; // force update
		}

	}
  
	// ydo must exist by now as we make one if need be
	if(ydo->xref_exists(DELTA_PLUS))
		ydo_delta_plus = ydo->get_xref(DELTA_PLUS);
	if(ydo->xref_exists(DELTA_MINUS))
		ydo_delta_minus = ydo->get_xref(DELTA_MINUS);


	// Apply Time Interval Mask if set
	QdObject_var TImaskObj = DataTab->_mask_data_fld->getSlotObject();
	if( !TImaskObj.is_nil()) {
		QdTimeInterval_var TImask = QdTimeInterval_var::narrow(TImaskObj);
		if( !TImask.is_nil()) {
			ydo = QuGetSubset(ydo, TImask);
			if(!ydo_end.is_nil()) ydo_end = QuGetSubset(ydo_end, TImask);
			if(!ydo_delta_plus.is_nil() && ydo_delta_plus->sequence_size() > 1) ydo_delta_plus = QuGetSubset(ydo_delta_plus, TImask);
			if(!ydo_delta_minus.is_nil() && ydo_delta_minus->sequence_size() > 1) ydo_delta_minus = QuGetSubset(ydo_delta_minus, TImask);
		}
	}

	if(ydo.is_nil()) return -1;

	_y_start_obj_id = ydo->get_id();
	if(!ydo_end.is_nil()) _y_end_obj_id = ydo_end->get_id();
		
	// fill the Y axis bins for each x bin (inhomogeneous arrays OK)
	int ny_data;
	if( !ydo_end.is_nil() )
	{
		ny_data = getYData( ydo, ydo_end);
	}
	else if( !ydo_delta_plus.is_nil() || !ydo_delta_minus.is_nil())
	{
		ny_data = getYData(ydo, ydo_delta_plus, ydo_delta_minus);
	}
	else
	{
		ny_data = getYData(ydo );
	}
	// NOTE: getYData() returns -1 for error, last array dim for sequences, array dim for matrix

	_ydo = ydo;
	
	return ny_data;

}

// -------------------------------------

int SpectrumItem::fill_Z_Data( int XtoY, bool inPlot)
{
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::fill_Z_Data" << endl;

	if(_nx_data != _y_start_all.size())
	{
		if ( QuSlotDebug() ) cout << "y dimension incorrect: check object has DEPEND_1 (and DEPEND_2 if XY spectrum)" << endl;
		return -1;
	}
	
	_z_data_all.clear();
	_z_data_all.resize(_nx_data);

	QdRMatrixSeq_var rmatrixseq_z = QdRMatrixSeq_var::narrow(_zdo);
	QdRMatrix_var    rmatrix_z    = QdRMatrix_var::narrow(_zdo);
  
	if( !rmatrixseq_z.is_nil() )
	{
   	 
		if(rmatrixseq_z->elt(0).rank() == 1)
		{
			// Try 1D Mat sequence
			for(int nx=0; nx<_nx_data; ++nx){
				int ny = _y_start_all[nx].size();
				_z_data_all[nx].resize(ny);

				_z_data_all[nx]=rmatrixseq_z->elt(nx);
			}
		}
		else if(rmatrixseq_z->elt(0).rank() == 2)
		{
			// Try 2D Mat sequence of 1 record 
			if (rmatrixseq_z->sequence_size() != 1) {
                if(inPlot)
				    plmtex("lv",-2.0,0.5,0.0, "Z data is a 2D matrix sequence with more than one record");
                else
                    std::cout << "Z data is a 2D matrix sequence with more than one record" << std::endl;
				return -1;
			}
	  
			double value_j;
			for(int nx=0; nx< _nx_data; ++nx){
				int ny = _y_start_all[nx].size();
				_z_data_all[nx].resize(ny);
				for(int j=0; j< ny; ++j)
				{
					if (XtoY == 1) value_j = (rmatrixseq_z->elt(0))[j][nx];
					else           value_j =  (rmatrixseq_z->elt(0))[nx][j];

					_z_data_all[nx][j] = value_j;
				}
			}
		}
		else
		{
            if(inPlot)
			    plmtex("lv",-2.0,0.5,0.0, "Z data is a matrix sequence, but not 1D or 2D with one record");
            else
                std::cout << "Z data is a matrix sequence, but not 1D or 2D with one record" << std::endl;
			return -1;
		}

	}
	else if( !rmatrix_z.is_nil() )
	{
		if(rmatrix_z->rank() == 2)
		{
			double value_j;
			for(int nx=0; nx< _nx_data; ++nx){
				int ny = _y_start_all[nx].size();
				_z_data_all[nx].resize(ny);
				for(int j=0; j< ny; ++j)
				{
					if (XtoY == 1) value_j =  (*rmatrix_z)[j][nx];
					else      value_j =  (*rmatrix_z)[nx][j];
			 
					_z_data_all[nx][j] = value_j;
				}
			}
		}
		else
		{
            if(inPlot)
			    plmtex("lv",-2.0,0.5,0.0, "Z data is a matrix, but not 2D or 1D sequence");
            else
                std::cout << "Z data is a matrix, but not 2D or 1D sequence" << std::endl;
			return -1;
		}
  
	}
	else
	{
        if(inPlot)
		    plmtex("lv",-2.0,0.5,0.0, "Z data is not a matrix or matrix sequence");
        else
            std::cout << "Z data is not a matrix or matrix sequence" << std::endl;
		return -1;
	}

	return 0;
}
// ----------------------------------

void SpectrumItem::setDefaultOptions()
{
	DataTab->timeAsSec_toggle->setChecked(false);
	DataTab->_XYtoggle->setChecked(false);
	DataTab->gap_toggle->setChecked(false);
	DataTab->interlace_toggle->setChecked(false);
	
	
	ColourscaleTab->_colourscale_min_fld->setText("");
	ColourscaleTab->_colourscale_max_fld->setText("");
	ColourscaleTab->_colourscale_auto_fld->setCurrentIndex(QP_FROM_META);
	ColourscaleTab->_colourscale_linlog_fld->setCurrentIndex(QP_LINEAR);
	ColourscaleTab->_colourscale_min_fld->setText("");
	ColourscaleTab->_colourscale_show_colour_bar_fld->setChecked(true);
	ColourscaleTab->_colourscale_colour_bar_width_fld->setValue(QPUI_COLOUR_BAR_WIDTH);
	ColourscaleTab->_colourscale_colour_bar_height_fld->setValue(QPUI_COLOUR_BAR_HEIGHT);
	ColourscaleTab->_colourscale_colour_bar_voffset_fld->setValue(QPUI_COLOUR_BAR_VOFFSET);
	ColourscaleTab->_colourscale_colour_bar_disp_fld->setValue(QPUI_COLOUR_BAR_DISP);
// 	ColourscaleTab->_colourscale_table_fld->setCurrentItem(QPUI_RAINBOW); //TODO
	ColourscaleTab->_colourscale_label_fld->setText("");
	ColourscaleTab->_colourscale_autolabel_fld->setChecked(true);
	
	updateAxes();
}



// --------------------------------------

void SpectrumItem::updateAxes()
{
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::updateAxes" << endl;
	
	frame()->updateAxes();
			
	if(ColourscaleTab->_colourscale_auto_fld->currentIndex() != QP_FROM_USER) ColourscaleTab->setColourscaleMinMax();

	if(ColourscaleTab->_colourscale_autolabel_fld->isChecked())  ColourscaleTab->setColourScaleAutoLabel();


}

void SpectrumItem::addCustomColourTableTo(ColourTablePalette& palette)
{
    ColourTable current=ColourscaleTab->_colourtable_fld->getCurrentTable();
    if(!stdPalette.contains(current))
    {
        if(!palette.contains(current))
        {
            palette.push_back(current);
        }
    }
}

// --------------------------------------

QpuiReturnStatus SpectrumItem::plot()
{
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::plot" << endl;
	
		// set the colour table
// 	page()->set_colour_table((QpuiColourTable)ColourscaleTab->_colourscale_table_fld->currentItem());
    ColourscaleTab->_colourtable_fld->getCurrentTable().apply();
	
	// plots the SpectroGram Object TS or XY
	
	double nominal_spacing = DBL_MAX;
	
	if ( _zdo.is_nil() ){
		plmtex("lv",-2.0,0.5,0.0,"No Z data object");
		return QPUI_FAILURE;
	}
	
	if(!DataTab->_z_data_fld->isValid())
	{
		plmtex("lv",-10.0,0.5,0.0,"Trace Plot ERROR: Invalid Z Slot");
		return QPUI_FAILURE;
	}
		
	if ( _nx_data == 0 ){
		plmtex("lv",-2.0,0.5,0.0,"Bad X axis object");
		return QPUI_FAILURE;
	}
		
	if ( _ny_data == -1 ){
		plmtex("lv",-2.0,0.5,0.0,"Bad Y axis object(s)");
		return QPUI_FAILURE;
	}
	
	if( !DataTab->_y_start_data_fld->text().isEmpty()){
		// test Z and Y joined
		
		if( !QuAreJoined(_ydo, _zdo) ) {
			plmtex("lv",-2.0,0.5,0.0,"Z data and Y axis objects not joined");
			_ydo = QdObject_var(); // set to null
			return QPUI_FAILURE;
		}
	}
	
	// --------------------------------------------------
		// Z data object
	if(_nx_data != _zdo->sequence_size() && _zdo->sequence_size() != 1)
	{
		plmtex("lv",-2.0,0.5,0.0,"Z and X lengths differ");
		return QPUI_FAILURE;
	}
	int XtoY = isZConformal(_nx_data, _ny_data);
	if( XtoY == -1)  {
		plmtex("lv",-2.0,0.5,0.0, "Z 2D matrix dimensions don't match axes");
		return QPUI_FAILURE;
	}

	if( fill_Z_Data( XtoY) == -1)  return QPUI_FAILURE;
	
	// set up colour scale
	double zmin = ColourscaleTab->_colourscale_min_fld->value();
	double zmax = ColourscaleTab->_colourscale_max_fld->value();

	
	if(ColourscaleTab->_colourscale_linlog_fld->currentIndex() == QP_LOG)  {
		zmin = SafeLog10(zmin);
		zmax = SafeLog10(zmax);
	}

    if(zmin >= zmax)
    {
        plmtex("lv",-2.0,0.5,0.0, "Invalid Colourscale Range");
        return QPUI_FAILURE;
    }
   
		// find  T axis offset for plot
	
	double x_offset=0.0;
	double x_min, x_max;
	
	if(frameType() == QP_TS)
	{
		x_min = frame()->tAxisTab->_axis_time_fld->getStart().getEpoch2000();
		x_max = frame()->tAxisTab->_axis_time_fld->getEnd().getEpoch2000();
	}
	else
	{
		x_min = frame()->xAxisTab->_axis_xy_fld->getStart();
		x_max = frame()->xAxisTab->_axis_xy_fld->getEnd();
	}
	
	if ( frameType() == QP_TS )
	{
		x_offset = x_min; // this is Epoch2000 value of range start in seconds
	}
	
	double zrange = zmax - zmin;
	PLFLT x[4], y[4];
	PLFLT minCellX = fabs( DataTab->minCellWidth->value() * (x_max - x_min)) ;
	
	// plot data 
	int plot_id=page()->PageTab->_xwin_num_fld->text().toInt();
	
	page()->setSmooth(plot_id, false);

	double y_min = panel()->yAxisTab->_axis_xy_fld->getStart();
	double y_max = panel()->yAxisTab->_axis_xy_fld->getEnd();

	double dx1, dx2, dy1, dy2, zval;
	bool isXLog=(frame()->xAxisTab->_axis_linlog_fld->currentIndex() == QP_LOG);
	bool isYLog=(panel()->yAxisTab->_axis_linlog_fld->currentIndex() == QP_LOG);
	bool isZLog=(ColourscaleTab->_colourscale_linlog_fld->currentIndex() == QP_LOG);
	
	for(int nx=0; nx<_x_start_all.size(); ++nx)
	{
		dx1 =  _x_start_all[nx] ;
		dx2 =  _x_end_all[nx];
		
		// skip if both bin edges are out of range
		if((dx1 < x_min || dx1 > x_max) && ((dx2 < x_min || dx2 > x_max))) continue;
		
		if(isXLog)
		{
			x[0] = (PLFLT) SafeLog10(dx1 - x_offset);
			x[2] = (PLFLT) SafeLog10(dx2 - x_offset);
		}
		else
		{
			x[0] = (PLFLT) (dx1 - x_offset);
			x[2] = (PLFLT) (dx2 - x_offset);
		}
		if(minCellX > 0.0){

			if( x[2] - x[0] < minCellX ) x[2] = x[0] + minCellX;
		}
		
		x[1] = x[0];
		x[3] = x[2];
		
		for(int j=0; j<_y_start_all[nx].size(); ++j)
		{
			dy1 = (_y_start_all[nx])[j];
			dy2 = (_y_end_all[nx])[j];
		
					// skip if out of range
			if((dy1 < y_min || dy1 > y_max) && (dy2 < y_min || dy2 > y_max)) continue;
					
			if(isYLog)
			{
				y[0] = SafeLog10(dy1);
				y[1] = SafeLog10(dy2);
			}
			else
			{
				y[0] = (PLFLT) dy1;
				y[1] = (PLFLT) dy2;
			}
			y[2] = y[1];
			y[3] = y[0];
		
			zval=(_z_data_all[nx])[j];
			if( zval != missing_value && isfinite(zval)) // ALBAN: changed for nothing plotted if data gap
			{
				PLFLT col;
				if(isZLog)
					col=(SafeLog10(zval) - zmin)/zrange;
				else
					col=(zval-zmin)/zrange;
					
				if(col <0.) col=0.;
				if(col>=0.999) col=0.999;
				plcol1(col);

				plfill(4, x, y);
			}
	
		}
	}

	plcol0(1);

	// set colour bar
	if(ColourscaleTab->_colourscale_show_colour_bar_fld->isChecked())
	{
		PLFLT x1win_save, x2win_save, y1win_save, y2win_save;
		PLFLT x1vp_save, x2vp_save, y1vp_save, y2vp_save;
	
		plgvpw(&x1win_save, &x2win_save, &y1win_save, &y2win_save);
		plgvpd(&x1vp_save, &x2vp_save, &y1vp_save, &y2vp_save);
	
		PLFLT chsize_ndc, dummy;
		
		chsize_ndc=getCharHeight();
	
		x[0] = x2vp_save + ColourscaleTab->_colourscale_colour_bar_disp_fld->value() * chsize_ndc*1.5;
		x[2] = x[0] +  ColourscaleTab->_colourscale_colour_bar_width_fld->value() * chsize_ndc;
		float ht = (y2vp_save - y1vp_save) ;
		y[0] = y1vp_save + ht * ColourscaleTab->_colourscale_colour_bar_voffset_fld->value();
		y[1] = y[0] + ht * ColourscaleTab->_colourscale_colour_bar_height_fld->value();
	
		plvpor(x[0],x[2], y[0], y[1]);
		
		plwind(0., 1., zmin, zmax);
	
		x[0]=0.;
		x[1]=0.;
		x[2]=1.;
		x[3]=1.;

		PLFLT zstep=(zmax-zmin)/200.;
		PLFLT i=0.;
		for(PLFLT col=zmin;col<zmax;col+=zstep, i+=1./200.)
		{
			y[0] = col;
			y[3] = col;
			y[1] = col+zstep;
			y[2] = y[1];
			if(i>=1.) i=0.999;
			plcol1(i);
			plfill(4, x, y);
		}
	
		plcol0(1);
		int plot_id=page()->PageTab->_xwin_num_fld->text().toInt();
		page()->setSmooth(plot_id, true);
		string yopt = "bcmtsv";
		if(ColourscaleTab->_colourscale_linlog_fld->currentIndex() == QP_LOG) yopt += "l";
		plschr(0., frame()->DisplayTab->_ticks_char_size_fld->value());
		
		plwid(QtWidth2PlWidth(frame()->DisplayTab->_line_width_fld->value()));

		PLFLT _ticks_length = ColourscaleTab->_colourscale_ticklength_fld->value();
		
		plsmaj(0., _ticks_length);
		plsmin(0., _ticks_length*0.6);

        PLLabelDefaults y_label_opt;
        y_label_opt.exp_label_disp = 2.0 + (PLFLT) ColourscaleTab->_exp_label_offset_y_fld->value();
        y_label_opt.exp_label_pos = 1.02 +(PLFLT) ColourscaleTab->_exp_label_offset_x_fld->value();
        y_label_opt.exp_label_just = 0.0;
        plslabelfunc(NULL, &y_label_opt);
		plbox("bc", 0.0, 0, yopt.c_str(), 0.0, 0);
        plslabelfunc(NULL, NULL);
        
		string label="";
	
		plwid(0);
		
		label = ColourscaleTab->_colourscale_label_fld->text().toLatin1().data();
		
		plmtex("l", 1., 0.5, 0.5, SafePGPString(label).c_str());
		plvpor(x1vp_save, x2vp_save, y1vp_save, y2vp_save);
		plwind(x1win_save, x2win_save, y1win_save, y2win_save);
		page()->setSmooth(plot_id, false);
	}

	return QPUI_SUCCESS;
}



//     ----------------------

QDomElement SpectrumItem::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("Plot_Item");
    xml.setAttribute("Type", "Spectrogram");
    xml.appendChild(createTextXMLNode(doc, "Name", getName()));
    xml.appendChild(DataTab->toXMLElement(doc, QP_SPECTROGRAM));
    QString linkText="";
    if(!getLinkText().isEmpty())
    {
        linkText=getLinkText()+"S";
    }
    xml.appendChild(createTextXMLNode(doc, "Link_Text", linkText));

    xml.appendChild(ColourscaleTab->toXMLElement(doc));

    xml.appendChild(createTextXMLNode(doc, "Missing_Value", missing_value));

    return xml;
}

void SpectrumItem::fromXMLElement(QDomElement& el)
{
    QString s;
    
    if(getXMLNodeText(el, "Name", s))
    {
        setName(s);
    }

    QDomElement dataElement=el.firstChildElement("Data_Tab");
    if(!dataElement.isNull())
    {
        DataTab->fromXMLElement(dataElement, I_Type);
		if(frameType()== QP_XY)
		{
			if(getXMLNodeText(el, "X_Start_Data", s))
            {
				DataTab->_x_start_data_fld->setText(s);
			}
			if(getXMLNodeText(el, "X_End_Data", s))
            {
	//			DataTab->_x_end_data_fld->setText(s);
			}
			if(getXMLNodeText(el, "Time_As_Sec", s))
            {
				DataTab->timeAsSec_toggle->setChecked(s.toInt());
			}

			DataTab->_x_start_data_fld->reValidate();
//			DataTab->_x_end_data_fld->reValidate();
		}
	}

    
    QDomElement colourscaleElement=el.firstChildElement("Colourscale_Tab");
    if(!colourscaleElement.isNull())
    {
        ColourscaleTab->fromXMLElement(colourscaleElement);
    }

	if(getXMLNodeText(el, "Link_Text", s))
	{
		setLinkText(s);
	}
	
    if(getXMLNodeText(el, "Missing_Value", s))
    {
        missing_value=(bool)s.toDouble();
    }
	
	QdObject_var o=QuGetObjByName(_name);
	if(!o.is_nil()) WLcontext=_name;
	
//	DataTab->_y_start_data_fld->reValidate();
//	DataTab->_z_data_fld->reValidate(true);
}

Vistream& operator>>(Vistream& v_is, SpectrumItem& spectrogram)
{
  // streams from virtual input stream v_is into spectrogram object

	PlotItem& plot_item = spectrogram;
	v_is >> plot_item;
	
	QdObject_var o=QuGetObjByName(spectrogram._name);
	if(!o.is_nil()) spectrogram.WLcontext=spectrogram._name;
	
	bool save_x = false;
	int type;
	v_is >> type;
	spectrogram.I_Type = (QpItemType) type;

	string z_data_str;
	v_is >> z_data_str;
	spectrogram.DataTab->_z_data_fld->setText(z_data_str.c_str());
	spectrogram.DataTab->old_BaseName = baseObjectText(z_data_str.c_str());
	spectrogram.DataTab->_z_data_fld->reValidate(false);


	string y_str;
	v_is >> y_str;
	spectrogram.DataTab->_y_start_data_fld->setText(y_str.c_str());
	v_is >> y_str;
	
	bool _fill_gaps;
	v_is >> _fill_gaps;
	spectrogram.DataTab->gap_toggle->setChecked(_fill_gaps);
	
	if(qsas_restore_release > 20402){
		bool _interlace_spectra;
		v_is >> _interlace_spectra;
		spectrogram.DataTab->interlace_toggle->setChecked(_interlace_spectra);
	}
  
	if(spectrogram.frameType() == QP_XY)
	{
		// only restore xy data if an xy spectrogram
		save_x = true;
		string x_str;
		v_is >> x_str;
		spectrogram.DataTab->_x_start_data_fld->setText(x_str.c_str());
		v_is >> x_str;
	//	spectrogram.DataTab->_x_end_data_fld->setText(x_str.c_str());

		bool TasSecToggle;;
		v_is >> TasSecToggle;
		spectrogram.DataTab->timeAsSec_toggle->setChecked(TasSecToggle);
	}

	string linkText;
	v_is >> linkText;
// 	spectrogram.setText(LINK_COL, linkText.c_str()); //TODO
	// do the relinking in listview restore after traces etc are there

	QpColourscaleTab* ct = spectrogram.ColourscaleTab;
	int cs_auto;
	v_is >> cs_auto;
	ct->_colourscale_auto_fld->setCurrentIndex(cs_auto);
	double _min;
	v_is >> _min;
	ct->_colourscale_min_fld->setDouble(_min);
	double _max;
	v_is >> _max;
	ct->_colourscale_max_fld->setDouble(_max);
	v_is >> spectrogram.missing_value;
	bool _linlog;
	v_is >> _linlog;
	ct->_colourscale_linlog_fld->setCurrentIndex(_linlog);
    if(qsas_restore_release < 20400)
    {
        int _colour_table;
        v_is >> _colour_table;
        if(_colour_table==QPUI_USER_TABLE)
        {
            ct->_colourtable_fld->setProfilePalette(0); // First item in profile palette - same behaviour as in 2.3.x
        }
        else
        {
            ct->_colourtable_fld->setStdPalette(_colour_table);
        }
    }
    else
    {
        double _exp_label_x;
        v_is >> _exp_label_x;
        ct->_exp_label_offset_x_fld->setValue(_exp_label_x);
        double _exp_label_y;
        v_is >> _exp_label_y;
        ct->_exp_label_offset_y_fld->setValue(_exp_label_y);
        
        int _colour_table_type;
        v_is >> _colour_table_type;
        int _colour_table;
        v_is >> _colour_table;
        if(_colour_table_type!=0)
        {
            ct->_colourtable_fld->setSessionPalette(_colour_table+importedPaletteOffset);
        }
        else
        {
            ct->_colourtable_fld->setStdPalette(_colour_table);
        }
    }
	string _label;
	v_is >> _label;
	ct->_colourscale_label_fld->setText(_label.c_str());
	bool _show_colour_bar;
	v_is >> _show_colour_bar;
	ct->_colourscale_show_colour_bar_fld->setChecked(_show_colour_bar);
	float _colour_bar_width;
	v_is >> _colour_bar_width;
	ct->_colourscale_colour_bar_width_fld->setValue(_colour_bar_width);
	float _colour_bar_disp;
	v_is >> _colour_bar_disp;
	ct->_colourscale_colour_bar_disp_fld->setValue(_colour_bar_disp);
	bool _autolabel;
	v_is >> _autolabel;
	ct->_colourscale_autolabel_fld->setChecked(_autolabel);
  
	float _colour_bar_height;
	v_is >> _colour_bar_height;
	ct->_colourscale_colour_bar_height_fld->setValue(_colour_bar_height);
	float _colour_bar_voffset;
	v_is >> _colour_bar_voffset;
	ct->_colourscale_colour_bar_voffset_fld->setValue(_colour_bar_voffset);

	if( qsas_restore_release > 20306)
	{
		float tickLength;
		v_is >> tickLength;
		ct->_colourscale_ticklength_fld->setValue(tickLength);
	}
	
//	spectrogram.DataTab->_z_data_fld->reValidate(true);
	
	return v_is;
}

// -----------------------------------------

Vostream& operator<<(Vostream& v_os, SpectrumItem& spectrogram)
{
	// streams spectrogram object to virtual output stream v_os

	PlotItem& plot_item = spectrogram;
	v_os=save(v_os ,plot_item);
	
	v_os << ListTypeEntry;
	v_os << spectrogram.I_Type;

	v_os << spectrogram.I_Type;
	v_os << string(spectrogram.DataTab->_z_data_fld->text().toLatin1().data());
	v_os << string(spectrogram.DataTab->_y_start_data_fld->text().toLatin1().data());
//	v_os << string(spectrogram.DataTab->_y_end_data_fld->text().toLatin1().data());
	v_os << " "; // y end data not used
	bool _fill_gaps = spectrogram.DataTab->gap_toggle->isChecked();
	v_os << _fill_gaps;

	// new at 2.4.03
	bool _interlace_spectra = spectrogram.DataTab->interlace_toggle->isChecked();;
	v_os << _interlace_spectra;
	

	bool save_x=false;
	if(spectrogram.frameType() == QP_XY)
	{
		// only save xy data if an XY SpectroGram
		save_x=true;

		v_os << string(spectrogram.DataTab->_x_start_data_fld->text().toLatin1().data());
		v_os << " "; // x end data not used
//		v_os << string(spectrogram.DataTab->_x_end_data_fld->text().toLatin1().data());
		bool TasSecToggle = spectrogram.DataTab->timeAsSec_toggle->isChecked();
		v_os << TasSecToggle;
		
	}

	string linkText;
	if(!spectrogram.getLinkText().isEmpty())
	{
		linkText = string(spectrogram.getLinkText().toLatin1().data());
		linkText += "S"; // temporary to avoid conflict with existing links on restore
	}
	else linkText="";
	v_os << linkText;
	

	QpColourscaleTab* ct = spectrogram.ColourscaleTab;
	int cs_auto = ct->_colourscale_auto_fld->currentIndex();
	v_os << cs_auto;
	double _min = ct->_colourscale_min_fld->value();
	v_os << _min;
	double _max = ct->_colourscale_max_fld->value();
	v_os << _max;
	v_os << spectrogram.missing_value;
	bool _linlog = ct->_colourscale_linlog_fld->currentIndex();
	v_os << _linlog;

    double _exp_label_x=ct->_exp_label_offset_x_fld->value();
    v_os << _exp_label_x;
    double _exp_label_y=ct->_exp_label_offset_y_fld->value();
    v_os << _exp_label_y;

//     int _colour_table = ct->_colourscale_table_fld->currentItem();
    ColourTable colourTable=ct->_colourtable_fld->getCurrentTable();

    int _colour_table_type;
    int _colour_table_id;
    if(stdPalette.contains(colourTable))
    {
        _colour_table_type=0;
        _colour_table_id=stdPalette.indexOf(colourTable);
    }
    else
    {
        _colour_table_type=1;
        _colour_table_id=savedPalette.indexOf(colourTable);
    }
	v_os << _colour_table_type;
    v_os << _colour_table_id;
	string _label = ct->_colourscale_label_fld->text().toLatin1().data();
	v_os << _label;
	bool _show_colour_bar = ct->_colourscale_show_colour_bar_fld->isChecked();
	v_os << _show_colour_bar;
	float _colour_bar_width = ct->_colourscale_colour_bar_width_fld->value();
	v_os << _colour_bar_width;
	float _colour_bar_disp = ct->_colourscale_colour_bar_disp_fld->value();
	v_os << _colour_bar_disp;
	bool _autolabel = ct->_colourscale_autolabel_fld->isChecked();
	v_os << _autolabel;
	float _colour_bar_height = ct->_colourscale_colour_bar_height_fld->value();
	v_os << _colour_bar_height;
	float _colour_bar_voffset = ct->_colourscale_colour_bar_voffset_fld->value();
	v_os << _colour_bar_voffset;
	float _colour_bar_ticklength = ct->_colourscale_ticklength_fld->value();
	v_os << _colour_bar_ticklength;
	
	return v_os;
}

QdObject_var makeIndexObject(QdObject_var zdo, int indxFlag){
	QdObject_var retObj;
	QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(zdo);
	QdRMatrix_var mm    = QdRMatrix_var::narrow(zdo);
	int indx = indxFlag-1;
	int len=0;
  
	if (indxFlag == 0 && !ms.is_nil() )
	{
     // use sequence length
		len = ms->sequence_size();
	}
	else if( !ms.is_nil() )
	{
        // use index
		if(ms->elt(0).rank() > indx)
		{
			len = ms->elt(0).dimdata()[indx];
		}
		else
		{
			len = ms->sequence_size();
		}
  
	}
	else if( !mm.is_nil() )
	{
        // use index
		if(mm->rank() > indx)
		{
			len = mm->dimdata()[indx];
		}
  
	}
  
  // create a matrix to return
	if (len > 0)
	{
        // create matrix whose values are the index 
		QdRMatrix_var dimIdxObj = new QdRMatrix(Dim(len));
		for (int i=0; i<len; i++) (*dimIdxObj)[i] = i+1;
		retObj = dimIdxObj;
	}
	else
	{
    // This should never be reached, but create something in case
		cout << "CANNOT create axis object " << indxFlag << " (t-axis=0; x-axis=1, yaxis=2)"  << endl;
		QdRMatrix_var dimIdxObj = new QdRMatrix(Dim(1));
		(*dimIdxObj)[0] = 0;
		retObj = dimIdxObj;
	}
  
	return retObj;
}
 
 
