//
//  Factory.cc
//
//  Factory class for cloning objects
//
//  David Burgess
//  March 2000
//

#include <vector>
#include "Factory.h"
#include "Matrix.cc"

namespace QDOS {

// String

QdString_var Factory::clone( const string& s )
{ return new QdString(s); }

QdString_var Factory::clone( QdString_var qs_var )
{ return new QdString( *qs_var ); }

// StringSeq

QdStringSeq_var Factory::clone( const vector<string>& s )
{ return new QdStringSeq( s ); }
QdStringSeq_var Factory::clone( const QdStringSeq& s )
{ return new QdStringSeq( s ); }
QdStringSeq_var Factory::clone( QdStringSeq_var qs_var )
{ return new QdStringSeq( *qs_var ); }

// Time

QdTime_var Factory::clone( const QmjdTime& t )
{ return new QdTime( t ); }
QdTime_var Factory::clone( QdTime_var t_var )
{ return new QdTime( *t_var ); }

// TimeSeq

QdTimeSeq_var Factory::clone( const QmjdTimeSeq& tseq )
{ return new QdTimeSeq( tseq ); }
QdTimeSeq_var Factory::clone( QdTimeSeq_var tseq_var )
{ return new QdTimeSeq( *tseq_var ); }

// TimeInterval

QdTimeInterval_var Factory::clone( const QmjdTimeInterval& tintvl )
{ return new QdTimeInterval( tintvl ); }
QdTimeInterval_var Factory::clone( QdTimeInterval_var tintvl_var )
{ return new QdTimeInterval( *tintvl_var ); }

// TimeIntervalSeq

QdTimeIntervalSeq_var Factory::clone( const QmjdTimeIntervalSeq& tintvl )
{ return new QdTimeIntervalSeq( tintvl ); }
QdTimeIntervalSeq_var Factory::clone( QdTimeIntervalSeq_var tintvl_var )
{ return new QdTimeIntervalSeq( *tintvl_var ); }


// RScalar

QdRScalar_var Factory::clone( double x )
{ return new QdRScalar( x ); }

QdRScalar_var Factory::clone( const QdRScalar& x )
{ return new QdRScalar( x ); }

QdRScalar_var Factory::clone( QdRScalar_var x_var )
{ return new QdRScalar( *x_var ); }

// RScalarSeq

QdRScalarSeq_var Factory::clone( const vector<double>& v )
{ QdRScalarSeq_var r = new QdRScalarSeq;
 r->assign( v ); return r;}
 
QdRScalarSeq_var Factory::clone( const vector<float>& v )
{ QdRScalarSeq_var r = new QdRScalarSeq;
 r->assign( v ); return r;}

QdRScalarSeq_var Factory::clone( const vector<int>& v )
{ QdRScalarSeq_var r = new QdRScalarSeq;
 r->assign( v ); return r;}
 
QdRScalarSeq_var Factory::clone( const RScalarSeq& rss )
{ return new QdRScalarSeq( rss ); }

QdRScalarSeq_var Factory::clone( QdRScalarSeq_var rss_var )
{ return new QdRScalarSeq( *rss_var ); }

// RMatrix

QdRMatrix_var Factory::clone( const RMatrix& m )
{ return new QdRMatrix( m ); }

QdRMatrix_var Factory::clone( const QdRMatrix_var& m_var )
{ return new QdRMatrix( *m_var ); }

// SMatrix

QdSMatrix_var Factory::clone( const SMatrix& sm )
{ return new QdSMatrix(sm ); }

QdSMatrix_var Factory::clone (const QdSMatrix_var & sm_var)
{ return new QdSMatrix( *sm_var ); }

// RMatrixSeq

QdRMatrixSeq_var Factory::clone( const RMatrixSeq& mseq )
{ return new QdRMatrixSeq( mseq ); }

QdRMatrixSeq_var Factory::clone( const QdRMatrixSeq_var& mseq_var )
{ return new QdRMatrixSeq( *mseq_var ); }

// SMatrixSeq

QdSMatrixSeq_var Factory::clone( const SMatrixSeq& smseq )
{ return new QdSMatrixSeq( smseq ); }

QdSMatrixSeq_var Factory::clone( const QdSMatrixSeq_var& smseq_var )
{ return new QdSMatrixSeq( *smseq_var ); }



} // end namespace QDOS
