/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/


/* ensure cdf.h is included before declaractions dependent on CDF types */


#if !defined(___cdf_h___)
#include "cdf.h"
#endif
 
#ifndef _QRTN_H_  
#define _QRTN_H_

/* c++ compliance... */

  
/* include files */

#include <stddef.h> 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <time.h> 
#include <dirent.h>  
#include <math.h>
#include <ctype.h>
#include <sys/param.h>

#include "qdutil.h" 
#include "qprofile.h"
#include "Qdos.h" 
#include "DvObj.h" 

using namespace QSAS;

/* environment variables */

#define MISSING 0
#define EXISTS 1

/* frames known */

#define GSE 0   /* Geocentric Solar Ecliptic */
#define GSM 1   /* Geocentric Solar Magnetic */
#define GEOC 2  /* Geographic (geocentric) */
#define SM 3    /* Solar Magnetic */
#define GEI 4   /* Geocentric Inertial */
#define MAGD 5  /* Geomagnetic (dipole) */
#define HAE 6   /* Heliospheric Aries Ecliptic */
#define HEE 7   /* Heliospheric Earth Ecliptic */
#define HEEQ 8  /* Heliospheric Earth Equatorial */
#define GSEQ 9 /* Geocentric Solar Equatorial */
#define UNKNOWN -1 


/* messages */

#define QRTN_OK 0         /* must be 0 for use of |= in accumulating errors */
#define QRTN_WARNING -1
#define QRTN_ERROR 1
#define NOT_YET_IMPLEMENTED 2
#define UNKNOWN_FRAME 3
#define SAME_FRAME 4
#define BAD_MDT_FILE 6
#define BAD_EPOCH 7
#define BAD_AXIS 8
#define CANNOT_ROTATE 9
#define QRTN_INIT_ERROR 10
#define CANNOT_GET_XYZ 11
#define CANNOT_GET_MAG_MODEL 12

/* sizes */

#define N_YEAR_MODELS 25
#define MAX_N_SYS 20
#define SYS_NAME_LEN 36
#define N_ROTNS 6 /* max number of rotns for each system */


/* Structures */

class QrVector
{
    int verbose;

    /* used for MAH routine disassembly */
    
    double year_0;
    int MaxTerm;
    int Qrtn_skip;
    char last_err[240];    /* message area for last error */
    
    /* MAH common blocks */
    
    int n_sys;                        /* number of coord systems handled (=NSYS) */
    char sys_name[MAX_N_SYS][SYS_NAME_LEN+1];/* full names of systems (=TSYS) */
    int sys_to_gse[MAX_N_SYS][N_ROTNS+1]; /* rtns to gse for each system (=DEFS) */
    /* sys_to_gse[k-1][j-1] = DEFS[j][k] */
    /* static initialises to zero */
    
    // field dipole set in QrInitQRTN()
    // New model needs to be added every 5 years
    // Estimated model IGRF is replaced with DGRF
    // Predicted future model replaced with IGRF
    
    double model_year[N_YEAR_MODELS];
    double G01[N_YEAR_MODELS];
    double G11[N_YEAR_MODELS];
    double H11[N_YEAR_MODELS];
    double G01_datum, G11_datum, H11_datum;
    
    int   Lterms[N_ROTNS];                  /* (=LTERMS) sequence of rotations */
    int   m_max;                            /* (=MMAX) */
    double M_pole_long;                      /* (=MLONG) */
    double M_pole_colat;                     /* (=MCOLAT) */
    double rotations[12];                    /* (=ROT) standard rotations */
    int rot_axis[12]; /* array of twelve ints = 0,1 or 2 for x, y, z */
    /* axis appropriate to rotations */
    /* constants */
    
    double value_of_pi;
    double deg2rad;
    double rad2deg;
    
public:
    int IsPositionVector;
    int in_frame;
    int out_frame;
    int n_recs;
    QdRMatrixSeq_var ptr;
    QdTimeSeq_var t;

    QrVector(){
        Qrtn_skip = 0;
        sprintf(last_err,"");
        rot_axis[0]=2;
        rot_axis[1]=2;
        rot_axis[2]=0;
        rot_axis[3]=0;
        rot_axis[4]=1;
        rot_axis[5]=1;
        rot_axis[6]=2;
        rot_axis[7]=2;
        rot_axis[8]=0;
        rot_axis[9]=2;
        rot_axis[10]=1;
        rot_axis[11]=2;
    }
    
    QdRMatrixSeq_var QrGetRotnMatx();
    long QrDoRotation();
    char * QrErrorMessage(long err_flag);
    void QrSetFrameAttr(int frame_flag, QdObject_var dobj);
    void QrSetToFrameAttr(int frame_flag, QdObject_var dobj);
    int QrGetFrameFromAttr(QdObject_var dobj);
    int QrIsPositionVector( QdObject_var dobj);
    int QrOriginsDiffer(int in_frame, int out_frame);
    int QrMJD(int year, int month, int day);

private:
        
    long QrGetModel(double year);
    long QrUpdateModel(int MJDdate, double ut);
    long QrEpochToMJD_UT(double epoch, int * MJDdate, double * ut);
    long QrInitQRTN();
    long QrGetMagPole();
    long QrGetTerms();
    long QrBuildTran(double (*T)[3][3], int n_terms, int *list_rtns);
    long QrMultVecbyMatx(double (*XX)[3], double T[3][3]);
    long QrMultMtxByMtx(double (*T)[3][3], double S[3][3]);
    long QrMakeMatrix(double theta, int axis, double (*T)[3][3]);
    int  QrGetAxis(int i);
    void QrUnitMatrix(double (*T)[3][3] );
    void QrNullMatrix(double (*T)[3][3] );
    double QrDegToRadMod2pi(double angle);
    double QrGST(double JulianC, double ut);
    double QrSunLong(double JulianC, double ut);
    double QrObliquity(double JulianC);
    long QrGSM_to_GSE(double *psi, double *mu, double latitude, double longitude);
    double QrSunEqator(int MJDdate);
    long QrSunEarth(double *theta , double *phi);
    long QrGetRotAngles(int MJDdate, double ut);
    double QrGetAngle(int i);
    long QrMakeMatx(int in_frame, int out_frame, double (*T)[3][3]);
                     
};

class RtnVector
{
    
private:
    int verbose;

    /* used for MAH routine disassembly */
    
    double year_0;
    int MaxTerm;
    int Qrtn_skip;
    char last_err[240];    /* message area for last error */
    
    /* MAH common blocks */
    
    int n_sys;                        /* number of coord systems handled (=NSYS) */
    char sys_name[MAX_N_SYS][SYS_NAME_LEN+1];/* full names of systems (=TSYS) */
    int sys_to_gse[MAX_N_SYS][N_ROTNS+1]; /* rtns to gse for each system (=DEFS) */
    /* sys_to_gse[k-1][j-1] = DEFS[j][k] */
    /* static initialises to zero */
    
    // field dipole set in QrInitQRTN()
    // New model needs to be added every 5 years
    // Estimated model IGRF is replaced with DGRF
    // Predicted future model replaced with IGRF
    
    double model_year[N_YEAR_MODELS];
    double G01[N_YEAR_MODELS];
    double G11[N_YEAR_MODELS];
    double H11[N_YEAR_MODELS];
    double G01_datum, G11_datum, H11_datum;
    
    int   Lterms[N_ROTNS];                  /* (=LTERMS) sequence of rotations */
    int   m_max;                            /* (=MMAX) */
    double M_pole_long;                      /* (=MLONG) */
    double M_pole_colat;                     /* (=MCOLAT) */
    double rotations[12];                    /* (=ROT) standard rotations */
    int rot_axis[12];                         /* array of twelve ints = 0,1 or 2 for x, y, z */
    /* axis appropriate to rotations */
    /* constants */
    
    double value_of_pi;
    double deg2rad;
    double rad2deg;
    
public:
    int IsPositionVector;
    int in_frame;
    int out_frame;
    int n_recs;
    DvObject_var ptr;
    DvObject_var t;

    RtnVector(){
        Qrtn_skip = 0;
        sprintf(last_err,"");
        rot_axis[0]=2;
        rot_axis[1]=2;
        rot_axis[2]=0;
        rot_axis[3]=0;
        rot_axis[4]=1;
        rot_axis[5]=1;
        rot_axis[6]=2;
        rot_axis[7]=2;
        rot_axis[8]=0;
        rot_axis[9]=2;
        rot_axis[10]=1;
        rot_axis[11]=2;
    }

    DvObject_var QrGetRotnMatx();
    long QrDoRotation();
    char * QrErrorMessage(long err_flag);
    void QrSetFrameAttr(int frame_flag, DvObject_var &dobj);
    void QrSetToFrameAttr(int frame_flag, DvObject_var &dobj);
    int QrGetFrameFromAttr(DvObject_var &dobj);
    int QrIsPositionVector( DvObject_var &dobj);
    int QrOriginsDiffer(int in_frame, int out_frame);
    int QrMJD(int year, int month, int day);

private:
    long QrGetModel(double year);
    long QrUpdateModel(int MJDdate, double ut);
    long QrEpochToMJD_UT(double epoch, int * MJDdate, double * ut);
    long QrInitQRTN();
    long QrGetMagPole();
    long QrGetTerms();
    long QrBuildTran(double (*T)[3][3], int n_terms, int *list_rtns);
    long QrMultVecbyMatx(double (*XX)[3], double T[3][3]);
    long QrMultMtxByMtx(double (*T)[3][3], double S[3][3]);
    long QrMakeMatrix(double theta, int axis, double (*T)[3][3]);
    int  QrGetAxis(int i);
    void QrUnitMatrix(double (*T)[3][3] );
    void QrNullMatrix(double (*T)[3][3] );
    double QrDegToRadMod2pi(double angle);
    double QrGST(double JulianC, double ut);
    double QrSunLong(double JulianC, double ut);
    double QrObliquity(double JulianC);
    long QrGSM_to_GSE(double *psi, double *mu, double latitude, double longitude);
    double QrSunEqator(int MJDdate);
    long QrSunEarth(double *theta , double *phi);
    long QrGetRotAngles(int MJDdate, double ut);
    double QrGetAngle(int i);
    long QrMakeMatx(int in_frame, int out_frame, double (*T)[3][3]);

};


#endif
