#include "QtPLWidget.h"
#include "qprofile.h"

double PlWidth2QtWidth(double width)
{
	return ((double)width)/10.;
}

int QtWidth2PlWidth(double width)
{
	return (int)(width*10.);
}

QtPLWidget::QtPLWidget(int i_iWidth, int i_iHeight, double i_dSaveWidth, double i_dSaveHeight, int resolution, QWidget * i_wiParent):QWidget(i_wiParent)
{
	m_painterP=new QPainter;
	m_iPreviousWidth=this->width();
	m_iPreviousHeight=this->height();
	m_dSaveWidth=(i_dSaveWidth<10.?10.:i_dSaveWidth);
	m_dSaveHeight=(i_dSaveHeight<10.?10.:i_dSaveHeight);
	
	setNewSaveSize(m_dSaveWidth, m_dSaveHeight);
	cursorParameters.isTracking=false;
	m_pixPixmap=NULL;
	m_iOldSize=0;
	resize(i_iWidth, i_iHeight);
	
	lastColour.r=-1;
	
	currentMode=PLOT_MODE_RASTER;
	printRes = resolution;
}

QtPLWidget::~QtPLWidget()
{
	clearBuffer();
	if(m_pixPixmap!=NULL) delete m_pixPixmap;
}

void QtPLWidget::captureMouse(double * xi, double* yi, double * xf, double * yf, bool horizontalLine)
{
	setMouseTracking(true);
	cursorParameters.isTracking=true;
	cursorParameters.showHorizontalLine=horizontalLine;
	cursorParameters.cursor_start_x=
			cursorParameters.cursor_start_y=
			cursorParameters.cursor_end_x=
			cursorParameters.cursor_end_y=-1.;
	cursorParameters.step=1;

	do
	{
		QCoreApplication::processEvents(QEventLoop::AllEvents, 10);
	} while(cursorParameters.isTracking);
    
	*xi=cursorParameters.cursor_start_x;
	*yi=cursorParameters.cursor_start_y;
	*xf=cursorParameters.cursor_end_x;
	*yf=cursorParameters.cursor_end_y;
    
}

void QtPLWidget::getPlotParameters(double & io_dXFact, double & io_dYFact, double & io_dXOffset, double & io_dYOffset/*, double& io_dRatio*/, double i_dWidth, double i_dHeight)
{
	if(i_dWidth<=0.) i_dWidth=width();
	if(i_dHeight<=0.) i_dHeight=height();
	
	if(i_dWidth/i_dHeight>m_dAspectRatio) //Too wide, h is the limitating factor
	{
		io_dYFact=i_dHeight;
		io_dXFact=i_dHeight*m_dAspectRatio;
		io_dYOffset=0.;
		io_dXOffset=(i_dWidth-io_dXFact)/2.;
// 		io_dRatio=i_dHeight/m_dSaveHeight;
	}
	else
	{
		io_dXFact=i_dWidth;
		io_dYFact=i_dWidth/m_dAspectRatio;
		io_dXOffset=0.;
		io_dYOffset=(i_dHeight-io_dYFact)/2.;
// 		io_dRatio=i_dWidth/m_dSaveWidth;
	}
	
}

void QtPLWidget::mouseMoveEvent(QMouseEvent* event)
{
	this->activateWindow();
	this->raise();
	
	if(!cursorParameters.isTracking) return;
    
	double x_fact, y_fact, x_offset, y_offset; //Parameters to scale and center the plot on the widget
// 	double ratio;
    
	getPlotParameters(x_fact, y_fact, x_offset, y_offset/*, ratio*/);
    
	PLFLT X=(PLFLT)(event->x()-x_offset)/x_fact;
	PLFLT Y=(PLFLT)(event->y()-y_offset)/y_fact;
    
	if(cursorParameters.step==1)
	{
		cursorParameters.cursor_start_x=X;
		cursorParameters.cursor_start_y=Y;
	}
	else
	{
		cursorParameters.cursor_end_x=X;
		cursorParameters.cursor_end_y=Y;
	}

	update();
}

void QtPLWidget::mousePressEvent(QMouseEvent * event)
{
	if(!cursorParameters.isTracking) return;
    
	double x_fact, y_fact, x_offset, y_offset; //Parameters to scale and center the plot on the widget
// 	double ratio;
    
	getPlotParameters(x_fact, y_fact, x_offset, y_offset/*, ratio*/);
    
	PLFLT X=(PLFLT)(event->x()-x_offset)/x_fact;
	PLFLT Y=(PLFLT)(event->y()-y_offset)/y_fact;
    
	if(cursorParameters.step==1)
	{
		cursorParameters.cursor_start_x=X;
		cursorParameters.cursor_start_y=Y;
		cursorParameters.step=2;
		update();
	}
}

void QtPLWidget::resizeEvent(QResizeEvent *)
{
	m_bAwaitingRedraw=true;
	if(m_pixPixmap!=NULL)
	{
		delete m_pixPixmap;
		m_pixPixmap=NULL;
	}
	double x_fact, y_fact, x_offset, y_offset; //Parameters to scale and center the plot on the widget
	double ratio;
				
	getPlotParameters(x_fact, y_fact, x_offset, y_offset/*, ratio*/);
    if(y_offset<=0.)
    {
        ratio=((double)height())/m_dSaveHeight;
    }
    else
    {
        ratio=((double)width())/m_dSaveWidth;
    }
	emit(resized(ratio));
}



void QtPLWidget::mouseReleaseEvent(QMouseEvent * event)
{
	double x_fact, y_fact, x_offset, y_offset; //Parameters to scale and center the plot on the widget
	
	getPlotParameters(x_fact, y_fact, x_offset, y_offset/*, ratio*/);
	
	if(!cursorParameters.isTracking) // This is a click to adjust an X/Y interval
	{
		if(QSAS_profiles->current()->timeIntervalButton==0 || (QSAS_profiles->current()->timeIntervalButton==1 && event->button()==Qt::RightButton))
		{
			PLFLT X_=(PLFLT)(event->x()-x_offset)/x_fact;
			PLFLT Y_=(PLFLT)(event->y()-y_offset)/y_fact;
			emit clicked(X_, Y_, event->x(), event->y());
		}
		return;
	}
    
	PLFLT X=(PLFLT)(event->x()-x_offset)/x_fact;
	PLFLT Y=(PLFLT)(event->y()-y_offset)/y_fact;
    
	if(cursorParameters.step!=1)
	{
		cursorParameters.cursor_end_x=X;
		cursorParameters.cursor_end_y=Y;
		cursorParameters.isTracking=false;
		setMouseTracking(false);
		update();
	}

}
void QtPLWidget::mouseDoubleClickEvent(QMouseEvent * event)
{
	double x_fact, y_fact, x_offset, y_offset; //Parameters to scale and center the plot on the widget
	
	getPlotParameters(x_fact, y_fact, x_offset, y_offset/*, ratio*/);
	
	if(!cursorParameters.isTracking) // This is a double click to launch interval widget
	{
		if(QSAS_profiles->current()->timeIntervalButton==0 || (QSAS_profiles->current()->timeIntervalButton==1 && event->button()==Qt::RightButton))
		{
			PLFLT X_=(PLFLT)(event->x()-x_offset)/x_fact;
			PLFLT Y_=(PLFLT)(event->y()-y_offset)/y_fact;
			emit doubleClicked(X_, Y_, event->x(), event->y());
		}
		return;
	}
  

}

void QtPLWidget::drawLine(short x1, short y1, short x2, short y2)
{
	BufferElement el;
	el.Element=LINE;
	el.Data.Line=new QLineF(QPointF((PLFLT)x1/m_dVirtualWidth, (PLFLT)(1.-y1/m_dVirtualHeight)), QPointF((PLFLT)x2/m_dVirtualWidth, (PLFLT)(1.-y2/m_dVirtualHeight)));
    
	m_listBuffer.append(el);
}

void QtPLWidget::drawPolyline(short * x, short * y, PLINT npts)
{
	BufferElement el;
	el.Element=POLYLINE;
	el.Data.Polyline=new QPolygonF;
	for(int i=0; i<npts; ++i)
	{
		(*el.Data.Polyline) << QPointF((PLFLT)(x[i])/m_dVirtualWidth, (PLFLT)(1.-(y[i])/m_dVirtualHeight));
	}
    
	m_listBuffer.append(el);
}

void QtPLWidget::drawPolygon(short * x, short * y, PLINT npts)
{
	BufferElement el;

	bool isRect=false;
	if(npts==4) // Check if it's a rectangle. If so, it can be made faster to display
	{
		if(x[0]==x[1] && x[2]==x[3] && y[0]==y[3] && y[1]==y[2])
		{
			isRect=true;
		}
		else if(x[0]==x[3] && x[1]==x[2] && y[0]==y[1] && y[2]==y[3])
		{
			isRect=true;
		}
	}
	if(npts==5)
	{
		if(x[0]==x[4] && y[0]==y[4])
		{
			if(x[0]==x[1] && x[2]==x[3] && y[0]==y[3] && y[1]==y[2])
			{
				isRect=true;
			}
			else if(x[0]==x[3] && x[1]==x[2] && y[0]==y[1] && y[2]==y[3])
			{
				isRect=true;
			}
		}
	}
	
	if(isRect)
	{
		el.Element=RECTANGLE;
		
		double x1, y1, x2, y2, x0, y0, width, height;
		x1=(PLFLT)(x[0])/m_dVirtualWidth;
		x2=(PLFLT)(x[2])/m_dVirtualWidth;
		y1=(PLFLT)(1.-(y[0])/m_dVirtualHeight);
		y2=(PLFLT)(1.-(y[2])/m_dVirtualHeight);
		if(x1<x2)
		{
			x0=x1;
			width=x2-x1;
		}
		else
		{
			x0=x2;
			width=x1-x2;
		}
		if(y1<y2)
		{
			y0=y1;
			height=y2-y1;
		}
		else
		{
			y0=y2;
			height=y1-y2;
		}
		el.Data.Rect=new QRectF(x0, y0, width, height);
// 		(*el.Data.Rect)=el.Data.Rect->normalized();
	}
	else
	{
		el.Element=POLYGON;
		el.Data.Polyline=new QPolygonF;
		for(int i=0; i<npts; ++i)
		{
			(*el.Data.Polyline) << QPointF((PLFLT)(x[i])/m_dVirtualWidth, (PLFLT)(1.-(y[i])/m_dVirtualHeight));
		}
	}
    
	m_listBuffer.append(el);

}

void QtPLWidget::setColor(int r, int g, int b, double alpha)
{
	if(lastColour.r!=r || lastColour.g!=g || lastColour.b!=b || lastColour.alpha!=alpha)
	{
		int alpha_int = (int)(alpha*255.);
		if(alpha_int < 0) alpha_int = 0;
		
		BufferElement el;
		el.Element=SET_COLOUR;
		el.Data.ColourStruct = new struct ColourStruct_;
		el.Data.ColourStruct->R=r;
		el.Data.ColourStruct->G=g;
		el.Data.ColourStruct->B=b;
		el.Data.ColourStruct->A=alpha_int;
	
		m_listBuffer.append(el);
		
		lastColour.r=r;
		lastColour.g=g;
		lastColour.b=b;
		lastColour.alpha=alpha;
	}
}

void QtPLWidget::setWidth(PLFLT w)
{
	BufferElement el;
	el.Element=SET_WIDTH;
	el.Data.fltParam=w;

	m_listBuffer.append(el);
}

void QtPLWidget::paintEvent( QPaintEvent * )
{
	static bool paintActive = false; // initialise to false to allow first call (static so only done 1st call)
	if(paintActive)	return; // block recursive repaints (should not happen now, just defensive)
	
	paintActive = true;
	
	double x_fact, y_fact, x_offset(0.), y_offset(0.); //Parameters to scale and center the plot on the widget
// 	double ratio;
	getPlotParameters(x_fact, y_fact, x_offset, y_offset/*, ratio*/);
    
	if(m_bAwaitingRedraw || m_pixPixmap==NULL || m_listBuffer.size()!=m_iOldSize  )
	{
		if(m_pixPixmap!=NULL) delete m_pixPixmap;
		m_pixPixmap=new QPixmap(width(), height());
		QPainter * painter=new QPainter;
		painter->begin(m_pixPixmap);
		painter->fillRect(0, 0, width(), height(), QBrush(Qt::white));
		painter->fillRect(0, 0, width(), height(), QBrush(Qt::gray, Qt::Dense4Pattern));
		painter->fillRect((int)x_offset, (int)y_offset, (int)floor(x_fact+0.5), (int)floor(y_fact+0.5), QBrush(Qt::white));
		plot(painter, x_fact, y_fact, x_offset, y_offset);
		painter->end();
		m_bAwaitingRedraw=false;
		m_iOldSize=m_listBuffer.size();
	}
	
	m_painterP->begin(this);
	
	m_painterP->drawPixmap(0, 0, *m_pixPixmap);

	if(cursorParameters.isTracking)
	{
		if(cursorParameters.step==1)
		{
			m_painterP->drawLine((int)(cursorParameters.cursor_start_x*x_fact+x_offset), 0, (int)(cursorParameters.cursor_start_x*x_fact+x_offset), height());
			if(cursorParameters.showHorizontalLine)
			{
				m_painterP->drawLine(0, (int)(cursorParameters.cursor_start_y*y_fact+y_offset), width(), (int)(cursorParameters.cursor_start_y*y_fact+y_offset));
			}
		}
		else
		{
			m_penCurrent.setStyle(Qt::DotLine);
			m_painterP->setPen(m_penCurrent);
			m_painterP->drawLine((int)(cursorParameters.cursor_start_x*x_fact+x_offset), 0, (int)(cursorParameters.cursor_start_x*x_fact+x_offset), height());
			if(cursorParameters.showHorizontalLine)
			{
				m_painterP->drawLine(0, (int)(cursorParameters.cursor_start_y*y_fact+y_offset), width(), (int)(cursorParameters.cursor_start_y*y_fact+y_offset));
			}
			m_penCurrent.setStyle(Qt::SolidLine);
			m_painterP->setPen(m_penCurrent);
			m_painterP->drawLine((int)(cursorParameters.cursor_end_x*x_fact+x_offset), 0, (int)(cursorParameters.cursor_end_x*x_fact+x_offset), height());
			if(cursorParameters.showHorizontalLine)
			{
				m_painterP->drawLine(0, (int)(cursorParameters.cursor_end_y*y_fact+y_offset), width(), (int)(cursorParameters.cursor_end_y*y_fact+y_offset));
			}
		}
	}
    
	m_painterP->end();
	paintActive = false;

}

void QtPLWidget::print(QPrinter& printer, double weight)
{
	QPainter * p=new QPainter;
	
	currentMode=PLOT_MODE_PS;
	
	QRect pageRect=printer.pageRect();
	
	double paperH = pageRect.height();
	double paperW = pageRect.width();

	double penScale = (paperH + paperW) / weight;

 	p->begin((QPaintDevice*)&printer);
 	plot(p, paperW, paperH, 0., 0., false, penScale); 
	currentMode=PLOT_MODE_RASTER;
	p->end();
}

void QtPLWidget::setPrintRes(int resolution)
{
	printRes = resolution;
}

void QtPLWidget::savePixmap(QString& name, const char * format, int w, int h, int compression, double weight)
{

	QImage im(w, h, QImage::Format_RGB32);
    
    
	double x_fact(w), y_fact(h); //Parameters to scale the plot
    
	m_painterP->begin(&im);

	m_painterP->fillRect(0, 0, w, h, QBrush(Qt::white));

	double penScale = 180. / (double)printRes / weight;
	plot(m_painterP, x_fact, y_fact, 0., 0., 1,  penScale);/*NoQTransform*/
    
	m_painterP->end();
	im.save(name, format);
    
}

void QtPLWidget::saveSVG(QString& name, int width, int height, int resolution, double weight)
{
	QSvgGenerator im;
    
	int w=(width*72)/resolution;
	int h=(height*72)/resolution;
	im.setFileName(name);
	im.setResolution(72);
	//im.setSize(QSize(w, h));
	im.setViewBox(QRect(0, 0, w, h));
	
	qApp->processEvents();

 
	m_painterP->begin(&im);

	m_painterP->fillRect(0, 0, w, h, QBrush(Qt::white));
	qApp->processEvents();
	double penScale = 1.4*(w + h)/weight; // svg needs thinner lines than the print route

	currentMode=PLOT_MODE_SVG;
	plot(m_painterP, w, h, 0., 0., false, penScale );
	currentMode=PLOT_MODE_RASTER;
    
	qApp->processEvents();

	m_painterP->end();

}


void QtPLWidget::plot(QPainter* p, double x_fact, double y_fact, double x_offset, double y_offset, bool useCosmeticPens, double penScale, double weight)
{
	QPen SolidPen;
    SolidPen.setCosmetic(useCosmeticPens); 
	QPen NoPen(Qt::NoPen);
	NoPen.setWidthF(0.); // Cosmetic pen
	p->setPen(SolidPen);
	bool hasPen=true;
	
	QBrush SolidBrush(Qt::SolidPattern);
	p->setBrush(SolidBrush);

#if QT_VERSION >= 0x040300
	QTransform trans;
	trans=trans.translate(x_offset, y_offset);
	trans=trans.scale(x_fact, y_fact);
	p->setTransform(trans, false);
#else

	QMatrix trans;
	trans=trans.translate(x_offset, y_offset);
	trans=trans.scale(x_fact, y_fact);
	
	p->setMatrix(trans);
#endif
	
	if(m_listBuffer.empty())
	{
		// p->fillRect(0, 0, 1, 1, QBrush());
		return;
	}
    
	for(QLinkedList<BufferElement>::const_iterator i=m_listBuffer.begin(); i!=m_listBuffer.end(); ++i)
	{
		switch(i->Element)
		{
			case LINE:
				if(!hasPen)
				{
					p->setPen(SolidPen);
					hasPen=true;
				}
                p->drawLine(*(i->Data.Line));
				break;
            
			case POLYLINE:
				if(!hasPen)
				{
					p->setPen(SolidPen);
					hasPen=true;
				}
				p->drawPolyline(*(i->Data.Polyline));
				break;
                
			case RECTANGLE:
				if(hasPen)
				{
					p->setPen(NoPen);
					hasPen=false;
				}
				p->drawRect(*(i->Data.Rect));
				break;
				
			case POLYGON:
				if(hasPen)
				{
					p->setPen(NoPen);
					hasPen=false;
				}
				p->drawConvexPolygon(*(i->Data.Polyline));
				break;
                
			case TEXT:
				p->save();
#if QT_VERSION >= 0x040300
				p->resetTransform();
#else
				p->resetMatrix();
#endif

				renderText(p, i->Data.TextStruct, x_fact, x_offset, y_fact, y_offset, weight);
				p->restore();
				break;
				
			case SET_WIDTH:
                if(useCosmeticPens)
                {
                    SolidPen.setWidthF(PlWidth2QtWidth(i->Data.fltParam)/penScale);
                }
                else
                {
                    SolidPen.setWidthF(PlWidth2QtWidth(i->Data.fltParam)/penScale);
                }

                if(hasPen)
				{
					p->setPen(SolidPen);
				}
				break;
                
			case SET_COLOUR:
				SolidPen.setColor(QColor(i->Data.ColourStruct->R, i->Data.ColourStruct->G, i->Data.ColourStruct->B, i->Data.ColourStruct->A));
				if(hasPen)
				{
					p->setPen(SolidPen);
				}
				SolidBrush.setColor(QColor(i->Data.ColourStruct->R, i->Data.ColourStruct->G, i->Data.ColourStruct->B, i->Data.ColourStruct->A));
				p->setBrush(SolidBrush);
				break;
                
			
			case SET_SMOOTH:
				p->setRenderHint(QPainter::Antialiasing, i->Data.intParam);
				break;
                
			default:
				break;
		}
	}

}

void QtPLWidget::setSmooth(bool b)
{
	BufferElement el;
	el.Element=SET_SMOOTH;
	el.Data.intParam=b;
	m_listBuffer.append(el);
}

void QtPLWidget::clearWidget()
{
	
	clearBuffer();
	m_bAwaitingRedraw=true;
	update();
}

void QtPLWidget::clearBuffer()
{
	lastColour.r=-1;
	for(QLinkedList<BufferElement>::iterator i=m_listBuffer.begin(); i!=m_listBuffer.end(); ++i)
	{
		switch(i->Element)
		{
			case LINE:
				delete i->Data.Line;
				break;
			case RECTANGLE:
				delete i->Data.Rect;
				break;
				
			case POLYLINE:
			case POLYGON:
				delete i->Data.Polyline;
				break;
            
			case TEXT:
				delete[] i->Data.TextStruct->text;
				delete i->Data.TextStruct;
				break;
				
			case SET_COLOUR:
				delete i->Data.ColourStruct;
				break;
			default:
				break;
		}
	}
    
	m_listBuffer.clear();
}



QFont QtPLWidget::getFont(PLUNICODE unicode)
{
	// Get new font parameters
	unsigned char fontFamily, fontStyle, fontWeight;

	plP_fci2hex(unicode, &fontFamily, PL_FCI_FAMILY);
	plP_fci2hex(unicode, &fontStyle, PL_FCI_STYLE);
	plP_fci2hex(unicode, &fontWeight, PL_FCI_WEIGHT);
			
	QFont f;

	switch(currentMode)
	{
		case PLOT_MODE_SVG:
		case PLOT_MODE_PS:
			f.setPointSize(currentFontSize*currentFontScale<4 ? 4 : (int)(currentFontSize*currentFontScale));
			break;
		default:
			f.setPixelSize(currentFontSize*currentFontScale<4 ? 4 : (int)(currentFontSize*currentFontScale));
			break;
	}


	switch(fontFamily) {
		case 1:
			f.setStyleHint(QFont::Serif);
			f.setFamily(QSAS_profiles->current()->SerifFont);
			break;
		case 2:
			f.setStyleHint(QFont::TypeWriter);
			f.setFamily(QSAS_profiles->current()->MonotypeFont);
			break;
		case 3:
			f.setStyleHint(QFont::Decorative);
			f.setFamily(QSAS_profiles->current()->DecorativeFont);
			break;
		case 0: default:
			f.setStyleHint(QFont::SansSerif);
			f.setFamily(QSAS_profiles->current()->SansFont);
			break;
	}

	if(fontStyle) f.setItalic(true);
	if(fontWeight) f.setWeight(QFont::Bold);
	else f.setWeight(QFont::Normal);
			
	f.setUnderline(underlined);
	f.setOverline(overlined);

	return f;
}

void QtPLWidget::drawTextInPicture(QPainter* p, const QString& text)
{
	QPicture tempPic;
	QPainter tempPainter(&tempPic);
	tempPainter.setFont(p->font());
#if QT_VERSION >= 0x040300
	QRectF rect(0., 0., 0., 0.);
	QRectF bounding;
	if((currentMode==PLOT_MODE_SVG || currentMode==PLOT_MODE_PS) && QSAS_profiles->current()->VectoriseSVGFonts)
	{
		bounding=tempPainter.boundingRect(rect, Qt::AlignHCenter|Qt::AlignVCenter|Qt::TextDontClip, text);
		QPen savePen=tempPainter.pen();
	
		QPen pen=savePen;
		pen.setStyle(Qt::NoPen);
		tempPainter.setPen(pen);
		
		QPainterPath path;
		path.addText (QPointF(bounding.left(), bounding.top()+0.65*bounding.height()), tempPainter.font(), text);
		tempPainter.drawPath(path);
		tempPainter.setPen(savePen);
	}
	else
	{
		tempPainter.drawText(rect, Qt::AlignHCenter|Qt::AlignVCenter|Qt::TextDontClip, text, &bounding);
	}
#else
	QRect rect(0., 1000, 0., 1000);
	QRect bounding;
	bounding=tempPainter.boundingRect(rect, Qt::AlignHCenter|Qt::AlignVCenter, text);
	tempPainter.drawText(0, 0,/* Qt::AlignHCenter|Qt::AlignVCenter,*/ text/*, &bounding*/);
#endif
	
	tempPainter.end();

	p->drawPicture((int)(xOffset+bounding.width()/2.), (int)-yOffset, tempPic);

	// Bug in Qt when not using "path-ified" texts: the bounding boxes are not updated.
	// So we update them now
	// Added, compared to the official plplot driver, to handle #n (new line)

	QPicture* currentPicture=dynamic_cast<QPicture*>(p->device());
	if(currentPicture!=NULL)
	{
		QRect currentBounding=currentPicture->boundingRect();
#if QT_VERSION >= 0x040300
		QRect boundingOffset=bounding.toRect();
#else
		QRect boundingOffset=bounding;
#endif
		boundingOffset.adjust((int)(xOffset+bounding.width()/2.), (int)-yOffset, (int)(xOffset+bounding.width()/2.), (int)-yOffset);
		currentBounding=currentBounding.united((const QRect&)boundingOffset);
		currentPicture->setBoundingRect(currentBounding);
	}

	xOffset+=bounding.width();
}

QPicture QtPLWidget::getTextPicture(PLUNICODE fci, PLUNICODE* text, int len, double chrht, double scale_fact, double weight)
{
	// This one differs from Qt3's and PLplot's as plplot does not handle "new line" (#n).
	// To implement this, we draw on a list of qpictures that get stacked on top of each other
	// Otherwise, it works similarly
	char plplotEsc;
	plgesc( &plplotEsc );
	
	double old_fontScale;
  
	QPicture currentPicture;
	QPainter p(&currentPicture);
	QVector<QPicture> stack;
	stack.push_back(currentPicture);
	
	QString currentString;

	yOffset=0.;
	xOffset=0.;

	switch(currentMode)
	{
		case PLOT_MODE_SVG:
			currentFontSize=chrht*scale_fact*PLOT_FONT_FACTOR_SVG*weight;
			break;
		case PLOT_MODE_PS:
			#if defined(Q_OS_MACX)
			currentFontSize=chrht*PLOT_FONT_FACTOR_PS*weight*scale_fact/600.;
			#else
			currentFontSize=chrht*PLOT_FONT_FACTOR_PS*weight;
			#endif
			break;
		default:
			currentFontSize=chrht*scale_fact*PLOT_FONT_FACTOR_RASTER*weight;//*/*POINTS_PER_INCH*/72./25.4;
			break;
	}
	
	currentFontScale=1.;
	underlined=false;
	overlined=false;
	
	p.setFont(getFont(fci));

	int i=0;
	while(i < len)
	{
		if(text[i]<PL_FCI_MARK) // Not a font change
		{
			if(text[i]!=(PLUNICODE)plplotEsc)
			{
				currentString.append(QString(QChar(text[i])));
				++i;
				continue;
			}
			++i; // Now analyse the escaped character
			switch(text[i])
			{
				case 'd':
					drawTextInPicture(&p, currentString);
					
// 					currentString.clear();
					currentString="";
					old_fontScale=currentFontScale;
					if( yOffset>/*0.0*/0.000000000001 )
						currentFontScale *= 1.25;  /* Subscript scaling parameter */
					else
						currentFontScale *= 0.8;  /* Subscript scaling parameter */
					yOffset -= currentFontSize * old_fontScale / 2.;
					
					p.setFont(getFont(fci));
					break;
				case 'u':
					drawTextInPicture(&p, currentString);
// 					currentString.clear();
					currentString="";
					if( yOffset</*0.0*/-0.000000000001 )
						currentFontScale *= 1.25;  /* Subscript scaling parameter */
					else
						currentFontScale *= 0.8;  /* Subscript scaling parameter */
					yOffset += currentFontSize * currentFontScale / 2.;
					p.setFont(getFont(fci));
					break;
				case '-':
					drawTextInPicture(&p, currentString);
					
// 					currentString.clear();
					currentString="";
					underlined=!underlined;
					p.setFont(getFont(fci));
					break;
				case '+':
					drawTextInPicture(&p, currentString);

// 					currentString.clear();
					currentString="";
					overlined=!overlined;
					p.setFont(getFont(fci));
					break;
				case '#':
					currentString.append(QString((QChar*)&(text[i]), 1));
					break;
				case 'n':
					drawTextInPicture(&p, currentString);
					currentString="";
					xOffset=0.;
					p.end();
					currentPicture=QPicture();
					stack.push_back(currentPicture);
					p.begin(&currentPicture);
					p.setFont(getFont(fci));
					break;
				default :
// 					std::cout << "unknown escape char " << ((QChar)text[i]).toLatin1() << std::endl;
					break;
			}
		}
		else // Font change
		{
			drawTextInPicture(&p, currentString);

// 			currentString.clear();
			currentString="";
			fci=text[i];
			p.setFont(getFont(fci));
		}
		++i;
	}
	drawTextInPicture(&p, currentString);
	p.end();
	
	if(stack.size()==1)
	{
		return currentPicture;
	}
	else
	{
		double height=0.;
		double width=0.;
		
		foreach(QPicture pic, stack)
		{
			if(width<pic.boundingRect().width())
				width=pic.boundingRect().width();
		}
		QPicture res;
		QPainter paintRes(&res);
		foreach(QPicture pic, stack)
		{
			paintRes.drawPicture(QPointF((width-pic.width())/2., height), pic);
			height+=pic.boundingRect().height()*0.8;
		}
		
		paintRes.end();
		xOffset=width;
		return res;
	}
	
	
}

void QtPLWidget::renderText(QPainter* p, struct TextStruct_* s, double x_fact, double x_offset, double y_fact, double y_offset, double weight)
{
	if(s->len<=0 || s->len>=500) return;
	QPicture picText=getTextPicture(s->fci, s->text, s->len, s->chrht, (x_fact+y_fact)/2.0, weight);
	double picDpi=picText.logicalDpiY();

	p->setClipping(true);
	p->setClipRect(QRectF(s->clipxmin*x_fact+x_offset, s->clipymax*y_fact+y_offset, (s->clipxmax-s->clipxmin)*x_fact, (-s->clipymax+s->clipymin)*y_fact), Qt::ReplaceClip);
	
	p->translate(s->x*x_fact+x_offset, s->y*y_fact+y_offset);
// 	p->rotate(-s->rotation);
	QMatrix rotShearMatrix(cos(s->rotation)*s->stride, -sin(s->rotation)*s->stride, cos(s->rotation)*sin(s->shear)+sin(s->rotation)*cos(s->shear), -sin(s->rotation)*sin(s->shear)+cos(s->rotation)*cos(s->shear), 0.,0.);
	p->setWorldMatrix(rotShearMatrix, true);

	p->translate(-s->just*xOffset*p->device()->logicalDpiY()/picDpi, 0.);
	
	p->drawPicture(0, 0, picText);
	
	p->setWorldMatrix(QMatrix());

	p->setClipping(false);
}

void QtPLWidget::drawText(PLStream* pls, EscText* txt)
{
	BufferElement el;
	el.Element=TEXT;
	el.Data.TextStruct=new struct TextStruct_;
	el.Data.TextStruct->x=txt->x/m_dVirtualWidth;
	el.Data.TextStruct->y=1.-txt->y/m_dVirtualHeight;
	el.Data.TextStruct->clipxmin=pls->clpxmi/m_dVirtualWidth;
	el.Data.TextStruct->clipymin=1.-pls->clpymi/m_dVirtualHeight;
	el.Data.TextStruct->clipxmax=pls->clpxma/m_dVirtualWidth;
	el.Data.TextStruct->clipymax=1.-pls->clpyma/m_dVirtualHeight;
	PLUNICODE fci;
	plgfci(&fci);
	el.Data.TextStruct->fci=fci;
	PLFLT rotation, shear, stride;
	plRotationShear( txt->xform, &rotation, &shear, &stride);
	rotation -= pls->diorot * M_PI / 2.0;
	el.Data.TextStruct->rotation=rotation;
	el.Data.TextStruct->shear=shear;
	el.Data.TextStruct->stride=stride;
	el.Data.TextStruct->just=txt->just;
	el.Data.TextStruct->text=new PLUNICODE[txt->unicode_array_len];
	memcpy(el.Data.TextStruct->text, txt->unicode_array, txt->unicode_array_len*sizeof(PLUNICODE));
	el.Data.TextStruct->len=txt->unicode_array_len;
	el.Data.TextStruct->chrht=pls->chrht;
	
	m_listBuffer.append(el);
}
