
#include <stdio.h>
#include <math.h>

#ifdef HAVE_IEEEFP_H
 #include <ieeefp.h>
#endif

#include <qglobal.h>
#include <qapplication.h>
#include <qmainwindow.h>
#include <qpalette.h>
#include <qlineedit.h>
#include <qinputdialog.h>
#include <qstatusbar.h>
#include <qregexp.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qdatetime.h> 
#include <qstring.h> 
#include <qwidget.h>
#include <qcheckbox.h>

#include "Qdos.h"
#include "Xrefs.h"
#include "qdutil.h"
#include "DvUnit.h"
#include "qwui.h"

using namespace QSAS;
 qwuiWindow* qsasWindow;

ObjectNamesTable_var QDOSlist = new ObjectNamesTable;
static double CLOSE_ENOUGH = 1.e-8;
extern int qsas_restore_release;
double _PI = 4.*atan(1.0);


QdRScalarSeq_var QuRegression(QdObject_var dobj, QdObject_var userDep0)
{
	QdObject_var dep0ret;
	QdRScalarSeq_var ret;
	QdRScalarSeq_var scalarDep0 = new QdRScalarSeq(2);
	QdRScalarSeq_var yobj = QdRScalarSeq_var::narrow(dobj);
	if(yobj.is_nil()) return ret;
	
	QdObject_var dep;
	if( userDep0.is_nil() ) dep = get_Depend_0(dobj);
	else  dep = userDep0;
	
	if(dep.is_nil()) return ret;

	QdString_var dep_units_str;	
	QdString_var dep_SIc_str;	
	
	QdRScalarSeq_var xobj = QdRScalarSeq_var::narrow(dep);
	
	if(xobj.is_nil()){
		QdTimeSeq_var tt = QdTimeSeq_var::narrow(dep);
		if(tt.is_nil()) return ret;
		QdTimeSeq_var dep0 = new QdTimeSeq(2);
		dep0[0] = tt[0];
		dep0[1] = tt[tt->sequence_size()-1];
		dep0ret = dep0;
		
		xobj = new QdRScalarSeq(tt->sequence_size());
		for(int i=0; i<tt->sequence_size(); i++){
			xobj[i] = tt[i] - tt[0];
		}
		dep_units_str = new QdString("s");
		dep_SIc_str = new QdString("1>s");
		
	}
	else{
		dep0ret = scalarDep0;
		if(dep->xref_exists(UNITS)){
			QdObject_var dep_units_xref = dep->get_xref(UNITS);
			dep_units_str = QdString_var::narrow(dep_units_xref);
		}
		if(dep->xref_exists(SI_CONVERSION)){
			QdObject_var dep_SIc_xref = dep->get_xref(SI_CONVERSION);
			dep_SIc_str = QdString_var::narrow(dep_SIc_xref);
		}
	}
	if(!dep_units_str.is_nil()) dep0ret->change_xref(UNITS, dep_units_str);
	if(!dep_SIc_str.is_nil()) dep0ret->change_xref(SI_CONVERSION, dep_SIc_str);

	
	if(yobj->sequence_size() != xobj->sequence_size()) return ret;
	
	int n = yobj->sequence_size();
	double sigxy = 0;
	double sigxx = 0;
	double sigx = 0;
	double sigy = 0;
	double sigyy = 0;
	double x, y;
	
	double min = xobj[0];
	double max = xobj[xobj->sequence_size()-1];
	
	for(int i=0; i<n; i++){
		x = xobj[i];
		if(min > x) min = x;
		if(max < x) max = x;
		
		y = yobj[i];
		sigxy += y * x;
		sigxx += x * x;
		sigx  += x;
		sigy  += y;
		sigyy += y * y;
	}
	double m = (n*sigxy - sigx * sigy) /(n*sigxx - sigx * sigx);
	double b = (sigy - m * sigx) / n;
	double r = (n*sigxy - sigx * sigy) / sqrt( (n*sigxx - sigx*sigx) * (n*sigyy - sigy * sigy) );
	
	ret = new QdRScalarSeq(2);
	ret[0] = m*min + b;
	ret[1] = m*max + b;
	
	scalarDep0[0] = min;
	scalarDep0[1] = max;
	
	
	QdRScalar_var mObj = new QdRScalar(m);
	QdRScalar_var bObj = new QdRScalar(b);
	QdRScalar_var rObj = new QdRScalar(r);

	if(dobj->xref_exists(UNITS)) {
		QdObject_var units_xref = dobj->get_xref(UNITS);
		bObj->change_xref(UNITS, units_xref);
		QdString_var units_str = QdString_var::narrow(units_xref);
		
		if(!dep_units_str.is_nil() && !units_str.is_nil() ) {
			QdString_var slope_units = new QdString(*units_str);
			*slope_units += "/";
			*slope_units += *dep_units_str;
			mObj->change_xref(UNITS, slope_units);
		}
			
	}
	if(dobj->xref_exists(SI_CONVERSION)) {
		QdObject_var SIc_xref = dobj->get_xref(SI_CONVERSION);
		bObj->change_xref(SI_CONVERSION, SIc_xref);
		QdString_var SIc_str = QdString_var::narrow(SIc_xref);
		
		if(!dep_SIc_str.is_nil() && !SIc_str.is_nil() ) {
			QdString_var slope_SIc = QuSIC_ratio(SIc_str, dep_SIc_str);
			mObj->change_xref(SI_CONVERSION, slope_SIc);
		}
		
	}
		
	ret->change_xref(DEPEND_0, dep0ret);
	ret->change_xref("Slope", mObj);
	ret->change_xref("Intercept", bObj);
	ret->change_xref("CoeffCorrel", rObj);
	 
	return ret;
	
}

QdString_var QuSIC_ratio(QdString_var SI1, QdString_var SI2){


	QdString_var null_SIC = new QdString("1 > ()");

	if(SI1.is_nil() || SI2.is_nil() ) return null_SIC;

	string siconv1 = string(SI1->c_str());
	string siconv2 = DvUnitGetPowerSI(SI2->c_str(), -1);
	
	size_t ptr1, ptr2;
	
	if( (ptr1 = siconv1.find(">")) == string::npos) return null_SIC;
	double value1 = atof(siconv1.substr(0,ptr1).c_str());
  
	if( (ptr2 = siconv2.find(">")) == string::npos) return null_SIC;
	
	double value2 = atof(siconv2.substr(0,ptr2).c_str());
	double value = value1 * value2;

   	
	static char buffer[100];
	sprintf(buffer, "%16g >", value);
	
	string base_str = string(buffer);
	base_str += siconv1.substr(ptr1+1, string::npos); 
	base_str += " ";
	base_str += siconv2.substr(ptr2+1, string::npos);
    
	QdString_var SIC_ptr =  new QdString(DvUnitGetTidySI(base_str.c_str()));
  
	return SIC_ptr;

}


QDomElement createTextXMLNode(QDomDocument& doc, const QString& parentName, const QString& s)
{
    QDomElement res=doc.createElement(parentName);
    res.appendChild(doc.createTextNode(s));
    return res;
}

QDomElement createTextXMLNode(QDomDocument& doc, const QString& parentName, const std::string& s)
{
    QDomElement res=doc.createElement(parentName);
    res.appendChild(doc.createTextNode(QString::fromStdString(s)));
    return res;
}

QDomElement createTextXMLNode(QDomDocument& doc, const QString& parentName, const char* s)
{
    QDomElement res=doc.createElement(parentName);
    res.appendChild(doc.createTextNode(QString::fromLatin1(s)));
    return res;
}

QDomElement createTextXMLNode(QDomDocument& doc, const QString& parentName, const double d)
{
    QDomElement res=doc.createElement(parentName);
    res.appendChild(doc.createTextNode(QString::number(d)));
    return res;
}

QDomElement createTextXMLNode(QDomDocument& doc, const QString& parentName, const int i)
{
    QDomElement res=doc.createElement(parentName);
    res.appendChild(doc.createTextNode(QString::number(i)));
    return res;
}

QDomElement createTextXMLNode(QDomDocument& doc, const QString& parentName, const bool b)
{
    QDomElement res=doc.createElement(parentName);
    res.appendChild(doc.createTextNode(QString::number((int)b)));
    return res;
}

bool getXMLNodeText(QDomElement& el, const QString& tag, QString& val)
{
    QDomElement subEl=el.firstChildElement(tag);
    if(!subEl.isNull())
    {
        val=subEl.firstChild().nodeValue();
        return true;
    }
    return false;
}

// Used by xdr, does nothing
QString encode(const QString& input)
{
	QString res(input);
	
	return res;
}

QString decode(const QString& input)
{
	QString res(input);
	
	return res;
}


char * QuNewSTR(const char * in_str)
{
    char *new_str;
    if(in_str != NULL){
        new_str = (char *) malloc(strlen(in_str)+1);
        strcpy(new_str, in_str);
    }
    else {
        new_str = (char *) malloc(1);
        new_str[0] = '\0';
    }

    return &(new_str[0]);
}

int QuAreUnitsSame(QdObject_var ptr1,
                    QdObject_var ptr2)
{
    std::string base_str1;
    std::string base_str2;

    int status;

    status = QDUTIL_OK;
  
    string SIc1 = QuGetAttrText("SI_conversion", ptr1);
    if (SIc1.empty())   status = QDUTIL_WARN;

    string SIc2 = QuGetAttrText("SI_conversion", ptr2);
    if (SIc2.empty())  status = QDUTIL_WARN;

    if(status != QDUTIL_WARN)
    {
        base_str1 = DvUnitGetBaseSI(SIc1.c_str());
        base_str2 = DvUnitGetBaseSI(SIc2.c_str());

        if(base_str1 != base_str2) status = QDUTIL_ERR;
  
    }
   
    return status;
  
} // end QuAreUnitsSame

// ------------------------------------------


int QuAreFramesSame(QdObject_var ptr1,
                     QdObject_var ptr2)
{

    string F1, F2;
    string attr_ptr;

    attr_ptr = QuGetFrameAttr(ptr1);  // this gets CSDS or CAA frame info
      
    if ( attr_ptr.find( "unk") != string::npos ) return QDUTIL_WARN; // no frame found
    
    F1 = QuToUpper(attr_ptr);
    
    attr_ptr = QuGetFrameAttr(ptr2);
    if ( attr_ptr.find( "unk") != string::npos )
    {
        return QDUTIL_WARN;
    }
    F2 = QuToUpper(attr_ptr);
        
    if(F1 != F2)
    {
        return QDUTIL_ERR;
    }
    
    return QDUTIL_OK;
    
} // end QuAreFramesSame

// ------------------------------------------

QString QuGetDefaultComp(const QString& WL, const QString& compDef)
{
	if(QuSlotDebug()) cout << "qdutil::QuGetDefaultComp: " << compDef.toLatin1().data() << " for " << WL.toLatin1().data() << endl;
	QString outText = QString(WL);
	
	if(compDef.isEmpty()) return outText;
	
	QString compTxt = " [";
	QdObject_var dobj = QuGetObjByName(WL);
	QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(dobj);
	if(ms.is_nil()) return outText;
	if((ms[0].dims()).size()!=3 ) return outText;
	if(!ms->is_homogeneous_seq()) return outText;
	
	int di;
	bool foundTslice=false;
	
	if(compDef.contains("[") || compDef.contains("<"))
	{
		compTxt = QString(" ")+compDef;
	}
	else for (int i=0; i<compDef.length(); i++)
	{
		if(compDef[i] == QChar('i'))
		{
			if((ms[0].dims()).size() > i) di = (ms[0].dims())[i];
			else return outText;
			compTxt +=  QString("(1,%1) ").arg(di);
		}
		else if(compDef[i] == QChar('S'))
		{
			if((ms[0].dims()).size() > i) di = (ms[0].dims())[i];
			else return outText;
			compTxt +=  QString("Sum(1,%1) ").arg(di);
		}
		else if(compDef[i] == QChar('A'))
		{
			if((ms[0].dims()).size() > i) di = (ms[0].dims())[i];
			else return outText;
			compTxt +=  QString("Ave(1,%1) ").arg(di);
		}
		else if(compDef[i] == QChar('t'))
		{
			foundTslice = true;
		}
		else return outText;
	}
	if(foundTslice) compTxt +=  "] <0>";
	else compTxt +=  "]";
	outText += QString(compTxt);
	
	return outText;
}

int QuEpochType(QdObject_var obj){
	
	// It is not obvious we ever get epoch types now as we convert to QmjdTime
	
	if(obj.is_nil()) return 0;
	
	if( !obj->xref_exists(SI_CONVERSION) ) return 0;
	
	QdObject_var attr = obj->get_xref(SI_CONVERSION);
	
	QdString_var attrStr = QdString_var::narrow(attr);
	if(attrStr.is_nil()) return 0;
	
	string base_str1 = DvUnitGetBaseSI(attrStr->c_str());
	string base_str2 = DvUnitGetBaseSI("1.e-3>s");

	if(base_str1 == base_str2) return 1;

	string base_str3 = DvUnitGetBaseSI("1.e-12>s");
	
	if(base_str1 == base_str3) return 2;

	return false;
	
}

void QuReplaceTokens(QString &newText, QString &oldText)
{
	int ptr = oldText.indexOf("?");
	while( ptr != -1)
	{
		oldText.remove(0, ptr+1);

		int ptr2 = oldText.indexOf("=");
		QString before = QString(oldText); 
		before.truncate(ptr2);
		oldText.remove(0, ptr2+1);
		
		int ptr3 = oldText.indexOf("?");
		QString after = QString(oldText); 
		after.truncate(ptr3);

		newText.replace(before, after, Qt::CaseInsensitive);
		
		oldText.remove(0, ptr3+1);

		ptr = oldText.indexOf("?");

	}
}

bool QuIsMatch(double value, double fill){
	// used by Filters in Calc and Filter plugins
	// Does not check strip toggles on main window

    if( fill == 0 && value == fill) return true;
    if( fill == -1 && value == fill) return true;
    else return isFill(value, fill);
}

QCheckBox * checkBadData(QWidget *parent){
   static QCheckBox *StripBadData = NULL;
   
   if(StripBadData == NULL) {
      StripBadData = new QCheckBox("Strip NaN   ", parent);
   }
   return StripBadData;
}

QCheckBox * checkFills(QWidget *parent){
   static QCheckBox *StripFills = NULL;
   
   if(StripFills == NULL) {
      StripFills = new QCheckBox("Strip Fill Values   ", parent);
   }
   return StripFills;
}

bool QuSlotDebug(){

	return (QSAS_profiles->current()->Debug);
}


string QuCommonLabl(string str1, string str2){
	string commonStr="";
	string units="";
	string units2="";
  
	// remove mod signs "|"
	size_t pos = str1.find("|");
	if(pos != string::npos) {
    	str1 = str1.substr(pos+1, str1.size() - pos -1);
		pos = str1.find("|");
		if(pos != string::npos) {
       	str1 = str1.substr(0,pos);
		}
  	}
  
	pos = str2.find("|");
	if(pos != string::npos) {
    	str2 = str2.substr(pos+1, str2.size() - pos -1);
		pos = str2.find("|");
		if(pos != string::npos) {
      	 str2 = str2.substr(0,pos);
		}
  	}

	pos = 0;
	
	// remove and keep units for later
    for(int i=0; i<str2.size(); i++){
		if(str2[i] == '(') 
		{
			pos = i;
			do
			{
				units2 += str2[i];
				if (str2[i] == ')' ) break;
				i++;
			}
			while(i < str2.size());
			break;
		}
	}
	if(pos > 0) str2 = str2.substr(0,pos);

    for(int i=0; i<str1.size(); i++){
		if(str1[i] == '(') 
		{
			pos = i;
			do
			{
				units += str1[i];
				if (str1[i] == ')' ) break;
				i++;
			}
			while(i < str1.size());
			break;
		}
	}
	if(pos > 0) str1 = str1.substr(0,pos);

  if (str1 == str2) 
  {
  	if(units == units2) str1 += units;
  	else if(units == "") str1 += units2;
  	else if(units2 == "") str1 += units;
  	return str1;
  }
  
  // break into stem and component
/*  pos =  str2.find("_");
  if(pos != string::npos) {
    string s1 = str2.substr(0,pos);
    string s2 = str2.substr(pos+1, str2.size() - pos -1);
	pos = s2.find(" ");
	if(pos != string::npos) s2 = s2.substr(0,pos);
	
	pos = str1.find(s1);
	if(pos != string::npos) commonStr = s1;
	pos = str1.find(s2);
	if(pos != string::npos) commonStr = s2;
  }
  else */
  {
    for(int i=0; i<str1.size(); i++){
	// handle escaped characters first
	if( str1[i] == '#' )
	{
		if( strncmp(&(str1[i]), &(str2[i]), 3) != 0 ) break;
		else {
			commonStr.append(&(str1[i]), 3);
			i+=2; // add only 2 as i++ adds one
			continue;
		}
	}
	
	// if we get here char was not escaped
	if(str1[i] == str2[i] ) commonStr += str1[i];
	  else break;
	}
  }
  // remove leading and trainmling '-'
  if(commonStr[0] == '_') commonStr = commonStr.substr(1);
  if(commonStr[commonStr.size()-1] == '_') commonStr = commonStr.substr(0, commonStr.size()-1);

	if(units == units2) commonStr += units;
  	else if(units == "") commonStr += units2;
  	else if(units2 == "") commonStr += units;
  
  return commonStr;
}


QdTimeSeq_var get_timetags(const QdObject_var& obj)
{
    QdTimeSeq_var nil_obj;
    if(obj.is_nil()) return  nil_obj;  // NULL var

    QdTimeSeq_var ts=QdTimeSeq_var::narrow(obj);
    if(ts.is_nil())
        if(obj->xref_exists(DEPEND_0))
            ts=QdTimeSeq_var::narrow(obj->get_xref(DEPEND_0));

    return ts;
}

QdObject_var get_Depend_0(const QdObject_var& obj)
{
    QdObject_var ret_obj;
    if(obj.is_nil()) return  ret_obj;  // NULL var

	if(obj->xref_exists(DEPEND_0))
            ret_obj = obj->get_xref(DEPEND_0);

    return ret_obj;
}

QdObject_var getcopy_Depend_0(const QdObject_var& obj)
{
    QdObject_var dep_obj = get_Depend_0(obj);
    if(dep_obj.is_nil()) return  dep_obj;  // NULL var

	QdObject_var ret_obj = Factory::clone(dep_obj);
	if(!ret_obj.is_nil()) ret_obj->copy_xrefs_from(dep_obj);

    return ret_obj;
}

QdTimeInterval_var get_timeinterval(const QdObject_var& obj)
{
    QdTimeInterval_var tint;
    if(obj.is_nil()) return tint; // must test return with is_nil()
    tint=QdTimeInterval_var::narrow(obj);

    if(tint.is_nil())
    {
        QdTimeSeq_var ts=get_timetags(obj);
        if(!ts.is_nil())
            tint=new QdTimeInterval(ts->get_interval_bounds());
    }
    return tint;
}

bool get_ScalarInterval(const QdObject_var& obj, double &start, double &end)
{
    if(obj.is_nil()) return false; 
	
	QdObject_var D0 = get_Depend_0(obj);
	if(D0.is_nil()){
		// try object itself
		QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(obj);
		if(ss.is_nil()) return false;
		
		start = ss[0];
		end = ss[ss->sequence_size()-1];
		return true;
	}
	
	QdRScalarSeq_var stags = QdRScalarSeq_var::narrow(D0);
	if(stags.is_nil() || stags->sequence_size() < 2) return false;
	
	start = stags[0];
	end = stags[stags->sequence_size()-1];
	return true;
}


QdTimeSeq_var get_timetags_inrange(const QdTimeSeq_var& ts_in, const QdTimeInterval_var& tint)
{
    int recStart, recEnd;
	bool OK = QuGetIvlRecBound( ts_in, recStart, recEnd, tint->start(), tint->end());
    if( OK )
	{
	    QdTimeSeq_var ts_out =new QdTimeSeq(recEnd - recStart +1);
        for(int i=recStart, j=0; i<= recEnd; i++, j++) ts_out[j] = ts_in->elt(i);

	    return ts_out;
	}
   
	QdTimeSeq_var nilObj;
    return nilObj;
}


double get_time_spacing(const QdObject_var& obj)
{
    double spacing;
    if(obj.is_nil()) return  -1.0;

    QdTimeSeq_var ts=get_timetags(obj);
    if(ts.is_nil()) return -1.0;
    if(ts->sequence_size() < 2) spacing = -1;
    else{
      spacing = fabs(ts[1].getDiffSecs(ts[0]));
    }
    for(int i=2; i<ts->sequence_size(); i++){
      double next = fabs(ts[i].getDiffSecs(ts[i-1]));
      if( next < spacing ) spacing = next;
    }
    // data glitches can give spacing too small

    return spacing;
}

double get_scalar_spacing(const QdObject_var& obj)
{
    double spacing;
    if(obj.is_nil()) return  -1.0;

	QdObject_var D0 = get_Depend_0(obj);
    QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(D0);
    if(ss.is_nil()) return -1.0;
	
    if(ss->sequence_size() < 2) spacing = -1;
    else{
      spacing = fabs(ss[1] - ss[0]);
    }
    for(int i=2; i<ss->sequence_size(); i++){
      double next = fabs(ss[i] - ss[i-1]);
      if( next < spacing ) spacing = next;
    }
    // data glitches can give spacing too small
    return spacing;
}

void set_timetags(QdObject_var obj, QdTimeSeq_var ts)
{
    obj->change_xref(DEPEND_0,ts);
}


int num_timetags_inrange(const QdTimeSeq_var& ts, const QdTimeInterval_var& tint)
{
    if(ts.is_nil() || tint.is_nil() ) return 0;
    int recStart, recEnd;
	
    if( QuGetIvlRecBound( ts, recStart, recEnd, tint->start(), tint->end())  ) return recEnd - recStart + 1;
    else return 0;

}

bool is_empty(const QdObject_var& obj)
{
    // if var pointer is nil then empty
    if(obj.is_nil())
        return true;

    // if a sequence then empty if sequence_size() is zero
    if(obj->is_sequence())
        return (obj->sequence_size()==0);

    // time interval ... empty if start=end time
    QdTimeInterval_var timeinterval_var = QdTimeInterval_var::narrow( obj );
    if( !timeinterval_var.is_nil() )
        return timeinterval_var->empty();

    // ..otherwise if a single object, always set up with a default value
    // so "not empty"
    return false;
}

 /***********************************************************************/

/// \fn  dimTripleStack getDimTripleStack(QString &compTxt)
/// \brief Method that parses text in QguLineEdit and finds the array reduction requested
/// \param     QString
/// \exception None
/// \return    dimTripleStack

dimTripleStack  getDimTripleStack(QString &compTxt){

  dimTripleStack arrayTriples;
  int ptr = compTxt.indexOf("[");
   compTxt.remove(0, ptr+1); // remove "[" 

  bool notEmpty = true;
   while (notEmpty){
      dimTriple nextDim = getDimTriple(compTxt);
	 if (nextDim.size() == 3) arrayTriples.push_back(nextDim);
	 else notEmpty = false;
   }

   return arrayTriples;
 }


/// \fn  dimTriple * getDimTriple(QString &compTxt)
/// \brief Method that parses text in QguLineEdit and finds the array reduction requested
/// \param     QString
/// \exception None
/// \return    dimTriple *

dimTriple  getDimTriple(QString &compTxt){

   dimTriple newTriple;
   int ptr;

   // get the next dimension only and cut from input str
   ptr = compTxt.indexOf(")");
   if(ptr == -1) return newTriple; // badly formed, newTriple will have size() == 1

   QString thisDim =  QString(compTxt.unicode(), ptr); // deep copy of first n=ptr characters
   compTxt.remove(0, ptr+1); // remove ")" for next call

   ptr = thisDim.indexOf("Sum");
   int option;
   if(ptr != -1){
     option = 1;   // sum
     thisDim.remove(0, ptr+3);
   }
   else {
	  ptr = thisDim.indexOf("Ave");
	  if(ptr != -1){
        option = 3;   // ave
        thisDim.remove(0, ptr+3);
      }
	  else {
	  	ptr = thisDim.indexOf("AtT");
	  	if(ptr != -1){
        	option = 4;   // alias to time
        	thisDim.remove(0, ptr+3);
      	}
	  	else option = 0;  // elements (or slice done below)
	  }
   }

   ptr = thisDim.indexOf("(");
   if(ptr == -1) return newTriple; // no more dimensions, newTriple will have size() == 1

   thisDim.remove(0, ptr+1); // remove " ("

   ptr = thisDim.indexOf(",");
   int start;
   int stop;
   bool ok;
   if (ptr == -1) {
     // one index, slice requested
     start = thisDim.toInt(&ok);
	 if(ok){
	 stop = start;
	 option = 2;
	   newTriple.push_back(option);
	   newTriple.push_back(start);
	   newTriple.push_back(stop);
     }
   }
   else{
     QString startStr = QString(thisDim.unicode(), ptr); // deep copy of first n=ptr characters
	 thisDim.remove(0, ptr+1); // remove ")" for next call

	 newTriple.push_back(option);
     start = startStr.toInt(&ok);
	 if(ok){
	   newTriple.push_back(start);
     }

     stop = thisDim.toInt(&ok);
	 if(ok){
	   newTriple.push_back(stop);
     }

   }

   return newTriple;

}

/// \fn vector <dimTriple *> getSliceTriple()
/// \brief Method that parses text in QguLineEdit and finds the time slice requested
/// \param     None
/// \exception None
/// \return    vector <dimTriple *>

dimTriple getSliceTriple(const QString &Txt)
{
   if ( QuSlotDebug() )
	printf( "getSliceTriple this = %s \n", Txt.toLatin1().data() );

   dimTriple sliceTriple;
   int option = 0; // default is subset
   int start;
   int stop;
   QString compTxt = QString( Txt); // deep copy
   int ptr = compTxt.indexOf(" <");
   if(ptr == -1) return sliceTriple;  // no T slice section found, sliceTriple is empty vector

   compTxt.remove(0, ptr+2); // remove " <"
   ptr = compTxt.indexOf(">");
   compTxt.truncate(ptr); // remove ">"


   ptr = compTxt.indexOf("Ave");
   if(ptr != -1) {
     option = 1;   // average
     compTxt.remove(0, ptr+3);
   }

   ptr = compTxt.indexOf(",");

   bool ok;
   if (ptr == -1) {
      // one index, slice requested
	  start = compTxt.toInt(&ok);
	  if(ok){
	    stop = start;
	    option = 2;
	    sliceTriple.push_back(option);
	    sliceTriple.push_back(start);
	    sliceTriple.push_back(stop);
	  }
   }
   else{
	 sliceTriple.push_back(option);

     QString startStr = QString(compTxt.unicode(), ptr); // deep copy of first n=ptr characters
	 compTxt.remove(0, ptr+1); // remove "," for next call

     start = startStr.toInt(&ok);
	 if(ok){
        sliceTriple.push_back(start);
	 }

     stop = compTxt.toInt(&ok);
	 if(ok){
	    sliceTriple.push_back(stop);
	 }
  }
  return sliceTriple;
 }

QString compText(QString objTxt)
{
   QString compTxt(objTxt); // deep copy
   int compStartAt = compTxt.indexOf(" [");
   int sliceStartAt = compTxt.indexOf(" <");
   int startAt=-1;
   if(compStartAt>=0 && sliceStartAt>=0)
     startAt=std::min(compStartAt,sliceStartAt);
   else if(compStartAt>=0)
      startAt=compStartAt;
   else if(sliceStartAt>=0)
     startAt=sliceStartAt;
   if(startAt != -1)
     compTxt.remove(0,startAt);
   else
     compTxt.remove(0,compTxt.length());

   return compTxt;
}

QString compText(QString objTxt, const dimTriple& slice_triple)
{
  QString compTxt(objTxt);
  int sliceStartAt = compTxt.indexOf(" <");
  if(sliceStartAt!=-1)
  {
    int sliceEndAt=compTxt.indexOf(">");
    compTxt.remove(sliceStartAt,1+sliceEndAt-sliceStartAt);
  }
  compTxt+=" < ";
  if(slice_triple[0]==1)
    compTxt+="Ave ";
  compTxt+=QChar(slice_triple[1]);
  if(slice_triple[1]!=slice_triple[2])
  {
      compTxt+=",";
      compTxt+=QChar(slice_triple[2]);
  }
  compTxt+=" >";
  return compTxt;
}

/// \fn QString getXrefName(QString objTxt)
/// \brief Method that returns the xref name from the text in the input slot
/// \param     objTxt of type QString
/// \exception None
/// \return    text of type QString
///

QString getXrefName(QString objTxt)
{
   QString txt = QString(objTxt); // deep copy
   int ptr = txt.indexOf("[=>");
   if (ptr == -1) return QString("");
   txt.remove(0, ptr+4);
   ptr = txt.indexOf(" ]");
   txt.truncate(ptr);
   return txt;
}

/// \fn QString baseObjectText()
/// \brief Method that returns the base Object text with component text removed
/// \param     None
/// \exception None
/// \return    text of type QString
///
QString baseObjectText(QString objTxt)
{

   QString baseTxt(objTxt);
   int endAt = baseTxt.indexOf(" [");
   if(endAt != -1) baseTxt.truncate(endAt);

	endAt = baseTxt.indexOf(" <");
	if(endAt != -1) baseTxt.truncate(endAt);
   
   return baseTxt;
}

/// \fn QguComp compEntry(const char* cmpTxt, const char* cmpType)
/// \brief Method that returns the component entry in the dropdown box from the type of component and the component text
/// \param     cmpTxt of type const char*
/// \param     cmpType of type const char*
/// \exception None
/// \return    comp of type QguComp
///

QguComp compEntry(const char* cmpTxt, const char* cmpType)
{
  QguComp comp=UNKNOWN_COMP;
  if(strcmp(cmpType,"VECTOR") ==0)
  {
    if(strcmp(cmpTxt,R_STR_CMP)==0)
      comp=R_COMP;
    else if(strcmp(cmpTxt,X_STR_CMP)==0)
      comp=X_COMP;
    else if(strcmp(cmpTxt,Y_STR_CMP)==0)
      comp=Y_COMP;
    else if(strcmp(cmpTxt,Z_STR_CMP)==0)
      comp=Z_COMP;
    else if(strcmp(cmpTxt,R_STR_CMP)==0)
      comp=R_COMP;
    else if(strcmp(cmpTxt,Ld_STR_CMP)==0)
      comp=LAT_DEG_COMP;
    else if(strcmp(cmpTxt,Td_STR_CMP)==0)
      comp=THETA_DEG_COMP;
    else if(strcmp(cmpTxt,Pd_STR_CMP)==0)
      comp=PHI_DEG_COMP;
    else if(strcmp(cmpTxt,Lr_STR_CMP)==0)
      comp=LAT_RAD_COMP;
    else if(strcmp(cmpTxt,Tr_STR_CMP)==0)
      comp=THETA_RAD_COMP;
    else if(strcmp(cmpTxt,Pr_STR_CMP)==0)
      comp=PHI_RAD_COMP;
    else
      comp=UNKNOWN_COMP;
  }
  else if(strcmp(cmpType,"TIME") ==0)
  {
    if (strcmp(cmpTxt,T_STR_START)==0)
      comp=T_START_COMP;
    else if (strcmp(cmpTxt,T_STR_CENTRE)==0)
       comp=T_CENTRE_COMP;
    else if (strcmp(cmpTxt,T_STR_END)==0)
      comp=T_END_COMP;
    else
      comp=UNKNOWN_COMP;
  }
  else if(strcmp(cmpType,"ARRAY") ==0)
    comp=ARRAY_COMP;
  else if(strcmp(cmpType,"XREF")==0)
    comp=XREF_OBJ;
  else if(strcmp(cmpType,"SLICE") ==0)
    comp=SLICE_COMP;
  else
    comp=UNKNOWN_COMP;

  return comp;
}

/// \fn QguComp compEntry(const char* cmpTxt)
/// \brief Method that returns the component entry in the dropdown box from the component text
/// \param     cmpTxt of type const char*
/// \exception None
/// \return    comp of type QguComp
///

QguComp compEntry(const char* cmpTxt)
{
  QguComp comp=UNKNOWN_COMP;
  if(strcmp(cmpTxt,R_STR_CMP)==0)
    comp=R_COMP;
  else if(strcmp(cmpTxt,X_STR_CMP)==0)
    comp=X_COMP;
  else if(strcmp(cmpTxt,Y_STR_CMP)==0)
    comp=Y_COMP;
  else if(strcmp(cmpTxt,Z_STR_CMP)==0)
    comp=Z_COMP;
  else if(strcmp(cmpTxt,R_STR_CMP)==0)
    comp=R_COMP;
  else if(strcmp(cmpTxt,Ld_STR_CMP)==0)
    comp=LAT_DEG_COMP;
  else if(strcmp(cmpTxt,Td_STR_CMP)==0)
    comp=THETA_DEG_COMP;
  else if(strcmp(cmpTxt,Pd_STR_CMP)==0)
    comp=PHI_DEG_COMP;
  else if(strcmp(cmpTxt,Lr_STR_CMP)==0)
    comp=LAT_RAD_COMP;
  else if(strcmp(cmpTxt,Tr_STR_CMP)==0)
    comp=THETA_RAD_COMP;
  else if(strcmp(cmpTxt,Pr_STR_CMP)==0)
    comp=PHI_RAD_COMP;
  else if (strcmp(cmpTxt,T_STR_START)==0)
    comp=T_START_COMP;
  else if (strcmp(cmpTxt,T_STR_CENTRE)==0)
   comp=T_CENTRE_COMP;
  else if (strcmp(cmpTxt,T_STR_END)==0)
    comp=T_END_COMP;
  else if (strncmp(cmpTxt, " [=>",4) == 0)
    comp=XREF_OBJ;
  else if (strncmp(cmpTxt, " [",2)==0)
    comp=ARRAY_COMP;
  else if (strncmp(cmpTxt, " <",2)==0)
    comp=SLICE_COMP;
  else
    comp=WL_OBJ;
  return comp;
}

QdObject_var QuSetCompFromObjText(const QString& from, QString& to)
{
  // Uses component and time slice of from object to get comp and slice of to obj
  // Care is taken since the slice is by record, so must be converted to an interval
   if ( QuSlotDebug() )
	printf( "QuSetCompFromObjText from = %s to = %s\n", from.toLatin1().data(), to.toLatin1().data() );

  // get from object
  QdObject_var nil_obj;
  QString to_obj_txt=baseObjectText(to);
  QString from_obj_txt=baseObjectText(from);

  // Check this isn't just the same object and selection
  if(to_obj_txt == from_obj_txt) 
  {
     to = from; // reset slot in plot
     return QuGetObjByNameAndComp(from);
  }
  // get 'to' object
  QdObject_var to_obj=QuGetObjByName(to_obj_txt);
  if(to_obj.is_nil())  return nil_obj;
  
  // get component string from from_obj
  QString from_comp_str=compText(from);

  // get slice info from from_obj
  dimTriple slice_trip=getSliceTriple(from);
  // do we have slice info
  bool newSlice = false;
  if(slice_trip.size()==3)
  {
     // slice 'to' object
     QdObject_var from_obj = QuGetObjByName(from_obj_txt);
	
     if( !from_obj.is_nil() ) {
     
       // find time interval from slice and from_object
	   QdTimeInterval_var tint=QuGetIvlFromSlice(from_obj,slice_trip);
       if( !tint.is_nil() )
	   {
         // update slice for to_object using time interval
	 	 if(QuGetSliceFromIvl(to_obj, tint, slice_trip))
	         newSlice = true; // safe to replace time slice (it exists)
         }
	  }
   }
   
   // update component text with new slice info
   if( newSlice )
   {
      to_obj_txt += compText(from_comp_str, slice_trip);
   }
   else{
      // remove any slice info - cannot apply it
      int cutAt = from_comp_str.indexOf(" <");
	  if (cutAt != -1 ) to_obj_txt += from_comp_str.left(cutAt); // time, < >, is always to rt of comp, [ ]
      to_obj_txt += from_comp_str;
   }
   // try and fetch this object
   QdObject_var obj=QuGetObjByNameAndComp(to_obj_txt);
   // if nil then not a valid object so return to unchanged
   if(obj.is_nil())
     return QuGetObjByNameAndComp(to);
 
   // update 'to' and new object
   to=to_obj_txt;
   return obj;
  
}

// -----------------------------------------------------------------

QdTimeInterval_var QuGetIvlFromSlice(QdObject_var obj, const dimTriple& slice_trip)
{
   QdTimeInterval_var tint;
   if(obj.is_nil())
     return tint;
  // fetch ttags from from_obj
  QdTimeSeq_var tt = get_timetags(obj);
  if(tt.is_nil())
    return tint;
  // ensure time slice/subset
  if(slice_trip.size()!=3)
    return tint;
  // fetch from and to times
  if( tt->sequence_size() > slice_trip[2])
  {
    tint = new QdTimeInterval(tt[slice_trip[1]], tt[slice_trip[2]] );
    return tint;
  }
  else
    return tint;
}

// -----------------------------------------------------------------

bool QuGetSliceFromIvl(QdObject_var obj,QdTimeInterval_var tint, dimTriple& slice_trip)
{
  if(obj.is_nil()) return false;
  
  // fetch ttags from obj
  QdTimeSeq_var tt = get_timetags(obj);

  if(tt.is_nil()) return false;
  
  // ensure it is a time slice/subset
  if(slice_trip.size()!=3) return false;

  bool found_ivl=false;
  int rec_start,rec_end;
  // find bounding records in tt for these start and end times
  found_ivl=QuGetIvlRecBound(tt,rec_start,rec_end,tint->start(),tint->end());

  if(found_ivl)
  {
    slice_trip[1]=rec_start;
    slice_trip[2]=rec_end;
  }
  return found_ivl;
}

// ---------------------------------------------------

QdObject_var QuGetObjByNameAndComp(const QString& name)
{  
    if ( QuSlotDebug() )
	{
		printf( "Utility: QuGetObjByNameAndComp '%s'\n", name.toLatin1().data() );
		fflush(stdout);
	}

  QdObject_var retObj;

  QdObject_var baseObj;
  if(name.isNull() || name.isEmpty())  return retObj;

  QString baseObjStr;
  int compAt = name.indexOf(" [");
  int timeAt = name.indexOf(" <"); 
  if(compAt == -1 && timeAt == -1) baseObjStr = name; 
  else if(compAt != -1) {
  	int endAt = name.indexOf("]");
	if(endAt == -1) return retObj; // incomplete component
	
  	baseObjStr = name.left(compAt);
  }
  else {
  	int endAt = name.indexOf(">");
	if(endAt == -1) return retObj; // incomplete t component
	
  	baseObjStr = name.left(timeAt);
  }
  
  Name baseObjName(baseObjStr.toLatin1().data());
  try
  {
     // if name is not found then resolve throws a NameNotFound exception
     baseObj=QDOSlist->resolve(baseObjName);
  }
  catch(Exception &e)
  {
	 return retObj;
  }
  
  retObj = baseObj; // default for no component processed

  if(compAt != -1)
  {  
    QString compStr=QString(name);
	compStr.remove(0,compAt);
	
    if (compStr.contains("(mag)") ) 
      retObj = QuGetVectorComponent(baseObj, "r(mag)" );
    else if (compStr.contains("x") )
      retObj = QuGetVectorComponent(baseObj, "x" );
    else if (compStr.contains("y") )
      retObj = QuGetVectorComponent(baseObj, "y" );
    else if (compStr.contains("z") )
      retObj = QuGetVectorComponent(baseObj, "z" );
    else if (compStr.contains("Lat")  && compStr.contains("deg") )
      retObj = QuGetVectorComponent(baseObj, "Lat(deg)" );
    else if (compStr.contains("Lat")  && compStr.contains("rad") )
      retObj = QuGetVectorComponent(baseObj, "Lat(rad)" );
    else if (compStr.contains("theta")  && compStr.contains("deg") )
      retObj = QuGetVectorComponent(baseObj, "theta(deg)" );
    else if (compStr.contains("theta")  && compStr.contains("rad") )
      retObj = QuGetVectorComponent(baseObj, "theta(rad)" );
    else if (compStr.contains("phi")  && compStr.contains("deg") )
      retObj = QuGetVectorComponent(baseObj, "phi(deg)" );
    else if (compStr.contains("phi")  && compStr.contains("rad") )
      retObj = QuGetVectorComponent(baseObj, "phi(rad)" );
    else if (compStr.contains("Start") )
      retObj = QuGetTimeComponent(baseObj, "Tstart" );
    else if (compStr.contains("Centre") )
      retObj = QuGetTimeComponent(baseObj, "Tcentre" );
    else if (compStr.contains("End") )
      retObj = QuGetTimeComponent(baseObj, "Tend" );
    else if (compStr.contains("[=>") )
    {
       QString xref_name=getXrefName(compStr);
       try{
         QdObject_var xrefObj = baseObj->get_xref(xref_name.toLatin1().data());
         retObj = xrefObj;
       }
       catch(Exception &e){
         QuMsgBar("Xref not found: ", xref_name.toLatin1().data(), true);
		 QdObject_var nilObj;
		 return nilObj;
       }
    }
    else if (compStr.contains("[") > 0)
    {
	   string arrComp_xref = string(compStr.toLatin1().data());
	   arrComp_xref[0] = '_';
//       compStr.remove(0,2);
       dimTripleStack dimStack=getDimTripleStack(compStr);
	   if(dimStack.size() < 1) return retObj; // empty, this component not understood
       retObj = QuGetArrayComponent(baseObj, dimStack, arrComp_xref );
    }

 }
 if(timeAt != -1) 
 { 
    // see if this exact object already exists (comp and time slice)
	QString tCompStr = QString(name);
	tCompStr.remove(0, timeAt);
	string tComp = string(tCompStr.toLatin1().data());
	tComp[0] = '_';

	// Use cache if found
    if(retObj->xref_exists(tComp.c_str())) return retObj->get_xref(tComp.c_str());
	
    // now subset on time slice if relevant
    dimTriple sliceTriple = getSliceTriple(tCompStr);

    if(sliceTriple.size() == 3){
      vector <string> objNameList;
      objNameList.push_back(baseObjName.srep());
	  QdObject_var slicedObj = QuTakeSlice(objNameList, retObj, sliceTriple, false);
	  retObj->change_xref(tComp.c_str(), slicedObj);
	  retObj = slicedObj;
    }
 }

 return retObj;
}


// ------------------------------------------------

QdObject_var QuGetTimeByNameAndComp(const QString& name)
{  
    if ( QuSlotDebug() )
	{
		printf( "Utility: QuGetTimeByNameAndComp %s\n", name.toLatin1().data() );
		fflush(stdout);
	}

	QdObject_var retObj;

	if(name.isEmpty())  return retObj;
	QString baseObjStr = baseObjectText(name);
	int compAt = name.indexOf(" [");
	int timeAt = name.indexOf(" <"); 
	
	bool isEvent = false;
	QdObject_var baseObj = QuGetObjByName(baseObjStr);

	// if Time component selected, get that
	if(compAt != -1)
	{  
		QdTimeIntervalSeq_var ET = QdTimeIntervalSeq_var::narrow(baseObj);
		if(!ET.is_nil()){
			// Event list selected event
			if(timeAt != -1) 
			{ 
				bool ok;
				QString eventID = QString(name);
				eventID.remove(0, timeAt+2);
				int endAt = eventID.indexOf(">");
				eventID.truncate(endAt);
				
				int i = eventID.toInt(&ok);
				if(ok){
					QdTimeInterval_var event = new QdTimeInterval(ET[i]);
					baseObj = event;
					isEvent = true;
				}
			}
		}
		
		QString compStr=QString(name);
		compStr.remove(0,compAt);
	
		if (compStr.contains(T_STR_START) > 0)
			retObj = QuGetTimeComponent(baseObj, "Tstart" );
		else if (compStr.contains(T_STR_CENTRE) > 0)
			retObj = QuGetTimeComponent(baseObj, "Tcentre" );
		else if (compStr.contains(T_STR_END) > 0)
			retObj = QuGetTimeComponent(baseObj, "Tend" );
		else if (compStr.contains("[=>") > 0)
		{
			QString xref_name=getXrefName(compStr);
			try{
				QdObject_var xrefObj = baseObj->get_xref(xref_name.toLatin1().data());
				retObj = xrefObj;
			}
			catch(Exception &e){
				QuMsgBar("Xref not found: ", xref_name.toLatin1().data(), true);
			}
		}

		
		if( !retObj.is_nil()) return retObj;
	}
 
  
	// is this a time object
	QdTime_var tobj = QdTime_var::narrow(baseObj);
	if( !tobj.is_nil() )
	{
		retObj = tobj;
		return retObj;
	}
	
	// get time sequence object (this gets self or tt)
	QdTimeSeq_var tt = get_timetags(baseObj); // default for no component processed
	retObj = tt;
	if(retObj.is_nil()) return retObj;
	
	// subsample if requested
	if(timeAt != -1 && !isEvent) 
	{ 
		// see if this exact object already exists (comp and time slice)
		QString tCompStr = QString(name);
		tCompStr.remove(0, timeAt);
		string tComp = string(tCompStr.toLatin1().data());
		tComp[0] = '_';

		// Use cache if found
		if(retObj->xref_exists(tComp.c_str())) return retObj->get_xref(tComp.c_str());
	
		// else subset on time choice if relevant
		dimTriple sliceTriple = getSliceTriple(tCompStr);

		if(sliceTriple.size() == 3){
		
			if(sliceTriple[0] == 0 || sliceTriple[0] == 1){

				int record = sliceTriple[1];
				int recCount = sliceTriple[2] - record + 1;
				QdTimeSeq_var ttNew = new QdTimeSeq();
				tt->get_subsequence(*ttNew, record, recCount);
				retObj = (QdObject_var) ttNew;
				tt->change_xref(tComp.c_str(), retObj);
			}
			else if(sliceTriple[0] == 2){

				int record = sliceTriple[1];

				QdTime_var tNew = new QdTime(tt[record]);
				retObj = (QdObject_var) tNew;
				tt->change_xref(tComp.c_str(), retObj);
			}		
			
		}
	}

	return retObj;
}


// ------------------------------------------------

QdObject_var QuGetIntervalsByNameAndComp(const QString& name)
{

	// initialise var pointer here (will be nil if resolve fails)
	QdObject_var baseObj;
	if(name.isNull() || name.isEmpty())  return baseObj;
 
	QdObject_var retObj;
 
	QString baseObjStr;
	int compAt = name.indexOf(" [");
	int timeAt = name.indexOf(" <"); 
	if(compAt == -1 && timeAt == -1) baseObjStr = name; 
	else if(compAt != -1) baseObjStr = name.left(compAt);
	else baseObjStr = name.left(timeAt);
 

	Name baseObjName(baseObjStr.toLatin1().data());

	try
	{
		// if name is not found then resolve throws a NameNotFound exception
		baseObj=QDOSlist->resolve(baseObjName);
	}
	catch(Exception &e)
	{
		cout << "Cannot find base interval object named " << baseObjStr.toLatin1().data() << endl;
		return retObj;
	}

	// Get TI object or TI from event table
	QdTimeInterval_var Ti = QdTimeInterval_var::narrow(baseObj);
	if( !Ti.is_nil()) return Ti;
	
	QdTimeIntervalSeq_var Tis = QdTimeIntervalSeq_var::narrow(baseObj);
	if( !Tis.is_nil()) 
	{

		if(timeAt == -1) return Tis; //  TI seq
		
		QString ivlStr=QString(name);
		ivlStr.remove(0,timeAt+2); 
		int terminator = ivlStr.indexOf(">");
		ivlStr.truncate(terminator);
		bool OK;
		int nEvent = ivlStr.toInt(&OK);
		QdTimeInterval_var Ti;
		if(OK && nEvent < Tis->sequence_size()) Ti= new QdTimeInterval(Tis[nEvent]);
		return Ti;
	}
	

	// Get interval from Data Object with time tags (possibly sliced on time)
	string ivl_xref;
	if( timeAt != -1) 
	{
		QString ivlStr=QString(name);
		ivlStr.remove(0,timeAt); 
		ivl_xref = "_ivl" + string(ivlStr.toLatin1().data());
	}
	else ivl_xref = "_ivl";
	
  
	if(baseObj->xref_exists(ivl_xref)) return baseObj->get_xref(ivl_xref);
  
	// Apply time slice if relevant
	if(ivl_xref != "_ivl") 
	{
		dimTriple sliceTriple = getSliceTriple(name);
		if(sliceTriple.size() == 3){
			vector <string> objNameList;
			objNameList.push_back(baseObjName.srep());
			retObj = QuTakeSlice(objNameList, baseObj, sliceTriple, true); // true returns just the interval
	  
			baseObj->change_xref(ivl_xref, retObj);
			return retObj;
		}
	}
  
	// else get ivl of base object
	retObj = get_timeinterval(baseObj);

	baseObj->change_xref(ivl_xref, retObj);
	
	return retObj;
}

 /***********************************************************************/

QdObject_var QuTakeSlice(vector <string> dataObjectNames, QdObject_var dobj, dimTriple sliceTriple, bool getInterval){

 // initialise var pointer here (will be nil if resolve fails)
 QdObject_var retObject;
 
 if(dobj.is_nil())    return retObject;
 
 QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(dobj);
 QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(dobj);
 QdTimeSeq_var    ts = QdTimeSeq_var::narrow(dobj);

    // subsample of sequence chosen
	if(QuSlotDebug())   cout << "QuTakeSlice: SUBSAMPLE " << sliceTriple[0] <<" "<< sliceTriple[1] <<" "<< sliceTriple[2] << endl;

   if(sliceTriple[0] == 0 && getInterval){
      // Return interval only
	  // ------------------------
      if(QuSlotDebug())	  cout << "QuTakeSlice: SUBSET INTERVAL" << endl;

	  int recStart = sliceTriple[1];
	  int recEnd = sliceTriple[2];
      QdTimeSeq_var tt = get_timetags(dobj);
	  if( !tt.is_nil() && tt->sequence_size() > recEnd){
	         QdTimeInterval_var ti = new QdTimeInterval(tt[recStart], tt[recEnd] );

			 return (QdObject_var) ti;
	  }
      else{
	        // not scalar or matrix sequence
	        QuMsgBar("Time Slice failed", false);
	        return dobj;
      }

   }
   else  if(sliceTriple[0] == 0){
      // Subset
	  // ------------------------
      if(QuSlotDebug()) cout << "QuTakeSlice: SUBSET" << endl;

	  int record = sliceTriple[1];
	  int recCount = sliceTriple[2] - record + 1;
	  if(recCount <= 0) return retObject; // will be is_nil

	  if( !ss.is_nil() && ss->sequence_size() > sliceTriple[2]){

	     QdRScalarSeq_var retScalar = new QdRScalarSeq();
	     ss->get_subsequence(*retScalar, record, recCount);
	     retObject = (QdObject_var) retScalar;
	  }
	  else if( !ms.is_nil() && ms->sequence_size() > sliceTriple[2]) {

	         QdRMatrixSeq_var retMatrix = new QdRMatrixSeq();
	   	     ms->get_subsequence(*retMatrix, record, recCount);
	         retObject = (QdObject_var) retMatrix;
	  }
	  else if( !ts.is_nil() && ts->sequence_size() > sliceTriple[2]) {

	         QdTimeSeq_var ttNew = new QdTimeSeq();
	   	     ts->get_subsequence(*ttNew, record, recCount);
	         retObject = (QdObject_var) ttNew;
	  }
      else{
	        // not scalar or matrix sequence
	        QuMsgBar("Time Slice failed", false);
	        return dobj;
      }
      
   }
   else if(sliceTriple[0] == 1){
      // Average
	  //-------------------------
	  if(QuSlotDebug()) cout << "QuTakeSlice: AVE" << endl;

	  int recStart = sliceTriple[1];
	  int recEnd = sliceTriple[2] ;
	  
	  if(recEnd < recStart) return retObject; // will be is_nil

	  if( !ss.is_nil() && ss->sequence_size() > recEnd){

	     QdRScalarSeq_var retScalar = new QdRScalarSeq();
	     double aveScalar=0;
	     for(int i=recStart; i<=recEnd; i++) aveScalar += ss[i];
	     aveScalar= aveScalar/(recEnd - recStart + 1);
		 retScalar->push_back( QdRScalar(aveScalar) );
	     retObject = (QdObject_var) retScalar;
	  }
	  else if( !ms.is_nil() && ms->sequence_size() > recEnd) {

		  QdRMatrixSeq_var retMatrix = new QdRMatrixSeq();
	      QdRMatrix_var aveMatrix = new QdRMatrix(ms[recStart]);
		  for(int i=recStart+1; i<=recEnd; i++) *aveMatrix += ms[i];
		  *aveMatrix = *aveMatrix/(recEnd - recStart + 1);
		  retMatrix->push_back(*aveMatrix);
	      retObject = (QdObject_var) retMatrix;
	  }
	  else if( !ts.is_nil() && ts->sequence_size() > recEnd) {
           QdTime newTime;
		   newTime.setFromMJD((ts[recStart].getMJD() + ts[recEnd].getMJD())/2.0);
		   QdTimeSeq_var ttNew = new QdTimeSeq();
		   ttNew->push_back(newTime);
		   retObject = (QdObject_var) ttNew;
	  }
	  else{
	        // not scalar or matrix sequence
	        QuMsgBar("Time Slice failed", false);
	        return dobj;
      }
   }
    else if(sliceTriple[0] == 2){
      // Slice
	  //--------------------------

      if(QuSlotDebug()) cout << "QuTakeSlice: SLICE" << sliceTriple[1] << endl;

	  int record = sliceTriple[1];
	  if( !ss.is_nil() && ss->sequence_size() > record){

	     QdRScalarSeq_var retScalar = new QdRScalarSeq();
	     ss->get_subsequence(*retScalar, record, 1);
	     retObject = (QdObject_var) retScalar;
	  }
	  else if( !ms.is_nil() && ms->sequence_size() > record) {

	         QdRMatrixSeq_var retMatrix = new QdRMatrixSeq();
	   	     ms->get_subsequence(*retMatrix, record, 1);
	         retObject = (QdObject_var) retMatrix;
	  }
	  else if( !ts.is_nil() && ts->sequence_size() > record) {

	         QdTimeSeq_var ttNew = new QdTimeSeq();
	   	     ts->get_subsequence(*ttNew, record, 1);
	         retObject = (QdObject_var) ttNew;
	  }
	  else{
	        // not scalar or matrix sequence
	        QuMsgBar("Time Slice failed", false);
			return dobj;
      }

   }
   else{
      QuMsgBar("Time Slice failed", false);
      return dobj;
   }

	// slice xrefs with same sequence size (if sequence_size > 1)
	// This also slices Time Tags
    vector<string> sl;
	dobj->list_xref_names(sl);
	for(int i=0; i<sl.size(); i++)
	{
	    for(int j=0; j<dataObjectNames.size(); j++){
	       if(sl[i] == dataObjectNames[j]) {
		      sl[i][0] = CACHE_XREF_CHAR; // break infinite loop if circularity returns to an ancestor
			  break;
		   }
		}
		if(sl[i][0]==CACHE_XREF_CHAR) continue;
		QdObject_var xref_obj = dobj->get_xref(sl[i]);
		if( xref_obj->sequence_size() > 1 &&  xref_obj->sequence_size() == dobj->sequence_size() ){
		    dataObjectNames.push_back(sl[i]);
			if(QuSlotDebug()) cout<< " slice Xref " << sl[i].c_str() << endl;
			QdObject_var sliced_xref_obj = QuTakeSlice(dataObjectNames, xref_obj, sliceTriple, false);
			retObject->change_xref(sl[i], sliced_xref_obj);
		}
		else retObject->change_xref(sl[i], xref_obj);
	}

    // Add new xref to identify slice etc
    // Add new xref to identify slice etc
	QdObject_var d0 = get_Depend_0(dobj);
	QdTimeSeq_var tt_in = QdTimeSeq_var::narrow(d0);
	QdRScalarSeq_var ss_in = QdRScalarSeq_var::narrow(d0);
  	
	if( !tt_in.is_nil()){
		QdString_var timeSliceAttr;

		if(sliceTriple[0] == 0){
			timeSliceAttr = new QdString(tt_in[sliceTriple[1]].iso_srep() + " to " + tt_in[sliceTriple[2]].iso_srep());
		}
		else  if(sliceTriple[0] == 1){
			timeSliceAttr = new QdString("Ave " + tt_in[sliceTriple[1]].iso_srep() + " to " + tt_in[sliceTriple[2]].iso_srep());
			
		}
		else  if(sliceTriple[0] == 2){
			timeSliceAttr = new QdString("At " + tt_in[sliceTriple[1]].iso_srep());
		}
	
	 	retObject->change_xref("Time_Slice", timeSliceAttr);
	}
	else if(!ss_in.is_nil()){
		QdString_var depSliceAttr;

		if(sliceTriple[0] == 0){
			depSliceAttr = new QdString(double_to_string(ss_in[sliceTriple[1]]) + " to " + double_to_string(ss_in[sliceTriple[2]]));
		}
		else  if(sliceTriple[0] == 1){
			depSliceAttr = new QdString("Ave " + double_to_string(ss_in[sliceTriple[1]]) + " to " + double_to_string(ss_in[sliceTriple[2]]));

			QdObject_var d0ret = get_Depend_0(retObject);
			
			QdRScalar_var delta = new QdRScalar((ss_in[sliceTriple[2]] - ss_in[sliceTriple[1]]) * 0.5);
			d0ret->change_xref(DELTA_PLUS, delta);
			d0ret->change_xref(DELTA_MINUS, delta);
			
		}
		else  if(sliceTriple[0] == 2){
			depSliceAttr = new QdString("At " + double_to_string(ss_in[sliceTriple[1]]));
		}
	
	 	retObject->change_xref("Record_Slice", depSliceAttr);
	}

	return retObject;

}

 /***********************************************************************/
 
 QdObject_var QuGetSubset(QdObject_var obj, QdTimeInterval_var tint){
 
      // Subset any ts object on interval
	  QdTimeSeq_var tt = get_timetags(obj);
	  if(tt.is_nil()) return obj;
	  
	  int start, end;
	  
	  bool OK = QuGetIvlRecBound(tt, start, end, tint->start(), tint->end());

	  if(!OK) return obj;
	  	  
	  QdObject_var retObj = QuGetSubset(obj, start, end);
	  
	  return retObj;
 
 }
  /***********************************************************************/

 
 QdObject_var QuGetSubset(QdObject_var ts_in, int recStart, int recEnd){
 
      // Subset any ts object on records
       
      if(QuSlotDebug()) cout << "QuGetSubset" << endl;
    
      int recCount = recEnd - recStart +1;

      // note that sub_sequence() call uses size of sequence to determine length of subseq
      
      QdTimeSeq_var ts = QdTimeSeq_var::narrow(ts_in);
      if( !ts.is_nil() && ts->sequence_size() > recEnd) {

	    QdTimeSeq_var ttNew = new QdTimeSeq();
	    ts->get_subsequence(*ttNew, recStart, recCount);
	    return (QdObject_var) ttNew;
      }

      QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(ts_in);     
      if( !ss.is_nil() && ss->sequence_size() > recEnd){

			QdRScalarSeq_var retScalar = new QdRScalarSeq();
			ss->get_subsequence(*retScalar, recStart, recCount);
			return (QdObject_var) retScalar;

      }
      
      QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(ts_in);
      if( !ms.is_nil() && ms->sequence_size() > recEnd) {


	   QdRMatrixSeq_var retMatrix = new QdRMatrixSeq();
	   ms->get_subsequence(*retMatrix, recStart, recCount );
	   return (QdObject_var) retMatrix;

      }
      
      QdObject_var nil_obj;
      return nil_obj;
   }

 /***********************************************************************/

QdObject_var QuGetObjByName(const QString &name){

	// initialise var pointer here (will be nil if resolve fails)
	QdObject_var obj;
	Name objName(name.toLatin1().data());


	try
	{
		// if name is not found then resolve throws a NameNotFound exception
		obj=QDOSlist->resolve(objName);
	}
	catch(Exception &e)
	{
		// do nothing
	}
	return obj;

}

/***********************************************************************/

void QuDeleteObject(const QString& name){

	try
	{
		QDOSlist->unbind(Name(name.toLatin1().data()));    
		if(!(name.startsWith("Temp/")) ) QuMsgBar(name.toLatin1().data(), "deleted", false);
		else if(QuSlotDebug()) 
		{
			cout << name.toLatin1().data() << " deleted " << endl;
			fflush(stdout);
		}
	}
	catch(Exception &e)
	{
		// fail silently
	}
	return;
}

/***********************************************************************/

void QuReplaceObject(const QString& name, QdObject_var new_obj){

  QdObject_var old_obj = QuGetObjByName(name);
  
  // safe against old object not existing
  if ( !old_obj.is_nil() ){
    try
    {
        QDOSlist->unbind(Name(name.toLatin1().data()));    
    }
    catch(Exception &e)
    {
        QuMsgBar( e.msg().c_str(), name.toLatin1().data(), true);
    }
  }
  
  try
  {
      QuSetObjByName(new_obj, name);
  }
  catch(Exception &e)
  {
      QuMsgBar( e.msg().c_str(), name.toLatin1().data(), true);
  }

  
 return;    

}

/***********************************************************************/

QdObject_var QuDuplicateObject(QString &name){

	QdObject_var duplicateObj;
	Name objName(name.toLatin1().data());
 
	try
	{
		// if name is not found then resolve throws a NameNotFound exception
		QdObject_var obj=QDOSlist->resolve(objName);
		duplicateObj = Factory::clone( (QdObject_var) obj);
	}
	catch(Exception &e)
	{
		QuMsgBar("duplicate failed:", name.toLatin1().data(), true);
	}
	return duplicateObj;
}

/***********************************************************************/

void QuRenameObject(QString oldName, QString newName){
  QdObject_var obj;
  
  try{
     obj = QuGetObjByName(oldName);
  }
  catch(Exception &e)
  {
     QuMsgBar(oldName.toLatin1().data(), "not found", true);
     return;
  }
 
  // unbind old name
  try{
      QDOSlist->unbind(Name(oldName.toLatin1().data()));    
  }
  catch(Exception &e)
  {
     QuMsgBar("cannot unbind", oldName.toLatin1().data(), true); 
     return;
  }  
    // bind new name 
  
  try{
      QDOSlist->bindx(Name(newName.toLatin1().data()), obj);    // and safe to replace
  }
  catch(Exception &e)
  {
     QuMsgBar("cannot bind", oldName.toLatin1().data(), true);
     return;
   }  
   QuSetOK();
}

//*********************************************************

bool QuSetObjByName(QdObject_var ToSetObj, QString name){
  QdString_var name_ptr;
  QdObject_var obj = ToSetObj;
  
  Name objName(name.toLatin1().data());

  // Find if objects are to be cleaned before putting on WL

  if ( checkBadData()->isChecked() || checkFills()->isChecked() ){
       obj = QuCleanObject(ToSetObj);
  }

  if( is_empty(obj) ) {
    QuMsgBar("Empty object: try unchecking Strip Fill Values", true);
    return false;
  }
  
  QdObject_var tmp = QuGetObjByName( name );
  bool name_found = !tmp.is_nil();

   if( name_found )
        QDOSlist->unbind( objName );  // remove binding if name already bound     
  
   try{
       QDOSlist->bindx(objName, obj);    // and safe to replace
   }
   catch(Exception &e)
   {
      e.print_msg();
      QuMsgBar( "QuSetObjByName: error binding ", name.toLatin1().data(), false);
      return false;
   }

   try{      
      int n = objName.num_components();
      name_ptr = new QdString(objName.get_component(n-1).get_id());
      obj->change_xref("ObjectName", name_ptr);
   }
   catch(Exception &e){
      QuMsgBar("QuSetObjByName: cannot add xref ", name_ptr->c_str(), true);
      return false;
   }
   try{     
		// catch old qsas created time tags
		if(obj->xref_exists("TIME_TAGS") && !obj->xref_exists(DEPEND_0))
		{
			QdObject_var tt = obj->get_xref("TIME_TAGS");
			if(!tt.is_nil()) obj->change_xref(DEPEND_0, tt);
		}
   }
   catch(Exception &e){
      QuMsgBar("QuSetObjByName: cannot convert TIME_TAGS to DEPEND_0  xref ", name_ptr->c_str(), true);
      return false;
   }
      
   return true;
   

}
/***********************************************************************/
void QuSetTxtAttr(const char *attrName, const char *value, QdObject_var obj)
{
   // create Xref DO
    try{
	 
      QdString_var DOx = new QdString(value);
    
// Add Xref called "attrName" to parent Object
// In this case DOx (the attribute) does NOT have a name of its own

      obj->change_xref(attrName, DOx);
    }
    catch(Exception &e){
      QuMsgBar("cannot set xref ", attrName, true);
    }

  return;

 } // end QuSetTextAttr


/***********************************************************************/
void applyMaskToXrefs(int oldSeqLen, QdObject_var ms_out, SequenceMask msk)
{
	QdObject_var d0 = get_Depend_0(ms_out);
	bool hasD0 = !d0.is_nil();
		  
	// Recursively fix sequence type xrefs 
	
	// Do DEPEND_0 first
		
	if(hasD0 && d0->sequence_size() == oldSeqLen) { // catches D0 already done

		QdRScalarSeq_var xref_ss = QdRScalarSeq_var::narrow(d0);
		QdTimeSeq_var xref_tt = QdTimeSeq_var::narrow(d0);
		if( !xref_ss.is_nil() ){
			QdRScalarSeq_var xref_i_out = new QdRScalarSeq;
			xref_ss->get_subsequence(*xref_i_out, msk);
			xref_i_out->copy_xrefs_from(xref_ss);
			if( xref_i_out->xref_exists(DEPEND_0) )	xref_i_out->delete_xref(DEPEND_0);

			d0 = xref_i_out;
			
			applyMaskToXrefs(oldSeqLen, d0, msk);
			ms_out->change_xref(DEPEND_0, d0);
		}
		else if( !xref_tt.is_nil() ){
			QdTimeSeq_var xref_i_out = new QdTimeSeq;
			xref_tt->get_subsequence(*xref_i_out, msk);
			xref_i_out->copy_xrefs_from(xref_tt);
			if( xref_i_out->xref_exists(DEPEND_0) )	xref_i_out->delete_xref(DEPEND_0);
			
			d0 = xref_i_out;
			
			applyMaskToXrefs(oldSeqLen, d0, msk);
			ms_out->change_xref(DEPEND_0, d0);
		}
	}

	vector<string> sl;
	ms_out->list_xref_names(sl);
	for(int i=0;i<sl.size();i++)
	{
		if(sl[i][0]==CACHE_XREF_CHAR) continue;
		if(sl[i]==DEPEND_0) continue;
		
		QdObject_var xref_i = ms_out->get_xref(sl[i]);
		
		if(xref_i.is_nil() || xref_i->sequence_size() != oldSeqLen) continue; 

		QdRScalarSeq_var xref_ss = QdRScalarSeq_var::narrow(xref_i);
		QdRMatrixSeq_var xref_ms = QdRMatrixSeq_var::narrow(xref_i);
		QdTimeSeq_var xref_tt = QdTimeSeq_var::narrow(xref_i);
		if( !xref_ss.is_nil() ){
			QdRScalarSeq_var xref_i_out = new QdRScalarSeq;
			xref_ss->get_subsequence(*xref_i_out, msk);
			xref_i_out->copy_xrefs_from(xref_ss);
			if( xref_i_out->xref_exists(DEPEND_0)) xref_i_out->delete_xref(DEPEND_0);
			
			applyMaskToXrefs(oldSeqLen, xref_i_out, msk);
			ms_out->change_xref(sl[i].c_str(), xref_i_out);
		}
		else if( !xref_tt.is_nil() ){
			QdTimeSeq_var xref_i_out = new QdTimeSeq;
			xref_tt->get_subsequence(*xref_i_out, msk);
			xref_i_out->copy_xrefs_from(xref_tt);
			if( xref_i_out->xref_exists(DEPEND_0)) xref_i_out->delete_xref(DEPEND_0);
			
			applyMaskToXrefs(oldSeqLen, xref_i_out, msk);
			ms_out->change_xref(sl[i].c_str(), xref_i_out);
		}
		else if( !xref_ms.is_nil() ){
			QdRMatrixSeq_var xref_i_out = new QdRMatrixSeq;
			xref_ms->get_subsequence(*xref_i_out, msk);
			xref_i_out->copy_xrefs_from(xref_ms);
			if( xref_i_out->xref_exists(DEPEND_0) ) xref_i_out->delete_xref(DEPEND_0);

			applyMaskToXrefs(oldSeqLen, xref_i_out, msk);
			ms_out->change_xref(sl[i].c_str(), xref_i_out);

		}
		// else leave as is
	  }
      return;
}	   

/***********************************************************************/
QdObject_var QuCleanObject(QdObject_var dobj){
	QdObject_var fill_xref;
	double fill = -10.e30;
	QdObject_var obj_out;
	bool objChanged = false;

	if(dobj.is_nil()) return dobj;
	
    if (dobj->xref_exists(FILL_VALUE))
		fill_xref = dobj->get_xref(FILL_VALUE);
	else 
		fill_xref = new QdRScalar(-10.e+30); // in case xref not set

	bool hasTT, hasSS;
	QdTimeSeq_var ts_in;
	QdRScalarSeq_var ss_in;
	
	QdObject_var qd0 = get_Depend_0( dobj );
	if ( !qd0.is_nil() ) {
		ts_in = QdTimeSeq_var::narrow(qd0);
		ss_in = QdRScalarSeq_var::narrow(qd0);	
	}

	hasTT = !(ts_in.is_nil());
	hasSS = !(ss_in.is_nil());

	QdTimeSeq_var ts_out=new QdTimeSeq;
	QdRScalarSeq_var ss_out=new QdRScalarSeq;

	QmjdTime badTime = QmjdTime(0.0); // <<<<< this is actually a valid time, change to NaN trap
	double badScalar = -10.e30; // <<<<<  change to NaN trap

    // is sequence a matrix sequence
    QdRMatrixSeq_var ms_in=QdRMatrixSeq_var::narrow(dobj);
    if(!ms_in.is_nil())
    {
      if(ms_in->empty())
          return obj_out;

      // if so, do we have a single fill-value or a matrix of different fill
      // values for each dimension

      QdRMatrix_var fill_m=QdRMatrix_var::narrow(fill_xref);

      // ...either way create matrix of fill values
      // this assumes the MatrixSeq is homogeneous i.e. each matrix
      // element has the same no. and sizes of dimensions

      if(fill_m.is_nil())
      {
          QdRScalar_var fill_v=QdRScalar_var::narrow(fill_xref);
          fill_m=new QdRMatrix(ms_in[0].dimdata(),fill_v->scalar());

      }

      // initialise matrix output sequence and reserve max space
      QdRMatrixSeq_var ms_out=new QdRMatrixSeq;
      ms_out->reserve(ms_in->size());
    
	// initialise mask
      SequenceMask msk(ms_in->size(),true);


      // get underlying data for fill matrix
      valarray<double> v_fill(fill_m->data()[fill_m->getgslice()]);
	for(register int i=0;i<ms_in->size();i++)
	{
		// catch bad time tags
		if(hasTT && ts_in[i] == badTime) {
			msk[i] = false;
			continue;
		}
		if(hasSS && ss_in[i] == badScalar) {
			msk[i] = false;
			continue;
		}

		bool valid_data=true;

		// get underlying data for matrix element
		valarray<double> v_in(ms_in[i].data()[ms_in[i].getgslice()]);
		for(register int j=0;j<v_in.size();j++)
		{
			const double& dfill = v_fill[j%v_fill.size()];
			valid_data=!QuIsBadData(v_in[j], dfill);
			if(!valid_data){
				msk[i] = false;
				break;
			}
		}
		// if valid update output matrix sequence
		if(valid_data)
		{
			ms_out->push_back(ms_in[i]);
		}
		if(msk[i] == false) objChanged=true; 
	}

		if(!objChanged) return ms_in;
		
		ms_out->copy_xrefs_from(ms_in);
	  

		applyMaskToXrefs(ms_in->sequence_size(), ms_out, msk);
	  
		return ms_out;
    }

    // scalar sequence case

   QdRScalarSeq_var rss_in=QdRScalarSeq_var::narrow(dobj);
   if(!rss_in.is_nil())
   {
    if(rss_in->empty())
          return obj_out;

      QdRScalar_var fill_v=QdRScalar_var::narrow(fill_xref);
      if( !fill_v.is_nil())
          fill = (double) *fill_v;

      // initialise mask
      SequenceMask msk(rss_in->size(),true);

      for(int i=0;i<rss_in->size();i++){
          msk[i] = !QuIsBadData(rss_in[i], fill);
		// catch bad time tags
		if(hasTT && ts_in[i] == badTime) msk[i] = false;
		if(hasSS && ss_in[i] == badScalar) msk[i] = false;
		
		if(msk[i] == false) objChanged=true;
      }
	  
	  if(!objChanged) return rss_in;
	  
      QdRScalarSeq_var rss_out=new QdRScalarSeq;
      rss_in->get_subsequence(*rss_out,msk);
      rss_out->copy_xrefs_from(rss_in);

      
      applyMaskToXrefs(rss_in->sequence_size(), rss_out, msk);
      return rss_out;
   }


    QdRMatrix_var m_in=QdRMatrix_var::narrow(dobj);
    if(!m_in.is_nil())
    {

      // if so, do we have a single fill-value or a matrix of different fill
      // values for each dimension

      QdRMatrix_var fill_m=QdRMatrix_var::narrow(fill_xref);

      // ...either way create matrix of fill values
      // this assumes the MatrixSeq is homogeneous i.e. each matrix
      // element has the same no. and sizes of dimensions

      if(fill_m.is_nil())
      {
          QdRScalar_var fill_v=QdRScalar_var::narrow(fill_xref);
          if(fill_v.is_nil())
              fill_v = new QdRScalar(fill);

          fill_m=new QdRMatrix(m_in->dimdata(),fill_v->scalar());

      }


      // iterator for fill values
      RMatrix::iterator fitr=fill_m->begin();

      // iterator for matrix
      RMatrix::iterator itr=m_in->begin();
      int j=0;
      while(itr!=m_in->end() )
      {
          const double& d=*itr++;
          if( QuIsBadData(d, *(fitr+j++) ) )
             return obj_out; // empty ptr
      }

      // catch bad time tags
      if(hasTT && ts_in[0] == badTime) return obj_out;
      if(hasSS && ss_in[0] == badScalar) return obj_out;

      return dobj;    // return original object

    }

   QdRScalar_var s_in=QdRScalar_var::narrow(dobj);
   if(!s_in.is_nil())
   {
      QdRScalar_var fill_v=QdRScalar_var::narrow(fill_xref);
      if( !fill_v.is_nil())
          fill = (double) *fill_v;

      if( QuIsBadData( *s_in, fill))
          return obj_out; // empty ptr

      // catch bad time tags
      if(hasTT && ts_in[0] == badTime) return obj_out;
      if(hasSS && ss_in[0] == badScalar) return obj_out;

      return dobj; // return original object as clean
   }

   // string and other input types not cleaned - return input object

   return dobj;

}
/***********************************************************************/

bool QuIsBadData(double value, double fill){
  if( checkFills()->isChecked()){
    if( fill == 0 && value == fill) return true;
    else if ( isFill(value, fill) ) return true;
  }

  if(checkBadData()->isChecked() ){
    if ( !isfinite(value) ) return true; // finds NaN, Infinity, -Infinity
  }

  return false;
}

/***********************************************************************/

bool QuIsBad(double value, double fill){

	// base level test, does not ask if strip is set
	if( fill == 0 && value == fill) return true;
	else if ( isFill(value, fill) ) return true;
  
	if ( !isfinite(value) ) return true; // finds NaN, Infinity, -Infinity

	return false;
}

/***********************************************************************/

QString QuNameOnly(QString nameContext){
 
  int i = nameContext.lastIndexOf('/');
  int len = nameContext.length() - i -1;
  
  if (i < 0 || len <= 0 ) return nameContext;
  
  return nameContext.right((uint) len);

}

/***********************************************************************/

void QuMsgBar(const char* text, bool error){

  if(qsasWindow==NULL) return;
  
  if(error)
  {
	  QPalette palette;
	  palette.setColor(qsasWindow->statusBar()->foregroundRole(), QColor("#ff0000"));
	  qsasWindow->statusBar()->setPalette(palette);
  }
  else
  {
	  QPalette palette;
	  palette.setColor(qsasWindow->statusBar()->foregroundRole(), qApp->palette().color(QPalette::WindowText));
	  qsasWindow->statusBar()->setPalette(palette);
  }
  
  qsasWindow->statusBar()->showMessage(text);

}

/***********************************************************************/

void QuMsgBar(const char* text1, const char* text2, bool error){
  static QString SStxt;

  SStxt = text1;
  SStxt += " ";
  SStxt += text2;
  QuMsgBar(SStxt.toLatin1(), error);

}

/***********************************************************************/

void QuSetOK(){
  static QString OKtxt = "  ok  ";

  // restore colour palette as last write may have been in error colours
  QPalette *palette = new QPalette(qApp->palette());
  
  if(qsasWindow!=NULL)
  {
	  qsasWindow->statusBar()->setPalette(*palette);
	  qsasWindow->statusBar()->showMessage(OKtxt,10000);
  }

}

/***********************************************************************/

int QuStrcpy(char * To, const char * From, int ToLen){

  /* ToLen is the size of the To string available
     (i.e. malloc as To[ToLen+1] ) */

 size_t FromLen;

   FromLen = strlen(From);
   if (FromLen > ToLen){
     strncpy(To, From, ToLen);
     To[ToLen] = '\0';
     QuMsgBar("String truncated on cpy", From, true);
   }
   else strcpy(To, From);

   return 0;
}

/***********************************************************************/

int QuStrcat(char * To, const char * From, int ToLen){

  /* ToLen is the remaining size of the To string available 
     (i.e. malloc as To[UsedLen + ToLen + 1] ) */
  
 size_t FromLen;
   
   FromLen = strlen(From);
   if (FromLen > ToLen){
     strncat(To, From, ToLen);
     To[ToLen] = '\0';
     QuMsgBar("String truncated on cat", From, true);
   }
   else strcat(To, From);
   
   return 0;
}
/***********************************************************************/

void QuSetFrame(QdObject_var obj, const char *txt, const char *frm, const char *rep){
// Accepts either CSDS syntax frame or components of frame "txt>frm_rep"
// txt = vector, rot, scalar, tensor
// frm = gse, gsm, sc, sc2 etc
// rep = xyz, rtp, rlp ( or rpz, xy, rp, which are not handled by QSAS yet)
// if txt is in "vector>gse_xyz" syntax dis-assemble
 
  string flag;
  string frame="";
  string representation;
  string frameAttr;
  double i_order;

  i_order = 0;
  if(QuToUpper(txt).find("SCALAR") != string::npos) {
    flag = "scalar";
    frame = "na";
	frameAttr = "scalar>na";
    i_order = 0;
  }
  else{
     if(strstr(txt,">") != 0){
         // parse txt
         string buffer =  string(txt);
	  
	     size_t ptr = buffer.find(">");
	     if(ptr != string::npos) {
	        flag = buffer.substr(0, ptr);
	        size_t ptr2 = buffer.find("_");
            if (ptr2 != string::npos){
	           frame = buffer.substr(ptr+1, ptr2 - ptr -1);
	           representation = buffer.substr(ptr2+1);
	        }
	     }
     }
     else{
	     flag = string(txt);
	     if(frm != 0) frame = string(frm);
	     if(rep != 0) representation = string(rep);
     }
	 
	 // assemble CSDS syntax Frame
	 frameAttr = flag;
	 frameAttr += ">";
	 frameAttr += frame;
	 if( !representation.empty() ){
	   frameAttr += "_";
	   frameAttr += representation;
	}
	
	if(QuToUpper(flag).find("VECTOR") != string::npos) i_order = 1;
	else if(QuToUpper(flag).find("TENSOR") != string::npos) i_order = 2; // We only go to order 2 in QSAS so far
	else if(QuToUpper(flag).find("ROT") != string::npos) i_order = 2;
	else i_order = 0; // no idea so this will block s/w doing vector & tensor stuff
	
  }// end else on scalar
  
  // Inputs parsed, now set attributes
  
  QuSetTxtAttr("Frame", frameAttr.c_str(), obj);
  QuSetTxtAttr("REPRESENTATION", representation.c_str() , obj);
 
  
  if(QuToUpper(frame).find("GSE") != string::npos) frame = "GSE>Geocentric Solar Ecliptic";
  else if (QuToUpper(frame).find("GSM") != string::npos) frame = "GSM>Geocentric Solar Magnetic";

  if( QuToUpper(frame).find("NA") == string::npos) {
  	QuSetTxtAttr("COORDINATE_SYSTEM", frame.c_str(), obj);
  }
  
  if(representation.size() >= 3){
  	QdSMatrix_var rep1 = new QdSMatrix(Dim(3), "");

  	for(int j=2;j>=0;j--){
		(*rep1)[j] = QdString(representation.substr(j,1));
  	}
  	obj->change_xref("REPRESENTATION_1", rep1);
  }

  try{ 
	   QdRScalar_var Torder = new QdRScalar(i_order);
        obj->change_xref("TENSOR_ORDER", Torder);
  }
  catch(Exception &e){
      QuMsgBar("cannot set xref ","TENSOR_ORDER" , true);
  }
  return;
}

/***********************************************************************/

string QuGetFrameAttr(QdObject_var obj){
	// returns CSDS syntax frame string "flag>frame_rep"
	int MaxLen = 80;
	string frameAttr("scalar>na");

	string frm   = QuGetFrame(obj);
	string rep   = QuGetRep(obj);
	string order = QuGetOrder(obj);

	if( frm.empty() || rep.empty()  || order.empty() )
	{
		return frameAttr;
	}

	if(order.find( "vector") != string::npos){
		frameAttr = "vector>";
		frameAttr += frm;
		frameAttr += "_";
		frameAttr += rep;
	}
	else if(order.find("rot") != string::npos){
		frameAttr = "rot>";
		frameAttr += frm;
		frameAttr += "_";
		frameAttr += rep;
	}
	else if(order.find("tensor") != string::npos){
		frameAttr = "tensor>";
		frameAttr += frm;
		frameAttr += "_";
		frameAttr += rep;
	}
   
	return frameAttr;
}

/***********************************************************************/

string QuGetFrameAttr(QdObject_var obj1, QdObject_var obj2){
   string frameAttr;
   string fr = QuGetFrame(obj1);
   if(!fr.empty() ) {
      frameAttr = QuGetFrameAttr(obj1);
   }
   else{
      frameAttr = QuGetFrameAttr(obj2); // will be scalar if none found
   }
   return frameAttr;  // QuGetFrameAttr always returns valid string, so this does too
}

/***********************************************************************/

string QuGetOrder(QdObject_var obj){

   string order(""); // return empty string if not found
   
   string attrPtr = QuGetAttrText( "Frame", obj );
   if( attrPtr.empty() ){

      // didn't find frame, look for TENSOR_ORDER
	   if(obj->xref_exists("TENSOR_ORDER")){
           QdObject_var xref=obj->get_xref("TENSOR_ORDER");
           QdRScalar_var xrefS= QdRScalar_var::narrow(xref);
           QdString_var xrefStr= QdString_var::narrow(xref);
           if( !xrefS.is_nil()) {
		     int i_order = (int) *xrefS;
		     if(i_order == 0) order = "scalar";
		     else if(i_order == 1) order = "vector";
		     else order = "tensor";
		   }
		   else if ( !xrefStr.is_nil())
		   {
		     int i_order = atoi(xrefStr->c_str());
		     if(i_order == 0) order = "scalar";
		     else if(i_order == 1) order = "vector";
		     else order = "tensor";
		   }
        }
   }
   else{
      // use Frame attribute if present, allows user to override CAA metadata
	 size_t ptr;
	 ptr = attrPtr.find(">");

	 if(ptr != string::npos) {
	   order = attrPtr.substr(0,  ptr);
	 }
   }
   
   return order;
}

/***********************************************************************/

string QuGetFrame(QdObject_var obj){

	string frame(""); // always return something, so empty this first
   
	string attrPtr = QuGetAttrText( "Frame", obj );

	if( attrPtr.empty()){
		// didn't find frame, look for COORDINATE_SYSTEM
		attrPtr = QuGetAttrText( COORDINATE_SYSTEM, obj );
		if(attrPtr.empty()) return frame;
		size_t ptr = attrPtr.find(">");
		if(ptr == string::npos) return attrPtr;
		return attrPtr.substr(0, ptr);
	}
	
	size_t ptr = attrPtr.find(">");

	if(ptr == string::npos) return attrPtr;

	// if scalar return empty
	if(QuToUpper(attrPtr.substr(0, ptr)).find("SCALAR") != string::npos) return frame; // will be empty string
	    
	size_t ptr2 = attrPtr.find("_");
	if (ptr2 == string::npos) return attrPtr.substr(ptr+1);
	
	frame =  attrPtr.substr(ptr+1, ptr2 - ptr -1);
	return frame;
}
/***********************************************************************/

string QuGetToFrame(QdObject_var obj){

	string frame(""); // always return something, so empty this first
   
	string attrPtr = QuGetAttrText( "ToFrame", obj );

	if( !attrPtr.empty()){
		size_t ptr = attrPtr.find(">");

		if(ptr != string::npos) {
	    
			size_t ptr2 = attrPtr.find("_");
			if (ptr2 != string::npos){
				frame =  attrPtr.substr(ptr+1, ptr2 - ptr -1);

			}
		}
 
	}
	return frame;
}

/***********************************************************************/


string QuGetRep(QdObject_var obj){
   string rep(""); // always return something, so empty this first
   
   string attrPtr = QuGetAttrText( "Frame", obj );
   if( attrPtr.empty() ){
      // didn't find frame, look for REPRESENTATION
	 attrPtr = QuGetAttrText( "REPRESENTATION", obj );
	 if( attrPtr.empty()){
	     attrPtr = QuGetAttrCondensed("REPRESENTATION_1", obj);
		int j=0;
		while(j < attrPtr.size() && isdigit(attrPtr[j]) ){
		  j++;
		}
		rep =  attrPtr.substr(j);
		return  rep;
	 }
	 return  attrPtr; 
   }
   else{

	 size_t ptr1 =  attrPtr.find(">");
	 if(ptr1 != string::npos) {
	    // if scalar return empty
	    if(QuToUpper(attrPtr.c_str()).find("SCALAR") != string::npos) return rep; // will be empty string
	    
	    size_t ptr2 = attrPtr.find("_"); // gets frame
         if (ptr2 != string::npos){
	      rep = attrPtr.substr(ptr2+1);
	    }
	 }
 
   }

   return rep;
}

/***********************************************************************/

bool QuIsThreeVector(QdObject_var obj)
{
	if(obj.is_nil()) return false;
	
  bool frameFound;
  // first test for frame (CSDS or CAA syntax)
  string framePtr = QuGetFrameAttr(obj);

  if(framePtr.empty() ) frameFound = false; // neither Frame nor COORDINATE_SYSTEM present
  else 
  {
     frameFound = true;
     if(framePtr.find("vector") == string::npos) return false;
  }
 
  QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
  if(rmatrix.is_nil())
  {
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(rmatrixseq.is_nil()) return false;
    if(!rmatrixseq->sequence_size())  return false;
    try
    {
      if((rmatrixseq[0].dims()).size()!=1) return false;
      if((rmatrixseq[0].dims())[0]!= 3) return false;
      if(!rmatrixseq->is_homogeneous_seq()) return false;
	 if(frameFound == true) return true;
	 else{
	   QuMsgBar("3D object with no frame", false);
	   return false;
	 }
    }
    catch(Exception &e)
    {
       e.print_msg();
       return false;
    }
  } 
  
  try
  {
    if((rmatrix->dims()).size()!=1) return false;
    if((rmatrix->dims())[0]!=3) return false;
    if(frameFound == true)  return true;
    else{
	 QuMsgBar("3D object with no frame", false);
	 return false;
    }
  }
  catch(Exception &e)
  {
    e.print_msg();
    return false;
  }
}

int QuArrayDims(QdObject_var obj)
{
  QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
  if(rmatrix.is_nil())
  {
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(rmatrixseq.is_nil())
      return 0;
    if(!rmatrixseq->sequence_size())
      return 0;
    try
    {
      if((rmatrixseq[0].dims()).size()>0 && rmatrixseq->is_homogeneous_seq())
        return (rmatrixseq[0].dims()).size();
    }
    catch(Exception &e)
    {
       e.print_msg();
       return 0;
    }
  } 
  
  try
  {
    if((rmatrix->dims()).size() > 0) return (rmatrix->dims()).size();
  }
  catch(Exception &e)
  {
    e.print_msg();
    return 0;
  }
  return 0;
}

/***********************************************************************/

bool QuIsScalar(QdObject_var dobj){

  if ( dobj.is_nil() ) return false;
  
  QdRScalar_var sc=QdRScalar_var::narrow(dobj);
  if(!sc.is_nil()) return true;
  
  QdRScalarSeq_var scs=QdRScalarSeq_var::narrow(dobj);
  if(!scs.is_nil()) return true;
 
  return false;
}


/***********************************************************************/

string QuGetAttrCondensed(const char *attrName, QdObject_var obj)
{
	string str ="";
	int lenNoWhite;
  
	if(obj.is_nil()) return str;
  
	if(obj->xref_exists(attrName))
	{
		QdObject_var xref=obj->get_xref(attrName);
		QdSMatrix_var Sxref = QdSMatrix_var::narrow(xref);
		if ( Sxref.is_nil() ) 
		{
			QdString_var sx =  QdString_var::narrow(xref);
			if(sx.is_nil()) return str;
			for(int i=0; i< strlen(sx->c_str()); i++)
			{
				if(sx->c_str()[i] != ',' && sx->c_str()[i] != ' ') str += sx->c_str()[i];
			}
		}
		else
		{
			for(QdSMatrix::iterator itr=Sxref->begin(); itr != Sxref->end(); itr++){
				lenNoWhite = itr->size();
				while (lenNoWhite > 0 && (*itr)[lenNoWhite-1] == ' ' ) lenNoWhite--; // remove Themis trailing space
				str += itr->substr(0, lenNoWhite);
			}
		}
		return str; 
	}
	else
		return str;
}



/***********************************************************************/
string QuGetAttrText(const char *attrName, QdObject_var obj)
{
  static StringRep string_rep;
  string_rep.set_token(StringRep::SR_TOKEN_STRING_DELIMITER,"");
  string str="";
  
  if(obj.is_nil()) return str;
  
  if(obj->xref_exists(attrName))
  {
    QdObject_var xref=obj->get_xref(attrName);

    str=string_rep(xref);
	
	if(str.size() > 80) {
		str = str.substr(0, 80);
		str += "...\0";
	}
  }
  return str; 

}
/***********************************************************************/
string QuGetVectorRep(QdObject_var obj)
{
    string rep("unk");

    string attr=QuGetRep(obj);
	
    if(attr.empty() ) return rep;
 
    rep = attr.substr(0, 3);
    
    // default is Rad
    if(QuToUpper(rep.c_str()).find("RTP") != string::npos || QuToUpper(rep.c_str()).find("RLP") != string::npos)
    {
      if( QuIsAngleDegrees( obj) )
        rep += "(deg)";
	 
    }
    
    return rep;
}
/***********************************************************************/

QString QuTextInputDialog(const char *caption, const char *label,
                          const char * defText, bool *OK, QWidget * parent){

QString input;

//     input = QInputDialog::getText(QString(caption), QString(label),
//                 QLineEdit::Normal, QString(defText), OK, parent, "QuInput_3");
	input=QInputDialog::getText(parent, QString(caption), QString(label), QLineEdit::Normal, QString(defText), OK);

return input;

}
/***********************************************************************/

QdObject_var QuGetVectorXYZ(QdObject_var dov){
  
 if ( !QuIsThreeVector(dov) ) return (QdObject_var) 0;


 const double RadToDeg = 45.0 / atan(1.0);  
 const double DegToRad = atan(1.0) / 45.0;  

 // if xyz already return input var
    
 string in_rep = QuGetRep(dov);
 if( in_rep == "xyz")  return dov;

 // if rtp convert to xyz
 
 if (in_rep =="rtp"){
   bool IsDeg = QuIsAngleDegrees(dov);
   try{
     // try sequence first
     QdRMatrixSeq_var in_seq = QdRMatrixSeq_var::narrow(dov);
     if(!in_seq.is_nil()){
       int sz = in_seq->sequence_size();
       
       QdRMatrixSeq_var out_seq = new QdRMatrixSeq(sz, Dim(3));
       out_seq->copy_xrefs_from(in_seq); 
       string attr = QuGetAttrText("Frame", out_seq);
       QuSetRep("xyz", out_seq);
  
       for(int i=0; i<sz; i++){
         RMatrix xyz(Dim(3),0.);

         if( IsDeg ){
           rtp_deg_to_xyz(in_seq[i], xyz);  //convert rtp to xyz
	      out_seq[i] = xyz;
         }
         else{
           rtp_to_xyz(in_seq[i],xyz);  //convert rtp to xyz
	      out_seq[i] = xyz;
         }
       } 
       return (QdObject_var) out_seq;
     }
     else {
       QdRMatrix_var in_var = QdRMatrix_var::narrow(dov);
       if(!in_var.is_nil()){

         QdRMatrix_var out_var = new QdRMatrix( Dim(3), 0);
         out_var->copy_xrefs_from(in_var);
	    QuSetRep("xyz", out_var);

         if( IsDeg ){
           rtp_deg_to_xyz(*in_var,*out_var);  //convert rtp to xyz
         }
         else{
           rtp_to_xyz(*in_var, *out_var);  //convert rtp to xyz
         }
         return (QdObject_var) out_var;
       }
     } // end if/else on sequence
   }
   catch(Exception &e){
      e.print_msg();
      return (QdObject_var) 0;   
   }
 }

 // if rlp convert to  xyz
 
 if (in_rep == "rlp" ) {
   bool IsDeg = QuIsAngleDegrees(dov);
   try{
     // try sequence first
     QdRMatrixSeq_var in_seq = QdRMatrixSeq_var::narrow(dov);
     if(!in_seq.is_nil()){
       int sz = in_seq->sequence_size();
       QdRMatrixSeq_var out_seq = new QdRMatrixSeq(sz, Dim(3));
       out_seq->copy_xrefs_from(in_seq); 
       QuSetRep("xyz", out_seq);

       for(int i=0; i<sz; i++){
         RMatrix xyz(Dim(3),0.);

         if( IsDeg ){
           rlp_deg_to_xyz(in_seq[i],xyz);  //convert rlp to xyz
	   
	   out_seq[i] = xyz;
         }
         else{
           rlp_to_xyz(in_seq[i],xyz);  //convert rlp to xyz
	   out_seq[i] = xyz;
         }
       } 
       return (QdObject_var) out_seq;
     }
     else {
       QdRMatrix_var in_var = QdRMatrix_var::narrow(dov);
       if(!in_var.is_nil()){
       
         QdRMatrix_var out_var = new QdRMatrix( Dim(3), 0);
         out_var->copy_xrefs_from(in_var); 
         QuSetRep("xyz", out_var);

         if( IsDeg ){
           rlp_deg_to_xyz(*in_var, *out_var);  //convert rlp to xyz
         }
         else{
           rlp_to_xyz(*in_var, *out_var);  //convert rlp to xyz
         }
         return (QdObject_var) out_var;
       }
     } // end if/else on sequence
   }
   catch(Exception &e){
      e.print_msg();
      return (QdObject_var) 0;   
   }
 }
 
 // can't handle other frame representations
 
 return (QdObject_var) 0;
 
}
/***********************************************************************/
 
void QuSetRep(const char* rep, QdObject_var do_out){
  QdString_var frame_xref = QdString_var::narrow(do_out->get_xref(FRAME));
  if( !frame_xref.is_nil() ){
    size_t pos = frame_xref->find("_");
    if(pos != string::npos){
       string stem = frame_xref->substr(0,pos+1);
       stem += rep;
       QdString_var new_frame_xref= new QdString(stem);
       do_out->change_xref(FRAME, new_frame_xref);
     } 
  }
  
  // and SI_conversion
  
  QdString_var SI_xref = QdString_var::narrow(do_out->get_xref(SI_CONVERSION));
  if( !SI_xref.is_nil() ){
    size_t pos = SI_xref->find(" ");
    if(pos != string::npos){
       string stemSI = SI_xref->substr(0,pos+1);
       QdString_var new_SI_xref= new QdString(stemSI);
       do_out->change_xref(SI_CONVERSION, new_SI_xref);
     } 
  }
  
    // and UNITS
  
  QdString_var units_xref = QdString_var::narrow(do_out->get_xref(UNITS));
  if( !units_xref.is_nil() ){
    size_t pos = units_xref->find(" ");
    if(pos != string::npos){
       string stemUnits = units_xref->substr(0,pos+1);
       QdString_var new_units_xref= new QdString(stemUnits);
       do_out->change_xref(UNITS, new_units_xref);
     } 
  }
  
  return;
}

/***********************************************************************/

void QuParseSIC(string &str, double &multiplier){
	// returns units portion in str
	multiplier = 0;
	size_t ptr = str.find(">");
	if(ptr == string::npos) return;

	multiplier = strtod(str.c_str(), NULL);
	
	str = str.substr(ptr+1);
	
}

/***********************************************************************/

bool QuIsAngleDegrees(QdObject_var dobj){
  StringRep string_rep;
  static string str;

  string_rep.set_token( StringRep::SR_TOKEN_STRING_DELIMITER, "" );

  QdObject_var xref;

  if(dobj->xref_exists(SI_CONVERSION)){
    xref = dobj->get_xref(SI_CONVERSION);
	str=string_rep(xref);
	double multiplier;
	QuParseSIC(str, multiplier);
	if (QuToUpper(str.c_str()).find("DEG") != string::npos){
		if ( fabs(multiplier - 1.) < CLOSE_ENOUGH ) return true;
		else return false;
	}
	if (QuToUpper(str.c_str()).find("RAD") != string::npos ){
		if( fabs(multiplier - 1.74532925e-2) < CLOSE_ENOUGH ) return true;
		else return false;
	}
  }
  else if(dobj->xref_exists(UNITS)){
    xref = dobj->get_xref(UNITS); 
	str=string_rep(xref);

  	if (QuToUpper(str.c_str()).find("DEG") == string::npos ) return false;

  }
  
  if( xref.is_nil() ) {
    printf("No SI_conversion: Units for polar angle assumed to be in degrees\n");
	fflush(stdout);
    return true;  // default to assuming degrees
  }

  // default is to assume degrees
  return true;


}

/***********************************************************************/

bool QuIsAngle(QdObject_var dobj, bool &isDeg){

  StringRep string_rep;
  static string str;
  
  isDeg = false;
  
  string_rep.set_token( StringRep::SR_TOKEN_STRING_DELIMITER, "" );

  QdObject_var xref;

  if(dobj->xref_exists(SI_CONVERSION)){
    xref = dobj->get_xref(SI_CONVERSION);
	str=string_rep(xref);
	double multiplier;
	QuParseSIC(str, multiplier);
	if (QuToUpper(str.c_str()).find("DEG") != string::npos){
		if ( fabs(multiplier - 1.) < CLOSE_ENOUGH ) isDeg = true;
	}
	else if( QuToUpper(str).find("RAD") != string::npos ){
		if( fabs(multiplier - 1.74532925e-2) < CLOSE_ENOUGH ) isDeg = true;
	}
	else return false;
  }
  else if(dobj->xref_exists(UNITS)){
    xref = dobj->get_xref(UNITS); 
	str=string_rep(xref);

  	if (QuToUpper(str).find("DEG") != string::npos ) isDeg = true;
  	else if(QuToUpper(str).find("RAD") != string::npos ) isDeg = false;
	else return false;
  }
  else return false; // do not assume this is an angle
  
  return true;


}

/***********************************************************************/

bool QuIsPhiAngle(QdObject_var dobj, bool isDeg, int index, double &minVal, int &kStart, int &kEnd){

	// returns true if this is the Phi angle, and minVal used for lat to colat theta shift
	// returns kStart and kEnd (kStart must be initialised to 0)
	
	double conv2rad=1.;
	if(isDeg) conv2rad=_PI/180.;
	
	QdRMatrixSeq_var o_seq=QdRMatrixSeq_var::narrow(dobj);
	QdRMatrix_var o_const=QdRMatrix_var::narrow(dobj);
	minVal=1.e30;
    double maxVal=-1.e30;
	
	bool isPhi = false;
	
	double fillVal = -1.e31;
	if(dobj->xref_exists(FILL_VALUE)){
		QdRScalar_var fill_ptr = QdRScalar_var::narrow(dobj->get_xref(FILL_VALUE));
		if(!fill_ptr.is_nil()) fillVal = *fill_ptr;
	}
			
	if(!o_seq.is_nil())
	{
		int n_dim = o_seq[0].dimdata()[0];
		for(int k=0; k<n_dim; k++)
		{
			double i_val = o_seq[0][k] * conv2rad;

			if( notFill(i_val, fillVal) )
			{
				if(kStart == 0) kStart = k+1; // slots use range (1, n)
				kEnd = k+1;
				if(i_val < minVal) minVal = i_val;
				if(i_val > maxVal) maxVal = i_val;
			}
		}
	}
	else if(!o_const.is_nil()){
		int n_dim=o_const->dimdata()[0];
		
		for(int k=0; k<n_dim; k++)
		{
			double i_val = (*o_const)[k]  * conv2rad;

			if( notFill(i_val, fillVal) )
			{
				if(kStart == 0) kStart = k+1; // slots use range (1, n)
				kEnd = k+1;
				if(i_val < minVal) minVal = i_val;
				if(i_val > maxVal) maxVal = i_val;
			}
		}
	}
	
	if( fabs(maxVal-minVal)>_PI ) isPhi = true;
	
	if(!isPhi) // Let's see if the variable name gives us a hint...
	{
		QString name=QuGetAttrText("FIELDNAM", dobj).c_str();
		isPhi=(name.contains("phi") || name.contains("Phi")|| name.contains("azimut")|| name.contains("Azimut"));
	}

	if(!isPhi)
	{
		QString representation=QuGetAttrText("REPRESENTATION", dobj).c_str();
		isPhi=(representation=="rtp" || representation=="rlp")&&(index==3);
	}

	return isPhi;

}

/***********************************************************************/

QdObject_var QuAngleMod(QdObject_var depObj, int angMax, const char * rad_deg){
	
	QdObject_var newObj;
	double threshold = 180.;
	if(strcmp(rad_deg, "rad") == 0) threshold = 4.*atan(1.);
	
	double shift = 2.*threshold;
	
	// DEPEND may be 1D matrix or 1D matrix sequence
	QdRMatrix_var m = QdRMatrix_var::narrow(depObj);
	QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(depObj);
			
	if( !m.is_nil() ){
		QdRMatrix_var m_xref = Factory::clone(m);
		
		for(QdRMatrix::iterator itr = m_xref->begin(); itr != m_xref->end(); itr++){
			if(angMax == 360 && *itr < 0) *itr += shift;
			if(angMax == 180 && *itr > threshold) *itr -= shift;
		}
		newObj = m_xref;
	}
	else if(!ms.is_nil()){
		QdRMatrixSeq_var ms_xref = Factory::clone(ms);
		
		for(int j=0; j<ms_xref->sequence_size(); j++) {
			const QdRMatrix &mat = ms_xref[j];
			for(QdRMatrix::iterator itr = mat.begin(); itr != mat.end(); itr++){
				if(angMax == 360 && *itr < 0) *itr += shift;
				if(angMax == 180 && *itr > threshold) *itr -= shift;
			}
		}
		newObj = ms_xref;
	}
	
	newObj->copy_xrefs_from(depObj);

	return newObj;
}

/***********************************************************************/

string QuAngleUnits(QdObject_var dobj){

	// unit object is an angle, else ""
	StringRep string_rep;
	string str = ""; 
  
	string_rep.set_token( StringRep::SR_TOKEN_STRING_DELIMITER, "" );

	QdObject_var xref;

	if(dobj->xref_exists(SI_CONVERSION)) xref = dobj->get_xref(SI_CONVERSION);
	else if (dobj->xref_exists(UNITS)) xref = dobj->get_xref(UNITS);
	
	if( !xref.is_nil() ) {
    	
		str=string_rep(xref);

  		if (QuToUpper(str).find("DEG") != string::npos)  str = string("deg");
  		else if (QuToUpper(str).find("RAD") != string::npos)  str = string("rad");
		else str = string("");
	}
	
  	return str;


}
/***********************************************************************/

QdObject_var QuGetArrayComponent(QdObject_var doa, dimTripleStack dimStack, string arrComp_xref_in)
{
   string arrComp_xref;
   size_t posn = arrComp_xref_in.find(" <");
   if(posn != string::npos) arrComp_xref = arrComp_xref_in.substr(0, posn);
   else  arrComp_xref = string(arrComp_xref_in.c_str());
   
   if (arrComp_xref != ""){
	 // look for cache
     if(doa->xref_exists(arrComp_xref)) return doa->get_xref(arrComp_xref);
   }
   
  
   QdObject_var reducedArr = doa;
   int i_offset = 1;
   // Reduce each dimension in turn according to operation choice
   for(int i=0; i<dimStack.size(); i++)
   {
      int choice = dimStack[i][0];
      if(choice == 0 )
	  {
	   if(dimStack[i][1] > dimStack[i][2]){
	      // elements cannot be sliced cyclic
		 QuMsgBar("start entry higher than end for element selection", true);
	      return doa;
	   }
	    // elements  (keeps DEPEND_i but modifies - can have m x 1 array, not exactly same as slice)
	    try{
	       reducedArr = QuSubsetArray(reducedArr, i + i_offset, dimStack[i][1], dimStack[i][2]);
	    }
	    catch(Exception &e){
	       cerr << e.msg() << endl;
		  QuMsgBar("element selection: slice out of range", true);
            return doa;
	    }
		// this option does not remove dimension i
	 }
	 else  if(choice == 1)
	 {
	    // sum (removes DEPEND_i)
	    reducedArr = QuSumArray(reducedArr, i + i_offset, dimStack[i][1], dimStack[i][2], false);
	    i_offset--;  // dimension has been removed, next call in loop must be adjusted
	 }
	 else  if(choice == 3)
	 {
	    // ave (removes DEPEND_i)
	    reducedArr = QuSumArray(reducedArr, i + i_offset, dimStack[i][1], dimStack[i][2], true);
	    i_offset--;  // dimension has been removed, next call in loop must be adjusted
     }
	 else  if(choice == 4)
	 {
	    // alias to time (removes DEPEND_i, modifies Depend_0 and removes D_0 deltas)
	    reducedArr = QuAliasArray(reducedArr, i + i_offset, dimStack[i][1], dimStack[i][2]);
	    i_offset--;  // dimension has been removed, next call in loop must be adjusted
     }
	 else
	 {
	    // slice (removes DEPEND_i)
	    reducedArr = QuSliceArray(reducedArr, i + i_offset, dimStack[i][1]);
	    i_offset--; // dimension has been removed, next call in loop must be adjusted
	 }
	 if (reducedArr.is_nil() ) return doa; // break from loop and give up
   }
      
   if (arrComp_xref != "" && !reducedArr.is_nil()) doa->change_xref(arrComp_xref.c_str(), reducedArr);
   
   return reducedArr;
}


/***********************************************************************/

QdObject_var QuSliceArray(QdObject_var doa, int indx, int entry )
{
   // Reduce dimension by slicing at entry along dimension indx
   QdRScalar_var s_out;
   QdRMatrix_var m_out;
   QdRScalarSeq_var ss_out;
   QdRMatrixSeq_var ms_out;

   QdObject_var reducedArr; // output object for return  
   if(doa.is_nil()) return reducedArr;
   
   Dim mDim;
   int mRank;

   // GUI works [1,n] qdos works [0,n-1]
   int ii = indx - 1;
   int ei = entry - 1;

   QdRMatrix_var Mat_in = QdRMatrix_var::narrow(doa);
   QdRMatrixSeq_var MatSeq_in = QdRMatrixSeq_var::narrow(doa);

     // Handle sequence and single matrix separately
     // Matrix first
   if ( !Mat_in.is_nil()  ) {
   
      // dimension sizes
      mDim = Mat_in->dimdata();
      mRank = mDim.rank();
 
      if(ii >= mRank || ii < 0){
        cout << "Array Slice: ERROR - index out of range" << endl;
        return reducedArr; 
      }
      if(ei >= (Mat_in->dims())[ii] || ei < 0){
        cout << "Array Slice: ERROR - entry out of range for this index" << endl;
        return reducedArr; 
      }
   
      // handle each case separately
      if(mRank == 1){
        s_out = new QdRScalar((*Mat_in)[ei]);
        reducedArr = (QdObject_var) s_out;
      }
      else if(mRank == 2){
        if(ii == 0){
          m_out = new QdRMatrix((*Mat_in)[ei][all]);
          reducedArr = (QdObject_var) m_out;
        }
        else if(ii == 1){
          m_out = new QdRMatrix((*Mat_in)[all][ei]);
          reducedArr = (QdObject_var) m_out;
        }
        else{
          cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
          return reducedArr;
        }
      }
      else if(mRank == 3){
        if(ii == 0){
          m_out = new QdRMatrix((*Mat_in)[ei][all][all]);
          reducedArr = (QdObject_var) m_out;
        }
        else if(ii == 1){
          m_out = new QdRMatrix((*Mat_in)[all][ei][all]);
          reducedArr = (QdObject_var) m_out;
        }
        else if(ii == 2){
          m_out = new QdRMatrix((*Mat_in)[all][all][ei]);
          reducedArr = (QdObject_var) m_out;
        }
        else{
          cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
          return reducedArr;
        }
     }
     else if(mRank == 4){
       if(ii == 0){
         m_out = new QdRMatrix((*Mat_in)[ei][all][all][all]);
         reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 1){
         m_out = new QdRMatrix((*Mat_in)[all][ei][all][all]);
         reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 2){
         m_out = new QdRMatrix((*Mat_in)[all][all][ei][all]);
         reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 3){
         m_out = new QdRMatrix((*Mat_in)[all][all][all][ei]);
         reducedArr = (QdObject_var) m_out;
       }
       else{
         cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }
     else{
        cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
        return reducedArr; 
     }
   } 	

   // try sequence
   else if ( !MatSeq_in.is_nil() ) {
     // sequence size
     int ndata = MatSeq_in->sequence_size();

     // dimension sizes
     mDim = MatSeq_in[0].dimdata();
     mRank = mDim.rank();

     if(ii >= mRank || ii < 0){
       cout << "Array Slice: ERROR - index out of range" << endl;
       return reducedArr; 
     } 
     if(ei >= (MatSeq_in[0].dims())[ii] || ei < 0){
       cout << "Array Slice: ERROR - entry out of range for this index" << endl;
       return reducedArr; 
     }

     // handle each case separately
     if(mRank == 1){
       ss_out = new QdRScalarSeq((*MatSeq_in)[all][ei]);
       reducedArr = (QdObject_var) ss_out;
     }
     else if(mRank == 2){
       if(ii == 0){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][ei][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 1){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][ei]);
         reducedArr = (QdObject_var) ms_out;
       }
       else{
         cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }
     else if(mRank == 3){
       if(ii == 0){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][ei][all][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 1){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][ei][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 2){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][ei]);
         reducedArr = (QdObject_var) ms_out;
       }
       else{
         cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }  
     else if(mRank == 4){
       if(ii == 0){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][ei][all][all][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 1){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][ei][all][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 2){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][ei][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 3){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][all][ei]);
         reducedArr = (QdObject_var) ms_out;
       }
       else{
         cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }
     else{
       cout << "Array Slice: Sorry, only matrices up to Rank 4 implemented" << endl;
       return reducedArr; 
     }
  
   }
   else{	
     cout << "Array Slice: ERROR - input not an array or array series!" << endl;
     return reducedArr; 
   }        

   
   // copy basic Xrefs across...  
   // this will ensure new object points to same time tags.
       
   reducedArr->copy_xrefs_from(doa); 

   // create new LABLAXIS 

   char num_str[50];
   string labstr;
   static StringRep string_rep;

   if (reducedArr->xref_exists(LABLAXIS) ){
  
     // handle case when LABLAXIS is an array 
     QdObject_var xref=reducedArr->get_xref(LABLAXIS);
     QdStringSeq_var labi = QdStringSeq_var::narrow(xref);
     if(!labi.is_nil() && labi->size() > ei){
       // works for 1-D ARRAY ONLY await cef final spec.
       labstr = string(labi[ei]);
     }
     else{
       // regular LABLAXIS string
       labstr = QuGetAttrText(LABLAXIS, reducedArr);
     }
   }
   else if (reducedArr->xref_exists("ObjectName") ){
     labstr = QuGetAttrText("ObjectName", reducedArr);
   }
   QString labl_attr = QString("LABEL_%1").arg(indx);
   QString dep_attr = QString("DEPEND_%1").arg(indx);

   if (reducedArr->xref_exists(labl_attr.toLatin1().data()) ){
     QdObject_var xref=reducedArr->get_xref(labl_attr.toLatin1().data());
     QdStringSeq_var labi = QdStringSeq_var::narrow(xref);
     if (!labi.is_nil() && labi->size() > ei){
       labstr += (*labi)[ei];  
     }
     else {
       labstr += " entry ";
       sprintf(num_str, "%d", ei+1);
       labstr += string(num_str);
       labstr += " from: ";
       labstr += QuGetAttrText(labl_attr.toLatin1(), reducedArr);
     }
   }
   else if (reducedArr->xref_exists(dep_attr.toLatin1().data())){
      labstr += string(" Slice(");
      QdObject_var d_xref=reducedArr->get_xref(dep_attr.toLatin1().data());
      QdRMatrix_var Dxref=QdRMatrix_var::narrow(d_xref);
      QdRMatrixSeq_var DSxref=QdRMatrixSeq_var::narrow(d_xref);
      if(!Dxref.is_nil()){
        if (Dxref->xref_exists(LABLAXIS)){
          labstr += QuGetAttrText(LABLAXIS, Dxref);  
        }
        else if (Dxref->xref_exists(FIELDNAM)){
          labstr += QuGetAttrText(FIELDNAM, Dxref);   
        }
        if (Dxref->size() > ei){
          labstr += " ";
		  QdRScalar_var elt = new QdRScalar((*Dxref)[ei]);
          labstr += string_rep(elt);
        }
        if (Dxref->xref_exists(UNITS)){
          labstr += " ";
          labstr += QuGetAttrText(UNITS, Dxref);
        }
      }
      else if(!DSxref.is_nil()){
        cout <<"CAUTION: Index dependency is RECORD VARYING!  Slice may not be meaningful!" << endl;;

        if (DSxref->xref_exists(LABLAXIS)){
          labstr += QuGetAttrText(LABLAXIS, DSxref);  
        }
        else if (DSxref->xref_exists(FIELDNAM)){
          labstr += QuGetAttrText(FIELDNAM, DSxref);   
        }
        if (DSxref->size() > ei){
          labstr += " ";
          QdRScalar_var elt = new QdRScalar((*DSxref)[0][ei]);
          labstr += string_rep(elt);
        }
        if (DSxref->xref_exists(UNITS)){
          labstr += " ";
          labstr += QuGetAttrText(UNITS, DSxref);
        }
      }
      else{
        char compstr[3];
	    sprintf(compstr, "%1d", ei+1); // Note this is %one-d not %ell-d
        labstr += string(compstr);
      }
      labstr += string(")");
   }
   QuSetTxtAttr(LABLAXIS, labstr.c_str(), reducedArr);

    string mods_str = "Array sliced at entry ";
    sprintf(num_str, "%d", entry);
    mods_str += string(num_str);
    mods_str += " of index "; 
    sprintf(num_str, "%d", indx);
    mods_str += string(num_str);
    mods_str += " (see LABLAXIS)"; 
    QuSetTxtAttr("MODS", mods_str.c_str(), reducedArr);

    QString label_i_1 = "?";
    QString depend_i_1 ="?";
 
    for(int i=1; i <= mRank; i++)
    {
      if(i < indx) continue;  // indices below slice are unaffected
	 
      QString labl_i = QString("LABEL_%1").arg(i);
      QString dep_i  = QString("DEPEND_%1").arg(i);

      if(i == indx){
        // slice removes this dimension
        if (reducedArr->xref_exists(labl_i.toLatin1().data()) ) reducedArr->delete_xref(labl_i.toLatin1().data());
        if (reducedArr->xref_exists(dep_i.toLatin1().data()) )  reducedArr->delete_xref(dep_i.toLatin1().data());
	   label_i_1 = QString(labl_i.toLatin1().data()); // deep copy
	   depend_i_1 = QString(dep_i.toLatin1().data()); // deep copy
      }
     else{
        // shuffle higher dimensions down
        if(reducedArr->xref_exists(labl_i.toLatin1().data())) reducedArr->rename_xref(labl_i.toLatin1().data(), label_i_1.toLatin1().data());
        if(reducedArr->xref_exists(dep_i.toLatin1().data()))  reducedArr->rename_xref(dep_i.toLatin1().data(), depend_i_1.toLatin1().data());
	   label_i_1 = QString(labl_i.toLatin1()); // deep copy
	   depend_i_1 = QString(dep_i.toLatin1()); // deep copy
      }
    }
 
   
   return reducedArr;
   
   // END QuSliceArray
}  

/***********************************************************************/

QdObject_var QuSumDown(QdRMatrix &Mat_in, int indx, int fromK, int toK, bool average, double fill ){
	
	if(fromK > toK) return QuSumExterior(Mat_in, indx, fromK, toK, average, fill );
	
	QdObject_var ret;

	Dim mDim = Mat_in.dimdata();
	int mRank = mDim.rank();
	int ii = indx-1;
	int newsize=1;
	
	valarray <size_t> newdims(mRank-1);	
	int nextdim=0;
	
	int ni0 = mDim[0];
	int ni1, ni2;
	
	if(mRank > 1) ni1 = mDim[1];
	else ni1 = 1;
	
	if(mRank > 2) ni2 = mDim[2];
	else ni2 = 1;
	
	int s0 = 0;
	int e0 = ni0;
    if(ii == 0 ) {
		s0 = fromK-1;
		e0 = toK;
	}
	else{
		newdims[nextdim] = ni0;
		newsize *= ni0;
		nextdim++;
	}
	
	int s1 = 0;
	int e1 = ni1;
    if(ii == 1 ) {
		s1 = fromK-1;
		e1 = toK;
	}
	else if(nextdim < mRank-1){
		newdims[nextdim] = ni1;
		newsize *= ni1;
		nextdim++;
	}
	
	int s2 = 0;
	int e2 = ni2;
    if(ii == 2 ) {
		s2 = fromK-1;
		e2 = toK;
	}
	else if(nextdim < mRank-1){
		newdims[nextdim] = ni2;
		newsize *= ni2;
		nextdim++;
	}
	
	if(mRank == 1) {
		QdRScalar_var S_out = new QdRScalar( 0.0);
		ret = S_out;
		
		int nAve = 0;
	
		for(int i0=s0; i0<e0; i0++ ){
			if( !isFill(Mat_in[i0], fill) ){
				*S_out += Mat_in[i0];
				nAve ++;
			}

		}
	
		// handle empty element and average option
		if( nAve == 0) *S_out = fill;
		else if(average) *S_out /= nAve;
	
	}
	else{
		QdRMatrix_var Mat_out = new QdRMatrix(Dim(newdims), 0.0);
		ret = Mat_out;
		
		valarray <int> nAve(0, newsize);
	
		int ie=0;
		double * matptr = Mat_in.ptr();
		for(int i0=s0; i0<e0; i0++ ){
			for(int i1=s1; i1<e1; i1++ ){
				for(int i2=s2; i2<e2; i2++ ){
					if(ii == 0) ie = i1 *ni2 + i2;
					else if(ii == 1) ie = i0 * ni2 + i2;
					else ie = (i0 * ni1 +i1);
					if( !isFill( matptr[(i0 * ni1 +i1) *ni2 + i2], fill) ){
						Mat_out->ptr()[ie] += matptr[(i0 * ni1 +i1) *ni2 + i2];
						nAve[ie] ++;
					}
					
				}
			}
		}
	
		// handle empty element and average option
		for(int i=0; i<nAve.size(); i++){
			if( nAve[i] == 0) Mat_out->ptr()[i] = fill;
			else if(average) Mat_out->ptr()[i] /= nAve[i];
		}
	}
	
	return ret;
}


/***********************************************************************/

QdObject_var QuSumExterior(QdRMatrix &Mat_in, int indx, int fromK, int toK, bool average, double fill){
		
	QdObject_var ret;
	
	
	Dim mDim = Mat_in.dimdata();
	int mRank = mDim.rank();
	int ii = indx-1;
	int newsize=1;
	
	valarray <size_t> newdims(mRank-1);	
	int nextdim=0;
	
	int ni0 = mDim[0];
	int ni1, ni2;
	
	if(mRank > 1) ni1 = mDim[1];
	else ni1 = 0;
	
	if(mRank > 2) ni2 = mDim[2];
	else ni2 = 0;
	
	int s0 = ni0;
	int e0 = 0;
    if(ii == 0 ) {
		s0 = toK-1;
		e0 = fromK-1;
	}
	else{
		newdims[nextdim] = ni0;
		newsize *= ni0;
		nextdim++;
	}
	
	int s1 = ni1;
	int e1 = 0;
    if(ii == 1 ) {
		s1 = toK-1;
		e1 = fromK-1;
	}
	else if(nextdim < mRank-1){
		newdims[nextdim] = ni1;
		newsize *= ni1;
		nextdim++;
	}
	
	int s2 = ni2;
	int e2 = 0;
    if(ii == 2 ) {
		s2 = toK-1;
		e2 = fromK-1;
	}
	else if(nextdim < mRank-1){
		newdims[nextdim] = ni2;
		newsize *= ni2;
		nextdim++;
	}
	
	if(mRank == 1) {
		QdRScalar_var S_out = new QdRScalar( 0.0);
		ret = S_out;
		
		int nAve = 0;
	
		for(int i0=0; i0<ni0; i0++ ){
			if(i0 > s0 && i0 < e0 ) continue;
			if( !isFill(Mat_in[i0], fill) ){
				*S_out += Mat_in[i0];
				nAve ++;
			}

		}
	
		// handle empty element and average option
		if( nAve == 0) *S_out = fill;
		else if(average) *S_out /= nAve;
	
	}
	else{
		QdRMatrix_var Mat_out = new QdRMatrix(Dim(newdims), 0.0);
		ret = Mat_out;
		
		
		valarray <int> nAve(0, newsize);
	
	    // exterior sums, continue if interior to (toK, fromK)
		int ie=0;
		double * matptr = Mat_in.ptr();

		for(int i0=0; i0<ni0; i0++ ){
			if(i0 > s0 && i0 < e0 ) continue;
			for(int i1=0; i1<ni1; i1++ ){
				if(i1 > s1 && i1 < e1 ) continue;
				for(int i2=0; i2<ni2; i2++ ){
					if(i2 > s2 && i2 < e2 ) continue;
					
					if(ii == 0) ie = i1 *ni2 + i2;
					else if(ii == 1) ie = i0 * ni2 + i2;
					else ie = (i0 * ni1 +i1);
					if( !isFill( matptr[(i0 * ni1 +i1) *ni2 + i2], fill) ){
						Mat_out->ptr()[ie] += matptr[(i0 * ni1 +i1) *ni2 + i2];
						nAve[ie] ++;
					}
					
				}
			}
		}
	
		// handle empty element and average option
		for(int i=0; i<nAve.size(); i++){
			if( nAve[i] == 0) Mat_out->ptr()[i] = fill;
			else if(average) Mat_out->ptr()[i] /= nAve[i];
		}
	}
	
	return ret;
}





/***********************************************************************/

QdObject_var QuSumArray(QdObject_var doa, int indx, int fromK, int toK, bool average )
{
	// Reduce dimension by summing along dimension indx

	// fillval for sums and averages
	double fill = -1.e31;
	if(doa->xref_exists(FILL_VALUE) )
	{
   		QdObject_var fill_ptr = doa->get_xref(FILL_VALUE);
   		QdRScalar_var dbl_ptr = QdRScalar_var::narrow(fill_ptr);
		if(!dbl_ptr.is_nil()) fill = *dbl_ptr;
	}

	QdObject_var reducedArr; // output object for return  
	if(doa.is_nil()) return reducedArr;

    
	Dim mDim;
	int mRank;
	int nave; 
	// GUI works [1,n] qdos works [0,n-1]
	int ii = indx - 1;

	QdRMatrix_var Mat_in = QdRMatrix_var::narrow(doa);
	QdRMatrixSeq_var MatSeq_in = QdRMatrixSeq_var::narrow(doa);

	// Handle sequence and single matrix separately
	// Matrix first
	if ( !Mat_in.is_nil()  ) {
   
      // dimension sizes
      mDim = Mat_in->dimdata();
      mRank = mDim.rank();
 
      if(ii >= mRank || ii < 0){
        cout << "Array Sum: ERROR - index out of range" << endl;
        return reducedArr; 
      }
      if(fromK > mDim[ii] || fromK < 1){
        cout << "Array Sum: ERROR - 'from' entry out of range for this index" << endl;
        return reducedArr; 
      }
      if(toK > mDim[ii] || toK < 1){
        cout << "Array Sum: ERROR - 'to' entry out of range for this index" << endl;
        return reducedArr; 
      }
   
     reducedArr = QuSumDown( *Mat_in,  indx,  fromK,  toK,  average, fill );
	 

   }
   // try sequence
   else if ( !MatSeq_in.is_nil() ) {

     // sequence size
     int ndata = MatSeq_in->sequence_size();

	 QdRScalarSeq_var ss_out;
	 QdRMatrixSeq_var ms_out;
	 
     // dimension sizes
     mDim = MatSeq_in[0].dimdata();
     mRank = mDim.rank();

     if(ii >= mRank || ii < 0){
       cout << "Array Sum: ERROR - index out of range" << endl;
       return reducedArr; 
     } 
     if(fromK > mDim[ii] || fromK < 1){
       cout << "Array Sum: ERROR - entry out of range for this index" << endl;
       return reducedArr; 
     }
     if(toK > mDim[ii] || toK < 1){
       cout << "Array Sum: ERROR - entry out of range for this index" << endl;
       return reducedArr; 
     }

	 if(mRank == 1) {
	 	ss_out = new QdRScalarSeq(ndata);		
	 	for(int i=0; i<ndata; i++){
			QdRMatrix_var elt = new QdRMatrix(MatSeq_in[i]);
			QdObject_var newObj = QuSumDown( *elt,  indx,  fromK,  toK,  average, fill );
			QdRScalar_var newS = QdRScalar_var::narrow(newObj);
			if( !newS.is_nil()) ss_out[i] = *newS;
			else return reducedArr;
	 	}
		reducedArr = ss_out;
	 }
	 else {
	 	ms_out = new QdRMatrixSeq(ndata);		
	 	for(int i=0; i<ndata; i++){
			QdRMatrix_var elt = new QdRMatrix(MatSeq_in[i]);
			QdObject_var newObj = QuSumDown( *elt,  indx,  fromK,  toK,  average, fill );
			QdRMatrix_var newM = QdRMatrix_var::narrow(newObj);
			if( !newM.is_nil()) ms_out[i] = *newM;
			else return reducedArr;
	 	}
		reducedArr = ms_out;
	 }
   }      

   if (reducedArr.is_nil() ) {
     cout << "Array Sum/Ave: Unable to create summed array object" << endl;
     return reducedArr; 
   }
   
   // copy basic Xrefs across...  
     reducedArr->copy_xrefs_from(doa); 

   // create new LABLAXIS 

   char * lab=(char*)"";
   char * comp=(char*)""; 
   char * unit=(char*)"";
   char num_str[20];
   string labstr;

   QString labl_attr = QString("LABEL_%1").arg(indx);
   QString dep_attr  = QString("DEPEND_%1").arg(indx);


   if (reducedArr->xref_exists(labl_attr.toLatin1().data()) ){
     QdObject_var xref=reducedArr->get_xref(labl_attr.toLatin1().data());
     QdStringSeq_var labi = QdStringSeq_var::narrow(xref); // try pointer
	
     if (!labi.is_nil()){
       // works only for 1-D array, await cef final spec...
       if (average) labstr = "Ave ";
	   else labstr = "Sum ";
	  if (fromK <= labi->size() && toK <= labi->size()){
	     if( toK >= fromK){
	       for (int ei=fromK - 1; ei<toK; ei++){
              if(ei != fromK - 1) labstr += ", ";
              labstr += labi[ei];
            }
	     }
	     else{
	       for (int ei=fromK - 1; ei<labi->size(); ei++){
              if(ei != fromK - 1) labstr += ", ";
              labstr += labi[ei];
            }
	       for (int ei=0; ei<toK; ei++){
              labstr += ", ";
              labstr += labi[ei];
            }
	     }
       }
       else {
         if (average) labstr += string(" ave down ");
		 else labstr += string(" sum down ");
         labstr += QuGetAttrText(labl_attr.toLatin1(), reducedArr);
       }
	}
   }
   else if ( reducedArr->xref_exists(LABLAXIS)) {
     // look for case when LABLAXIS is an array 
     QdObject_var xref=reducedArr->get_xref(LABLAXIS);
     QdStringSeq_var labi = QdStringSeq_var::narrow(xref);
	if (!labi.is_nil()){
       // works only for 1-D array, await cef final spec...
       if (average) labstr = "Ave ";
       else labstr = "Sum ";
	  if (fromK <= labi->size() && toK <= labi->size()){
	     if( toK >= fromK){
	       for (int ei=fromK - 1; ei<toK; ei++){
              if(ei != fromK - 1) labstr += ", ";
              labstr += labi[ei];
            }
	     }
	     else{
	       for (int ei=fromK - 1; ei<labi->size(); ei++){
              if(ei != fromK - 1) labstr += ", ";
              labstr += labi[ei];
            }
	       for (int ei=0; ei<toK; ei++){
              labstr += ", ";
              labstr += labi[ei];
            }
	     }
       }
       else {
         if (average) labstr += string(" ave down ");
		 else labstr += string(" sum down ");
         labstr += QuGetAttrText(labl_attr.toLatin1(), reducedArr);
       }
	}

   }
   else if (reducedArr->xref_exists(dep_attr.toLatin1().data())){
      if (average) labstr += string(" Ave(");
      else labstr += string(" Sum(");
      QdObject_var dxref=reducedArr->get_xref(dep_attr.toLatin1().data());
      QdRMatrix_var Dxref=QdRMatrix_var::narrow(dxref);
      QdRMatrixSeq_var DSxref=QdRMatrixSeq_var::narrow(dxref);
      if(!Dxref.is_nil()){
        if (Dxref->xref_exists(LABLAXIS)){
          labstr += QuGetAttrText(LABLAXIS, Dxref);  
        }
        else if (Dxref->xref_exists(FIELDNAM)){
          labstr += QuGetAttrText(FIELDNAM, Dxref);   
        }
        if (Dxref->xref_exists(UNITS)){
          labstr += " ";
          labstr += QuGetAttrText(UNITS, Dxref);
        }
      }
      else if(!DSxref.is_nil()){
        cout <<"CAUTION: Index dependency is RECORD VARYING!  Sum may not be meaningful!" << endl;;

        if (DSxref->xref_exists(LABLAXIS)){
          labstr += QuGetAttrText(LABLAXIS, DSxref);  
        }
        else if (DSxref->xref_exists(FIELDNAM)){
          labstr += QuGetAttrText(FIELDNAM, DSxref);   
        }
        if (DSxref->xref_exists(UNITS)){
          labstr += " ";
          labstr += QuGetAttrText(UNITS, DSxref);
        }
      }
      labstr += string(")");
   }
      
   else if (reducedArr->xref_exists(LABLAXIS) ){
  
      // handle case when LABLAXIS is not an array 
      QdObject_var xref=reducedArr->get_xref(LABLAXIS);
      QdStringSeq_var labi = QdStringSeq_var::narrow(xref);
      if(labi.is_nil() ){
          // regular LABLAXIS string
          labstr = QuGetAttrText(LABLAXIS, reducedArr);
	 }
   }
   else if (reducedArr->xref_exists("ObjectName") ){
     labstr = QuGetAttrText("ObjectName", reducedArr);
   }
   else {
      if (average) labstr = string("Ave");
	  else labstr = string("Sum");
   }
   QuSetTxtAttr(LABLAXIS, labstr.c_str(), reducedArr);

    string mods_str;
	if (average) mods_str = "Array averaged over index ";
	else mods_str = "Array summed over index ";
    sprintf(num_str, "%d", indx);
    mods_str += string(num_str);
    mods_str += " (see LABLAXIS)"; 
    QuSetTxtAttr("MODS", mods_str.c_str(), reducedArr);

   if (reducedArr->xref_exists(UNITS)){
      string units_str = QuGetAttrText(UNITS, reducedArr);
      if (average) units_str += " averaged";  
	  else  units_str += " summed"; 
      QuSetTxtAttr("UNITS", units_str.c_str(), reducedArr);
    }

    QString label_i_1 = "?";
    QString depend_i_1 ="?";
 
    for(int i=1; i <= mRank; i++)
    {	 
		QString labl_i = QString("LABEL_%1").arg(i);
		QString dep_i  = QString("DEPEND_%1").arg(i);

		if(i == indx){
			// slice removes this dimension
			if (reducedArr->xref_exists(labl_i.toLatin1().data()) ) reducedArr->delete_xref(labl_i.toLatin1().data());
			if (reducedArr->xref_exists(dep_i.toLatin1().data()) )  reducedArr->delete_xref(dep_i.toLatin1().data());
			label_i_1 = QString(labl_i.toLatin1()); // deep copy
			depend_i_1 = QString(dep_i.toLatin1()); // deep copy
		}
		else{
			// if summed, modify SCALEMAX
			if(!average)
			{
				// change the object SCALEMAX
				if(reducedArr->xref_exists(SCALEMAX))
				{
					QdObject_var obj_max = reducedArr->get_xref(SCALEMAX);
					if(!obj_max.is_nil())
					{
						QdRScalar_var s = QdRScalar_var::narrow(obj_max);
						if( !s.is_nil()) *s *= nave;
					}
				}
			}
			if(i > indx) 
			{
				// shuffle higher dimensions down
				if(reducedArr->xref_exists(labl_i.toLatin1().data())) reducedArr->rename_xref(labl_i.toLatin1().data(), label_i_1.toLatin1().data());
				if(reducedArr->xref_exists(dep_i.toLatin1().data()))  reducedArr->rename_xref(dep_i.toLatin1().data(), depend_i_1.toLatin1().data());
				label_i_1 = QString(labl_i.toLatin1()); // deep copy
				depend_i_1 = QString(dep_i.toLatin1()); // deep copy
			}
		}
    }
 

	return   reducedArr;
   
	// END QuSumArray
}


/***********************************************************************/

QdObject_var QuAliasArray(QdObject_var doa, int indx, int fromK, int toK )
{
	// Reduce dimension by opening dimension indx along time

	QdRScalarSeq_var ss_out;
	QdRMatrixSeq_var ms_out;

	QdObject_var reducedArr; // output object for return  
	if(doa.is_nil()) return reducedArr;
   
	Dim mDim;
	int mRank;
	int ii = indx - 1;  // GUI works [1,n] qdos works [0,n-1]

	QdRMatrixSeq_var MatSeq_in = QdRMatrixSeq_var::narrow(doa);

	if ( MatSeq_in.is_nil() ) {
		cout << "Array Alias to Time requires matrix sequence data" << endl;
		return reducedArr; 
	}        

   
	// sequence size
	int ndata = MatSeq_in->sequence_size();

	// dimension sizes
	mDim = MatSeq_in[0].dimdata();
	mRank = mDim.rank();

	if(ii >= mRank || ii < 0){
		cout << "Array Alias: ERROR - index out of range" << endl;
		return reducedArr; 
	} 
	if(fromK > mDim[ii] || fromK < 1){
		cout << "Array Alias: ERROR - entry out of range for this index" << endl;
		return reducedArr; 
	}
	if(toK > mDim[ii] || toK < 1){
		cout << "Array Alias: ERROR - entry out of range for this index" << endl;
		return reducedArr; 
	}

	int nElem = toK - fromK + 1; 
	if(fromK > toK ) nElem = fromK - toK + 1; // elements in reverse order


	if(mRank == 1){
		if( fromK <= toK ){
			ss_out = new QdRScalarSeq(ndata*nElem);
			for(int i=0; i<ndata; i++){
				for (int ei=fromK-1; ei < toK; ei++){
					ss_out[i*nElem + ei - fromK + 1] = (*MatSeq_in)[i][ei];
				}
			}
		}
	  	else{
			// reverse order
          	ss_out = new QdRScalarSeq(ndata*nElem);
			for(int i=0; i<ndata; i++){
				int count=0;
				for (int ei=fromK-1; ei >= toK-1; ei--){
            		ss_out[i*nElem + count] = (*MatSeq_in)[i][ei];
					count++;
          		}
			}
		}
		reducedArr = (QdObject_var) ss_out;
	}
	else if(mRank == 2){
		

		if(ii == 0){
		
			Dim newDim(mDim[1]);
			
			if( fromK <= toK ){
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei < toK; ei++){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][ei][all];
						count++;
 					}
				}
			}
			else{
	        	// reverse order
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei >= toK-1; ei--){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][ei][all];
						count++;
					}
				}
			}
			reducedArr = (QdObject_var) ms_out;
		}
		else if(ii == 1){
		
			Dim newDim(mDim[0]);
			
			if( fromK <= toK ){
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei < toK; ei++){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][all][ei];
 						count++;
					}
				}
			}
			else{
	        	// reverse order
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei >= toK-1; ei--){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][all][ei];
						count++;
					}
				}
			}
			reducedArr = (QdObject_var) ms_out;
		}
		else{
			cout << "Array Alias to Time: ERROR - matrix rank error, tell CSC support" << endl;
			return reducedArr; 
		}
	}
	else if(mRank == 3){

		if(ii == 0){
			Dim newDim(mDim[1], mDim[2]);
			
			if( fromK <= toK ){
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei < toK; ei++){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][ei][all][all];
						count++;
 					}
				}
			}
			else{
	        	// reverse order
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei >= toK-1; ei--){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][ei][all][all];
						count++;
					}
				}
			}
			reducedArr = (QdObject_var) ms_out;
		}
		else if(ii == 1){
		
			Dim newDim(mDim[0], mDim[2]);
		
			if( fromK <= toK ){
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei < toK; ei++){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][all][ei][all];
 						count++;
					}
				}
			}
			else{
	        	// reverse order
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei >= toK-1; ei--){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][all][ei][all];
						count++;
					}
				}
			}
			reducedArr = (QdObject_var) ms_out;
		}
		else if(ii == 2){
		
			Dim newDim(mDim[0], mDim[1]);
			
			if( fromK <= toK ){
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei < toK; ei++){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][all][all][ei];
						count++;
 					}
				}
			}
			else{
	        	// reverse order
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei >= toK-1; ei--){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][all][all][ei];
						count++;
					}
				}
			}
			reducedArr = (QdObject_var) ms_out;
		}
	}
	else if(mRank == 4){
			
		if(ii == 0){
				
			Dim newDim(mDim[1], mDim[2], mDim[3]);

			if( fromK <= toK ){
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei < toK; ei++){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][ei][all][all][all];
						count++;
 					}
				}
			}
			else{
	        	// reverse order
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei >= toK-1; ei--){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][ei][all][all][all];
						count++;
					}
				}
			}
			reducedArr = (QdObject_var) ms_out;
		}
		else if(ii == 1){
				
			Dim newDim(mDim[0], mDim[2], mDim[3]);
		
			if( fromK <= toK ){
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei < toK; ei++){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][all][ei][all][all];
						count++;
 					}
				}
			}
			else{
	        	// reverse order
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei >= toK-1; ei--){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][all][ei][all][all];
						count++;
					}
				}
			}
			reducedArr = (QdObject_var) ms_out;
		}
		else if(ii == 2){
				
			Dim newDim(mDim[0], mDim[1], mDim[3]);
		
			if( fromK <= toK ){
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei < toK; ei++){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][all][all][ei][all];
						count++;
 					}
				}
			}
			else{
	        	// reverse order
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei >= toK-1; ei--){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][all][all][ei][all];
						count++;
					}
				}
			}
			reducedArr = (QdObject_var) ms_out;
		}
		else if(ii == 3){
				
			Dim newDim(mDim[0], mDim[1], mDim[2]);
		
			if( fromK <= toK ){
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei < toK; ei++){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][all][all][all][ei];
						count++;
 					}
				}
			}
			else{
	        	// reverse order
				ms_out = new QdRMatrixSeq(ndata*nElem, newDim);
				for(int i=0; i<ndata; i++){
					int count=0;
					for (int ei=fromK-1; ei >= toK-1; ei--){
						ms_out[i*nElem + count] = (*MatSeq_in)[i][all][all][all][ei];
						count++;
					}
				}
			}
			reducedArr = (QdObject_var) ms_out;
		}
	}
	else{
		cout << "Array Alias to Time: ERROR - matrix rank, tell CSC support" << endl;
		return reducedArr; 
	} 
   
	if (reducedArr.is_nil() ) {
		cout << "Array Alias to Time: Unable to create unwrapped array sequence" << endl;
		return reducedArr; 
		}
   
   // copy basic Xrefs across...  

     reducedArr->copy_xrefs_from(doa); 

	// create new MODS 

	string labstr("Aliased to time over dimension ");

	QString dep_attr = QString("DEPEND_%1").arg(indx);

	if (reducedArr->xref_exists(dep_attr.toLatin1().data()) ){
		QdObject_var dep_xref=reducedArr->get_xref(dep_attr.toLatin1().data());
		if(dep_xref->xref_exists(LABLAXIS)){
		
			QdObject_var labVar = dep_xref->get_xref(LABLAXIS);
			QdString_var labSvar = QdString_var::narrow(labVar); 
	
			if (!labSvar.is_nil()){
				labstr += "(";
				labstr += labSvar->c_str();
				labstr += ")";
            }
		}
	}
	QuSetTxtAttr("MODS", labstr.c_str(), reducedArr);

	// shuffle depends down
	
    QString label_i_1 = "?";
    QString depend_i_1 ="?";
 
    for(int i=1; i <= mRank; i++)
    {	 
		QString labl_i = QString("LABEL_%1").arg(i);
		QString dep_i  = QString("DEPEND_%1").arg(i);

		if(i == indx){
			// slice removes this dimension
			if (reducedArr->xref_exists(labl_i.toLatin1().data()) ) reducedArr->delete_xref(labl_i.toLatin1().data());
			if (reducedArr->xref_exists(dep_i.toLatin1().data()) )  reducedArr->delete_xref(dep_i.toLatin1().data());
			label_i_1 = QString(labl_i.toLatin1()); // deep copy
			depend_i_1 = QString(dep_i.toLatin1()); // deep copy
		}
		else{
			if(i > indx) 
			{
				// shuffle higher dimensions down
				if(reducedArr->xref_exists(labl_i.toLatin1().data())) reducedArr->rename_xref(labl_i.toLatin1().data(), label_i_1.toLatin1().data());
				if(reducedArr->xref_exists(dep_i.toLatin1().data()))  reducedArr->rename_xref(dep_i.toLatin1().data(), depend_i_1.toLatin1().data());
				label_i_1 = QString(labl_i.toLatin1()); // deep copy
				depend_i_1 = QString(dep_i.toLatin1()); // deep copy
			}
		}
    }
 
 	// Need new timetags and other record dependent attributes
	
	applyAliasToXrefs(doa->sequence_size(),  reducedArr,  nElem);
	 

   return   reducedArr;
   
   // END QuAliasArray
}


/***********************************************************************/
void adjustDeltas(QdObject_var dobj, int nElem)
{

	// divide deltas by number of elements unwrapped along time axis

	QdObject_var d0;
	if(dobj->xref_exists(DEPEND_0)) d0 = dobj->get_xref(DEPEND_0);
	if(d0.is_nil()) return;
	
	
	QdObject_var dp;
	if(d0->xref_exists(DELTA_PLUS)) dp = d0->get_xref(DELTA_PLUS);
	if(!dp.is_nil() ){
		QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(dp);
		QdRScalar_var sv = QdRScalar_var::narrow(dp);
		if(!ss.is_nil() ){
			QdRScalarSeq_var newss = new QdRScalarSeq(ss->sequence_size());
			for(int i=0; i<ss->sequence_size(); i++) newss[i] = ss[i] / nElem;
			d0->change_xref(DELTA_PLUS, newss);
		}
		else if(!sv.is_nil()){
			QdRScalar_var newsv = new QdRScalar(*sv);
			*newsv  /= nElem;
			d0->change_xref(DELTA_PLUS, newsv);
		}
	}
	
	QdObject_var dm;
	if(d0->xref_exists(DELTA_MINUS)) dm = d0->get_xref(DELTA_MINUS);
	if(!dm.is_nil() ){
		QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(dm);
		QdRScalar_var sv = QdRScalar_var::narrow(dm);
		if(!ss.is_nil() ){
			QdRScalarSeq_var newss = new QdRScalarSeq(ss->sequence_size());
			for(int i=0; i<ss->sequence_size(); i++) newss[i] = ss[i] / nElem;
			d0->change_xref(DELTA_MINUS, newss);
		}
		else if(!sv.is_nil()){
			QdRScalar_var newsv = new QdRScalar(*sv);
			*newsv /= nElem;
			d0->change_xref(DELTA_MINUS, newsv);
		}

	}
	

}

/***********************************************************************/
void applyAliasToXrefs(int oldSeqLen, QdObject_var dobj, int nElem)
{
		  
	// Do DEPEND_0 differently
	QdObject_var d0;
	if(dobj->xref_exists(DEPEND_0)) d0 = dobj->get_xref(DEPEND_0);

	if( !d0.is_nil() ) { 
		
		QdRScalarSeq_var xref_ss = QdRScalarSeq_var::narrow(d0);
		QdTimeSeq_var xref_tt = QdTimeSeq_var::narrow(d0);
		
		if( !xref_ss.is_nil() ){
			QdRScalarSeq_var xref_i_out = new QdRScalarSeq(oldSeqLen*nElem);
			
			xref_i_out->copy_xrefs_from(xref_ss);
			if( xref_i_out->xref_exists(DEPEND_0) )	xref_i_out->delete_xref(DEPEND_0);
			double delta;
			QuIsSSRegular(xref_ss, &delta);
			delta = delta / nElem;
			
			for(int i=0; i<oldSeqLen; i++){
				for(int k=0; k<nElem; k++){
					// smear along axis
					xref_i_out[i*nElem + k] = xref_ss[i] + k*delta;
				}
			}
			
			dobj->change_xref(DEPEND_0, xref_i_out);
			applyAliasToXrefs(oldSeqLen, xref_i_out, nElem);
		}
		else if( !xref_tt.is_nil() ){
			QdTimeSeq_var xref_i_out = new QdTimeSeq(oldSeqLen*nElem);
			
			xref_i_out->copy_xrefs_from(xref_tt);
			if( xref_i_out->xref_exists(DEPEND_0) )	xref_i_out->delete_xref(DEPEND_0);
			
			double delta;
			QuIsTSRegular(xref_tt, &delta);
			delta = delta / nElem;
			
			for(int i=0; i<oldSeqLen; i++){
				for(int k=0; k<nElem; k++){
					// smear along axis
					xref_i_out[i*nElem + k] = xref_tt[i] + k*delta;
				}
			}			
			
			dobj->change_xref(DEPEND_0, xref_i_out);
			applyAliasToXrefs(oldSeqLen, xref_i_out, nElem);
		}
	}
	
	// now subdivide delta plus/minus for Depend 0
	adjustDeltas(dobj, nElem); // if a sequence it will get records added later

	vector<string> sl;
	dobj->list_xref_names(sl);
	for(int i=0;i<sl.size();i++)
	{
		if(sl[i][0]==CACHE_XREF_CHAR) continue;
		if(sl[i]==DEPEND_0) continue; // done already 
		
		QdObject_var xref_i = dobj->get_xref(sl[i]);
	
		if( xref_i.is_nil() || oldSeqLen != xref_i->sequence_size()) continue; // not record varying
			  
		QdRScalarSeq_var xref_ss = QdRScalarSeq_var::narrow(xref_i);
		QdRMatrixSeq_var xref_ms = QdRMatrixSeq_var::narrow(xref_i);
		QdTimeSeq_var xref_tt = QdTimeSeq_var::narrow(xref_i);
		if( !xref_ss.is_nil() ){
			QdRScalarSeq_var xref_i_out = new QdRScalarSeq(oldSeqLen*nElem);
			for(int j=0; j<oldSeqLen; j++){
				for(int k=0; k<nElem; k++){
					// duplicate along axis
					xref_i_out[j*nElem + k] = xref_ss[j];
				}
			}			
			xref_i_out->copy_xrefs_from(xref_ss);
			if( xref_i_out->xref_exists(DEPEND_0) ) xref_i_out->delete_xref(DEPEND_0);
		
			dobj->change_xref(sl[i].c_str(), xref_i_out);
			applyAliasToXrefs(oldSeqLen, xref_i_out, nElem);
		}
		else if( !xref_tt.is_nil() ){
			QdTimeSeq_var xref_i_out = new QdTimeSeq(oldSeqLen*nElem);
			for(int j=0; j<oldSeqLen; j++){
				for(int k=0; k<nElem; k++){
					// duplicate along axis
					xref_i_out[j*nElem + k] = xref_tt[j];
				}
			}			
			xref_i_out->copy_xrefs_from(xref_tt);
			if( xref_i_out->xref_exists(DEPEND_0) ) xref_i_out->delete_xref(DEPEND_0);
		
			dobj->change_xref(sl[i].c_str(), xref_i_out);
			applyAliasToXrefs(oldSeqLen, xref_i_out, nElem);
		}
		else if( !xref_ms.is_nil() ){
			QdRMatrixSeq_var xref_i_out = new QdRMatrixSeq(oldSeqLen*nElem);
			for(int j=0; j<oldSeqLen; j++){
				for(int k=0; k<nElem; k++){
					// duplicate along axis
					xref_i_out[j*nElem + k] = xref_ms[j];
				}
			}			
			xref_i_out->copy_xrefs_from(xref_ms);
			if( xref_i_out->xref_exists(DEPEND_0) ) xref_i_out->delete_xref(DEPEND_0);
		
			dobj->change_xref(sl[i].c_str(), xref_i_out);
			applyAliasToXrefs(oldSeqLen, xref_i_out, nElem);

		}
		// else leave as is
	}
		
	
	return;
}	   




/***********************************************************************/

QdObject_var QuSubsetArray(QdObject_var doa, int indx, int fromK, int toK )
{
   
     // Remove elements from an array

   QdRMatrix_var m_out;
   QdRMatrixSeq_var ms_out;

   QdObject_var reducedArr; // output object for return  
   if(doa.is_nil()) return reducedArr;
   
   Dim mDim;
   int mRank;

   // GUI works [1,n] qdos works [0,n-1]
   int ii = indx - 1;
   int j = fromK - 1;
   int k = toK - 1;

   QdRMatrix_var Mat_in = QdRMatrix_var::narrow(doa);
   QdRMatrixSeq_var MatSeq_in = QdRMatrixSeq_var::narrow(doa);

     // Handle sequence and single matrix separately
     // Matrix first
   if ( !Mat_in.is_nil()  ) {
   
      // dimension sizes
      mDim = Mat_in->dimdata();
      mRank = mDim.rank();
 
	 if( fromK == 1 && toK == mDim[ii] ){
	    // change nothing, return
	    return doa;
	 }
	 
      if(ii >= mRank || ii < 0){
        cout << "Array Elements: ERROR - index out of range " << ii << " for range 0," << mRank << endl;
        return reducedArr; 
      }
      if(j >= mDim[ii] || j < 0){
        cout << "Array Elements: ERROR - start entry out of range for this index" << endl;
        return reducedArr; 
      }
      if(k >= mDim[ii] || k < 0){
        cout << "Array Elements: ERROR - end entry out of range for this index" << endl;
        return reducedArr; 
      }
   
      // handle each case separately
      if(mRank == 1){
        m_out = new QdRMatrix((*Mat_in)[slice(j,k-j+1,1)]);
        reducedArr = (QdObject_var) m_out;
      }
      else if(mRank == 2){
        if(ii == 0){
          m_out = new QdRMatrix((*Mat_in)[slice(j,k-j+1,1)][all]);
          reducedArr = (QdObject_var) m_out;
        }
        else if(ii == 1){
          m_out = new QdRMatrix((*Mat_in)[all][slice(j,k-j+1,1)]);
          reducedArr = (QdObject_var) m_out;
        }
        else{
          cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
          return reducedArr;
        }
      }
      else if(mRank == 3){
        if(ii == 0){
          m_out = new QdRMatrix((*Mat_in)[slice(j,k-j+1,1)][all][all]);
          reducedArr = (QdObject_var) m_out;
        }
        else if(ii == 1){
          m_out = new QdRMatrix((*Mat_in)[all][slice(j,k-j+1,1)][all]);
          reducedArr = (QdObject_var) m_out;
        }
        else if(ii == 2){
          m_out = new QdRMatrix((*Mat_in)[all][all][slice(j,k-j+1,1)]);
          reducedArr = (QdObject_var) m_out;
        }
        else{
          cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
          return reducedArr;
        }
     }
     else if(mRank == 4){
       if(ii == 0){
         m_out = new QdRMatrix((*Mat_in)[slice(j,k-j+1,1)][all][all][all]);
         reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 1){
         m_out = new QdRMatrix((*Mat_in)[all][slice(j,k-j+1,1)][all][all]);
         reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 2){
         m_out = new QdRMatrix((*Mat_in)[all][all][slice(j,k-j+1,1)][all]);
         reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 3){
         m_out = new QdRMatrix((*Mat_in)[all][all][all][slice(j,k-j+1,1)]);
         reducedArr = (QdObject_var) m_out;
       }
       else{
         cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }
     else{
        cout << "Array Elements: Matrix Rank error - tell csc support" << endl;
        return reducedArr; 
     }
   } 	

   // try sequence
   else if ( !MatSeq_in.is_nil() ) {
     // sequence size
     int ndata = MatSeq_in->sequence_size();

     // dimension sizes
     mDim = MatSeq_in[0].dimdata();
     mRank = mDim.rank();

	if( fromK == 1 && toK == mDim[ii] ){
	    // change nothing, return
	    return doa;
     }
	
     if(ii >= mRank || ii < 0){
       cout << "Array Elements: ERROR - index out of range at " << indx << " for rank " << mRank << endl;
       return reducedArr; 
     } 
     if(j >= mDim[ii] || j < 0){
       cout << "Array Elements: ERROR - start entry out of range for this index" << endl;
       return reducedArr; 
     }
     if(k >= mDim[ii] || k < 0){
       cout << "Array Elements: ERROR - end entry out of range for this index" << endl;
       return reducedArr; 
     }

     // handle each case separately
     if(mRank == 1){
       ms_out = new QdRMatrixSeq((*MatSeq_in)[all][slice(j,k-j+1,1)]);
       reducedArr = (QdObject_var) ms_out;
     }
     else if(mRank == 2){
       if(ii == 0){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][slice(j,k-j+1,1)][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 1){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][slice(j,k-j+1,1)]);
         reducedArr = (QdObject_var) ms_out;
       }
       else{
         cout << "Array Elements: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }
     else if(mRank == 3){
       if(ii == 0){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][slice(j,k-j+1,1)][all][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 1){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][slice(j,k-j+1,1)][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 2){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][slice(j,k-j+1,1)]);
         reducedArr = (QdObject_var) ms_out;
       }
       else{
         cout << "Array Elements: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }  
     else if(mRank == 4){
       if(ii == 0){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][slice(j,k-j+1,1)][all][all][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 1){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][slice(j,k-j+1,1)][all][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 2){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][slice(j,k-j+1,1)][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 3){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][all][slice(j,k-j+1,1)]);
         reducedArr = (QdObject_var) ms_out;
       }
       else{
         cout << "Array Elements: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }
     else{
       cout << "Array Elements: Sorry, only matrices up to Rank 4 implemented" << endl;
       return reducedArr; 
     }
  
   }
   else{	
     cout << "Array Elements: ERROR - input not an array or array series!" << endl;
     return reducedArr; 
   }        

   // copy basic Xrefs across...  
   // this will ensure new object points to same time tags.
       
   reducedArr->copy_xrefs_from(doa); 

   // create new LABLAXIS 

   char * lab=(char*)"";
   char * comp=(char*)""; 
   char * unit=(char*)"";
   char num_str[50];
   QdString_var labstr;
   static StringRep string_rep;
   QdStringSeq_var labjk;
   QdStringSeq_var lab_i_jk;
   QdStringSeq_var dlab_i_jk;
   
   
   QString labl_attr = QString("LABEL_%1").arg(indx);
   QString dep_attr = QString("DEPEND_%1").arg(indx);
   if (reducedArr->xref_exists(LABLAXIS) ){
  
     // handle case when LABLAXIS is an array 
     QdObject_var xref=reducedArr->get_xref(LABLAXIS);
     QdStringSeq_var labi = QdStringSeq_var::narrow(xref);
     if(!labi.is_nil() && labi->size() > k){
        // works for 1-D ARRAY ONLY await cef final spec.
	   for (int kidx=j; kidx <= k; kidx++){
	      string lab_kidx = (labi[kidx]).c_str();
		 labjk->push_back(lab_kidx);
	   }
     }
     else{
       // regular LABLAXIS string
       labstr = new QdString(QuGetAttrText(LABLAXIS, reducedArr).c_str());
     }
   }
   else if (reducedArr->xref_exists("ObjectName") ){
     labstr = new QdString(QuGetAttrText("ObjectName", reducedArr).c_str());
   }
   if( !labjk.is_nil() )
   {
      reducedArr->change_xref(LABLAXIS, labjk);
   }
   else if ( !labstr.is_nil() )
   {
      QuSetTxtAttr(LABLAXIS, labstr->c_str(), reducedArr);
   }
   // handle LABEL_i, 
   if (reducedArr->xref_exists(labl_attr.toLatin1().data()) ){
     QdObject_var xref=reducedArr->get_xref(labl_attr.toLatin1().data());
     QdStringSeq_var labi = QdStringSeq_var::narrow(xref);
	 QdSMatrix_var mlabi = QdSMatrix_var::narrow(xref);
	 if (!mlabi.is_nil() ){
	    QdSMatrix_var mlab_jk = new QdSMatrix((*mlabi)[slice(j,k-j+1,1)]);
		reducedArr->change_xref(labl_attr.toLatin1().data(), mlab_jk);
	    mlab_jk->copy_xrefs_from(mlab_jk); // ensure labels have their own xrefs
	 }

     if (!labi.is_nil() && labi->size() > k){
	   for (int kidx=j; kidx <= k; kidx++){
	      string lab_kidx = (labi[kidx]).c_str();
		  lab_i_jk->push_back(lab_kidx);
	   }
       reducedArr->change_xref(labl_attr.toLatin1().data(), lab_i_jk);
     }
	 
   }
   // handle DEPEND_i, 
   if (reducedArr->xref_exists(dep_attr.toLatin1().data()) ){

      QdObject_var dxref=reducedArr->get_xref(dep_attr.toLatin1().data());
      QdRMatrix_var Dxref=QdRMatrix_var::narrow(dxref);
      QdRMatrixSeq_var DSxref=QdRMatrixSeq_var::narrow(dxref);
      QdSMatrix_var dlabi = QdSMatrix_var::narrow(dxref);
      if (!dlabi.is_nil() ){
	    QdSMatrix_var dxref_jk = new QdSMatrix((*dlabi)[slice(j,k-j+1,1)]);
		reducedArr->change_xref(dep_attr.toLatin1().data(), dxref_jk);
	    dxref_jk->copy_xrefs_from(dlabi); // ensure depends have their own xrefs
      }
      if(!Dxref.is_nil()){  
	    QdRMatrix_var dxref_jk = new QdRMatrix((*Dxref)[slice(j,k-j+1,1)]);
         reducedArr->change_xref(dep_attr.toLatin1().data(), dxref_jk);
	    dxref_jk->copy_xrefs_from(Dxref); // ensure depends have their own xrefs
	    
	    // handle case when delta plus/minus are also arrays
		if(dxref_jk->xref_exists("DELTA_PLUS")){
			QdObject_var dp_xref=dxref_jk->get_xref("DELTA_PLUS");
			QdRMatrix_var dpMat_xref=QdRMatrix_var::narrow(dp_xref);
			if( !dpMat_xref.is_nil() ) {
				QdRMatrix_var dpMat_xref_jk = new QdRMatrix((*dpMat_xref)[slice(j,k-j+1,1)]);
				dxref_jk->change_xref("DELTA_PLUS", dpMat_xref_jk);
			}
		}
		if(dxref_jk->xref_exists("DELTA_MINUS")){
			QdObject_var dm_xref=dxref_jk->get_xref("DELTA_MINUS");
			QdRMatrix_var dmMat_xref=QdRMatrix_var::narrow(dm_xref);
			if( !dmMat_xref.is_nil() ) {
				QdRMatrix_var dmMat_xref_jk = new QdRMatrix((*dmMat_xref)[slice(j,k-j+1,1)]);
				dxref_jk->change_xref("DELTA_MINUS", dmMat_xref_jk);
			}
		}
      }
      if(!DSxref.is_nil()){    
	    cout <<"CAUTION: Index dependency is RECORD VARYING!  Slice may not be meaningful!" << endl;;
	    QdRMatrixSeq_var dsxref_jk = new QdRMatrixSeq((*DSxref)[all][slice(j,k-j+1,1)]);
         reducedArr->change_xref(dep_attr.toLatin1().data(), dsxref_jk);
	    dsxref_jk->copy_xrefs_from(DSxref); // ensure depends still have xrefs & time tags as t varying

	    // handle case when delta plus/minus are also arrays
	 QdObject_var dps_xref;
	 if(dsxref_jk->xref_exists("DELTA_PLUS"))
           dps_xref=dsxref_jk->get_xref("DELTA_PLUS");
         QdRMatrix_var dpMat_xref=QdRMatrix_var::narrow(dps_xref);
         QdRMatrixSeq_var dpMatS_xref=QdRMatrixSeq_var::narrow(dps_xref);
	    if( !dpMat_xref.is_nil() ) {
	      QdRMatrix_var dpMat_xref_jk = new QdRMatrix((*dpMat_xref)[slice(j,k-j+1,1)]);
           dsxref_jk->change_xref("DELTA_PLUS", dpMat_xref_jk);
	    }
	    else if( !dpMatS_xref.is_nil() ) { // DeltaPlus itself a sequence of matrices
	      QdRMatrixSeq_var dpMatS_xref_jk = new QdRMatrixSeq((*dpMatS_xref)[all][slice(j,k-j+1,1)]);
           dsxref_jk->change_xref("DELTA_PLUS", dpMatS_xref_jk);
	    }

	 QdObject_var dms_xref;
	 if(dsxref_jk->xref_exists("DELTA_MINUS"))
           dms_xref=dsxref_jk->get_xref("DELTA_MINUS");
         QdRMatrix_var dmMat_xref=QdRMatrix_var::narrow(dms_xref);
         QdRMatrixSeq_var dmMatS_xref=QdRMatrixSeq_var::narrow(dms_xref);
	    if( !dmMat_xref.is_nil() ) {
	      QdRMatrix_var dmMat_xref_jk = new QdRMatrix((*dmMat_xref)[slice(j,k-j+1,1)]);
           dsxref_jk->change_xref("DELTA_MINUS", dmMat_xref_jk);
	    }
	    if( !dmMatS_xref.is_nil() ) {  // DeltaMinus itself a sequence of matrices
	      QdRMatrixSeq_var dmMatS_xref_jk = new QdRMatrixSeq((*dmMatS_xref)[all][slice(j,k-j+1,1)]);
           dsxref_jk->change_xref("DELTA_MINUS", dmMatS_xref_jk);
	    }
	   
      }
   }
    string mods_str = "Array subset between entry ";
    sprintf(num_str, "%d", fromK);
    mods_str += string(num_str);
    mods_str += " and entry "; 
    sprintf(num_str, "%d", toK);
    mods_str += string(num_str);
    mods_str += " of index "; 
    sprintf(num_str, "%d", indx);
    mods_str += string(num_str); 
    QuSetTxtAttr("MODS", mods_str.c_str(), reducedArr);

   // index is retained, no need to shuffle LABEL and DEPEND indices down
   return   reducedArr; 
   
   // END QuSubsetArray
}

/***********************************************************************/
QdObject_var QuGetVectorComponent(QdObject_var obj, const char* vcomp)
{
	if(obj.is_nil() ) return obj;

   // First check for cached component
	string vcomp_str(vcomp);
	string vcomp_xref("_"+vcomp_str);
	if(obj->xref_exists(vcomp_xref)) return obj->get_xref(vcomp_xref);
   
   // not cached, so construct component

	QdObject_var vcomp_obj=0;
   
	if(!QuIsThreeVector(obj))  return vcomp_obj;
   
	QdRMatrixSeq_var in_seq=QdRMatrixSeq_var::narrow(obj);
	QdRMatrix_var in_matx=QdRMatrix_var::narrow(obj);
	if(in_seq.is_nil() && in_matx.is_nil()) return vcomp_obj;
   
	string inrep=QuGetVectorRep(obj);
	static CoordRepConversion<RMatrix,RScalar> coord_rep_conv;
	try{
		
		if(!in_seq.is_nil())
		{
			int sz=obj->sequence_size();
			QdRScalarSeq_var out_seq=new QdRScalarSeq(sz);
			vcomp_obj=out_seq;
			coord_rep_conv.get_vcomp(inrep,vcomp,*in_seq,*out_seq);
		}
		else if(!in_matx.is_nil())
		{
			QdRScalar_var out_scalar=new QdRScalar;
			vcomp_obj=out_scalar;
			coord_rep_conv.get_vcomp(inrep,vcomp,*in_matx,*out_scalar);
		}
		vcomp_obj->copy_xrefs_from(obj);
	 
		string label = "";
		QdString_var label_axis_str;
		QdSMatrix_var label_axis;
	 
		if(obj->xref_exists(LABEL_1))
		{
	   // Try LABEL_1 (CAA/CSDS)
			QdObject_var label_one_obj=obj->get_xref(LABEL_1);	     
			label_axis=QdSMatrix_var::narrow(label_one_obj);
			vcomp_obj->delete_xref(LABEL_1);
		}
		else if(obj->xref_exists(LABL_PTR_1))
		{
	   // Try LABL_PTR_1 (ISTP)
			QdObject_var label_ptr_obj=obj->get_xref(LABL_PTR_1);	     
			label_axis=QdSMatrix_var::narrow(label_ptr_obj);
			vcomp_obj->delete_xref(LABL_PTR_1);
		}

		if(!label_axis.is_nil())
		{
			if(label_axis->size()==3 && label_axis->rank()==1)
			{
				QdString_var laVar = new QdString((*label_axis)[0]);
				label= laVar->c_str();
				// get stem only
				size_t pos=label.find("x");
				if(pos!=string::npos)  label=label.substr(0,pos);
			}
		}
		
		if(label.size() == 0 &&  obj->xref_exists(LABLAXIS))
		{
			QdObject_var label_axis_obj=obj->get_xref(LABLAXIS);	     
			label_axis_str=QdString_var::narrow(label_axis_obj);
			
			if(!label_axis_str.is_nil()) label = (*label_axis_str).c_str();
			else{
				QdSMatrix_var strLabMat = QdSMatrix_var::narrow(label_axis_obj);
				if(!strLabMat.is_nil()){
				
					valarray<string> ms_data = strLabMat->data();
					label = ms_data[0].c_str();
					for(register int i=1;i<ms_data.size();i++){

						label = QuCommonLabl(label, ms_data[i].c_str());
					}
				}
				else label = "";
			}
		}
		
     
		if(obj->xref_exists(FIELDNAM)){
			QdString_var fieldnam=new QdString(QuGetAttrText(FIELDNAM,in_seq).c_str());
	 
			if(label.size() == 0 && fieldnam->size() > 0 ) label =  (*fieldnam).c_str() ;
	 
			fieldnam->operator+=("(");
			if(vcomp==string(MAG))
			{
				fieldnam->operator+=("magnitude");
			}
			else{
				fieldnam->operator+=(string("")+vcomp);
			}
	 
			fieldnam->operator+=(")");
			vcomp_obj->change_xref(FIELDNAM, fieldnam);

		}
	 
		if(label.size() == 0 && obj->xref_exists(OBJECT_NAME)){
			// last gasp, try ObjectName created by qie
			QdString_var objectName=new QdString(QuGetAttrText(OBJECT_NAME,in_seq).c_str());
	 
			if( objectName->size() >0) label = (*objectName).c_str();
	 
		}
	 
		QdString_var frame=new QdString("scalar>na");
		vcomp_obj->change_xref(FRAME,frame);
	 
		QdString_var si_conversion;
		QdString_var units;
		if(vcomp==string(LAT_DEG) || vcomp==string(THETA_DEG) || vcomp==string(PHI_DEG))
		{
			si_conversion=new QdString("1>degree");
			units=new QdString("deg");

			size_t pos=vcomp_str.find("(deg");
			if(pos!=string::npos)
				label.insert(0,vcomp_str.substr(0,pos)+" ");
			else
				label.insert(0,vcomp_str+" ");
		}
		else if(vcomp==string(LAT) || vcomp==string(THETA) || vcomp==string(PHI))
		{
			si_conversion=new QdString("1>rad");
			units=new QdString("rad");
			size_t pos=vcomp_str.find("(rad");
			if(pos!=string::npos)
				label.insert(0,vcomp_str.substr(0,pos)+" ");
			else
				label.insert(0,vcomp_str+" ");
	
		}
		else if(vcomp==string(MAG))
		{
			si_conversion=new QdString(QuGetAttrText(SI_CONVERSION,obj).c_str());
			units=new QdString(QuGetAttrText(UNITS,obj).c_str());

			label.insert(0,"|");
			label.append("|");

		}
		else
		{
			si_conversion=new QdString(QuGetAttrText(SI_CONVERSION,obj));
			units=new QdString(QuGetAttrText(UNITS,obj).c_str());
			label.append("_");
			label.append(vcomp);
		}
     
		if(si_conversion->size()==0) si_conversion->operator+=("1>(unknown)");
	   
		if( !si_conversion.is_nil() ) vcomp_obj->change_xref(SI_CONVERSION, si_conversion);
		if( !units.is_nil() ) vcomp_obj->change_xref(UNITS, units);
		if( label.size() != 0) 
		{
			QdString_var labelVar = new QdString(label);
			vcomp_obj->change_xref(LABLAXIS, labelVar);
		}
		QdString_var scid=new QdString(QuGetAttrText(SC_ID, obj).c_str());
		if(scid->size()!=0) vcomp_obj->change_xref(SC_ID,scid);
	 
		obj->change_xref(vcomp_xref,vcomp_obj);
	}
	catch(Exception& e)
	{ 
		e.print_msg();
		cout << "Failed to get component from a vector. " << endl;
		cout << "Note: vectors require a Frame attribute with syntax, e.g. 'vector>gse_xyz' " << endl;
	}


	return vcomp_obj;
}

// ------------------------------------------------------

QdObject_var QuGetTimeComponent(QdObject_var obj, const char* tcomp)
{
   // First check for cached component
   string tcomp_str(tcomp);
   string tcomp_xref("_"+tcomp_str);
   if(obj->xref_exists(tcomp_xref)) return obj->get_xref(tcomp_xref);

   // otherwise create and cache component
   QdObject_var out_obj=0;

   QdTimeInterval_var obj_ivl=get_timeinterval(obj);
   if(obj_ivl.is_nil()) return obj_ivl;

   QdTime_var tobj = 0;

   if(strcmp(tcomp, "Tstart") == 0 )  tobj = new QdTime(obj_ivl->start_abs());
   if(strcmp(tcomp, "Tcentre") == 0 ) tobj = new QdTime(obj_ivl->start_abs() + (obj_ivl->duration_abs()/2.0));
   if(strcmp(tcomp, "Tend") == 0 )    tobj = new QdTime(obj_ivl->end_abs());

   out_obj = (QdObject_var) tobj;
   out_obj->copy_xrefs_from(obj_ivl);
   
   // cache component
   obj->change_xref(tcomp_xref, out_obj);
   
   return out_obj;

}
// ------------------------------------------------------


double ConvertThetaRad(double theta){
  const double PiByTwo = 2.0 * atan(1.0);

  // works for colatitude to latitude and vice versa

  return (double) (PiByTwo - theta);
}

// -------------------------------------------------------

double ConvertThetaDeg(double theta){

  // works for colatitude to latitude and vice versa

  return (double)(90.0 - theta);
}

// -------------------------------------------------------


bool QuIsTSRegular(QdTimeSeq_var tt, double *spacing){
  double err;
  double Gap, Gaptest;
   
  *spacing = 0;
    
  if ( tt.is_nil() ) return false;
  if( !tt->is_sequence() ) return false;

  int n_seq = tt->sequence_size();
  
  Gap = tt[1].getDiffSecs(tt[0]);

  *spacing = Gap;
  if (Gap < 10.*CLOSE_ENOUGH) return false; 
  
  for (int i=2; i<n_seq; i++){
    Gaptest = tt[i].getDiffSecs(tt[i-1]);
    err = fabs((Gap - Gaptest)/Gap);
    if (err > CLOSE_ENOUGH ) return false;
  }
  return true;
}
// -------------------------------------------------------


bool QuIsSSRegular(QdRScalarSeq_var ss, double *spacing){
  double err;
  double Gap, Gaptest;
   
  *spacing = 0;
    
  if ( ss.is_nil() ) return false;
  if( !ss->is_sequence() ) return false;

  int n_seq = ss->sequence_size();
  
  Gap = ss[1] - ss[0];

  *spacing = Gap;
  if (Gap < 10.*CLOSE_ENOUGH) return false; 
  
  for (int i=2; i<n_seq; i++){
    Gaptest = ss[i] - ss[i-1];
    err = fabs((Gap - Gaptest)/Gap);
    if (err > CLOSE_ENOUGH ) return false;
  }
  return true;
}

// -------------------------------------------------------

string QuSanitizeName ( const char * oldName){

  string newName;

  int n_sl = 0;
  for(int i=0; i<strlen(oldName); i++)
    if(oldName[i] == '/') n_sl++;
    
  for(int i=0; i<strlen(oldName); i++){
    if(oldName[i] == '/'){
      newName += "_sl_";
    }
    else{
      newName += oldName[i];
    }
  }

  return newName;
}
/***********************************************************************/

string QuToUpper(const char *InStr)
{
  int i;
  string upper="";
  int s_length;

  s_length = strlen(InStr);

  for (int i=0; i < s_length; i++){
    upper += toupper(InStr[i]);
  }

  return upper;
}
/***********************************************************************/

string QuToUpper(string InStr)
{
	return QuToUpper(InStr.c_str()); 
}

/***********************************************************************/

bool QuStrCmp(const char *A, const char *B){
  
  string strA = QuToUpper(A);
  string strB = QuToUpper(B);
  
  if( strA == strB) return true;

  return false;

}
/***********************************************************************/

bool QuIsJoined(QdObject_var obj1, QdObject_var target){
 
    if(obj1.is_nil() || target.is_nil() ) {
      if(QuSlotDebug())
	  {
		printf("QuIsJoined - Null object\n");
		fflush(stdout);
	  }
      return false;
    }
    
    if(obj1->sequence_size() != target->sequence_size())  {
      if(QuSlotDebug())
	  {
		printf("QuIsJoined - sequences of different lengths %d and %d \n", obj1->sequence_size(), target->sequence_size());
		fflush(stdout);
	  }
      return false;
    }
    
    QdObject_var D0_1 = get_Depend_0(obj1);
	if(D0_1.is_nil() ){
		if(QuSlotDebug())
		{
			printf("QuIsJoined - no DEPEND_0 on object \n");  
			fflush(stdout);
		}
		return false;
	}
	
    QdTimeSeq_var tt1 = QdTimeSeq_var::narrow(D0_1);
    QdTimeSeq_var tt2 = QdTimeSeq_var::narrow(target);
    
    if(tt1.is_nil() || tt2.is_nil() ) {
   
		QdRScalarSeq_var ss1 = QdRScalarSeq_var::narrow(D0_1);
		QdRScalarSeq_var ss2 = QdRScalarSeq_var::narrow(target);
		if ( ss1.is_nil() ||  ss2.is_nil()) {
			if(QuSlotDebug())
			{
				printf("QuIsJoined - is_nil() scalar and time sequence input\n");
				fflush(stdout);
			}
			return false;
		}
		
		if(*ss1 == *ss2) {
			if(QuSlotDebug())
			{
				printf("QuIsJoined - joined: same scalar sequence DEP_0 object_var \n");    
				fflush(stdout);
	  		}
      		return true;
		}
		if(ss1->size() != ss2->size()) {
			if(QuSlotDebug())
			{
				printf("QuIsJoined - number of time tags differ \n");  
				fflush(stdout);
			}
			return false;
		}
		for(int i=0; i<ss1->size(); i++){
			if( fabs(ss1[i] - ss2[i]) > 1.e-30 ) return false;  
		}
		return true;
	}
    else{
		// Try Time tags
		if(*tt1 == *tt2) {
			if(QuSlotDebug())
			{
				printf("QuIsJoined - joined: same timetag object_var \n");    
				fflush(stdout);
			}
			return true;
		}
    
		if(tt1->size() != tt2->size()) {
			if(QuSlotDebug())
			{
				printf("QuIsJoined - number of time tags differ \n");  
				fflush(stdout);
			}
			return false;
		}
    
		for(int i=0; i<tt1->size(); i++){
			// epoch variable can have 2 msec jitter due to double as msec from start date of 0AD
			if( fabs(tt1[i] - tt2[i]) > .002) return false;  
		}
    
		if(QuSlotDebug())
		{
			printf("QuIsJoined - joined to within 2 msec \n");  
			fflush(stdout);
		}
    }
    return true;
}
/***********************************************************************/

bool QuAreJoined(QdObject_var obj1, QdObject_var obj2){
 
    if(obj1.is_nil() || obj2.is_nil() ) {
      if(QuSlotDebug())
	  {
		printf("QuIsJoined - Null object\n");
		fflush(stdout);
	  }
      return false;
    }
    
    if(obj1->sequence_size() != obj2->sequence_size())  {
      if(QuSlotDebug())
	  {
		printf("QuIsJoined - sequences of different lengths %d and %d \n", obj1->sequence_size(), obj2->sequence_size());
		fflush(stdout);
	  }
      return false;
    }
    
    QdObject_var D0_1 = get_Depend_0(obj1);
    QdObject_var D0_2 = get_Depend_0(obj2);
	if(D0_1.is_nil() || D0_2.is_nil()){
		if(QuSlotDebug())
		{
			printf("QuIsJoined - no DEPEND_0 on one object \n");  
			fflush(stdout);
		}
		return false;
	}
	
    QdTimeSeq_var tt1 = QdTimeSeq_var::narrow(D0_1);
    QdTimeSeq_var tt2 = QdTimeSeq_var::narrow(D0_2);
    
    if(tt1.is_nil() || tt2.is_nil() ) {
   
		// Try for scalar DEPEND_0
		
		QdRScalarSeq_var ss1 = QdRScalarSeq_var::narrow(D0_1);
		QdRScalarSeq_var ss2 = QdRScalarSeq_var::narrow(D0_2);
		if ( ss1.is_nil() ||  ss2.is_nil()) {
			if(QuSlotDebug())
			{
				printf("QuIsJoined - is_nil() scalar and time sequence input\n");
				fflush(stdout);
			}
			return false;
		}
		
		if(*ss1 == *ss2) {
			if(QuSlotDebug())
			{
				printf("QuIsJoined - joined: same scalar sequence DEP_0 object_var \n");    
				fflush(stdout);
	  		}
      		return true;
		}
		if(ss1->size() != ss2->size()) {
			if(QuSlotDebug())
			{
				printf("QuIsJoined - number of time tags differ \n");  
				fflush(stdout);
			}
			return false;
		}
		for(int i=0; i<ss1->size(); i++){
			if( fabs(ss1[i] - ss2[i]) > 1.e-28 ) return false;  
		}
		return true;
	}
    else{
		// Try Time tags
		if(*tt1 == *tt2) {
			if(QuSlotDebug())
			{
				printf("QuIsJoined - joined: same timetag object_var \n");    
				fflush(stdout);
			}
			return true;
		}
    
		if(tt1->size() != tt2->size()) {
			if(QuSlotDebug())
			{
				printf("QuIsJoined - number of time tags differ \n");  
				fflush(stdout);
			}
			return false;
		}
    
		for(int i=0; i<tt1->size(); i++){
			// epoch variable can have 2 msec jitter due to double as msec from start date of 0AD
			if( fabs(tt1[i] - tt2[i]) > .002) return false;  
		}
    
		if(QuSlotDebug())
		{
			printf("QuIsJoined - joined to within 2 msec \n");  
			fflush(stdout);
		}
    }
    return true;
}

/***********************************************************************/

bool copyXref(const char * name, QdObject_var from_ptr, QdObject_var to_ptr){

  if( from_ptr.is_nil() || to_ptr.is_nil() ) return false;
  
  if( from_ptr->xref_exists(name) ){
     QdObject_var xrefCopy = from_ptr->get_xref(name);
     to_ptr->change_xref(name, xrefCopy);
     return true;
  }
  
  return false;
}

/***********************************************************************/

bool copyDimXrefs( QdObject_var from_ptr, QdObject_var to_ptr){

 int mtxRank;
 char xrefName[13];

  if( from_ptr.is_nil() ) return false;

  QdRMatrix_var mtx = QdRMatrix_var::narrow(from_ptr);
  if(mtx.is_nil()) {
    QdRMatrixSeq_var mtxseq = QdRMatrixSeq_var::narrow(from_ptr);
    if(mtxseq.is_nil()) return false;
    if(mtxseq->sequence_size() == 0) return false;
    mtxRank = (mtxseq[0]).rank();
  }
  else
     mtxRank = mtx->rank();
  
     // NOTE: copyXref() is safe against non-existent xrefs, so just try it in each case and see.
  bool OK = false;
  for (int k=1; k <= mtxRank; k++){
    sprintf(xrefName, "DEPEND_%1.1d", k);
    if( copyXref(xrefName, from_ptr, to_ptr) ) OK=true;     
    sprintf(xrefName, "LABEL_%1.1d", k);
    if( copyXref(xrefName, from_ptr, to_ptr) ) OK=true;     
    sprintf(xrefName, "REFERENCE_%1.1d", k);
    if( copyXref(xrefName, from_ptr, to_ptr) ) OK=true;     
  }
  
  return OK;

}
/***********************************************************************/

void innerProdDimXrefs( QdObject_var A_ptr,QdObject_var B_ptr, QdObject_var to_ptr){

	// dims after matrix multiplication
	int mRank;
	char xrefName[13];

	if( A_ptr.is_nil() || B_ptr.is_nil()) return;

	// if either is a scalar, use refs from other object unchanged
	QdRScalar_var Sptr = QdRScalar_var::narrow(A_ptr);
	if( !Sptr.is_nil()) {
		copyDimXrefs(B_ptr, to_ptr);
		return;
	}
	Sptr = QdRScalar_var::narrow(B_ptr);
	if( !Sptr.is_nil()) {
		copyDimXrefs(A_ptr, to_ptr);
		return;
	}
	QdRScalarSeq_var SSptr = QdRScalarSeq_var::narrow(A_ptr);
	if( !SSptr.is_nil()) {
		copyDimXrefs(B_ptr, to_ptr);
		return;
	}
	SSptr = QdRScalarSeq_var::narrow(B_ptr);
	if( !SSptr.is_nil()) {
		copyDimXrefs(A_ptr, to_ptr);
		return;
	}
	
	// copy first rank()-1 dimensions from Matrix A
	
	QdRMatrix_var mA = QdRMatrix_var::narrow(A_ptr);
	if(mA.is_nil()) {
		QdRMatrixSeq_var msA = QdRMatrixSeq_var::narrow(A_ptr);
		if(msA.is_nil()) return;
		if(msA->sequence_size() == 0) return;
		mRank = (msA[0]).rank();
	}
	else
		mRank = mA->rank();
  
	int newRank = 0;
	for (int k=1; k <= mRank-1; k++){
		newRank++;
		sprintf(xrefName, "DEPEND_%1.1d", k);
		copyXref(xrefName, A_ptr, to_ptr);     
		sprintf(xrefName, "LABEL_%1.1d", k);
		copyXref(xrefName, A_ptr, to_ptr);     
		sprintf(xrefName, "REFERENCE_%1.1d", k);
		copyXref(xrefName, A_ptr, to_ptr);     
	}
  
	// copy last rank()-1 depends from matrix B
	
	QdRMatrix_var mB = QdRMatrix_var::narrow(B_ptr);
	if(mB.is_nil()) {
		QdRMatrixSeq_var msB = QdRMatrixSeq_var::narrow(B_ptr);
		if(msB.is_nil()) return;
		if(msB->sequence_size() == 0) return;
		mRank = (msB[0]).rank();
	}
	else
		mRank = mB->rank();
  
	for (int k=2; k <= mRank; k++){
		newRank++;
		sprintf(xrefName, "DEPEND_%1.1d", k);
		if(B_ptr->xref_exists(xrefName) )
		{
			QdObject_var xref = B_ptr->get_xref(xrefName);
			sprintf(xrefName, "DEPEND_%1.1d", newRank);
			to_ptr->change_xref(xrefName, xref);
		}
		
		sprintf(xrefName, "LABEL_%1.1d", k);
		if(B_ptr->xref_exists(xrefName) )
		{
			QdObject_var xref = B_ptr->get_xref(xrefName);
			sprintf(xrefName, "LABEL_%1.1d", newRank);
			to_ptr->change_xref(xrefName, xref);
		}

		sprintf(xrefName, "REFERENCE_%1.1d", k);
		if(B_ptr->xref_exists(xrefName) )
		{
			QdObject_var xref = B_ptr->get_xref(xrefName);
			sprintf(xrefName, "REFERENCE_%1.1d", newRank);
			to_ptr->change_xref(xrefName, xref);
		}
	}
  
return;

}

/***********************************************************************/

QColor QuGetColour(QColor inC, bool *newChoice, QWidget * parent){

  *newChoice = false;
  static bool first = true;
  
  if(first)
  {
	int Ncustom = QColorDialog::customCount();
	if (Ncustom >= 16) 
	{
		QColorDialog::setCustomColor(0, QColor(255, 255, 255).rgb());
		QColorDialog::setCustomColor(1, QColor(0, 0, 0).rgb());
		QColorDialog::setCustomColor(2, QColor(255, 0, 0).rgb());
		QColorDialog::setCustomColor(3, QColor(0, 255, 0).rgb());
		QColorDialog::setCustomColor(4, QColor(0, 0, 255).rgb());
		QColorDialog::setCustomColor(5, QColor(0, 255, 255).rgb());
		QColorDialog::setCustomColor(6, QColor(255, 0, 255).rgb());
		QColorDialog::setCustomColor(7, QColor(255, 255, 0).rgb());
		QColorDialog::setCustomColor(8, QColor(255, 128, 0).rgb());
		QColorDialog::setCustomColor(9, QColor(128, 255, 0).rgb());
		QColorDialog::setCustomColor(10, QColor(0, 255, 128).rgb());
		QColorDialog::setCustomColor(11, QColor(0, 128, 255).rgb());
		QColorDialog::setCustomColor(12, QColor(128, 0, 255).rgb());
		QColorDialog::setCustomColor(13, QColor(255, 0, 128).rgb());
		QColorDialog::setCustomColor(14, QColor(85, 85, 85).rgb());
		QColorDialog::setCustomColor(15, QColor(170, 170, 170).rgb());
	}
	
	first = false;
  }
/*  
#if defined(Q_OS_MACX)

  QRgb newRGB = QColorDialog::getRgba(inC.rgb(), newChoice, parent );
  QColor c = QColor(newRGB);
  if(newChoice) return c; // changed, return new colour
  return inC;  // Cancelled - return same as input
     
#else
*/
  QColor c=QColorDialog::getColor(inC, parent);
  if(c.isValid() && c != inC){
    // 
    if(newChoice) *newChoice = TRUE;  // ensure valid bool pointer newChoice was passed in before setting it
    return c;
  }
  else{
    if(newChoice) *newChoice = FALSE;
    return inC;
  }

// #endif

} // end QuGetColour

/***********************************************************************/
// Find records bounding specified time interval using binary search on two ends

bool QuGetIvlRecBound(QdTimeSeq_var tt, int &recStart, int &recEnd, const QmjdTime& tStart, const QmjdTime& tEnd){

   int recL=0;
   int recU=tt->size()-1;
      
	if(tStart > tt[recU] || tEnd < tt[recL]) return false; // ivl outside tt range

	if(tStart < tt[recL] ) recStart = 0; // outside tt range
	else{
		QuGetBounds(tt, recL, recU, tStart);
		if(tt[recL] == tStart ) recStart = recL;
		else recStart = recU;
	}

	recL=0;
	recU=tt->size()-1;
	if(tEnd > tt[recU] ) recEnd = recU; // outside tt range
	else{
		QuGetBounds(tt, recL, recU, tEnd);
		if(tt[recU] == tEnd ) recEnd = recU;
		else recEnd = recL;
	}
	return true;
}

// -------------------------------------------------------------------
// Find the records that bound a time value using recursive binary search

void QuGetBounds(QdTimeSeq_var tt, int &recL, int &recU, const QmjdTime& t){

	if( recU == recL+1) return; // found bounding records
	
	int recH =  (recU + recL) / 2;
	if( tt[recH] < t) recL = recH;
	else recU = recH;
	
	return QuGetBounds(tt, recL, recU, t);
}

// -------------------------------------------------------------------

void QuNow(const char * txt, bool reset){
	static long start = clock(); // initial call
	if(reset) start = clock();
	
	QDateTime  qdatetime ; 
	qdatetime = QDateTime::currentDateTime();
//	printf(" >>>>Timed ------------------- %s at  %s\n",txt, qdatetime.toString( "dd.MM.yyyy-hh:mm:ss.zzz" ).toLatin1().data());
	
	printf(" >>>>seconds ------------------- %s at %.6f \n",txt, ((double)(clock()-start))/CLOCKS_PER_SEC);
}

// -------------------------------------------------------------------

int QuLength(QdObject_var obj){
   int seqLen=1; 
   QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(obj);
   QdRScalar_var    sv = QdRScalar_var::narrow(obj);
   QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(obj);
   QdRMatrix_var    mv = QdRMatrix_var::narrow(obj);
   QdTimeSeq_var    tt = QdTimeSeq_var::narrow(obj);

   if( !tt.is_nil() ) return tt->sequence_size();
   if( !ss.is_nil() ) return ss->sequence_size();
   if( !sv.is_nil() ) return sv->sequence_size();
   if( !ms.is_nil() && ms->sequence_size() == 1) {
      if((ms[0].dimdata()).rank() == 1) return (ms[0].dims())[0];
   }
   if( !mv.is_nil() ) {
      if((mv->dimdata()).rank() == 1) return (mv->dims())[0];
   }
   return seqLen;
}

// --------------------------------------------------------------------

bool QuSet_Array(int ndata, double *dArray, QdObject_var obj, QdTime_var offset){
	// Note that QuLength call ensures we cannot run off array
	
    QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(obj);
    if( !ss.is_nil() ) {
        for(int i=0; i<ndata; i++){
            dArray[i] = (double)ss[i];
        }
        return true;
    }
	
    QdRScalar_var    sv = QdRScalar_var::narrow(obj);
    if( !sv.is_nil() ) {
        dArray[0] = (double)*sv;
        return true;
    }
	
    QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(obj);
    if( !ms.is_nil() && ms->sequence_size() == 1) {
        for(int i=0; i<ndata; i++){
            if((ms[0].dimdata()).rank() == 1) dArray[i] = (double)(ms[0])[i];
        }
        return true;
    }
	
    QdRMatrix_var    mv = QdRMatrix_var::narrow(obj);
    if( !mv.is_nil() ) { 
        for(int i=0; i<ndata; i++){
            if((mv->dimdata()).rank() == 1) dArray[i] = (double)(*mv)[i];
        }
        return true;
    }
	
    QdTimeSeq_var    ts = QdTimeSeq_var::narrow(obj);
    if( !ts.is_nil() && ts->sequence_size() > 0) {
		if(offset.is_nil() ) offset = new QdTime(0);
		
        for(int i=0; i<ndata; i++){
            dArray[i] = ts[i].getDiffSecs(*offset);
        }
        return true;
    }

    return false;
}


double getSpacingTolerance(QdRScalarSeq_var rseq){
   // find minimum spacing between elements
   int len = rseq->sequence_size();
   double diff_sign = 1.0;
   double diffi;
   
   if(len == 0) return 0;
   if(len == 1) return (rseq[0] * 0.1); // allow something to be visible on the plot

   double diff = fabs(rseq[len-1] - rseq[0]);
   for(int i=1; i<rseq->sequence_size(); i++){
      diffi =  rseq[i] - rseq[i-1] ;
      if(fabs(diffi) < diff) diff = fabs(diffi);
	  diff_sign = diffi / diff;
   }
   return (diff * diff_sign);
}

// --------------------------------------------------------------------

QdObject_var makeSequence(QdObject_var inObj, int seqSize )
{  
    if(QuSlotDebug()) cout << " qdutil - makeSequence length " << seqSize << endl;

	fflush(stdout);
   QdObject_var outObj;

   QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(inObj);
   QdRScalar_var s = QdRScalar_var::narrow(inObj);
   QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(inObj);
   QdRMatrix_var m = QdRMatrix_var::narrow(inObj);
   fflush(stdout);
   if( !ss.is_nil() ){
      QdRScalarSeq_var out_ss = new QdRScalarSeq(seqSize, ss[0]);
      out_ss->copy_xrefs_from(inObj);
      outObj = out_ss;
   }
   else if ( !s.is_nil() ){
      QdRScalarSeq_var out_ss = new QdRScalarSeq(seqSize, *s);
      out_ss->copy_xrefs_from(inObj);
      outObj = out_ss;
   }
   else if ( !ms.is_nil() ){
      QdRMatrixSeq_var out_ms = new QdRMatrixSeq(seqSize, ms[0]);
      out_ms->copy_xrefs_from(inObj);
      outObj = out_ms;
   }
   else if ( !m.is_nil() ){
      QdRMatrixSeq_var out_ms = new QdRMatrixSeq(seqSize, *m);
      out_ms->copy_xrefs_from(inObj);
      outObj = out_ms;
   }
   else return inObj;
   
   return outObj;
}

// -------------------------------------------------------

bool QuGetDataRange(QdObject_var obj, double &min_val, double &max_val)
{
	if(QuSlotDebug()) cout << " QuGetDataRange " << endl;
	
	if(obj.is_nil()) return false;
	
	QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(obj);
	if(!ss.is_nil())
	{
		if(ss->size() < 1) return false;
		
		min_val = ss[0];
		max_val = ss[0];
		for(int i=1; i<ss->size(); i++)
		{
			if(ss[i] < min_val) min_val = ss[i];
			if(ss[i] > max_val) max_val = ss[i];
		}
		return true;
	}
	
	QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(obj);
	if(!ms.is_nil())
	{
		if(ms->size() < 1) return false;
		
		double value = min(ms[0]);
		min_val = value;
		value = max(ms[0]);
		max_val = value;
		for(int i=1; i<ms->size(); i++)
		{
			value = min(ms[i]);
			if(value < min_val) min_val = value;
			
			value = max(ms[i]);
			if(value > max_val) max_val = value;
		}	

		return true;
	}
        
	QdRMatrix_var m = QdRMatrix_var::narrow(obj);
	if(!m.is_nil())
	{	
		min_val = min(*m);
		max_val = max(*m);

		return true;
	}

	
	QdTimeSeq_var ts = QdTimeSeq_var::narrow(obj);
	if(!ts.is_nil())
	{
		if(ts->size() < 1) return false;
		QmjdTimeInterval ivl = ts->get_interval_bounds();
		min_val = ivl.start().getEpoch2000();
		max_val = ivl.end().getEpoch2000();
		return true;
	}

	return false;
}


void setRestoreVersion(const char*version)
{
	int ptr=0;
	int len = strlen(version);
	
	while( ptr < len && !isdigit(version[ptr]) ) ptr++;
	if(ptr == len) 
	{ 
		qsas_restore_release = 0;
		return;
	}
	
	int r1 = strtol( &(version[ptr]), NULL, 10);
	while( ptr < len &&  version[ptr] != '.' ) ptr++;
	ptr++;
	if(ptr >= len) 
	{ 
		qsas_restore_release = r1*10000;
		return;
	}
	
	int r2 = strtol( &(version[ptr]), NULL, 10);
	while( ptr < len && version[ptr] != '.' ) ptr++;
	ptr++;
	if(ptr >= len ) 
	{ 
		qsas_restore_release = r1*10000 + r2*100;
		return;
	}

	int r3 = strtol( &(version[ptr]), NULL, 10);
	
	qsas_restore_release = r1*10000 + r2*100 + r3;
	return;
}



QdObject_var QuMergeObjects(QdObject_var &dobj1, QdObject_var &dobj2)
{
	QdObject_var retObj;

	// if either object is empty, just return the other
	if(dobj1.is_nil()) 
	{
		return dobj2;
	}
	if(dobj2.is_nil()) 
	{
		QuMsgBar("second object is empty", true);
		return dobj1;
	}
	
	// reject object with no DEPEND_0 as this is merge on ordered dependency
	QdObject_var d0_1 = get_Depend_0(dobj1);
	if(d0_1.is_nil()) 
	{
		QuMsgBar("object has no Depend_0", true);
		return dobj2;
	}
	QdObject_var d0_2 = get_Depend_0(dobj2);
	if(d0_2.is_nil())  
	{
		QuMsgBar("object has no Depend_0", true);
		return dobj1;
	}
	// DEPEND_0 either time or scalar seq
	QdTimeSeq_var tt1 = QdTimeSeq_var::narrow(d0_1);
	QdTimeSeq_var tt2 = QdTimeSeq_var::narrow(d0_2);
	QdRScalarSeq_var ss1 = QdRScalarSeq_var::narrow(d0_1);
	QdRScalarSeq_var ss2 = QdRScalarSeq_var::narrow(d0_2);
	
	int nD1=0, nD2=0;
	// either both tt exist or both ss exist else sequence is of zero length
	if(!tt1.is_nil() && !tt2.is_nil()){
		nD1 = tt1->sequence_size();
		nD2 = tt2->sequence_size();
	}
	else if(!ss1.is_nil() && !ss2.is_nil()){
		nD1 = ss1->sequence_size();
		nD2 = ss2->sequence_size();
	}
	
	if(QuAreUnitsSame(dobj1, dobj2) == QDUTIL_ERR)
	{
		QuMsgBar("merge failed: base units different", true);
		return retObj;
	}
	
	if(QuAreFramesSame(dobj1, dobj2) == QDUTIL_ERR)
	{
		QuMsgBar("merge failed: different reference frames", true);
		return retObj;
	}
	

	// create mask for final object with input source for each record
	// 0= empty record due to earlier duplicates
	// 1= dobj1
	// 2= dobj2
	// 3= duplicate, take either but increment both on read
	vector <int> pairMask;
	pairMask.assign(nD1 + nD2, 0); // initialise to zero in case of dropped duplicates
	
	int n1=0;
	int n2=0;
	int nMask=0;
	if(!tt1.is_nil()){
		while(n1<nD1 && n2<nD2)
		{
			if(tt1[n1] == tt2[n2]) 
			{
				pairMask[nMask] = 3;
				n1++;
				n2++;
				nMask++;
			}
			else if(tt1[n1] < tt2[n2])
			{
				pairMask[nMask] = 1;
				n1++;
				nMask++;
			}
			else
			{
				pairMask[nMask] = 2;
				n2++;
				nMask++;
			}
		}
	}
	else if(!ss1.is_nil()){
		while(n1<nD1 && n2<nD2)
		{
			if(ss1[n1] == ss2[n2]) 
			{
				pairMask[nMask] = 3;
				n1++;
				n2++;
				nMask++;
			}
			else if(ss1[n1] < ss2[n2])
			{
				pairMask[nMask] = 1;
				n1++;
				nMask++;
			}
			else
			{
				pairMask[nMask] = 2;
				n2++;
				nMask++;
			}
		}
	}
	// finish off dobj2 if dobj1 ran out first
	while(n2<nD2)
	{
		pairMask[nMask] = 2;
		n2++;
		nMask++;
	}
	// finish off dobj1 if dobj2 ran out first
	while(n1<nD1)
	{
		pairMask[nMask] = 1;
		n1++;
		nMask++;
	}
	
	// we now have a mask with the order of records to use, even if one object is nested inside the other
	// for object and t dependent xrefs do merge
	
	retObj = QuMergeVar(nMask, pairMask, dobj1, dobj2);
	
	if(retObj.is_nil())
	{
		QuMsgBar("merge failed: incompatible data types", true);
		return dobj2;
	}
	
	retObj->copy_xrefs_from(dobj1);
	
	// modify t-dep xrefs (inc time tags)
	vector<string> sl;
	dobj1->list_xref_names(sl);
	for(int i=0; i<sl.size(); i++)
	{
		if(sl[i][0]==CACHE_XREF_CHAR) 
		{
			// remove
			retObj->delete_xref(sl[i]); 
			continue;
		}
		QdObject_var xref_obj = dobj1->get_xref(sl[i]);
		if( xref_obj->sequence_size() > 1 &&  xref_obj->sequence_size() == dobj1->sequence_size() ){
			QdObject_var xref_obj2;
			if(dobj2->xref_exists(sl[i])) xref_obj2 = dobj2->get_xref(sl[i]);
			QdObject_var merged_xref_obj = QuMergeVar(nMask, pairMask, xref_obj, xref_obj2);
			merged_xref_obj->copy_xrefs_from(xref_obj);
			
			retObj->change_xref(sl[i], merged_xref_obj);
		}
	}

	return retObj;
}

// -------------------------------------------------------------------

QdObject_var QuMergeVar(int nMask, vector <int> &pairMask, QdObject_var &dobj1, QdObject_var &dobj2)
{
	if(dobj1.is_nil()) return dobj2;
	if(dobj2.is_nil()) return dobj1;
	
	QdObject_var retObj;
	int n1=0;
	int n2=0;
	
	// create new object from inputs according to mask
	QdRScalarSeq_var s1 = QdRScalarSeq_var::narrow(dobj1);
	QdRMatrixSeq_var m1 = QdRMatrixSeq_var::narrow(dobj1);
	QdStringSeq_var st1 = QdStringSeq_var::narrow(dobj1);
	QdTimeSeq_var t1 = QdTimeSeq_var::narrow(dobj1);
	QdTimeIntervalSeq_var ti1 = QdTimeIntervalSeq_var::narrow(dobj1);
	
	if( !s1.is_nil())
	{
		QdRScalarSeq_var s2 = QdRScalarSeq_var::narrow(dobj2);
		if(s2.is_nil()) return retObj; // incompatible objects
		
		QdRScalarSeq_var newS = new QdRScalarSeq(nMask);
		
		for(int i=0; i<nMask; i++)
		{
			if(pairMask[i] == 1)
			{
				newS[i] = s1[n1];
				n1++;
			}
			else if(pairMask[i] == 2) 
			{
				newS[i] = s2[n2];
				n2++;
			}
			else 
			{
				newS[i] = s1[n1];
				n1++;
				n2++;
			}
		}
		retObj = newS;
	}
	else if( !m1.is_nil())
	{
		QdRMatrixSeq_var m2 = QdRMatrixSeq_var::narrow(dobj2);
		if(m2.is_nil() || m2->sequence_size() < 1 || m1->sequence_size() < 1) return retObj; // incompatible objects
		
		Dim mDim1 = m1[0].dimdata();
		Dim mDim2 = m2[0].dimdata();
		if(mDim1.size() != mDim2.size())
		{
			QuMsgBar("merge failed: matrix dimensions differ", true);
			return retObj;
		}
		
		QdRMatrixSeq_var newM = new QdRMatrixSeq(nMask);
		
		for(int i=0; i<nMask; i++)
		{
			if(pairMask[i] == 1)
			{
				newM[i] = m1[n1];
				n1++;
			}
			else if(pairMask[i] == 2) 
			{
				newM[i] = m2[n2];
				n2++;
			}
			else 
			{
				newM[i] = m1[n1];
				n1++;
				n2++;
			}
		}
		retObj = newM;
	}	
	else if( !st1.is_nil())
	{
		QdStringSeq_var st2 = QdStringSeq_var::narrow(dobj2);
		if(st2.is_nil()) return retObj; // incompatible objects
		
		QdStringSeq_var newSt = new QdStringSeq(nMask);
		
		for(int i=0; i<nMask; i++)
		{
			if(pairMask[i] == 1)
			{
				newSt[i] = st1[n1];
				n1++;
			}
			else if(pairMask[i] == 2) 
			{
				newSt[i] = st2[n2];
				n2++;
			}
			else 
			{
				newSt[i] = st1[n1];
				n1++;
				n2++;
			}
		}
		retObj = newSt;
	} 
	else if( !t1.is_nil())
	{
		QdTimeSeq_var t2 = QdTimeSeq_var::narrow(dobj2);
		if(t2.is_nil()) return retObj; // incompatible objects
		
		QdTimeSeq_var newT = new QdTimeSeq(nMask);
		
		for(int i=0; i<nMask; i++)
		{
			if(pairMask[i] == 1)
			{
				newT[i] = t1[n1];
				n1++;
			}
			else if(pairMask[i] == 2) 
			{
				newT[i] = t2[n2];
				n2++;
			}
			else 
			{
				newT[i] = t1[n1];
				n1++;
				n2++;
			}
		}
		retObj = newT;
	} 
	else if( !ti1.is_nil())
	{
		QdTimeIntervalSeq_var ti2 = QdTimeIntervalSeq_var::narrow(dobj2);
		if(ti2.is_nil()) return retObj; // incompatible objects
		
		QdTimeIntervalSeq_var newTi = new QdTimeIntervalSeq(nMask);
		
		for(int i=0; i<nMask; i++)
		{
			if(pairMask[i] == 1)
			{
				newTi[i] = ti1[n1];
				n1++;
			}
			else if(pairMask[i] == 2) 
			{
				newTi[i] = ti2[n2];
				n2++;
			}
			else 
			{
				newTi[i] = ti1[n1];
				n1++;
				n2++;
			}
		}
		retObj = newTi;
	} 
	else
	{
		// not a sequence, so return first object
		return dobj1; // should not happen, just defensive
	}
	
	return retObj;
}

// -------------------------------------------------------------------

QdObject_var QuStripDuplicates(QdObject_var dobj, double tolerance)
{
	// removes duplicate DEPEND_0 values (time tags or scalar) (within t1 - t2 < tolerance)
	if(dobj.is_nil()) return dobj;

	bool hasTT=false;
	bool hasSS=false;
	
	SequenceMask msk(dobj->sequence_size(), true);
	QdObject_var retObj;
	QdObject_var d0 = get_Depend_0(dobj);
	if(d0.is_nil())
	{
		if(QuSlotDebug()) cout << "Util::QuStripDuplicates  No DEPEND_0 found" << endl;
		return retObj;
	}
	
	if(d0->sequence_size() != dobj->sequence_size())
	{
		if(QuSlotDebug()) cout << "Util::QuStripDuplicates  Depend_0 and data sequence lengths differ" << endl;
		return retObj;
	}
	
	QdTimeSeq_var tt = QdTimeSeq_var::narrow(d0);
	QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(d0);
	
	// remove duplicates
	if(!tt.is_nil()){
		hasTT = true;
		
		QmjdTime* tlast = &(tt[0]);
		for(int i=1; i<dobj->sequence_size(); i++)
		{
			double diff = tt[i] - *tlast;
			if(fabs(diff) <= tolerance )
			{
				msk[i] = false;
			}
			else tlast = &(tt[i]);
		}
	}
	else if(!ss.is_nil()){
		hasSS = true;
		double slast = ss[0];
		for(int i=1; i<dobj->sequence_size(); i++)
		{
			double diff = ss[i] - slast;
			if(fabs(diff) <= tolerance )
			{
				msk[i] = false;
			}
			else slast = ss[i];
		}
	}
	else{
		if(QuSlotDebug()) cout << "Util::QuStripDuplicates  Depend_0 not time or scalar sequence" << endl;
		return retObj;
	}
	
	// create new object from inputs according to mask
	QdRScalarSeq_var s1 = QdRScalarSeq_var::narrow(dobj);
	QdRMatrixSeq_var m1 = QdRMatrixSeq_var::narrow(dobj);
	QdStringSeq_var st1 = QdStringSeq_var::narrow(dobj);
	QdTimeSeq_var t1 = QdTimeSeq_var::narrow(dobj);
	QdTimeIntervalSeq_var ti1 = QdTimeIntervalSeq_var::narrow(dobj);
	
	if( !s1.is_nil())
	{
		QdRScalarSeq_var newS = new QdRScalarSeq;
		s1->get_subsequence(*newS,msk);
		newS->copy_xrefs_from(s1);
		
		retObj = newS;
	}
	else if( !m1.is_nil())
	{
		QdRMatrixSeq_var newM = new QdRMatrixSeq;
		m1->get_subsequence(*newM,msk);
		newM->copy_xrefs_from(m1);
		
		retObj = newM;
	}	
	else if( !st1.is_nil())
	{
		QdStringSeq_var newSt = new QdStringSeq;
		st1->get_subsequence(*newSt,msk);
		newSt->copy_xrefs_from(st1);

		retObj = newSt;
	} 
	else if( !t1.is_nil())
	{
		QdTimeSeq_var newT = new QdTimeSeq;
		t1->get_subsequence(*newT,msk);
		newT->copy_xrefs_from(t1);

		retObj = newT;
	} 
	else if( !ti1.is_nil())
	{
		QdTimeIntervalSeq_var newTi = new QdTimeIntervalSeq;
		ti1->get_subsequence(*newTi,msk);
		newTi->copy_xrefs_from(ti1);

		retObj = newTi;
	} 
	else
	{
		// not a sequence, so return first object
		return dobj; // should not happen, just defensive
	}

	
	applyMaskToXrefs(dobj->sequence_size(), retObj, msk);

	return retObj;
}

// -------------------------------------------------------------------

QdObject_var QuMakeMonotonic(QdObject_var dobj)
{
	// shuffles elements so tt are monotonic
	if(dobj.is_nil() || dobj->sequence_size() <= 1) return dobj;
	
	// create sequence holding current ordering
	vector <int> order(dobj->sequence_size());
	
	QdObject_var retObj;
	QdObject_var d0 = get_Depend_0(dobj);
	if(d0.is_nil())
	{
		if(QuSlotDebug()) cout << "Util::QuStripDuplicates  No DEPEND_0 found" << endl;
		return retObj;
	}
	
	if(d0->sequence_size() != dobj->sequence_size())
	{
		if(QuSlotDebug()) cout << "Util::QuStripDuplicates  Depend_0 and data sequence lengths differ" << endl;
		return retObj;
	}
	
	QdTimeSeq_var tt = QdTimeSeq_var::narrow(d0);
	QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(d0);

	if(!tt.is_nil()){	
		// sort order assuming they are nearly monotonic
		order[0] = 0;
		for(int i=1; i<dobj->sequence_size(); i++)
		{
			// compare next tt with latest time found
			double diff = tt[i] - tt[order[i-1]];
			if(diff >=  0) order[i] = i;
			else
			{
				order[i] = order[i-1];
				order[i-1] = i;
				// place in correct place
				for(int j=i-1; j>0; j--)
				{
					if(tt[order[j]] - tt[order[j-1]] < 0.0) 
					{	
						int hold = order[j];
						order[j] = order[j-1];
						order[j-1] = hold;
					}
					else break;
				}
			}
		}
	}
	else if(!ss.is_nil()){	
		// sort order assuming they are nearly monotonic
		order[0] = 0;
		for(int i=1; i<dobj->sequence_size(); i++)
		{
			// compare next tt with latest time found
			double diff = ss[i] - ss[order[i-1]];
			if(diff >=  0) order[i] = i;
			else
			{
				order[i] = order[i-1];
				order[i-1] = i;
				// place in correct place
				for(int j=i-1; j>0; j--)
				{
					if( ss[(int)order[j]] - ss[(int)order[j-1]] < 0.0 ) 
					{	
						int hold = order[j];
						order[j] = order[j-1];
						order[j-1] = hold;
					}
					else break;
				}
			}
		}
	}
	else{
		if(QuSlotDebug()) cout << "Util::QuStripDuplicates  Depend_0 not time or scalar sequence" << endl;
		return retObj;
	}
	
	// the vector order now holds the indices of the elements in time order
	// re-order the data object and all its sequence ordered metadata
	
	retObj = applyOrder(order, dobj);
	
	retObj->copy_xrefs_from(dobj);
	
	QdString_var r_xref = new QdString("Made Monotonic in QSAS");
	retObj->change_xref("Re-ordered", r_xref);

	vector<string> sl;
	retObj->list_xref_names(sl); // this skips cache xrefs

	for(int i=0;i<sl.size();i++)
	{
		QdObject_var xref_i = retObj->get_xref(sl[i]);
		if(xref_i.is_nil() || xref_i->sequence_size() != order.size()) continue; 
		
		QdObject_var newXref = applyOrder(order, xref_i);
		retObj->change_xref(sl[i], newXref);
	}
	
	return retObj;
}

QdObject_var applyOrder( vector <int> &order, QdObject_var dobj)
{
	QdObject_var newObj;
	
	// avoid infinite loop from circular xrefs
	if(dobj->xref_exists("Re-ordered")) return dobj;
	
	// create new object from inputs according to mask
	QdRScalarSeq_var s1 = QdRScalarSeq_var::narrow(dobj);
	QdRMatrixSeq_var m1 = QdRMatrixSeq_var::narrow(dobj);
	QdStringSeq_var st1 = QdStringSeq_var::narrow(dobj);
	QdTimeSeq_var t1 = QdTimeSeq_var::narrow(dobj);
	QdTimeIntervalSeq_var ti1 = QdTimeIntervalSeq_var::narrow(dobj);
	
	if( !s1.is_nil())
	{
		QdRScalarSeq_var newS = Factory::clone(s1);
		for(int i=0; i<s1->sequence_size(); i++)
		{
			if(i != order[i]) newS[i] = s1[order[i]];
		}		
		newObj = newS;
	}
	else if( !m1.is_nil())
	{
		QdRMatrixSeq_var newM = Factory::clone(m1);
		for(int i=0; i<m1->sequence_size(); i++)
		{
			if(i != order[i]) newM[i] = m1[order[i]];
		}		
		newObj = newM;
	}	
	else if( !st1.is_nil())
	{
		QdStringSeq_var newSt = Factory::clone(st1);
		for(int i=0; i<st1->sequence_size(); i++)
		{
			if(i != order[i]) newSt[i] = st1[order[i]];
		}		
		newObj = newSt;
	} 
	else if( !t1.is_nil())
	{
		QdTimeSeq_var newT = Factory::clone(t1);
		for(int i=0; i<t1->sequence_size(); i++)
		{
			if(i != order[i]) newT[i] = t1[order[i]];
		}		
		newObj = newT;
	} 
	else if( !ti1.is_nil())
	{
		QdTimeIntervalSeq_var newTi = Factory::clone(ti1);
		for(int i=0; i<ti1->sequence_size(); i++)
		{
			if(i != order[i]) newTi[i] = ti1[order[i]];
		}		
		newObj = newTi;
	} 
	else
	{
		// not a sequence, so return first object
		return dobj; // should not happen, just defensive
	}
	
	vector<string> sl;
	newObj->list_xref_names(sl); // this skips cache xrefs

	for(int i=0;i<sl.size();i++)
	{
		QdObject_var xref_i = newObj->get_xref(sl[i]);
		if(xref_i.is_nil() || xref_i->sequence_size() != order.size()) continue; 
		
		QdObject_var newXref = applyOrder(order, xref_i);
		newObj->change_xref(sl[i], newXref);
	}


	return newObj;

	
}
// -----------------------------------------------------------------

void cleanWLtext(string &text)
{
	for(int i=0; i<text.size(); i++)
	{
		if(text[i] == '}') text[i] = '/';
	}
}


// --------------------------------------------------------------------



string QuSIConv_product(QdObject_var in1_ptr, 
                         QdObject_var in2_ptr){
						 
 string siconv1, siconv2, siconv;
 char base_str[30];
 float value1, value2, value;
 size_t ptr1, ptr2;
 
 static string null_unit("1 > ()");

  string txt1 = QuGetAttrText("SI_conversion", in1_ptr);
  if( txt1.empty() ) {
    siconv1 = null_unit;
  }
  else{
    siconv1 = txt1;  
  }
  
  string txt2 = QuGetAttrText("SI_conversion", in2_ptr);
  if( txt2.empty() ) {
    siconv2 = null_unit;
  }
  else{
    siconv2 = txt2;  
  }

  if( (ptr1 = siconv1.find(">")) == string::npos) return NULL;
  value1 = atof(siconv1.c_str());
  ptr1++;
  
  if( (ptr2 = siconv2.find(">")) == string::npos) return NULL;
  value2 = atof(siconv2.c_str());
  ptr2++;
  value = value1 * value2;

  sprintf(&(base_str[0]), "%16g >", value);
  siconv = string(&(base_str[0]));
  
  siconv += siconv1.substr(ptr1); 
  siconv += " "; 
  siconv += siconv2.substr(ptr2);
    
  siconv =  DvUnitGetTidySI(siconv.c_str());
  
//   printf("siC1= %s, \nsiC2= %s, \nbase= %s \nsic= %s\n", txt1,txt2,base_str,siconv); fflush(stdout);
  
  
  return siconv;
  
} // end QarSIConv_product

// *************************************************************************

string QuSIConv_inverse(QdObject_var in1_ptr){

 string siconv;
 char * buffer;

  siconv = QuGetAttrText("SI_conversion", in1_ptr);
         
  return DvUnitGetPowerSI(siconv.c_str(), -1);
  
} // copy of QarSIConv_inverse

// *************************************************************************

string QuSIConv_power(QdObject_var in1_ptr, double constB){

  string siconv;

  siconv = QuGetAttrText("SI_conversion", in1_ptr);

  return  DvUnitGetPowerSI(siconv.c_str(), constB);

  
} // copy of QarSIConv_power

// --------------------------------------------------------------------

namespace QSAS
{

SequenceMask UNITmsk(1, true);

// --------------------

void QuMultiJoin( QdObjectSeq_var        in_obj_seq,
                  QdObject_var           target,
                  QdObjectSeq_var        joined_obj_seq,
				  vector <double>		 &gap_or_width,
				  QdStringSeq_var		 gap_method,
				  QdStringSeq_var		 join_method)
{
	bool gapsRemoved=false;
	
    if ( in_obj_seq.is_nil() )
    {
		QuMsgBar("[QuMultiJoin]: No Input Series ", true );
		return;
    }

    if (target.is_nil() )
    {
		QuMsgBar("[QuMultiJoin]: No target sequence ", true );
		return;
    }

    if (gap_method.is_nil() || join_method.is_nil() )
    {
		QuMsgBar("[QuMultiJoin]: No gap/join option ", true );
		return;
    }

    int n_series = in_obj_seq->size();

    if ( gap_or_width.size() != n_series || gap_method->sequence_size() != n_series || join_method->sequence_size() != n_series)
    {
		QuMsgBar("[QuMultiJoin]: Options Wrong Size", true );
		return;
    }

    if (joined_obj_seq.is_nil() )
    {
       joined_obj_seq = new QdObjectSeq( n_series );
    }
	else
    {
        joined_obj_seq->resize( n_series );
    }

	SequenceMask Gmsk(target->sequence_size(), true);
    for( int i = 0; i < n_series; i++ )
    {  
        if (in_obj_seq[i]->sequence_size() <= 1)
        {
			// replace input scalar with sequence of right length and all values the same
			joined_obj_seq[i] = makeSequence( in_obj_seq[i], target->sequence_size() );
	     	joined_obj_seq[i]->change_xref(DEPEND_0, target) ; // is automatically joined
			continue;
        }

        
		if(gap_method[i] == String(REMOVE_GAP)) gapsRemoved = true; // if any object removes gaps, will need to apply mask after
		
			    	   
	   	bool doGaps = false; // QuMultijoin applies mask after all series done
		if(join_method[i] == string(LINEAR)){ 

			joined_obj_seq[i] = QuLinearJoin( in_obj_seq[i],  target,  gap_or_width[i],  gap_method[i],  doGaps, Gmsk);
			
		}
		else if(join_method[i] == string(BOXCAR)){
			
			joined_obj_seq[i] = QuBoxcarJoin( in_obj_seq[i],  target,  gap_or_width[i],  gap_method[i],  doGaps, Gmsk);


		}
		else{
			QuMsgBar("[QuMultiJoin]: Unknown join method", join_method[i].c_str(), true );
			return;
		}
	
   }

    
    // Apply gap mask to all objects
	if(gapsRemoved){
		QdObject_var NULLobj;
		target = QuRemoveGaps(target, Gmsk, NULLobj);
		for(int i=0;i<n_series;i++){
			joined_obj_seq[i] = QuRemoveGaps(joined_obj_seq[i], Gmsk, target);
		} 
	}
	

}

// ---------------------------------------------

QdObject_var QuLinearJoin(QdObject_var obj, QdObject_var target, double gap, String gap_opt, bool doGaps, SequenceMask &Gmsk){

	QdObject_var result;
	QdTimeSeq_var tt = QdTimeSeq_var::narrow(target);
	QdTimeSeq_var tto = get_timetags(obj);
	QdRScalarSeq_var ss;
	QdRScalarSeq_var sso;
	
	// convert times to doubles for speed
	if(!tt.is_nil() && !tto.is_nil() ) {
		ss = new QdRScalarSeq(tt->sequence_size());
		for(int i=0; i<ss->sequence_size(); i++){
			ss[i] = tt[i].getEpoch2000();
		}
		sso = new QdRScalarSeq(tto->sequence_size());
		for(int i=0; i<sso->sequence_size(); i++){
			sso[i] = tto[i].getEpoch2000();
		}
		
	}
	else{
		ss = QdRScalarSeq_var::narrow(target);
		sso = QdRScalarSeq_var::narrow(get_Depend_0(obj));
		if(ss.is_nil() || sso.is_nil() ) {
			QuMsgBar("[QuLinearJoin]: target Depend_0 not found", true );
			return result;
		}
	}
	

	// invert decreasing tags
	if(ss[0] > ss[ss->sequence_size()-1]){
		if(sso[0] > sso[sso->sequence_size()-1]){
			*sso *= -1.;
			*ss  *= -1.;
		}
		else{
			QuMsgBar("[QuLinearJoin]: target Depend_0 decreasing, but data Depend_0 increasing", true );
			return result;
		}
	}
	
	if(sso[0] > ss[ss->sequence_size()-1] || ss[0] > sso[sso->sequence_size()-1]){
		QuMsgBar("[QuLinearJoin]: Disjoint data sequences in join", true );
		return result;
	}
	
	// fillval
	QdRScalar_var   qdfillval;
	double fillval = 0.1e-30;
	if ( obj->xref_exists( FILL_VALUE ) )
	{
		qdfillval  = QdRScalar_var::narrow(obj->get_xref(FILL_VALUE ));
		if(!qdfillval.is_nil()) fillval = *qdfillval;
	}
	

	// gap tolerance in seconds
	if(gap <= 0){
		gap = get_time_spacing(obj);
		if(gap <=0) gap = get_scalar_spacing(obj);
		if(gap <=0) QuMsgBar( "[QuLinearJoin]: Cannot determine data spacing", true );
	
		gap *=1.5; 
	}

	// do join
	
	QdRScalarSeq_var Sobj = QdRScalarSeq_var::narrow(obj);
	QdRMatrixSeq_var Mobj = QdRMatrixSeq_var::narrow(obj);
	
	// local mask for this sequence
	SequenceMask msk(ss->sequence_size(), true);
		
	if(!Sobj.is_nil()) {
		QdRScalarSeq_var Snew = QuLinearJoinScalar( msk,  Sobj,  sso,  ss,  gap,  gap_opt, fillval);
		result = Snew;
	}
	else if(!Mobj.is_nil()) {
		QdRMatrixSeq_var Mnew = QuLinearJoinMatrix(msk,  Mobj,  sso,  ss,  gap,  gap_opt, fillval);
		result = Mnew;
	}
	
	if(Gmsk.size() != 1 && Gmsk.size() == msk.size()){
		// Gmsk has only one element unless called for variable from MultiJoin
		// Simple join call or xref joins don't get here
		
		for(int i=0; i<msk.size(); i++){
			// accumulate union of gaps
			if(!msk[i]) Gmsk[i] = false;
		}
	}
	
	if(result.is_nil()) return result;


	// attach new Depend_0

	result->change_xref(DEPEND_0, target);

	// join record varying xrefs
	QuJoinXrefs(obj, result, gap);
	
	if(doGaps){
	
		// do gap mask recursively now as this was not called by QuMultijoin.
		if(gap_opt == String(REMOVE_GAP)) {
			QdObject_var NULLobj;
			target = QuRemoveGaps(target, msk, NULLobj);

			obj = QuRemoveGaps(obj, msk, target);
		}
	}
	
	return result;
}

// ---------------

QdRScalarSeq_var QuLinearJoinScalar(SequenceMask &msk, QdRScalarSeq_var Sobj, QdRScalarSeq_var sso, QdRScalarSeq_var ss, double gap, String gap_opt, double fillval){
	// do join
	
	QdRScalarSeq_var Snew = new QdRScalarSeq(ss->sequence_size());

	int i_obj=0;
	int i_ss=0;
	
	if(ss[0] < sso[0]){
		// data starts after target, get within gap tolerence of data
		while (ss[i_ss] + gap < sso[0] ) {
			// treat as gap
			if(gap_opt == String(FILL_GAP)) {
				Snew[i_ss] = fillval;
			}
			else if(gap_opt == String(LINEAR_INTERP_GAP)) {
				if(i_obj + 1 < sso->sequence_size() )
					Snew[i_ss] = LinearInterpScalar( Sobj,  sso,  i_obj+1, fillval, ss[i_ss]);
				else Snew[i_ss] = Sobj[i_obj];
			}
			else if(gap_opt == String(NEAREST_NEIGHBOUR_GAP)) {
				Snew[i_ss] = Sobj[i_obj];
			}
			else if(gap_opt == String(REMOVE_GAP)) {
				Snew[i_ss] = 0.;
				msk[i_ss]=false; // remove (done later)
			}
			else Snew[i_ss] = 0.; // zero fill
			i_ss++;
		}
		
		if(ss[i_ss] < sso[0]){
			// extrapolate (same call as interpolate)
			Snew[i_ss] =  LinearInterpScalar( Sobj,  sso,  i_obj+1, fillval, ss[i_ss]);
			i_ss++;
		}
		// else fall through to interpolation
		
	}
	
	// sso[i_obj] starts above ss[i_ss] 
	while(i_ss<ss->sequence_size()){
	
		double target = ss[i_ss];
		
		// find data pt above target tag
		while(sso[i_obj] < target  &&  i_obj<Sobj->sequence_size()-1) i_obj++;
		
		// is this a data gap or off the end of data by more than gap?
		
		if(i_obj > 0 && ( sso[i_obj] - sso[i_obj-1] > gap || ss[i_ss] - sso[i_obj] > gap) ){
		
			// data gap
			
			if(gap_opt == String(FILL_GAP)) {
				Snew[i_ss] = fillval;
			}
			else if(gap_opt == String(LINEAR_INTERP_GAP)) {
				Snew[i_ss] = LinearInterpScalar( Sobj,  sso,  i_obj, fillval, target);
			}
			else if(gap_opt == String(NEAREST_NEIGHBOUR_GAP)) {
				if( ((double)sso[i_obj] - target) < (target - (double)sso[i_obj-1]) ) Snew[i_ss] = Sobj[i_obj];
				else Snew[i_ss] = Sobj[i_obj-1];
			}
			else if(gap_opt == String(REMOVE_GAP)) {
				Snew[i_ss] = 0.; // remove (done later)
				msk[i_ss]=false; 
			}
			else Snew[i_ss] = 0.; // zero fill
			
			
		}
		else{
			// Not a gap, normal interpolation
			
			// if we are off end of data then interpolation algorithm works as extrapolation
		
			Snew[i_ss] =  LinearInterpScalar( Sobj,  sso,  i_obj, fillval, target);
		}
			
		i_ss++;
	}

	
	// this object has no xrefs yet and has not had the gap mask applied
	return Snew;
	
}
// ---------------

QdRMatrixSeq_var QuLinearJoinMatrix(SequenceMask &msk, QdRMatrixSeq_var Mobj, QdRScalarSeq_var sso, QdRScalarSeq_var ss, double gap, String gap_opt, double fillval){
	// do join
	
	QdRMatrixSeq_var Mnew = new QdRMatrixSeq(ss->sequence_size());
	QdRMatrix_var fill_ptr = new QdRMatrix(Mobj[0].dims(), fillval);
	QdRMatrix_var zero_ptr = new QdRMatrix(Mobj[0].dims(), 0.0);
	
	int i_obj=0;
	int i_ss=0;
	
	// do first element before main while loop
	if(ss[0] < sso[0]){
		// data starts after target, get within gap tolerence of data
		while (ss[i_ss] + gap < sso[0] ) {
			// treat as gap 
			if(gap_opt == String(FILL_GAP)) {
				Mnew[i_ss] = *fill_ptr;
			}
			else if(gap_opt == String(LINEAR_INTERP_GAP)) {
				if(i_obj + 1 < sso->sequence_size() )
					 LinearInterpMatrix( Mnew[i_ss], Mobj,  sso,  i_obj+1, fillval, ss[i_ss]);
				else 
					Mnew[i_ss] = Mobj[i_obj];
			}
			else if(gap_opt == String(NEAREST_NEIGHBOUR_GAP)) {
				Mnew[i_ss] = Mobj[i_obj];
			}
			else if(gap_opt == String(REMOVE_GAP)) {
				Mnew[i_ss] = *zero_ptr; 
				msk[i_ss]=false; // remove done later
			}
			else Mnew[i_ss] = *zero_ptr; // zero fill 
			
			i_ss++;
		}
		
		if(ss[i_ss] < sso[0]){
			// extrapolate (same call as interpolate)
			LinearInterpMatrix( Mnew[i_ss], Mobj,  sso,  i_obj+1, fillval, ss[i_ss]);
			i_ss++;
		}
		// else fall through to interpolation
		
	}

	// sso[i_obj] starts above ss[i_ss] 
	while(i_ss<ss->sequence_size()){
	
		double target = ss[i_ss];
		
		// find data pt above target tag
		while(sso[i_obj] < target  &&  i_obj<Mobj->sequence_size()-1) i_obj++;
		
		// is this a data gap or off the end of data by more than gap?
		
		if(i_obj > 0 && ( sso[i_obj] - sso[i_obj-1] > gap || ss[i_ss] - sso[i_obj] > gap) ){
		
			// data gap
			
			if(gap_opt == String(FILL_GAP)) {
				Mnew[i_ss] = *fill_ptr;
			}
			else if(gap_opt == String(LINEAR_INTERP_GAP)) {
				LinearInterpMatrix( Mnew[i_ss], Mobj,  sso,  i_obj, fillval, target);
			}
			else if(gap_opt == String(NEAREST_NEIGHBOUR_GAP)) {
				if( ((double)sso[i_obj] - target) < (target - (double)sso[i_obj-1]) ) Mnew[i_ss] = Mobj[i_obj];
				else Mnew[i_ss] = Mobj[i_obj-1];
			}
			else if(gap_opt == String(REMOVE_GAP)) {
				Mnew[i_ss] = *zero_ptr; 
				msk[i_ss]=false; //remove (done later)
			}
			else Mnew[i_ss] = *zero_ptr; // zero fill 
			
			
		}
		else{
			// Not a gap, normal interpolation
			
			// if we are off end of data then interpolation algorithm works as extrapolation
		
			LinearInterpMatrix( Mnew[i_ss], Mobj,  sso,  i_obj, fillval, target);
		}
			
		i_ss++;
	}

	
	// this object has no xrefs yet and has not had the gap mask applied
	return Mnew;
	
}
// ------------------

double LinearInterpScalar(QdRScalarSeq_var Sobj, QdRScalarSeq_var sso, int i_obj, double fillval, double target){
	
	if(i_obj > 0 ){
		// use only one if other is fillval (if both are fill then fillval is returned)
		if(isFill(Sobj[i_obj-1], fillval) ) return Sobj[i_obj];
		else if(isFill(Sobj[i_obj], fillval) ) return Sobj[i_obj-1];
		
		// neither is fillval, get linear interp
		return (Sobj[i_obj-1] * ((double)sso[i_obj] - target) + Sobj[i_obj] * (target - (double)sso[i_obj-1])) / (double)(sso[i_obj] - sso[i_obj-1]);
	}
	else 
		return Sobj[i_obj];

}

// ---------------------

void LinearInterpMatrix( QDOS::Matrix<double> &result, QdRMatrixSeq_var Mobj, QdRScalarSeq_var sso, int i_obj, double fillval, double target){
		
	if(i_obj > 0 ){
		double factor =  (target - (double)sso[i_obj-1]) / (double)(sso[i_obj] - sso[i_obj-1]);
		result =  (Mobj[i_obj] - Mobj[i_obj-1]);
		result *= factor;
		result += Mobj[i_obj-1];
	}
	else 
		result = Mobj[i_obj];

}

// ---------------------

void QuJoinXrefs(QdObject_var preJoinObj, QdObject_var joinedObj, double gap_width, bool useBoxcar){

	// xrefs will come from the pre-joined object (we need their original time tags)
	// and be attached to the joined object
	  
	vector<string> sl;
	preJoinObj->list_xref_names(sl);
	for(int ix=0;ix<sl.size();ix++)
	{
		if(sl[ix][0]==CACHE_XREF_CHAR) continue;
		if(upper_case(sl[ix]) == "DEPEND_0" ) continue;  // helps avoid infinite loop
		  
		QdObject_var xref_obj = preJoinObj->get_xref(sl[ix]);
		// attach xref to new object
		joinedObj->change_xref(sl[ix], xref_obj);

		if( xref_obj->sequence_size() != preJoinObj->sequence_size()) continue; // not conformal sequence
		  
		QdNObject_var nobj = QdNObject_var::narrow(xref_obj);
		if( nobj.is_nil() ) continue; // not numeric xref, can't interpolate
		  
		QdObject_var D0_obj = get_Depend_0(xref_obj);
		if(D0_obj.is_nil() ) {
			// sequence length is same as object, so use tt from object
			D0_obj = get_Depend_0(preJoinObj);
			xref_obj->change_xref(DEPEND_0, D0_obj); // keep them for recursive call for xrefs of xrefs
		}
			 			 
		// this is recursive, leave gap mask till last
		bool doGaps = false;
		QdObject_var xref_result;
		if(useBoxcar){	
			xref_result = QuBoxcarJoin(xref_obj, get_Depend_0(joinedObj),  gap_width, String(NEAREST_NEIGHBOUR_GAP), doGaps);
		}
		else{
			xref_result = QuLinearJoin(xref_obj, get_Depend_0(joinedObj),  gap_width, String(NEAREST_NEIGHBOUR_GAP), doGaps);
		}
		joinedObj->change_xref(sl[ix], xref_result);


	} // end for over variable's xrefs
		
}


// ---------------------

QdObject_var QuRemoveGaps(QdObject_var obj, SequenceMask &msk, QdObject_var target){

	QdObject_var result;
	
	int oldLength = obj->sequence_size();
	
	// apply gap mask to object and xrefs
	QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(obj);
	if(!ss.is_nil()){
		// Apply mask
		QdRScalarSeq_var Sresult = new QdRScalarSeq();
		ss->get_subsequence(*Sresult, msk);
		
		result = Sresult;
	}
	
	QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(obj);
	if(!ms.is_nil()){
		// Apply mask
		QdRMatrixSeq_var Mresult = new QdRMatrixSeq();
		ms->get_subsequence(*Mresult, msk);
		result = Mresult;
	}

	QdTimeSeq_var tt = QdTimeSeq_var::narrow(obj);
	if(!tt.is_nil()){
		QdTimeSeq_var tdep0 = new QdTimeSeq();
		tt->get_subsequence(*tdep0, msk);
		result = tdep0;
	}

	
	vector<string> sl;
	obj->list_xref_names(sl);
	for(int ix=0;ix<sl.size();ix++)
	{
		if(sl[ix][0]==CACHE_XREF_CHAR) continue;
		  
		QdObject_var xref_obj = obj->get_xref(sl[ix]);
		if(xref_obj.is_nil()) continue;
			
		// copy xref to result
		result->change_xref(sl[ix], xref_obj);
			
		if( xref_obj->sequence_size() != oldLength) continue; // not conformal sequence
		
		// dep 0 done separately
		if(upper_case(sl[ix]) != "DEPEND_0" ){ 
			
			QdObject_var newXref = QuRemoveGaps(xref_obj, msk, target);
			
			// replace xref as masked
			result->change_xref(sl[ix], newXref);			
		}
			
	} // end for over variable's xrefs
	
	
	if( !target.is_nil()) result->change_xref(DEPEND_0, target);			

	return result;
}


// ---------- BOXCAR ----------

// ---------------------------------------------

QdObject_var QuBoxcarJoin(QdObject_var obj, QdObject_var target, double width, String gap_opt, bool doGaps, SequenceMask &Gmsk){

	QdObject_var result;
	
	QdTimeSeq_var tt = QdTimeSeq_var::narrow(target);
	QdTimeSeq_var tto = get_timetags(obj);
	QdRScalarSeq_var ss;
	QdRScalarSeq_var sso;
	
	// convert times to doubles for speed
	if(!tt.is_nil() && !tto.is_nil() ) {
		ss = new QdRScalarSeq(tt->sequence_size());
		for(int i=0; i<ss->sequence_size(); i++){
			ss[i] = tt[i].getEpoch2000();
		}
		sso = new QdRScalarSeq(tto->sequence_size());
		for(int i=0; i<sso->sequence_size(); i++){
			sso[i] = tto[i].getEpoch2000();
		}
		
	}
	else{
		ss = QdRScalarSeq_var::narrow(target);
		sso = QdRScalarSeq_var::narrow(get_Depend_0(obj));
	}
	

	// invert decreasing tags
	if(ss[0] > ss[ss->sequence_size()-1]){
		if(sso[0] > sso[ss->sequence_size()-1]){
			*sso *= -1.;
			*ss  *= -1.;
		}
		else{
			QuMsgBar("[QuBoxcarJoin]: target Depend_0 decreasing, but data Depend_0 increasing", true );
			return result;
		}
	}
	
	// fillval
	QdRScalar_var   qdfillval;
	double fillval = 0.1e-30;
	if ( obj->xref_exists( "FILLVAL" ) )
	{
		qdfillval  = QdRScalar_var::narrow(obj->get_xref("FILLVAL" ));
		if(!qdfillval.is_nil()) fillval = *qdfillval;
	}
	

	// boxvcar in seconds
	if(width <= 0){
		width = get_time_spacing(obj);
		if(width <=0) width = get_scalar_spacing(obj);
		if(width <=0)  QuMsgBar( "[QuBoxcarJoin]: Cannot determine data spacing", true );
	
		width *=10.; 
	}

	double halfWidth = width*0.5;
	
	// do join
	
	QdRScalarSeq_var Sobj = QdRScalarSeq_var::narrow(obj);
	QdRMatrixSeq_var Mobj = QdRMatrixSeq_var::narrow(obj);
	
	// local mask for this sequence
	SequenceMask msk(ss->sequence_size(), true);
		
	if(!Sobj.is_nil()) {
		QdRScalarSeq_var Snew = QuBoxcarJoinScalar( msk,  Sobj,  sso,  ss,  halfWidth,  gap_opt, fillval);
		result = Snew;
	}
	else if(!Mobj.is_nil()) {
		QdRMatrixSeq_var Mnew = QuBoxcarJoinMatrix(msk,  Mobj,  sso,  ss,  halfWidth,  gap_opt, fillval);
		result = Mnew;
	}
	
	if(Gmsk.size() != 1 && Gmsk.size() == msk.size()){
		// Gmsk has only one element unless called for variable from MultiJoin
		// Simple join call or xref joins don't get here
		
		for(int i=0; i<msk.size(); i++){
			// accumulate union of gaps
			if(!msk[i]) Gmsk[i] = false;
		}
	}
	
	if(result.is_nil()) return result;


	// attach new Depend_0

	result->change_xref(DEPEND_0, target);

	// join record varying xrefs
	bool useBoxcar=true;
	QuJoinXrefs(obj, result, halfWidth, useBoxcar);
	
	if(doGaps){
	
		// do gap mask recursively now as this was not called by QuMultijoin.
		if(gap_opt == String(REMOVE_GAP)) {
			QdObject_var NULLobj;
			target = QuRemoveGaps(target, msk, NULLobj);

			obj = QuRemoveGaps(obj, msk, target);
		}
	}
	
	return result;
}

// ---------------

QdRScalarSeq_var QuBoxcarJoinScalar(SequenceMask &msk, QdRScalarSeq_var Sobj, QdRScalarSeq_var sso, QdRScalarSeq_var ss, double halfWidth, String gap_opt, double fillval){

	// do join
	
	QdRScalarSeq_var Snew = new QdRScalarSeq(ss->sequence_size());
	int i_obj=0;
	int i_ss=0;
	
	if(ss[0] < sso[0]){
		// data starts after target boxcar
		while (ss[i_ss] + halfWidth < sso[0] ) {
			// treat as gap
			if(gap_opt == String(FILL_GAP)) {
				Snew[i_ss] = fillval;
			}
			else if(gap_opt == String(LINEAR_INTERP_GAP)) {
				// this will extrapolate
				if(i_obj + 1 < sso->sequence_size() )
					Snew[i_ss] = LinearInterpScalar( Sobj,  sso,  i_obj+1, fillval, ss[i_ss]);
				else Snew[i_ss] = Sobj[i_obj];
			}
			else if(gap_opt == String(NEAREST_NEIGHBOUR_GAP)) {
				Snew[i_ss] = Sobj[i_obj];
			}
			else if(gap_opt == String(REMOVE_GAP)) {
				Snew[i_ss] = 0.;
				msk[i_ss]=false; // remove (done later)
			}
			else Snew[i_ss] = 0.; // zero fill
			i_ss++;
		}
		
		// else fall through to interpolation
		
	}
	
	// sso[i_obj] within half width of ss[i_ss] 
	
	while(i_ss<ss->sequence_size()){
	
		double target = ss[i_ss];
		
		// find data pt in boxcar
		while(sso[i_obj] < target - halfWidth &&  i_obj<Sobj->sequence_size()-1 ) i_obj++;
		
		// try to get boxcar, if not set as gap
		
		if( i_obj == Sobj->sequence_size()-1 || BoxcarInterpScalar(Snew[i_ss], Sobj,  sso,  i_obj, halfWidth, fillval, target) == false ){
		
			// data gap
			
			if(gap_opt == String(FILL_GAP)) {
				Snew[i_ss] = fillval;
			}
			else if(gap_opt == String(LINEAR_INTERP_GAP)) {
				// ss[i_obj] is above boxcar min, but no pt in boxcar, 
				// so this is 1st pt above boxcar and this interpolates across gap
				Snew[i_ss] = LinearInterpScalar( Sobj,  sso,  i_obj, fillval, target);
			}
			else if(gap_opt == String(NEAREST_NEIGHBOUR_GAP)) {
				// ss[i_obj] is above boxcar min, but no pt in boxcar, 
				// so this is 1st pt above boxcar  
				if( ((double)sso[i_obj] - target) < (target - (double)sso[i_obj-1]) ) Snew[i_ss] = Sobj[i_obj];
				else Snew[i_ss] = Sobj[i_obj-1];
			}
			else if(gap_opt == String(REMOVE_GAP)) {
				Snew[i_ss] = 0.; // remove (done later)
				msk[i_ss]=false; 
			}
			else Snew[i_ss] = 0.; // zero fill
			
		}
			
		i_ss++;
	}

	// this object has no xrefs yet and has not had the gap mask applied
	return Snew;
	
}

// ---------------

QdRMatrixSeq_var QuBoxcarJoinMatrix(SequenceMask &msk, QdRMatrixSeq_var Mobj, QdRScalarSeq_var sso, QdRScalarSeq_var ss, double halfWidth, String gap_opt, double fillval){

	// do join
	
	QdRMatrixSeq_var Mnew = new QdRMatrixSeq(ss->sequence_size());
	QdRMatrix_var fill_ptr = new QdRMatrix(Mobj[0].dims(), fillval);
	QdRMatrix_var zero_ptr = new QdRMatrix(Mobj[0].dims(), 0.0);

	int i_obj=0;
	int i_ss=0;
	
	if(ss[0] < sso[0]){
		// data starts after target boxcar
		while (ss[i_ss] + halfWidth < sso[0] ) {
			// treat as gap
			if(gap_opt == String(FILL_GAP)) {
				Mnew[i_ss] = *fill_ptr;
			}
			else if(gap_opt == String(LINEAR_INTERP_GAP)) {
				// this will extrapolate
				if(i_obj + 1 < sso->sequence_size() )
					LinearInterpMatrix( Mnew[i_ss], Mobj,  sso,  i_obj+1, fillval, ss[i_ss]);
				else Mnew[i_ss] = Mobj[i_obj];
			}
			else if(gap_opt == String(NEAREST_NEIGHBOUR_GAP)) {
				Mnew[i_ss] = Mobj[i_obj];
			}
			else if(gap_opt == String(REMOVE_GAP)) {
				Mnew[i_ss] = *zero_ptr;
				msk[i_ss]=false; // remove (done later)
			}
			else Mnew[i_ss] = *zero_ptr; // zero fill
			i_ss++;
		}
		
		// else fall through to interpolation
		
	}
	
	// sso[i_obj] within half width of ss[i_ss] 
	
	while(i_ss<ss->sequence_size()){
	
		double target = ss[i_ss];
		
		// find data pt in boxcar
		while(sso[i_obj] < target - halfWidth &&  i_obj<Mobj->sequence_size()-1 ) i_obj++;
		
		// try to get boxcar, if not set as gap
		
		if( i_obj == Mobj->sequence_size()-1 || BoxcarInterpMatrix(Mnew[i_ss], Mobj,  sso,  i_obj, halfWidth, fillval, target) == false ){
		
			// data gap
			
			if(gap_opt == String(FILL_GAP)) {
				Mnew[i_ss] = *fill_ptr;
			}
			else if(gap_opt == String(LINEAR_INTERP_GAP)) {
				// ss[i_obj] is above boxcar min, but no pt in boxcar, 
				// so this is 1st pt above boxcar and this interpolates across gap
				LinearInterpMatrix(Mnew[i_ss], Mobj,  sso,  i_obj, fillval, target);
			}
			else if(gap_opt == String(NEAREST_NEIGHBOUR_GAP)) {
				// ss[i_obj] is above boxcar min, but no pt in boxcar, 
				// so this is 1st pt above boxcar  
				if( ((double)sso[i_obj] - target) < (target - (double)sso[i_obj-1]) ) Mnew[i_ss] = Mobj[i_obj];
				else Mnew[i_ss] = Mobj[i_obj-1];
			}
			else if(gap_opt == String(REMOVE_GAP)) {
				Mnew[i_ss] = *zero_ptr; // remove (done later)
				msk[i_ss]=false; 
			}
			else Mnew[i_ss] = *zero_ptr; // zero fill
			
		}
			
		i_ss++;
	}

	// this object has no xrefs yet and has not had the gap mask applied
	return Mnew;
	
}
// ------------------

bool BoxcarInterpScalar(QDOS::Scalar<double> &result, QdRScalarSeq_var Sobj, QdRScalarSeq_var sso, int i_obj, double halfWidth, double fillval, double target){

	// i_obj starts above lower edge of boxcar
	int i = i_obj;
	int length = sso->sequence_size();
	result = 0.;
	// accumulate all values up to top edge of boxcar
	while( i < length && sso[i] < target + halfWidth){
		if( !isFill(Sobj[i], fillval) ) {
			result += Sobj[i];
			i++;
		}
	}
	
	if(i > i_obj ){
		result = result / (double) (i - i_obj);
		return true;
	}
	else return false;
}


bool BoxcarInterpMatrix( QDOS::Matrix<double> &result, QdRMatrixSeq_var Mobj, QdRScalarSeq_var sso, int i_obj, double halfWidth, double fillval, double target){
	
	// i_obj starts above lower edge of boxcar
	int i = i_obj;
	int length = sso->sequence_size();
	
	// initialise boxcar
	if( i < length && sso[i] < target + halfWidth ) {
		result = Mobj[i_obj];
		i++;
	}
		
	// accumulate all values up to top edge of boxcar
	while( i < length && sso[i] < target + halfWidth){
		result += Mobj[i];
		i++;
	}
	
	if(i > i_obj ){
		result = result / (double) (i - i_obj);
			
		return true;
	}
	
	return false;

}
		
		
} // end namespace QSAS

	vector <int> idList;

// ---------------------

DvObject_var toDVOS(QdObject_var &qobj, int id){
	
	// convert QDOS object to DVOS
	if(qobj.is_nil()) {
		return new DvObject();
	}
	if(id == 0) idList.clear();
	
	idList.push_back(id);
	
	QdRScalar_var s = QdRScalar_var::narrow(qobj);
	QdRMatrix_var m = QdRMatrix_var::narrow(qobj);
	QdString_var str = QdString_var::narrow(qobj);
	QdSMatrix_var strm = QdSMatrix_var::narrow(qobj);
	QdTime_var t = QdTime_var::narrow(qobj);
	QdTimeInterval_var ti = QdTimeInterval_var::narrow(qobj);
	QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(qobj);
	QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(qobj);
	QdStringSeq_var strs = QdStringSeq_var::narrow(qobj);
	QdTimeSeq_var ts = QdTimeSeq_var::narrow(qobj);
	QdSMatrixSeq_var strms = QdSMatrixSeq_var::narrow(qobj);
	QdTimeIntervalSeq_var tis = QdTimeIntervalSeq_var::narrow(qobj);
	if(!s.is_nil()){
		DvObject_var res = new DvObject(*s);
		addXrefsToDVOS(qobj, res);
		return res;	
	}
	else if(!ss.is_nil()){
		int n_recs = ss->sequence_size();
		valarray <double> dval(n_recs);
		for(int n=0; n < n_recs; n++){
        	dval[n] = ss[n]; 
		}
		
		DvObject_var res = new DvObject(dval);
		addXrefsToDVOS(qobj, res);
		
		return res;		
	}
	else if(!m.is_nil()){
		size_t nArray=1.;
		vector <size_t> dims;
		for(int j=0; j<(m->dims()).size(); j++) {
        	dims.push_back((size_t) (m->dims())[j]);
        	nArray *= dims[j];
		}
		valarray <double> dval(nArray);
          
		int i=0;
        for(QdRMatrix::iterator itr=m->begin(); itr != m->end(); itr++, i++){
           dval[i] = *itr;
        }

		DvObject_var res = new DvObject(dval, dims);
		addXrefsToDVOS(qobj, res);
		return res;		
	}
	else if(!ms.is_nil()){
		int n_recs = ms->sequence_size();
		size_t nArray=1.;
		vector <size_t> dims;
		for(int j=0; j<(ms[0].dims()).size(); j++) {
        	dims.push_back((size_t) (ms[0].dims())[j]);
        	nArray *= dims[j];
		}
		valarray <double> dval(nArray*n_recs);
          
		int i=0;
		for(int n=0; n < n_recs; n++){
           		
			// get underlying data for matrix element
			valarray<double> v_in(ms[n].data()[ms[n].getgslice()]);
			for(register int j=0;j<v_in.size();j++, i++)
			{
				dval[i] = v_in[j];
			}
		}

		DvObject_var res = new DvObject(dval, dims, n_recs);
		addXrefsToDVOS(qobj, res);
		return res;		
	
	}
	else if(!str.is_nil()){
		DvObject_var res = new DvObject(str->c_str());
		addXrefsToDVOS(qobj, res);
		return res;	
	}
	else if(!strs.is_nil()){
		int n_recs = strs->sequence_size();
		valarray <DvString> dval(n_recs);
		for(int n=0; n < n_recs; n++){
        	dval[n] = strs[n]; 
		}
		DvObject_var res = new DvObject(dval);
		addXrefsToDVOS(qobj, res);
		return res;			
	}
	else if(!strm.is_nil()){
		size_t nArray=1.;
		vector <size_t> dims;
		for(int j=0; j<(strm->dims()).size(); j++) {
        	dims.push_back((size_t) (strm->dims())[j]);
        	nArray *= dims[j];
		}
		valarray <DvString> dval(nArray);
          
		int i=0;
        for(QdSMatrix::iterator itr=strm->begin(); itr != strm->end(); itr++, i++){
           dval[i] = *itr;
        }

		DvObject_var res = new DvObject(dval, dims);
		addXrefsToDVOS(qobj, res);
		return res;		
	
	}
	else if(!strms.is_nil()){
		int n_recs = strms->sequence_size();
		size_t nArray=1.;
		vector <size_t> dims;
		for(int j=0; j<(strms[0].dims()).size(); j++) {
        	dims.push_back((size_t) (strms[0].dims())[j]);
        	nArray *= dims[j];
		}
		valarray <DvString> dval(nArray*n_recs);
          
		int i=0;
		for(int n=0; n < n_recs; n++){
			const QdSMatrix &mat = strms[n];
			for(QdSMatrix::iterator itr=mat.begin(); itr != mat.end(); itr++, i++){
           		dval[i] = *itr;
			}
		}
		DvObject_var res = new DvObject(dval, dims, n_recs);
		addXrefsToDVOS(qobj, res);
		return res;		
	}
	else if(!t.is_nil()){
		DvTime tval(t->getEpoch2000());
		DvObject_var res = new DvObject(tval);
		addXrefsToDVOS(qobj, res);
		return res;		
	}
	else if(!ts.is_nil()){
		int n_recs = ts->sequence_size();
		valarray <DvTime> dval(n_recs);
		for(int n=0; n < n_recs; n++){
        	dval[n] = ts[n].getEpoch2000(); 
		}
		DvObject_var res = new DvObject(dval);
		addXrefsToDVOS(qobj, res);
		return res;			
	}
	else if(!ti.is_nil()){
		double start = ti->start().getEpoch2000();
		double end = ti->end().getEpoch2000();
		DvEvent eval(start, end);
		DvObject_var res = new DvObject(eval);
		addXrefsToDVOS(qobj, res);
		return res;			
	}
	else if(!tis.is_nil()){
		int n_recs = tis->sequence_size();
		valarray <DvEvent> dval(n_recs);
		for(int n=0; n < n_recs; n++){
			double start = tis[n].start().getEpoch2000();
			double end = tis[n].end().getEpoch2000();
        	dval[n] = DvEvent(start, end); 
		}
		DvObject_var res = new DvObject(dval);
		addXrefsToDVOS(qobj, res);
		return res;				
	}
	return new DvObject();

}

void addXrefsToDVOS(QdObject_var &qobj, DvObject_var &dobj){

	// recursively add xrefs, watch for inf loop in QDOS
    vector<string> sl;
	qobj->list_xref_names(sl);
	for(int i=0; i<sl.size(); i++){
		if(sl[i][0] == '_') continue; // skip cache
		QdObject_var xref_var = qobj->get_xref(sl[i].c_str());
		if(xref_var.is_nil()) continue;
		bool found = false;
		for(int j=0; j<idList.size(); j++) {
			if( xref_var->get_id() == idList[j]) {
				// avoid infinite loop
				found = true;
				break;
			}
		}
		if(!found){
			DvObject_var xref = toDVOS(xref_var, xref_var->get_id());
			dobj->change_xref(sl[i].c_str(), xref);
		}
	}
	
}

// ---------------------

QdObject_var toQDOS(DvObject_var &dobj){
	
	// convert DVOS object to QDOS

	QdObject_var res;
	if(dobj.is_nil()) {
		return res;
	}
	
	if(dobj->is_dbl() ) {
		if(dobj->seqSize() == 1){
			if(dobj->arraySize() == 1){
				QdRScalar_var s = new QdRScalar(dobj->dbl());
				res = s;
			}
			else{
				vector <size_t> dvdims = dobj->Dims();
				vector <int> qd;
				for(int i=0; i<dvdims.size(); i++) qd.push_back( (int) dvdims[i] );
				Dim qdims(qd);
				QdRMatrix_var m = new QdRMatrix(qdims, 0.0);
				size_t i=0;
        		for(QdRMatrix::iterator itr=m->begin(); itr != m->end(); itr++, i++){
					*itr = dobj->dbl(i);
        		}
				res = m;
			}	
		}
		else{
			size_t n_recs = dobj->seqSize();
			if(dobj->arraySize() == 1){
				QdRScalarSeq_var ss = new QdRScalarSeq(n_recs, 0.0);
				for(int n=0; n<n_recs; n++) ss[n] = dobj->dbl(n);
				res = ss;
			}
			else{
				vector <size_t> dvdims = dobj->Dims();
				vector <int> qd;
				for(int i=0; i<dvdims.size(); i++) qd.push_back( (int) dvdims[i] );
				Dim qdims(qd);
				QuNow("new matrix ");
				QdRMatrixSeq_var ms = new QdRMatrixSeq(n_recs, qdims);
				QuNow("new matrix done");
				int arrSize = dobj->arraySize();
				for(int n=0; n<n_recs; n++){
					size_t i=0;
					// get underlying data for matrix element
					ms[n].data()[ms[n].getgslice()] = dobj->record(n);
				}
				QuNow("new matrix filled");
				res = ms;
			}	
		}
	}
	if(dobj->is_int()) {
		if(dobj->seqSize() == 1){
			if(dobj->arraySize() == 1){
				QdRScalar_var s = new QdRScalar((double) dobj->itg());
				res = s;
			}
			else{
				vector <size_t> dvdims = dobj->Dims();
				vector <int> qd;
				for(int i=0; i<dvdims.size(); i++) qd.push_back( (int) dvdims[i] );
				Dim qdims(qd);
				QdRMatrix_var m = new QdRMatrix(qdims, 0.0);
				size_t i=0;
        		for(QdRMatrix::iterator itr=m->begin(); itr != m->end(); itr++, i++){
					*itr = (double)dobj->itg(i);
        		}
				res = m;
			}	
		}
		else{
			size_t n_recs = dobj->seqSize();
			if(dobj->arraySize() == 1){
				QdRScalarSeq_var ss = new QdRScalarSeq(n_recs, 0.0);
				for(int n=0; n<n_recs; n++) ss[n] = (double) dobj->itg(n);
				res = ss;
			}
			else{
				vector <size_t> dvdims = dobj->Dims();
				vector <int> qd;
				for(int i=0; i<dvdims.size(); i++) qd.push_back( (int) dvdims[i] );
				Dim qdims(qd);
				QdRMatrixSeq_var ms = new QdRMatrixSeq(n_recs, qdims);
				int arrSize = dobj->arraySize();
				for(int n=0; n<n_recs; n++){
					size_t i=0;
        			// get underlying data for matrix element
					valarray<double> v_in(ms[n].data()[ms[n].getgslice()]);
					for(register int j=0;j<v_in.size();j++)
					{
						v_in[j] = (double)dobj->itg(n*arrSize + i);
        			}
				}
				res = ms;
			}	
		}
	}
	else if(dobj->is_str()) {
		if(dobj->seqSize() == 1){
			if(dobj->arraySize() == 1){
				QdString_var str = new QdString(dobj->str().c_str());
				res = str;
			}
			else{
				vector <size_t> dvdims = dobj->Dims();
				vector <int> qd;
				for(int i=0; i<dvdims.size(); i++) qd.push_back( (int) dvdims[i] );
				Dim qdims(qd);
				QdSMatrix_var strm = new QdSMatrix(qdims );
				size_t i=0;
        		for(QdSMatrix::iterator itr=strm->begin(); itr != strm->end(); itr++, i++){
					*itr = dobj->str(i).c_str();
        		}
				res = strm;
			}	
		}
		else{
			size_t n_recs = dobj->seqSize();
			if(dobj->arraySize() == 1){
				QdStringSeq_var strs = new QdStringSeq(n_recs);
				for(int n=0; n<n_recs; n++) strs[n] = dobj->str(n).c_str();
				res = strs;
			}
			else{
				vector <size_t> dvdims = dobj->Dims();
				vector <int> qd;
				for(int i=0; i<dvdims.size(); i++) qd.push_back( (int) dvdims[i] );
				Dim qdims(qd);
				QdSMatrixSeq_var mstrs = new QdSMatrixSeq(n_recs, qdims);
				int arrSize = dobj->arraySize();
				for(int n=0; n<n_recs; n++){
					size_t i=0;
        			for(QdSMatrix::iterator itr=mstrs[n].begin(); itr != mstrs[n].end(); itr++, i++){
						*itr = dobj->str(n*arrSize + i).c_str();
        			}
				}
				res = mstrs;
			}	
		}
	}
	else if(dobj->is_time()) {
		if(dobj->seqSize() == 1){
			if(dobj->arraySize() == 1){
				QdTime_var t = new QdTime(dobj->time().getEpoch2000());
				res = t;
			}
			else{
				// no matrix time type
				return res;
			}	
		}
		else{
			size_t n_recs = dobj->seqSize();
			if(dobj->arraySize() == 1){
				QdTimeSeq_var ts = new QdTimeSeq(n_recs);
				for(int n=0; n<n_recs; n++) ts[n] = QdTime(dobj->time(n).getEpoch2000());
				res = ts;
			}
			else{
				// no matrix seq time type
				return res;
			}	
		}
	}
	else if(dobj->is_event()) {
		if(dobj->seqSize() == 1){
			if(dobj->arraySize() == 1){
				QdTimeInterval_var ti = new QdTimeInterval((dobj->event()).start().getEpoch2000(), (dobj->event()).end().getEpoch2000());
				res = ti;
			}
			else{
				// no matrix time type
				return res;
			}	
		}
		else{
			size_t n_recs = dobj->seqSize();
			if(dobj->arraySize() == 1){
				QdTimeIntervalSeq_var tis = new QdTimeIntervalSeq(n_recs);
				for(int n=0; n<n_recs; n++) tis[n] = QdTimeInterval((dobj->event(n)).start().getEpoch2000(), (dobj->event(n)).end().getEpoch2000());
				res = tis;
			}
			else{
				// no matrix seq time type
				return res;
			}	
		}
	}
	addXrefsToQDOS(dobj, res);
	return res;				

}

void addXrefsToQDOS(DvObject_var &dobj, QdObject_var &qobj){

	// recursively add xrefs
    vector<DvString> sl;
	dobj->list_xref_names(sl);
	for(int i=0; i<sl.size(); i++){
		DvObject_var xref_var = dobj->get_xref(sl[i].c_str());
		if( !xref_var.is_nil()){
			QdObject_var xref = toQDOS(xref_var);
			qobj->change_xref(sl[i].c_str(), xref);
		}
	}
	
}


